PRO applycomb, strin, strout, wdfin, comb=comb, intervals=nint, sweep=sweep,$
   edit=edit

;**********************************************************************
;* Program to analyze the laser comb on a streak and apply the 
;* time dispersion to the full streak.
;*
;* strin = input streak image in pixel units
;* strout = output streak imabe in time units
;* wdfin = wdf array containing lineout of the comb
;*       *note: wdf array should contain only the comb, no extra space
;* comb = peak-to-peak interval of comb in ns (Default = 4 ns)
;* intervals = number of intervals to use in analyzing the comb
;* sweep = sweep speed on the streak camera
;* edit = keyword to allow editing of points prior to fitting
;**********************************************************************

if (n_elements(comb) eq 0) then comb = 4.
if (n_elements(nint) eq 0) then nint = 1.
if (n_elements(sweep) eq 0) then sweep=430.
frange=sweep/comb/40500.*[0.5,1.5]

w2i,xx,yy,wdfin
interval=(max(xx)-min(xx))/nint
mxfreq=dblarr(nint)
midx=dblarr(nint)
for i=0,nint-1 do begin
   xfr,wdfin,61
   lim,61,xx(0)+i*interval,xx(0)+(i+1)*interval
   midx(i)=((xx(0)+i*interval)+(xx(0)+(i+1)*interval))/2.
   fftf,61,y,freq
   ind=where(freq gt frange(0) and freq lt frange(1))
   i2w,freq(ind),abs(y(ind)),62,/xy
   fw=fwhm(62,base=0.,first=left,second=right)
   cent=(left+right)/2.
   mxfreq(i)=cent
   plot,freq,abs(y),xran=frange,xtitle='Frequency',ytitle='Amplitude'
   oplot,mxfreq(i)*[1.,1.],!y.crange,line=2
   wait,0.25
endfor

i2w,midx,comb*mxfreq,63,/xy
if keyword_set(edit) then wdedit,63
wdfit,63,64,deg=2,coef=fit
plo,63,2,psym=[8,0],col=[0,1]
wait,0.25

xv=get_array(strin,1)
yv=get_array(strin,2)
zv=get_array(strin,-1)
tdisp=fit(0)+yv*fit(1)+yv^2.*fit(2)
nyv=n_elements(yv)
yint=yv(20)-yv(19)
t=dblarr(nyv)
t(0)=0
for i=1,nyv-1 do begin
   t(i)=t(i-1)+tdisp(i)*yint
endfor
arr2str,strout,zv,xv,t,xlabel='x (microns)',ylabel='Time (ns)',clab=strcomment(strin)


return
end