      subroutine idl_main (lcterm, mode, prompt, xsize, ysize,
     &   hist_size, command_out, ierr)
c
c     This routine provides a means of getting strings from IDL.
c
c     Two types of interface are possible
c        LCTerm  =  0  Widget interface
c                      The widget interface will maintain a command buffer.
c                      The commands for mode = 0 and mode = 1 will be different.
c                      The commands can be recalled by double click or by 
c                         hitting return when they are designated.
c        LCTerm not 0  Terminal interface (xterm, etc)
c     For each type of interface two modes are possible.
c        Mode  = 0   Commands will be processed in idl until a EXIT command
c                    is entered.  When EXIT is entered, the widget will be 
c                    destroyed (If there is a widget.)
c        Mode  = 1   Commands will be returned to the calling program.
c                    If there is a widget, the the widget will stay active.
c     Prompt         String used to indicate instructions.
c                    For LCTerm = 0, Default is:
c                         "Enter Command Below"
c                      (No maximum length)
c                    For LCTerm not 0, Default is
c                      for mode = 0: "IDL>"
c                      for mode = 1: "ATHETA>"
c                      (Maximum length ~ 32 characters)
c     Xsize          Size of window width in characters 
c                      Default = 50
c     Ysize          Number of commands visible at one time
c                      Default = 20
c     Hist_size      Total number of commands available.
c                      Default = 100
c     Command_out    Command entered with mode = 1
c     IERR           Error Parameter
c     
      integer  lcterm, xsize, ysize, mode, hist_size, ierr
      character *(*) prompt, command_out
      character *4 cmd, cexit
      character *96 get_string
      character *16 atheta_string, idl_main_prompt
      character *48 myprompt
      integer errcount, mxerror
      integer mxchar
      parameter (mxchar = 256)
      parameter (mxerror = 10)
      character*(mxchar) command
      character*(mxchar) cmd_string
      parameter (cexit = 'EXIT')
      parameter (atheta_string = 'ATHETA_CMDSTRING')
      parameter (idl_main_prompt = 'IDL_MAIN_PROMPT ')

c **********************************************************************
      integer ier, i, j
      integer lcreset
      integer nchar
c  start idl_main
      errcount = 0
      call cidl_strtrim(prompt, nchar, i, j)
      if (lcterm .eq. 0) then
        ier = 0
        if (nchar .eq. 0) then
          myprompt = 'Enter Command Below '
        else
          myprompt = prompt(i:j)
        endif
        call send_str_var(idl_main_prompt, myprompt, ier)
        if (ier .ne. 0 ) then 
          write(cmd_string, 104) ier
          ierr = ier
104       format('IDL_MAIN: Error sending prompt. IER =', i4)
          ier = 0
          call idl_write(cmd_string, ier)
          return
        endif
        lcreset = 0
       
        write (cmd_string, 103) atheta_string, xsize, ysize, 
     &     mode, hist_size
103     format('GET_IDL_COMMAND, ', a, ', xsize =', i4, ', ysize =', 
     &  i3, ', mode =', i4, ', max_stack =', i5, 
     &   ', prompt = idl_main_prompt')
      else
        if (nchar .eq. 0) then 
          if (mode .eq. 0) then 
            myprompt = 'IDL>'
            j = 4
          else 
            myprompt = 'ATHETA>'
            j = 7
          endif
        else
          myprompt = prompt(i:j)
          j = nchar
        endif
        write (get_string, 105), atheta_string, myprompt(1:j), 
     &    atheta_string
105     format(a, ' = '' '' &  read, prompt= ''', a, 
     &    ' '', ', a)
      endif
      ier = 0
2     continue
      if (lcterm .eq. 0) then
c       use widget interface
        call send_command(cmd_string, ier)
      else
c       lcterm ne 0 - use the old interface
        call send_command(get_string, ier)
      endif
      call get_str_var(atheta_string, command, nchar, ier) 
      if (mode .ne. 0) then 
        ierr = ier
        command_out = command
        return
      endif
      if (ier .ne. 0) then
        if (ier .lt. 100) then
          write (6, 101) ier
101       format ('%IDL_MAIN: Error getting IDL command:', I5)
          return
        endif
        write(command, 102) mxchar
102     format('%IDL_MAIN: Commands limited to', i4, 'characters')
        call idl_write (command(1:44), 0)
        errcount = errcount+1
        if (errcount .eq. mxerror) then
          command = '% IDL_MAIN: Maximum number of errors exceeded '
          call idl_write (command(1:46), 0)
          command_out = 'EXIT'
          return
        endif 
        goto 2
      endif
      errcount = 0
      call cidl_strtrim(command, nchar, i, j)
      if (nchar .eq. 0) go to 2
      if (nchar .eq. 4) then
        cmd = command (i:j)
        call cidl_upcase(cmd, nchar)
        if (cmd .eq. cexit) return
      endif
      call send_command(command(i:j), ier)
      goto 2

      end
