PRO exray_ip, MainWidget, nocheck=nocheck, noplot=noplot, nosave=nosave, angcor=angcor,$
               angle=ang

;********************************************************************
;* Image plate model from:
;*   Meadowcroft et al., RSI 79 (2008)
;********************************************************************

widget_control, MainWidget,get_uvalue=exraystr

filmtype=exraystr.wfilm.filmtype


if (exraystr.plotflag eq 4) then $
   data=exraystr.scaledwdf(exraystr.activedata) $
else $
   data=exraystr.datawdf(exraystr.activedata)

;Subtract the background
Error_Status=0
CATCH,Error_Status
if Error_status NE 0 THEN begin
    whoops=dialog_message('There is a problem subtracting the fog.'$
             +' Please verify the settings',/ERROR)
    CATCH,/CANCEL
    return
endif
if (exraystr.wfilm.fogflag eq 1) then $
   sub,data,exraystr.wfilm.fogwdf,103 $
else $
   sub,data,0,exraystr.wfilm.fog,103
w2i,lambda,PSL,103
CATCH,/CANCEL

if keyword_set(angcor) then begin
    ;calculate the angle of incidence associated with each ray
    widget_control,exraystr.widget.Graphics,GET_VALUE=winID
    autoflag=exraystr.autoflag
    exraystr.autoflag=1
    exray_buildptr,exraystr,temptr
    (*temptr).numsource=1
    exray_go,50,temptr,windID=winID,noplot=noplot,angle=ang
    exraystr.autoflag=autoflag
    w2i,xf,yf,50
    ang=interpol(ang,yf,lambda)
endif else ang=!pi/2.0

path=exraystr.projpath
if (filmtype eq 6) then begin
   ;TR type image plate
   readasc,path+'atomic_data\xsections\TR_IP_atten.txt',2,hnu,att,skip=2
   mj=[0.54,-5.0e-3,0.29,0.18] ;mPSL/keV
   cj=[0.02,3.19,-0.78,-1.21] ;mPSL
   thick=44.74 ;microns (actual thickness is 60 microns but TR_IP_atten file assumes 34% higher density)
endif
eta=1.-exp(-thick/att)
nhnu=n_elements(hnu)
psi=hnu
hnukev=hnu/1.e3
ind1=where(hnukev le 6.0)
ind2=where(hnukev gt 6.0 and hnukev le 13.5)
ind3=where(hnukev gt 13.5 and hnukev le 37.4)
ind4=where(hnukev gt 37.4 and hnukev le 100.)
if (ind1(0) ge 0) then $
   psi(ind1)=eta(ind1)*(mj(0)*hnukev(ind1)+cj(0)) ;mPSL/photon
if (ind2(0) ge 0) then $
   psi(ind2)=eta(ind2)*(mj(1)*hnukev(ind2)+cj(1))
if (ind3(0) ge 0) then $
   psi(ind3)=eta(ind3)*(mj(2)*hnukev(ind3)+cj(2))
if (ind4(0) ge 0) then $
   psi(ind4)=eta(ind4)*(mj(3)*hnukev(ind4)+cj(3))
     
lam=12398.53/hnu
psi2=psi(sort(lam))
lam=lam(sort(lam))
psii=interpol(psi2,lam,lambda)

;From Stephanie Hansen:
;I've been multiplying the mPSL/photon values below by [ 400 um^2/pixel ] * [1 PSL / 1000 mPSL] * 0.714 
;(this last factor to account for the decay in signal for a time > 100 minutes between exposure and readout). 
;I thus obtain a response curve in PSL * um^2 / photon. Dividing the measured PSL by the response curve gives intensities in photons/um^2.
psii=psii*400./1000.*0.714
spect_data=PSL/psii

if not keyword_set(noplot) then begin
   yrange=[min(spect_data),max(spect_data)]

   plot,lambda,spect_data,col=0,line=0,xrange=exraystr.xrange,yrange=yrange $
       ,xtitle='Wavelength (Angstroms)',pos=[0.15,0.1,0.95,0.957],xstyle=1,ystyle=1 $
       ,psym=0,ytitle='Intensity photons/um^2'

   wait,0.0
endif

if keyword_set(nosave) then begin
   ;store the result in the scratch wdf array 60
   i2w,lambda,spect_data,60,/xy
endif else begin
   if keyword_set(nocheck) then $
      check='Yes'$
   else $
      check=dialog_message('Is this acceptable?',/Question)
   if (check eq 'Yes') then begin
      ;permanently overwrite the active scaledwdf
      if (exraystr.plotflag eq 4) then $
         i2w,lambda,spect_data,exraystr.scaledwdf(exraystr.activedata),/xy $
      else $
         i2w,lambda,spect_data,exraystr.datawdf(exraystr.activedata),/xy
   endif
   if not keyword_set(noplot) then begin
      plotflag=exraystr.plotflag
      exraystr.plotflag = 0
      widget_control,MainWidget,set_uvalue=exraystr
      widget_control,exraystr.widget.plotlist,send_event=$
        { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
         HANDLER:MainWidget, INDEX:plotflag }
   endif
endelse

return
end

return
end
