PRO Exray_Convex_BuildPtr, exraystr, convexptr

wnumsource=exraystr.wnumsource
wnumxstals=exraystr.wnumxstals
wnumorders=exraystr.wnumorders
wnumlambda=exraystr.wnumlambda

if (exraystr.autoflag eq 1) then begin
   exraystr.lamset(0) = 0
   exraystr.serset(0) = 1
   exraystr.wsource(*).lambda(*,*)=0.
   exraystr.wxstal(*).beta(*,*)=0.
   if (exraystr.wfilm.circleflag ne 1) then begin
      ;a line detector
      phi=exraystr.wfilm.phi-exraystr.wxstal(0).chi
      mu=exraystr.wfilm.mu+!pi/2.-exraystr.wfilm.phi
      rf=exraystr.wfilm.rf
      rc=exraystr.wxstal(0).rc
      L=exraystr.wfilm.L
      yfplus=rf*sin(phi) + rc + L/2.*sin(mu)
      xfplus=rf*cos(phi) - L/2.*cos(mu)
      yfminus=rf*sin(phi) + rc - L/2.*sin(mu)
      xfminus=rf*cos(phi) + L/2.*sin(mu)
      exraystr.wxstal(*).beta(0,0)=!pi-atan((yfplus-rc)/xfplus)
      exraystr.wxstal(*).beta(0,1)=!pi-atan((yfminus-rc)/xfminus)
      exraystr.wxstal(*).beta(0,2)= (exraystr.wxstal(*).beta(0,1)-exraystr.wxstal(*).beta(0,0)) $
          /exraystr.wnumlambda
   endif else begin
      ind=where(exraystr.wsource(*).a ne 0)
      aa=exraystr.wsource(ind(0)).a
      bb=exraystr.wsource(ind(0)).b
      rc=exraystr.wxstal(0).rc
      chi=exraystr.wxstal(0).chi
      ap=sqrt(aa^2.+(rc-bb)^2.)*sin(atan(aa/(rc-bb))+chi)
      bp=sqrt(aa^2.+(rc-bb)^2.)*cos(atan(aa/(rc-bb))+chi)+rc
      ;maximum beta is tangential ray to crystal - 1 degrees
      betamax=!pi/2.+atan(abs(bp/ap))+acos(rc/sqrt(ap^2.+bp^2.))
      exraystr.wxstal(*).beta(0,0)=!pi*(1./2.+1./180.)
      ;exraystr.wxstal(*).beta(0,1)=!pi*(1.-1./180.)
      exraystr.wxstal(*).beta(0,1)=betamax-!pi/180.
      exraystr.wxstal(*).beta(0,2)= (exraystr.wxstal(*).beta(0,1)-exraystr.wxstal(*).beta(0,0)) $
          /exraystr.wnumlambda
      if (exraystr.LOSCFlag eq 2) then begin 
         ;CRITR transmission geometry
         exraystr.wxstal(*).beta(0,0)=exraystr.wxstal(*).beta(0,0)+3.*!pi/4.
         exraystr.wxstal(*).beta(0,1)=exraystr.wxstal(*).beta(0,1)+!pi/4.
      endif
   endelse
endif

;***************************************************************************
;Size the arrays
;***************************************************************************
numsource=long(0)
numxstals=long(0)
numorders=long(0)
numlambda=long(0)

for i=0,wnumsource-1 do $
   if (exraystr.sorset(i) eq 1) then numsource=numsource+1
for i=0,wnumxstals-1 do $
   if (exraystr.xstalset(i) eq 1) then numxstals=numxstals+1
for i=0,wnumorders-1 do begin
   if (exraystr.wsource(0).lambda(i,0) gt 0.) then begin
      numorders=numorders+1
      if (exraystr.serset(i) eq 0) then begin
         ;specific lambdas are given
         index=where(exraystr.wsource(0).lambda(i,*) gt 0.)
         if (n_elements(index) gt numlambda) then numlambda=n_elements(index)
      endif else begin
         ;a series of lambdas is given
         num=long((exraystr.wsource(0).lambda(i,1)-exraystr.wsource(0).lambda(i,0))/ $
                   exraystr.wsource(0).lambda(i,2))
         if (num gt numlambda) then numlambda=num
      endelse
   endif else if (exraystr.wxstal(0).beta(i,0) gt 0.) then begin
      numorders=numorders+1
      if (exraystr.serset(i) eq 0) then begin
         ;specific betas are given
         index=where(exraystr.wxstal(0).beta(i,*) gt 0.)
         if (n_elements(index) gt numlambda) then numlambda=n_elements(index)
      endif else begin
         ;a series of betas is given
         num=long((exraystr.wxstal(0).beta(i,1)-exraystr.wxstal(0).beta(i,0))/ $
                   exraystr.wxstal(0).beta(i,2))
         if (num gt numlambda) then numlambda=num
      endelse
   endif
endfor

;**********************************************************************
; Define the structures that will be used in EXRAY Computations
;**********************************************************************
source={NAME: 'SOURCE VARIABLES',$
        lambda:dblarr(numorders,numlambda),$ ;the wavelengths to calculate
        n:lonarr(numorders),$ ;the order number to consider
        a:dblarr(numxstals),$ ;the source x position is a function of coordinate rotation
        b:dblarr(numxstals)} ;the source y position is a function of coordinate rotation

source=replicate(source,numsource)

xstal={NAME: 'CRYSTAL VARIABLES',$
       twod:double(0),$ ;twice the crystal plane spacing
       theta:dblarr(numsource,numorders,numlambda),$ ;the bragg diffraction angles
       beta:dblarr(numorders,numlambda),$ ;the user-defined beta values
       xc:dblarr(numsource,numorders,numlambda),$ ;the x position where lambda hits the crystal
       yc:dblarr(numsource,numorders,numlambda),$ ;the y position where lambda hist the crystal
       rc:double(0),$ ;the crystal radius
       chi:double(0),$ ;the crystal rotation
       p:double(0.) } ;the lattice rotation

xstal=replicate(xstal,numxstals)

film={NAME: 'FILM VARIABLES',$
      rf:double(0),$ ;the radius of the film location
      phi:double(0),$ ;the rotation of the film location
      mu: double(0),$ ;the rotation of the film about its center
      L: double(0),$ ;the length of the film
      circleflag: long(0),$ ;flag to say if circular or linear film plane
      xlambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;x position where lambda hits film
      ylambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;y position where lambda hits film
      d: dblarr(numxstals,numsource,numorders,numlambda)} ;the position of lambda in film coordinates

;**********************************************************************
; Transfer Data to the new structures
;**********************************************************************

sorindex=long(-1)
for i=0,wnumsource-1 do begin
   if (exraystr.sorset(i) eq 1) then begin
      sorindex=sorindex+1
      source(sorindex).a=exraystr.wsource(i).a
      source(sorindex).b=exraystr.wsource(i).b
   endif
endfor
xindex=long(-1)
for i=0,wnumxstals-1 do begin
   if (exraystr.xstalset(i) eq 1) then begin
      xindex=xindex+1
      xstal(xindex).rc=exraystr.wxstal(i).rc
      xstal(xindex).chi=exraystr.wxstal(i).chi
      xstal(xindex).twod=exraystr.wxstal(i).twod
      xstal(xindex).p=exraystr.wxstal(i).p
   endif
endfor
ordindex=long(-1)
for i=0,wnumorders-1 do begin
   if (exraystr.wsource(0).lambda(i,0) gt 0.) then begin
      ordindex=ordindex+1
      source(*).n(ordindex)=exraystr.wsource(0).n(i)
      if (exraystr.serset(i) eq 0) then begin
         ;specific lambdas are given
         if numlambda gt wnumlambda then numlam = wnumlambda $
         else numlam=numlambda
         for j=0,numlam-1 do $
            source(*).lambda(ordindex,j)=exraystr.wsource(0).lambda(i,j)
      endif else begin
         ;a series of lambdas is given
         tmp1=exraystr.wsource(0).lambda(i,0)
         tmp2=exraystr.wsource(0).lambda(i,1)
         tmp3=exraystr.wsource(0).lambda(i,2)
         if tmp2 eq 0. then tmp2=tmp1+1.0
         if tmp3 eq 0. then tmp3=1.0
         nvalues=long((tmp2-tmp1)/tmp3)
         valarray=findgen(nvalues)*tmp3+tmp1
         for j=0,nvalues-1 do $
             source(*).lambda(ordindex,j)=valarray(j)
         if nvalues lt numlambda then $
            for j=nvalues,numlambda-1 do $
                source(*).lambda(ordindex,j)=0.
      endelse
   endif else if (exraystr.wxstal(0).beta(i,0) gt 0.) then begin
      ordindex=ordindex+1
      source(*).n(ordindex)=exraystr.wsource(0).n(i)
      if (exraystr.serset(i) eq 0) then begin
         ;specific betas are given
         if numlambda gt wnumlambda then numlam = wnumlambda $
         else numlam=numlambda
         for j=0,numlam-1 do $
            xstal(*).beta(ordindex,j)=exraystr.wxstal(0).beta(i,j)
      endif else begin
         ;a series of betas is given
         tmp1=exraystr.wxstal(0).beta(i,0)
         tmp2=exraystr.wxstal(0).beta(i,1)
         tmp3=exraystr.wxstal(0).beta(i,2)
         if tmp2 eq 0. then tmp2=tmp1+1.0
         if tmp3 eq 0. then tmp3=1.0
         nvalues=long((tmp2-tmp1)/tmp3)
         valarray=findgen(nvalues)*tmp3+tmp1
         for j=0,nvalues-1 do $
             xstal(*).beta(ordindex,j)=valarray(j)
         if nvalues lt numlambda then $
            for j=nvalues,numlambda-1 do $
                xstal(*).beta(ordindex,j)=0.
      endelse
   endif
endfor

   film.rf=exraystr.wfilm.rf
   film.phi=exraystr.wfilm.phi
   film.mu=exraystr.wfilm.mu
   film.L=exraystr.wfilm.L
   film.circleflag=exraystr.wfilm.circleflag

;************************************************************
; Calculate some dependent values to be used by many routines
;************************************************************

;the bragg angles
for l=0,numxstals-1 do begin
   twod=xstal(l).twod
   for i=0,numsource-1 do begin
      for j=0,numorders-1 do begin
          n=source(i).n(j)
          for k=0,numlambda-1 do begin
              lambda=source(i).lambda(j,k)
              p=xstal(l).p
              if (lambda gt 0) and (n*lambda le twod) then begin
                 xstal(l).theta(i,j,k) = asin(n*lambda/twod);+p
              endif else $
                 xstal(l).theta(i,j,k) = -1
          endfor
      endfor
   endfor
endfor

;the film parameter transformations to the xstal coordinate frame
film.phi=film.phi-exraystr.wxstal(0).chi
film.mu=film.mu+!pi/2.-film.phi

;ensure the y position of the crystal equals the crystal radius if Default has been specified
if (widget_info(exraystr.widget.DefaultC,/button_set) eq 1) then $
   source(0).b = exraystr.wxstal(0).rc

;the source position transformations:
R=dblarr(numxstals)
delta=dblarr(numxstals)
for i=0,numsource-1 do begin
   R(*)=sqrt(source(i).a(*)^2+(xstal(*).rc-source(i).b(*))^2)
   etap=dblarr(numxstals)
   for j=0,numxstals-1 do begin
      if (xstal(j).rc-source(i).b(j) ge 0) then $
         etap(j)=atan(abs(source(i).a(j))/(xstal(j).rc-source(i).b(j))) $
      else $
          etap(j)=atan(abs(source(i).a(j))/(xstal(j).rc-source(i).b(j)))+!pi
   endfor
   ;etap=atan(abs(source(i).a(*))/(xstal(*).rc-source(i).b(*)))
   delta(*)=!pi-etap-xstal(*).chi
   source(i).a(*)=-R(*)*sin(delta(*))
   source(i).b(*)=R(*)*cos(delta(*))+xstal(*).rc
endfor

;*****************************************************************
; Define the pointer which will be shared by EXRAY_CONVEX routines
;*****************************************************************

convexstr={source:source, $
          xstal:xstal, $
          film:film, $
          numorders:numorders, $
          numxstals:numxstals, $
          numsource:numsource,$
          numlambda:numlambda }

convexptr=ptr_new(convexstr,/no_copy)

return
end