PRO EXRAY_CONVEX_PLOTPREP,convexptr,xrange,yrange,over,coord,trans=trans,windID=winID

;*************************************************
; Intialize the plot window
;*************************************************
if (over eq 0) then begin
   if (n_elements(winID) eq 0) then begin
      window,0,xsize=650,ysize=450,title='Convex Crystal Ray Tracing'
      plot,[0,0],[-100,100],line=1,xran=xrange,yran=yrange,xtitle='X (cm)',ytitle='Y (cm)',$
                         xstyle=1,ystyle=1,position=[0.1,0.1,0.7,0.9667],charsize=1.2
   endif else begin
      wset,winID
      plot,[0,0],[-1000,1000],line=1,xran=xrange,yran=yrange,xtitle='X (cm)',ytitle='Y (cm)',$
                         xstyle=1,ystyle=1,position=[0.15,0.1,0.95,0.957],charsize=1.2
   endelse
   oplot,[-1000,1000],[0,0],line=1
   linelook=0
endif else linelook=1

;*************************************************
; Plot the Spectrometer opening and film limits
;*************************************************
if (coord eq 1) then begin
   ;opening for TIXTLs
   oplot,-4.5*[1.,1.],[-1.55/2.,-5.],thick=5,col=1
   oplot,-4.5*[1.,1.],[1.55/2.,10.],thick=5,col=1
   ;Film limits for TIXTLs
   thetaf1=!pi-asin(7.72/8.0)
   thetaf2=-asin(0.334/8.0)
   thetafs=findgen(1000)/999.*(thetaf1-thetaf2)+thetaf2
   oplot,8.0+thetafs*0.,thetafs,/polar,thick=2,col=1
endif else if (coord eq 2) then begin
   ;opening for PODD TIXTL PEAS
   oplot,-5.363*[1.,1.],[-(1.25/2.+0.125),-2.]*2.54,thick=5,col=1
   oplot,-5.363*[1.,1.],[1.25/2.-0.125,5.]*2.54,thick=5,col=1
   ;Film limits for PODD TIXTL PEAS
   thetaf1=!pi-atan(3.139/.841)
   thetaf2=atan(0.841/3.139)
   thetafs=findgen(1000)/999.*(thetaf1-thetaf2)+thetaf2
   oplot,3.205*2.54+thetafs*0.,thetafs,/polar,thick=2,col=1
endif else if (coord eq 3) then begin
   ;opening for CRITR
   oplot,-5.*[1.,1.],[-3.5/2.,-5.],thick=5,col=1
   oplot,-5.*[1.,1.],[3.5/2.,5.],thick=5,col=1
   ;Film limits for CRITR
   xfilm=(findgen(1000)*2.*12.3/1000)-12.3
   yfilm = -sqrt(12.4^2-xfilm^2)+12.6
   yfilm=yfilm-25.
   thetaf=atan(yfilm/xfilm)
   indf=where(xfilm lt 0)
   if (indf(0) ge 0) then $
         thetaf(indf)=!pi+atan(abs(yfilm(indf))/abs(xfilm(indf)))
   rfp=sqrt(xfilm^2.+yfilm^2.)
   xfilm=rfp*cos(thetaf+100.*!pi/180.)
   yfilm=rfp*sin(thetaf+100.*!pi/180.)
   xfmin=21.348
   xfmax=24.172
   ind=where((xfilm ge xfmin) and (xfilm le xfmax) and (yfilm gt 0))
   if (ind(0) ge 0) then begin
      xfilm=xfilm(ind)
      yfilm=yfilm(ind)
   end
   oplot,xfilm,yfilm,col=1,thick=2
endif


;*************************************************
; Plot the xstals
;*************************************************
numxstals=(*convexptr).numxstals
rc0=(*convexptr).xstal(0).rc
chi0=(*convexptr).xstal(0).chi
a0=(*convexptr).source(0).a(0)
b0=(*convexptr).source(0).b(0)
for l=0,numxstals-1 do begin
   rc=(*convexptr).xstal(l).rc
   chi=(*convexptr).xstal(l).chi
   delta_r=rc-rc0
   delta_chi=chi-chi0
   xxstal=(findgen(1000)*2.*rc/1000)-rc
   yxstal = sqrt(rc^2-xxstal^2)
   xstalcol=l
   if (coord eq 1 or coord eq 2) then begin
      ;The crystal size is limited to a chord-length of ~44 mm
      xclim=2.2
      ind=where((xxstal ge -1.*xclim) and (xxstal le xclim))
      if (ind(0) ge 0) then begin
         xxstal=xxstal(ind)
         yxstal=yxstal(ind)
      end
      ;transfer to LOS coordinate system
      yxstal=yxstal-rc0
      thetac=atan(yxstal/xxstal)
      indc=where(xxstal lt 0)
      if (indc(0) ge 0) then $
         thetac(indc)=!pi-atan(yxstal(indc)/abs(xxstal(indc)))
      rx=sqrt(xxstal^2.+yxstal^2.)
      xxstal=rx*cos(thetac+chi0)
      yxstal=rx*sin(thetac+chi0)
      xstalcol=1
   endif else if (coord eq 3) then begin
      ;The crystal size is limited to a arc-length of ~35.5 mm
      thetamin=(3.55/rc0)/2.
      xclim=rc*sin(thetamin)
      ind=where((xxstal ge -1.*xclim) and (xxstal le xclim))
      if (ind(0) ge 0) then begin
         xxstal=xxstal(ind)
         yxstal=yxstal(ind)
      end
      ;transfer to LOS coordinate system
      yxstal=yxstal-rc0
      thetac=atan(yxstal/xxstal)
      indc=where(xxstal lt 0)
      if (indc(0) ge 0) then $
         thetac(indc)=!pi-atan(yxstal(indc)/abs(xxstal(indc)))
      rx=sqrt(xxstal^2.+yxstal^2.)
      xxstal=rx*cos(thetac+chi0)
      yxstal=rx*sin(thetac+chi0)
      xstalcol=1
   endif
   oplot,xxstal,yxstal,col=xstalcol,thick=2,line=linelook
endfor

rf=(*convexptr).film.rf
chi=(*convexptr).xstal(0).chi
circleflag=(*convexptr).film.circleflag
;*************************************************
; Plot the film plane (if linear film)
;*************************************************
if (rf ge 0.) and (circleflag ne 1) then begin
   phi=(*convexptr).film.phi
   mu=(*convexptr).film.mu
   L=(*convexptr).film.L
   xf0=rf*cos(phi)
   yf0=rf*sin(phi) + rc0
   xfminus=xf0-L/2.*cos(mu)
   xfplus=xf0+L/2.*cos(mu)
   yfminus=yf0+L/2.*sin(mu)
   yfplus=yf0-L/2.*sin(mu)
   if (coord eq 0) then begin
   endif
   oplot,[xfminus,xfplus],[yfminus,yfplus],thick=2,line=linelook
endif else begin
;*************************************************
; Plot the film plane (if circular film)
;*************************************************
      ;transmission geometry - film is on the other side of the crystal and on roland circle
   xfilm=(findgen(1000)*2.*rf/1000)-rf
   yfilm = sqrt(rf^2-xfilm^2)
   if keyword_set(trans) then $ ;film is on Roland circle with top of film circle at crystal apex.
      yfilm=-yfilm+rc0-rf $
   else $
      yfilm=yfilm+rc0
   filmcol=0
   if (coord ne 0) then begin
      ;transfer to LOS coordinate system
      yfilm=yfilm-rc0
      thetaf=atan(yfilm/xfilm)
      indf=where(xfilm lt 0)
      if (indf(0) ge 0) then $
          if (coord eq 1 or coord eq 2) then $
            thetaf(indf)=!pi-atan(abs(yfilm(indf))/abs(xfilm(indf))) $
          else if (coord eq 3) then $
            thetaf(indf)=!pi+atan(abs(yfilm(indf))/abs(xfilm(indf)))
      rfp=sqrt(xfilm^2.+yfilm^2.)
      xfilm=rfp*cos(thetaf+chi0)
      yfilm=rfp*sin(thetaf+chi0)
      ;The film size is limited
      if (coord eq 1) then begin
         yfmin=-0.334
         xfmin=-sqrt(8.^2.-7.72^2.)
         ind=where((yfilm ge yfmin) and (xfilm ge xfmin))
      endif else if (coord eq 2) then begin
         xfmin=-0.841*2.54
         xfmax=3.319*2.54
         ind=where((xfilm ge xfmin) and (xfilm le xfmax))
      endif else if (coord eq 3) then begin
         xfmin=21.348
         xfmax=24.172
         ind=where((xfilm ge xfmin) and (xfilm le xfmax) and (yfilm gt 0))
      endif
      if (ind(0) ge 0) then begin
         xfilm=xfilm(ind)
         yfilm=yfilm(ind)
      end
      filmcol=1
   endif
   oplot,xfilm,yfilm,col=filmcol,thick=2,line=linelook
endelse


return
end