PRO EXRAY_FILMPOSITION,exrayptr,whichxstal,whichsource,whichorder,whichlambda

;Calculate where the ray hits the LINEAR or CIRCULAR film plane

rf=(*exrayptr).film.rf
phi=(*exrayptr).film.phi
mu=(*exrayptr).film.mu
L=(*exrayptr).film.L
theta=(*exrayptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)
xc=(*exrayptr).xstal(whichxstal).xc(whichsource,whichorder,whichlambda)
yc=(*exrayptr).xstal(whichxstal).yc(whichsource,whichorder,whichlambda)
a=(*exrayptr).source(whichsource).a
b=(*exrayptr).source(whichsource).b
betamin=(*exrayptr).xstal(whichxstal).betamin
betamax=(*exrayptr).xstal(whichxstal).betamax

if ((xc eq 0) and (yc eq 0)) then return ;the x-ray won't diffract off this crystal


if ((*exrayptr).film.Jflag eq 3) then begin
;**Circular film plane
   ;calculate coordinates where x-ray hits the film
   gamma=2.*theta-atan((yc-b)/(xc-a))
   tangam=tan(gamma)
   ac=1.+tangam^2.
   bc=2.*yc*tangam+2.*xc*tangam^2.
   cc=yc^2.+2.*yc*xc*tangam+xc^2.*tangam^2.-rf^2.
   if (gamma le !pi/2.) then $
      xlambda=(-bc+sqrt(bc^2.-4*ac*cc))/(2.*ac) $
   else $
      xlambda=(-bc-sqrt(bc^2.-4*ac*cc))/(2.*ac)
   ylambda=-sqrt(rf^2.-xlambda^2.)

   ;calculate minimum film position
   xmin=rf*cos(betamin)
   ymin=-rf*sin(betamin)

   ;calculate distance from end of film to x-ray
   alpha=acos((xmin*xlambda+ymin*ylambda)/rf^2.)
   dlambda=rf*alpha

   ;calculate angle at which the x-ray hits the film
   ad=sqrt(xc^2.+yc^2.)
   bd=rf
   cd=sqrt((xlambda-xc)^2.+(ylambda-yc)^2.)
   angle=acos((bd^2.+cd^2.-ad^2.)/(2.*bd*cd))
   if (abs(angle) lt !pi/180.) then angle=0.

endif else begin
;**Linear film plane

   gamma=2.*theta-atan((yc-b)/(xc-a))
   xf0=-rf*cos(phi)
   yf0=rf*sin(phi)
   yfplus=yf0+L/2.*sin(mu)

   xlambda=((yf0-yc)+xf0*tan(mu)-xc*tan(gamma))/(tan(mu)-tan(gamma))
   ylambda=yf0-tan(mu)*(xlambda-xf0)
   dlambda=(yfplus-ylambda)/sin(mu)


   ;The angle of incidence on the film is approximated assuming that all the rays
   ; pass through the cross-over and that the film plane is normal to the central ray
   ;angle=atan(abs(dlambda-L/2.)/rf) ;radians

   ;Angle of incidence for arbitrary source and arbitrary film position
   rayang=atan((xc-xlambda)/abs(yc-ylambda))
   filmnorm=3.*!pi/2.-phi-(mu-(phi-!pi/2.))
   angle=rayang+filmnorm

endelse

;Calculate the angle of the ray relative to the cross-over filter 
; (assuming cross-over filter is same orientation as film)
; (normal incidence = 0 degrees)
xangle=!pi/2.-(mu-gamma)

   (*exrayptr).film.d(whichxstal,whichsource,whichorder,whichlambda)=dlambda

(*exrayptr).film.xlambda(whichxstal,whichsource,whichorder,whichlambda)=xlambda
(*exrayptr).film.ylambda(whichxstal,whichsource,whichorder,whichlambda)=ylambda
(*exrayptr).film.angle(whichxstal,whichsource,whichorder,whichlambda)=angle
(*exrayptr).film.xangle(whichxstal,whichsource,whichorder,whichlambda)=xangle

return
end