FUNCTION polynom, x
   ;This function returns the result of a Nth order polynomial
   ;with coefficients c at the value x

   ;the coefficient vector is stored in wdf array 81
   w2i,dummy,c,81

   nc=n_elements(c)
   result=0.
   for i=0,nc-1 do begin
      result=result+c(i)*x^i
   endfor

   return, result

End

PRO exray_johann_filmpos, exrayptr,whichxstal,whichsource,whichorder,whichlambda,$
                          degree=order, nodisp=nodisp

  ;default to 5th order polynomial fit to the Johann film plane
  if (n_elements(order) eq 0) then order=5

  if (whichlambda eq 0) then begin
    wdfit,(*exrayptr).film.filmwdf,coef=c,degree=order,/double
    ;wdf array 80 contains the coefficients that describe the film plane
    i2w,findgen(order+1),c,80,/xy
  endif

  xc=(*exrayptr).xstal(whichxstal).xc(whichsource,whichorder,whichlambda)
  yc=(*exrayptr).xstal(whichxstal).yc(whichsource,whichorder,whichlambda)
  theta=(*exrayptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)
  a=(*exrayptr).source(whichsource).a
  b=(*exrayptr).source(whichsource).b

  ;calculate the angle of the propagating ray
  gamma=2.*theta-atan((yc-b)/(xc-a))

  ;create the coefficient vector and store it in wdf array 81
  w2i,dummy,c,80
  c2=c
  c2(0)=c(0)-yc-xc*tan(gamma)
  c2(1)=c(1)+tan(gamma)
  i2w,findgen(order+1),c2,81,/xy

  ;use the min, median, and max of the filmwdf for the initial guess
  w2i,xj,yj,(*exrayptr).film.filmwdf
  guess=[xj(0),total(xj)/(1.*n_elements(xj)),xj(n_elements(xj)-1)]

  ;find the root of the non-linear equation (x position of where the ray hits the film)
  root=fx_root(guess,'polynom',/double)

  xxover=root(0)
  yxover = yc - (xxover-xc)*tan(gamma)

if (nodisp eq 0) then begin
  ;calculate d by summing up along the curve up to the wavelength of interest
    w2i,xj,yj,(*exrayptr).film.filmwdf
    nxj=n_elements(xj)
    mxj=max(xj)
;    if (whichlambda eq 0) then begin
       ;use 10um spacing in x or a minimum of 10 points
       dx=abs(xxover-mxj)
       nx=long(max([dx*1000.,10]))
       if (xxover gt mxj) then begin
       ;ray is off end of film... d will be negative
          xx=mxj+findgen(nx)/(nx-1.)*dx
          yy=dblarr(nx)
          for i=0,n_elements(c)-1 do yy=yy+c(i)*xx^i
          d=-1.*total(sqrt((yy(1:nx-1)-yy(0:nx-2))^2+(xx(1:nx-1)-xx(0:nx-2))^2))
       endif else begin
       ;ray is to the left of the right end of the film... d is positive
          xx=mxj-findgen(nx)/(nx-1.)*dx
          yy=dblarr(nx)
          for i=0,n_elements(c)-1 do yy=yy+c(i)*xx^i
          d=total(sqrt((yy(1:nx-1)-yy(0:nx-2))^2+(xx(1:nx-1)-xx(0:nx-2))^2))
       endelse
;    endif else begin
;       lastd=(*exrayptr).film.d(whichxstal,whichsource,whichorder,whichlambda-1)
;       lastx=(*exrayptr).film.xxover(whichxstal,whichsource,whichorder,whichlambda-1)
       ;use 1um spacing in x or a minimum of 10 points
;       dx=abs(xxover-lastx)
;       nx=max([dx*10000.,10])
;       if (xxover lt (*exrayptr).film.xxover(whichxstal,whichsource,whichorder,whichlambda-1)) then begin
          ;ray is to the left of the last ray... d is larger than last d
;          xx=lastx-findgen(nx)/(nx-1)*dx
;          yy=dblarr(nx)
;          for i=0,n_elements(c)-1 do yy=yy+c(i)*xx^i
          ;d=lastd+total(sqrt((yy(1:nx-1)-yy(0:nx-2))^2+(xx(1:nx-1)-xx(0:nx-2))^2))
;          d=lastd
;          for i=0,nx-2 do d=d+sqrt((yy(i+1)-yy(i))^2+(xx(i+1)-xx(i))^2)
;       endif else begin
          ;ray is to the right of the last ray... d is smaller than last d
;          xx=lastx+findgen(nx)/(nx-1)*dx
;          yy=dblarr(nx)
;          for i=0,n_elements(c)-1 do yy=yy+c(i)*xx^i
          ;d=lastd-total(sqrt((yy(1:nx-1)-yy(0:nx-2))^2+(xx(1:nx-1)-xx(0:nx-2))^2))
;          d=lastd
;          for i=0,nx-2 do d=d-sqrt((yy(i+1)-yy(i))^2+(xx(i+1)-xx(i))^2)
;       endelse
;    endelse
;    print,d

   rayang=atan((xc-xxover)/abs(yc-yxover))
   xfp=xxover+10.e-4
   xfm=xxover-10.e-4
   yfp=0.
   yfm=0.
   for i=0,n_elements(c)-1 do begin
      yfp=yfp+c(i)*xfp^i
      yfm=yfm+c(i)*xfm^i
   endfor
   filmnorm=atan((yfp-yfm)/abs(xfp-xfm))
   angle=abs(rayang+filmnorm)

  (*exrayptr).film.angle(whichxstal,whichsource,whichorder,whichlambda)=angle
  (*exrayptr).film.d(whichxstal,whichsource,whichorder,whichlambda)=d
endif

  (*exrayptr).film.xxover(whichxstal,whichsource,whichorder,whichlambda)=xxover
  (*exrayptr).film.yxover(whichxstal,whichsource,whichorder,whichlambda)=yxover
  (*exrayptr).film.xlambda(whichxstal,whichsource,whichorder,whichlambda)=xxover
  (*exrayptr).film.ylambda(whichxstal,whichsource,whichorder,whichlambda)=yxover

  return
end