PRO exray_read,exrayptr,fname

line=' '
type=' '
numxstals=long(1)
numsource=long(1)

;if N_Elements(Error_Status) eq 0 then Error_Status=0
;CATCH,Error_Status

;if Error_status NE 0 THEN begin
;    print,'There is a problem with the input deck.'
;    print,'Please compare it with the example.'
;    CATCH,/CANCEL
;    retall
;endif

;**************************************************************
; Size the data structures
;**************************************************************
openr,u,fname,/get
  count=long(0)
  sized=long(0)
  while(not eof(u)) do begin
    readf,u,line
    if strpos(line,'number of elliptical') ne -1 then begin
      readf,u,numxstals
    endif
    if strpos(line,'number of source') ne -1 then begin
      readf,u,numsource
    endif

    if strpos(line,'start') ne -1 then begin

      count=count+1
      readf,u,line
      readf,u,line
      readf,u,line
      readf,u,line
      readf,u,line

      if strpos(line,'series') ne -1 then begin
        readf,u,v1
        readf,u,v2
        readf,u,v3
        if n_elements(v3) eq 0 or v3 eq 0.0 then v3=1.0
        lamda=findgen((v2-v1)/v3)*v3+v1
        sized=(n_elements(lamda)>sized)
      endif else begin
        d=strpos(line,'=')
        sized=long(strmid(line,d+1,strlen(line)-d)) > sized
      endelse

    endif
  endwhile
close,u
free_lun,u
numorders=count
numlambda=sized

;**********************************************************************
; Define the structures that will be used in EXRAY
;**********************************************************************
source={NAME: 'SOURCE VARIABLES',$
        lambda:dblarr(numorders,numlambda),$ ;the wavelengths to calculate
        n:lonarr(numorders),$ ;the order number to consider
        a:double(0),$ ;the source x position
        b:double(0)} ;the source y position

source=replicate(source,numsource)

xstal={NAME: 'CRYSTAL VARIABLES',$
       twod:double(0),$ ;twice the crystal plane spacing
       theta:dblarr(numsource,numorders,numlambda),$ ;the bragg diffraction angles
       beta:dblarr(numorders,numlambda),$ ;the user-defined beta values
       xc:dblarr(numsource,numorders,numlambda),$ ;the x position where lambda hits the crystal
       yc:dblarr(numsource,numorders,numlambda),$ ;the y position where lambda hist the crystal
       R0:double(0),$ ;the focal length of the crystal
       h:double(0),$ ;the henke h parameter
       ecc:double(0),$ ;the crystal eccentricity
       betamin:double(0),$ ;the minimum beta angle for the crystal
       betamax:double(0)} ;the maximum beta angle for the crystal

xstal=replicate(xstal,numxstals)

film={NAME: 'FILM VARIABLES',$
      rf:double(0),$ ;the radius of the film location
      phi:double(0),$ ;the rotation of the film location
      mu: double(0),$ ;the rotation of the film about its center
      L: double(0),$ ;the length of the film
      xlambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;x position where lambda hits film
      ylambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;y position where lambda hits film
      xxover:dblarr(numxstals,numsource,numorders,numlambda),$ ;x position of johann crossover curve
      yxover:dblarr(numxstals,numsource,numorders,numlambda),$ ;y position of johann crossover curve
      d: dblarr(numxstals,numsource,numorders,numlambda)} ;the position of lambda in film coordinates

;**********************************************************************
; Read in Data
;**********************************************************************
xstal(*).beta(*,*)=-1.  ;initialize beta to -1 as a flag for later
JohannFlag=0
count=0

openr,u,fname,/get
while(not eof(u)) do begin
  readf,u,line

  ;crystal plane spacing (2d)
  if strpos(line,'crystal plane') ne -1 then begin
     for i=0,numxstals-1 do begin
        readf,u,temp
        xstal(i).twod=double(temp)
     endfor
  endif

  ;crystal h parameter (h)
  if strpos(line,'henke') ne -1 then begin
     for i=0,numxstals-1 do begin
        readf,u,temp
        xstal(i).h=double(temp)
     endfor
  endif

  ;crystal focal length (R0)
  if strpos(line,'focal lengths') ne -1 then begin
     for i=0,numxstals-1 do begin
        readf,u,temp
        xstal(i).R0=double(temp)
     endfor
  endif

  ;crystal boundaries (Beta_min and Beta_max)
  if strpos(line,'crystal boundary') ne -1 then begin
     for i=0,numxstals-1 do begin
        readf,u,temp1,temp2
        xstal(i).betamin=double(temp1)*!pi/180.
        xstal(i).betamax=double(temp2)*!pi/180.
     endfor
  endif

  ;source positions (a,b)
  if strpos(line,'source position') ne -1 then begin
     for i=0,numsource-1 do begin
        readf,u,temp1,temp2
        source(i).a=double(temp1)
        source(i).b=double(temp2)
     endfor
  endif

  ;source wavelengths (lambda) and orders
  if strpos(line,'start wavelength') ne -1 then begin
    readf,u,line
    point_lun,-u,place
    readf,u,line
    ;which sources contain these wavelengths
    i=0
    j=0
    while (i ne -1) do begin
        i=strpos(line,',',i)
        if (i ne -1) then begin
           j = j+1
           i = i+1
        endif
    endwhile
    sourcearr=lonarr(j+1)
    point_lun,u,place
    readf,u,sourcearr
    readf,u,line

    ;get the order numbers
    readf,u,temp
    for i=0,j do begin
      source(sourcearr(i)-1).n(count)=long(temp)
    endfor

    readf,u,line
    if strpos(line,'series') ne -1 then begin
      readf,u,v1
      readf,u,v2
      readf,u,v3
      if n_elements(v3) eq 0 or v3 eq 0.0 then v3=1.0
      nvalues=long((v2-v1)/v3)
      valarray=(findgen((v2-v1)/v3)*v3+v1)
      for i=0,j do begin
         for k=0,nvalues-1 do $
            source(sourcearr(i)-1).lambda(count,k)=valarray(k)
         if nvalues lt numlambda then $
            for k=nvalues,numlambda-1 do $
               source(sourcearr(i)-1).lambda(count,k)=0.
      endfor
    endif else begin
      d=strpos(line,'=')
      sized=float(strmid(line,d+1,strlen(line)-d))
      for i=0,sized-1 do begin
        readf,u,temp
        for k=0,j do begin
           source(sourcearr(k)-1).lambda(count,i)=double(temp)
        endfor
      endfor
    endelse

    count=count+1
  endif


  ;xstal betas and orders
  if strpos(line,'start beta') ne -1 then begin
    readf,u,line
    point_lun,-u,place
    readf,u,line
    ;which xstals considered for these betas
    i=0
    j=0
    while (i ne -1) do begin
        i=strpos(line,',',i)
        if (i ne -1) then begin
           j = j+1
           i = i+1
        endif
    endwhile
    xstalarr=lonarr(j+1)
    point_lun,u,place
    readf,u,xstalarr
    readf,u,line

    ;get the order numbers
    readf,u,temp
    for i=0,numsource-1 do begin
      source(i).n(count)=long(temp)
    endfor

    readf,u,line
    if strpos(line,'series') ne -1 then begin
      readf,u,v1
      readf,u,v2
      readf,u,v3
      if n_elements(v3) eq 0 or v3 eq 0.0 then v3=1.0
      nvalues=long((v2-v1)/v3)
      valarray=(findgen((v2-v1)/v3)*v3+v1)*!pi/180.
      for i=0,j do begin
         for k=0,nvalues-1 do $
            xstal(xstalarr(i)-1).beta(count,k)=valarray(k)
         if nvalues lt numlambda then $
            for k=nvalues,numlambda-1 do $
               xstal(xstalarr(i)-1).beta(count,k)=0.
      endfor
    endif else begin
      d=strpos(line,'=')
      sized=float(strmid(line,d+1,strlen(line)-d))
      for i=0,sized-1 do begin
        readf,u,temp
        for k=0,j do begin
           xstal(xstalarr(k)-1).beta(count,i)=double(temp)*!pi/180.
        endfor
      endfor
    endelse

    count=count+1
  endif


  ;detector length (L)
  if strpos(line,'length of detector') ne -1 then begin
        readf,u,temp
        film.L=double(temp)
  endif

  ;detector positioning (rf,phi)
  if strpos(line,'detector position') ne -1 then begin
        point_lun,-u,place
        readf,u,line
        if strpos(line,'Johann line') ne -1 then begin
           ;the user has asked for a line detector located at
           ; the Johann crossover at beta_avg
           JohannFlag=1 ;a flag to signal a Johann line detector is desired
           if strpos(line,'beta =') ne -1 then begin
             ;the user has specified a beta_avg to use as the film rotation angle
              hold=strpos(line,'=')
              beta_avg=double(strmid(line,hold+1,strlen(line)-hold))*!pi/180.
           endif else if strpos(line,'theta =') ne -1 then begin
             ;the user has specified a theta to use as the mean ray
              hold=strpos(line,'=')
              theta_avg=double(strmid(line,hold+1,strlen(line)-hold))*!pi/180.
           endif
        endif else if strpos(line,'Johann locus') ne -1 then begin
           ;the user has asked for the detector to be the locus of
           ; points at each lambda's johann crossover point.
           ; Set rf to be -1. as a flag for future consideration.
           film.rf=-1.
           ;set phi to the global average beta + pi
           film.phi=(max(xstal(*).betamax)+min(xstal(*).betamin))/2.+!pi
        endif else begin
           point_lun,u,place
           readf,u,temp1,temp2
           film.rf=double(temp1)
           film.phi=double(temp2)*!pi/180.
        endelse
  endif


  ;detector rotation from normal (mu-phi+90)
  if strpos(line,'detector rotation') ne -1 then begin
        readf,u,temp
        film.mu=(temp+film.phi*180./!pi-90.)*!pi/180.
  endif

endwhile

close,u
free_lun,u

;************************************************************
; Calculate some dependent values to be used by many routines
;************************************************************
;the eccentricities
for i=0,numxstals-1 do begin
   h=xstal(i).h
   R0=xstal(i).R0
   xstal(i).ecc = sqrt(1+(h/R0)^2)-h/R0
endfor

;the bragg angles
for l=0,numxstals-1 do begin
   twod=xstal(l).twod
   for i=0,numsource-1 do begin
      for j=0,numorders-1 do begin
          n=source(i).n(j)
          for k=0,numlambda-1 do begin
              lambda=source(i).lambda(j,k)
              if (lambda gt 0) and (n*lambda le twod) then begin
                 xstal(l).theta(i,j,k) = asin(n*lambda/twod)
              endif else $
                 xstal(l).theta(i,j,k) = -1
          endfor
      endfor
   endfor
endfor


if (JohannFlag eq 1) then begin
   ;the user has asked for a Johann Line detector at the average theta position
   if ((n_elements(beta_avg) eq 0) and (n_elements(theta_avg) eq 0)) then begin
      ;the user has not specitified a beta_avg, so use the average beta
      ; for the entire substrate
      betamax=max(xstal(*).betamax)
      betamin=min(xstal(*).betamin)
      beta_avg=(betamax+betamin)/2.
   endif else if (n_elements(beta_avg) eq 0) then begin
      ;the user has specified a theta_avg, so calculate the corresponding beta
      ; assuming the source is at the source focus
      betavg_global=(max(xstal(*).betamax)+min(xstal(*).betamin))/2.
      diff = 100.
      for i=0,numxstals-1 do begin
         ;calculate the beta_avg for each crystal and then determine which
         ; crystal the theta_avg is defined for.  If defined for multiple
         ; crystals, then assume it to be the one closest to the center
         ecc=xstal(i).ecc
         betamax=xstal(i).betamax
         betamin=xstal(i).betamin
         betavg=theta_avg+acos(cos(theta_avg)/ecc)
         if (i eq 0) then beta_avg = betavg ;in case betavg not on any crystal
         if ((betavg lt betamax) and (betavg gt betamin)) then $
            if (abs(betavg_global-betavg) lt diff) then begin
               diff = abs(betavg_global-betavg)
               beta_avg = betavg
            endif
      endfor
   endif

   index = where((xstal(*).betamax gt beta_avg) $    ;find which crystal beta diffracts off
                 and (xstal(*).betamin lt beta_avg))
   if (index(0) lt 0) then index(0) = 0  ;beta doesn't exist for these crystals so use crystal 1
   h=xstal(index(0)).h
   R0=xstal(index(0)).R0
   ecc = sqrt(1.+(h/R0)^2)-h/R0
   thetaJ= atan((1.-ecc*cos(beta_avg))/(ecc*sin(beta_avg)))
   rcurve=h*(1.-2.*ecc*cos(beta_avg)+ecc^2.)^(3./2.)/(1.-ecc*cos(beta_avg))^3.
   rho=h/(1.-ecc*cos(beta_avg))
   film.rf=rcurve*sin(thetaJ)-rho
   xc_Bavg=-rho*cos(beta_avg)
   yc_Bavg=rho*sin(beta_avg)
   alpha=atan(yc_Bavg/(xc_Bavg+R0))
   film.phi=2.*thetaJ-alpha+!pi
   film.mu=film.mu+film.phi
   print,format='(A37,D6.3,A12,D7.3)',$
      'At the central Johann crossover, rf= ',film.rf,'cm and phi= ',film.phi*180./!pi
endif

;************************************************************
; Define the pointer which will be shared by EXRAY routines
;************************************************************

exraystr={source:source, $
          xstal:xstal, $
          film:film, $
          numorders:numorders, $
          numxstals:numxstals, $
          numsource:numsource,$
          numlambda:numlambda }

exrayptr=ptr_new(exraystr,/no_copy)


;CATCH,/CANCEL
RETURN
END
