pro Exray_RTprint_BASE_event, Event


  wWidget =  Event.top

  case Event.id of

    Widget_Info(wWidget, FIND_BY_UNAME='Save_BUTTON'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then begin
        Widget_Control, wWidget, get_uvalue = rtstr
        output=rtstr.output
        outfile = ''
        outfile = dialog_pickfile(title='Save As:', /WRITE,$
                            filter=['*.txt'])
        if (outfile ne '') then begin
           if (strpos(outfile,'.txt') eq -1) then begin
              root=outfile
              outfile=outfile+'.txt'
           endif else begin
              len=strlen(outfile)
              form='(A'+strtrim(string(len-4),2)+')'
              root=string(outfile,format=form)
           endelse
           xstalfile=root+'_xtal_coordinates.ufo'
           filmfile=root+'_film_coordinates.ufo'
           openw,u,outfile,/get
           for i=0,n_elements(output)-1 do printf,u,output(i)
           close,u
           ;print the crystal coordinates
           wufo,rtstr.xxstal,rtstr.yxstal,file=xstalfile
           if (rtstr.rf lt 0) then begin
              ;print the film coordinates
              wufo,rtstr.xjohann,rtstr.yjohann,file=filmfile
           endif
           free_lun,u
           widget_control, Event.top, /DESTROY
           return
        endif
      endif
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Done_BUTTON'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then begin
        ;quit without saving anything
        widget_control, Event.top, /DESTROY
        return
      endif
    end
    else:
  endcase


end


PRO EXRAY_RT,exrayptr,rot,LOS,alpham,R0m,xjohann,yjohann


;find the xstal parameters associated with the mean ray (betam)
numxstals=(*exrayptr).numxstals
betam=(*exrayptr).film.phi - !pi
betamax=(*exrayptr).xstal(*).betamax
betamin=(*exrayptr).xstal(*).betamin
index=where((betamax gt betam) and (betamin lt betam))
if (index(0) lt 0) then index(0)=0
eccm=(*exrayptr).xstal(index(0)).ecc
R0m=(*exrayptr).xstal(index(0)).R0
hm=(*exrayptr).xstal(index(0)).h

;calculate the film-limited beta angles
rf=(*exrayptr).film.rf
L = (*exrayptr).film.L
phi=(*exrayptr).film.phi
mu = (*exrayptr).film.mu
xf0=-rf*cos(phi)
yf0=rf*sin(phi)
betaminf=atan(abs((yf0+L/2.*sin(mu))/(xf0-L/2.*cos(mu))))
betamaxf=!pi-atan(abs((yf0-L/2.*sin(mu))/(xf0+L/2.*cos(mu))))


;Create a widget to display the TREX design features

Exray_RTprint_BASE = Widget_Base( GROUP_LEADER=wGroup,  $
      UNAME='Exray_RTprint_BASE' ,XOFFSET=5 ,YOFFSET=5 $
      ,TITLE='Configuration Features' ,SPACE=3 ,XPAD=3 ,YPAD=3 $
      ,COLUMN = 1)

Info_TEXT = Widget_Text(Exray_RTprint_BASE,xsize=80 $
            ,ysize=13+16*numxstals,font="Courier*8")

Quit_BASE = Widget_Base(Exray_RTprint_BASE, UNAME='Quit_BASE' $
      ,COLUMN=2,/ALIGN_RIGHT)

Done_BUTTON = Widget_Button(Quit_BASE,  $
      UNAME='Done_BUTTON' ,FRAME=1 ,XOFFSET=10  $
      ,YOFFSET=370 ,SCR_XSIZE=60 ,SCR_YSIZE=36 ,/ALIGN_CENTER  $
      ,VALUE='Done')


Save_BUTTON = Widget_Button(Quit_BASE,  $
      UNAME='Save_BUTTON' ,FRAME=1 ,XOFFSET=80  $
      ,YOFFSET=370 ,SCR_XSIZE=60 ,SCR_YSIZE=36 ,/ALIGN_CENTER  $
      ,VALUE='Save')

;print out the design parameters
output=strarr(13+18*numxstals)

output(0)='ellipse configuration features:'
output(1)='-------------------------------'
for l=0,numxstals-1 do begin
   h=(*exrayptr).xstal(l).h
   R0=(*exrayptr).xstal(l).R0
   ecc=(*exrayptr).xstal(l).ecc
   betamin=(*exrayptr).xstal(l).betamin
   betamax=(*exrayptr).xstal(l).betamax
   ;*********************************************
   ;calculate the ellipse curve coordinates
   xmin=-h*cos(betamin)/(1.-ecc*cos(betamin))
   ymin= h*sin(betamin)/(1.-ecc*cos(betamin))
   xmax=-h*cos(betamax)/(1.-ecc*cos(betamax))
   ymax= h*sin(betamax)/(1.-ecc*cos(betamax))
   xxstal=(xmax-xmin)*(findgen(1000)*.001)+xmin
   yxstal=sqrt(h^2/(1.-ecc^2)-(xxstal+R0/2.)^2*(1.-ecc^2))
   dist=sqrt((R0m+xxstal)^2+yxstal^2)
   xxstal=LOS-dist*cos(alpham+asin(-yxstal/dist))
   yxstal=dist*sin(alpham+asin(-yxstal/dist))
   ;**********************************************
 ;  if (betamin lt betaminf) then betamin=betaminf
 ;  if (betamax gt betamaxf) then betamax=betamaxf
   thetamin=atan((1-ecc*cos(betamin))/(ecc*sin(betamin)))
   thetamax=atan((1-ecc*cos(betamax))/(ecc*sin(betamax)))
   thetamind=thetamin*180./!pi
   thetamaxd=thetamax*180./!pi
   output(1+l*16+1)='xstal #'+strtrim(string(l+1),2)
   output(1+l*16+2)='h = '+strtrim(string(h,form='(F6.3)'),2)$
      +' cm ; '+strtrim(string(h/2.54,form='(F6.3)'),2)+' inches'
   output(1+l*16+3)='focal length = ' $
      +strtrim(string(R0,form='(F6.2)'))+' cm ; '+strtrim(string(R0/2.54,form='(F6.2)'))$
      +' inches'
   output(1+l*16+4)= 'eccentricity = ' $
      +strtrim(string(ecc,form='(F7.5)'))
   output(1+l*16+5)='beta range with this configuration = ' $
      +strtrim(string(betamin*180./!pi,form='(F6.2)'))+' - ' $
      +strtrim(string(betamax*180./!pi,form='(F6.2)'))+' degrees'
   output(1+l*16+6)='bragg angle range with this configuration = '$
      +strtrim(string(thetamind,form='(F6.2)'))+' - '+strtrim(string(thetamaxd,form='(F6.2)'))$
      +' degrees'
   output(1+l*16+7)='wavelength range with KAP = '$
      +strtrim(string(26.62*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(26.62*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+8)='wavelength range with KAP 2 = '$
      +strtrim(string(13.31*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(13.31*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+9)='wavelength range with MICA 1 = ' $
      +strtrim(string(20.0*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(20.0*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+10)='wavelength range with MICA 2 = ' $
      +strtrim(string(10.0*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(10.0*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+11)='wavelength range with MICA 3 = '$
      +strtrim(string(20.0/3.0*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(20.0/3.0*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+12)='wavelength range with PET = '$
      +strtrim(string(8.742*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(8.742*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+13)='wavelength range with ADP = '$
      +strtrim(string(10.66*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(10.66*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+14)='wavelength range with Quartz1011 = '$
      +strtrim(string(6.6862*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(6.6862*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+15)='wavelength range with Quartz1010 = '$
      +strtrim(string(8.5096*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(8.5096*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+16)='wavelength range with Ge111 = '$
      +strtrim(string(6.532*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(6.532*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+17)='wavelength range with Ge220 = '$
      +strtrim(string(4.000*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(4.000*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+18)='wavelength range with LiF200 = '$
      +strtrim(string(4.026*sin(thetamin),form='(F6.3)'))+' - ' $
      +strtrim(string(4.026*sin(thetamax),form='(F6.3)'))+' Angstroms'
   output(1+l*16+19)=$
    '---------------------------------------------------------------------'
endfor


output(1+numxstals*18+1)=''
output(1+numxstals*18+2)='fabrication specifications:'
output(1+numxstals*18+3)='---------------------------'
output(1+numxstals*18+4)='origin location at x = '$
   +strtrim(string(LOS/2.54,form='(F7.3)'))+', y =  0.000 inches'
;calculate the new ellipse origin
xshift=-R0m*cos(alpham)+LOS
yshift=R0m*sin(alpham)
output(1+numxstals*18+5)='ellipse focal point at x = '$
   +strtrim(string(xshift/2.54,form='(F7.3)'))+'  , y = '$
   +strtrim(string(yshift/2.54,form='(F7.3)'))+' inches'
if (rf gt 0) then begin
   if ((*exrayptr).film.Jflag eq 3) then begin
      ;print the film radius
      output(1+numxstals*18+6)='Film Radius from focal point = '$
         +strtrim(string(rf/2.54,form='(F7.3)'))+'  inches'
   endif else begin
      ;calculate the new MCP center
      dist=sqrt((R0m+xf0)^2+yf0^2)
      xf0=LOS-dist*cos(alpham+asin(-yf0/dist))
      yf0=dist*sin(alpham+asin(-yf0/dist))
      output(1+numxstals*18+6)='MCP center point at x = '$
         +strtrim(string(xf0/2.54,form='(F7.3)'))+'  , y = '$
         +strtrim(string(yf0/2.54,form='(F7.3)'))+' inches'
      output(1+numxstals*18+7)='MCPangle = '$
         +strtrim(string((alpham+betam)*180./!pi,form='(F7.3)'))+' degrees'
   endelse
endif else begin
   output(1+numxstals*18+6)='MCP center point at x = N/A    y = N/A'
   output(1+numxstals*18+7)='MCPangle = N/A'
endelse
output(1+numxstals*18+8)='overcutting for ellipse = 0.006 inches'
;calculate the clipping angle
;gamma_min=!pi/2.-betaminf-alpham
;rho_min=h/(1-eccm*cos(betaminf))
;ynew_min=-1.0*(rho_min*cos(gamma_min)-yshift)
;output(1+numxstals*16+9)='clipping half angle = '$
;   +strtrim(string((alpham+betam)*180./!pi,form='(F7.3)'))+'degrees'
betamax=max((*exrayptr).xstal(*).betamax)
betamin=min((*exrayptr).xstal(*).betamin)
output(1+numxstals*18+10)='beta angles subtended by crystal substrate = '$
   +strtrim(string(betamin*180./!pi,form='(F7.3)'))+' - '$
   +strtrim(string(betamax*180./!pi,form='(F7.3)'))+' degrees'
output(1+numxstals*18+11)=''

;send output to the text widget
for i=0,13+18*numxstals-1 do $
   widget_control,Info_TEXT,set_value=output(i),/append

;****** realize the widget and hand control to the event manager **********
  rtstr = {NAME: 'RT_Structure',$
              output:output,$
              xxstal:xxstal/2.54,$
              yxstal:yxstal/2.54,$
              xjohann:xjohann/2.54,$
              yjohann:yjohann/2.54,$
              rf:rf}


  Widget_Control, /REALIZE, Exray_RTprint_BASE

  Widget_Control, Exray_RTprint_BASE, set_uvalue = rtstr


  XManager, 'Exray_RTprint_BASE', Exray_RTprint_BASE,/No_Block

return
end


