PRO Exray_PlotControls, MainWidget, Event

;*********************************************************************************************
;Graphical User-Interface Controls
;*********************************************************************************************

Widget_Control,MainWidget,get_uvalue=exraystr
;get the "as realized" size of the graphics window
graphicsize=widget_info(exraystr.widget.graphics,/geometry)

   wdy = graphicsize.ysize   ;the window height
   pdy = 0.857  ;the plot height (in fraction of window height)
   pxmin = 0.15 ;the x position of the plot lower left corner (in fraction of window height)
   wdx= graphicsize.xsize     ;the window width
   pdx = 0.8    ;the plot width (in fraction of window height)
   pymin = 0.1  ;the y position of the plot lower left corner (in fraction of window height)

if ((Event.type eq 2) and (exraystr.pick eq 1)) then begin
;******* Plot a line at cursor position during reference picking ********
; this only works if the 'pick reference' button has been selected

  dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
  xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
  tv,exraystr.image,TRUE=1
  oplot,[xnow,xnow],exraystr.yrange,col=3,line=0
  legend2,['Reference Position = '+string(xnow,format='(F8.2)')+' microns'],$
    col=3,line=0,psym=0,pos=[0.15,0.98],/normal

endif else if ((Event.type eq 1) and (exraystr.pick eq 1)) then begin
;************ Pick a reference wavelength and position *******************
; this only works if the 'pick reference' button has been selected

  exraystr.pick=0
  widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=0
  dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
  dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)
  exraystr.xref=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
  w2i,xdata,ydata,exraystr.datawdf(exraystr.activedata)
  justify,exraystr.xref,xdata,xref,ydata,yintensity
  exraystr.xref=xref
  ytemp=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
  xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
  tv,exraystr.image,TRUE=1
  oplot,[xnow,xnow],exraystr.yrange,col=3,line=0
  oplot,[exraystr.xref,exraystr.xref],exraystr.yrange,col=1,line=0
  legend2,['Justified Reference Position = '+string(exraystr.xref,format='(F8.2)')+' microns'],$
    col=1,line=0,psym=0,pos=[0.1,0.98],/normal
  widget_control,MainWidget,set_uvalue=exraystr
  exray_lamref,MainWidget
  widget_control,MainWidget,get_uvalue=exraystr

endif else if (exraystr.trimflag eq 1) then begin
;******** Trim the active data set to the left or right of the line *************
; this only works if the 'trimdata' button has been selected and a data set is being viewed

   dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
   dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)

  if (Event.type eq 1) then begin
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=0
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      plots,[xnow,xnow],exraystr.yrange,line=0,col=3
      if (exraystr.plotflag eq 3) then $
         w2i,xtemp,ytemp,exraystr.datawdf(exraystr.activedata) $
      else if (exraystr.plotflag eq 4) then $
         w2i,xtemp,ytemp,exraystr.scaledwdf(exraystr.activedata)
      if (Event.modifiers eq 1) then begin
         ;On hold of shift key, trim data to the right of the trim line
         index = where(xtemp lt xnow)
      endif else begin
         ;Otherwise, trim data to the left of the trim line
         index = where(xtemp gt xnow)
      endelse
      if (index(0) ge 0) then begin
         xtemp=xtemp(index)
         ytemp=ytemp(index)
         ;wdf array 103 is the temporary holder of data
         i2w,xtemp,ytemp,103,/xy
         if (exraystr.plotflag eq 3) then $
            plo,103,col=0,line=0,xran=[min(xtemp),max(xtemp)],yran=[min(ytemp),max(ytemp)] $
             ,leg=-1,xtitle='Film Position (microns)',pos=[0.15,0.1,0.95,0.957],xstyle=1,ystyle=1 $
             ,psym=0 $
         else if (exraystr.plotflag eq 4) then $
            plo,103,col=0,line=0,xran=[min(xtemp),max(xtemp)],yran=[min(ytemp),max(ytemp)] $
             ,leg=-1,xtitle='Wavelength (Angstroms)',pos=[0.15,0.1,0.95,0.957],xstyle=1,ystyle=1 $
             ,psym=0
         check=dialog_message('Is this acceptable?',/Question)
         if (check eq 'Yes') then begin
            w2i,xtemp,ytemp,103
            ;permanently overwrite the active datawdf
            if (exraystr.plotflag eq 3) then $
               i2w,xtemp,ytemp,exraystr.datawdf(exraystr.activedata),/xy $
            else if (exraystr.plotflag eq 4) then $
               i2w,xtemp,ytemp,exraystr.scaledwdf(exraystr.activedata),/xy
         endif
         exraystr.plotflag = exraystr.plotflag - 1 ;force a rescale of the axis
         widget_control,exraystr.widget.plotlist,send_event=$
            { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
             HANDLER:MainWidget, INDEX:exraystr.plotflag + 1 }
      endif
      exraystr.trimflag =0
      widget_control,exraystr.widget.trimdata,set_value=exraystr.images.cut,/bitmap
   endif else if (Event.type eq 2) then begin
      ;plot the ruler line
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      tv,exraystr.image,TRUE=1 ;redraw the background image
      if (Event.modifiers eq 1) then begin
         legend2,['Remove all points > x = '+strtrim(string(xnow,format='(F9.3)'),2)], $
              col=3,line=-1,pos=[0.25,0.98],/normal
          plots,[exraystr.xrange(1),xnow],[exraystr.yrange(1),exraystr.yrange(0)],line=0,col=1
          plots,[exraystr.xrange(1),xnow],exraystr.yrange,line=0,col=1
      endif else begin
         legend2,['Remove all points < x = '+strtrim(string(xnow,format='(F9.3)'),2)], $
              col=3,line=-1,pos=[0.25,0.98],/normal
          plots,[exraystr.xrange(0),xnow],[exraystr.yrange(1),exraystr.yrange(0)],line=0,col=1
          plots,[exraystr.xrange(0),xnow],exraystr.yrange,line=0,col=1
      endelse

      plots,[xnow,xnow],exraystr.yrange,line=0,col=3
   endif

endif else if (exraystr.rulerflag eq 1) then begin
;******** Make a measurement between two points on screen *************
; this only works if the 'ruler' button has been selected
;   -This routine uses the xzoom and yzoom variables

   dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
   dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)

   if (Event.type eq 0) then begin
      exraystr.xzoom(0)=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      exraystr.yzoom(0)=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      plots,exraystr.xzoom(0),exraystr.yzoom(0),psym=2,col=3,symsize=1.4
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=1
      exraystr.rulerflag2=1
   endif else if (Event.type eq 1) then begin
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=0
      exraystr.xzoom(1)=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      exraystr.yzoom(1)=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      plots,exraystr.xzoom(1),exraystr.yzoom(1),psym=2,col=3,symsize=1.4
      exraystr.rulerflag =0
      exraystr.rulerflag2 =0
      widget_control,exraystr.widget.ruler,set_value=exraystr.images.ruler,/bitmap
   endif else if (Event.type eq 2) then begin
      ;plot the ruler line
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      tv,exraystr.image,TRUE=1 ;redraw the background image
      legend2,['('+strtrim(string(xnow,format='(F11.5)'),2)+', '$
           +strtrim(string(ynow,format='(F11.5)'),2)+')'], $
           col=3,line=-1,pos=[0.7,0.98],/normal
      if (exraystr.rulerflag2 eq 1) then $
;      if (exraystr.plotflag eq 1) or (exraystr.plotflag eq 2) then begin
      if (exraystr.plotflag ne 0) then begin
        if (Event.modifiers eq 1) then begin
         distance=abs(ynow-exraystr.yzoom(0))
         plots,exraystr.xzoom(0),exraystr.yzoom(0),psym=2,col=3,symsize=1.4
         plots,[exraystr.xzoom(0),exraystr.xzoom(0)],[exraystr.yzoom(0),ynow],$
               line=0,col=3,thick=2
         plots,exraystr.xrange,[exraystr.yzoom(0),exraystr.yzoom(0)],line=1,col=3
         plots,exraystr.xrange,[ynow,ynow],line=1,col=3
         ypoint=exraystr.yzoom(0)-0.025*(exraystr.yrange(1)-exraystr.yrange(0))
         if (exraystr.xzoom(0) gt (exraystr.xrange(1)+exraystr.xrange(0))/2.) then $
            xpoint=exraystr.xzoom(0)-0.4*(exraystr.xrange(1)-exraystr.xrange(0)) $
         else xpoint=exraystr.xzoom(0)
         legend2,['('+strtrim(string(exraystr.xzoom(0),format='(F11.5)'),2)+', ' $
              +strtrim(string(exraystr.yzoom(0),format='(F11.5)'),2)+')'], $
              col=3,line=-1,thick=2,pos=[xpoint,ypoint]
         legend2,['seperation = '+strtrim(string(distance,format='(F11.5)'),2)], $
               col=3,line=0,psym=2,pos=[0.15,0.98],/normal,thick=2
        endif else begin
         distance=abs(xnow-exraystr.xzoom(0))
         plots,exraystr.xzoom(0),exraystr.yzoom(0),psym=2,col=3,symsize=1.4
         plots,[exraystr.xzoom(0),xnow],[exraystr.yzoom(0),exraystr.yzoom(0)],$
               line=0,col=3,thick=2
         plots,[exraystr.xzoom(0),exraystr.xzoom(0)],exraystr.yrange,line=1,col=3
         plots,[xnow,xnow],exraystr.yrange,line=1,col=3
         ypoint=exraystr.yzoom(0)-0.025*(exraystr.yrange(1)-exraystr.yrange(0))
         if (exraystr.xzoom(0) gt (exraystr.xrange(1)+exraystr.xrange(0))/2.) then $
            xpoint=exraystr.xzoom(0)-0.4*(exraystr.xrange(1)-exraystr.xrange(0)) $
         else xpoint=exraystr.xzoom(0)
         legend2,['('+strtrim(string(exraystr.xzoom(0),format='(F11.5)'),2)+', ' $
              +strtrim(string(exraystr.yzoom(0),format='(F11.5)'),2)+')'], $
              col=3,line=-1,thick=2,pos=[xpoint,ypoint]
         legend2,['seperation = '+strtrim(string(distance,format='(F11.5)'),2)], $
               col=3,line=0,psym=2,pos=[0.15,0.98],/normal,thick=2
        endelse
      endif else begin
         distance=sqrt((xnow-exraystr.xzoom(0))^2 + $
                       (ynow-exraystr.yzoom(0))^2)
         plots,exraystr.xzoom(0),exraystr.yzoom(0),psym=2,col=3,symsize=1.4
         plots,[exraystr.xzoom(0),xnow],[exraystr.yzoom(0),ynow],$
               line=0,col=3,thick=2
         legend2,['distance = '+strtrim(string(distance,format='(F11.5)'),2)+' cm'], $
               col=3,line=0,psym=2,pos=[0.15,0.98],/normal,thick=2
      endelse
   endif

endif else if (Event.modifiers eq 1) then begin

;************** On Hold of Shift Key, Zoom the window ********************
; plot a box showing the zoom window

   dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
   dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)

   if (Event.type eq 0) then begin
      exraystr.xzoom(0)=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      exraystr.yzoom(0)=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=1
   endif else if (Event.type eq 1) then begin
      exraystr.xzoom(1)=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      if (exraystr.zoomsquare eq 1) then begin
      ;keep plot area square
         ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
         sign=(exraystr.yzoom(0)-ynow)/abs(exraystr.yzoom(0)-ynow)
         deltax=abs(exraystr.xzoom(1)-exraystr.xzoom(0))
         deltay=deltax*(exraystr.yrange(1)-exraystr.yrange(0))/ $
                       (exraystr.xrange(1)-exraystr.xrange(0))
         exraystr.yzoom(1)=exraystr.yzoom(0)-sign*deltay
      endif else begin
      ;arbitrary zoom area
         exraystr.yzoom(1)=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      endelse
      exraystr.xsave=exraystr.xrange
      exraystr.ysave=exraystr.yrange
      exraystr.xrange=[min(exraystr.xzoom),max(exraystr.xzoom)]
      exraystr.yrange=[min(exraystr.yzoom),max(exraystr.yzoom)]
      widget_control,MainWidget,set_uvalue=exraystr
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=0
      widget_control,exraystr.widget.plotlist,get_value=listnames
      if (exraystr.plotflag eq 5) then begin
         index=where(listnames eq 'Locus Points')
         ind=index(0)
      endif else ind=exraystr.plotflag
      widget_control,exraystr.widget.plotlist,send_event=$
        { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
        HANDLER:MainWidget, INDEX:ind }

   endif else if (Event.type eq 2) then begin
      ;plot the zoom box
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      if (exraystr.zoomsquare eq 1) then begin
      ;keep plot area square
         ytemp=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
         sign=(exraystr.yzoom(0)-ytemp)/abs(exraystr.yzoom(0)-ytemp)
         deltax=abs(xnow-exraystr.xzoom(0))
         deltay=deltax*(exraystr.yrange(1)-exraystr.yrange(0))/ $
                       (exraystr.xrange(1)-exraystr.xrange(0))
         ynow=exraystr.yzoom(0)-sign*deltay
      endif else begin
      ;arbitrary zoom area
         ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      endelse
     tv,exraystr.image,TRUE=1 ;redraw the background image
     plots,[exraystr.xzoom(0),xnow],[exraystr.yzoom(0),exraystr.yzoom(0)],$
            line=0,col=0
     plots,[exraystr.xzoom(0),xnow],[ynow,ynow],$
            line=0,col=0
     plots,[exraystr.xzoom(0),exraystr.xzoom(0)],[exraystr.yzoom(0),ynow],$
            line=0,col=0
     plots,[xnow,xnow],[exraystr.yzoom(0),ynow],$
            line=0,col=0
   endif

endif else if (exraystr.zoomflag ne 0) then begin
;*********** Zoom the screen about the cursor click *********************
; this only works if the 'zoom in' or 'zoom out' button has been selected
   dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
   dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)
   xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
   ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)

   if ((event.type eq 1) and (exraystr.zoomflag eq 1)) then begin
     ;Zoom Out
       exraystr.zoomflag = 0
       halfxrange=(exraystr.xrange(1)-exraystr.xrange(0))*exraystr.xzoomfac/2.
       halfyrange=(exraystr.yrange(1)-exraystr.yrange(0))*exraystr.yzoomfac/2.
       exraystr.xrange=[xnow-halfxrange,xnow+halfxrange]
       exraystr.yrange=[ynow-halfyrange,ynow+halfyrange]
      widget_control,exraystr.widget.zoomout,set_value=exraystr.images.zoomout,/bitmap
       widget_control,exraystr.widget.plotlist,send_event=$
          { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
            HANDLER:MainWidget, INDEX:exraystr.plotflag }

   endif else if (event.type eq 1) then begin
      ;Zoom In
        exraystr.zoomflag = 0
        halfxrange=(exraystr.xrange(1)-exraystr.xrange(0))/exraystr.xzoomfac/2.
        halfyrange=(exraystr.yrange(1)-exraystr.yrange(0))/exraystr.yzoomfac/2.
        exraystr.xrange=[xnow-halfxrange,xnow+halfxrange]
        exraystr.yrange=[ynow-halfyrange,ynow+halfyrange]
        widget_control,exraystr.widget.zoomin,set_value=exraystr.images.zoomin,/bitmap
        widget_control,exraystr.widget.plotlist,send_event=$
           { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
             HANDLER:MainWidget, INDEX:exraystr.plotflag }
   endif

endif else if ((exraystr.showtrans eq 1) and ((exraystr.plotflag eq 4) $
       or ((exraystr.plotflag eq 3) and (exraystr.xrange(1) lt 100.)))) then begin
;************** Snap cursor click to the nearest known transition ********************

   dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
   dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)

   if (Event.type eq 0) then begin
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=1
   endif else if (Event.type eq 1) then begin
      widget_control,exraystr.widget.Graphics,DRAW_MOTION_EVENTS=0
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      tv,exraystr.image,TRUE=1 ;redraw the background image
      exray_neartrans,xnow,exraystr,/dialog
   endif else if (Event.type eq 2) then begin
      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      tv,exraystr.image,TRUE=1 ;redraw the background image
      exray_neartrans,xnow,exraystr
   endif

endif else if (exraystr.pick eq 0) then begin
;************** On window click, Print Coordinates ********************
  if (Event.type eq 1) then begin
      dxdp=(exraystr.xrange(1)-exraystr.xrange(0))/(pdx*wdx)
      dydp=(exraystr.yrange(1)-exraystr.yrange(0))/(pdy*wdy)

      xnow=(event.x-(pxmin*wdx))*dxdp+exraystr.xrange(0)
      ynow=(event.y-(pymin*wdy))*dydp+exraystr.yrange(0)
      plots,xnow,ynow,psym=2,col=1
      if (exraystr.plotflag ne 2) then $
         xyouts,xnow,ynow,' ('+string(xnow,format='(F7.3)')+', '+$
      string(ynow,format='(F7.3)')+')' ,charsize=0.85 $
      else $
         xyouts,xnow,ynow,' ('+string(xnow,format='(F6.3)')+', '+$
      string(ynow,format='(F8.3)')+')',charsize=0.85
   endif
endif

Widget_Control,MainWidget,set_uvalue=exraystr

return
end
