PRO exray_eff, MainWidget, save=dosave, nocheck=nocheck, noplot=noplot, $
               noextract=noextract, image = image

widget_control, MainWidget,get_uvalue=exraystr

widget_control,exraystr.widget.Graphics,GET_VALUE=winID

filt1=exraystr.wfilm.filter(0)
filt2=exraystr.wfilm.filter(1)
filt3=exraystr.wfilm.filter(2)

ssiz = size(image)
lcimage = (ssiz[0] eq 2) ? 1 : 0

;retrieve the dataset
if (exraystr.plotflag eq 4) then $
   data=exraystr.scaledwdf(exraystr.activedata) $
else $
   data=exraystr.datawdf(exraystr.activedata)
   
w2i,lamdat,dat,data

;wdf array 60 holds the working data array
if (not keyword_set(noextract)) then begin
   if (dosave eq 0) then $
      xfr,data,60 $
   else begin
      i2w,lamdat,lamdat/lamdat,60,/xy
   endelse
endif

;*************************** filter corrections *********************************

if (exraystr.wfilm.filtflag eq 1) then begin
   ;calculate the correction due to the filter transmission

   ;open the pff file containing mass absorption curves
   pff=exraystr.projpath+'atomic_data'+exraystr.sepchar+'filter_material.pff'
   openpff,pff,fileid=fid

   firstflag=0
   for i=0,2 do begin
      ;retrieve the filter choices (as in the order given in effwidget)
      filtindex=exraystr.wfilm.filter(i)
      if (filtindex ne 0) then begin
         ;determine the index of the selected filter in the pff file
         case filtindex of
            1: begin  ;Aluminum
                index=2
                rho=2.6989
               end
            2: begin  ;Beryllium
                index=10
                rho=1.848
               end
            3: begin  ;Kapton
                index=115
                rho=1.42
               end
            4: begin  ;Kimfol
                index=117
                rho=1.20
               end
            5: begin  ;Mylar
                index=122
                rho=1.397
               end
            6: begin  ;Parylene N
                index=126
                rho=1.11
               end
            7: begin  ;Polystyrene
                index=131
                rho=1.056
               end
            8: begin   ;Tantalum
                index=83
                rho=16.654
               end
            9: begin   ;Titanium
                index=91
                rho=4.54
               end
           10: begin   ;Tungsten
                index=92
                rho=19.3
               end
         endcase

         ;get the absorption curve from the pff file (use wdf array 61 as work array)
         re,61,index,fileid=fid
         w2i,energy,sigma,61
         lambda=12398.53/energy ;angstroms
         sigma=sigma(sort(lambda))
         lambda=lambda(sort(lambda))
         mu=interpol(sigma*rho,lambda,lamdat) ;cm^-1
         
         ;*** elliptical geometry
         if (exraystr.convexflag eq 0 and exraystr.wfilm.xoverflag(i) eq 1) then begin
            ;calculate the angle through the xover
            exray_buildptr,exraystr,temptr
            (*temptr).numsource=1
            exray_go,50,temptr,windID=winID,noplot=noplot
            xangle=(*temptr).film.xangle(0,0,0,*)
            xangle=interpol(xangle,(*temptr).source.lambda(0,*),lamdat)
            thickness=exraystr.wfilm.filtwidth(i)/cos(xangle)*1.e-4 ;cm
      
         endif else begin
            thickness=exraystr.wfilm.filtwidth(i)*1.e-4 ;cm
         endelse
         
         ftrans=exp(-1.0*mu*thickness)
         i2w,lamdat,ftrans,62

         ;wdf array 63 holds the total filter transmission for all filters
         if (firstflag eq 0) then begin
            xfr,62,63
            firstflag=1
         endif else mul,63,62

      endif
   endfor
    closepff,fid

;calculate the transmission corrected data
    if lcimage then begin
      i2w, getxf(60), make_array(ssiz[1], value = 1.0), get_maxwdf()
      mul, get_maxwdf(), 63
      tmp = getyf(get_maxwdf())
      for j = 0, ssiz[2]-1 do image[*, j] /= tmp 
    endif else div,60,63
endif


;****** Xstal reflectivity and Spectrometer Efficiency Corrections ************************
if (exraystr.wfilm.xstalflag eq 1) then begin

   ;get the diffraction order
   n=exraystr.wsource(0).n(0)

  if (exraystr.wfilm.refflag ne 3) then begin
   if (exraystr.convexflag eq 0) then $ ;elliptical geometry
      whichXstal=widget_info(exraystr.widget.Xstals,/Droplist_Select) $
   else $ ;convex geometry
      whichXstal=widget_info(exraystr.widget.XstalC,/Droplist_Select)
   case whichXstal of
      1: begin   ;KAP
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'KAP_'+strtrim(string(n),2)+'.txt'
      end
      2: begin   ;Mica
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'Mica_'+strtrim(string(n),2)+'.txt'
      end
      3: begin   ;PET
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'PET_'+strtrim(string(n),2)+'.txt'
      end
      6: begin   ;LiF(220)
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'LiF220_'+strtrim(string(n),2)+'.txt'
      end
      7: begin   ;LiF(200)
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'LiF200_'+strtrim(string(n),2)+'.txt'
      end
   endcase

   readasc,refile,10,xt,yt
   index=sort(yt(*,8))           ;index for ascending wavelength order
   lambda=yt(index,8)            ;Angstroms
  endif

   ;Get the appropriate reflectivity (Radians)
   if (exraystr.wfilm.refflag eq 0) then $       ;Average reflectivity
      ref=(yt(index,2)+yt(index,3))*1.e-3/2. $
   else if (exraystr.wfilm.refflag eq 1) then $  ;Prins reflectivity
      ref=yt(index,2)*1.e-3 $
   else if (exraystr.wfilm.refflag eq 2) then $  ;Mosaic reflectivity
      ref=yt(index,3)*1.e-3 $
   else begin                                    ;Uniform reflectivity (1 mrad)
      w2i,lambda,ref,data
      ref=ref*0.+1.e-3
   endelse

   autoflag=exraystr.autoflag
   exraystr.autoflag=1

   ;*** elliptical geometry
   if (exraystr.convexflag eq 0) then begin

      ;calculate the theta associated with each beta
      exray_buildptr,exraystr,temptr
      (*temptr).numsource=1
      exray_go,50,temptr,windID=winID,noplot=noplot
      twod=(*temptr).xstal(0).twod
      n=(*temptr).source(0).n(0)
      betas=(*temptr).xstal(0).beta(0,*)
      thetas=(*temptr).xstal(0).theta(0,0,*)
      i2w,thetas,betas,51,/xy
      dif,51,52  ;wdf array 52 is d(Beta)/d(theta)
      sub,52,0,2.,53  ;wdf array 53 is -d(Chi)/d(theta)
      w2i,dummy,dBdt,52
      w2i,dummy,dCdt,53
      theta=asin(n*lambda/twod)
      index=where((theta ge thetas(0)) and (theta le thetas(n_elements(thetas)-1)))
      if (index(0) ge 0) then begin
         ;interpolate the reflectivity onto the larger thetas grid
         ref2=interpol(ref,theta,thetas)
      endif else begin
         whoops =dialog_message(['The reflectivity curve for this crystal does not have the proper domain.',$
                                 'Please check your spectrometer configuration.'],/Error)
         ptr_free,temptr
         return
      endelse
      lambda=twod/n*sin(thetas)

      xs=(*temptr).source(0).a
      ys=(*temptr).source(0).b
      xc=(*temptr).xstal(0).xc(0,0,*)
      yc=(*temptr).xstal(0).yc(0,0,*)
      xf=(*temptr).film.xlambda(0,0,0,*)
      yf=(*temptr).film.ylambda(0,0,0,*)

      ;the total x-ray path length from source to film
      L=dblarr(n_elements(thetas))
      L=sqrt((xc-xs)^2+(yc-ys)^2)+sqrt((xc-xf)^2+(yc-yf)^2)
      ;the distance from x-over to film
      rf=sqrt(xf^2+yf^2)

      ;Henke's efficiency correction
      S0_dNdA=1.602e-19*rf*L*12398.53*tan(thetas)/(dCdt/dBdt*lambda^2*ref2)  ;J/sr/Angstrom

      wait,0.0

  ;*** convex geometry
   endif else begin

      ;calculate the theta associated with each beta
      exray_convex_buildptr,exraystr,temptr
      (*temptr).numsource=1
      exray_convex_go,50,temptr,windID=winID,noplot=noplot

      dif,50,51  ;wdf array 51 holds the dispersion dlambda/dx
      w2i,xlam,dLdx,51

      twod=(*temptr).xstal(0).twod
      n=(*temptr).source(0).n(0)
      theta=asin(n*lambda/twod)
      thetas=(*temptr).xstal(0).theta(0,0,*)
      indt=where(thetas ne 0)
      lambda=twod/n*sin(thetas(indt))
      sorter=sort(lambda)
      lambda=lambda(sorter)
      thetas=thetas(sorter)
      index=where((theta ge thetas(0)) and (theta le thetas(n_elements(thetas)-1)))
      if (index(0) ge 0) then begin
         ;interpolate the reflectivity onto the larger thetas grid
         ref2=interpol(ref,theta,thetas)
      endif else begin
         whoops =dialog_message(['The reflectivity curve for this crystal does not have the proper domain.',$
                                 'Please check your spectrometer configuration.'],/Error)
         ptr_free,temptr
         return
      endelse

      xs=(*temptr).source(0).a
      ys=(*temptr).source(0).b
      xc=(*temptr).xstal(0).xc(0,0,sorter)
      yc=(*temptr).xstal(0).yc(0,0,sorter)
      xf=(*temptr).film.xlambda(0,0,0,sorter)
      yf=(*temptr).film.ylambda(0,0,0,sorter)
      rc=(*temptr).xstal(0).rc

      ;the total x-ray path length from source to film
      L=dblarr(n_elements(thetas))
      L(*)=sqrt((xc(*)-xs)^2+(yc(*)-ys)^2)+sqrt((xc-xf)^2+(yc-yf)^2)
      ;the distance from source to crystal axis
      s=dblarr(n_elements(thetas))
      s(*)=sqrt(xs^2+ys^2)
      ;geometry correction
      dGdt=dblarr(n_elements(thetas))
      dGdt(*)=1.-1./sqrt(1.+xs^2/(rc^2*(sin(thetas))^2))

      ;Brown's efficiency correction
      S0_dNdA=1.602e-19*s*L*12398.53/(dGdt*rc*sin(thetas)*lambda*ref2*dLdx)  ;J/sr/Angstrom

      ;Henke's efficiency correction
;      rf=sqrt((xc-xf)^2+(yc-yf)^2)
;      rf=(*temptr).film.rf
;      S0_dNdA=2.*1.602e-19*s*L*rf*12398.53/(rc*cos(thetas)*lambda^2*ref2)  ;J/sr/Angstrom

  endelse

   exraystr.autoflag=autoflag
   ptr_free,temptr

   ;calculate the source brightness in J/sr/cm^2/Angstrom
   i2w,lambda,S0_dNdA,61

  ;reconcile units
   units=exraystr.wfilm.units
   fudge =[1e8, 1e2, 1.0]
   fudge = fudge[units]
;   case units of
;     0: mul,60,0,1.e8         ;covert data from photons/um^2 to photons/cm^2
;     1: mul,60,0,1.e2         ;covert data from photons/mm^2 to photons/cm^2
;     2:
;   endcase


   ;if (dosave eq 0) then $
   if lcimage then begin
     i2w, getxf(60), make_array(ssiz[1], value = fudge), get_maxwdf()    
     mul, get_maxwdf(), 61  
     tmp = getyf(get_maxwdf())
     for j = 0, ssiz[2]-1 do image[*, j] *= tmp
   endif else begin
      mul,60,61             ;wdf array 60 holds the scaled data array
      if fudge gt 1.0 then mul, 60, 0, fudge
   endelse
   ;else $
   ;   div,61,63,60           ;wdf array 60 holds the correction factor
endif

;*************************** MCP photocathode corrections *********************************

if (exraystr.wfilm.mcpflag eq 1) then begin
    w2i,xdat,ydat,60

    ;calculate the angle of incidence associated with each ray (angle)
    widget_control,exraystr.widget.Graphics,GET_VALUE=winID
    autoflag=exraystr.autoflag
    exraystr.autoflag=1
    exray_buildptr,exraystr,temptr
    (*temptr).numsource=1
    exray_go,61,temptr,windID=winID,noplot=noplot,angle=ang
    exraystr.autoflag=autoflag
    w2i,xang,yang,61
    angle=!pi/2.-interpol(ang,yang,xdat)

    ;determine angle with respect to pore axis (eta)
    pang=exraystr.wfilm.poreangle
    eta=acos(cos(pang)*cos(angle))

    ;find MFP of SiO2 (rho = 2.2 g/cc) and determine Q
    exray_mcp_model,12398.53/xdat,180./!pi*eta,G,volts=400.,path=exraystr.projpath

    ;**NOTE**
    ;*G is the response (#photons out) per incident photon energy
    ;*we want the response per incident photon, so multiply by the photon energy
    ;*********
    Q=G*12398.53/xdat

    ;normalize Q
    Q=Q/max(Q)
    ;normalize Q by the value at the central ray
    ;index=where(ang eq max(ang))
    ;if (index(0) ge 0) then begin
    ;   mfp0=interpol(ymfp,xmfp,yang(index(0)))
    ;endif else mfp0=1.
    ;Q=tan(pang)*mfp0*Q
    if lcimage then for j = 0, ssiz[2]-1 do image[*, j] /= q else $
      i2w,xdat,ydat/Q,60,/xy
endif

;****************** Plot it and ask if user is satisfied ***********************
w2i,lamda,spect_data,60

if (dosave eq 0) then begin
  jump1: wait,0.0
  yrange=[min(spect_data),max(spect_data)]

  if not(keyword_set(noplot)) then $
  plot,lamda,spect_data,col=0,line=0,xrange=exraystr.xrange,yrange=yrange $
       ,xtitle='Wavelength (Angstroms)',pos=[0.15,0.1,0.95,0.957],xstyle=1,ystyle=1 $
       ,psym=0,ytitle='Intensity photons/um^2'

  wait,0.0
  if (not keyword_set(nocheck)) then $
     check=dialog_message('Is this acceptable?',/Question) $
  else check = 'Yes'
  if (check eq 'Yes') then begin
     ;permanently overwrite the active scaledwdf
     if (exraystr.plotflag eq 4) then $
        i2w,lamda,spect_data,exraystr.scaledwdf(exraystr.activedata),/xy $
     else $
        i2w,lamda,spect_data,exraystr.datawdf(exraystr.activedata),/xy
  endif
endif else begin
   ;save the correction factor
   nametemp=''
   nametemp = dialog_pickfile(title='Save As:', /WRITE,$
                        filter='*.ufo',path=exraystr.path)
   if (nametemp eq '') then return $
   else if (strpos(nametemp,'.ufo') eq -1) then nametemp=nametemp+'.ufo'
   wufo,lamda,spect_data,file=nametemp
   check=dialog_message('Would you like to correct the data?',/Question)
      if (check eq 'Yes') then begin
         xfr,data,61
         mul,60,61
         w2i,lamda,spect_data,60
         goto,jump1
      endif
endelse

if (not keyword_set(noplot)) then begin
   plotflag=exraystr.plotflag
   exraystr.plotflag = 0
   widget_control,MainWidget,set_uvalue=exraystr
   widget_control,exraystr.widget.plotlist,send_event=$
      { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
         HANDLER:MainWidget, INDEX:plotflag }
endif

return
end
