PRO exray_mcp_model,hnu,angle,G,volts=volts,G0=G0,path=path

els=[ 'O',  'Si',  'C',   'Pb',   'Zn',   'Cs',   'Ba',   'N' ]
fa= [ 62.,   25.,   7.,    3.,     2.,    0.7,    0.5,    0.3 ] /100.
A=  [ 16.,  28.1,  12.0,  207.2,  65.4,  132.9,  137.3,   14.0]

if (n_elements(volts) eq 0) then begin
   V_V0=400./375.    ;Ratio of DC voltage to unity gain voltage
endif else begin
   V_V0=volts(0)/375.
endelse
rho = 2.9  ;density of MCP material
L=460.     ;Length of MCP pore
D=10.0     ;Diameter of MCP pore
dw=2.0     ;MCP wall thickness
ang2=8.0     ;MCP bias angle
n=32        ;number of pores to calculate for
fast=5.3e-5 ;ratio of primary to secondary electrons

nels=n_elements(els)
for i=0,nels-1 do begin
   readasc,path+'atomic_data\xsections\'+els(i)+'_atten.txt',2,x,y,skip=2
   i2w,x,1./y,61+i,/xy
   cha,61+i,clab=els(i)
endfor

fm=fa*A/total(fa*A)
;fm=fa
;print,fm
for i=0,nels-1 do mul,61+i,0,fm(i)
add,61,62,69
for i=3,nels do add,69,60+i

;compute mean free path
w2i,xx,yy,69
x=hnu
y=interpol(yy,xx,hnu)
sigma=y*1.e4  ;mass absorption in cm^2/g
mfp=1./(sigma*rho)  ;mean free path in cm
mfp=mfp*1.e4      ;mean free path in microns

;compute the front and back surface yields
ang=angle*!pi/180.    ;incident angle in radians
ang2=ang2*!pi/180.    ;pore angle in radians
t=dw/sin(ang)       ;Path length through MCP wall
dxf=mfp*cos(ang)    ;delta x for birth of front yield electrons
dxb=dxf

;loop over photon energy to compute the relative gain at each energy
nhv=n_elements(x)
G=dblarr(nhv)
G0=dblarr(nhv)   ;the signal from channel 0
G0f=dblarr(nhv)  ;the signal from front surface of channel 0
G0b=dblarr(nhv)  ;the signal from back surface of channel 0
Gf=dblarr(nhv)  ;the total front surface signal
Gb=dblarr(nhv)  ;the total back surface signal
y0=dblarr(nhv)  ;the yield from channel wall n=0
yft=dblarr(nhv)  ;the total front surface yield
ybt=dblarr(nhv)  ;the total back surface yield
yst=dblarr(nhv)  ;total secondary yield
ypt=dblarr(nhv)  ;total primary yield
in=indgen(n)+1
for i=0,nhv-1 do begin ;loop over photon energy
   ;where incident rays enter pore
   DD=findgen(10)/9.*(D-0.5*dw*tan(ang(i)))+0.5*dw*tan(ang(i))
   ;DD=findgen(10)*0.+D/2.
for j=0,9 do begin ;loop over where incident ray enters pore
   ;front yield
   yf=dblarr(n)
   yf=((1-fast)*y(i)+fast*x(i)^0.88*y(i))*exp(-1.*t(i)/mfp(i)*(in-1))
   ;back yield
   yb=dblarr(n)
   yb=((1-fast)*y(i)+fast*x(i)^0.88*y(i))*exp(-1.*t(i)/mfp(i))*exp(-1.*t(i)/mfp(i)*(in-1))
   ;front birth depth
   xf=dblarr(n)
   xf=(DD(j)/2.+(in-1)*(D+dw))/tan(ang(i))+dxf(i)
   ;back birth depth
   xb=dblarr(n)
   xb=xf

   y0(i)=((1-fast)*y(i)+fast*x(i)^0.88*y(i))
   yft(i)=total(yf)
   ybt(i)=total(yb)
   yst(i)=total((1-fast)*y(i)*exp(-1.*t(i)/mfp(i)*(in-1)))
   ypt(i)=total(fast*x(i)^0.88*y(i)*exp(-1.*t(i)/mfp(i)*(in-1)))


   ;compute the Gain
   G0f(i)=yf(0)*V_V0^(0.25/D*(L-xf(0)))
   G0b(i)=yb(0)*V_V0^(0.25/D*(L-xb(0)))
   Gf(i)=total(yf*V_V0^(0.25/D*(L-xf)))
   Gb(i)=total(yb*V_V0^(0.25/D*(L-xb)))
   G0(i)=G0(i)+G0f(i)+G0b(i)
   G(i)=G(i)+Gf(i)+Gb(i)
endfor
endfor
G=G/tan(ang)
G0=G0/tan(ang)

return
end