;
;+
; NAME:			write_ufo_111.pro
; PURPOSE:
; 	IDL routine to write in a simple x,y file. UFO format #111 assumed.
;	 Format is assumed to consist of a 5 line header, followed by
;	 N lines of x,y data pairs, concluded with a -999,-999 data pair.
; CATEGORY:	I/O, UFO-like file
; CALLING SEQUENCE:	write_ufo_111,x,y [, filename=filename ]
; INPUTS:		x,y : data arrays
; KEYWORDS:
;	filename : optional filename to read from. If not specified, user is
;		   prompted for one.
; Unix KEYWORDS:	none
; VMS KEYWORDS:		none
; OUTPUTS:		none
; COMMON BLOCKS:	none
; SIDE EFFECTS:		filename is prompted for and written
; RESTRICTIONS:		none
; MODIFICATION HISTORY:
;	Adapted WENelson's wwrite routine, done by Alex Filuk, 9/18/91.
;	NOTE: this routine is to be kept exactly the same as wufo.pro, which
;	is simply a shorter named version (for typing convenience)
;	A.B. Filuk (11/6/92) - removed initial whitespace at start of first
;		four lines of header, to make format COMPLETELY UFO 111
;		compatible!
;	A.B. Filuk (1/13/93) - added free_lun to free logical unit!
;-

 PRO wufo, xvalues, yvalues, filename=filename

on_error, 2			;return to caller on error
num_pts = n_elements(xvalues)
IF n_elements(yvalues) ne num_pts THEN BEGIN
  print,' WRITE_UFO_111: x and y do not have same dimensions.'
  GOTO, bottom
ENDIF
num_hdr_lines = 5
fmt_str = '(2x,G22.14,2x,G22.14)'
if not keyword_set(filename) then begin
	filename = ''
	READ,' Enter filename ',filename
	endif
openw, unit, filename, /get_lun

header = strarr(num_hdr_lines)
header(0) = '111 = FORMAT #'
header(1) = '2 = # TABLE COLUMNS'
header(2) = '1  1 = ITYPE, IORDER'
header(3) = '0.00E+00  0.00E+00 = ABSTOL, RELTOL'
header(4) = '             X             Y'

; first write the header stuff
FOR i=0,num_hdr_lines-1 DO  $
  printf, unit, header(i)

; now write data pairs
FOR i=0,num_pts-1 DO  $
  printf, unit, FORMAT=fmt_str,xvalues(i),yvalues(i)

printf, unit, FORMAT='(2x,G22.14,2x,G22.14)',-999.,-999.

close, unit
free_lun, unit

bottom:
return
end
