PRO Exray_AutoScale,exraystr,doneOK,noplot=noplot,fit=fit

   ;***note:
   ;   only the first active source and the specified diffraction order
   ;   for that source are considered for the data scaling

widget_control,/hourglass
active=exraystr.activedata ;the current data set

tempstr=replicate(exraystr,1)
whatsource=where(tempstr.sorset gt 0)

if (n_elements(whatsource) gt 1) then $
   whoops=dialog_message('Only the first active source location will '$
                        +'be used for the dispersion calculation.')

tempstr.sorset(*)=0
if (whatsource(0) lt 0) then begin
   whoops=dialog_message('An error occured during data scale...' $
           +'There is not a valid source position',/ERROR)
   doneOK=0
   return
endif
tempstr.xstalset(*)=0
tempstr.xstalset(0)=1
tempstr.sorset(whatsource(0))=1

if (exraystr.fitflag ne 2) then begin
   tempstr.autoflag=0
   tempstr.lamset(*)=0
   tempstr.lamset(1)=1
   tempstr.wsource(*).lambda(*,*)=0.
   tempstr.wsource(*).n(*)=tempstr.wsource(whatsource(0)).n(0)
   tempstr.wsource(*).lambda(1,0)=tempstr.lamref
   tempstr.serset(*)=0
   tempstr.serset(0)=1
   if (exraystr.fitflag eq 0) then begin
   ;automatic beta range
      tempstr.wxstal(*).beta(0,0)=tempstr.wxstal(0).betamin
      tempstr.wxstal(*).beta(0,1)=tempstr.wxstal(0).betamax
      tempstr.wxstal(*).beta(0,2)= $
         (tempstr.wxstal(0).betamax-tempstr.wxstal(0).betamin)/tempstr.wnumlambda
   endif else begin
   ;specified wavelength range
      tempstr.lamset(0)=1
      tempstr.wsource(*).lambda(0,0)=tempstr.minwave
      tempstr.wsource(*).lambda(0,1)=tempstr.maxwave
      tempstr.wsource(*).lambda(0,2)=tempstr.intwave
   endelse
endif else begin
   ;use wavelength/beta group 1 for scaling range
   tempstr.lamset(1)=1
   tempstr.wsource(*).lambda(1,*)=0.
   tempstr.wsource(*).n(*)=tempstr.wsource(whatsource(0)).n(0)
   tempstr.wsource(*).lambda(1,0)=tempstr.lamref
endelse
exray_buildptr,tempstr,temptr
;xstalnum=0
;for i=0,tempstr.wnumxstals-1 do $
;if (tempstr.xstalset(i) eq 1) then $
;xstalnum=xstalnum+1
;wdfa=findgen(xstalnum)+1
widget_control,exraystr.widget.Graphics,GET_VALUE=winID
exray_go,80,temptr,windID=winID,/noprint,noplot=noplot
wait,1.5
w2i,xdat,ydat,exraystr.datawdf(active)
w2i,xdisp,ydisp,80
index=where(ydisp eq tempstr.lamref)
if (index(0) lt 0) then begin
   whoops=dialog_message('An error occured during data scale...' $
           +'please check reference wavelength and order',/ERROR)
   doneOK = 0
   delwdf,80
   return
endif

xdisp=xdisp*10000 ;convert from cm to um

xdisp=xdisp-(xdisp(index(0))-tempstr.xref)

index2=where((xdisp ge 0.) and (xdisp le tempstr.wfilm.L*10000))
if (index2(0) lt 0) then begin
   whoops=dialog_message('An error occured during data scale...' $
           +'please check reference wavelength and order',/ERROR)
   doneOK = 0
   delwdf,80
   return
endif
xdisp=xdisp(index2)
ydisp=ydisp(index2)

fit=poly_fit(xdisp,ydisp,exraystr.fitorder,chisq=chi2,status=state) ;fit a nth order polynomial to dispersion
warning=''
if (state ne 0) then begin
  case state of
  1: begin
     whoops=dialog_message('An error occured during polynomial fit',/Error)
     doneOK=0
     return
  end
  2: warning=['WARNING:  A small pivot element was used during polynomial fit.',$
               'Accuracy of fit may be poor.']
  3: warning=['WARNING:  Undefined error estimate was encountered.',$
               'Accuracy of fit may be poor.']
  endcase
endif
x=(max(xdisp)-min(xdisp))/10000.*findgen(10000) + min(xdisp)
y=poly(fit, x)

if not keyword_set(noplot) then begin
   plot,xdisp,ydisp,line=0,col=0,xtitle='Film Position',ytitle='Wavelength',$
        pos=[0.1,0.1,0.9,0.9],xstyle=1,ystyle=1
   oplot,x,y,line=0,col=1
   legend2,['Calculated Dispersion','Polynomial Fit'],line=[0,0],col=[0,1],psym=0
   verify='OK'
   if ((chi2 gt 0.1) or (warning ne '')) then $
     verify=dialog_message(['Chi-Squared of fit = '+strtrim(string(chi2,format='(F9.3)'),2),$
                          warning,'Click "OK" to continue'],/INFORMATION,/Cancel) $
   else wait,2.0
   if (verify eq 'Cancel') then begin
      doneOK = 0
      return
   endif
endif

lamscale= poly(fit, xdat)
;now shift the result so that the reference wavelength is in the same spot
index3=where(xdat eq tempstr.xref)
if (index3(0) lt 0) then index3=where(xdat gt 0.9999*tempstr.xref and xdat lt 1.0001*tempstr.xref)

;scale the reference wavelength if it is in a different order than the dispersion calculation
lamref=tempstr.lamref*tempstr.orderef/tempstr.wsource(whatsource(0)).n(0)

lamscale=lamscale-(lamscale[index3[0]]-lamref)
i2w,lamscale,ydat,exraystr.scaledwdf(active),/xy
doneOK=1

exraystr.scaleflag(exraystr.activedata) = 1

delwdf,80
end
