pro exray_betacalc,exrayptr,whichxstal,whichsource,whichorder,whichlambda,$
                   limit,parabolic,ignore

  betamin=(*exrayptr).xstal(whichxstal).betamin
  betamax=(*exrayptr).xstal(whichxstal).betamax
  h=(*exrayptr).xstal(whichxstal).h
  ecc=(*exrayptr).xstal(whichxstal).ecc
  a=(*exrayptr).source(whichsource).a
  b=(*exrayptr).source(whichsource).b
  theta=(*exrayptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)

  ; **************************************************************
  ; ***  this part calculates where the ray strikes            ***
  ; ***  the crystal.  It uses a Secant iterative solver       ***
  ; ***  (similar to a bisection solver) to find the point.    ***
  ; ***  It should also be noted that there are 4 solutions    ***
  ; ***  Applied Numerical Analysis 5th ed. Gerald and Wheatly pp 35
  ; **************************************************************
  tol = 1.0d-10
  t0  = betamin
  t1  = betamax
  err = 1.0e6
  check = 0
  Px = a
  Py = b

  While(tol LT abs(err) and check le 100) do begin
      x =-h*cos(t0)/(1.0-ecc*cos(t0))
      y = h*sin(t0)/(1.0-ecc*cos(t0))
      if (parabolic eq 0) then begin
         ;the xstal is elliptical
         dydx = -(ecc-cos(t0))/sin(t0)
      endif else begin
         ;the xstal is parabolic
         dydx = -h/y
      endelse
      LHS = (x-Px)+(y-Py)*dydx
      RHS = SQRT((x-Px)^2 +(y-Py)^2)*SQRT(1+dydx^2) * cos(theta)
      f0 = LHS-RHS

      x =-h*cos(t1)/(1.0-ecc*cos(t1))
      y = h*sin(t1)/(1.0-ecc*cos(t1))
      if (parabolic eq 0) then begin
         ;the xstal is elliptical
         dydx = -(ecc-cos(t1))/sin(t1)
      endif else begin
         ;the xstal is parabolic
         dydx = -h/y
      endelse
      LHS = (x-Px)+(y-Py)*dydx
      RHS = SQRT((x-Px)^2 +(y-Py)^2)*SQRT(1+dydx^2) * cos(theta)
      f1 = LHS-RHS

      if check eq 0.0d0 then begin
        if abs(f0) LT abs(f1) then begin
          temp = t0
          t0 = t1
          t1 = temp

          temp = f0
          f0 = f1
          f1 = temp
        endif
      endif

      t2 = t0-f0*(t0-t1)/(f0-f1)
      err = t2-t1
      t0 = t1
      t1 = t2

      if check GE 99 then begin
          ;there is a problem in the calculation...notify user and set xc=yc=theta=0
          if (ignore eq 0) then $
             whoops=dialog_message('No solution for lambda = ' $
                +strtrim(string((*exrayptr).source(whichsource).lambda(whichorder,whichlambda) $
                ,format='(F7.3)'))+ ' Angstroms',/Error)
          (*exrayptr).xstal(whichxstal).xc(whichsource,whichorder,whichlambda)=0.
          (*exrayptr).xstal(whichxstal).yc(whichsource,whichorder,whichlambda)=0.
          (*exrayptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)=0.
          return
      endif

      check = check+1
  endwhile

  ;***************************************************************
  ; Save the final solution in xc and yc
  ;***************************************************************

  if (((t2 ge betamin) and (t2 le betamax)) or limit eq 0) then begin
         xc =-h*cos(t2)/(1.0-ecc*cos(t2))
         yc = h*sin(t2)/(1.0-ecc*cos(t2))
  endif else begin
    ;otherwise set this theta = 0. as a flag that the x-ray will not
    ; diffract off this crystal
     xc = 0.
     yc = 0.
     (*exrayptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)=0.
  endelse


(*exrayptr).xstal(whichxstal).xc(whichsource,whichorder,whichlambda)=xc
(*exrayptr).xstal(whichxstal).yc(whichsource,whichorder,whichlambda)=yc

return
end
