PRO Exray_BuildPtr, exraystr, exrayptr

wnumsource=exraystr.wnumsource
wnumxstals=exraystr.wnumxstals
wnumorders=exraystr.wnumorders
wnumlambda=exraystr.wnumlambda

if (exraystr.autoflag eq 1) then begin
   exraystr.lamset(0) = 0
   exraystr.serset(0) = 1
   exraystr.wsource(*).lambda(*,*)=0.
   exraystr.wxstal(*).beta(*,*)=0.
   exraystr.wxstal(*).beta(0,0)=exraystr.wxstal(*).betamin
   exraystr.wxstal(*).beta(0,1)=exraystr.wxstal(*).betamax
   exraystr.wxstal(*).beta(0,2)= $
     (exraystr.wxstal(*).betamax-exraystr.wxstal(*).betamin)/exraystr.wnumlambda
endif

;***************************************************************************
;Size the arrays
;***************************************************************************
numsource=long(0)
numxstals=long(0)
numorders=long(0)
numlambda=long(0)

for i=0,wnumsource-1 do $
   if (exraystr.sorset(i) eq 1) then numsource=numsource+1
for i=0,wnumxstals-1 do $
   if (exraystr.xstalset(i) eq 1) then numxstals=numxstals+1
for i=0,wnumorders-1 do begin
   if (exraystr.wsource(0).lambda(i,0) gt 0.) then begin
      numorders=numorders+1
      if (exraystr.serset(i) eq 0) then begin
         ;specific lambdas are given
         index=where(exraystr.wsource(0).lambda(i,*) gt 0.)
         if (n_elements(index) gt numlambda) then numlambda=n_elements(index)
      endif else begin
         ;a series of lambdas is given
         num=long((exraystr.wsource(0).lambda(i,1)-exraystr.wsource(0).lambda(i,0))/ $
                   exraystr.wsource(0).lambda(i,2))
         if (num gt numlambda) then numlambda=num
      endelse
   endif else if (exraystr.wxstal(0).beta(i,0) gt 0.) then begin
      numorders=numorders+1
      if (exraystr.serset(i) eq 0) then begin
         ;specific betass are given
         index=where(exraystr.wxstal(0).beta(i,*) gt 0.)
         if (n_elements(index) gt numlambda) then numlambda=n_elements(index)
      endif else begin
         ;a series of betas is given
         num=long((exraystr.wxstal(0).beta(i,1)-exraystr.wxstal(0).beta(i,0))/ $
                   exraystr.wxstal(0).beta(i,2))
         if (num gt numlambda) then numlambda=num
      endelse
   endif
endfor

;**********************************************************************
; Define the structures that will be used in EXRAY Computations
;**********************************************************************
source={NAME: 'SOURCE VARIABLES',$
        lambda:dblarr(numorders,numlambda),$ ;the wavelengths to calculate
        n:lonarr(numorders),$ ;the order number to consider
        a:double(0),$ ;the source x position
        b:double(0)} ;the source y position

source=replicate(source,numsource)

xstal={NAME: 'CRYSTAL VARIABLES',$
       twod:double(0),$ ;twice the crystal plane spacing
       theta:dblarr(numsource,numorders,numlambda),$ ;the bragg diffraction angles
       beta:dblarr(numorders,numlambda),$ ;the user-defined beta values
       xc:dblarr(numsource,numorders,numlambda),$ ;the x position where lambda hits the crystal
       yc:dblarr(numsource,numorders,numlambda),$ ;the y position where lambda hist the crystal
       R0:double(0),$ ;the focal length of the crystal
       h:double(0),$ ;the henke h parameter
       ecc:double(0),$ ;the crystal eccentricity
       betamin:double(0),$ ;the minimum beta angle for the crystal
       betamax:double(0)} ;the maximum beta angle for the crystal

xstal=replicate(xstal,numxstals)

film={NAME: 'FILM VARIABLES',$
      rf:double(0),$ ;the radius of the film location
      phi:double(0),$ ;the rotation of the film location
      mu: double(0),$ ;the rotation of the film about its center
      L: double(0),$ ;the length of the film
      JFlag:exraystr.wfilm.JFlag,$ ;Flag to say what type of film plane
      xlambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;x position where lambda hits film
      ylambda:dblarr(numxstals,numsource,numorders,numlambda),$ ;y position where lambda hits film
      angle:dblarr(numxstals,numsource,numorders,numlambda),$ ;the angle of incidence on the film
      xangle:dblarr(numxstals,numsource,numorders,numlambda),$ ;the angle of incidence on the xover filter
      xxover:dblarr(numxstals,numsource,numorders,numlambda),$ ;x position of johann crossover curve
      yxover:dblarr(numxstals,numsource,numorders,numlambda),$ ;y position of johann crossover curve
      d: dblarr(numxstals,numsource,numorders,numlambda),$ ;the position of lambda in film coordinates
      filmwdf: long(0)} ;wdf array containing the Johann film coordinates

;**********************************************************************
; Transfer Data to the new structures
;**********************************************************************

sorindex=long(-1)
for i=0,wnumsource-1 do begin
   if (exraystr.sorset(i) eq 1) then begin
      sorindex=sorindex+1
      source(sorindex).a=exraystr.wsource(i).a
      source(sorindex).b=exraystr.wsource(i).b
   endif
endfor
xindex=long(-1)
for i=0,wnumxstals-1 do begin
   if (exraystr.xstalset(i) eq 1) then begin
      xindex=xindex+1
      xstal(xindex).h=exraystr.wxstal(i).h
      xstal(xindex).R0=exraystr.wxstal(i).R0
      xstal(xindex).betamin=exraystr.wxstal(i).betamin
      xstal(xindex).betamax=exraystr.wxstal(i).betamax
      xstal(xindex).twod=exraystr.wxstal(i).twod
   endif
endfor
ordindex=long(-1)
for i=0,wnumorders-1 do begin
   if (exraystr.wsource(0).lambda(i,0) gt 0.) then begin
      ordindex=ordindex+1
      source(*).n(ordindex)=exraystr.wsource(0).n(i)
      if (exraystr.serset(i) eq 0) then begin
         ;specific lambdas are given
         for j=0,numlambda-1 do $
            source(*).lambda(ordindex,j)=exraystr.wsource(0).lambda(i,j)
      endif else begin
         ;a series of lambdas is given
         tmp1=exraystr.wsource(0).lambda(i,0)
         tmp2=exraystr.wsource(0).lambda(i,1)
         tmp3=exraystr.wsource(0).lambda(i,2)
         if tmp2 eq 0. then tmp2=tmp1+1.0
         if tmp3 eq 0. then tmp3=1.0
         nvalues=long((tmp2-tmp1)/tmp3)
         valarray=findgen(nvalues)*tmp3+tmp1
         for j=0,nvalues-1 do $
             source(*).lambda(ordindex,j)=valarray(j)
         if nvalues lt numlambda then $
            for j=nvalues,numlambda-1 do $
                source(*).lambda(ordindex,j)=0.
      endelse
   endif else if (exraystr.wxstal(0).beta(i,0) gt 0.) then begin
      ordindex=ordindex+1
      source(*).n(ordindex)=exraystr.wsource(0).n(i)
      if (exraystr.serset(i) eq 0) then begin
         ;specific betas are given
         for j=0,numlambda-1 do $
            xstal(*).beta(ordindex,j)=exraystr.wxstal(0).beta(i,j)
      endif else begin
         ;a series of betas is given
         tmp1=exraystr.wxstal(0).beta(i,0)
         tmp2=exraystr.wxstal(0).beta(i,1)
         tmp3=exraystr.wxstal(0).beta(i,2)
         if tmp2 eq 0. then tmp2=tmp1+1.0
         if tmp3 eq 0. then tmp3=1.0
         nvalues=long((tmp2-tmp1)/tmp3)
         valarray=findgen(nvalues)*tmp3+tmp1
         for j=0,nvalues-1 do $
             xstal(*).beta(ordindex,j)=valarray(j)
         if nvalues lt numlambda then $
            for j=nvalues,numlambda-1 do $
                xstal(*).beta(ordindex,j)=0.
      endelse
   endif
endfor
if (exraystr.wfilm.JFlag eq 2) then begin
   ;the user has asked for the detector to be the locus of
   ; points at each lambda's johann crossover point.
   ; Set rf to be -1. as a flag for future consideration.
   film.rf=-1.
   ;set phi to the global average beta + pi
   film.phi=(max(xstal(*).betamax)+min(xstal(*).betamin))/2.+!pi
   JohannFlag=0

   if (exraystr.wfilm.jsflag eq 0) then begin
      ;calculate locus of points and store as wdf array
         jh=xstal(0).h
         jR0=xstal(0).R0
         jecc=sqrt(1+(jh/jR0)^2)-jh/jR0
         jbetamin=xstal(0).betamin
         jbetamax=xstal(0).betamax
         jbeta=findgen(10000)/9999.*(jbetamax-jbetamin)+jbetamin
         jtheta=atan((1-jecc*cos(jbeta))/(jecc*sin(jbeta)))
         rcurve=jh*(1.-2.*jecc*cos(jbeta)+jecc^2.)^(3./2.)/(1.-jecc*cos(jbeta))^3.
         jrho=jh/(1.-jecc*cos(jbeta))
         rlambda=rcurve*sin(jtheta)-jrho
         xj=rlambda*cos(jbeta)
         yj=-rlambda*sin(jbeta)
         i2w,xj,yj,exraystr.wfilm.filmwdf,/xy

   endif else begin
      ;read in locus of points and store as wdf array
         rufo,xj,yj,file=exraystr.wfilm.filmfile
         i2w,xj,yj,exraystr.wfilm.filmwdf,/xy
   endelse
   ;determine length of film plane (needed for autoscale)
   nxj=n_elements(xj)
   dj=total(sqrt((yj(1:nxj-1)-yj(0:nxj-2))^2+(xj(1:nxj-1)-xj(0:nxj-2))^2))
   exraystr.wfilm.L=dj

   film.filmwdf = exraystr.wfilm.filmwdf

endif else if (exraystr.wfilm.JFlag eq 1) then begin
   JohannFlag=1
   film.mu=exraystr.wfilm.mu-exraystr.wfilm.phi+!pi/2.
   film.L=exraystr.wfilm.L
endif else begin
   JohannFlag=0
   film.rf=exraystr.wfilm.rf
   film.phi=exraystr.wfilm.phi
   film.mu=exraystr.wfilm.mu
   film.L=exraystr.wfilm.L
endelse

;************************************************************
; Calculate some dependent values to be used by many routines
;************************************************************
;the eccentricities
for i=0,numxstals-1 do begin
   h=xstal(i).h
   R0=xstal(i).R0
   xstal(i).ecc = sqrt(1+(h/R0)^2)-h/R0
endfor

;the bragg angles
for l=0,numxstals-1 do begin
   twod=xstal(l).twod
   for i=0,numsource-1 do begin
      for j=0,numorders-1 do begin
          n=source(i).n(j)
          for k=0,numlambda-1 do begin
              lambda=source(i).lambda(j,k)
              if (lambda gt 0) and (n*lambda le twod) then begin
                 xstal(l).theta(i,j,k) = asin(n*lambda/twod)
              endif else $
                 xstal(l).theta(i,j,k) = -1
          endfor
      endfor
   endfor
endfor


if (JohannFlag eq 1) then begin
   ;the user has asked for a Johann Line detector at the average theta position
   if ((n_elements(beta_avg) eq 0) and (n_elements(theta_avg) eq 0)) then begin
      ;the user has not specitified a beta_avg, so use the average beta
      ; for the entire substrate
      betamax=max(xstal(*).betamax)
      betamin=min(xstal(*).betamin)
      beta_avg=(betamax+betamin)/2.
   endif else if (n_elements(beta_avg) eq 0) then begin
      ;the user has specified a theta_avg, so calculate the corresponding beta
      ; assuming the source is at the source focus
      betavg_global=(max(xstal(*).betamax)+min(xstal(*).betamin))/2.
      diff = 100.
      for i=0,numxstals-1 do begin
         ;calculate the beta_avg for each crystal and then determine which
         ; crystal the theta_avg is defined for.  If defined for multiple
         ; crystals, then assume it to be the one closest to the center
         ecc=xstal(i).ecc
         betamax=xstal(i).betamax
         betamin=xstal(i).betamin
         betavg=theta_avg+acos(cos(theta_avg)/ecc)
         if (i eq 0) then beta_avg = betavg ;in case betavg not on any crystal
         if ((betavg lt betamax) and (betavg gt betamin)) then $
            if (abs(betavg_global-betavg) lt diff) then begin
               diff = abs(betavg_global-betavg)
               beta_avg = betavg
            endif
      endfor
   endif

   index = where((xstal(*).betamax gt beta_avg) $    ;find which crystal beta diffracts off
                 and (xstal(*).betamin lt beta_avg))
   if (index(0) lt 0) then index(0) = 0  ;beta doesn't exist for these crystals so use crystal 1
   h=xstal(index(0)).h
   R0=xstal(index(0)).R0
   ecc = sqrt(1.+(h/R0)^2)-h/R0
   thetaJ= atan((1.-ecc*cos(beta_avg))/(ecc*sin(beta_avg)))
   rcurve=h*(1.-2.*ecc*cos(beta_avg)+ecc^2.)^(3./2.)/(1.-ecc*cos(beta_avg))^3.
   rho=h/(1.-ecc*cos(beta_avg))
   film.rf=rcurve*sin(thetaJ)-rho
   xc_Bavg=-rho*cos(beta_avg)
   yc_Bavg=rho*sin(beta_avg)
   alpha=atan(yc_Bavg/(xc_Bavg+R0))
   film.phi=2.*thetaJ-alpha+!pi
   film.mu=film.mu+film.phi-!pi/2.
;   output = 'At the central Johann crossover, rf= '+string(film.rf,format='(F6.3)')+'cm and phi= '$
;             +string(film.phi*180./!pi,format='(F8.3)')
;   widget_control,exraystr.widget.Output,SET_VALUE=output
endif

;************************************************************
; Define the pointer which will be shared by EXRAY routines
;************************************************************

newexraystr={source:source, $
          xstal:xstal, $
          film:film, $
          numorders:numorders, $
          numxstals:numxstals, $
          numsource:numsource,$
          numlambda:numlambda }

exrayptr=ptr_new(newexraystr,/no_copy)

return
end