PRO exray_eff, MainWidget

widget_control, MainWidget,get_uvalue=exraystr


filt1=exraystr.wfilm.filter(0)
filt2=exraystr.wfilm.filter(1)
filt3=exraystr.wfilm.filter(2)

;retrieve the dataset
if (exraystr.plotflag eq 4) then $
   data=exraystr.scaledwdf(exraystr.activedata) $
else $
   data=exraystr.datawdf(exraystr.activedata)

;wdf array 50 holds the working data array
xfr,data,50


;*************************** filter corrections *********************************

if (exraystr.wfilm.filtflag eq 1) then begin
   ;calculate the correction due to the filter transmission

   ;open the pff file containing mass absorption curves
   pff=exraystr.projpath+'atomic_data'+exraystr.sepchar+'filter_material.pff'
   openpff,pff,fileid=fid

   firstflag=0
   for i=0,2 do begin
      ;retrieve the filter choices (as in the order given in effwidget)
      filtindex=exraystr.wfilm.filter(i)
      if (filtindex ne 0) then begin
         ;determine the index of the selected filter in the pff file
         case filtindex of
            1: begin  ;Aluminum
                index=2
                rho=2.6989
               end
            2: begin  ;Beryllium
                index=10
                rho=1.848
               end
            3: begin  ;Kapton
                index=115
                rho=1.42
               end
            4: begin  ;Kimfol
                index=117
                rho=1.20
               end
            5: begin  ;Mylar
                index=122
                rho=1.397
               end
            6: begin  ;Parylene N
                index=126
                rho=1.11
               end
            7: begin  ;Polystyrene
                index=131
                rho=1.056
               end
            8: begin   ;Tantalum
                index=83
                rho=16.654
               end
            9: begin   ;Titanium
                index=91
                rho=4.54
               end
           10: begin   ;Tungsten
                index=92
                rho=19.3
               end
         endcase

         ;get the absorption curve from the pff file (use wdf array 60 as work array)
         re,60,index,fileid=fid
         w2i,energy,sigma,60
         lambda=12398.53/energy ;angstroms
         mu=sigma*rho ;cm^-1
         thickness=exraystr.wfilm.filtwidth(i)*1.e-4 ;cm
         ftrans=exp(-1.0*mu*thickness)
         i2w,lambda,ftrans,61

         ;wdf array 62 holds the total filter transmission for all filters
         if (firstflag eq 0) then begin
            xfr,61,62
            firstflag=1
         endif else mul,62,61

      endif
   endfor

   ;calculate the transmission corrected data
   div,50,62
endif

closepff,fid

;****** Xstal reflectivity and Spectrometer Efficiency Corrections ************************
if (exraystr.wfilm.xstalflag eq 1) then begin

   ;get the diffraction order
   n=exraystr.wsource(0).n(0)

   if (exraystr.convexflag eq 0) then $ ;elliptical geometry
      whichXstal=widget_info(exraystr.widget.Xstals,/Droplist_Select) $
   else $ ;convex geometry
      whichXstal=widget_info(exraystr.widget.XstalC,/Droplist_Select)
   case whichXstal of
      1: begin   ;KAP
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'KAP_'+strtrim(string(n),2)+'.txt'
      end
      2: begin   ;Mica
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'Mica_'+strtrim(string(n),2)+'.txt'
      end
      3: begin   ;PET
         refile=exraystr.projpath+'crystal_data'+exraystr.sepchar+$
                  'PET_'+strtrim(string(n),2)+'.txt'
      end
   endcase

   readasc,refile,10,xt,yt
   index=sort(yt(*,8))           ;index for ascending wavelength order
   lambda=yt(index,8)            ;Angstroms

   ;Get the appropriate reflectivity (Radians)
   if (exraystr.wfilm.refflag eq 0) then $       ;Average reflectivity
      ref=(yt(index,2)+yt(index,3))*1.e-3/2. $
   else if (exraystr.wfilm.refflag eq 1) then $  ;Prins reflectivity
      ref=yt(index,2)*1.e-3 $
   else $                                        ;Mosaic reflectivity
      ref=yt(index,3)*1.e-3

   nlam=n_elements(lambda)  ;the number of wavelengths in reflectivity curve
   widget_control,exraystr.widget.Graphics,GET_VALUE=winID
   tempstr=replicate(exraystr,1)                      ;make a working copy of exraystr
   tempstr.autoflag=0                                 ;turn off autoscaling
   index=where(tempstr.wsource(0).lambda(*,0) gt 0.) ;the first set order
   if (index(0) lt 0) then $                         ;betas have been defined by the user
      index=where(tempstr.wxstal(0).beta(0,0,*,0) gt 0.)
   tempstr.wsource(0).lambda(index(0),*)=1.0         ;make sure maximum number of lambdas
   tempstr.wxstal(0).beta(index(0)+1,0)=!pi/4.       ;make sure I can later set average beta
   tempstr.serset(index(0))=0

   ;*** elliptical geometry
   if (exraystr.convexflag eq 0) then begin
      h=exraystr.wxstal(0).h
      R0=exraystr.wxstal(0).R0

      ;calculate where the lambdas hit the crystal and the film
      exray_buildptr,tempstr,temptr
      (*temptr).numsource=1
      (*temptr).numorders=2
      (*temptr).numxstals=1
      (*temptr).source(0).n(1)=(*temptr).source(0).n(0)
      (*temptr).source(0).lambda(*,*)=0.
      (*temptr).source(0).lambda(0,0:nlam-1)=lambda
      (*temptr).numlambda=nlam
      (*temptr).xstal(0).beta(*,*)=0.
      (*temptr).xstal(0).beta(1,0)=(*temptr).xstal(0).betamin
      (*temptr).xstal(0).beta(1,1)=(*temptr).xstal(0).betamax
      twod=(*temptr).xstal(0).twod
      n=(*temptr).source(0).n(0)
      (*temptr).xstal(0).theta(*,*,*)=0.
      (*temptr).xstal(0).theta(0,0,0:nlam-1)=asin(n*lambda/twod)
      exray_go,50,temptr,windID=winID,/limit,/ignore
      xc=(*temptr).xstal(0).xc(0,0,0:nlam-1)
      yc=(*temptr).xstal(0).yc(0,0,0:nlam-1)
      xf=(*temptr).film.xlambda(0,0,0,0:nlam-1)
      yf=(*temptr).film.ylambda(0,0,0,0:nlam-1)
      theta=(*temptr).xstal(0).theta(0,0,0:nlam-1)

      L=sqrt(xc^2+yc^2)+sqrt((xc-xf)^2+(yc-yf)^2)
      rf=sqrt(xf^2+yf^2)

      ;some lambdas may not diffract off this crystal
      ;  ...assume either or min or max L and rf for these
      index=where(yc eq 0)
      if (index(0) ge 0) then begin
         xcmin=(*temptr).xstal(0).xc(0,1,0)
         ycmin=(*temptr).xstal(0).yc(0,1,0)
         xcmax=(*temptr).xstal(0).xc(0,1,1)
         ycmax=(*temptr).xstal(0).yc(0,1,1)
         xfmin=(*temptr).film.xlambda(0,0,1,0)
         yfmin=(*temptr).film.ylambda(0,0,1,0)
         xfmax=(*temptr).film.xlambda(0,0,1,1)
         yfmax=(*temptr).film.ylambda(0,0,1,1)
         L(index)=sqrt(xcavg^2+ycavg^2)+sqrt((xcavg-xfavg)^2+(ycavg-yfavg)^2)
         rf(index)=sqrt(xfavg^2+yfavg^2)
         theta(index)=asin(n*lambda(index)/twod)
      endif

      ;Henke's efficiency correction
      So_dNdA=3.204e-19*rf*L*R0*12398.53/(h*lambda^2*ref*sin(theta)*cos(theta)) ;J/sr/Angstrom

      ptr_free,temptr

  ;*** convex geometry
   endif else begin
      whichXstal=widget_info(exraystr.widget.XstalC,/Droplist_Select)
   endelse

   exraystr.autoflag=autoflag

endif

;****************** Plot it and ask if user is satisfied ***********************
w2i,lamda,spect_data,50

yrange=[min(spect_data),max(spect_data)]

plot,lamda,spect_data,col=0,line=0,xrange=exraystr.xrange,yrange=yrange $
       ,xtitle='Wavelength (Angstroms)',pos=[0.15,0.1,0.95,0.957],xstyle=1,ystyle=1 $
       ,psym=0,ytitle='Intensity photons/um^2'

wait,0.0
check=dialog_message('Is this acceptable?',/Question)
if (check eq 'Yes') then begin
   ;permanently overwrite the active scaledwdf
   if (exraystr.plotflag eq 4) then $
      i2w,lamda,spect_data,exraystr.scaledwdf(exraystr.activedata),/xy $
   else $
      i2w,lamda,spect_data,exraystr.datawdf(exraystr.activedata),/xy
endif
plotflag=exraystr.plotflag
exraystr.plotflag = 0
widget_control,MainWidget,set_uvalue=exraystr
widget_control,exraystr.widget.plotlist,send_event=$
      { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
         HANDLER:MainWidget, INDEX:plotflag }

return
end
