PRO exray_horder, MainWidget

Widget_Control, MainWidget, get_uvalue = exraystr
lim1= exraystr.wfilm.hocrange(0)
lim2= exraystr.wfilm.hocrange(1)
n1 = exraystr.wsource(0).n(0)
n2 = exraystr.wfilm.ncor
nr = float(n2)/float(n1)

;save a copy of the data in case user not satisified with result
if (exraystr.plotflag eq 4) then $
   w2i,xorig,yorig,exraystr.scaledwdf(exraystr.activedata) $
else $
   w2i,xorig,yorig,exraystr.datawdf(exraystr.activedata)

;limit the range to the fall within the data domain
if (lim1 lt min(xorig)) then lim1=min(xorig)
if (lim2 gt max(xorig)/nr) then lim2=max(xorig)/nr

;run d2i in the original order
;-----------------------------
exray_d2i,MainWidget,angcor=exraystr.wfilm.d2iangflag,/noplot,/nocheck,angle=ang
Widget_Control, MainWidget, get_uvalue = exraystr
if (exraystr.plotflag eq 4) then $
   w2i,xi,yi,exraystr.scaledwdf(exraystr.activedata) $
else $
   w2i,xi,yi,exraystr.datawdf(exraystr.activedata)

;save the exposure in the requested correction limits
if (exraystr.wfilm.hoflag eq 0) then begin
   index=where(xi ge lim1 and xi le lim2)
   xil=xi(index)
   yil=yi(index)
endif else begin
   w2i,x,y,exraystr.wfilm.howdf
   index=where(xorig ge min(x) and xorig le max(x))
   xx=xorig(index)
   yy=interpol(y,x,xx)
   index=where(xx ge lim1 and xx le lim2)
   xil=xx(index)
   yil=yy(index)
endelse


;correct for reflectivity, filter transmission, and spectrometer geometry
;------------------------------------------------------------------------
exray_eff,MainWidget,/noplot,/nocheck,save=0
Widget_Control, MainWidget, get_uvalue = exraystr
if (exraystr.plotflag eq 4) then $
   w2i,xeff,yeff,exraystr.scaledwdf(exraystr.activedata) $
else $
   w2i,xeff,yeff,exraystr.datawdf(exraystr.activedata)

;Find the exposure in the order to be corrected for
;-----------------------------------------------------
;first find correction due to geometry and reflectivity in original order
mcpflag = exraystr.wfilm.mcpflag
filtflag = exraystr.wfilm.filtflag
xstalflag = exraystr.wfilm.xstalflag
refflag = exraystr.wfilm.refflag
exraystr.wfilm.mcpflag = 0
exraystr.wfilm.filtflag = 0
exraystr.wfilm.xstalflag = 1
;exraystr.wfilm.refflag = 3
nw=(lim2-lim1)/0.001
waves=findgen(nw)/(nw-1.)*(lim2-lim1)+lim1
if (exraystr.plotflag eq 4) then $
   i2w,waves,waves/waves,exraystr.scaledwdf(exraystr.activedata),/xy $
else $
   i2w,waves,waves/waves,exraystr.datawdf(exraystr.activedata),/xy
Widget_Control, MainWidget, set_uvalue = exraystr
exray_eff,MainWidget,/noplot,/nocheck,save=0
Widget_Control, MainWidget, get_uvalue = exraystr
if (exraystr.plotflag eq 4) then $
   w2i,xn1,yn1,exraystr.scaledwdf(exraystr.activedata) $
else $
   w2i,xn1,yn1,exraystr.datawdf(exraystr.activedata)

;now find correction due to geometry and reflectivity in order to be corrected for
exraystr.wsource(*).n(0) = n2
if (exraystr.plotflag eq 4) then $
   i2w,waves,waves/waves,exraystr.scaledwdf(exraystr.activedata),/xy $
else $
   i2w,waves,waves/waves,exraystr.datawdf(exraystr.activedata),/xy
Widget_Control, MainWidget, set_uvalue = exraystr
exray_eff,MainWidget,/noplot,/nocheck,save=0
Widget_Control, MainWidget, get_uvalue = exraystr
if (exraystr.plotflag eq 4) then $
   w2i,xn2,yn2,exraystr.scaledwdf(exraystr.activedata) $
else $
   w2i,xn2,yn2,exraystr.datawdf(exraystr.activedata)

;restore efficiency correction flags
exraystr.wfilm.mcpflag = mcpflag
exraystr.wfilm.filtflag = filtflag
exraystr.wfilm.xstalflag = xstalflag
;exraystr.wfilm.refflag = refflag
exraystr.wsource(*).n(0) = n1
Widget_Control, MainWidget, set_uvalue = exraystr

c1=interpol(yn1,xn1,xil)
c2=interpol(yn2,xn2,xil)
cdif=c1/c2


;Calculate the exposure in the order to be corrected for
;-------------------------------------------------------
index4=where(xorig ge lim1*nr and xorig le lim2*nr)
xil2=xorig(index4)
yil2=interpol(yil*cdif,xil*nr,xil2)
dtot=yorig(index4)
;exray_findi1,MainWidget,16.,3.,0.88856+0.1,yil1,angcor=exraystr.wfilm.d2iangflag,angle=ang2
exray_findi1,MainWidget,xil2,yil2,dtot,yil1,angcor=exraystr.wfilm.d2iangflag,angle=ang2
yinew=yi
yinew(index4)=yil1

;Check that user is satisfied with result
;----------------------------------------
yran=[0.,max([yi,yinew,yil2])]
plot,xi,yi,xtitle='Wavelength (Angstroms)',ytitle='Exposure (photons/um!E2!N)',yrange=yran,/ys
oplot,xil2,yil2,col=3
oplot,xi,yinew,col=1
legend,['Original Data','Corrected Data',strtrim(string(lim1,form='(F6.3)'),2)+'-'$
        +strtrim(string(lim2,form='(F6.3)'),2)+' in order '+strtrim(string(n2,form='(I2)'),2)],$
        col=[0,1,3],line=[0,0,0]
wait,0.0
check=dialog_message('Is this acceptable?',/Question)
if (check eq 'Yes') then begin
     ;permanently overwrite the active scaledwdf
     if (exraystr.plotflag eq 4) then $
        i2w,xi,yinew,exraystr.scaledwdf(exraystr.activedata),/xy $
     else $
        i2w,xi,yinew,exraystr.datawdf(exraystr.activedata),/xy
endif else begin
;restore original plot
   if (exraystr.plotflag eq 4) then $
      i2w,xorig,yorig,exraystr.scaledwdf(exraystr.activedata),/xy $
   else $
      i2w,xorig,yorig,exraystr.datawdf(exraystr.activedata),/xy
endelse
   plotflag=exraystr.plotflag
   exraystr.plotflag = 0
   widget_control,MainWidget,set_uvalue=exraystr
   widget_control,exraystr.widget.plotlist,send_event=$
      { WIDGET_DROPLIST, ID:exraystr.widget.plotlist, TOP:MainWidget, $
         HANDLER:MainWidget, INDEX:plotflag }

Widget_Control, MainWidget, set_uvalue = exraystr


return
end