PRO exray_pffcalc,pffstr

;open the pff and read into structure
openpff,pffstr.pfffile
readstr,1
xdat=get_array(1,1)
vdat=get_array(1,2)
scaledarray=get_array(1,-1)
yarray=reform(scaledarray)
xfrstr,1,2

;Open background ufo if required
if (pffstr.backflag eq 1) then begin
  rufo,xback,yback,file=pffstr.backfile
  if (pffstr.d2iflag eq 0) then begin
   if ((xback(0) gt xdat(0)) or $
       (xback(n_elements(xback)-1) lt xdat(n_elements(xdat)-1))) then begin
        whoops=dialog_message('Background data must span the entire x-axis of the pff.',/error)
        closepff
        return
   endif else begin
        yback=interpol(yback,xback,xdat)
     endelse
   endif
endif

;save the active data set to be restored at the end
active=pffstr.exraystr.activedata
xfr,pffstr.exraystr.datawdf(active),get_maxwdf()
xfr,pffstr.exraystr.scaledwdf(active),get_maxwdf()-1


plotstr,1,/xstyle,/ystyle,cn=2
wait,1.0

;Do wavelength scaling on first value to get required scaling data
if (pffstr.waveflag eq 1) then begin
  ydat=yarray(*,0)
  if (pffstr.waveflag eq 1) then $
     i2w,xdat,ydat,pffstr.exraystr.datawdf(active),/xy $
  else $
     i2w,xdat,ydat,pffstr.exraystr.scaledwdf(active),/xy
  if (pffstr.exraystr.convexflag eq 0) then $
     exray_autoscale,pffstr.exraystr,/noplot,fit=fit $
  else $
     exray_convex_autoscale,pffstr.exraystr,/noplot,fit=fit
endif

ny=n_elements(yarray(0,*))

;Loop over spatial dimension
for i=0,ny-1 do begin
  ydat=yarray(*,i)

  ;*********************************************
  ;apply the background subtraction (if desired)
  ;*********************************************
  if ((pffstr.backflag eq 1) and (pffstr.d2iflag eq 0)) then begin
     ydat=ydat-yback
  endif

  if (pffstr.waveflag eq 1) then $
     i2w,xdat,ydat,pffstr.exraystr.datawdf(active),/xy $
  else $
     i2w,xdat,ydat,pffstr.exraystr.scaledwdf(active),/xy

  ;******************************************
  ;apply the wavelength scale (if desired)
  ;******************************************
  if (pffstr.waveflag eq 1) then begin
    xref=pffstr.exraystr.xref
    orderef=pffstr.exraystr.orderef
    lamref=pffstr.exraystr.lamref
    whatsource=where(pffstr.exraystr.sorset gt 0)
    order=pffstr.exraystr.wsource(whatsource(0)).n(0)
    lamscale=fit(0)
    for j=1,pffstr.exraystr.fitorder do $
      lamscale=lamscale+xdat^j*fit(j)
    ;now shift the result so that the reference wavelength is in the same spot
    index3=where(xdat eq xref)
    if (index3(0) lt 0) then $
       index3=where(xdat gt 0.9999*xref and xdat lt 1.0001*xref)

    ;scale the reference wavelength if it is in a different order than the dispersion calculation
    lamref=lamref*orderef/order

    lamscale=lamscale-(lamscale(index3(0))-lamref)
    i2w,lamscale,ydat,pffstr.exraystr.scaledwdf(active),/xy
    if (i eq 300) then begin
       i2w,lamscale,ydat,12,/xy
    endif
  endif

  ;******************************************
  ;apply the density to intensity conversion
  ;******************************************
  if (pffstr.d2iflag eq 1) then begin
     Widget_Control, pffstr.MainWidget, set_uvalue = pffstr.exraystr
     wait,0.01
     exray_d2i,pffstr.MainWidget,/nosave,/nocheck,/noplot
     ;data has been stored by exray_d2i in wdf array 60
     if (pffstr.backflag eq 1) then begin
        w2i,xdat2,ydat2,60
        if (i eq 0) then begin
           ybacki=interpol(yback,xback,xdat2)
        endif
        ydat2=ydat2-ybacki
        i2w,xdat2,ydat2,60,/xy
     endif
     xfr,60,pffstr.exraystr.scaledwdf(active)
  endif

  ;******************************************
  ;apply the efficiency corrrection (if desired)
  ;******************************************
  if (pffstr.effcorflag eq 1) then begin
      xfr,pffstr.exraystr.scaledwdf(active),60
      Widget_Control, pffstr.MainWidget, set_uvalue = pffstr.exraystr
      wait,0.01
      exray_eff,pffstr.MainWidget,save=0,/nocheck,/noplot,/noextract
      xfr,60,pffstr.exraystr.scaledwdf(active)
  endif

  w2i,x,y,pffstr.exraystr.scaledwdf(active)
  if (i eq 0) then begin
     nx=n_elements(x)
     nv=n_elements(vdat)
     newarray=dblarr(nx,nv)
     scaledx=x
     ;nx=n_elements(scaledarray(*,0,0))
     ;newx=findgen(nx)/(nx-1.)*(x(nx-1)-x(0))+x(0)
     ;put_array,2,newx,1
  endif
  ;if (n_elements(y) ne nx) then $
  ;   y=interpol(y,x,newx)
  ;scaledarray(*,i,0)=y
  newarray(*,i)=y

  plot,scaledx,newarray(*,i),/xstyle,/ystyle,$
       title='i = '+strtrim(string(i+1),2)+' (out of '+strtrim(string(ny),2)+')'
  wait,0.01
endfor

;put_array,2,scaledarray,-1
arr2str,2,newarray,scaledx,vdat

closepff
plotstr,2,/xstyle,/ystyle,cn=2
savefile=''
savefile=dialog_pickfile(title='Save Processed pff As:',/write,path=pffstr.path,filter='*.pff')
if (savefile eq '') then begin
   whoops=dialog_message('Processed pff Not Saved.',/info)
endif else begin
   if (strpos(savefile,'.pff') eq -1) then savefile=savefile+'.pff'
   createpff,savefile
   wristr,2
   closepff
endelse

;restore the data array
xfr,get_maxwdf(),pffstr.exraystr.datawdf(active)
xfr,get_maxwdf()-1,pffstr.exraystr.scaledwdf(active)
widget_control,pffstr.exraystr.widget.DataFile,SET_VALUE=pffstr.exraystr.datafile(active)
widget_control,pffstr.exraystr.widget.plotlist,send_event=$
          { WIDGET_DROPLIST, ID:pffstr.exraystr.widget.plotlist, TOP:pffstr.MainWidget, $
            HANDLER:pffstr.MainWidget, INDEX:3 }
return
end
