pro exray_convex_thetacalc,convexptr,whichxstal,whichsource,whichorder,whichlambda,limit

  rc=(*convexptr).xstal(whichxstal).rc
  a=(*convexptr).source(whichsource).a(whichxstal)
  b=(*convexptr).source(whichsource).b(whichxstal)
  beta=(*convexptr).xstal(whichxstal).beta(whichorder,whichlambda)

  ; **************************************************************
  ; ***  this part calculates the theta associated with the    ***
  ; ***  the defined beta.  It uses a Secant iterative solver  ***
  ; ***  (similar to a bisection solver) to find the point.    ***
  ; ***  It should also be noted that there are 4 solutions    ***
  ; ***  Applied Numerical Analysis 5th ed. Gerald and Wheatly pp 35
  ; **************************************************************
  thetamin=1.0*!pi/180.
  thetamax=89.*!pi/180.
  tol = 1.0d-10
  t0  = thetamin
  t1  = thetamax
  err = 1.0e6
  check = 0
  psi=atan(b/abs(a))
  RHS = !pi+psi-beta
  sqrta2b2 = sqrt(a*a+b*b)

  While(tol LT abs(err) and check le 100) do begin
      LHS = asin(rc*sin(!pi/2.-t0)/sqrta2b2)+2.*t0
      f0 = LHS-RHS

      LHS = asin(rc*sin(!pi/2.-t1)/sqrta2b2)+2.*t1
      f1 = LHS-RHS

      if check eq 0 then begin
        if abs(f0) LT abs(f1) then begin
          temp = t0
          t0 = t1
          t1 = temp

          temp = f0
          f0 = f1
          f1 = temp
        endif
      endif

      t2 = t0-f0*(t0-t1)/(f0-f1)
      err = t2-t1
      t0 = t1
      t1 = t2

      if check GE 99 then begin
          ;there is a problem in the calculation...notify user and set xc=yc=theta=0
          whoops=dialog_message('No solution for beta = ' $
              +strtrim(string((*convexptr).xstal(whichxstal).beta(whichorder,whichlambda) $
              ,format='(F8.3)'))+ ' degrees',/Error)
          (*convexptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)=0.
          return
      endif

      check += 1
  endwhile

;***************************************************************
; Save the final solution in theta and calculate lambda
;***************************************************************

  if ((t2 ge thetamin) and (t2 le thetamax)) then begin
    n=(*convexptr).source(whichsource).n(whichorder)
    twod=(*convexptr).xstal(whichxstal).twod
    p=(*convexptr).xstal(whichxstal).p
    (*convexptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)= $
       t2+p
  endif else begin
    ;otherwise set this theta = 0. as a flag that the x-ray will not
    ; diffract off this crystal
     (*convexptr).xstal(whichxstal).theta(whichsource,whichorder,whichlambda)=0.
  endelse

end
