PRO Exray_NearTrans,xnow,exraystr,dialog=dialog

delta=0.01 ;the delta-lambda to use for finding maxima/minima
if (exraystr.plotflag eq 4) then $
   xfr,exraystr.scaledwdf(exraystr.activedata),105 $
else $
   xfr,exraystr.datawdf(exraystr.activedata),105
w2i,xdata,ydata,105
diff=1000. ;the difference between xnow and the transition wavelength
mindex=-1  ;the index of the array element containing the minimum diff
mintrans=-1 ;the index of the series which contains the minimum diff
minele=-1 ;the index of the element which contains the minimum diff

absflag=exraystr.absflag

;find the chosen elements
index=where(exraystr.whichelements gt 0)
for i=0,n_elements(index)-1 do begin
   ;find the chosen transitions
   index2=where(exraystr.whichtrans(index(i),*) gt 0)
   for j=0,n_elements(index2)-1 do begin

      case index2(j) of

      0: begin ;The Lymann series
         ;find if xnow is closest to one of these transitions
         index3=where(abs(exraystr.element(index(i)).Ly.lambda-xnow) lt diff)
         if (index3(0) ge 0) then begin
            diff=min(abs(exraystr.element(index(i)).Ly.lambda(index3)-xnow),mindex)
            mindex=index3(mindex)
            mintrans=0
            minele=index(i)
         endif
      end
      1: begin ;The He-Like series
         ;find if xnow is closest to one of these transitions
         index3=where(abs(exraystr.element(index(i)).He.lambda-xnow) lt diff)
         if (index3(0) ge 0) then begin
            diff=min(abs(exraystr.element(index(i)).He.lambda(index3)-xnow),mindex)
            mindex=index3(mindex)
            mintrans=1
            minele=index(i)
         endif
      end
      2: begin ;The Li-Like series
         ;find if xnow is closest to one of these transitions
         index3=where(abs(exraystr.element(index(i)).Li.lambda-xnow) lt diff)
         if (index3(0) ge 0) then begin
            diff=min(abs(exraystr.element(index(i)).Li.lambda(index3)-xnow),mindex)
            mindex=index3(mindex)
            mintrans=2
            minele=index(i)
         endif
      end
      3: begin ;The ATBASE calculations from the user-defined .atm file
         ;find if xnow is closest to one of these transitions
         if (exraystr.element(index(i)).atm.LSFlag eq 1) then $
            index5=where(exraystr.element(index(i)).atm.LS eq 1) $
         else $
            index5=where(exraystr.element(index(i)).atm.lambda gt 0.)
         index3=where(abs(exraystr.element(index(i)).atm.lambda(index5)-xnow) lt diff)
         if (index3(0) ge 0) then begin
            diff=min(abs(exraystr.element(index(i)).atm.lambda(index5(index3))-xnow),mindex)
            mindex=index5(mindex)
            mintrans=3
            minele=index(i)
         endif
      end

      endcase
   endfor
endfor

;find the nearest maxima (emissions)/minima (absorptions)
if (mindex ge 0) and (mintrans ge 0) and (minele ge 0) then begin
   ;a nearest transition was found
   case mintrans of
     ;find the data indices within +-delta of the nearest transition

   0: $  ;The Lymann Series
      index4=where((xdata ge exraystr.element(minele).Ly.lambda(mindex)-delta) $
                     and (xdata le exraystr.element(minele).Ly.lambda(mindex)+delta))
   1: $  ;The He-Like Series
      index4=where((xdata ge exraystr.element(minele).He.lambda(mindex)-delta) $
                     and (xdata le exraystr.element(minele).He.lambda(mindex)+delta))
   2: $  ;The Li-Like Series
      index4=where((xdata ge exraystr.element(minele).Li.lambda(mindex)-delta) $
                     and (xdata le exraystr.element(minele).Li.lambda(mindex)+delta))
   3: $  ;The ATBASE calculations from the user-defined .atm file
       index4=where((xdata ge exraystr.element(minele).atm.lambda(mindex)-delta) $
                  and (xdata le exraystr.element(minele).atm.lambda(mindex)+delta))

   endcase

   if (index4(0) ge 0) then begin
      ;the wavelength is in the range of the data
      if (absflag eq 1) then $
      mint=mx(105,left=xdata(index4(0)), $
              right=xdata(index4(n_elements(index4)-1)),/quiet) $
      else $
      mint=mn(105,left=xdata(index4(0)), $
              right=xdata(index4(n_elements(index4)-1)),/quiet)

      case mintrans of
        ;Plot the point of the nearest transition

      0: $  ;The Lymann Series
         if (exraystr.linetrans eq 0) then $
            plots,exraystr.element(minele).Ly.lambda(mindex),mint,line=0,psym=5,$
                        col=1 $
         else $
            oplot,exraystr.element(minele).Ly.lambda(mindex)*[1.,1.],!y.crange,line=0,col=1
      1: $  ;The He-Like Series
         if (exraystr.linetrans eq 0) then $
            plots,exraystr.element(minele).He.lambda(mindex),mint,line=0,psym=5,$
                        col=1 $
         else $
            oplot,exraystr.element(minele).He.lambda(mindex)*[1.,1.],!y.crange,line=0,col=1
      2: $  ;The Li-Like Series
         if (exraystr.linetrans eq 0) then $
            plots,exraystr.element(minele).Li.lambda(mindex),mint,line=0,psym=5,$
                        col=1 $
         else $
            oplot,exraystr.element(minele).Li.lambda(mindex)*[1.,1.],!y.crange,line=0,col=1
      3: $  ;The ATBASE calculations from the user-defined .atm file
         if (exraystr.linetrans eq 0) then $
            plots,exraystr.element(minele).atm.lambda(mindex),mint,line=0,psym=5,$
                        col=1 $
         else $
            oplot,exraystr.element(minele).atm.lambda(mindex)*[1.,1.],!y.crange,line=0,col=1

   endcase

      if keyword_set(dialog) then begin
         ;Tell the user the transition details
           ;set the platform-specific carriage return character (for display purposes)
           if (!Version.OS_FAMILY eq 'MacOS') then cr=string(13B) else cr=''
         wait,0.0 ;program needs a split second to update the plot
         case mintrans of

         0: $;The Lymann Series
            info=dialog_message([exraystr.element(minele).Ly.config(mindex),$
                                 exraystr.element(minele).Ly.term(mindex),$
                                 strtrim(string(exraystr.element(minele).Ly.lambda(mindex),$
                                 format='(F6.3)'),2)$
                                 +' Angstroms'],/info)
         1: $;The He-Like Series
            info=dialog_message([exraystr.element(minele).He.config(mindex),$
                                 exraystr.element(minele).He.term(mindex),$
                                 strtrim(string(exraystr.element(minele).He.lambda(mindex),$
                                 format='(F6.3)'),2)$
                                 +' Angstroms'],/info)
         2: $;The Li-Like Series
            info=dialog_message([exraystr.element(minele).Li.config(mindex),$
                                 exraystr.element(minele).Li.term(mindex),$
                                 strtrim(string(exraystr.element(minele).Li.lambda(mindex),$
                                 format='(F6.3)'),2)$
                                 +' Angstroms'],/info)
         3: $;The ATBASE calculations from the user-defined .atm file
            info=dialog_message([exraystr.element(minele).atm.ion(mindex)+cr,$
                                 exraystr.element(minele).atm.config(mindex)+cr,$
                                 exraystr.element(minele).atm.configE(mindex)+cr,$
                                 exraystr.element(minele).atm.term(mindex)+cr,$
                                 strtrim(string(exraystr.element(minele).atm.lambda(mindex),$
                                 format='(F6.3)'),2)$
                                 +' Angstroms'],/info)
         endcase

      endif
   endif
endif

return
end