Function Theta_Xroot,x
common params,R0,a0,theta0,a01,theta01
    common angles,theta1
    term1=0.0
    term2=0.0
    term3=theta1
    term1=atan( (a0*sin(theta0)-R0+SQRT(R0*R0-x*x)) / (a0*cos(theta0)-x) )
    term2=atan( x / SQRT(R0*R0-x*x) )
    return,term1-term2-term3
END

Function Theta_X,x
    common params,R0,a0,theta0
    term2= sqrt((R0 -x)*(R0+x))
    term1=atan( (a0*sin(theta0)-R0+term2) / (a0*cos(theta0)-x) )
    term2=atan( x / term2 )
    return,term1-term2
END

Function A_x,x
    common params,R0,a0,theta0
    term2= sqrt((R0 -x)*(R0+x))
    term1=(a0*cos(theta0)-x)^2
    term2=(a0*sin(theta0)-R0+SQRT(R0*R0-x*x)) * (a0*sin(theta0)-R0+SQRT(R0*R0-x*x))
    return,SQRT(term1+term2)
END

Function M_x,x,ax,thetax
    common params,R0,a0,theta0
    return,R0/(2*ax*sin(thetax)-R0)
END

Function B_x,x,ax,thetax
    common params,R0,a0,theta0
    return,ax*R0/(2*ax*sin(thetax)-R0)
END

Function Omega_x,x
    common params,R0,a0,theta0
    term1=0.0  ;eta
    term2=0.0  ;phi
    term1=atan( (a0*sin(theta0)-R0+SQRT(R0*R0-x*x)) / (a0*cos(theta0)-x) )
    term2=atan( x / SQRT(R0*R0-x*x) )
    return,term1-2*term2
END

pro All_spherical_param, x, Yc, Tharr, Axarr, mxarr, bxarr, omega
common params,R0,a0,theta0,a01,theta01
;   Y value from center of crystal bend to x
    Yh =  sqrt((R0 -x)*(R0+x))
    Yc = R0 - yh
;   Y value to the source for the crystal at x
    YS = A0 *sin(theta0) - Yc
    YS = (a01 eq 0.0) ? A0 *sin(theta0) - Yc : A01 *sin(theta01) - Yc
;   X value to the source from X
    XS = A0*cos(theta0)-x
    XS = (a01 eq 0.0) ? A0*cos(theta0)-x : A01*cos(theta01)-x
;   Angle to the source from X
    Thetax = atan(YS/XS)
;   Slant angle of the crystal plane at X
    ThetaXC = atan (x/Yh)
;   Bragg angle at X
    Tharr = thetax - thetaxc
;   Source to Crystal distance vs X value
    Axarr = sqrt(XS *XS +YS *YS)
;   Magnigication  with film at 1/a +1/b = 2*sin(theta)/R0
    Mxarr = R0/(2*sin(tharr)*axarr - R0)
;   Distance from Crystal to film
    bxarr = mxarr*axarr
;   Angle from crystal to the film
    omega = thetax - 2*thetaxc
    end

pro FSSR2Dmodel,R0in,R_2din,R_Lin,a0in,theta0in,Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,xGarr,wdfi=wdfi
; ******************************************************************
; Program for modeling the FSSR2D geometry
; This program generates a series of arrays for use in interpreting
; spectra generated on the FSSR2D instrument on the Z facility.
; Inputs:
;     R0:     Bending radius of crystal (mm)
;     R_2d:   2d spacing of crystal (angstroms)
;     R_L:    Working area of crystal (mm)
;     a0:     Source-to-Xtal dist. (mm)
;     theta0: Bragg angle at center of crystal (radians)
;     Xtalarr: Array of x-axis values (-R_L/2 < Xtalarr < R_L/2)
; Outputs: (all functions of Xtalarr)
;     Lamarr: Lambda values (angstroms)
;     Tharr:  Theta values (radians)
;     Axarr:  Source-to-crystal values (mm)
;     Bxarr:  Crystal-to-Film values (mm)
;     Mxarr:  Magnification values
;     xGarr:  Film position values (mm) (in plane of film)
; Film scans can be synchronized to xGarr and then the other
; outputs will also be synchronized to the film scan.
; ******************************************************************
; BEGIN PROGRAM
; ******************************************************************
common params,R0,a0,theta0,a01,theta01
common angles,theta1
; ------------------------------------------------------------------------
; Using Xtalarr, calculate all other parameters
; ------------------------------------------------------------------------
R0=R0in
a0=a0in
a1 = 0.0
theta01 = 0.0
R_2d=R_2din
theta0=theta0in
; Tharr=Theta_x(Xtalarr)
; Axarr=A_x(Xtalarr)
; Mxarr=M_x(Xtalarr,Axarr,Tharr)
; Bxarr=B_x(Xtalarr,Axarr,Tharr)
 All_spherical_param, Xtalarr, YTALARR, Tharr, Axarr, mxarr, bxarr, omega
 Lamarr=R_2d*sin(Tharr)
;
Ytalarr=R0-SQRT(R0*R0-Xtalarr*Xtalarr)
xFarr=Xtalarr-Bxarr*cos(Omega)
yFarr=Ytalarr+Bxarr*sin(Omega)
a0x= [1.,1.,1.]* (cos(theta0)*a0)
a0y= [1.,1.,1.]* (sin(theta0)*a0)
result1=poly_fit(xFarr,yFarr,2)
print,result1
i2w,xFarr,yFarr,wdfi,/xydata,clab='Film Position', xlab = 'mm', ylab ='mm'
i2w,Xtalarr,Ytalarr,wdfi+1,/xydata,clab='Xtal Position', xlab = 'mm', ylab ='mm'
i2w,a0x,a0y,wdfi+2,/xydata,clab='Source Position', xlab = 'mm', ylab ='mm'
plo, wdfi, 3, psym = [0, 0, 8], /fill, /nl, color = [1, 2, 4], nchar = 30
; plo,[wdfi,wdfi+1,wdfi+2],nchar=30
hak,/mess
;
; Apply an orthogonal transformation into the film coordinates
;
; distance from center of the crystal to the film
b0 = a0*R0/(2*a0*sin(theta0)-R0)
; Angle of the film plane 
alpha0=atan(tan(theta0)*(a0-B0)/(a0+B0)) 
cosalpha = cos(alpha0)
sinalpha = sin(alpha0)
xGarr= xFarr*cosalpha+yFarr*sinalpha
yGarr=-xFarr*sinalpha+yFarr*cosalpha
i2w,xGarr,yGarr,wdfi+3,/xydata,clab='Film position in Film coordinates'
plo,wdfi+3,nchar=40, yr =[221, 222]
hak,/mess
i2w,xGarr,Lamarr,wdfi+4,/xydata,clab='Lambda vs. Film Position', $ 
   ylab ='Wavelength (A)', xlab = 'Distance (mm)'
tsh,wdfi+4,320.0
;Larr2=2.91980+4.21e-6*xGarr*1000.+1.845e-11*xGarr*xGarr*1000.*1000.
;i2w,xGarr,Larr2,wdfi+5,/xydata,clab='Predicted dispersion'
plo,[wdfi+4],nchar=40
hak,/mess
;
Dearr=3.23823-1.13926e-5*xGarr+2.45759e-11*xGarr*xGarr
;
i2w,Lamarr,Tharr,wdfi+10,/xydata,clab='Theta vs. Lambda'
i2w,Lamarr,Xtalarr,wdfi+11,/xydata,clab='Crystal X vs. Lambda', $
  xlab = 'Wavelength (A)', ylab = 'Crystal X (mm)'
i2w,Lamarr,Axarr,wdfi+12,/xydata,clab='Source-xtal dist. vs. Lambda'
i2w,Lamarr,Mxarr,wdfi+13,/xydata,clab='Mag. vs. Lambda (Analytic)'
i2w,Lamarr,Dearr,wdfi+14,/xydata,clab='Demag. vs. Lambda (Empirical)'
i2w,Lamarr,Bxarr,wdfi+15,/xydata,clab='Xtal-Film dist. vs. Lambda'
Dxarr=1.0/Mxarr
i2w,Lamarr,Dxarr,wdfi+16,/xydata,clab='Demag. vs. Lambda (Analytic)'
;plo,[wdfi+14,wdfi+26],nchar=40

; ******************************************************************
; END PROGRAM
; ******************************************************************
print,'The program is finished'
END
