Function Theta_Xroot,x
    common params,R0,a0,theta0,a01,theta01
    common angles,theta1
    term1=0.0
    term2=0.0
    term3=theta1
    term1=atan( (a0*sin(theta0)-R0+SQRT(R0*R0-x*x)) / (a0*cos(theta0)-x) )
    term2=atan( x / SQRT(R0*R0-x*x) )
    return,term1-term2-term3
END

Function Theta_X,x
    common params,R0,a0,theta0,a01,theta01
    term1=0.0
    term2=0.0
    term1=atan( (a01*sin(theta01)-R0+SQRT(R0*R0-x*x)) / (a01*cos(theta01)-x) )
    term2=atan( x / SQRT(R0*R0-x*x) )
    return,term1-term2
END

Function A_x,x
    common params,R0,a0,theta0,a01,theta01
    term1=0.0
    term2=0.0
    term1=(a01*cos(theta01)-x)*(a01*cos(theta01)-x)
    term2=(a01*sin(theta01)-R0+SQRT(R0*R0-x*x)) * (a01*sin(theta01)-R0+SQRT(R0*R0-x*x))
    return,SQRT(term1+term2)
END

;Function M_x,x,ax,thetax
;    common params,R0,a0,theta0,a01,theta01
;    return,R0/(2*ax*sin(thetax)-R0)
;END
;
Function B_x,x,ax,thetax
    common params,R0,a0,theta0,a01,theta01
    return,ax*R0/(2*ax*sin(thetax)-R0)
END
;
Function Omega_x,x
    common params,R0,a0,theta0,a01,theta01
    term1=0.0  ;eta
    term2=0.0  ;phi
    term1=atan( (a0*sin(theta0)-R0+SQRT(R0*R0-x*x)) / (a0*cos(theta0)-x) )
    term2=atan( x / SQRT(R0*R0-x*x) )
    return,term1-2*term2
END

pro FSSR2Dmodel_D,R0in,R_2din,R_Lin,a0in,theta0in,a01in,theta01in,Xtalarr,Lamarr,Tharr,Axarr,Bxarr,Mxarr,mMxarr,xGarr,ThFarr,wdfi=wdfi
; ******************************************************************
; Program for modeling the FSSR2D geometry
; This program generates a series of arrays for use in interpreting
; spectra generated on the FSSR2D instrument on the Z facility.
; Inputs:
;     R0:     Bending radius of crystal (mm)
;     R_2d:   2d spacing of crystal (angstroms)
;     R_L:    Working area of crystal (mm)
;     a0:     Source-to-Xtal dist. (mm)
;     theta0: Bragg angle at center of crystal (radians)
;     Xtalarr: Array of x-axis values (-R_L/2 < Xtalarr < R_L/2)
; Outputs: (all functions of Xtalarr)
;     Lamarr: Lambda values (angstroms)
;     Tharr:  Theta values (radians)
;     Axarr:  Source-to-crystal values (mm)
;     Bxarr:  Crystal-to-Film values (mm)
;     Mxarr:  Magnification values
;     xGarr:  Film position values (mm) (in plane of film)
;     ThFarr: Angle of incidence values (radians)
; Film scans can be synchronized to xGarr and then the other
; outputs will also be synchronized to the film scan.
;
; This is a modified version of FSSR2Dmodel. The original program
; performed calculations for a perfectly aligned system. In reality,
; it is very difficult to align the spectrometer properly relative
; to the source, which can change things. This program assumes that
; the crystal and film are aligned properly with respect to one
; another for a source at a0, theta0. However, the actual source
; position is allowed to vary relative to the crystal and is
; assumed to be at a01,theta01. Thus, some calculations had to be
; altered to accomodate the change.
;
; Tharr is now calculated using a01,theta01 instead of a0,theta0
; Axarr is now calculated using a01,theta01 instead of a0,theta0
; Lamarr is calculated using Tharr as before
; xFarr and yFarr are calculated using a line intercept routine
; xGarr and yGarr are calculated as before (orthogonal transform.)
; Bxarr is the dist. between Farr points and Xtal surface points
; Mxarr is calculated to be (Bxarr/Axarr)
;
; ******************************************************************
; BEGIN PROGRAM
; ******************************************************************
common params,R0,a0,theta0,a01,theta01
common angles,theta1
; ------------------------------------------------------------------------
; Using Xtalarr, calculate all other parameters
; ------------------------------------------------------------------------
R0=R0in
a0=a0in
a01=a01in
R_2d=R_2din
theta0=theta0in
theta01=theta01in
Xtalarr=DINDGEN(10001)
Xtalarr=(Xtalarr-5001)*R_Lin/10000.0
Tharr=Theta_x(Xtalarr)
Lamarr=R_2d*sin(Tharr)
Axarr=A_x(Xtalarr)
;Mxarr=M_x(Xtalarr,Axarr,Tharr)
B0=B_x(0,a0,theta0)
; ------------------------------------------------------------------------
; Calculate the intercept points with the film to get Farr points
; ------------------------------------------------------------------------
; Film plane is described by y=mfilm*x+bfilm
; mfilm=(a0*sin(theta0)-b0*sin(theta0))/(a0*cos(theta0)+b0*cos(theta0))
; bfilm=a0*sin(theta0)-mfilm*a0*cos(theta0)
;
; Ray coming off crystal described by y=mray*x+bray
; mray=-tan(omega), where omega = eta - 2*phi
; bray=R0-SQRT(R0*R0-xtalpt*xtalpt)-mray*xtalpt
; bray=ytalpt-mray*xtalpt
;
; Intercept points:
; xFarr=-(bfilm-bray)/(mfilm-mray)
; yFarr=mfilm*xFarr+bfilm
;
; Angle of incidence points:
; ThFarr=!pi-(!pi+atan(mray))+atan(mfilm)
; ------------------------------------------------------------------------
mfilm=(a0*sin(theta0)-B0*sin(theta0))/(a0*cos(theta0)+B0*cos(theta0))
bfilm=a0*sin(theta0)-mfilm*a0*cos(theta0)
mrayarr=-tan(Omega_x(Xtalarr))
Ytalarr=R0-SQRT(R0*R0-Xtalarr*Xtalarr)
brayarr=Ytalarr-mrayarr*Xtalarr
xFarr=-(bfilm-brayarr)/(mfilm-mrayarr)
yFarr=mfilm*xFarr+bfilm
Bxarr=SQRT( (xFarr-Xtalarr)*(xFarr-Xtalarr) + (yFarr-Ytalarr)*(yFarr-Ytalarr) )
;
Mxarr=Bxarr/Axarr     ; Sagittal magnification
;
ThFarr=!pi-(!pi+atan(mrayarr))+atan(mfilm)
; ------------------------------------------------------------------------
; Calculate the meridional magnification
; ------------------------------------------------------------------------
; The meridional magnification will determine the spectral resolution of
; the spectrometer. It is the ratio of the distance between the film
; intercept and the Rowland circle intercept ("b2") to the distance between
; the source position and its Rowland circle intercept ("a2").
b2arr=SQRT( (xFarr+R0*sin(Tharr)*cos(Tharr))^2 + (yFarr-R0*sin(Tharr)*sin(Tharr))^2 )
a2arr=SQRT( (a01*cos(theta01)-R0*sin(Tharr)*cos(Tharr))^2 + (a01*cos(theta01)-R0*sin(Tharr)*sin(Tharr))^2 )
mMxarr=b2arr/a2arr
;
; ------------------------------------------------------------------------
; Plot the positions of the Film, Crystal, and Source
; ------------------------------------------------------------------------
i2w,xFarr,yFarr,wdfi,/xydata,clab='Film Position'
i2w,Xtalarr,Ytalarr,wdfi+1,/xydata,clab='Xtal Position'
i2w,a0x,a0y,wdfi+2,/xydata,clab='Source Position'
plo,[wdfi,wdfi+1,wdfi+2],nchar=30
hak,/mess
; ------------------------------------------------------------------------
; Apply an orthogonal transformation to xFarr points to put them
; into the plane of the film (xGarr points)
; ------------------------------------------------------------------------
alpha0=atan(tan(theta0) * ( (a0-B_x(0,a0,theta0))/(a0+B_x(0,a0,theta0)) ) )
xGarr= xFarr*cos(alpha0)+yFarr*sin(alpha0)
yGarr=-xFarr*sin(alpha0)+yFarr*cos(alpha0)
i2w,xGarr,yGarr,wdfi+3,/xydata,clab='Film position in Film coordinates'
plo,wdfi+3,nchar=40,psym=-3
hak,/mess
i2w,xGarr,Lamarr,wdfi+4,/xydata,clab='Lambda vs. Film Position'
tsh,wdfi+4,320.0
plo,[wdfi+4],nchar=40
hak,/mess
;
Dearr=3.23823-1.13926e-5*xGarr+2.45759e-11*xGarr*xGarr
;
i2w,Lamarr,Tharr,wdfi+10,/xydata,clab='Theta vs. Lambda'
i2w,Lamarr,Xtalarr,wdfi+11,/xydata,clab='Crystal X vs. Lambda'
i2w,Lamarr,Axarr,wdfi+12,/xydata,clab='Source-xtal dist. vs. Lambda'
i2w,Lamarr,Mxarr,wdfi+13,/xydata,clab='Sagittal Mag. vs. Lambda (Analytic)'
i2w,Lamarr,Dearr,wdfi+14,/xydata,clab='Demag. vs. Lambda (Empirical)'
i2w,Lamarr,Bxarr,wdfi+15,/xydata,clab='Xtal-Film dist. vs. Lambda'
Dxarr=1.0/Mxarr
i2w,Lamarr,Dxarr,wdfi+16,/xydata,clab='Demag. vs. Lambda (Analytic)'
i2w,Lamarr,mMxarr,wdfi+17,/xydata,clab='Meridional Mag. vs. Lambda'
;plo,[wdfi+14,wdfi+26],nchar=40
;
; ----- Generate spectral resolution curve -----
Sdiam=1.0
SRarr=Sdiam*mMxarr     ; Width of source projected on the film (mm)
DL_Dxarr=Deriv(xGarr,Lamarr)
SRarr=SRarr*DL_Dxarr   ; Multiply by local dispersion derivative (-->angstroms)
SRarr=Lamarr/SRarr     ; Divide into dispersion curve to get spectral resolution (L/dL)
i2w,Lamarr,SRarr,wdfi+18,/xydata,clab='Spectral Resolution vs. Lambda (Analytic)'
plo,[wdfi+18],/xst,/yst,nchar=50
hak,/mess
; ******************************************************************
; END PROGRAM
; ******************************************************************
print,'The program is finished'
END