; ------------------------------------------------------------------------ 
; $Id: ahelp.pro,v 1.5 2009/04/17 16:50:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro ahelp, names= pname, material = pmat , update = update
;+
; NAME:
;       AHELP
;
; PURPOSE: 
;   List the ACIS parts that are currently read
; NOTE: This is the same as LIST_ACIS_PARTS.;    NAMES    - Part names
;
; FORMAT:
;    AHELP [, NAMES = names] [, MATERIAL = pmat] [, UPDATE = UPDATE]
; 
;  WHERE:
;    NAMES    - Part names
;    MATERIAL - Part material
;    UPDATE   - Read the names and materials from the ACIS database
;
;
; EXAMPLE:
;    Find out the current parts and materials.
;      AHELP
;
; MODIFICATION HISTORY:
;   Original Version, L. P. Mix
;   Added Formating from LIST_ACIS_PARTS, April 10, 2009
;   Change formatting of the help.
;-

on_error, 1
common acis_parts_com, part_name, material_name, npart_save

if n_elements(npart_save) eq 0 then npart_save = 0
if npart_save eq 0 then begin
  print, '% AHelp: No ACIS parts exist'
  return
endif
if keyword_set(update) then readacis, /update
pchrs = max(strlen(part_name))
mchrs = max(strlen(material_name))
ptit = 'Part Name'
mtit = 'Material Name'
npt = strlen(ptit)
nmt = strlen(mtit)
pwin = pchrs > npt
mwin = mchrs > nmt
form='(' + strtrim(string(pwin - npt + 2),2) + 'x,a)'

print,form='($,2a)','Part Number  ',ptit
print,form=form,mtit
dash = string(make_array((pwin > mwin),/byte,val=byte('-')))
print,'-----------  ',strmid(dash,0,pwin),'  ',strmid(dash,0,mwin)
for i=0,npart_save-1 do begin
  nsp = pwin - strlen(part_name[i]) + 2
  form = '(i7,6x,a,' + strtrim(string(nsp),2) + 'x,a)'
  print,form=form,i+1,part_name[i],material_name[i]
endfor

if arg_present(pname) then pname = part_name
if arg_present(pmat) then pmat = material_name
 end
