; ------------------------------------------------------------------------
; $Id: clear_acis_parts.pro,v 1.2 2009/04/10 17:55:43 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro clear_acis_parts

;+
; ; NAME:
;       clear_acis_parts
;
; PURPOSE:
;       Clear all ACIS parts that are currently loaded.
;
; CALLING SEQUENCE:
;       CLEAR_ACIS_PARTS
; INPUTS:
;       none
;  
; KEYWORD PARAMETERS:
;       none
;
; COMMON BLOCKS:
;       ACISCOM
;       ACIS_PARTS_COM
;
; SEE ALSO:
;      READ_ACIS, ACIS2CIM
;
; MODIFICATION HISTORY:
;      Original version, D. B. Seidel, Sept. 26, 2008
;-
;
on_error, 1

common aciscom, initacis, acis_exe
common acis_parts_com, part_name, material_name, npart_save
;
;     check to see if ACIS has been initialized
;
if (n_elements(initacis) eq 0) then init_acis $
else if (initacis eq 0) then init_acis
if initacis eq 0 then begin
  print, '% CLEAR_ACIS_PARTS: ACIS libraries are not supported by this System'
  return
endif

if npart_save eq 0 then begin
  print, '% CLEAR_ACIS_PARTS: No ACIS parts have been read'
  return
endif

;
if initacis eq 1 then begin
; HP
  junk = call_external(ACIS_EXE, 'acis_clear' )
endif else if initacis eq 2 then begin
; Sun, Linux
  junk = call_external(ACIS_EXE, 'acis_clear' )
endif else if initacis eq 3 then begin
; VMS
    junk = call_external(ACIS_EXE, 'acis_clear' )
endif else if initacis eq 4 then begin
; WIN
  junk = call_external(ACIS_EXE, 'ACIS_CLEAR@8', 0)
endif else if initacis eq 5 then begin
; MAC
  junk = call_external(ACIS_EXE, 'acis_clear')
endif
      
if junk ne 0 then begin
  print,'Error encountered in ACIS_CLEAR: ',junk
  return
endif

npart_save = 0L
part_name = ''
material_name = ''
end

