; ------------------------------------------------------------------------ 
; $Id: makebox.pro,v 1.2 2005/03/09 22:20:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; Name: MAKEBOX
; Purpose:
;     Make a parallelepiped in acis.
;        Note: MAKECONF is the same as MAKEBOX with two points.
; format: makebox, p1, p2, partname= partname, material = material
;   or    makebox, p1, p2, p3, p4, partname= partname, material = material
;    
;     where:
;        p1 are the coordinates of the origin 
;        p2 is the opposite corner of a rectangular box
;           or an adjacent corner to p1 of the general hexahedron.
;        p3 is an adjacent corner to p1 of the general hexahedron
;        p4 is an adjacent corner to p1 of the general hexahedron
;        partname - Name of part. Default = 'Box_part'
;        material - Name of material. Default = 'Conductor'
; Example:
;    Make a unit cube with one vertex at the origin.
;      Makebox, [0,0,0], [1,1,1]
; or   Makebox, [0,0,0], [1, 0,0], [0,1,0], [0,0, 1]
; or   Makeconf, [0,0,0], [1,1,1]
;
;-

 pro makebox, p1, p2, p3, p4, partname =pname, material = matname
on_error, 1
  common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save

  npar = n_params()
  if npar eq 0 then begin
    print, '% Format: MakeBox, p1, p2, p3, p4, partname = part, material= mat'
    return
  endif

;
;     check to see if ACIS has been initialized
;
  if (n_elements(initacis) eq 0) then init_acis else $
    if (initacis eq 0) then init_acis
  if initacis eq 0 then begin
    print, 'ACIS libraries are not supported by this Operating System'
    return
  endif
  npart = 0L
  if n_elements (p1) ge 3 and n_elements(p2) ge 3 then begin
    if npar eq 2 then begin
      x_acis = make_array(6, /double)
      x_acis(0:2) = p1(0:2)
      x_acis(3:5) = p2(0:2)
      if x_acis(0) gt x_acis(3) then begin
        x_acis(3) = p1(0)
        x_acis(0) = p2(0)
      endif
      if x_acis(1) gt x_acis(4) then begin
        x_acis(4) = p1(1)
        x_acis(1) = p2(1)
      endif
      if x_acis(2) gt x_acis(5) then begin
        x_acis(5) = p1(2)
        x_acis(2) = p2(2)
      endif
      npart = -2L
    endif else if ((npar eq 4) and $
       (n_elements (p3) ge 3) and (n_elements(p4) ge 3)) then begin
      npart = -1L
      x_acis = make_array(12, /double)
      x_acis(0:2) = p1(0:2)
      x_acis(3:5) = p2(0:2)
      x_acis(6:8) = p3(0:2)
      x_acis(9:11) = p4(0:2)
    endif else begin
print, '% MAKEBOX: X, Y and Z components must be provided for adjacent corners'
      return
    endelse
  endif else begin 
print, '% MAKEBOX: X, Y and Z components must be provided for opposite corners'
    return
  endelse 
  if n_elements(pname) eq 0 then pname = 'Box_Part' else begin
    pname = strtrim (pname, 2)
    len = strlen(pname)
    if len eq 0 then pname = 'Box_Part'
  endelse
  if n_elements(matname) eq 0 then matname = 'Conductor' else begin
    matname = strtrim (matname, 2)
    len = strlen(matname)
    if len eq 0 then matname = 'Conductor'
  endelse
  pnameb = [byte(pname), 0B]
  matnameb = [byte(matname), 0B]
;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_box', x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_box',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 3 then begin
; VMS
        junk = call_external(ACIS_EXE, 'acis_box',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_BOX@8',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_box',  x_acis,  $
          pnameb, matnameb, npart)
      endif

  if npart le 0 then begin
    Print, '% MakeBox: Unable to make box'
    return
  endif
  npart_save = npart
  if npart eq 1 then begin
    part_name = pname
    material_name = matname
  endif else begin
    part_name = [part_name, pname]
    material_name = [material_name, matname]
  endelse 
  return
  end


