; ------------------------------------------------------------------------ 
; $Id: makecyl.pro,v 1.2 2005/03/09 22:20:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; format: makecyl, p1, p2, r1, partname= partname, material = material
;   or    makecyl, p1, p2, r1, r2, partname= partname, material = material
;    
;     where:
;        p1 are the coordinates of the center of the bottom cylinder face
;        p2 are the coordinates of the center of the top cylinder face
;        r1 is the radius of the cylinder at p1- must be > 0.0
;        r2 is the radius of the cylinder at p2- 
;              If missing or <= 0 then r2 = r1. 
;        partname - Name of part. Default = 'Cylinder_Part'
;        material - Name of material. Default = 'Conductor'
; Example:
;    Make cylinder along the z axis, 5 units long with a radius of 1
;      makecyl, [0,0,0], [0, 0, 5], 1
; or   makecyl, [0,0,0], [0, 0, 5], 1, 1
;    Make cylinder along the z axis, 5 units long with a radius of 2 at the
;    bottom and 1 at the top. Tilt the cylinder 30 deg in the y-z plane.
;      makecyl, [0,0,0], 2.5*[0, sqrt(3.0), 1], 1, 2
;-
 pro makecyl, p1, p2, p3, p4, partname =pname, material = matname
on_error, 1
  common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save

  npar = n_params()
  if npar eq 0 then begin
    print, '% Format: MakeCyl, P1, P2, R1, R2, partname = part, material= mat'
    return
  endif

;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% MakeCyl: ACIS libraries are not supported by this System'
        return
      endif

  x_acis = make_array(12, /double)
  npart = -1L

  if npar lt 3 then begin
    print, '%MakeCyl: Three parameters are required'
    return
  endif
  if n_elements(p1) lt 3 or n_elements(p2) lt 3 or n_elements(p3) eq 0 $
    then begin
    print, '% MakeCyl: Two points and a radius are required.'
    return
  endif
  if p3(0) le 0.0 then begin
    print, '% MakeCyl: Radius must be greater than zero'
    return
  endif
  x_acis(0:2) = p1(0:2)
  x_acis(3:5) = p2(0:2)
  x_acis(6) = p3(0)
  if n_elements(p4) gt 0 then begin
    if p4(0) gt 0.0 then x_acis(7) = p4(0) else x_acis(7) = x_acis(6) 
  endif else x_acis(7) = x_acis(6)

  if n_elements(pname) eq 0 then pname = 'Cylinder_Part' else begin
    pname = strtrim (pname, 2)
    len = strlen(pname)
    if len eq 0 then pname = 'Cylinder_Part'
  endelse
  if n_elements(matname) eq 0 then matname = 'Conductor' else begin
    matname = strtrim (matname, 2)
    len = strlen(matname)
    if len eq 0 then matname = 'Conductor'
  endelse
  pnameb = [byte(pname), 0B]
  matnameb = [byte(matname), 0B]
;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_cylinder', x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_cylinder',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 3 then begin
; VMS
        file = string(p3)
        junk = call_external(ACIS_EXE, 'acis_cylinder',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_CYLINDER@8',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_cylinder',  x_acis,  $
          pnameb, matnameb, npart)
      endif

  if npart le 0 then begin
    Print, '% MakeCyl: Unable to make cylinder'
    return
  endif
  npart_save = npart
  if npart eq 1 then begin
    part_name = pname
    material_name = matname
  endif else begin
    part_name = [part_name, pname]
    material_name = [material_name, matname]
  endelse
  return
  end


