; ------------------------------------------------------------------------ 
; $Id: makesph.pro,v 1.2 2005/03/09 22:20:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; Name: MAKESPH
; Purpose:
;     Make a sphere box in acis.
;    
; format: makeconf, p1, r, partname= partname, material = material
;     where:
;        p1  - the coordinates of the origin 
;        r   - the radius of the sphere
;        partname - Name of part. Default = 'Sphere_part'
;        material - Name of material. Default = 'Conductor'
; Example:
;    Make a unit cube with one vertex at the origin.
;      Makeconf, [0,0,0], [1,1,1]
;
;-

 pro makesph, p1, p2, partname =pname, material = matname
on_error, 1
  common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save

  npar = n_params()
  if npar eq 0 then begin
    print, '% Format: MakeSph, Center, Radius, partname = part, material= mat'
    return
  endif

;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then init_acis else $
        if (initacis eq 0) then init_acis
      if initacis eq 0 then begin
        print, '% MakeSph: ACIS libraries are not supported by this System'
        return
      endif
  if n_elements (p1) ge 3 and n_elements(p2) ge 1 then begin
      x_acis = make_array(4, /double)
      x_acis(0:2) = p1(0:2)
      x_acis(3) = p2(0)
      npart = -2L
  endif else begin 
print, '% MakeSph: Center and Radius must be provided'
    npart = 0
    return
  endelse 
  if n_elements(pname) eq 0 then pname = 'Sphere_Part' else begin
    pname = strtrim (pname, 2)
    len = strlen(pname)
    if len eq 0 then pname = 'Sphere_Part'
  endelse
  if n_elements(matname) eq 0 then matname = 'Conductor' else begin
    matname = strtrim (matname, 2)
    len = strlen(matname)
    if len eq 0 then matname = 'Conductor'
  endelse
  pnameb = [byte(pname), 0B]
  matnameb = [byte(matname), 0B]
;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_sphere', x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_sphere',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 3 then begin
; VMS
        junk = call_external(ACIS_EXE, 'acis_sphere',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_SPHERE@8',  x_acis, $
          pnameb, matnameb, npart)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_sphere',  x_acis,  $
          pnameb, matnameb, npart)
      endif

  if npart le 0 then begin
    Print, '% MakeSph: Unable to make box'
    return
  endif
  npart_save = npart
  if npart eq 1 then begin
    part_name = pname
    material_name = matname
  endif else begin
    part_name = [part_name, pname]
    material_name = [material_name, matname]
  endelse 
  return
  end
