; ------------------------------------------------------------------------
; $Id: get_times.pro,v 1.4 2007/06/06 19:39:21 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_times
;+
; NAME:
;       Get_Times
;
; PURPOSE:
;	Return the time values for the current ExodusII file
;       This works only with the CDF directory routines.
;       See GET_EXOTIMES for EXODUS directory routines
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;       TIMES = GET_TIMES()
;
;
; INPUTS:
;       NONE
;
; OUTPUTS:
;       Time values for the current netCDF (ExodusII) file
;
; COMMON BLOCKS:
;       This routine accesses the common "netcdfcom" and "netcdfdatacom"
;
; EXAMPLE:
;       Get the times for the current ExodusII file.
;         Times = get_times()
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, March 20, 2003
;-

      on_error, 1
@netcdfcom.dat
;
if n_elements(initcdf) eq 0 then begin
  print, '% GET_TIMES: No netCDF files have been opened.'
  return, 0
endif 

if n_elements(time_whole) gt 0 then return, time_whole else return, 0

END
