; ------------------------------------------------------------------------ 
; $Id: load_cdf_data.pro,v 1.10 2008/02/27 22:51:09 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro load_cdf_data, dimname, dimvalue, verbose = verbose
;+ 
;  Load the cdf common block
;-
;
      on_error, 1
@netcdfcom.dat
      id = cdf_id
      fid =  fcdf_id[cdf_id]
      lcdfvalid = -1
      wdf_array = 0
      node_array = 0
      if n_elements(dimname) eq 0 then begin
        if (cdf_id  lt 0) or (cdf_id  ge max_cdffile) then begin
          print,'% LOAD_CDF_DATA: Invalid CDF ID.'
          return
        endif
        if array_cdfid(cdf_id, 1) eq 0 then begin
          print,'% LOAD_CDF_DATA: Invalid CDF ID, Ident = 0.'
          return
        endif
        glob = ncdf_inquire(fid)
        dimname = make_array(glob.ndims, /string)
        dimvalue = make_array(glob.ndims, /long)
;       get the names of the dimensional variables
        for i = 0, glob.ndims-1  do begin
          ncdf_diminq, fid, i, name, dimval
          dimname(i) = strlowcase(strtrim(name, 2))
          dimvalue(i) = dimval
        endfor
      endif
      if cdf_type(id) eq 1 then begin
;     EXODUS II parameters:
;       set rectilinear arrays to zero
;       
        apiversion = api_version[id]
        node_order = 0
        cell_order = 0
        lc_global = 0
        x1_array = 0
        x1_carray = 0
        x2_array = 0
        x2_carray = 0
        x3_array = 0.0
        z_array = 0.0
        z_array_err = 0.0
        z_carray = 0.0
        x3_carray = 0.0
        loc_triangles = 0
        tri_angles = 0

;       name_elem_var  - String array with Cell array names

        if num_elem_var[id] gt 0 then begin
          varid = ncdf_varid(fid, 'name_elem_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_elem_var= make_array(num_elem_var[id], /str)
            for i = 0, num_elem_var[id]-1 do  $
                name_elem_var(i)= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Cell Variables: ', $
              name_elem_var
          endif else begin
            print, '% LOAD_CDF_DATA: Cell variable names not found.'
            print, '                 Number of Cell Variables set to zero.'
            num_elem_var[id] = 0
          endelse
        endif

;   name_nod_var   - String array with Node array names
        if num_nod_var[id] gt 0 then begin
          varid = ncdf_varid(fid, 'name_nod_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_nod_var= make_array(num_nod_var(id), /str)
            for i = 0, num_nod_var(id)-1 do  $
              name_nod_var[i]= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Node Variables: ', $
              name_nod_var
          endif else begin
            print, '% LOAD_CDF_DATA: Node variable names not found.'
            print, '                 Number of Node Variables set to zero.'
            num_nod_var[id] = 0
          endelse
        endif

;   name_glo_var   - String array with WDF array names
        if num_glo_var(id) gt 0 then begin
          varid = ncdf_varid(fid, 'name_glo_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_glo_var= make_array(num_glo_var(id), /str)
            for i = 0, num_glo_var(id)-1 do  $
              name_glo_var(i)= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Global Variables: ', $
              name_glo_var
          endif else begin
            print, '% LOAD_CDF_DATA: Global variable names not found.'
            print, '                 Number of Global Variables set to zero.'
            num_glo_var(id) = 0
          endelse
        endif


;   num_dim        - Number of dimensions
        ok = where(dimname eq  'num_dim', nok)
        if nok eq 0 then $
          num_dim = 0 else num_dim= dimvalue(ok(0))
        if keyword_set(verbose) then print, ' Number of dimensions = ', $
            strtrim(dimvalue(ok(0)), 2)
        ok = where(dimname eq 'time_step', nok)
        if nok eq 0 then nt = 0 else nt = dimvalue(ok(0))
        if keyword_set(verbose) then print, ' Number of time steps = ', $
            strtrim(dimvalue(ok(0)), 2)
        if nt gt 0 then begin
;   time_whole     - Time values
          varid = ncdf_varid(fid, 'time_whole')
          if varid ge 0 then ncdf_varget, fid, varid, time_whole else begin
            print, '% LOAD_CDF_DATA: No time values. Check file type'
            return
          endelse
         endif else begin
           nt = 1
           time_whole = 0.0
         endelse

;   X1_Label       - Coordinate name(s) for first dimension
;   X2_Label       - Coordinate name(s) for second dimension
        if num_dim gt 0 then begin
          varid = ncdf_varid(fid, 'coor_names')
          if varid ge 0 then begin

            ncdf_varget, fid, varid, value
            x1_label = strtrim(value(*, 0), 2)
            if num_dim gt 1 then begin
              x2_label = strtrim(value(*, 1), 2)
              if num_dim gt 2 then x3_label = strtrim(value(*, 2), 2) else $
                x3_label = ''
            endif else begin
              x2_label = ''
              x3_label = ''
            endelse
          endif else begin
            x1_label = 'X'
            x2_label = 'Y'
            x3_label = 'Z'
          endelse
          lc_2d =  num_nod_var(id) +num_elem_var (id)
        endif else begin
          print, '% LOAD_CDF_DATA: Number of dimensions is zero'
          print, '               Setting number of node and cell variables to 0'
          num_nod_var(id) = 0
          num_elem_var(id) = 0
          lc_2d = 0
        endelse
;       lc_2d is the total number of node and element(cell) variables
        num_nodes = 0L
        lc_vesey = 0
        if lc_2d gt 0 then begin
          if apiversion ge api_change then begin
            varid = ncdf_varid(fid, 'coordx')
            if varid ge 0 then ncdf_varget, fid, varid, coordx else begin
              print, '% LOAD_CDF_DATA: Unable to read COORDX'
              return
            endelse
            num_nodes = n_elements(coordx)
            varid  = ncdf_varid(fid, 'coordy')
	    if varid ge 0 then ncdf_varget, fid, varid, coordy else begin
              print, '% LOAD_CDF_DATA: Unable to read COORDY'
              return
            endelse
            if num_dim eq 3 then begin
              varid  = ncdf_varid(fid, 'coordz')
	      if varid ge 0 then ncdf_varget, fid, varid, coordz else begin
                print, '% LOAD_CDF_DATA: Unable to read COORDZ'
                return
              endelse
              coord = reform([coordx, coordy, coordz], num_nodes, 3)
            endif else coord = reform ([coordx, coordy], num_nodes, 2)
          endif else begin
            varid = ncdf_varid(fid, 'coord')
            if varid ge 0 then ncdf_varget, fid, varid, coord else begin
              print, '% LOAD_CDF_DATA: Unable to read COORD'
              return
            endelse
            num_nodes = (size(coord))[1] 
          endelse
          if n_elements(coord) gt 6 then begin
;           coord = coord (num_nodes, num_dim) 
            if num_nod_var[id] gt 0 then begin
              okx = where (name_nod_var eq 'DISPLX', nokx)
              oky = where (name_nod_var eq 'DISPLY', noky)
              if nokx eq 0 then begin
; Fix for Roger Vesey!
                okx = where (name_nod_var eq 'NODE_R(t)', nokx)
                if nokx eq 1 then lc_vesey = 1
              endif
              okz = -1
            endif else begin
              nokx = 0L
              noky = 0L
              okx = -1L
              oky = -1L
              okz = -1L
            endelse
            if nokx eq 0 then lc_2d = 0  
            if ((num_dim gt 2) and (lc_2d gt 0)) then begin
              okz = where (name_nod_var eq 'DISPLZ', nokz)
              if nokz eq 0 then lc_2d = 0 else okz = okz[0]
            endif
            okx = okx(0)
            oky = oky(0)
      
            xcoord  = reform(coord(*, 0))
            if num_dim gt 1 then ycoord  = reform(coord(*, 1))
            if num_dim eq 3 then zcoord  = reform(coord(*, 2))
            coord_node = make_array(n_elements(xcoord), nt, num_dim)

            if apiversion lt api_change then begin
              varid= (num_nod_var(id) gt 0) ? ncdf_varid(fid, 'vals_nod_var') : -1

              if varid ge 0 then ncdf_varget, fid, varid, node_array 
;             help, name_nod_var
;             help, okx, coord_node, node_array, xcoord
              if okx ge 0 then begin
                if (lc_vesey) then  $
                   coord_node[*,*,0] = reform(node_array[*, okx, *]) else $
                   for iii = 0, nt-1 do coord_node[0,iii,0] = $
                     node_array[*, okx, iii] + xcoord
                  if num_dim gt 1 then for iii = 0, nt-1 do $
                  coord_node[0,iii,1] = node_array[*, oky, iii] + ycoord
                if num_dim eq 3 then for iii = 0, nt-1 do coord_node[0,iii,2] =  $
                  node_array[*, okz, iii] + zcoord
              endif else begin
                for iii = 0, nt-1 do coord_node[0, iii, 0] = xcoord 
                if num_dim gt 1 then for iii = 0, nt-1 do coord_node[0, iii, 1]= ycoord
                if num_dim gt 2 then for iii = 0, nt-1 do coord_node[0, iii, 2]= zcoord
              endelse
            endif else begin
;             API_VERSION ge api_change
              displx = 'vals_nod_var'+strtrim(okx+1,2)
              disply = 'vals_nod_var'+strtrim(oky+1,2)
              if num_dim gt 2 then displz = 'vals_nod_var'+strtrim(okz+1,2)
              if okx ge 0 then begin
                varidx =  ncdf_varid(fid, displx)
                ncdf_varget, fid, varidx, dispx
              endif
              if oky ge 0 then begin
                varidy =  ncdf_varid(fid, disply)
                ncdf_varget, fid, varidy, dispy
              endif
              if okz ge 0 then begin
                varidz =  ncdf_varid(fid, displz)
                ncdf_varget, fid, varidz, dispz
              endif
              if okx ge 0 then begin
                if (lc_vesey) then  $
                   coord_node[*,*,0] = dispx  else $
                   for iii = 0, nt-1 do coord_node[0,iii,0] = $
                     dispx[*, iii] + xcoord
                  if num_dim gt 1 then for iii = 0, nt-1 do $
                  coord_node[0,iii,1] = dispy[*, iii] + ycoord
                if num_dim eq 3 then for iii = 0, nt-1 do coord_node[0,iii,2] =  $
                  dispz[*, iii] + zcoord
              endif else begin
                for iii = 0, nt-1 do coord_node[0, iii, 0] = xcoord
                if num_dim gt 1 then for iii = 0, nt-1 do coord_node[0, iii, 1]= ycoord
                if num_dim gt 2 then for iii = 0, nt-1 do coord_node[0, iii, 2]= zcoord
              endelse
            endelse

            if num_elem_var(id) gt 0 then begin
              ok = where(dimname eq  'num_el_blk', nok)
              if nok eq 0 then begin
                num_elem_var(id) = 0
                print,'% LOAD_CDF_DATA: Number of cell variables set to zero'
                print,'         No block parameters'
              endif else begin
                num_el_blk = dimvalue(ok(0))
;               num_el_blk     - Number of blocks
                nblo = num_el_blk
                loc_triangles = make_array(/long, nblo+1)
                nelb = make_array(nblo, /long)
;               coord_cell     - cell values
                for i = 1, nblo do begin
                  ok = where(dimname eq  'num_el_in_blk'+strtrim(i,2), nok)
                  if nok gt 0 then nelb(i-1)= dimvalue(ok(0)) else $
                    nelb(i-1)=0
                endfor
                nel = total(nelb)
                triangles = 0
                if nel gt 0 and min(nelb) gt 0 then begin
                  coord_cell = make_array(nel, nt, num_dim)
                  loc_f = make_array(/long, nblo+1)
                  for i = 1, nblo do begin
                    varid = ncdf_varid(fid, 'connect'+strtrim(i, 2))
                    if varid ge 0 then begin
                      ncdf_varget, fid, varid, connect
                      connect = connect-1
                      np = n_elements(connect(*, 0))
                      if np mod 2 ne 0 then lcbad = 1
                      for j = 0L, nelb(i-1)-1 do begin
                        coord_cell(loc_f(i-1)+j, *, 0)=  $
                          total (coord_node(connect(*,j), *, 0), 1)/np 
                        if num_dim gt 1 then coord_cell(loc_f(i-1)+j, *, 1)=  $
                          total (coord_node(connect(*,j), *, 1), 1)/np 
                        if num_dim eq 3 then coord_cell(loc_f(i-1)+j, *, 2) $
                          = total (coord_node(connect(*,j), *, 2), 1)/np
                      endfor
                      if num_dim gt 1 then begin
                      if i eq 1 then begin
                        triangles = make_triangles(connect, ndim = num_dim)
                        ss = size(triangles)
                          loc_triangles(1) = ss(2)
                        if nblo gt 1 then triangles = $
                            reform (triangles, 3*ss(2), /overwrite)
                      endif else begin
                        tri= make_triangles(connect, ndim = num_dim)
                        ss = size(tri)
                        loc_triangles(i) = loc_triangles(i-1)+ss(2)
                        triangles= [ triangles, $
                             reform(tri, 3*ss(2), /overwrite)]
                      endelse
                      endif
                    endif else num_elem_var(id) = 0
                  loc_f(i) = loc_f(i-1)+nelb(i-1)
                  endfor
                  if nblo ge 1 and num_dim gt 1 then $
                    triangles = reform(triangles, 3, loc_triangles(nblo), $
                      /overwrite )
                  tri_angles = temporary(triangles)
                endif else num_elem_var(id) = 0
              endelse
            endif
            lc_2d = num_nod_var(id) +num_elem_var(id)
          endif else lc_2d = 0
        endif
        if lc_2d eq 0 then begin
            coord_node = 0
            coord_cell = 0
          print, '% LOAD_CDF_DATA: No multi-dimensional data for this file' 
        endif else if num_dim eq 3 then begin
          x3 = reform(coord_node(*,0,2))
          ntot = n_elements(x3)
          i = sort (x3)
          z = x3(i)
          jf = 0L
          nz =0L
;  values assumed equal if they differ by less than 1/2^14 =(6e-5)
;         delta = 1.0/16384 = 6.1035156e-05
          del = (z(ntot-1)-z(0))*6.1035156e-05
          for j =1L, ntot-1 do begin
            if z(j)-z(j-1) gt del then begin
              z(nz) = total (z(jf:j-1))/(j-jf)
              jf = j
              nz = nz +1L
            endif
          endfor
          z(nz) = total (z(jf:ntot-1))/(ntot-jf)
          z_array = z(0:nz)
          z_array_err = del
          x3 = reform(coord_cell(*,0,2))
          ntot = n_elements(x3)
          i = sort (x3)
          z = x3(i)
          jf = 0L
          nz =0L
;  values assumed equal if they differ by less than 1/2^14 =(6e-5)
;         delta = 1.0/16384 = 6.1035156e-05
          del = (z(ntot-1)-z(0))*6.1035156e-05
          for j =1L, ntot-1 do begin
            if z(j)-z(j-1) gt del then begin
              z(nz) = total (z(jf:j-1))/(j-jf)
              jf = j
              nz = nz +1L
            endif
          endfor
          z(nz) = total (z(jf:ntot-1))/(ntot-jf)
          z_carray = z(0:nz)

        endif
      endif else if cdf_type(cdf_id) eq 2 then begin
;   X1_Label       - Coordinate name(s) for first dimension
;   X2_Label       - Coordinate name(s) for second dimension
;   wdf_array      - Packed array of WDF arrays
;   Coord_node     - Maximum number of datasets.
;   Coord_cell     - Number of points
;   num_el_blk     - Number of datasets (WDF arrays)
;   Loc_f          - Pointer Array for each dataset
;   name_glo_var   - String array with WDF array names(clabel)
        ndset_id = ncdf_varid(fid, 'numb_dataset')
        locf_id = ncdf_varid(fid, 'pointer_array')
        wdf_id = ncdf_varid(fid, 'wdf_array')
        np_id = ncdf_varid(fid, 'numb_of_pnts')
        x1lab_id = ncdf_varid(fid, 'x1_label')
        x2lab_id = ncdf_varid(fid, 'x2_label')
        clab_id = ncdf_varid(fid, 'comment_label')
        ncdf_varget, fid, ndset_id, num_el_blk
        ncdf_varget, fid, locf_id, loc_f
        if num_el_blk gt 0 then $
           ncdf_varget, fid, wdf_id, wdf_array else wdf_array=0
        ncdf_varget, fid, np_id, coord_cell
        ncdf_varget, fid, x1lab_id,  label
        num_label = (size(label))(1)
        x1_label= strtrim(label, 2)
        ncdf_varget, fid, x2lab_id,  label
        x2_label= strtrim(label, 2)
        ncdf_varget, fid, clab_id,  label
        num_comment = (size(label))(1)
        name_glo_var = strtrim(label, 2)
        coord_node = n_elements(coord_cell)
        label = ''
      endif 
        
      lcdfvalid = array_cdfid(id, 1)
      return
      end
