; ------------------------------------------------------------------------ 
; $Id: wricdf.pro,v 1.3 2007/06/06 19:39:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  wricdf, wdfin, fileid = fid, clabel = clabel, ylabel = ylabel, $
         xlabel= xlabel
;
;+
;  This procedure writes a WDF array to a PFF file.  
;  Format: WRICDF, WD1 [, FILEID = FID] [, CLABEL = CLABEL]  [,  YLABEL = YLABEL]
;             [, XLABEL = XLABEL]
;  where:
;    wd1 - WDF array number
;    fileid - PFF file id.  If fileid  is missing or 0, the current PFF 
;             file is used.
;    clabel - dataset comment.  If not specified, the WDF comment is used.
;             Dataset comment string limited to 1024 characters.
;    xlabel - dataset x-axis label.  If not specified, the WDF x-axis 
;             label is used.  Label string limited to 64 characters.
;    ylabel - dataset block label (y-axis label).  If not specified, 
;             then the y-axis label is used as the block label.
;             Label string limited to 64 characters.
;  Examples:
;  Write WDF array 1 to the current CDF file.
;    wricdf, 1
;  Write WDF array 3 to the CDF file with a file id of 4.
;    wricdf, 3, f = 4
;  Write WDF array 1 to the PFF file with a file id of 1 and use the dataset 
;  comments and block labels indicated.
;    wricdf, 1, f= 1,c='Calculated Diode Power, Shot 4443', y='Power-W'
;-   
;
on_error, 1
;     add common blocks
@com.dat
@netcdfcom.dat

;     check to see if cdf has been initialized
;
      if n_elements(initcdf) eq 0 then init_cdf
;
      if n_elements(initwdf) eq 0 then wdfinit
;
;
      if n_params() eq 0 then begin
        print, '% Format: WRICDF, WD1 [, FILEID = FID] [, CLABEL = CLABEL]'
        print, '           [,  YLABEL = YLABEL] [, XLABEL = XLABEL] '
        return
      endif
      if n_elements(wdfin) eq 0 then begin
        print,'WDF array value must be provided'
        return
      endif
      wdf = wdfin(0)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if wdfnp(wdf-1) lt 2 then begin
         print, 'No valid data in WDF array ', strtrim(wdf,2)
        return
      endif
      if n_elements(clabel) eq 0 then clabel=wdfcomment(wdf-1)
      if n_elements(ylabel) eq 0 then ylabel=wdfylabel(wdf-1)
      if n_elements(xlabel) eq 0 then xlabel=wdfxlabel(wdf-1)
      if n_elements (fid) eq 0 then fid = array_cdfid(cdf_id, 0) $
        else fid = long (fid(0))
;
      if array_cdfid(cdf_id, 0) eq fid then begin
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endif else begin
        ok = where (fid eq array_cdfid(*, 0), nok)
        if nok ne 1 then begin
          print, '% WRICDF: ', strtrim(nok,2), ' netCDF files with a fid of ', $
            strtrim(fid, 2)
          print, '         No Data written'
          return
        endif
        cdf_id = ok(0)
        if lcdfvalid ne array_cdfid(cdf_id, 1) then load_cdf_data
      endelse
      if array_cdfid(cdf_id, 2) ne 1 then begin
        print, '% WRICDF: File not opened with write access'
        return
      endif
      if coord_node eq num_el_blk then begin
        print, '% WRICDF: File contains the maximum number of datasets.'
        return
      endif
;
      nx = long(wdfnp(wdf-1))
      get_wdfarray, array, wdf, /xydata
      nel = n_elements(array)
      nd = num_el_blk+1
      loc_f(nd) = loc_f(nd-1)+nel
      ncdf_varput, cdf_id, locf_id, loc_f(nd), offset= nd
      if nd eq 1 then wdf_array = array else wdf_array = [wdf, array]
      ncdf_varput, cdf_id, wdf_id, array, offset= loc_f(num_el_blk)
      coord_cell(num_el_blk) = nx
      ncdf_varput, cdf_id, np_id, nx, offset = num_el_blk
      x1_label(num_el_blk) = xlabel
      x2_label(num_el_blk) = ylabel
      name_glo_var(num_el_blk) = clabel
      str2byt, xlabel, p1, num_label
      ncdf_varput, cdf_id, x1lab_id, p1, offset= [0, num_el_blk]
      str2byt, ylabel, p1, num_label
      ncdf_varput, cdf_id, x2lab_id, p1, offset= [0, num_el_blk]
      str2byt, clabel, p1, num_comment
      ncdf_varput, cdf_id, clab_id, p1, offset= [0, num_el_blk]
      ncdf_varput, cdf_id, ndset_id, nd
      num_glo_var(cdf_id) = nd
      oldcdf = cdf_id
      ncdf_close, cdf_id
      cdf_id = ncdf_open(file_name(cdf_id), /write)
      file_name(cdf_id) = file_name(oldcdf)
      array_cdfid(cdf_id, *) = array_cdfid(oldcdf, *)
      array_cdfid(oldcdf, *) = 0
      file_title (cdf_id) = file_title(oldcdf)
      cdf_type (cdf_id) = cdf_type(oldcdf)
      num_glo_var (cdf_id) = num_glo_var (oldcdf)
      num_nod_var (cdf_id) = num_nod_var (oldcdf)
      num_elem_var (cdf_id) = num_elem_var (oldcdf)
;
      num_el_blk = nd
      return
      end
