; ------------------------------------------------------------------------
; $Id: get_exotimes.pro,v 1.2 2007/06/06 19:40:41 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_exotimes, tindex
;+
; NAME:
;       Get_ExoTimes
;
; PURPOSE:
;	Return the time values for the current ExodusII file
;       File must be opened with OPENEXO.
;
; CATEGORY:
;       Utility
;
; CALLING SEQUENCE:
;       TIMES = GET_EXOTIMES([INDEX])
;
;
; INPUTS:
;       INDEX:  Optional index array with values from 1 to the number of times.
;                 Invalid values will be ignored
;               Default: INDEX = UNDEFINED ; All times are returnded
;               Index is an integer.
;
; OUTPUTS:
;       Time values for the current ExodusII file
;
; COMMON BLOCKS:
;       This routine accesses the common in the file: exoduscom.dat
;
; EXAMPLE:
;       Get the times for the current ExodusII file.
;         Times = get_exotimes()
;
;       Get the first 100 odd times from a simulation and store in T_odd.
;         T_odd = get_exotimes(indgen(100)*2+1)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, October 4, 2005
;-

      on_error, 1
@exoduscom.dat

if n_elements(init_exo) eq 0 then begin
  print, '% GET_EXOTIMES: No Exodus file have been opened.'
  return, 0
endif

nt = n_elements(time_whole)
if n_elements(tindex) eq 0 then $
  if nt gt 0 then return, time_whole else return, 0
tin = round(tindex)-1
ok = where ((tin ge 0) and (tin lt nt), nok)
If (nok gt 0) then return, time_whole[tin[ok]] else return, 0
END
