; ------------------------------------------------------------------------ 
; $Id: showexo.pro,v 1.2 2007/06/06 19:40:42 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro showexo, current= p1, ufid = p2, number = number, ndset = ndset, $
              filename = filename, full = full, quiet = quiet
;
;+
;  Show all opened Exodus files and their file id.
;  Indicate the location of the current file pointer.
;  The current file id, and arrays of the user file ids and filesnames
;    can be received as keywords.
;  Format: SHOWCDF [, current = current] [, ufid = ufid]
;              [, filename = filename]  [, NUMBER= number]
;  where
;         current - user file id of the current file
;         ufid    - array of user file id's
;         filename- Array of file names corresponding to the ufid's
;         number  - number of files open
;         full    - Show all information about the file
;                   This includes title, type, and the number of datasets
;         ndset   - Number of datasets; ndset = ndset(number, 3)
;                   where the second index is represents the number of 
;                     global/wdf arrays, node arrays, and cell/element arrays
;         quiet   - If present and not zero, send no output to terminal
;  Example:
;  Show the status, including file id, of all opened Exodus files
;    SHOWEXO
;  
;  Get the filenames and user file id's for all open files
;  Do not send any output to the terminal
;    SHOWEXO, current = cur, ufid=ufid, num= n, filename= filename, /quiet
;
;  Written by:     L. P. Mix, October 10, 2005
;-
;
on_error, 1
@exoduscom.dat

      if n_elements(init_exo) eq 0 then begin
        print, '% SHOWEXO: No Exodus files have been opened.'
        return
      endif
;
      ok = where (array_exoid(*, 1) ne 0, number)
      if ((exo_id lt 0) or (number eq 0)) then begin
        PRINT, '% SHOWEXO: No Exodus files are open'
        return
      endif
      filename = file_name(ok)
      p2 = array_exoid(ok)
      p1 = array_exoid(exo_id)
      ndset = make_array(number, 3)
      ndset (*, 0) = num_glo_var(ok)
      ndset (*, 1) = num_nod_var(ok)
      ndset (*, 2) = num_elem_var(ok)
      if keyword_set(quiet) then return
      print, '  FID  Current  File Name'
      print, '  ---  -------  ---------'
      for i = 0, number-1 do begin
        j = ok(i)
        if j eq exo_id then cur ='===>>' else cur = '     '
        print, format='(i5,3x,a5,a2,  a)', $
            array_exoid(j), cur, '  ', file_name(j)
        if keyword_set(full) then begin
          print, '   Title: ', file_title(j)
          print, '     WDF arrays= ', strtrim(num_glo_var(j), 2), $
             ', Node arrays = ', strtrim(num_nod_var(j), 2), $
             ', Cell arrays = ', strtrim(num_elem_var(j), 2)
        endif
      endfor
      end
