; ------------------------------------------------------------------------
; $Id: get_grid_connect.pro,v 1.3 2009/09/02 16:39:15 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       GET_GRID_CONNECT
;
; PURPOSE:
;       Return the connection points for the grid
;
; CALLING SEQUENCE:
;
;       GET_GRID_CONNECT, CONNECT, LOCX, LOCV
;
; where:
;       CONNECT:     An array of node numbers for the grid
;       LOCX: The average value of the quantity obtained by averaging
;                      the element value for every element that touches the node.
;       Triangles:     Values of the nodes for each element
;                      Size is [ndim+1,  Num_Elements]
;                      TRIANGLE values are zero based, ie,  valuse are [0: num_nodes-1]
;           Note: Num_elements may be larger than the problem 
;                 if some of the problem elements are not tets
;                 or triangles.
; EXAMPLE:
;       Calculate the value of the electric field at each node and return the 
;       connectivity information in the variable, TRI
;         ELEM2NODE, ELEC_X, NodeEX, tri = tri
;       
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, October 12, 2005
;
;-
;
  pro ELEM2NODE, v_elem, v_node, triangles = tets

  on_error, 1
; add common block
@exoduscom.dat
;
  v_node =0
  tets = -1
  if n_elements(init_exo) eq 0 then begin
    print, '% ELEM2NODE: No Exodus files have been opened.'
    return
  endif
  if exo_id lt 0 then begin
    print, '% ELEM2NODE: No Exodus files are open.'
    return
  endif
  nblo = num_elem_blk[exo_id]
  if nblo lt 1 then begin
    print, '% ELEM2NODE: No blocks in this dataset.'
    return
  endif

  fid = fexo_id[exo_id]
  ndim = num_dim[exo_id]
  lctets = (arg_present(tets)) ? 1 : 0
  v_node = make_array(num_nodes, value = 0.0d0)
  i_node = make_array(num_nodes, value = 0)
  itet = 0
  if (lctets) then begin
    tets = make_array(ndim+1, loc_v[nblo],  /long)
    mxtet = loc_v[nblo]
  endif else tets = -1
  for i = 1, nblo do begin
    varid = ncdf_varid(fid, 'connect'+strtrim(i, 2))
    ncdf_varget, fid, varid, connect
    connect = connect-1
    ss = size(connect)
    if ss[0] eq 1 then ss=[2, ss[1], 1, ss[2], ss[3]]
;   nnode is number of nodes per connect block
;   nelemb is the number of connection blocks in this block
    nnode = ss[1]
    nelemb = ss[2]
    for j = 0L, nelemb-1 do $
      v_node[connect[*, j]] += v_elem[connect[*, j]]
    for j = 0L, nelemb-1 do i_node[connect[*, j]] += 1

    if lctets then begin
      if ndim eq 2 then begin
        
      if nnode gt ndim +1 then begin
        if ndim eq 2 then begin
           tri = make_triangles (connect)
        endif else tri = make_array(ndim+1, size_V[i-1])
      endif else tri = temporary(connect)
      ss = size(tri)
      if ss[2]+itet gt mxtet then begin
        mxtet = mxtet+ mxtet
        tets = reform (tets, n_elements(tets), /over)
        tets = [tets, tets]
        tets = reform(tets, ndim+1, mxtet, /over)
      endif
      tets[*, itet: itet+ss[2]-1] = tri
      itet = itet+ss[2]
    endif
  endfor
  ok = where (i_node eq 0, nok)
  if nok gt 0 then i_node[ok] = 1

  v_node = v_node/i_node
  if lctets && (itet lt mxtet) then tets = tets[*, 0:itet-1]
  end
