 pro write_exo_header, cdfid, nblock = nblock, nnode = nnode, nelem = nelem, $ 
 blo_type = blo_type, title = title, numnodevar= numnodevar, $
    numelemvar = numelemvar, ierror = ierror
;
; Write dimensions and header information for an Exodus II file
; Strings limited to 32 characters.

@write_exo_header_com.dat
 on_error, 1
 npar = n_params()
 ierror =1L
 if npar eq 0 then begin
   print, '% Format: WRITE_EXO_HEADER: CDFID, NBLOCK= nblock, NNODE= nnode, '
   print, '          NELEM = nelem, BLO_TYPE = blo_type, TITLE = title,'
   print, '          NUMNODEVAR = numnodevar, NUMELEMVAR = numelemvar,'
   return
 endif
 
 num_elem= total(nelem)

 if n_elements(cdfid) eq 0 then begin
   print, '% WRITE_EXO: CDF File ID must be provided'
   return
 endif
 fid = cdfid
 if n_elements(nblock) eq 0 then begin
   print, '% WRITE_EXO: Number of blocks must be specified'
   return
 endif
 nblock = long(nblock)

 numnodevar = (n_elements(numnodevar) eq 0) ?  0L : long(numnodevar)
 numelemvar = (n_elements(numelemvar) eq 0) ?  0L : long(numelemvar)
 if n_elements(title) eq 0 then title = 'IDL/ExodusII: '+systime(0)
; atitle = bytarr(strlen(title)+1)
; atitle[0] = byte(title)
 atitle = string(title)

 block_types = ['SPHERE', 'SHELL', 'TETRA', 'HEX']
 block_elements=[1L, 4L, 4L, 8L]

 ncdf_attput, cdfid, /global, 'api_version', 4.68000
 ncdf_attput, cdfid, /global, 'version', 4.68000
 ncdf_attput, cdfid, /global, 'floating_point_word_size', 4L
 ncdf_attput, cdfid, /global, 'file_size', 1L
 ncdf_attput, cdfid, /global, 'title', atitle

 lensid = ncdf_dimdef(cdfid, 'len_string', 33L)
 lenlid = ncdf_dimdef(cdfid, 'len_line', 81L)
 id4   = ncdf_dimdef(cdfid, 'four', 4L)
 timeid = ncdf_dimdef(cdfid, 'time_step', /unlimited)
 dimid = ncdf_dimdef(cdfid, 'num_dim', 3L)
 nnodeid = ncdf_dimdef(cdfid, 'num_nodes', long(nnode))
 nelemid = ncdf_dimdef(cdfid, 'num_elem', num_elem)
 nbloid = ncdf_dimdef(cdfid, 'num_el_blk', long(nblock))
 nnodelemss1id = make_array(nblock, /long)
 nconid = make_array(nblock, /long)
 elemtype= make_array(nblock, /long)
 for i = 0, nblock-1 do begin
   elemtype[i] = get_match(blo_type[i], block_types, nelem = nel)
   if nel ne 1 then begin
     print, '% WRITE_EXO: Unable to find block type: "'+blo_type[i]+'"'
     print, '%            Only the following block types are supported'
     for i = 0, n_elements(block_types)-1 do print, block_types[i]
     return
   endif
   nconid[i] = ncdf_dimdef(cdfid, 'num_el_in_blk'+strtrim(i+1,2), nelem[i])
   nnodelemss1id[i] = ncdf_dimdef(cdfid, 'num_nod_per_el'+strtrim(i+1,2), block_elements[elemtype[i]])
 endfor
 if numnodevar gt 0 then $
   numnodvarid = ncdf_dimdef(cdfid, 'num_nod_var', numnodevar)
 if numelemvar gt 0 then $
   nelemvarid = ncdf_dimdef(cdfid, 'num_elem_var', numelemvar)
;
; End of Dimensions
 timewholeid = ncdf_vardef(cdfid, 'time_whole', timeid)
 ebsid = ncdf_vardef(cdfid, 'eb_status', nbloid, /long)
 ebpid = ncdf_vardef(cdfid, 'eb_prop1', nbloid, /long)
 ncdf_attput, cdfid, ebpid, 'name', 'ID'
 coordxid = ncdf_vardef(cdfid, 'coordx', nnodeid )
 coordyid = ncdf_vardef(cdfid, 'coordy', nnodeid )
 coordzid = ncdf_vardef(cdfid, 'coordz', nnodeid )

 coordnamid = ncdf_vardef(cdfid, 'coor_names', [lensid, dimid], /char)
 ebnameid = ncdf_vardef(cdfid, 'eb_names', [lensid, nbloid], /char)
 elemmapid = ncdf_vardef(cdfid, 'elem_map', nelemid, /long)
 elemnummapid = ncdf_vardef(cdfid, 'elem_num_map', nelemid, /long)
 nodenummapid = ncdf_vardef(cdfid, 'node_num_map', nnodeid, /long)
 connectid = make_array(nblock, /long)
 for i =0, nblock-1 do begin
   connectid[i] = ncdf_vardef(cdfid, 'connect'+strtrim(i+1,2), [nnodelemss1id[i], nconid[i]], /long )
   ncdf_attput, cdfid, connectid[i], 'elem_type', block_types[elemtype[i]]
 endfor
 if numnodevar gt 0 then begin
   nodevarid = make_array(numnodevar, /long)
   for i = 0, numnodevar-1 do $
     nodevarid[i] = ncdf_vardef(cdfid, 'vals_nod_var'+strtrim(i+1, 2), [nnodeid, timeid] )
   nodenamid = ncdf_vardef(cdfid, 'name_nod_var', [lensid, numnodvarid], /char )
 endif
 if numelemvar gt 0 then begin
   elemvarid = make_array(numelemvar, nblock, /long)
   for i = 0, numelemvar-1 do begin
     for j = 0, nblock-1 do $
     elemvarid[i, j] = ncdf_vardef(cdfid, 'vals_elem_var'+strtrim(i+1, 2)+'eb'+strtrim(j+1, 2), $
         [nconid[j], timeid] )
   endfor
   elemnamid = ncdf_vardef(cdfid, 'name_elem_var', [lensid, nelemvarid], /char )
 endif

 ncdf_control, cdfid, /endef
 
 ncdf_varput, cdfid, ebsid, make_array(nblock, value = 1L)
 ncdf_varput, cdfid, ebpid, 1+lindgen(nblock)
 ncdf_varput, cdfid, elemmapid, make_array(num_elem, value =0L)
 ncdf_varput, cdfid, elemnummapid, lindgen(num_elem)+1
 ncdf_varput, cdfid, nodenummapid, lindgen(nnode)+1
 ierror =0
 end
