; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro comp_ufiles, filea, fileb, WDF = WDF

;+
; NAME:
;       COMP_UFILES
;       Compare two PFF files with uniform data, dataset by dataset
;
; CALLING SEQUENCE:
;
;       COMP_FILES, FILE1, FILEB
; Where:
; File1:    Name of first file 
; File2:    Name of second file
; Current version limited to 300 datasets
; Maximum and minimum differences are stored in the following WDF arrays
; 601   max x difference
; 602   min x difference
; 603   max y difference
; 604   min y difference
; 
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., October 22, 2008
;-
on_error, 1

if get_maxwdf() lt 1000 then set_maxwdfarray, 1000

openpff, filea, ndset = ndseta
re, 1, ''
closepff

openpff, fileb, ndset = ndset
if ndset ne ndseta then begin
  print, 'Number of datasets is different - quit'
  print, ndseta, ndset
  return
endif else if ndset eq 0 then begin
  print, 'Number of datasets is ZERO'
  return
endif

; make arrays to hold the differences
maxy = make_array(ndset)
miny = maxy
startx = maxy
deltax = maxy
numb = make_array(ndset, /long)

for i = 0, ndset-1 do begin
  re, 301, '^'+wdcomment(i+1)+'$', narray = na
  if na eq 0 then $
    print, '% COMP_UFILES: Unable to match dataset comment: ', wdcomment(i+1) $
  else begin
    get_wdfarray, xa, i+1
    get_wdfarray, xb, 301
;   range of data
    rang =max(xa[2:*], min = m) - m
    startx[i] = xa[0]-xb[0]
    deltax[i] = xa[1]-xb[1]
    numb[i]= n_elements(xa) - n_elements(xb)
    miny[i] = min(xa[2:*]-xb[2:*], max = m)/rang
    maxy[i] = m/rang
  endelse
endfor
closepff
if ndset lt 3 then begin
  print, 'Start X =', startx
  print, 'Delta X =', deltax
  print, 'MIN Y =', miny
  print, 'MAX Y =', maxy
  print, 'NUMB Dif =', numb
  return
endif

i2w, [1., 1], startx, 601, clab= 'Start X'
i2w, [1., 1], deltax, 602, clab= 'Delta X'
i2w, [1., 1], maxy, 603, clab= 'Max Y'
i2w, [1., 1], miny, 604, clab= 'Min Y'
i2w, [1., 1], numb, 605, clab= 'Number Diff'

  print, 'Start X Maximum =', max(startx, /abs)
  print, 'Delta X Maximum =', max(deltax, /abs)
  print, 'MIN Y Maximum =', max(miny, /abs)
  print, 'MAX Y Maximum =', max(maxy, /abs)
  print, 'NUMB Dif Maximum =', max(numb, /abs)

plo, 601, 5, /color, /over
end
