 pro make_exo_conductor,  fileout, filein
  on_error,1
@write_exo_header_com.dat

  if n_elements (fileout) eq 0 then fileout = 'qsgeo.exo'
  if n_elements (filein) eq 0 then fname = 'qsgeo.pff' else fname = filein
  info = file_info(fname)
  if (info.exists + info.read) ne 2b then begin
    print, '% MAKE_EXO_CONDUCTOR: File does not exist with read permission'
    print, '           File Name = ', fname
    print, '           File Exists =', info.exists
    print, '           Read Permission =', info.read
    return
  endif
  openpff, fname, ndset = ndset 

  ndset = ndset 

  if n_elements(fileout) eq 0 then begin
    i = strpos(fname, '.pff')
    fileout = (i gt 0) ? strmid(fname, 0, i) : fname
    fileout = fileout + '.exo'
  endif
  fileout = find_newfile(fileout, /quiet)

; get the names of the conductors
  get_dir, 3, ndset, comment = comment
;  glob_planes 
  ncon = ndset-2
  help, ncon

  dset = 3L+lindgen(ncon)
  loc_planes = make_array(ncon+1, value =0L)
  glob_planes = make_array(3, info.size/6)
  cnames = make_array(33, ncon, /byte )
  for i = 0L, ncon-1 do begin
    readvtx, plane, dset[i], /cond , spare = spare, clab = clab
    cnames[0,i]=byte(strtrim (clab, 2))

    ss = size(plane)
    if (ss[0] ne 2) or (ss[1] ne 3) then begin
      print, '% MAKE_EXO_CONDUCTOR: Conductor plane is not correct'
      help, plane
      return
    endif
    if ss[2] lt 4 then begin
      print, '% MAKE_EXO_CONDUCTOR: Unable to read dataset: ', dset[i]
      help, plane
      return
    endif
    loc_planes[1+i] = loc_planes[i]+ss[2]
;    plane = reform(plane, ss[4], /overwrite)
    
    glob_planes[0, loc_planes[i]] = plane 
  endfor
  closepff
  
  nsides = loc_planes[ncon]
  glob_planes = temporary(glob_planes[*, 0:loc_planes[ncon]-1])
  get_uniq_nodes, glob_planes, newcoord, ind
  if spare[0] eq 2 then begin
    ctheta = reform(cos(newcoord[1, *]))
    stheta = reform(sin(newcoord[1, *]))
    newcoord[1, *] = newcoord[0, *]*stheta
    newcoord[0, *] = newcoord[0, *]*ctheta
    ctheta =0
    stheta =0
  endif

  nplanes = nsides/4
 
  newcoord = transpose(newcoord)
  ss = size(newcoord)
  num_nodes = ss[1]
  loc_elem=loc_planes/4L
  num_elem = loc_elem[1:ncon]-loc_elem[0:ncon-1]
  print, ' num_elem ', num_elem 
  
  dlconnect = reform (ind, 4, nplanes, /overwrite) +1L
  block_type = make_array(ncon, value = 'SHELL', /string)
  title = 'IDL/ExodusII: '+systime(0)
  cdfid = ncdf_create(fileout, clobber = clobber)
  help, cdfid
  write_exo_header, cdfid, nblock = ncon, nnode = num_nodes, nelem= num_elem, $
    blo_type = block_type, title = title, numnodevar =0, numelemvar=0
  help, ncon
  ncdf_varput, cdfid, coordxid, newcoord[*, 0]
  ncdf_varput, cdfid, coordyid, newcoord[*, 1]
  ncdf_varput, cdfid, coordzid, newcoord[*, 2]
  coord_names = bytarr(33, 3)
  
  coord_names[0:5, 0] = byte('X axis')
  coord_names[0:5, 1] = byte('Y axis')
  coord_names[0:5, 2] = byte('Z axis')
help, ebnameid, cnames
  ncdf_varput, cdfid, ebnameid, cnames
  ncdf_varput, cdfid, coordnamid, coord_names
  for i = 1, ncon do $
    ncdf_varput, cdfid, connectid[i-1], dlconnect[*, loc_elem[i-1]:loc_elem[i]-1]

  ncdf_close, cdfid
  end
