pro read_ndata , sample = test, threshold = threshold, redo = redo

; 
; Threshold = default = 10%
; Redo, if set, use stored images 
;               Assume that the data is in get_maxstr()-8
;               Assume the focus file is in get_maxstr()-7
;
on_error, 1

if n_elements(threshold) eq 0 then threshold = 0.075
if threshold gt 1.0 then threshold = 0.075

; Modified Streak camera Channels
xarray = findgen(85) * 350 + 700
xarray[9:17] = xarray[9:17]+650
xarray[18:41] = xarray[18:41]+1300
xarray[42:65] = xarray[42:65]+1950
xarray[66:74] = xarray[66:74]+2600
xarray[75:83] = xarray[75:83]+3250
; add an extra
numb_fiber = 84 ; 18+ 48 + 18
xarray[84]= xarray[83]+ 35000

xspace = [findgen(18), findgen(48)*0.5+ 18, 18+24.5+findgen(18)]

; Comb
comb = make_array(5)
comb[0]= total(xarray[8:9])*0.5
comb[1]= total(xarray[17:18])*0.5
comb[2]= total(xarray[41:42])*0.5
comb[3]= total(xarray[65:66])*0.5
comb[4]= total(xarray[74:75])*0.5
; background
backgr = make_array(2)
backgr[0]= 350
backgr[1]= xarray[83]+350

; scale for images (microns/pixel)
Xscale = 18.044
xdata = xarray/xscale
xdata = xdata -xdata[0]

; work arrays for wdf and images

work = get_maxwdf()-12
wstr = get_maxstr()-8

if keyword_set(redo) then begin
  img = wstr+1
  flt = wstr+3
  wstr = wstr+4
  datafile = strcomment(img)
  focfile = strcomment(flt)
endif else begin

  if keyword_set(test) then begin
    datafile = '/remote/lpmix/consult/sportil/New_data_011605/flatshot676.TIF'
    backfile = '/remote/lpmix/consult/sportil/New_data_011605/postshot676.TIF'
    focfile = '/remote/lpmix/consult/sportil/New_data_011605/shot679TRSD.TIF'
    ff_backfile = '/remote/lpmix/consult/sportil/New_data_011605/postshot676.TIF'
  endif else begin
    datafile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
      /must_exist, Title= 'Pick the DATA file')
    backfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the Background file', path = oldpath)
    Focfile = dialog_pickfile(default_extension = '.tif', get_path= oldpath, $
       /must_exist, Title= 'Pick the focus file', path = oldpath)
    ff_backfile = dialog_pickfile(default_extension = '.tif', get_path=oldpath, $
       /must_exist, Title= 'Pick the focus background file', path = oldpath)
    if strlen(ff_backfile) lt 1 then begin
      Print, '% READ_DATA: No background file selected for the Focus File'
      print, '             Background for data file will be used'
      ff_backfile= backfile
    endif
  endelse

; Read the data
  img = read_tiff(datafile)
  bkg = read_tiff(backfile)
  ff = read_tiff(focfile)
  ff_bkg = read_tiff(ff_backfile)

; filter the data
  print, ' Filtering the data'
  print, ' Image data range = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  img = median(img, 3)
  print, ' Image data range after filter = [ ', strtrim(min(img, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Background data range = [ ', strtrim(min(bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  bkg = median(bkg, 3)
  print, ' Background data range after filter = [ ', $
    strtrim(min(bkg, max = m), 2), ', ', strtrim (m, 2), ']'
  print, ' Focus data range = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff = median(ff, 3)
  print, ' Focus data range after filter = [ ', strtrim(min(ff, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  print, ' Focus background data range = [ ', strtrim(min(ff_bkg, max = m), 2), $
    ', ', strtrim (m, 2), ']'
  ff_bkg = median(ff_bkg, 3)
  print, ' Focus background data range after filter = [ ', $
    strtrim(min(ff_bkg, max = m), 2), ', ', strtrim (m, 2), ']'
; Save the image
  img = (img - bkg)
  arr2str, wstr, float(img) , clab = 'image - '+datafile, file = datafile
  img = wstr & wstr = wstr+1
  alignstr, img, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  img = wstr
  wstr = wstr+1
  ff = (ff-ff_bkg)
  arr2str, wstr, float(ff) , clab = 'Flat-Field - '+datafile, file = datafile
  flt = wstr & wstr = wstr+1
; plotstr, flt, cn=2
   alignstr, flt, wstr, angle= angle , width = 10./2000, yr= [500, 1500]
  flt= wstr
  wstr = wstr+1
endelse

imag = get_array(img, -1)
flat = get_array(flt, -1)
ss = size(imag) 
sf = size(flat) 

delwdf, work, /quiet

print, '*** Select the signal region ***'
while (wdvalid(work, /valid, /quiet))[0] lt 0 do $
  linstr, img, work, /vert, wid = 120
filt, work, work+1, d = 8.0

delwdf, work+2, /quiet
while (wdvalid(work+2, /valid, /quiet))[0] lt 0 do $
  linstr, flt, work+2, /vert, wid = 120
filt, work+2, work+3, d = 8.0


st = make_array(128, /long)
en = st
stf = st
enf = st
middle = Make_array(128, /long, value = 65536)
middlef = middle
imag_val = middle
flat_val = middle

nfib = 0L

; Width of fiber = 9
wfiber= 9.5
skip = 11.5
nex  = 20
bnex = 55
hwid = wfiber/2
finish = 0
maxx = sf[2] -31
ans = ''
yes_no_ques, lcno, 'Is the first channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first visible data channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif
zoomw, work+1, xc = xc, instructions = $
  'Use right mouse button in right window to indicate the first data channel'

mid = xc[0]
offset = mid -xdata[star-1]
idata = xdata+offset

lef = mid - skip
ri = mid+skip
junk = mx(work+1, lef= lef, ri= ri, /quiet, min = mm, time = nt)

while ri le maxx do begin
  peak = junk
  time = round(nt)
  m = min(abs(idata-nt), jmid)
;
  middle[nfib] = time
  st[nfib] = time - hwid
  en[nfib] = time + hwid  
  imag_val[nfib] = jmid

  fudge = nt -idata[jmid]
  if abs(fudge) gt 2 then begin
    idata = idata +fudge
    fudge =0
  endif
  nfib= nfib+1
  if nfib eq numb_fiber then break
  jmid = jmid+1
  newmid = idata[jmid]+fudge
  lef = newmid - skip
  ri =  newmid + skip
  repeat begin
    junk = mx(work+1, lef= lef, ri= ri, min= mm, /quiet, time = nt) 
    if (nt-lef lt 2.1) || (ri-nt lt 2.1) then junk =0.0
    ri = ri + nex
  endrep until junk gt 0.4*peak || ri gt maxx
endwhile

i2w, middle[0:nfib-1], 60, clab = 'Middle for images'
image = make_array(ss[2])
for i = 0, nfib-1 do image[st[i]:en[i] ] = 1.0
i2w, image, work+4, clab = 'Image Fibers'
nfibd = nfib
; now do the same for the flat field
help, nfib
yes_no_ques, lcno, 'Is the first channel visible?','Fiber Check'
star = 1
if lcno ne 0 then $
  read,  prompt='Enter the first visible data channel (1 to 84): ', star
if star lt 1 or star gt 84 then begin
  print, '% READ_NDATA: The first visible line may not have a value of ', star
  print, '              Use "READ_NDATA, /REDO" to rerun'
  return
endif

zoomw, work+3, xc = xc, instructions = $ 
  'Use right mouse button in right window to indicate the first data channel'
mid = xc[0]
offset = mid -xdata[star-1]
idata = xdata+offset

if imag_val[0] gt 0 then mid = idata[imag_val[0]]
lef = mid-skip
ri = mid +skip
nfib =0
junk = mx(work+3, lef= 0, ri= ri, min = mm, /quiet, time = nt)
nok =0
while ri le maxx do begin
  peak = junk
  time = round(nt)
  m = min(abs(idata-time), jmid)
;
; we need one background channel for every data channel
; jmid will be at least as large as imag_val because of the 
; way the middle is set.
;
  if jmid gt imag_val[nfib] then begin
    delt = idata[jmid] - nt
    jmid = imag_val[nfib]
    nt = idata[jmid] -delt
    time = round(nt)
  endif
  middlef[nfib] = time
  stf[nfib] = time - hwid
  enf[nfib] = time + hwid
  flat_val[nfib] = jmid
  fudge = nt -idata[jmid]
  if abs(fudge) gt 2 then begin
    idata = idata +fudge
    fudge =0
  endif
  nfib= nfib+1
  jmid = jmid+1

  
  if nfib eq nfibd then break
  newmid = idata[imag_val[nfib]] + fudge
  lef = newmid - skip
  ri =  newmid + skip

  repeat begin
    junk = mx(work+3, lef= lef, ri= ri, min = mm, /quiet, time = nt)
    if (nt-lef lt 2.1) || (ri-nt lt 2.1) then junk =0.0
    ri = ri + nex
  endrep until junk gt 0.4*peak || ri gt maxx
endwhile

image = make_array(sf[2])
for i = 0, nfib-1 do image[stf[i]:enf[i] ] = 1
i2w, image, work+5, clab= 'Flat Field Fibers'
i2w, middlef[0:nfib-1], 61, clab= 'Middle for focus'
nfibf = nfib
; check data and focus for the same number of points
; and see if they are identical

if nfibf ne nfibd then begin
  print, '% READ_NDATA: Number of fibers is different'
  help, nfibf, nfibd
  print, '              Please contact Paul Mix if this is not resolved'
  return
endif

; stf = stf[0:nfib-1]
; enf = enf[0:nfib-1]
; st = st[0:nfib-1]
; en = en[0:nfib-1]

; print, st, en, stf, enf
; Convert fiber positions to spatial positions
;
values = xspace[imag_val[0:nfib-1]]
image = make_array(ss[1], nfib)
for i = 0, nfib-1 do image[*, i] = total(imag[*, st[i]:en[i]], 2)
ffield = make_array(sf[1], nfib)
for i = 0, nfib-1 do ffield[*, i] = total(flat[*, stf[i]:enf[i]], 2)
arr2str, wstr, image, get_array(img, 1), values,  $
  xlab = 'Distance', ylab ='Fiber Position (mm)', $
    clab = 'Fiber Data - ' + datafile
arr2str, wstr+1, ffield, get_array(flt, 1), values,  $
  xlab = 'Distance', ylab ='Fiber Position (mm)', $
    clab = 'Flat Field Data - ' + focfile
nor = total(ffield, 1)
for i = 0, nfib-1 do image[*, i] = image[*, i]/nor[i]
arr2str, wstr+2, image, get_array(img, 1), values,  $
  xlab = 'Distance', ylab ='Fiber (mm)', $
    clab = 'Normalized Fiber Data - ' + datafile
i2w, nor, work+6, clab='Flat Field Total', xlab= 'Fiber', ylab = 'Value'
end
