; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
; +
;  Show current NTOF data
;  Format: SHOWNTOF [, FILENAME]
;  where
;         FILENAME- FileName that can be used to store the data.
;                   If FILENAME is not present, NTOF data is sent to the screen.
;  Example:
;  Show the current NTOF data
;    showntof
;
;-

 pro showntof, filename

on_error,1
@ntof_common.dat

If n_elements(ntof_init) eq 0 then begin
  print, '% SHOWNTOF:  No NTOF data to print.'
  return
endif

if n_elements(filename) eq 0 then unit =-1 else begin
  openw, unit, filename, /get_lun, error = ier
  if ier ne 0 then begin
    print, '% SHOWNTOF:  Unable to open file: '+strtrim (filename, 2)
    return
  endif
endelse

Printf, unit, 'Neutron Time of Flight Data'
printf, unit, '  Shot Number: '+strtrim(shot_number, 2)
Printf, unit,  systime(0)
for i = 0, numb_los-1 do begin
  printf, unit, ' '
  printf, unit, 'LOS #'+strtrim(i, 2)+' '+name_los[i]
  for j = 0, numb_detect[i]-1 do begin
    printf, unit, 'Detector '+strtrim(detect_name[j, i], 2)+', Distance = ' + $
      strtrim(detect_dist[j, i], 2)+' m'
    printf, unit, 'DD neutron peak:'
    printf, unit, '   Peak Time(ns): '+  strtrim(dd_time[j, i].time, 2) + $
                  ', Est Error: '+strtrim(dd_time[j, i].t_error, 2)
    printf, unit, '   Peak Width(ns): '+  strtrim(dd_time[j, i].width, 2) + $
                  ', Est Error: '+strtrim(dd_time[j, i].width_error, 2)
    printf, unit, '   Ion Energy(keV): '+  strtrim(dd_time[j, i].energy, 2) + $
                  ', Est Error: '+strtrim(dd_time[j, i].energy_error, 2)
    printf, unit, '   Peak Area: '+  strtrim(dd_time[j, i].area, 2)
    printf, unit, 'DT neutron peak:'
    printf, unit, '   Peak Time(ns): '+  strtrim(dt_time[j, i].time, 2) + $
                  ', Est Error: '+strtrim(dt_time[j, i].t_error, 2)
    printf, unit, '   Peak Width(ns): '+  strtrim(dt_time[j, i].width, 2) + $
                  ', Est Error: '+strtrim(dt_time[j, i].width_error, 2)
    printf, unit, '   Ion Energy(keV): '+ strtrim( dt_time[j, i].energy, 2) + $
                  ', Est Error: '+strtrim(dt_time[j, i].energy_error, 2)
    printf, unit, '   Peak Area: '+  strtrim(dt_time[j, i].area, 2)
  endfor
  printf, unit, ''
  printf, unit, 'Neutron Energy from Peak Spacing'
  printf, unit, '  DD Energy(MeV): '+strtrim(fit_energy[0, i], 2)
  printf, unit, '  DT Energy(MeV): '+strtrim(fit_energy[1, i], 2)
  printf, unit, 'Neutron Creation Time from Peak Spacing'
  printf, unit, '  DD Bang Time(ns): '+strtrim(bang_time[0, i], 2)
  printf, unit, '  DT Bang Time(ns): '+strtrim(bang_time[1, i], 2)
  printf, unit, ''
endfor
if max(lsfit_param) gt 0 then begin
  if lsfit_param[0, 0] gt 0 then begin
    printf, unit, ' '
    printf, unit, ' Least Square Fit to Deuterium Data:'
    printf, unit, ' Bang Time(ns) = '+ strtrim(lsfit_param[0,0], 2)+ $
           ',  DD Neutron Energy(MeV) = ', Strtrim(lsfit_param[0,1]*1e-6, 2)
  endif 
  if lsfit_param[1, 0] gt 0 then begin
    printf, unit, ' '
    printf, unit, ' Least Square Fit to Tritium Data:'
    printf, unit, ' Bang Time(ns) = '+ strtrim(lsfit_param[1,0], 2)+ $
           ',  DT Neutron Energy(MeV) = ', Strtrim(lsfit_param[1,2]*1e-6, 2)
  endif 
  if lsfit_param[2, 0] gt 0 then begin
    printf, unit, ' '
    printf, unit, ' Least Square Fit to Deuterium & Tritium Data:'
    printf, unit, ' Bang Time(ns) = '+ strtrim(lsfit_param[2,0], 2)+ $
           ',  DD Neutron Energy(MeV) = ', Strtrim(lsfit_param[2,1]*1e-6, 2)
    printf, unit, ' Bang Time(ns) = '+ strtrim(lsfit_param[2,0], 2)+ $
           ',  DT Neutron Energy(MeV) = ', Strtrim(lsfit_param[2,2]*1e-6, 2)
  endif 
endif
  if unit gt 1 then free_lun, unit
end
