; ------------------------------------------------------------------------ 
; $Id: build_vertex_struct.pro,v 1.2 2006/08/30 20:30:42 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro build_vertex_struct, current, index, label, crdflg, num_v, vertex 

@qs_enums.com

;  Build vertex structure from vertex data


if crdflg eq CARTESIAN then xlab = ['x', 'y', 'z'] $
else if crdflg eq CYLINDRICAL then xlab = ['r', 'theta', 'z'] $
else if crdflg eq SPHERICAL then xlab = ['r', 'theta', 'phi']


nslant = 0L
mxslant = 4096L
slant = make_array(mxslant*6L)
vecnormal = make_array(mxslant*3L)
tot = 0L
totinc = num_v/4
b = size(vertex)
p = lindgen(4)
plane = make_array(6*totinc)
normal = make_array(totinc, /int)
maxsp = max(vertex, min= minsp)
err = 7.62939e-06*max([maxsp-minsp, abs(maxsp), abs(minsp)])
for j = 0L, b[2]-1, 4 do begin
  minx = min(vertex[0, j+p], max = mx)
  miny = min(vertex[1, j+p], max = my)
  minz = min(vertex[2, j+p], max = mz)
  k = (tot+j/4)
  k6 = 6*k
; rather than taking the cross product - look for equal values.
  plane[k6:k6+5] = [minx, mx, miny, my, minz, mz]
  if(mx - minx) lt err then normal[k] = I_DIR else $
  if(my - miny) lt err then normal[k] = J_DIR else $
  if(mz - minz) lt err then normal[k] = K_DIR else begin
    if nslant+1 gt mxslant then begin
      mxslant = mxslant+mxslant
      slant = [slant, slant]
      vecnormal= [vecnormal, vecnormal]
    endif
    slant[nslant*6] = [vertex[*, j], vertex[*, j+2]]
;
;    slnorm[3, size[0]] - unit slant normal from point slant(*, 0, i)
;                         unit cross product of vector(0, 1) X vector(0, 3)
;                         where vector(0, 1) = vect(1)-vect(0) and 
;                               vector(0, 3) = vect(3)-vect(0)
;    store as vecnormal
;
    veca = vertex[*, j+1]- vertex[*, j]
    vecb = vertex[*, j+3]- vertex[*, j]
    vecaxb = [veca[1]*vecb[2]-veca[2]*vecb[1], $
              veca[2]*vecb[0]-veca[0]*vecb[2], $
              veca[0]*vecb[1]-veca[1]*vecb[0]]
    vecnormal[nslant*3] = [vecaxb/sqrt(total(vecaxb*vecaxb))]
    nslant = nslant+1
    normal[k] = 0
  endelse
endfor
tot = tot+totinc
;

nsize = make_array(5, /long)
plane = reform (plane, 2, 3, tot, /overwrite)
nsize [4] = num_v/4
n0 = where (normal eq 0, ns)
nsize[0] = ns
n1 = where (normal eq I_DIR, ns)
nsize[1]= ns
n2 = where (normal eq J_DIR, ns)
nsize[2] = ns
n3 = where (normal eq K_DIR, ns)
nsize[3] = ns

nel = n_elements(vertex)
vertex = reform (vertex, 3, nel/3, /overwrite)
connectivity = make_planes(vertex, n0, n1, n2, n3, loc_con)

if nslant gt 0 then begin
  slant = temporary(slant[0:(6*nslant-1)])
  slant = reform (slant,3,2,nsize[0], /overwrite)
  vecnormal = temporary(vecnormal[0:(3*nslant-1)])
  vecnormal = reform (vecnormal,3,nsize[0], /overwrite)
endif else begin
  slant = 0.0
  vecnormal = 0.0
endelse

struct = {type:4L, ndum:3, block:1L, $
          size:nsize, plane:plane, slant:slant, slnorm:vecnormal, $
          n0:n0, n1:n1, n2:n2, n3:n3, xlab:xlab, $
          space:vertex, connectivity:connectivity, loc_con:loc_con, $
          file:'passed_data', spare:[crdflg,index,0,0,0], tlab:'facet', $
          clab:label}

i2s, struct, current, /no_copy

return
end
