; ------------------------------------------------------------------------ 
; $Id: line_vertex.pro,v 1.1 2007/05/16 17:09:21 rscoats Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
function line_vertex, pdi, imax, jmax, kmax, xi, xj, xk, $
                      system = systemin

@qs_enums.com
                 
 on_error , 2

  if n_elements(systemin) gt 0 then sys = systemin else sys = CARTESIAN

  add_chunk = 4000L
  max_v = add_chunk
  vertices = make_array(3, add_chunk, /float)
  size_v = 0L

  toler = 0.01

  if pdi[4] eq I_DIR then begin

    istep = pdi[1] - pdi[0]
    if pdi[2] eq 1 then xtol = toler*(xj[pdi[2]+1] - xj[pdi[2]]) $
                   else xtol = toler*(xj[pdi[2]] - xj[pdi[2]-1])
    if pdi[3] eq 1 then ytol = toler*(xk[pdi[3]+1] - xk[pdi[3]]) $
                   else ytol = toler*(xk[pdi[3]] - xk[pdi[3]-1])
    x_face = [xj[pdi[2]] - xtol, xj[pdi[2]] + xtol]
    y_face = [xk[pdi[3]] - ytol, xk[pdi[3]] + ytol]

    for i=pdi[0], pdi[1]-1, istep do begin
      if size_v + 16 gt  max_v then begin
        vertices = [[vertices], [make_array(3, add_chunk, /float)]]
        max_v = max_v + add_chunk
      endif
      if size_v + 16 le max_v then begin
        for n=0, 1 do begin
          vertices[I_DIR-1, size_v]   = xi[i]
          vertices[I_DIR-1, size_v+1] = xi[i+istep]
          vertices[I_DIR-1, size_v+2] = xi[i+istep]
          vertices[I_DIR-1, size_v+3] = xi[i]
          vertices[J_DIR-1, size_v:size_v+3] = x_face[n]
          vertices[K_DIR-1, size_v:size_v+1] = y_face[1]
          vertices[K_DIR-1, size_v+2:size_v+3] = y_face[0]
          size_v = size_v + 4
        endfor
        for n=0, 1 do begin
          vertices[I_DIR-1, size_v]   = xi[i]
          vertices[I_DIR-1, size_v+1] = xi[i+istep]
          vertices[I_DIR-1, size_v+2] = xi[i+istep]
          vertices[I_DIR-1, size_v+3] = xi[i]
          vertices[K_DIR-1, size_v:size_v+3] = y_face[n]
          vertices[J_DIR-1, size_v:size_v+1] = x_face[1]
          vertices[J_DIR-1, size_v+2:size_v+3] = x_face[0]
          size_v = size_v + 4
        endfor
      endif
    endfor

  endif else if pdi[4] eq J_DIR then begin

    if sys eq CYLINDRICAL or sys eq SPHERICAL then jstep = 1 $
    else jstep = pdi[1] - pdi[0]
    if pdi[2] eq 1 then xtol = toler*(xk[pdi[2]+1] - xk[pdi[2]]) $
                   else xtol = toler*(xk[pdi[2]] - xk[pdi[2]-1])
    if pdi[3] eq 1 then ytol = toler*(xi[pdi[3]+1] - xi[pdi[3]]) $
                   else ytol = toler*(xi[pdi[3]] - xi[pdi[3]-1])
    x_face = [xk[pdi[2]] - xtol, xk[pdi[2]] + xtol]
    y_face = [xi[pdi[3]] - ytol, xi[pdi[3]] + ytol]
    
    for j=pdi[0], pdi[1]-1, jstep do begin
      if size_v + 16 gt  max_v then begin
        vertices = [[vertices], [make_array(3, add_chunk, /float)]]
        max_v = max_v + add_chunk
      endif
      if size_v + 16 le max_v then begin
        for n=0, 1 do begin
          vertices[J_DIR-1, size_v]   = xj[j]
          vertices[J_DIR-1, size_v+1] = xj[j+jstep]
          vertices[J_DIR-1, size_v+2] = xj[j+jstep]
          vertices[J_DIR-1, size_v+3] = xj[j]
          vertices[K_DIR-1, size_v:size_v+3] = x_face[n]
          vertices[I_DIR-1, size_v:size_v+1] = y_face[1]
          vertices[I_DIR-1, size_v+2:size_v+3] = y_face[0]
          size_v = size_v + 4
        endfor
        for n=0, 1 do begin
          vertices[J_DIR-1, size_v]   = xj[j]
          vertices[J_DIR-1, size_v+1] = xj[j+jstep]
          vertices[J_DIR-1, size_v+2] = xj[j+jstep]
          vertices[J_DIR-1, size_v+3] = xj[j]
          vertices[I_DIR-1, size_v:size_v+3] = y_face[n]
          vertices[K_DIR-1, size_v:size_v+1] = x_face[1]
          vertices[K_DIR-1, size_v+2:size_v+3] = x_face[0]
          size_v = size_v + 4
        endfor
      endif
    endfor

  endif else if pdi[4] eq K_DIR then begin

    if sys eq SPHERICAL then kstep = 1 $
    else kstep = pdi[1] - pdi[0]
    if pdi[2] eq 1 then xtol = toler*(xi[pdi[2]+1] - xi[pdi[2]]) $
                   else xtol = toler*(xi[pdi[2]] - xi[pdi[2]-1])
    if pdi[3] eq 1 then ytol = toler*(xj[pdi[3]+1] - xj[pdi[3]]) $
                   else ytol = toler*(xj[pdi[3]] - xj[pdi[3]-1])
    x_face = [xi[pdi[2]] - xtol, xi[pdi[2]] + xtol]
    y_face = [xj[pdi[3]] - ytol, xj[pdi[3]] + ytol]

    for k=pdi[0], pdi[1]-1, kstep do begin
      if size_v + 16 gt  max_v then begin
        vertices = [[vertices], [make_array(3, add_chunk, /float)]]
        max_v = max_v + add_chunk
      endif
      if size_v + 16 le max_v then begin
        for n=0, 1 do begin
          vertices[K_DIR-1, size_v]   = xk[k]
          vertices[K_DIR-1, size_v+1] = xk[k+kstep]
          vertices[K_DIR-1, size_v+2] = xk[k+kstep]
          vertices[K_DIR-1, size_v+3] = xk[k]
          vertices[I_DIR-1, size_v:size_v+3] = x_face[n]
          vertices[J_DIR-1, size_v:size_v+1] = y_face[1]
          vertices[J_DIR-1, size_v+2:size_v+3] = y_face[0]
          size_v = size_v + 4
        endfor
        for n=0, 1 do begin
          vertices[K_DIR-1, size_v]   = xk[k]
          vertices[K_DIR-1, size_v+1] = xk[k+kstep]
          vertices[K_DIR-1, size_v+2] = xk[k+kstep]
          vertices[K_DIR-1, size_v+3] = xk[k]
          vertices[J_DIR-1, size_v:size_v+3] = y_face[n]
          vertices[I_DIR-1, size_v:size_v+1] = x_face[1]
          vertices[I_DIR-1, size_v+2:size_v+3] = x_face[0]
          size_v = size_v + 4
        endfor
      endif
    endfor

  endif

  if size_v gt 0 then return, vertices[*,0:size_v-1] else return, 0
  end
