; ------------------------------------------------------------------------ 
; $Id: main_widget.pro,v 1.2 2005/03/09 22:47:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;	Unauthorized reproduction prohibited.
;+
; NAME:
;	Main_Widget
;
; PURPOSE:
;	This procedure defines a new QUICKSILVER simulation setup
;
; CATEGORY:
;	MERCURY
;
; CALLING SEQUENCE:
;	GlobalSetup [, Name=name] [, System=system] [, Active=active] $
;       [, Periodic=periodic] [, Units=units] [, Convert=convert] $
;       [, /Particle] [, /FieldsOnly] [, /NoGUI]
;
; KEYWORD PARAMETERS:
;       NoGUI:  Set this keyword to not display the GlobalSetup widget.
;
; RESTRICTIONS:
;	
;
; EXAMPLE:
;	Define a 2D simulation including the r-z plane in cylindrical
;       coordinates with units of cm:
;		GlobalSetup, System='cylindrical', Active=[1,0,1], $
;               Units='cm'
;
;	Define a 3D simulation in cartesian coordinates, default units
;       of meters, and fields-only:
;		GlobalSetup, /FieldsOnly
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 12/97.
;-

;---------------------------------------------------------------------

PRO Main_Widget_Event, event

common params, nblk, block

 help,/str, event

END

;---------------------------------------------------------------------

PRO Main_Widget

common params, nblk, block

  On_Error, 2


  mainBase = Widget_Base(Column=1, Title='Main')
  topID = mainBase

  stateBase = Widget_Base(mainBase, Column=1, Frame=1, $
  Kill_Notify='StatePtr_Cleanup')

  closeBase = Widget_Base(mainBase, Align_Right=1, Column=1)
  updateID = Widget_Button(closeBase, /Frame, Value='   Refresh   ', $
  UValue='REFRESH')
  doneID = Widget_Button(closeBase, /Frame, Value='   Close   ', $
  UValue='CLOSE')

  Widget_Control, mainBase, /Realize

; Create simulation object and plot widget

print, 'nblk: ', nblk, 'block: ',block
;;  Q2DPlot, simSession, Group_Leader=topID, $
;;  refreshID=refreshID, WidID=wID, ObjPlot2D=objPlot2D

;  Store state information in structure for event handler
state = 0
  statePtr = Ptr_New(state, /No_Copy)
  Widget_Control, stateBase, Set_UValue=statePtr

  XManager, 'Main_Widget', mainBase, /No_Block

END
