; ------------------------------------------------------------------------ 
; $Id: qs_geom_in.pro,v 1.3 2007/05/16 17:09:21 rscoats Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro qs_geom_in, crdflg, nblk, imax, jmax, kmax, $
                locib, locjb, lockb, $
                bgxif, bgxjf, bgxkf

@qs_data.com


;  Repack loc and grid arrays to remove ghost cells

mblo = nblk+1
n = 0
locx = make_array(mblo+1, /long)
locy = make_array(mblo+1, /long)
locz = make_array(mblo+1, /long)
locx[n] = 0
locy[n] = 0
locz[n] = 0
locx[n+1] = locx[n] + imax[n]
locy[n+1] = locy[n] + jmax[n]
locz[n+1] = locz[n] + kmax[n]
x1 = bgxif[locib[n]+1:locib[n]+imax[n]]
x2 = bgxjf[locjb[n]+1:locjb[n]+jmax[n]]
x3 = bgxkf[lockb[n]+1:lockb[n]+kmax[n]]
for n=1, nblk do begin
  x1 = [x1, bgxif[locib[n]+1:locib[n]+imax[n]]]
  x2 = [x2, bgxjf[locjb[n]+1:locjb[n]+jmax[n]]]
  x3 = [x3, bgxkf[lockb[n]+1:lockb[n]+kmax[n]]]
  locx[n+1] = locx[n] + imax[n]
  locy[n+1] = locy[n] + jmax[n]
  locz[n+1] = locz[n] + kmax[n]
endfor

; Find range for each block and overall range

range = make_array(2, 3, mblo+1)
for i = 0, mblo-1 do begin
  range [0, 0, i+1] = min(x1[locx[i]:locx[i+1]-1], max = mm)
  range [1, 0, i+1] = mm
  range [0, 1, i+1] = min(x2[locy[i]:locy[i+1]-1], max = mm)
  range [1, 1, i+1] = mm
  range [0, 2, i+1] = min(x3[locz[i]:locz[i+1]-1], max = mm)
  range [1, 2, i+1] = mm
endfor
range [0, 0, 0] = min(range[0, 0, 1:mblo])
range [0, 1, 0] = min(range[0, 1, 1:mblo])
range [0, 2, 0] = min(range[0, 2, 1:mblo])
range [1, 0, 0] = max(range[1, 0, 1:mblo])
range [1, 1, 0] = max(range[1, 1, 1:mblo])
range [1, 2, 0] = max(range[1, 2, 1:mblo])

locx = transpose(reform([locx, locy, locz], mblo+1, 3))

ssize = make_array(3, mblo, /long)
ssize[0, *]= imax
ssize[1, *]= jmax
ssize[2, *]= kmax

ispare = make_array(5, /long, value=0L)
ispare[0] = crdflg

x1lab = make_array(mblo, /string, value='x')
x2lab = make_array(mblo, /string, value='y')
x3lab = make_array(mblo, /string, value='z')

; Create grid structure in QS common block
; Use a named structure to make it easier to extract data later

qs_grid = {type:3L, ndim:3L, block:mblo, two:0L, range:range, $
           size:ssize, locx:locx, slice:0.0,  $
           x1:x1, x2:x2, x3:x3, x1lab:x1lab, x2lab:x2lab, $
           x3lab:x3lab, proc:0, file:'passed_data', spare:ispare, $
           tlab:'grid', clab:'QS grid data' }

;  Initialize counters and first structure # if none have been defined
;  They will be reset later if any are created
if n_elements(numbr_cond) eq 0 then numbr_cond = 0
if n_elements(first_cond) eq 0 then first_cond = 20
if n_elements(numbr_diel) eq 0 then numbr_diel = 0
if n_elements(first_diel) eq 0 then first_diel = 21
numbr_bc = 0
first_bc = first_diel + numbr_diel
numbr_diag = 0
first_diag = first_bc + numbr_bc

return
end
