; ------------------------------------------------------------------------ 
; $Id: closepdb.pro,v 1.5 2007/06/06 19:44:55 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro closepdb, p1, allfiles= allfiles
;
;+
;   Close a pdb file. If the current file is closed, the current file pointer
;   will be set to the next file opened or to the previous file if it is the
;   last file.
;   format:   CLOSEPDB, [, fileid] [, /allfiles
;   where:
;      fileid    - file id or an array of file ids of the file(s) to be closed
;                  If it is omitted or zero, the current file is closed.
;      allfiles  - If set, all files are closed. FILEID is ignored.
;   Example:
;      Close the current file.
;         closepdb
;   <or>  closepdb, 0
;
;      Close the file with a file id of 3
;         closepdb, 3
;
;      Close all files
;         closepdb, /all
;-
;
      on_error, 1
;
@pdbcom.dat
; PDB common block
;     check to see if pdb has been initialized
;
      if (n_elements(initpdb) eq 0) then initpdb = 0
      if (initpdb eq 0) then  begin
        print, '% CLOSEPDB: No PDB files are open'
        return
      endif
;
;     see if there is an open file
; 
      if curfid lt 0 then begin
        print, '% CLOSEPDB: No PDB files are now open '
        return
      endif

;     get integers
;     
      ok = -1
      if keyword_set(allfiles) then ok = where(ufid ne 0, nok) else begin
        if (n_elements(p1) eq 0) then p1= 0
        nelp = n_elements(p1)
        valid = where( p1 ne 0, nvalid)
        if (nvalid eq 0) then begin
          ok = curfid
          nok = 1
        endif else begin
          p1 = p1(valid)
          for i = 0, nvalid-1 do $
            ok = [ok, where (p1[i] eq ufid)]
          okg = where (ok ge 0, nok)
          if nok gt 0 then ok = ok[okg]
        endelse
      endelse 
      if nok eq 0 then begin 
        print, '% CLOSEPDB: Specified File IDs do not exist'
        return
      endif
  for i = 0, nok-1 do begin
    offset = ok[i]*max_pdb_file

    str = *family_info[ok[i]]
    nfiles = str.nfile
    for j = 0, nfiles-1 do begin
      fstr = *file_info[offset+j]
      pid = fstr.pds_fileid
      ier = 0L
      if pid gt 0 then close_pdb, pid, ier
      ptr_free, file_info[offset+j]
    endfor
    ufid[ok[i]] = 0
    ptr_free, family_info[ok[i]]
    n_pdbfile[ok[i]]=0

  endfor
      if ufid[curfid] eq 0 then begin
        ok = where (ufid ne 0, nok)
        if nok eq 0 then curfid = -1 else curfid = ok[nok-1]
      endif
;
      return
      end
