; ------------------------------------------------------------------------ 
; $Id: curpdb.pro,v 1.4 2007/06/06 19:44:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function curpdb,str,time= time, index = index, quiet = quiet, $
           point = point,  _extra = extrastuff, ijcoord = ijcoord
;+
;   Name: CURPDB
;  
;   A wrapper for IDL cursor which returns the cell K, L components for the
;     cursor position. 
;   Note: PDB indices are assumed to start at 1
;   NOTE: XY2KL is used to calculate the indices.
;   NOTE: This feature may be accessed from the plotgrd command.
;   
;   Format:
;       POINT = CURPDB(STR [, /QUIET] [, POINT = XYpoint]
;              [, TIME = time] [, INDEX = index]  [, _EXTRA = extrastuff])
;     where
;        STR        - Grid structure used for the current plot.
;
;        POINT      - Node number of the cell - 1 to number of nodes
;                     If IJCOORD is set then I, J coordinates of the zone 
;                       of the cursor location 
;                     In case of an error, -1 is returned
;                     If n_elements(time) ne 0 and POINT is a 2 element vector,
;                     then point is taken as the cursor location.
;                     (I and J go from 1 to nx,ny)
;        TIME       - If defined then used to determine the time of the grid
;                     with the values in point.
;                     Default is the value used for the last plot.
;                     If zero or integer 1, then the first grid time is used.
;                     Data is not interpolated.
;        INDEX      - If set, Time is taken as an index =[1, 2, ...N_time]
;                     If time = zero or integer=1 then time index is 1
;        XYpoint    - X, Y coordinates of the cursor location
;        QUIET      - If set, no output to the terminal
;        extrastuff - any valid keywords for the IDL cursor command
;    Example:
;        Print the location of a point on a grid plot and get the components 
;        in ind.  Structure with the grid data is 12
;           ind = curpdb(12)
;-
 common plot_grid_last, yeng_scale, xeng_scale
on_error,1


;  Common block
@qsstrcom.dat
@qscom.dat

      npar = n_params()
      if npar eq 0 then begin
        print,'% Format: POINT = CURPDB (STR [, /QUIET] [, POINT = XYpoint] '
        print,'          [, _EXTRA = extrastuff])'
        return, -1
      endif
      if n_elements(str) eq 0 then begin
        print, '% CURPDB: Grid structure must be defined'
        return, -1
      endif
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      if n_elements(yeng_scale) eq 0 then begin
        xeng_scale = 1.00
        yeng_scale = 1.00
      endif


      print, ' Use the cursor to pick the desired cell'
      cursor , aa, bb, _extra = extrastuff 
      if xeng_scale ne 1.00 then aa = aa/xeng_scale
      if yeng_scale ne 1.00 then bb = bb/yeng_scale
      point= [aa, bb]
      if n_elements (time ) eq 0 then time = zvalueold
      ijval = xy2kl(aa, bb, str, time, index=index, quiet=quiet, scalar=scalar)
        
      if keyword_set(ijcoord) then return, ijval else return, scalar
      end
