; ------------------------------------------------------------------------ 
; $Id: open_pdb_doit.pro,v 1.5 2008/11/11 17:06:33 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro open_pdb_doit, p1, ifile, mode, ntimes, times, ssize, lcpar, error =error
;+
;  Open a PDB file and store the data.
;  
;  Format: open_pdb_doit, file, ifile, ntimes, times, ssize, lcpar, error
;  where:
;    File   - name of the PDB file to be opened.  
;    ifile  - File info index for this file
;    Mode   - Open mode: Read Only, Write (Create), Append (READ/WRITE)
;             (Default = Read Only)
;    ntimes - number of time values
;    Times  - values for the times
;    Ssize   - Size of space array ([nz, nr])
;    Error  - Error parameter
;             If this error is zero, no error encountered.
;             If Error = -99, file not found
;    lcpar - 15 if file contains zt(1), rt(2), ireg(4) and time(8) data
;-
;
;
      on_error, 1
;
@pdbcom.dat
; PDB common block
; common pdbcommon, initpdb, pdb_exe, ufid, pdbfid, maxfid, curfid, file_info
;
;     check on the number of parameters
;
      error = -999
      nparams = n_params()
      if nparams ne 7 then begin
        print, '% OPEN_PDB_DOIT: Error with number of parameters: ', nparams
        return
      endif
      maxt = 0L
      nz0 = 0L
      nr0 = 0L
      nt = 0L
      lcpar =0

      pdb_open_only, p1, lstr, mode, ndset, pid, error

      if error le 0 then begin
        print, '% PDB_OPEN_DOIT: Unable to open PDB file:'
        print, p1
        error = 1
        return
      endif
      if mode eq 2 then begin
        ntimes = 0
        times = 0.0
        ssize  = 0
        lcpar = pid
        error = 0L
        return
      endif


      names = make_array(ndset, /string)
      p3 = make_array(256, /byte)
      ier = 0L
      nel = ndset
      type = strarr(nel)
      ndim = intarr(nel)
      dim = lonarr(10, nel)
      totlen = lonarr(nel)
      source = strarr(nel)
      i =0L
      for ii = 0L, ndset-1 do begin
        lstr = 256L
;
        nd= ii +1L
        if initpdb eq 1 then begin
; HP
          junk = call_external(PDB_EXE, 'pdb_get_name', nd, lstr, p3, ier)
        endif else if initpdb eq 2 then begin
; Sun
          junk = call_external(PDB_EXE, 'pdb_get_name_', nd, lstr, p3, ier)
        endif else if initpdb eq 4 then begin
; WIN
         junk = call_external(PDB_EXE, '_PDB_GET_NAME@8', nd, lstr, p3, ier)
        endif else if initpdb eq 5 then begin
; MAC
         junk = call_external(PDB_EXE, 'pdb_get_name', nd, lstr, p3, ier)
        endif
        if (ier ne 0) then begin
          names(i) =  strmid(p3, 0, lstr)
;    llnl convention to use the @ at the end of the routine to indicate source
          isour = strpos(names(i), '@')
          if isour gt 0 then source(i)= strmid(names(i), isour+1, lstr)
; strip off the initial directory stuff
          idir = strpos(names[i], '/')
          if idir eq 0 then names[i] = strmid(names[i], 1)
;          while idir ge 0 do begin
;            names[i] = strmid(names[i], idir+1, lstr)
;            idir = strpos(names[i], '/')
;          endwhile
          typez = bytarr(256)
          ltype = 256L
          var = names[i]
          lname = 256L
          str2byt, var, varb, 256
          ndimz = 0L
          dimz = lonarr(10)
          ier = 0L
          nz0=0L
          nr0=0L
          ssize = 0L
          if initpdb eq 1 then begin
; HP
            junk = call_external(PDB_EXE, 'pdb_inquire_var', $
              pid, varb, lname, ltype, typez, ssize, ndimz, dimz, ier)
          endif else if initpdb eq 2 then begin
; Sun
            junk = call_external(PDB_EXE, 'pdb_inquire_var_', $
              pid, p3, lname, ltype, typez, ssize, ndimz, dimz, ier)
          endif else if initpdb eq 4 then begin
; WIN
           junk = call_external(PDB_EXE, '_PDB_INQUIRE_VAR@8', $
              pid, varb, lname, ltype, typez, ssize, ndimz, dimz, ier)
          endif else if initpdb eq 5 then begin
; MAC
           junk = call_external(PDB_EXE, 'pdb_inquire_var', $
              pid, varb, lname, ltype, typez, ssize, ndimz, dimz, ier)
          endif
          if ier ne 0 then begin
            typez = strmid(typez , 0, ltype)
            if typez ne 'Directory' then begin
              type[i] = strmid(typez , 0, ltype)
              ndim[i] = ndimz
              totlen[i] = ssize
              dim(*,i) = dimz
;             if ndimz gt 2 then print, var, ndimz , dimz
              i = i+1
            endif
          endif else $
            print, '% OPENPDB: Error in PDB_INQUIRE_VAR', strtrim (p3,2)
        endif
      endfor
      if i lt ndset then begin
        ndset = i
        i = i-1
        type = type[0:i]
        ndim = ndim[0:i]
        totlen = totlen[0:i]
        dim = dim[*, 0:i]
        names = names[0:i]
        source = source[0:i]
      endif
      if ier ne 0 then begin
        if initpdb eq 1 then begin
; HP
          junk = call_external(PDB_EXE, 'pdb_delete_table', ier)
        endif else if initpdb eq 2 then begin
; Sun
          junk = call_external(PDB_EXE, 'pdb_delete_table_', ier)
        endif else if initpdb eq 4 then begin
; WIN
         junk = call_external(PDB_EXE, '_PDB_DELETE_TABLE@8', ier)
        endif else if initpdb eq 5 then begin
; MAC
         junk = call_external(PDB_EXE, 'pdb_delete_table', ier)
        endif
        
;       check to see if we have a known dataset type
        ireg = 0
        zt = 0
        rt = 0
        time = 0
        dim = reform(dim, 2, 5, ndset, /overwrite)
;       do the test first to avoit warnings
        ind= (get_match('^zt@', names, nel= nel))[0]
        if nel eq 0 then $
          ind= (get_match('^zt$', names, nel= nel))[0]
        if nel eq 1 then read_pdb_open, names[ind], zt, $
           pid, type[ind], totlen[ind], ndim[ind], dim[*,*,ind], ier 
           
        ind= (get_match('^rt@', names, nel= nel))[0]
        if nel eq 0 then $
          ind= (get_match('^rt$', names, nel= nel))[0]
        if nel eq 1 then read_pdb_open, names[ind], rt, $
           pid, type[ind], totlen[ind], ndim[ind], dim[*,*,ind], ier
 
        ind= (get_match('^ireg@', names, nel= nel))[0]
        if nel eq 0 then $
           ind= (get_match('^ireg$', names, nel= nel))[0]
        if nel eq 1 then begin
          read_pdb_open,  names[ind], ireg, $ 
            pid, type[ind], totlen[ind], ndim[ind], dim[*,*,ind], ier
          imat = ireg(*,*,0)
        endif
        if n_elements (zt) gt 1 then lcpar = 1 
        if n_elements (rt) gt 1 then lcpar = lcpar + 2 
        if (n_elements (ireg) lt 3) and (lcpar eq 3) then begin
          ind = size(zt)
          ireg = make_array(ind[1], ind[2], value = 1)
          imat = ireg
        endif 
        if (n_elements (ireg) gt 3) then begin
          lcpar = lcpar + 4 
          make_pact_triangles, imat, tri, bad = bad
          if bad[0] lt 0 then good = -1 else $
              cell2node, imat, imat, junk, bad = bad, good = good
        endif else good = 0
        ind= (get_match('^time@', names, nel= nel))[0]
        if nel eq 0 then $
          ind= (get_match('^time$', names, nel= nel))[0]

        if nel eq 1 then read_pdb_open, names[ind], time, $
           pid, type[ind], totlen[ind], ndim[ind], dim[*,*,ind], ier
        error = 0
        nelt = n_elements(time)
        if nelt gt 0 then lcpar = lcpar+8 else time =0
        ntimes = nelt
        times = time
        nt = nt+nelt
        if (lcpar and 3) eq 3 then begin
          ss = size(zt)
          if ss[0] lt 2 then error = 12
          if error eq 0 then begin
            if nz0 eq 0 then begin
              nz0 = ss[1]
              nr0 = ss[2]
            endif else $
              if nz0 ne ss[1] or nr0 ne ss[2] then error = 13
          endif
        endif else begin
          nz0 = 0L
          nr0 = 0L
        endelse 
      endif
      ssize = [nz0, nr0]
      close_pdb, pid, ier
      pid = 0L
      
      if error eq 0 then begin
        ntimes = nelt
        times = time         
        str = {file:p1, pds_fileid: pid, ndset: ndset, names:names, $
          length:totlen, type:type, ndim:ndim, source:source, dimensions:dim, $
          zt: zt, rt:rt, ireg: ireg, time: time, good: good}
        ptr_free, file_info[ifile]
        file_info [ifile]= ptr_new(str, /no_copy) 
      endif
      return
      end
