; ------------------------------------------------------------------------ 
; $Id: read_pdb_grid.pro,v 1.4 2007/06/06 19:45:15 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function read_pdb_grid ,double = double
;+
;  Utility function to read a PDB grid structure.
;
;  Format: structure = READ_PDB_GRID(  double = double)
;   where 
;      double  - If set, data will be saved in the same precision as written
;                Default is to change X and Y values to float
;      
;  All other parameters are contained in the common block.
;-
      on_error, 1
;     add common block
@pdbcom.dat
@qsstrcom.dat
;
;     check to see if pdb and structures have been initialized
;
      if n_elements(initpdb) eq 0 then init_pdb
      if n_elements (structinit) eq 0 then qsstruct_init
;
;
      If curfid lt 0 then begin
        print, '% READ_PDB_GRID:  No valid PDB files are open.'
        return, 0
      endif
;
      info = *family_info(curfid)
;     get time and ntime
; ntime
      ntime = info.ntime
; time
      if ntime eq 0 then begin
        print, '% READ_PDB_GRID: Error reading time@history', ind, nel
        return, 0
      endif
      time = info.time
      if not keyword_set(double) then time = float(time)
;
;     str = {file:p1, pds_fileid: error, ndset: ndset, datasets:names, $
;       length: totlen, type:type, ndim: ndim, source:source, $
;       dimensions: reform(dim, 2, 3, ndset, /overwrite)}

      clab = 'Grid Data - '+info.file
      tlab = 'PACT Grid Data'
      get_pdb_rzreg, x1, x2, ireg, ier, double = double
      if ier ne 0 then begin
        print, '% READ_PDB_GRID: Error with R, Z, IRegion data'
        return, 0
      endif
 
; region data
      region = ireg(*,*,0)
;

      make_pact_connect, region, conn, locx
      locv = [0, n_elements(locx)-1]
      nreg = n_elements(region)
;
      ss = size(x1)
      nx = ss(1)
      ny = ss(2)
      sgrid = [nx, ny]
      if ss(0) eq 3 then nz = ss(3) else nz = 1l
      x1 = reform (x1, nx*ny, nz, /overwrite)
      x2 = reform (x2, nx*ny, nz, /overwrite)
      ndim = 2
      block =1 
      type = 11L
      range = make_array(2, 2, 2)
      range(0, 0, *) = min(x1[conn,*], max = mm)
      range(1, 0, *) = mm
      range(0, 1, *) = min(x2[conn,*], max = mm)
      range(1, 1, *) = mm
      
      two = 3L
      zval = 0.0
      x3 = 0.0
      ss = size(region)
      ssize = ss(1:2)
      str = {type:type, ndim:ndim, block: block, two: two, ntime:ntime, $
             locx: locx, locv:locv, x1:x1, x2:x2, x3:x3, x4: time, $
             connect:conn, range:range,  size:sgrid, spare:[0,0,0,0,0], $
             x1lab:'Z value', x2lab:'R value', x3lab:'', x4lab: 'Time', $
             region:region, file: info.file, tlab:tlab, clab:clab }
 
      return, str
      end
