; ------------------------------------------------------------------------ 
; $Id: wri_var_pdb.pro,v 1.4 2007/06/06 19:44:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  wri_var_pdb, name, arrayin, fileid = fileid, ierror = ierror 
;
;+
;  Writes a variable to a PDB file
;  Note:  Use openpdb to open a new file.
;
;  Format:  WRI_VAR_PDB, Name, Array
;             [, FILEID = fileid] [, IERROR = Ierror]
;  where:
;    Name      - Name of variable
;    Array     - Data array ot be written.
;                Data should be character, float, double, short, or long
;                (LONGS are stored as PDB integers.)
;    FILEID    - File ID of the desired file.
;                Default:  FILEID = 0  ;  Current file
;    IERROR    - Zero if no problems
;  Examples:
;    Write array "X Axis (cm)" to the current file.
;      WRI_VAR_PDB,  "X Axis (cm)", x
;-
      on_error, 1
;     add common block
@pdbcom.dat
;
;     check to see if pdb and structures have been initialized
;
      if n_elements(initpdb) eq 0 then init_pdb
      if n_elements (structinit) eq 0 then qsstruct_init
;
      ierror =1L
      nparam = n_params()
      if (nparam lt 1 ) then begin
 print,'% Format: WRI_VAR_PDB, Name, Array
 print,'          [, FILEID = fileid] [, IERROR = Ierror]
	return
      endif
;
      if initpdb  le 0 then begin
        print, '% WRI_VAR_PDB: No netPDB files are open.'
        return
      endif

      If curfid lt 0 then begin
        print, '% WRI_VAR_PDB:  No valid PDB files are open.'
        return
      endif

      if n_elements (fileid) gt 0 then begin
        setpdb, fileid[0], ier=ierror
        if ierror ne 0 then begin
          print, '% WRI_VAR_PDB: Unable to set file pointer:' , fileid[0]
          showpdb
          return
        endif
      endif else ierror=0

      offset = curfid*max_pdb_file
      info = *file_info[offset]
      fam_info = *family_info[curfid]
      pid = info.pds_fileid
      p1 = info.file
      mode = 3L
      if pid le 0 then $
          pdb_open_only, p1, lstr, mode, ndset, pid, error


      if n_elements(name) eq 0 then begin
        print, '% WRI_VAR_PDB: Array name must be provided'
        return
      endif
      name = strtrim(name, 2)
      lnam = strlen(name)
      if lnam lt 1 then begin
        print, '% WRI_VAR_PDB: Non-blank array name must be provided'
        return
      endif
      if n_elements (arrayin) eq 0 then begin
        print, '% WRI_VAR_PDB: Array must be provided'
        return
      endif else array = arrayin

      atype = size (array, /type)
      if atype eq 2  then begin
        typ = 'short'
      endif else if atype eq 3 then begin 
        typ = 'integer'     
      endif else if atype eq 4 then begin 
        typ = 'float'
      endif else if atype eq 5 then begin
        typ = 'double'
      endif else if atype eq 7 then begin
        typ = 'char'
        array = byte(array)
      endif else begin
        print,  '% WRI_VAR_PDB: Only integers (SHORT,LONG), reals and doubles'
        print,  '               are supported.'
        return
      endelse
      ltyp = strlen(typ)

      lstr = 64L
      if strlen (name) gt 64 then begin
        print, '% WRI_VAR_PDB: Name will be shorted to ', strtrim(lstr,2), ' characters'
        print, '               Contact Paul Mix to change this limitation.'
        name = strmid(name, 0, 64)
      endif
      error = 0L
      str2byt, name, p3, lstr
      str2byt, typ, p4, lstr
      ss = size (array)
      ndim = ss[0]
      if ndim gt 0 then begin
        dim = make_array(3, ndim, value = 1L)
        dim[1, *] = ss[1:ndim]
      endif else dim = [1l,1l,1l]
; help, lnam, ltyp, lstr
;
      if initpdb eq 1 then begin
; HP
        junk = call_external(PDB_EXE, 'pdb_write', p3, p4, lstr, pid, array, $
          lnam, ltyp, ndim, dim, ierror)
      endif else if initpdb eq 2 then begin
; Sun
        junk = call_external(PDB_EXE, 'pdb_write_', p3, p4, lstr, pid, array, $
          lnam, ltyp, ndim, dim, ierror)
      endif else if initpdb eq 4 then begin
; WIN
       junk = call_external(PDB_EXE, '_PDB_WRITE@8', p3, p4, lstr, pid, array, $
          lnam, ltyp, ndim, dim, ierror)
      endif

      if error eq 1 then ierror = 0 else ierror = error     
      end
