; ------------------------------------------------------------------------ 
; $Id: get_pds_varid.pro,v 1.3 2007/06/06 19:50:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_pds_varid, par1, par2, file_handle=fhin, ier=ier, helpme=helpme 
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      GET_PDS_VARID
;
; PURPOSE:
;      Inquire the ID's of a particular dataset type
;      Uses: PDS_VARID
;
; CALLING SEQUENCE:
;
;       VARID   = GET_PDS_VARID([FILE_HANDLE,] NAME 
;               [, FILE_HANDLE = file_handle] [, IER = ier] [, /HELPme] )
;
; WHERE:
;        VARID        - Variable handle or id
;                       Set to -1 if there is a problem
;        FILE_HANDLE  - PDS file ID
;                       Default: File_Handle = current file
;                       Note: If two parameters are present then the keyword
;                             file_handle is ignored.
;        NAME         - Variable name
;        IER          - Error Parameter
;                       Returns 0 if successful
;                               -1 if unsuccessful or if help is selected
;        HELPme       - If set, a one user will be given a help message.
;
; EXAMPLE:
;       Get the VARID of the variable, POWER, from the current file
;         varid = get_pds_varid('POWER')
;       Get the VARID of the variable POWER from the PDS file with id =1
;         varid = get_pds_varid(1, 'POWER')
;   or    varid = get_pds_varid('POWER', file = 1)
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 14, 1998
;-


;
;
@pds_datacom.dat
      on_error, 1

@pds_info_types.dat
  ier = -1L
  if keyword_set (helpme) then begin
    print, '% Format: VARID   = GET_PDS_VARID([FILE_HANDLE,] NAME 
    print, '         [, FILE_HANDLE = file_handle] [, IER = ier] [, /HELPme] )
    return, 0
  endif
  npar = n_params()
  if n_elements(par1) eq 0 then begin
    print, '% GET_PDS_VARID: Variable name must be provided'
    return, -1
  endif
  if npar eq 1 then begin
    name = par1
    if n_elements(fhin) eq 0 then fh= pds_id else fh = long (fhin[0])
  endif else begin
    fh = long(par1[0])
    if n_elements(par2) eq 0 then begin
print, '% GET_PDS_VARID: Variable name must be provided after the file handle'
      return, -1
    endif
    name = par2
  endelse
  name = strtrim (name, 2)
; 
  strl = strlen(name)
  if strl eq 0 then begin
    Print, '% GET_PDS_VARID: Name must have a nonzero length'
    return, -1
  endif
  varid =  -1L
  nameb = [byte(name), 0B]
  junk= call_external(PDS_EXE, 'pdsp_varid_',fh, nameb, strl, varid)
  if varid ge 0 then ier = 0
  return, varid
  end
