; ------------------------------------------------------------------------ 
; $Id: set_pds_segment.pro,v 1.3 2007/06/06 19:50:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro set_pds_segment, PAR1, par2,  timeseg = timeseg, extension= extension, $
        file_handle = filehandle, topology = topology, $
          statesegments = statesegments, ier = ier 
;
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;+
; NAME:
;      SET_PDS_SEGMENT
;
; PURPOSE:
;      Select the PDS segment to process
;      Uses: PDS_SET_SEGMENT
;
; CALLING SEQUENCE:
;
;       SET_PDS_SEGMENT ([FILE_HANDLE,] INDEX, [, /TOPOLOGY] [, /STATESEGMENTS]
;             [, TIMESEG = time] [, EXTENSION = extension]
;             [, FILE_HANDLE = file_handle] [, IER = ier]
;
; WHERE:
;        FILE_HANDLE  - PDS file ID
;                       Default: File_Handle = current file
;                       Note: If two parameters are present then the keyword
;                             file_handle is ignored.
;        INDEX        - Time index for the segment
;        TOPOLOGY     - If set, set to topology segment for requested time index
;                       Default: set to a state segment
;        STATESEGMENT - Ignored.  If TOPOLOGY is not set, then statesegment will
;                       be set.
;        IER          - Error Parameter
;                       Returns 0 if successful
;                               -1 if unsuccessful or if help is selected
;
; EXAMPLE:
;       Set the file to the 2 state segment of the current file
;         SET_PDS_SEGMENT, 2
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, August 26, 1998
;-

@pds_datacom.dat
      on_error, 1

@pds_info_types.dat

 
 ier = -1L
 npar = n_params()
 if npar eq 0 then begin
   print, '% FORMAT: SET_PDS_SEGMENT [FILE_HANDLE,] INDEX, [, /TOPOLOGY]'
   print, '              [, /STATESEGMENTS] [, FILE_HANDLE = file_handle]'
   print, '              [, TIMESEG = time] [, EXTENSION = extension]'
   print, '              [, IER = ier]'
   return
 endif

 if n_elements(initpds) eq 0 then initpds = 0
 if initpds eq 0 then begin
   Print, '% SET_PDS_SEGMENT: No PDS files are open'
   return
 endif

 if initpds lt 0 then begin
   print, '% SET_PDS_SEGMENT: This installation does not support PDS'
   return
 endif

 if n_elements (par1) eq 0 then begin
   print, '% SET_PDS_SEGMENT: First parameter must be defined'
   return
 endif
 if npar eq 2 then begin
   if n_elements (par2) eq 0 then begin
     print, '% SET_PDS_SEGMENT: Second parameter must be defined'
     return
   endif
 endif

 if npar eq 1 then begin
   if keyword_set(filehandle) then fh = long(filehandle[0]) $
       else fh = long(pds_id)
   seg_index = long(par1[0])
 endif else begin
   fh = long(par1[0])
   seg_index = long(par2[0])
 endelse

 if keyword_set(topology) then type = PDS_TP  else type = PDS_ST
 type = long(type)
 num_seg = 0L
 junk = call_external(PDS_EXE, 'pdsp_inq_param_', fh, type, num_seg, ier)

 if ier ne 0 then begin
   print, '% SET_PDS_SEGMENT: Error retrieving the total number of segments'
   return
 end
 ier = 1L
;if seg_index  lt 0 or seg_index ge num_seg then begin
;   print,  '% SET_PDS_SEGMENT: Variable must be in the range of 0 to ', $
;     strtrim(num_seg-1,2)
;   return
;endif
 timeseg = -1.0D0
 extL = 32L
 ext = make_array(extL, /byte, value = 32B)
 junk = call_external(PDS_EXE, 'pdsp_set_segment_', $
          fh, type, seg_index, timeseg, ext, extL,ier)
 if ier ne 0 then $
    print, '% SET_PDS_SEGMENT: Error setting PDS segment', seg_index, type $
  else extension = strtrim (ext, 2)
  end
