; ------------------------------------------------------------------------ 
; $Id: closepff.pro,v 1.3 2007/06/06 19:59:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro closepff, p1
;
;+
;   Close a pff file. If the current file is closed, the current file pointer
;   will be set to the next file opened or to the previous file if it is the
;   last file.
;   format:   CLOSEPFF, [, fileid]
;   where:
;      fileid    - file id of the file to be closed
;                  If it is omitted, the current file is closed.
;                  If it is zero or 'all', then all files will be closed.
;   Example:
;      Close the current file.
;         closepff
;
;      Close the file with a file id of 3
;         closepff, 3
;
;      Close all files
;         closepff, 0
;    <or> closepff, 'all'
;-
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff = 0
      if (initpff eq 0) then  begin
        print, '% CLOSEPFF: No PFF files are open'
        return
      endif

;
;     check on the number of parameters
;
      nparams = long( n_params())
;
;     get integers
;
      if (n_elements(p1) eq 0) then p1= -1L
      j = size (p1)
      if (j(0) ne 0) then begin
        p1= p1(0,0,0,0)
        j = size(p1)
      endif
      if (j(1) eq 7) then begin
        p1 = strtrim(p1, 2)
        p1 = strupcase(p1)
        if (p1 eq 'ALL') then begin
          p1 = 0L 
        endif else begin
          print, ' Invalid parameter sent to to CLOSEPFF - ', p1
          return
        endelse
      endif
      fid = long(p1)
;
      ier = 0L
      if initpff eq 1  then begin
; HP
        junk = call_external(PFF_EXE,'pff_slclose',fid,ier)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slclose_',fid,ier)
      endif else if initpff eq 3 then begin
; VMS
        junk = call_external(PFF_EXE,'pff_close',fid,ier)
      endif else if initpff eq 4  then begin
; WIN
        junk = call_external(PFF_EXE,'_PFF_SLCLOSE@8',fid,ier)
      endif else if initpff eq 5 then begin
; MAC
        junk = call_external(PFF_EXE,'pff_slclose',fid,ier)
      endif
;
      return
      end
