; ------------------------------------------------------------------------ 
; $Id: dir.pro,v 1.4 2008/02/27 22:39:42 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dir, p1, p2, p3, PAGE = page, reset = reset, font= font, ndset = ndset, $
         xsize = xsize, truncate = truncate, no_block = no_block, $
         sortit = sortit
;+
;
;     this routine prints a directory of pff files
;
;     DIR [, P1] [, P2] [, P3] [, Page = Page] [, NDSET = ndset]
;         [, TRUNCATE = truncate] [, /sortit]
;         [, Reset = reset] [, Xsize = xsize]
;     where p1, p2, p3 are identical to dirpff
;           page is the number of lines on the directory
;                default is directorypage
;           reset if present and not zero, resets the directory
;                use this if the window has been closed
;             also: if the window is still open and you wish to leave it
;                open, use /reset to put the current command in a second
;                window.
;           ndset is the number of datasets returned
;          NOTE: By using the reset qualifier, a window can be
;                opened for each file with the filename at the top.
;           xsize is the width of the window in pixels (generally not set)
;           truncate - if set, the dataset comments will be truncated at ' - '
;                      This is a toggle; If not specified the last value will
;                      be used.  Default value =0 -- no truncate
;           sortit - If set, display the directory with comments sorted.
;                    IF SORTIT is a string, then sorting will begin following 
;                    the string.
;
;     READ funtion:
;          The DIR widget can be used to read files.
;          Datasets may be highlighted using the left mouse button. 
;          Shift-Left mouse button will select a range of datasets.
;          Control-Left mouse button will toggle individual datasets.
;          If a carriage return is entered in the window or if one of the 
;          above selections is performed with a double-click, PFIDL will read
;          selected datasets.  
;          All seleced datasets should be either structures or waveforms.
;         
;     Example:
;         Get a directory of the current pff file and the number of datasets.
;           dir, ndset = ndset
;         Get a directory of the current file and the pff file with id = 3
;           with an additional window for this widget
;           (to see two different files simultaneously)
;           dir & dir, 3, /reset
;
;-
      common dirsave, base, list, oldpage,  text1, truncate_save
;
      common pffcom, initpff, pff_exe
;
@qscom.dat
      on_error, 1
;
      if n_elements (truncate_save) eq 0 then truncate_save =0
      if n_elements (truncate) gt 0 then truncate_save = truncate
;

      if n_elements(qsplotinit) eq 0 then qsinit
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if (initpff eq 0) then begin
        print, '% DIR: No PFF files are open'
        return
      endif
;
      if n_elements (oldpage) eq 0 then oldpage = directorypage
      if n_elements (page) eq 0 then page = oldpage
      page = page >4
;
      if n_elements (base) eq 0 then begin
        base = 0
        text1 = 0
        text2 = 0
      endif
;
      get_dir, p1, p2, p3, line = line, filename = filename, number = ndset, $
              sortit = sortit
      if ndset eq 0 then begin
        print, '% DIR: No datasets in the file: ', filename
        return
      endif
      pff_info, fid = fid, fname = fname, /quiet
      ok = where (filename eq fname, nok)
      if nok eq 1 then fileid = fid[ok[0]] else fileid  = 0
  if fileid eq 0 then begin
    Print, '% DIR: This is a diagnostic - contact Paul Mix, 845-7493'
    print, 'Search for: ',filename
    print, 'From the following list:'
    for ijk = 0, n_elements(fname)-1 do print, fname[ijk]
  endif
      filename = 'File: '+ filename

;
      if keyword_set(truncate_save) then begin
        for i = 0, ndset-1 do begin
          ii = strpos(line(i), ' - ', 25)
          if ii gt 0 then line(i) = strmid(line(i), 0, ii)
        endfor
      endif

      if ndset eq 0 then return
      if keyword_set(reset) then base = 0
      if oldpage ne page then page = -page
      widget_directory, base, page, list, line, base_title = 'PFF Directory', $
        xsize = xsize, text1 = text1, value1 = filename,  dirtype =1, $
        text2 = text2, fileid = fileid, no_block = no_block,  $
        value2 = ' #   Data Type       Dataset Comment'

      oldpage = page
      end
