; ------------------------------------------------------------------------ 
; $Id: get_dir.pro,v 1.4 2007/06/06 19:59:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro get_dir, p1, p2, p3, dataset = dataset, type = type, filename = filename, $
    comment = comment, number = ndset, point = point, line = line, $
     no_trim = no_trim, sortit = sortit
          
;+
;  this routine returns up to four arrays of dataset numbers, dataset types,
;  dataset comments, and the entire dataset line.  The size of these arrays 
;  is returned in number.
;
;   Format: get_dir [,p1] [, p2] [, p3] [, dataset=dataset]
;           [,type=type] [,comment=comment] [, number= ndset] [, POINT= point]
;           [, line = line] [, filename = filename] [, /no_trim] [, /SortIT]
;   where:
;          p1, p2, p2 have the same possibilities as dirpff
;          That is:
;            [, FILEID] [, LOW] [, HIGH]
;        or  [, FILEID] ,STRING
;            where:
;                fileid   - file id of the file
;                low      - first dataset number
;                           If low lt 0 then make a directory +/- low around
;                             current dataset
;                high     - last dataset number
;                string   - search string for the dataset comments
;
;          dataset - integer array of dataset numbers
;          type    - string array of dataset type
;          comment - string array of dataset comments
;          number  - number of elements in the above arrays
;          point   - dataset pointer
;          line    - string array of the entire dataset directory line
;          filename - name of the pff file
;          NO_trim - If set, leave comment as read
;          SortIT  - If set sort directory by dataset comments
;                    Dataset pointer refers to the dataset number.
;                    IF SORTIT is a string, then sorting will begin following 
;                    the string.
;     Example:
;          Get a directory around current dataset of 5 datasets
;            GET_DIR, -5, line = line
;          Get a directory of datasets 95 to 100
;            GET_DIR, 95, 100, line = line
;          Get a listing of all datasets with 'MOCAM2?'
;            GET_DIR, 'MOCAM2?', line = line
;-
      common pffcom, initpff, pff_exe
      common getdircom, file
;
       on_error, 1
;
      ndset = 0
      if (n_elements(initpff) eq 0) then initpff = 0 
      if initpff eq 0 then begin
        print, '% GET_DIR: No PFF files are open'
        return
      endif

       if n_elements(p3) gt 0 then begin
         dirpff, p1, p2, p3, luout = file, point = point, width = 98, $
            filename=filename, number = ndset
       endif else if n_elements(p2) gt 0 then begin
         dirpff, p1, p2 ,luout = file, point = point, width = 98, $
            filename=filename, number = ndset
       endif else if n_elements(p1) gt 0 then begin
         dirpff, p1, luout = file, point = point, width = 98, $
            filename=filename, number = ndset
       endif else dirpff, luout = file, point = point, width = 98, $
            filename=filename, number = ndset
;
       if ndset eq 0 then begin
         openr, unit, file, /get, /delete, error = err
         if err ne 0 then $
          print, '% GET_DIR: Open error ', strtrim(err, 2),' - ', file
         free_lun, unit
         return
       endif
       dataset = make_array(ndset, /long)
       type = make_array(ndset, /string)
       comment = make_array(ndset, /string)
       line = make_array(ndset, /string)
       i = 0
       plus = 0
       c = '      '
       try = 0
tryagain:
       openr, unit, file, /get, /delete, error = err
       if err ne 0 then begin
         print, '% GET_DIR: Open error ', strtrim(err, 2),' - Try to Continue'
         if try eq 10 then message, ' Ten tries to open '+strtrim(file, 2) + $
                ', ERROR = '+strtrim(err, 2)
         try = try +1
         wait, 1
         goto, tryagain
       endif
readit:
       while (not eof(unit) and i lt ndset) do begin
         readf, unit, c
         if plus eq 2 then begin
           dataset(i) =  long(strmid(c, s1, 4))
           type[i] = strtrim(strmid(c, s2, n2), 2)
           if keyword_set(no_trim) then comment(i) = strmid(c,s3, n3) else $
             comment[i] = strtrim(strmid(c,s3, n3), 2)
           line[i] = strmid(c, s1, 4)+' -' + strmid(c, s2, n2)+ $
            ' - ' +  strip_spaces(strmid(c, s3, n3))
           i = i+1
         endif else if strpos(strmid(c, 0,10), '+') gt 0 then begin
           plus = plus+1 
         endif else if plus eq 1 then begin
           ok = where (byte(c) eq 124B, nok)
           if nok ne 4 then begin
             print, '%GET_DIR: Number of separators not equal to 4. n_sep =', $
               strtrim( nok, 2)
             ndset = 0
             goto, read_error
           endif
           s1 = ok(0)+3
           s2 = ok(1)+2
           n2 = ok(2)-ok(1)-3
           s3 = ok(2)+2
           n3 = ok(3)- ok(2)-3
         endif
       endwhile
       readf, unit, c
       if strmid(c, ok(0), 1) ne '+' then begin
         print, '%GET_DIR: Directory list may be incomplete'
       endif
read_error:
       free_lun, unit
       i = i > 1
       LINE = LINE[0:I-1]
       if type[i-1] eq 'END-OF-DATA' then i = i-1
       ndset = i
       if i eq 0 then i = 1
       dataset = temporary(dataset(0:i-1))
       type = temporary(type[0:i-1])
       comment = temporary(comment[0:i-1])
       if keyword_set (sortit) then begin
         if size(sortit, /type) eq 7 then begin
           key = sortit[0]
           ok = strpos(strupcase(comment), strupcase(key))
           ndset_orig = ndset
           okp = where (ok ge 0, ndset)
           if ndset eq 0 then begin
             print, '% GET_DIR: No datasets match the search key: ', key
             return
           endif
           if ndset lt ndset_orig then begin
             ok = ok[okp]
             comment = comment[okp]
             line= line[okp]
             dataset = dataset[okp]
             type = type[okp] 
           endif 
           comm = make_array(ndset, /string)
           ok  = ok + strlen(key)
           for i = 0L, ndset-1 do comm[i]=strmid(comment[i], ok[i])
         endif else comm = comment
         ii = sort(comm)
         dataset= dataset[ii]
         type = type[ii]
         comment = comment[ii]
         line = line [ii]
       endif
       end
