; ------------------------------------------------------------------------ 
; $Id: ndspff.pro,v 1.3 2007/06/06 19:59:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  ndspff, fileid
;
;+
;     This routine returns the maximum number of datasets in a pff file.
;     Note: Dir will also return this information with the ndset keyword
;
;     format: ndset = ndspff([ fileid ])
; 
;     where
;         ndset   - Number of datasets in pff file = fileid
;         fileid  - file id of the PFF file containing the data
;                   If fileid is undefined or zero, the number of datasets in 
;                     the current file is returned
; 
;
;     Example
;       store in n, the number of datasets in the current pff file
;          n = ndspff()
;-
 on_error, 1
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff=0

      if initpff eq 0 then begin
        print, '% NDSPFF: No PFF files are open'
        return, 0
      endif
;
;     check if the dataset pointer is defined
;
;     get integers for the fortran call
;
      if n_elements(fileid) eq 0 then fid = 0L else fid = long(fileid) 
      dirpt = 0L
      ier = 0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slsetdir',dirpt, fid, ier)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slsetdir_',dirpt, fid, ier)
      endif else if initpff eq 3 then begin
; VMS
        junk=call_external(PFF_EXE,'pff_setdir',dirpt, fid, ier)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLSETDIR@8',dirpt, fid, ier)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,'pff_slsetdir',dirpt, fid, ier)
      endif
      if ier ne 0 then print, ' ERROR in NDSPFF'
      return, dirpt
      end
