; ------------------------------------------------------------------------ 
; $Id: readgrd.pro,v 1.5 2007/06/06 19:59:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readgrd, array, datasetin, fileid = fid
;
;+
;
;  Reads a GRID dataset into an IDL structure.
;  Procedure options are explained below. PFF dataset type must be NG3
;  (nonuniform 3-D grid)  (See readNG3 for more details.)
;
;  Format:  READGRD, structure  [, dataset] [, FILEID = fileid]
;                    [, STRINGLEN = stringlen]
;  where:
;    structure - IDL structure name
;    dataset - indicates a PFF dataset number 
;              or is a search string for dataset comments.  
;              If dataset is missing or zero, then it is set to the 
;              current dataset.
;        Note: If dataset is a string, a leading `^' indicates that 
;              the string begins at the start of the dataset comment.
;    fileid -  file id of the PFF file containing the data
;    stringlen - Currently ignored
;                Maximum number of characters in labels -- currently 64
;
;  Structure Labels
;    type  - 3
;    ndim  - number of spatial dimensions = 3
;    block - number of blocks 
;    size  - array of the size of the spatial dimensions
;            size = size(ndim, block)
;    two   - two dimensional missing dimension
;            this is 0 for 3d data, 1 for yz data, 2 for zx data, 3 for xy data
;    range - (2, ndim, nblock+1) two dimensional array of the max and min for
;            each block.  Block (0) has overall values.
;    locx  - array of the location of the first point for each block
;            spatial array- locx = locx(ndim, block+1)
;            (first spatial dimension for block m is x and is defined by
;             x = x1(locx(0, m-1): locx(0, m)-1) )
;    x1, x2, x3 - spatial arrays
;    proc  - Processor number if available on parallel jobs
;    x1lab, x2lab, x3lab - spatial labels
;    tlab  - type label
;    clab  - dataset comment
;    file  - file data read from
;    spare - integer array (5)
;
;  Examples:
;    Read dataset 34 in file id 4 into structure g5
;      readgrd, g5, 34, fil=4
;    Read next dataset of the current file into structure gstart
;      readgrd, gstart
;-
;
      on_error, 1
      nparams =  n_params()
@pff_strings.com
      if (nparams lt 1 ) then begin
        print,'% Format: READGRD, Structure  [, dataset] [, FILEID = fileid]'
        print,'        [, STRINGLEN = stringlen]'
        return
      endif
      stringlen = max_str_len
      if n_elements (datasetin) eq 0 then dataset=0 else dataset=datasetin[0]
      readng3, array, dataset, fileid = fid, stringlen = stringlen
      if size(array, /type) ne 8 then $
         Print, '% READGRD: Unable to read dataset ', strtrim (dataset, 2)
      return
      end
