; ------------------------------------------------------------------------ 
; $Id: cur.pro,v 1.3 2007/06/06 20:03:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro cur, aa, bb, wait, screen = screen, quiet=quiet, real_value = real_value, $
        _extra = extrastuff
;+
;   Name: CUR
;  
;   A wrapper for IDL cursor which prints the coordinates of the point.
;   
;   Format: cur [, X] [, Y] [,WAIT] [, SCREEN = screen] [, /quiet] 
;             [, /REAL_VALUE] [, _EXTRA = extrastuff]
;     where
;           x, y    - Coordinates of the cursor location
;           wait    - Optional parameter to cursor command.
;                     Default = 3, return with a button down transition
;           Screen  - Coordinates in terms of the screen coordinates
;                     If engineering units are used, this may be 
;                     different than X and Y
;           Quiet   - If set, no output will be made
;           Real_VALUE - If set, X and Y are real (float) values
;                     Default is double precision for IDL 5.5.
;        extrastuff - any valid keywords for the IDL cursor command
;    Example:
;        Print the location of a point on a graph
;           cur
;-
 common plot_grid_last, yeng_scale, xeng_scale

      if n_elements(yeng_scale) eq 0 then begin
        xeng_scale = 1.00
        yeng_scale = 1.00
      endif
     if n_elements(wait) eq 0 then wait = 3 else wait = (wait[0] > 0) < 4
     cursor , aa, bb, wait, _extra = extrastuff 
     screen = [aa, bb]
     if xeng_scale ne 1.00 then aa = aa/xeng_scale
     if yeng_scale ne 1.00 then bb = bb/yeng_scale
     if keyword_set(real_value) then begin
       aa = float(aa)
       bb = float(bb)
     endif
     if keyword_set (quiet) then return
     print, aa, bb
     return
     end
