; ------------------------------------------------------------------------ 
; $Id: dif.pro,v 1.5 2010/01/06 14:31:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  dif, wdf1, wdf2, clabel= clab, ylabel= ylab, xlabel = xlab
;+
;     Differentiate a wdf array using a simple differential technique
;       this will reduce the number of points by 1
;       (First X value is at (x[0]+x[1])/2 
;        and
;        First Y value is (Y[1] - Y[0])/(X[1] - X[0])
;    
;     In the case of X-Y data, derivative is set to zero if dx=0
;
;     Format:
;       DIF, WDF1 [, WDF2] [, CLABEL = clabel] [, YLABEL= ylabel] [, XLABEL = xlabel]
;
;       Where:
;         WDF1   - Input wdf array number
;         WDF2   - Output WDF array number.
;                  If missing, store the result in WDF1
;         CLABEL - Comment label for output WDF array
;         XLABEL - X-axis label for output WDF array
;         YLABEL - Y-axis label for output WDF array
;
;     Example:
;       Differentiate array 3 and store the derivative in wdf array 5
;         DIF, 3, 5
;-
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: DIF, WD1 [, WD2] [, CLABEL = clabel] '
        print, '              [, XLABEL = xlabel] [, YLABEL = ylabel]'
        return
      endif

;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameter to DIF must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
;
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, 'No data in the first array'
        return
      endif
;
;     GET the array and calculate the differential
;
      get_wdfarray, array1, wdf1, /xydata
      if n_elements (array1) ne np+ 2 then begin
        xydata = 1
        array2 = make_array(2*(np-1))
        dx = array1 (1:np-1)-array1(0:np-2)
        ok = where (dx eq 0, nok)
        if nok gt 0 then dx(ok) = 1.0
        array2(np-1:*) = (array1(np+1:np+np-1)-array1(np:np+np-2))/dx
        if nok gt 0 then begin
          array2(np-1 +ok) = 0.0
          dx[ok]=0       
        endif
        array2(0:np-2) = array1(0:np-2)+dx*0.5
      endif else begin
        xydata = 0
        array2 = make_array(np+1)
        array2(2:np) = (array1(3:np+1)-array1(2:np)) / array1(1)
;       derivative is for points 1/2 interval after the initial point
        array2(0) = array1(0) + array1(1) *0.5
        array2(1) = array1(1)
      endelse
;
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array2, save, xydata = xydata, /no_copy
      wdf2 = save
      wdffile(save-1) = wdffile(wdf1-1)
      if n_elements(clab) ne 0 then wdfcomment(save-1) = clab else $
        wdfcomment(save-1) = wdfcomment(wdf1-1)
      if n_elements(xlab) ne 0 then wdfxlabel(save-1) = xlab else $
        wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      if n_elements(ylab) ne 0 then wdfylabel(save-1) = ylab else $
        wdfylabel(save-1)= wdfylabel(wdf1-1)
      array1=0
      sp=0
      im=0
      return
      end
