; ------------------------------------------------------------------------ 
; $Id: factors.pro,v 1.4 2007/06/06 20:03:46 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function factors,xin, nfactors = nf
;+
; Return the factors of an integer.
; Format:
;       fact = factors (XIN  [, NFactors = nf])
;   where:
;       X        - Number for which factors are desired.
;                  For this routine: 
;                   X = Long(ABS(XIN[0])) where XIN is the value input.
;       NFactors - Number of factors found
; Example:
;   Find the factors of 500
;     ff = factors (500, nf = nf)
;     print, nf
;       5
;     print, ff
;           2           2           5           5           5
;    
;     Original version: February 25, 1999, lpmix
;-
 on_error,1
 x = long(abs(xin[0]))
 nf = 0
 if x le 3 then begin
   nf = 1
   return, x
 endif
; allow for a maximum size of 2^32 which should be fine!!
; since x is a long
;
 a = make_array(32, /long)
 while ((x mod 2) eq 0) do begin
   a(nf) = 2L
   nf = nf+1
   x = x/2
 endwhile
 j= 3L
 while (j*j le x) do begin
   if x mod j eq 0 then begin
     a(nf) = j
     nf = nf +1
     x = x/j
   endif else j = j+2L
 endwhile
 a[nf] = x
 a = (x gt 1) ? a[0:nf] : a[0:nf-1]
 if x gt 1 then nf +=1
 return, a
 end
     
