; ------------------------------------------------------------------------ 
; $Id: get_resp.pro,v 1.4 2007/06/06 20:03:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  get_resp, wdf1, wdf2, wdf3, filter = filter, plot = plot
;
;+
; pro  get_resp, wdf1, wdf2, wdf3, filter = filter, plot = plot
;   this procedure produces the response function required to get wdf1 from wdf2
;   wdf3 is optional, if present the response will be stored in wdf3
;   filter is an optional filter multiplier, Default is zero (no filter)
;   plot if present and nonzero plots the fft of the various signals
;-
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,'% GET_RESP: Must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf1) eq 0 or  n_elements(wdf2) eq 0 then begin
        print, '% GET_RESP: Parameters must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, '% GET_RESP: WDF1 array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 2 then save = get_save_array(wdf3, wdf1) else save = wdf1
;
      np1 = wdfnp(wdf1-1)
      if np1 eq 0 then begin
        print, '% GET_RESP: No data in the first array'
        return
      endif
;
      if wdf2 lt 1 or wdf2 gt maxwdfarray then begin
        print, '% GET_RESP: WDF2 array must be greater than 0 and </=', maxwdfarray
        return
      endif
      np2 =  wdfnp(wdf2-1)
      if np2 eq 0 then begin
        print, '% GET_RESP: No data in the second array'
        return
      endif
      get_wdfarray, array1, wdf1
      get_wdfarray, array2, wdf2
;     put array 2 on same time base as 1 
;     get a new array
      n = long ( (np2-1) * array2(1)/array1(1)+1.00)
      if (n lt 6 ) then print,' Warning - True signal has only',n, ' elements'
      ivalue = findgen(n)* array1(1)/array2(1)
      array2 = interpolate (array2(2:np2+1), ivalue, /grid)
;
      nzero = max([n, np1])*2
      if nzero lt 32 then nzero = 32 $
 else if nzero le 80 then nzero = ((nzero-1)/8+1)*8 $
 else if nzero le 160 then nzero = ((nzero-1)/16+1)*16 $
 else if nzero le 320 then nzero = ((nzero-1)/32+1)*32 $
 else if nzero le 640 then nzero = ((nzero-1)/64+1)*64 $
 else if nzero le 1280 then nzero = ((nzero-1)/128+1)*128 $
 else if nzero le 2560 then nzero = ((nzero-1)/256+1)*256 $
 else if nzero le 5120 then nzero = ((nzero-1)/512+1)*512 $
 else if nzero le 10240 then nzero = ((nzero-1)/1024+1)*1024 $
 else if nzero le 20480 then nzero = ((nzero-1)/2048+1)*2048 $
 else nzero = ((nzero-1)/4096+1)*4096
      array3 = make_array(nzero)
      array4 = array3
      array3(0:np1-1) = array1(2:*)
      array4(0:n-1) = array2
      array4 = fft(array4, -1)
      a4 = abs(array4)
      ma4 = max(a4)
      a = where (a4 lt 1.0e-7* ma4, nel)
      if nel gt 0 then begin
        array4(a) = complex(1,0)
        print , '% GET_RESP:  Warning --  some elements ofthe response FFT are zero'
        print, '              These modes will be set to one.'
        a4 = abs(array4)
      endif
      if (n_elements(filter) ne 0) then begin 
        temp  = array4* (1+ 0.005*filter* ma4*ma4/(a4*a4)) 
        if (n_elements(plot) ne 0) then begin
          ymax = max(1.0/a4)
          abtemp = 1.0/abs(temp)
          ymin = min(abtemp)
          plot, 1.0/a4, yr = [ymin, ymax], /ytyp
          oplot, abtemp
          hak, /mes
          plot, abtemp, /ytyp, tit = 'Transform vs mode'
          hak, /mes
          array4 = temp
          temp = 0
          abtemp = 0
        endif else begin
          array4 = temp
          temp = 0
        endelse
      endif else if (n_elements(plot) ne 0) then plot, 1.0/a4, /ytype
      if a(0) ne -1 then array4(a) = complex(1.0, 0.0)
;
      array3 = float( fft(fft(array3, -1)/array4/nzero,1))
;
;     ma3 =  max(array3)
;     a = where (array3 gt 0.002*ma3)
;
;
;     last = max(a)
;     if last gt np1 then begin
;       last = np1/2
;     endif  
;     array1(2:last+2) = array3(0:last) > 0
;     I will let the user worry about shifts
;     array3 = shift(array3 , np1/2)
      array1(2:*) = array3(0:np1-1) > 0
;     a = where (array1(5:last+2) eq 0)
;     if a(0) gt 0.0 then last = 5+a(0)
;     array1= array1(0:last+2)
      if n_elements (save) eq 0 then save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      put_wdfarray, array1, save, /no_copy
      wdf3 = save
      wdffile(save-1) = '  '
      wdfcomment(save-1) = wdfcomment(wdf1-1)
      wdfxlabel(save-1)= wdfxlabel(wdf1-1)
      wdfylabel(save-1)= wdfylabel(wdf1-1)
      a4 = 0
      array2 = 0
      array3 = 0
      array4 = 0
      sp=0
      im=0
      end
