; ------------------------------------------------------------------------ 
; $Id: hak.pro,v 1.5 2007/06/06 20:03:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
pro HAK, dummy, message = mesg, widget_hak = widget_hak, $
              byte_return = byte_return
;+
;
; NAME:
;       HAK
; FORMAT: 
;       HAK [, CHAR ] [, MESSAGE = message] [, /WIDGET_HAK]
;           [, /BYTE_RETURN]
;  where:
;       MESSAGE:   If MESSAGE is present then:
;                  If MESSAGE is a string, print the string. 
;                  Otherwise, print, "Hit any key to continue..."
;                  If MESSAGE is omitted, nothing will be printed.
;       CHAR:      Character entered from keyboard.  Not returned if a 
;                    widget is used.
;                  Note: CHAR is a byte if it is less than 32 or
;                        greater than 126.
;       BYTE_RETURN: If set, return a byte regardless of the key struck
;       WIDGET_HAK: If set, use a widget to determine if user is ready to 
;                   continue.  No character or byte is returned.
; 
; PURPOSE:
;       HAK is a procedure that performs "Hit any key to continue".  It waits
;         for keyboard input, clears the type-ahead buffer and allows the
;         application to continue.
;       If the device is either Postscript or Encapsulated Postscript,
;         the procedure returns with no operation.
; EXAMPLE:
;       Plot first 10 waveform arrays 
;       Note: First way demonstrates HAK, second way is most efficient
;       for i = 1, 10 do begin, plo, i  & hak, /mes & end
;    <or> plo, 1, 10
;
;       Determine if a person is finished.
;         HAK, answ, /byte, message='Hit Q to quit'
;         if (answ eq 113B) || (answ eq 81B) then return
;
; Modification History:
;       This routine is a modified version of a user routine from PV WAVE.
;       Except for the name many changes have been made.
;
;-
;
;
; Check for the message keyword.
;
common get_idl_hak_command, hakflag
if n_elements(hakflag) eq 0 then $
  defsysv, "!callable_idl", exists=hakflag
if !d.name eq 'PS' then return
if !d.name eq 'PCL' then return
if hakflag eq 0 and not keyword_set(widget_hak) then begin
  if keyword_set(mesg) then begin
     sm = size(mesg)
     if sm(1) ne 7 then begin          ; Print default string
        print, 'Hit any key to continue...'
     endif else begin                  ; Print user-defined string
        print, mesg
     endelse
  endif
;
; Wait for keyboard input before continuing (returning).
;
  empty
; get the next key press
  repeat dumb = byte(get_kbrd(1)) until $ 
      ((dumb[0] gt 7B) and (dumb[0] lt 128B))
  dummy = dumb[0]
  if ~keyword_set(byte_return) && (dummy gt 31B) &&  (dummy lt 127B) $
          then  dummy = string (dummy)
; Clear the buffer
  repeat dumb = byte(get_kbrd(0)) until (dumb[0] eq 0B)
  return
endif 

; Widget HAK 
if keyword_set(mesg) then begin
   sm = size(mesg)
   if sm(1) ne 7 then begin          ; Print default string
      messag = 'Hit OK to continue...'
   endif else begin                  ; Print user-defined string
      messag = mesg
   endelse
endif else messag = 'Hit OK to continue...'
Winformation, messag, xsize = 400
dummy = ' '

end
