; ------------------------------------------------------------------------ 
; $Id: lab.pro,v 1.4 2009/06/17 15:51:07 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro Lab, wdf, comment, append = append
;+
;     this routine changes the comment of an wdf array
;     format: lab, wdf, string [, append = append]
;     where
;       wdf is the WDF array number or an array of WDF array numbers
;       string - the comment and is used for the title of plots
;                If string is an array, then it must have the same number of 
;                elements as the number of valid arrays in wdf.
;       Append - if not zero the the current comment will be modified
;                if gt 0 then string is appended to the current comment
;                if lt 0 then string is prepended to the current comment
;     Example:
;       Label array 2 as `Ion Current vs Time'
;         lab, 2, `Ion Current vs Time'
;
;       Add shot number to dataset comments for arrays 1 to 10
;       Assume shot is the shot number.
;         for i =1, 10 do lab, i, wdcomment(i)+' - '+ strtrim(shot,2)
;   or    lab, indgen(10)+1, ' - '+strtrim(shot, 2), /append
;-
;     include the common block
@com.dat
;
      on_error, 1
;
;     see if we are initialized
      if n_elements(initwdf) eq 0 then begin
        print, '% Format: LAB, WDF, COMMENT, APPEND = append'
        return
      endif
;
;
;     determine if wdfarray is good
;
      nelwdf = n_elements(wdf)
      if nelwdf eq 0 then begin
        print, '% LAB:  WDF array must be defined'
        return
      endif
      ok = where (wdf gt 0 or wdf le maxwdfarray, nok)
      if nok eq 0 then begin
        print, '% LAB: WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif 
;     determine if we have valid comment
      nelcom = n_elements(comment)
      if nelcom eq 0 then begin
        print, '% LAB: Comment must be defined'
        return
      endif
;     determine if we have valid data
      wdfm = wdf[ok]-1
      oknp = where (wdfnp[wdfm] gt 1, nok, complem= wnok, ncomp= nwnok)
      if nwnok gt 0 then  Print,'% LAB: No valid data in wdf array =' , $
        strtrim (wdf[ok[wnok]], 2)
      if nok gt 0 then begin
        wdfm = wdfm[oknp]
        if nelcom eq 1 or nelcom eq nok then begin
          if keyword_set (append) then begin
            if append gt 0 then $
                wdfcomment(wdfm)= wdfcomment(wdfm) + comment $
            else wdfcomment(wdfm)= comment + wdfcomment(wdfm)
          endif else wdfcomment(wdfm)=  comment
        endif else print, '% LAB: Comment must be a scalar or have ' + $
          strtrim (nok, 2) + ' elements'
      endif
      return
      end
