; ------------------------------------------------------------------------ 
; $Id: label.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro label, mylabel, x, y, reset = reset,  _extra= extrastuff

;+
; NAME:
;	LABEL
;
; PURPOSE:
;	Add a label or text string to a plot using the cursor for placement.
;       The location is determined with the cursor
;       XYOUTS is used for the label.
;
; CALLING SEQUENCE:
;
;	LABEL [, LABEL_STRING] [, Xval] [, Yval] [, /RESET] 
;             [, _extra = extrastuff]
;
; Optional INPUTS:
;	LABEL_STRING - String to be placed on a plot
;                      If string is undefined, zero length, or not a string,
;                        then the user will be prompted for a string.
;       Xval         - X-value for the label in data coordinates.
;       Yval         - Y-value for the label in data coordinates.
;       Note: If missing or if keyword (reset) is set, then the values will be
;             set interactively.
;       RESET        - If set, the user will be expected to set the position of
;                      the label using the cursor.
;       _extra       - Alignment, charsize, charthick, Text-Axes, color, ...
;                      Any valid keyword for XYOUTS.
;       
; EXAMPLE:
;      Label a plot at a point to be determined with a string to be entered.
;        LABEL
;
;      Label a plot on the screen and save the values for a postscript plot.
;        labelstring = '' or labelstring = 0 ; since zero is not a string
;        < plot command >
;        label, labelstring, x, y, /reset
;        startpost
;        < plot command >
;       <Note: X and Y (set with the cursor) and the text string, entered 
;         on the terminal, are in variables.> 
;        label, labelstring, x, y
;        endpost
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 15, 1997
;-

on_error, 1

 if n_elements (mylabel) eq 0 then mylabel = 0 else mylabel = mylabel(0)
 ss = size(mylabel)
 if ss(1) ne 7 then begin
   mylabel = ''
   while strlen(strtrim(mylabel, 2)) eq 0 do $
    read, prompt='Enter a label: ', mylabel
 endif 
 
 if n_elements (x) eq 0 or n_elements(y) eq 0 or keyword_set(reset) then begin
   print, 'Use the mouse to indicate a position for the label:'
   cur, x, y
 endif

 xyouts , x, y, mylabel, _extra = extrastuff
 return
END

