; ------------------------------------------------------------------------ 
; $Id: ls.pro,v 1.5 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  ls, p1, current = current, help= help, quiet = quiet,  $
        nfiles = nfiles, _extra = extrastuff
;
;+
; NAME:
;       LS
;
; PURPOSE:
;       Show files in current directory.
; CATEGORY:
;       UTILITY 
; CALLING SEQUENCE:
;       LS [, SPEC] [, CURRENT = current] [, NFILES = nfiles] [, /QUIET]
;                   [, _EXTRA = extrastuff] 
; OPTIONAL PARMAETERS
;       SPEC      - File filter
;                   If SPEC is an array, only the first element is used.
; KEYWORDS:
;       CURRENT   - Files in the current directory 
;                or files which match the SPEC file filter.
;       NFILES    - Number of files found
;                   The following syntax is used:
;                   current = FILE_SEARCH (COUNT = nfiles)
;       _EXTRA    - Any valid keyword to FILE_SEARCH
;       QUIET     - If set, no output.  Keyword CURRENT must be used.
; OUTPUTS:
;       CURRENT directory file names or file names which match SPEC.
; EXAMPLES:
;       Show all files in the current directory
;             ls
;
;       Show all files which end in .dat
;             ls , '*.dat'
;
;       Show full path names to all files that begin with pbfa2z_8'
;       Put the number of files in nf, and put the filse in fils
;         LS, 'pbfa2z_8*', nfiles = nf, /fully_qualify, current = fils
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, January 3, 1997
;-
;
;
      on_error, 1
;
      if keyword_set (help) then begin
        print, '% Format: LS, P1, CURRENT = current, /QUIET, /HELP'
        return
      endif

       cd , current = current
       if strupcase(strmid(!version.os,0,3)) eq 'LIN' then resolve_it, current

       if not keyword_set(quiet) then print, 'Present Directory: ', current
       if n_elements(p1) eq 0 then begin
         current = file_search(count = nfiles, _extra = extrastuff)
         p = 0
       endif else begin 
         p1 = strtrim(p1(0), 2)
         p = strlen(p1)
         if p gt 0 then $
           current = file_search(p1, count = nfiles, _extra = extrastuff) $
           else current = file_search(count = nfiles, _extra = extrastuff)
       endelse
       if keyword_set (quiet) then return
       nel = nfiles
       if nel gt 0 then begin
         if p gt 0 then print, 'Current Files with filter = ', p1 else $
           print, ' Current Files:'
         lmax = max(strlen(current)) +1
         ncol = 80/(lmax)>1
         wid = 80/ncol
         nf = nel/ncol
         if nf*ncol lt nel then begin
           nf = nf+1
           temp = make_array(nf*ncol, /string, val = '')
           temp(0:nel-1) = current
         endif else temp =current
         temp = transpose(reform (temp, nf, ncol, /overwrite))
         col_pos = 2+indgen(ncol)*wid
         format = '('
         for i = 0, ncol-2 do format = format+'t'+strtrim(col_pos(i), 2) + $
           ', a, '
         format = format+'t'+strtrim(col_pos(ncol-1), 2)+ ', a)'
         
         for i = 0, nf-1 do print, temp(*, i), format= format 
         return
       endif
       if P gt 0 then print, 'No files with filter: ', p1 else $
        print, 'No files in this directory'
       return
       end
