; ------------------------------------------------------------------------ 
; $Id: parser.pro,v 1.3 2007/06/06 20:03:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro Parser, stringin, array, np, integer = integer, ignore = ignore, $
    stringarray = stringarray, delimiter = delimiter
;+
;  This routine converts a string to a float, integer array, or string array.
; format:  
;    PARSER, string, array, np, integer = integer, ignore = ignore, 
;            stringarray = stringarray
;     string     - Input string to separate
;                  If a zero length string, np = 0 and no values returned.
;     array      - Output array
;     np         - Number of elements
;     INTEGER    - Optional keyword.  If set, Array is converted to an integer
;                    with ROUND
;     STRINGARRAY- Optional keyword.  If set, Array is a string array.
;                  If stringarray is set, then one string will be returned 
;                  even if string is all blanks.  All blanks will be removed.
;     IGNORE     - Any nonnumeric characters to be ignored are listed 
;                  in ignore. See example. They are converted to spaces
;     DELIMITER  - Default value is space = 32B 
;                  Other values are tab = 9B
;
; Example:
;     Parse string into an integer array, array 
;       parser, string, array, np, /integer
;
;     Read all remaining records in a file , unit, and put the numbers in new
;     Ignore ",", "$", and "&"
;       npmax = 4096 & new = make_array(npmax)
;       i = 0
;       str = ' '
;       ignore = [",", "$", "&"]
;       while (not eof(unit)) and i lt npmax do begin
;         readf, unit, str
;         parser, str, array, np, ignore = ignore
;         if i+np ge npmax then np = npmax-i
;         if np gt 0 then new (i:i+np-1) = array(0:np-1)
;         i = i+np
;       endwhile
;       if i gt 0 then new = new(0:i-1) else new = 0
;-
       on_error, 1
       if n_params() eq 0 then begin
         print, $
         '%format: PARSER, STRING, ARRAY, NP, [/INTEGER] [, IGNORE = ignore]'
         print, '         [, /STRINGARRAY] [, DELIMITER = delimiter] '
         return
       endif
       if n_elements (stringin) eq 0 then begin
         Print, '% PARSER: Input string must be defined'
         return
       endif
       np = 0
       if strlen (stringin) eq 0 then begin
         print, '% PARSER: Null string - no points returned'
         return
       endif
       str= stringin(0)
       nig = n_elements(ignore)
       if nig gt 0 then begin
         bst= byte(str)
         for i = 0, nig -1 do begin
           ok = where(bst eq (byte(ignore(i)))(0), nok)
           if nok gt 0 then bst(ok) = 32B
         endfor
         str = string(bst)
       endif
       if n_elements(delimiter) eq 0 then begin
         str = strtrim(strcompress(str), 2)
         bst= byte(str)
         bd = 32B
       endif else begin
         bst = byte(str) 
         bd = (byte(delimiter(0)))[0]
       endelse
       leng = strlen(str)
       if leng eq 0 then begin
         if  keyword_set (stringarray) then begin
           np = 1
           array = stringin
           return
         endif
         print, ' Blank string - no points returned'
         np = 0
         return
       endif
       leng = n_elements(bst)
       if bst(0) eq bd or bst(leng-1) eq bd then begin
         if bst(0) eq bd then st = 1 else st = 0
         if (bst(leng-1) eq bd) then leng  = leng -1
         if leng gt st then begin
           bst = temporary(bst(st:leng-1))
           leng = n_elements(bst)
           str = strmid(str, st, leng)
         endif else begin
           if  keyword_set (stringarray) then begin
             np = 1
             array = stringin
             return
           endif
           print, ' Blank string - no points returned'
           np = 0
           return
         endelse
       endif
; if first or last item is a delimiter then shorten the string
       ok = where (bst eq bd, nok)
       np = nok+1
       if keyword_set(stringarray) then begin
         array = make_array(np, /string)
         if nok eq 0 then array(0) = strtrim(str, 2) else begin
           array(0) = strtrim(strmid(str, 0, ok(0)), 2)
           if nok gt 1 then for i = 1, (nok-1) do $
              array(i) = strtrim(strmid(str, ok(i-1)+1, ok(i)-ok(i-1)-1), 2)
           if nok gt 0 then array (nok) =  $
                strtrim(strmid(str, ok(nok-1)+1, leng-ok(nok-1)-1), 2)
         endelse
       endif else begin
         array = make_array(np)
         if nok eq 0 then array(0) = float(strtrim(str,2)) else begin
           array(0) = float(strmid(str, 0, ok(0)))
           if nok gt 1 then for i = 1, (nok-1) do $
              array(i) = float(strmid(str, ok(i-1), ok(i)-ok(i-1)))
           if nok gt 0 then array (nok) =  $
                float(strmid(str, ok(nok-1), leng-ok(nok-1)))
         endelse
         if keyword_set(integer) then array = round(array)
       endelse
       return
       end
