; ------------------------------------------------------------------------ 
; $Id: plo.pro,v 1.24 2011/07/20 14:04:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  plo, wdfinp, ncurvinput, wselect = wselect, over = over, xrange= xr, $
      yrange = yrin, nsymbol = nsymbol, setdefault = setdefault, unset = unset, $
      title= title, grid = grid, legend = leg, nogrid = nogrid, data= data,  $
      psymbol = symbol, linestyle = linestyle, color = color, nchar = nchar,  $
      nlinestyle = nlinestyle, thick = thickin, lgsp=legsp, charsize=charsize, $
      lcharsize = lcharsize, allsymbol = allsym, nextpos = nextpos, skip = skip, $
      right = right, left = left, showdefault = showdefault, fill = symfill, $
      symsize =symsize,lsymsize = lsymsize, header = header, lname = lnamein, $
      truncate = truncate, reset = reset,  wait = wait, hak = hak, t3d = t3d, $
      engineering = engineering, quick = quick, zeroline = zeroline, $
      spectrum = spectrum, blegend = blegend, ymargin = ymargin, $
      keep_colors = keep_colors, start_offset = start_offset, $
      absolute_value = abs_value, logrange = lograngein, front = front,  $
      xstyle =xstyle, ystyle = ystyle,  aspect = aspect,  _extra = extrastuff
      
;
;
;+
; NAME:
;       PLO
;
; PURPOSE:
;       Plot a WDF array or multiple WDF arrays
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLO, WDA [, NCURVE] [, WSELECT = WSELECT] [, FRONT= front] [, XRANGE =xr][, YRANGE =yr] 
;     [, /OVER][, TITLE =title][, GRID =grid][, LEGEND =leg][, /NOGRID] [, /NSYMBOL] 
;     [, PSYMBOL =symbol] [, FILL = fill ] [, SYMSIZE = symsize] [, /SPECTRUM]
;     [, LSYMSIZE = lsymsize] [, LINESTYLE =linestyle] [, COLOR = color] [, /KEEP_COLORS]
;     [, LGSP = lgsp] [, ALLSYMBOL = allsymbol][, NCHAR = nchar][,LNAME = lname]
;     [, DATA = data] [, THICK = thick ] [, /NLINESTYLE] [, CHARSIZE =charsize]
;     [, LCHARSIZE = lcharsize] [, NEXTPOS = nextpos] [, /RIGHT] [, /LEFT] [, SKIP = skip]
;     [, /SETDEFAULT] [, UNSET = unset] [, SHOWDEFAULT = showdefault] 
;     [, /TRUNCATE ] [, /Engineering] [, /QUICK] [, /ZEROLINE] [, /ASPECT ]
;     [, /WAIT] [, /HEADER] [, RESET = reset] [, BLEGEND = blegend]
;     [, START_OFFSET= start_offset] [, ABSOLUTE_VALUE = absolute_value]
;     [, LOGRANGE = logrange] [, _EXTRA = extrastuff]
; INPUTS:
;       WDA:      WDF array number or an array of WDF array numbers
;                   or a search string ('*' represents all arrays)
;                   If WDF is an array of strings, the results of the search
;                   strings will be concatenated and unique values selected.
;                   If the first value is a '*', then other values are ignored.
;       Note:     Data with more than 19999 points will be thinned to 3000
;                 pairs for postscript and 1500 pairs for other displays unless
;                 QUICK is set. The pairs will be in the range of the plot.
;                 Set keyword QUICK = 0 to avoid this thinning.
;                 If xrange is specified the resultant array must have at least
;                   4500 points (9000 for PS) for thinning to occur.
;                 (If x is not monotonically increasing then no automatic 
;                  thinning will occur.)
;                 
; OPTIONAL INPUT:
;       NCURVE:   Number of curves to be plotted.  If WDA is a single WDF 
;                 array number, then WDA and the following ncurve-1 arrays
;                 will be plotted.  If WDA is an array or the search
;                 string returns more than 1 value, then NCURVE is ignored.
;       WSELECT:  Ignored unless WDA is a string.  If WDA is a string, then 
;                 only dataset comments containing WSELECT are used.
;                 If WSELECT is an array, it will be applied sequentially.
;                 EXAMPLE:  Plot datasets beginning with Current and Voltage for
;                 shot 2345 assuming these strings are in the dataset comments.
;                   PLO, ['^current', '^voltage'], wselect ='2345'
;                 See GET_MATCH for discussion of string matching.
;
; KEYWORD PARAMETERS:
;       FRONT:    Use WSHOW to pop current window to the front.
;                 This can be set with the /SETDEFAULT keyword.
;                 DEFAULT: FRONT = 1
;       SPECTRUM  Indicates a spectrum from red to blue to be used for the 
;                 curves.  See the RAINBOW command.  The user may also issue
;                 the spectrum command and specify colors in the PLO command. 
;                 Spectrum sets the following if they are not set:
;                   NLINESTYLE =  1
;                   NSYMBOL    =  1
;                   LEGEND     = -1
;                   LCHARSIZE  =0.6
;                 The following commands are identical:
;                    rainbow, 5
;                    plo, 1, 5, /over, /nl, /ns, leg = -1, color = 10+indgen(5)
;                 or
;                    plo, 1, 5, /over, /spectrum
;                 NOTE:  If you use /spectrum and desire a legend, you should use
;                        both the rainbow and PLO commands:
;                    rainbow, 10
;                    plo, 1, 10, /over, /spectrum, legend = [0.6, 0.94]
;                    loadxyct
;                    These commands will load the rainbow spectrum, do the plot and 
;                      restore the rainbow spectrum.  LOADXYCT restores the original 
;                      color table.  The default behavior loads the rainbow spectrum,
;                      does the plot, and then goes back to the default color table
;                      before the legend is called.
;       COLOR:    Indicates the color to be used for the curve.  
;                 See also SPECTRUM keyword.  If /OVER is 
;                 present the successive curves will be drawn in different 
;                 colors.  If a single color>0 is given, it will be used as the
;                 first color and the colors below will be sequenced over 
;                 color = (color +indgen(7) ) mod 7 + 1.
;                 If the color is zero, then color = !p.color. If 
;                   more than 1 color is specified, these colors will be sequenced.
;                 If color[0] is negative, it is converted to an array and the absolute 
;                   value used for the color index.  This allows specific colors to be 
;                   specified when /NOGRID is specified.
;                 The colors are: red(1), green(2), blue(3),
;                   magenta(4), orange(5), cyan(6) and yellow(7).
;                 Note: For a B/W plot, color = 0.  Actual value depends on !p.color
;                 The color of a particular curve is determinied by the following
;                 relationship:
;                   curve_color = color[Total_number_of_curves mod n_elements(color)]
;       KEEP_COLORS: This keyword is only checked if NOGRID is set.
;                 If NOGRID is set, then the color table will be the same as used on the 
;                 last plot (PLO) command. 
;                 NOTE: SETDEFAULT can be used to define a color table for an extended time.
;       XRANGE, YRANGE - two element vectors indicating the desired range for
;                 the plots in the horizontal and verticle directions. If XSTYLE
;                 and YSTYLE are not specified then these values will be rounded
;                 to "nice" values. If a single value is specified, the minimum
;                 and maximum of the data will be used.
;                 If xrange is specified and quick is not set to zero, the data 
;                   will be set to the XRANGE with a fudge of 25% of XRANGE
;                   additional data points on each end.
;                 If engineering is specified, these values will be automatically
;                 scaled.
;       LOGRANGE: If a log scale is specified with /ytype or /ylog, the minimum will 
;                 be specified as LOGRANGE*YRANGE[1] unless YRANGE is specified.
;                 LOGRANGE must be greater than zero and less than 1.
;       ENGINEERING: If set, maximum/minimum values will be in the interval 
;                 (1, 1000).  For example, if maximum is 1.0e7 volts and 
;                 250 e-9 sec  then the y values will be multiplied by 1e-6
;                 and ' x1e6' will be added to the y label and x values 
;                 multiplied by 1e9 and ' x1e-9' added to the x label.
;                 Caution - WARNING -Beware: If XTITLE or YTITLE are specified
;                   then the title will be as specified, regardless whether 
;                   the data has been scaled.
;                 If xrange and yrange are specified, they will be scaled.
;       ASPECT    If set, X and Y axes will have the same relative scale
;       TITLE     Plot Title. Default is the curve label unless over is set.
;                 If TITLE is an array, multiple lines will be printed.
;       GRID      Indicates the number of plots to a page with a scalar value or
;                 the number of plots in the horizontal and vertical directions
;                 with a vector.  With a scalar value, N, the number of plots 
;                 in the vertical direction is approximately sqrt(N). GRID is 
;                 ignored with overlay plots and for plots with a single curve.
;       HEADER    If specified a plot header, consisting of a date or dates 
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default 
;                 DEFAULT is HEADER = 1
;                 HEADER = 0 will eliminate the header.
;                 HEADER parameters (character size, spacing, alignment).
;                  If header = -1, the date is the current date and time.
;                  If header = -2, the date is the modification date of the file.
;                    (On certain installations, the dataset comment contains
;                     the string 'DATETIME='.  The string after this flag 
;                     is used as the time for the shot. A ';' is taken as 
;                     then end of this field. If a ; is not found the string is 
;                     assumed to be 20 bytes long. Time should be the last 
;                     parameter in this field; time of the form hh:mm:ss is 
;                     truncated at the second colon.)
;                 If the file is not specified, then the date is the current
;                   date regardless of the value of header.
;                 HEADER = -3 or any positive number(>0) will give both dates 
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;       LEGEND    Specifies the location of the legend on overlaid plots.
;                 If LEG = -1, no legend is drawn. Default is [0.04, .94] 
;                 relative to the grid
;                 If LEG = 1 (/LEGEND) and KEYWORD_SET(RIGHT) then 
;                 default is [0.76, .94]
;       TRUNCATE  If set to zero, the full dataset comment will be used as 
;                   a title. Default is to not limit the title to the characters
;                   before the first ' - ' in the comment. (TRUNCATE =0)
;                   If truncate is set only caracters before a ' - ' will be used
;       QUICK     If set and the dataset has more than 3*NQ points, reduce the 
;                 plotted data to 2*NQ where NQ = max(nn, quick) 
;                 (nn = 1500 for postscript, 750 otherwise.)
;          Note:  The actual data is not modified.
;          Note:  Strange results can occur if the data is non-uniform.
;                 Example:
;                    Plot array 1 with the default 500 intervals (1000 pts).
;                       plo, 1, /q
;                 Technique: Same as automatic filtering but the defaults are 
;                            less by a factor of 3. Also no check will be made on
;                            whether the abscissa values are monotonically
;                            increasing with XY data.
;                   1.  Reduce the data by a fixed ratio - n
;                           (up to (n-1) points can be lost for the plot)
;                           (if n is less than 2 ignore this keyword)
;                   2.  For each n points determine a minimum and a maximum.
;                   3.  Assign the minimum amplitude and the maximum to an 
;                       X-value near the midpoint of the bin.
;                   4.  Plot the thinned data.
;       LGSP      If specified, the legend spacing is multiplied by this 
;                 parameter.
;       NCHAR     Number of characters in the legend 
;                 Default is 30 for PostScript and 12 otherwise.
;       LNAME     Array of names (strings) for use in the legend.
;                 The number of elements must be the same as or greater than 
;                    the number of curves.
;                 Default is the dataset comment for each waveform.
;                 If NCHAR is not specified, then all characters are printed.
;                 If NCHAR is specified, then each element of LNAME will be 
;                        truncated to NCHAR characters.
;       DATA      If present and not zero, indicates LEG is in DATA coordinates 
;                 Default is DATA = 0 unless RIGHT is set.
;       LINESTYLE Indicates the type of line to be used.  Lines specified by
;                 LINESTYLE are: solid(0), dot(1), dash(2), dot-dash(3),
;                 dot-dot-dot-dash(5), and long dash(5).  If the keyword OVER
;                 is present then if LINESTYLE contains:
;                 a.  more than one value, then the linestyle types 
;                       are sequenced over the specified values.
;                 b.  a single value not equal to -1, then the linestyle types 
;                     are given by LINESTYLE= (LINESTYLE + [0,2,3,4,5,1]) mod 6
;                 c.  If LINESTYLE is -1, a solid line is used for all plots.
;                 If the keyword OVER is not present, then all plots are done
;                   with LINESTYLE (0) unless an array is specified.
;       /NOGRID   Indicates the curve should be plotted on the existing grid.
;       /OVER     Indicates curves are to be overlaid. A legend is printed to
;                 distinguish curves.  The location of the legend can be 
;                 specified with the keyword LEG. Linetypes and symbols are 
;                 used automatically. 
;                 Title by default is '' if over is set.
;       PSYMBOL   Indicates the type of line and symbol to be used.  If is less
;                 than zero, a symbol will be plotted at every point and no 
;                 line will be drawn.  If SYMBOL is greater than zero, only
;                 about 18 symbols will be plotted and lines will connect the
;                 points.  Values of SYMBOL are: plus sign(1), 2 (asterisk),
;                 3 (period), 4 (diamond), 5 (triangle), 6 (square), 7 ("X"),
;                 8 (USER Defined) and 10 is histogram mode with horizontal
;                 and vertical lines connecting the points.  Default for 
;                 USER Defined is a filled circle. 
;                 Note: PSYMOL = 10 and PSYMBOL = -10 have identical results.
;                 Note: GET_SYMBOL_POINTS is used to set symbol point position
;                       Value of the number of points is the set in WDERR.
;                       To have 10 points use:
;                         WDERR, /set , npoint = 10 
;                 (If the symbol value is greater than 10 then ll will be 
;                 subtracted from the value and MKSYMBOL will be called.)
;                 (See help on SYMBOL_LIST for a description of additional
;                 symbols.)
;                 Note: Symbols may also be specified as names:
;                   PSYM=['sq', 'ci', 'diamond', 'clover']
;                   These will be converted to numbers. [11, 12, 14, 21]
;                   A null or blank string('' or '  ') will result in no symbol.
;                 If the keyword OVER is present, then if SYMBOL contains:
;                 a.  more than one value, then the SYMBOL types 
;                       are sequenced over the specified values values.
;                 b.  otherwise symbol is the set value of PSYMBOL
;                     Default value of the saved value is [1,2,4,5,6,7]
;                 If OVER is not set then the default is no symbol.
;                 The absolute value of symbol is used and the frequency is
;                 governed by allsymbol. The keyword NSYMBOL will eliminate all
;                 symbols.
;      FILL       This has no effect on symbols 1 to 10. On symbols 11 and higher
;                 the symbols will be filled if FILL is not zero.
;                 IF FILL has n values, then for multiple curves FILL has the  
;                   value FILL(i mod n) for the ith curve.
;                 Example: psym = ['sq', 'sq', 'ci', 'ci'] , fill = [0, 1]
;                   will provide a filled and unfilled square and then the same
;                   for a circle.
;      SYMSIZE    Symbol size multiplier
;      LSYMSIXE   Legend symbols use LSYMSIZE as a multiplier to the default
;                 LSYMSIZE is a scalar 
;      ALLSYMBOL  If present and not zero, a symbol is plotted at every point.
;                 If allsym is negative, only symbols will be plotted.
;                  (This is the same as psymbol negative, but if both allsymbol
;                    and psymbol are negative then they will cancel.)
;                 If allsym is positive, a line will be drawn with a symbol at 
;                      every point
;      /NLINESTYLE  If present and nonzero, all the lines will be solid 
;                   This is the same as linestyle = -1
;                   Note: If NLINESTYLE is set, then linestyle is set to -1.
;      /NSYMBOL  If present and nonzero, no symbols will be plotted in the 
;                 overlay mode.  ALLSYMBOL is set to zero.
;                Note: If PSYMBOL is set, NSYMBOL is set to zero.
;      THICK     Line thickness, default is !p.thick
;      CHARSIZE  Parameter sent to the PLOT command to vary the character size
;                CHARSIZE is used to scale !P.CHARSIZE if it has been changed
;                  from the default value of zero.
;                This keyword also affects the legend text. See LCHARSIZE below.
;                Default is CHARSIZE = 1. * !p.charsize
;      LCHARSIZE Parameter sent to the LEGEND command to vary the character size
;                in the legend. The value sent is actually LCHARSIZE*CHARSIZE
;                Default is LCHARSIZE = 1.
;      NEXTPOS   Next legend position in data coordinates. This keyword can be
;                used to add text below a legend. 
;      LEFT      Indicates a new plot using the left axis.
;                A single axis will be drawn on the left side of the grid
;                For this plot XMARGIN will be set to 
;                   XMARGIN = [!X.margin(0), !X.margin(0)]
;                and YSTYLE = 8.
;                The LEGEND will add a line "LEFT AXIS" unless LEFT is less than zero.
;                NOTE: If LEFT is set, NOGRID is set to 0.
;                NOTE: If LEFT is set, OVER is set to 1 if more than 1 curve.
;                NOTE: Multiple plots may be made by setting NOGRID.  LEFT can only
;                      be set for the first plot command.
;      RIGHT     Indicates a NOGRID plot making and using the right axis.
;                If RIGHT is present and not zero, then:
;                  A single axis will be drawn on the right side of the existing
;                    grid
;                  The wdf array(s) will be overplotted on the existing grid.
;                Before using this command, a plot should be created using
;                  the keyword: /LEFT
;                The LEGEND will have a line "RIGHT AXIS" unless RIGHT is less than zero.
;                NOTE: If RIGHT is set then NOGRID is set to 1.
;                NOTE: After this command all memory of the original y-axis 
;                      scaling is lost
;                NOTE: Multiple plots may be made by setting NOGRID.  RIGHT can only
;                      be set for the first plot command on the right axis.
;      SKIP      To avoid a space between RIGHT and LEFT axis labels, set SKIP to zero.
;                "SKIP" lines will be left beween RIGHT and LEFT labels on two-axis plots.
;                DEFAULT:  IF RIGHT is set, SKIP = 1
;      ZEROLINE  Indicates a horizontal line at y = 0 is to be drawn.
;                If keyword_set(right) then left and right zero's will be 
;                  aligned.
;                If n_elements(zeroline) eq 1 and zeroline lt -255, 
;                  no zero line will be drawn.
;                If n_elements(Zeroline) eq 1 and zeroline gt 0, a solid line
;                  with color = !p.color will be drawn.
;                If n_elements(Zeroline) eq 1 and zeroline lt 0, a solid line
;                  with color = abs(zeroline) will be drawn.
;                If n_elements(Zeroline) ge 2 then a line with 
;                  color = abs(zeroline(0)) and style = abs(zeroline(1))
;                  will be drawn. (See LINESTYLE for options)
;
;      WAIT      If present and  not zero, PLO will pause after every plot until
;                a key is pressed. This can be used with statements such as:
;                  for i = 3, 9, 3 do plo, i, 3, /ov, /w
;      BLEGEND   Parameters for a box around the legend.
;                This is passed to the legend command as an extra keyword.
;                If BLEGEND is not a structure but is defined, then 
;                  BLEGEND={BOX:1} 
;                and a box will be drawn around the legend.
;                This keyword may also be used to set the color of the printing 
;                or any other keyword to the LEGENDP command.
;      _EXTRA    Any valid parameters for the plot command may be entered.
;                These inclued: XTITLE, YTITLE, XMARGIN, YMARGIN, 
;                XSTYLE, YSTYLE, XTYPE, YTYPE, T3D, Zvalue,  etc.
;     START_OFFSET - Character offset between top of the graph and the title.
;                Default: START_OFFSET = 0.5
;     ABSOLUTE_VALUE - If set, the plot will have only positive or negative 
;                numbers.
;                If ABSOLUTE_VALUE equals -1 then the ordinate y is defined
;                as ABS(Y), where y are the ordinate values of WDF array.
;                NOTE:  This is dangerous and could confuse some people.
;                If  ABSOLUTE_VALUE is not zero and not -1, then PLO will 
;                determine if the WDF array has a larger positive deflection
;                or a negative deflection.
;                If the largest deflection from zero is positive, yrange[0] = 0.0
;                If the largest deflection from zero is negative, yrange[1] = 0.0
;     SETDEFAULT-If present and not zero, then the following parameters will be 
;                saved and used as the default values in all future calls to PLO
;                If a keyword is specified, it will always be used rather than
;                the saved values.  This keyword may be used with a regular 
;                plot command; for example:
;                  PLO, 1, 3, /over, /ns, /nl, /co, xr = [0, 150e-9], /set 
;                will set default values of 1 for color, nlinestyle, over, 
;                xrange and nsymbol. 
;                To temporily change use the keyword; for example,
;                  PLO, 1, 3, xr = [20, 80]*1e-9 
;                will use the above keywords but xrange would now be 20 to 80ns
;                The default value remains 0 to 150 ns until a new value is set
;                  or until PLO is called with UNSET.
;                To temporarily use auto scaling for X: plo, 1, 3, xr = 0
;
;                Normal default values are listed below.
;                Keyword     Default value
;                COLOR       0
;                OVER        0
;                FRONT       1
;                XRANGE      0
;                YRANGE      0
;                LINESTYLE   [0,2,3,4,5,1] (LINESTYLE= -1 implies NLINESTYLE= 1)
;                GRID        0
;                PSYMBOL     [1,2,4,5,6,7]
;                FILL        0
;                HEADER      1
;                SYMSIZE     0
;                SPECTRUM    0
;                LEGEND      [0.04, 0.94]
;                NCHAR       0  (NCHAR = 0 implies using the default of 30/12)
;                BLEGEND     0
;                DATA        0
;                NSYMBOL     0
;                ALLSYMBOL   0
;                CHARSIZE    1
;                ZEROLINE    0
;                LCHARSIZE   1
;                ENGINEERING 0
;                TRUNCATE    0
;                LNAME       0 (Use dataset comment)
;                BLEGEND     0
;                ABSOLUTE_VALUE 0
;      RESET {or}
;      UNSET     If present and not zero, will set all save values of keywords
;                to their default values. This keyword can be used alone or in a
;                regular plot command with other plot values or keywords
;                including the SETDEFAULT keyword
;                Examples: 
;                  Set the above defaults:
;                        plo, /unset
;                  Set the above defaults, make over = 1 the new default and 
;                    plot curves [1,2,3] in an overlay plot: 
;                        plo, 1, 3, /over, /set, /unset
;      SHOWDEFAULT Show the present default plot values.
; OUTPUTS:
;       Designated datasets are plotted.
; COMMON BLOCKS:
;       COM.DAT  WDF common block 
; RESTRICTIONS:
;       WD1 array must be defined and have valid data
; PROCEDURE:
;       If NCURVE is specified:
;            WDA and NCURVE-1 datasets are plotted in the specified form.
;       Otherwise:
;            The dataset WDA or the array of datasets defined by WDA are plotted.
; EXAMPLES:
;  Plot array 1 using the default settings.
;    plo, 1
;
;  Plot array 2 using diamond symbols and a dashed line on the existing graph.
;    plo, 2, psym = 4, line= 2, /nogrid
;
;  Plot arrays 1,..., 6 using different colored lines on a single grid
;    plo, 1,6,/ov, /col
;
;  Plot array 1 in a log-log plot with exact axis scaling in the X direction.
;    plo, 1, xr = [1e6, 12e6], /xstyle, /xlog, /ylog 
;
;  Plot an overlay of arrays 1,..., 6 with  abscissa of 0 to 100 and 
;  range of 0 to 10
;    plo, 1,6, xr = [0, 100], yr = [0, 10], /over
;
;  Plot arrays 1,..., 10 on separate grids with 6 grids to a page
;    plo, 1, 10, g=[3,2]
;or: plo,indgen(10)+1,g=6
;
;  Plot arrays 1, 4,5, 9 on one grid
;    PLO, [1,4,5,9], /ov
;
;  Plot arrays 3 and 6 on the existing grid using colors 2 and 4.
;    PLO, [3, 6], /NOGRID, color = [-2, 4]
; or PLO, [3, 6], /NO, co = -[2,4]
; or PLO, 3, /no, col=-2 & plo, 6, col=-4, /no
;
;  Plot all arrays beginning with `mspin' on a common time base from 
;  0 to 200 ns.  This requires two commands.  Plot 4 horizontal and 3 
;  vertical to a page.
;    re, 1, `^mspin', nc = ncurve
;    plo, 1, ncurve, xr = [0, 200e-9], gr = [4, 3];
; In the above example, overlay plot only those arrays which begin 'mspin2'
;    plo, 'mspin2', /over
;
;  Plot wdf arrays 1 and 11 on the left axis, and 2 and 12 on the right axis.
;    plo, [1, 11], /left
;    plo, [1, 11]+1, /right
;
;  Plot the above but put the right legend on the right side of the plot.
;    plo, [1, 11], /left
;    plo, [1, 11]+1, /right, /leg
;
;  Plot the above but put the right legend at a different location.
;    plo, [1, 11], /left
;    plo, [1, 11]+1, /right, leg=[0.8, .7]
;
;  Plot the above but add arrows to the legend header and put the right legend
;  in the default right position.
;    plo, [1, 11], left ='<<-- Left Axis'
;    plo, [1, 11]+1, right = 'Right Axis -->>', /leg
;
; Plot the above in the format of 2007 and before.
;    plo, [1, 11], left=-1
;    plo, [1, 11]+1, right=-1, skip=0
;
; MODIFICATION HISTORY:
;      7/7/94   lbb   Added HEADER keyword
;      Original version, L. P. Mix, May 15, 1992
;-
; include wdf common blocks
;
@com.dat

@plo_defaults.com

 common plot_grid_last, yeng_scale, xeng_scale
 common mkcircle, lcmkcircle, nel_symbols
 common plo_color_keep_common, keep_colors_save

;
@symbol_list.pro
;  ; Special symbols available with MKSYMBOL 
;    and their number and character representation
; #    Name     Character
; 0    Square     SQ
; 1    Circle     CI or O
; 2    Triangle   TR
; 3    Diamond    DI
; 4    Big X      BI or X
; 5    Delta      DE
; 6    Star       ST
; 7    Plus       PL or +
; 8    Hexagon    HE
; 9    Pentagon   PE
; 10   Clover     CL
; 11   PACMAN     PA
; 12   Spiral     SP
; 
; symbol_list = ['SQUARE', 'CIRCLE', 'TRIANGLE', 'DIAMOND', 'BIGX', $
;            'DELTA', 'STAR', 'PLUS', 'HEXAGON', 'PENTAGON', $
;            'CLOVER', 'PACMAN', 'SPIRAL']
;
      if n_elements(nel_symbols) eq 0 then nel_symbols = 12
;
      on_error, 1
      npar = n_params()
      if n_elements(yeng_scale) eq 0 then begin
        xeng_scale = 1.00
        yeng_scale = 1.00
      endif
      if keyword_set(unset)  or n_elements (color_save) eq 0 then begin
        COLOR_SAVE      = 0
        FRONT_SAVE      = 1
        KEEP_COLORS_SAVE= 0
        ABSOLUTE_SAVE   = 0
        OVER_SAVE       = 0
        XRANGE_SAVE     = 0
        YRANGE_SAVE     = 0
        LINESTYLE_SAVE  = [0,2,3,4,5,1]
        OLD_LINEST_SAVE = LINESTYLE_SAVE
        GRID_SAVE       = 0
        PSYMBOL_SAVE    = [1,2,4,5,6,7]
        SYMFILL_SAVE    = 0
        SYMSIZE_SAVE    = 0
        HEADER_SAVE     = 1
        LNAME_SAVE      = 0
        LEGEND_SAVE     = [0.04, 0.94]
        BLEGEND_SAVE    = 0
;       THICK_SAVE      = !p.thick
;                THICK       !p.thick (0)
        NCHAR_SAVE      = 0  ; NCHAR = 0 implies using the default of 30/12
        DATA_SAVE       = 0
        SPECTRUM_SAVE   = 0
        NSYMBOL_SAVE    = 0
        ALLSYM_SAVE     = 0
        CHARSIZE_SAVE   = 1
        ENGINEERING_SAVE= 0
        TRUNCATE_SAVE   = 0
        LCHARSIZE_SAVE  = 1
        ZEROLINE_SAVE  = 1
      endif
      if n_elements(symbol) gt 0 then begin
        a = size(symbol)
        if a(a(0)+1) eq 7 then begin
          nel = a(a(0)+2) 
          temp = make_array(nel, /int)
          for i = 0, nel-1 do begin
            s = strupcase(strcompress(symbol(i), /remove_all))
            l = strlen(s)
            if l eq 1 then begin
              if s eq 'X' then s = 'BIGX' else $
              if s eq '+' then s = 'PLUS' else $
              if s eq 'O' then s = 'CIRCLE' 
              l = strlen(s)
            endif
            if l eq 0 then temp(i) = -11 else begin
              templist = strmid(symbol_list, 0,l)
              ok = where (s eq templist, nok )
              if nok eq 0 then begin
                print, 'A valid symbol string must be specified - ', s
                return
              endif else if nok eq 1 then temp(i) = ok else begin
                print, ' Multiple symbols specified, using ' , s
                print, symbol_list(ok)
                temp(i) = ok(0)
                print, '   Using a symbol of ', symbol_list(ok(0))
              endelse
            endelse
          endfor
          symbol = temp+11
        endif
      endif
      if keyword_set (setdefault) then begin
        if n_elements(color) gt 0 then color_save = color
        if n_elements(front) gt 0 then front_save = front[0]
        if n_elements(abs_value) gt 0 then absolute_save = abs_value
        if n_elements(linestyle) gt 0 then begin
          linestyle_save = linestyle
          OLD_LINEST_SAVE = LINESTYLE_SAVE
        endif
        if n_elements (nlinestyle) gt 0 then begin
          if nlinestyle ne 0 then linestyle_save = -1 else $
            linestyle_save = old_linest_save
        endif
        if n_elements(over) gt 0 then over_save = over
        if n_elements (xr) gt 0 then xrange_save = xr
        if n_elements (yrin) gt 0 then yrange_save = yrin
        if n_elements (grid) gt 0 then grid_save = grid
        if n_elements (lnamein) gt 0 then lname_save = lnamein
        if n_elements (symbol) gt 0 then psymbol_save = round(symbol)
        if n_elements (symfill) gt 0 then symfill_save = symfill
        if n_elements (symsize) gt 0 then symsize_save = symsize
        if n_elements (leg) gt 0 then legend_save = leg
        if n_elements (blegend) gt 0 then blegend_save = blegend
;       if n_elements (thickin) gt 0 then thick_save = thickin
        if n_elements (nchar) gt 0 then nchar_save = nchar
        if n_elements(data) then data_save = data
        if n_elements(nsymbol) gt 0 then NSYMBOL_SAVE = nsymbol[0] 
        if n_elements(allsym) gt 0 then ALLSYM_SAVE = allsym[0]
        if n_elements(charsize) gt 0 then CHARSIZE_SAVE = charsize
        if n_elements(zeroline) gt 0 then ZEROLINE_SAVE = zeroline
        if n_elements(header) gt 0 then HEADER_SAVE = header
        if n_elements(engineering) gt 0 then ENGINEERING_save = engineering
        if n_elements(truncate) gt 0 then TRUNCATE_save = truncate
        if n_elements(lcharsize) gt 0 then LCHARSIZE_SAVE  = lcharsize
        if  n_elements(spectrum) gt 0 then SPECTRUM_SAVE  = spectrum
      endif
      if keyword_set(showdefault) then begin
         print, 'Default parameters for the PLO command are:'
         print, '    COLOR      = ' , strtrim (color_save, 2)
         print, '    FRONT      = ' , strtrim (front_save, 2)
         print, '    OVER       = ' , strtrim (over_save, 2)
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    LINESTYLE  = ' , strtrim (LINESTYLE_SAVE, 2)
         print, '    SPECTRUM   = ' , strtrim (spectrum_save, 2)
         print, '    GRID       = ' , strtrim (grid_save, 2)
         print, '    PSYMBOL    = ' , strtrim (psymbol_save, 2)
         print, '    FILL       = ' , strtrim (symfill_save, 2)
         print, '    SYMSIZE    = ' , strtrim (symsize_save, 2)
         print, '    LEGEND     = ' , strtrim (legend_save, 2)
;        print, '    THICK      = ' , strtrim (thick_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
         print, '    NCHAR      = ' , strtrim (NCHAR_SAVE, 2)
         print, '    LNAME      = ' , LNAME_SAVE
         print, '    DATA       = ' , strtrim (DATA_SAVE, 2)
         print, '    NSYMBOL    = ' , strtrim (nsymbol_save, 2)
         print, '    ALLSYMBOL  = ' , strtrim (allsym_save, 2)
         print, '    CHARSIZE   = ' , strtrim (charsize_save, 2)
         print, '    ZEROLINE   = ' , strtrim (zeroline_save, 2)
         print, '    ENGINEERING= ' , strtrim (engineering_save, 2)
         print, '    TRUNCATE   = ' , strtrim (truncate_save, 2)
         print, '    ABSOLUTE   = ' , strtrim (absolute_save, 2)
         print, '    LCHARSIZE  = ' , strtrim (lcharsize_save, 2)
         if size(/type, blegend_save) eq 8 then begin
           print, '    BLEGEND:'
           help, /str, blegend_save
         endif else print, '    BLEGEND    = ' , strtrim (blegend_save, 2)
         if npar eq 0 then return
      endif
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) $
        or keyword_set(reset) ) then return

      if npar eq 0 then begin
 print,'% Format: PLO, WDA [, NCURVE][, /OVER][, XRANGE =xr][, YRANGE =yr]'
 print,'   [, TITLE =title][, GRID =grid][, LEGEND =leg][, /NOGRID] [, /NSYMBOL]'
 print,'   [, PSYMBOL =symbol] [, SYMSIZE = symsize] [, FILL = fill]'
 print,'   [, LINESTYLE =linestyle][, COLOR = color] [, LNAME = lname]'
 print,'   [, LGSP = lgsp] [, /ALLSYMBOL] [, NCHAR = nchar][, NEXTPOS=nextpos]'
 print,'   [, DATA = data] [, THICK = thick ] [, LCHARSIZE = lcharsize] '
 print,'   [, /NLINESTYLE] [, CHARSIZE = charsize] [, RESET = reset]'
 print,'   [, /QUICK] [, /ZEROLINE ] [, /ASPECT ] [, /LEFT] [, /RIGHT]'
 print,'   [, SETDEFAULT = setdefault] [, SHOWDEFAULT = showdefault]'
 print,'   [, TRUNCATE = truncate] [, /ENGINEERING] [, /WAIT] [, /HEADER]'
 print,'   [, ABSOLUTE_VALUE = absolute_value] [, FRONT = front]'
 print,'   [, /SPECTRUM] [, _EXTRA = extrastuff]'
        return
      endif
;
;     make sure we don't corrupt input data
      if n_elements(wdfinp) gt 0 then  wdf = wdfinp else begin
        print, '% PLO: WDF array(s) must be specified'
        return
      endelse

      a = size(wdf)
      if a(a(0)+1) eq 7 then begin
         nwdf = n_elements(wdf)
        if strtrim(wdf[0], 2) eq '*' then wdf = wdvalid() else begin
          wdfp = wdvalid(wdf[0])
          if nwdf gt 1 then begin
            for i = 1, nwdf-1 do wdfp = [wdfp, wdvalid(wdf[i])]
            ok = where (wdfp gt 0, nok)
            if nok gt 0 then wdfp = wdfp[ok]
          endif
          wdf = temporary(wdfp)
        endelse
        if wdf(0) lt 1 then begin
          print, '% PLO: No arrays found using the search string: ', wdf[0]
          if nwdf gt 1 then for i = 1, nwdf-1 do print, wdf[i]
          return
        endif
        if n_elements(wselect) gt 0 then begin
          wdcom = wdcomment(wdf)
          nsel = n_elements(wselect)
          nelem = n_elements(wdf)
          for i = 0, nsel-1 do begin
            wdfp = get_match(wselect[i], wdcom, nelem = nelem)
            if nelem eq 0 then begin
              print, '% PLO: No arrays found using the select string: ', wselect[i]  
              return
            endif
            if nelem lt nwdf then begin
              wdf = wdf[wdfp]
              wdcom = wdcom[wdfp]
              nwdf = nelem
            endif
          endfor 
        endif
      endif

      if n_elements(ncurvinput) gt 0 then ncurve = ncurvinput
      pthick = (!p.thick eq 0.0) ? 1:!p.thick
      thick = (n_elements (thickin) eq 0) ? pthick : thickin*pthick
;     if n_elements(thick) eq 0 then thick = thick_save
      
      lclname = n_elements(lnamein)
      if lclname gt 0 then lname = lnamein else begin
        i = size(lname_save)
        if i(i(0)+1) eq 7 then begin
          lname = lname_save
          lclname = N_elements(lname)
        endif
      endelse
      if n_elements (truncate) eq 0 then truncate = truncate_save
;     determine if we are in postscript and allow a longer legend if we are
      if n_elements (nchar) gt 0 then begin
        len_legend = abs(nchar)
        if lclname gt 0 then for i = 0, lclname-1 do $
            lname (i) = strtrim (strmid(lname(i), 0,len_legend))
      endif  else len_legend = nchar_save
      if !d.name eq 'PS' then begin
        if len_legend le 0 then len_legend = 30 
        lcpostscr =1 
      endif else begin
        if len_legend le 0 then len_legend = 12
        lcpostscr = 0
      endelse
;
;     determine if wdfarray is good
;
      front = (n_elements(front) eq 0) ? front_save : front[0]
      if keyword_set (hak) then wait = 1
      if n_elements(data) eq 0 then data= data_save
      if keyword_set(left) then nogrid = 0
      if n_elements(initwdf) eq 0 then wdfinit
      nel =  n_elements(wdf)
      if n_elements(ncurve) eq 0 then ncurve= 1
      if nel eq 1 then begin
        if ncurve le 0 then begin
          print, 'Number of curves must be GT ZERO'
          return
        endif
        wdfp = make_array(ncurve, /int, /index)+wdf
      endif else wdfp = wdf
      ncurve = n_elements(wdfp)
      startn = ncurve
      ok = where(wdfp gt 0 and wdfp le maxwdfarray , j)
      if j ne ncurve then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        if j eq 0 then begin
          print, 'No valid data for the plot'
          return 
        endif
        wdfp = wdfp(ok)
        ncurve = j
      endif
      ok = where (wdfnp(wdfp-1) gt 1, j)
      if j ne ncurve then begin
        print, 'Some curves contain no valid data'
        if j eq 0 then begin
          print, 'No arrays with data for the plot'
          return 
        endif
        wdfp = wdfp(ok)
        ncurve = j
      endif
      absolute =  (n_elements(abs_value) eq 0) ? absolute_save : abs_value
      if n_elements(engineering) eq 0 then engineering = engineering_save
      if ncurve gt 1 and keyword_set(left) then over = 1
      if n_elements(over) eq 0 then over = over_save
      if n_elements(xr) eq 0 then xr = xrange_save
      if n_elements(yrin) eq 2 then begin
        lcyrange_set = 1 
        yr = yrin
;       leave logrange undefined since we have two values for yrange
;       assume user knows what he is doing!
      endif else begin
        if n_elements(lograngein) gt 0 then logrange = lograngein[0]
        lcyrange_set = 0
        if n_elements(yrin) eq 0 then yr = yrange_save else yr = yrin
      endelse
      if n_elements(header) eq 0 then header = header_save
      lcheader = 0
      if keyword_set(right) then begin
        nogrid = 1
        xr = 0
      endif

      if n_elements (spectrum) eq 0 then spectrum = spectrum_save
      if keyword_set(spectrum) then begin
        color = 10 +indgen(ncurve > 2)
        tvlct, /get , rsave, gsave, bsave
        if n_elements(linestyle) eq 0 then nlinestyle = 1
        if n_elements(symbol) eq 0 then nsymbol = 1
        if n_elements(leg) eq 0 then leg = -1
        if n_elements(lcharsize) eq 0 then lcharsize = 0.6
        rainbow, ncurve > 2
      endif
      if keyword_set(nogrid) then begin
        if (n_elements(color) eq 0) && (keyword_set(keep_colors)) then $
          color = keep_colors_save
        if (n_elements(color) gt 0) && (color[0] lt 0) then begin
          color=abs([color, color])
          ncol = n_elements(color)
          color = shift(color, over_offset)
        endif 
        if keyword_set(over) then begin
          if keyword_set(right) then begin
            if n_elements(leg) eq 1 && leg eq 1 then leg = [0.76, 0.94]
            if n_elements(leg) eq 2 then $
              skip = n_elements(skip) eq 0 ? 0: skip else $
              skip = n_elements(skip) eq 0 ? 1: skip
          endif else skip = 0
          if  n_elements(leg) eq 0 then begin
            if n_elements(over_legend) eq 0 then leg = legend_save else begin
              leg = over_legend
              data = 1
            endelse
          endif
          if keyword_set(data) and n_elements(leg) ge 2 then begin
            data = 0
            leg = float(leg)
            if !x.type eq 0 then $
              leg(0) = (leg(0)-!x.crange(0))/(!x.crange(1)-!x.crange(0)) else $
              leg(0) = (alog10(leg(0))-!x.crange(0))/(!x.crange(1)-!x.crange(0)) 
            if !y.type eq 0 then $
              leg(1) = (leg(1)-!y.crange(0))/(!y.crange(1)-!y.crange(0)) else $
              leg(1) = (alog10(leg(1))-!y.crange(0))/(!y.crange(1)-!y.crange(0))
          endif
        endif
      endif else if n_elements(color) gt 0 then color = abs(color)
      if n_elements(leg) eq 0 then leg = legend_save
;
; check for t3d
;
       if n_elements (t3d) eq 0 then t3d = 0
;************************
; get the range for overlay plots and set the grid for single plots
      if (over ne 0  and ncurve gt 1) or keyword_set(nogrid) then begin
        if keyword_set(nogrid) then begin
          if !x.type eq 0 then xr = !x.crange/xeng_scale $
            else xr = 10^(!x.crange)/xeng_scale
          lcset = 1
        endif else if n_elements (xr) lt 2 then begin
          xr = [min(wdfminx(wdfp-1)), max(wdfmaxx(wdfp-1))] 
          lcset = 0
        endif else begin
; set the y range using the new values for XRANGE rather than the total 
; waveform 
          xr = [min(xr, max = mm), mm]
          lcset = 1
        endelse
        if n_elements (yr) lt 2 and lcset eq 1 then begin
            yrmin = make_array(ncurve)
            yrmax = yrmin
            yrier = make_array(ncurve, /int)
            eps = max([1.0e-4*(xr[1]-xr[0]), 3.0e-5*xr])
            xrl = xr[0]-eps
            xrr = xr[1]+eps
            for i = 0, ncurve -1 do begin
              yrmin(i)= mn(wdfp(i),left=xrl,right=xrr,/quiet, $
               max=maxx, ier = ierval)
              yrmax(i) = maxx
              yrier(i) = ierval
            endfor
            ok = where (yrier eq 0, nok)
            if nok  eq 0 then begin
              print, 'No data in the X-range of data, yr = [0, 1] plotted'
              yr = [0.0, 1.0]
            endif else begin
              yr = make_array(2)
              yr(0) = min(yrmin(ok))
              yr(1) = max(yrmax(ok))
            endelse
        endif else if n_elements (yr) lt 2 then yr = $
          [min(wdfmin(wdfp-1)), max(wdfmax(wdfp-1))]
        lcgridswitch = 0
      endif else begin
        if n_elements(grid) eq 0 then grid=grid_save
        pmultisave = !p.multi
        if ncurve gt 1 and total (grid) gt 1 then begin
          lcgridswitch = 1
          if n_elements (grid) ge 2 then !p.multi=[0, grid(0), grid(1)] $
          else begin
            y=fix(sqrt(grid))
            x = fix(grid*1.0/y+.99999)
            !p.multi = [0, x , y]
          endelse
        endif else lcgridswitch = 0
      endelse
      if over ne 0 then begin
        if lclname ge ncurve then begin
          if lclname gt ncurve then legendst = lname(0:ncurve-1) $
            else legendst= lname
        endif else begin
          lclname = 0
          legendst= make_array(ncurve, /string)
        endelse
      endif
      if n_elements(color) eq 0 then color = color_save
      nelcolor = n_elements (color)
      if nelcolor gt 0 then begin
        loadxyct , /quiet, /freeze
        if nelcolor eq 1 then begin
          if color eq 0 then pcolor = !p.color else begin
            color = color > 1
            pcolor = color -1 + indgen(7)
            pcolor = 1+ pcolor mod 7
          endelse
        endif else pcolor = color
      endif
      ok = where(pcolor eq 0, nok)
      if nok gt 0 then pcolor[ok] = !p.color
      keep_colors_save =pcolor
      nelcolor = n_elements(pcolor)
      if keyword_set(nlinestyle) then linestyle = -1
      nellines = n_elements(linestyle)
      if nellines eq 0 then begin
        if keyword_set(over) then begin
          linestyle = linestyle_save
          nellines = n_elements(linestyle)
        endif else begin 
          linestyle = linestyle_save(0)
          nellines = 1
        endelse
      endif
      if keyword_set(over) then begin
        if nellines eq 1 and linestyle(0) ge 0 then begin
          linestyle = (linestyle +[0,2,3,4,5,1]) mod 6
          nellines = n_elements(linestyle) 
        endif
      endif 
      ok = where (linestyle gt 5, nok) 
      if nok gt 0 then linestyle(ok) = 0
      if linestyle(0) eq -1 then linestyle = 0*linestyle
      nelsymb_orig = n_elements(symbol)
      if nelsymb_orig eq 0 then begin
        if  not keyword_set(over) then symbol = 0 $
          else symbol = psymbol_save
      endif
      if keyword_set(allsym) then nsymbol = 0
      nelsymb = n_elements(symbol)
;     if max(symbol) lt 0 then allsym = -1
      if n_elements (nsymbol) eq 0 then nsymbol = (nelsymb_orig eq 0) ? nsymbol_save : 0
      if keyword_set(nsymbol) then allsym = 0
      if n_elements(lcmkcircle) eq 0 then lcmkcircle = 0
      if lcmkcircle eq 0 then mkcircle
      if n_elements (allsym) eq 0 then allsym = allsym_save 
      if max(abs(symbol)) eq 0 then begin
        if allsym ne 0 then symbol = 8 else nsymbol = 1
      endif
      if nsymbol ne 0 then begin
        allsym = 0
        symbol = 0*symbol
      endif
      if n_elements (symfill) eq 0 then symfill = symfill_save
      nsymfill = n_elements (symfill)

      if n_elements (symsize) eq 0 then symsize  = symsize_save

      nsymsize = n_elements (symsize)
;     if keyword_set(over) then symbol = abs(symbol)
      if allsym gt 0 then allsym = 1 else if allsym lt 0 then allsym = -1
      if n_elements (nogrid) eq 0 then nogrid = 0
      if n_elements(charsize) eq 0 then charsize = charsize_save
      if !p.charsize ne 0 then charsize = charsize *!p.charsize
      if not keyword_set(nogrid)  then over_offset = 0
      if n_elements(over_offset) eq 0 then over_offset = 0
      if over eq 0 then over_offset = 0

      if n_elements(xr) gt 1 then begin
        xrp = [min(xr, max = mm), mm]
        xr = xrp
        wdfpm = wdfp-1
        ok=where(wdfminx(wdfpm) le xrp(1) and wdfmaxx(wdfpm) ge xrp(0), ncurve)
        if ncurve eq 0 then begin
          Print, '% PLO: No data in the designated range of the abscissa'
          return
        endif
        wdfp = wdfp(ok)
      endif
      if startn gt ncurve then begin
        print, 'The following arrays will be plotted:'
        ll = strlen(strtrim(max(long(wdfp)), 2))
        print,format = '(20(i'+strtrim(ll+1, 2)+', :, ","))', long(wdfp)
      endif
      if keyword_set(front) && (!d.window ge 0) then my_wshow, !d.window, iconic =0
      for i = 0, ncurve-1 do begin
        ioffset = i+over_offset
        if nelcolor ne 0 then colorcurve=pcolor(ioffset mod nelcolor) $
            else colorcurve= !p.color
        ip = wdfp(i)
        np = wdfnp(ip-1)
        get_wdfarray, array, ip, /xy
        if size(array, /type) gt 5 then begin
          Print, '% PLO: Unable to handle this dataset type: ' 
          wdfarray= ip
          help, wdfarray, array
          return
        endif
        nel = n_elements(array)
        if keyword_set (quick) then begin
          if lcpostscr eq 0 then nq = long(max ([quick, 750])) else $
            nq = long(max ([quick, 1500]))
          if np lt 3*nq then nq = 0
        endif else if n_elements(quick) eq 0 then begin
          if np ge 20000 then begin
            if lcpostscr eq 0 then nq = 2250 else nq = 4500
          endif else nq = 0
        endif else nq = 0
        if (nq eq 0) then begin
          if nel eq np+2 then begin
            x = array(0)+findgen(np)*array(1)
            array =  array(2:*)
          endif else begin
            x = array(0:np-1)
            array = array(np:*)
          endelse
        endif else begin
; Limit the data to the range of interest. Allow 25% outside Xrange each end.
; we have already checked that there is some overlap.
          ep = 0.25
          if nel eq np+2 then begin
            if n_elements (xr) gt 1 then begin
              xrp = xr[1]-xr[0]
              xrp = xrp*ep*[-1, 1]+xr
              if array(0) lt xrp(0) then begin
                ist = long((xrp[0]-array[0])/array[1]) > 0
                if ist gt 0 then begin
                  array[0] = array[0] + ist*array[1]
                  array = [array[0:1],array[2+ist:np+1]]
                  np = np-ist
                  nel = np+2
                endif
              endif
              if (wdfmaxx(ip-1) gt xrp(1)) then begin
                np = (ceil((xrp[1]- array[0])/array[1])+1)  < np
                nel = np+2
              endif
            endif
            if np lt 3*nq then begin
              x = findgen(np)*array(1)+array(0)
              array = array(2:np+1)
            endif else begin
              ysav = temporary(array)
;             get the points per bin
              ppbin  = np/nq
              nq = np/ppbin
              nq2 = nq+nq
              x = make_array(nq2)
              array = make_array(nq2)
              f = lindgen(nq)
              fp = f
              f = f+f
              x(f) = ysav(0)+ ppbin*0.5*ysav(1) + fp*(ysav(1)*ppbin)
              x(f+1) =x(f)
              fp = fp*ppbin+2
              fpt = fp+(ppbin-1)
              for iave = 0, nq-1 do begin
                array(f(iave)) = min([ysav(fp(iave):fpt(iave))], max = mm)
                array(f(iave)+1) = mm
              endfor
              ysav=0
            endelse
          endif else begin
; Limit the data to the range of interest. Allow 25% outside Xrange each end.
; EP defined above
            x = array(0:np-1)
            if n_elements (xr) gt 1 then begin
              fudge = (xr(1)-xr(0))*ep
              xrp = xr+[-fudge, fudge]
              if wdfminx(ip-1) ge xrp(0) and wdfmaxx(ip-1) le xrp(1) then $
                npp = np else begin
                if wdfminx(ip-1) ge xrp(0) then $
                  ok = where(x le xrp(1),npp) else begin
                  if wdfmaxx(ip-1) le xrp(1) then $
                    ok = where(x ge xrp(0),npp) else $
                      ok = where(x ge xrp(0) and x le xrp(1),npp)
                endelse
              endelse
              if npp lt 3*nq then begin
                if npp gt 2 then begin
                  x =x(ok)
                  array = array(np+ok)
                endif else begin
                  array = array(np:*)
                endelse
                np = -1
              endif else begin
                if npp lt np then begin
                  x = x(ok)
                  array = array(np+ok)
                  np = npp
                endif else array = array(np:*)
              endelse
              xmax = min([wdfmaxx(ip-1), xrp(1)])
              xmin = max([wdfminx(ip-1), xrp(0)])
            endif else begin
              xmax = wdfmaxx(ip-1)
              xmin = wdfminx(ip-1)
              array = array(np:*)
            endelse
            if ((not keyword_set (quick))  and (np gt 0))then begin
              der = x(1:np-1) - x(0:np-2)
              if min(der) lt 0 then np = -1
            endif
            if np gt 0 then begin
              deltax = (xmax - xmin)/nq
              his= histogram(x, binsize = deltax, min=xmin, max=xmax, rev=rev)
              nq = n_elements(his)
              ok= where (his gt 0, nqq)
              nq2 = nqq+nqq
              rev1= rev(0:nq-1)
              rev2= rev(1:nq)-1
              if nqq lt nq then begin
                rev1 = rev1(ok)
                rev2 = rev2(ok)
                his = his(ok)
              endif
              xx = make_array(nqq)
              for iv = 0, nqq-1 do xx(iv)=total(x(rev(rev1(iv):rev2(iv))))/his(iv)
              x = make_array(nq2)
              f = lindgen(nqq)*2
              x(f) = xx 
              x(f+1) = xx
              xxx = xx
              for iv = 0, nqq-1 do begin
                xx(iv) = min(array(rev(rev1(iv):rev2(iv))), max = mm)
                xxx(iv) = mm 
              endfor
              array = make_array(nq2)
              array(f) = xx
              array(f+1) = xxx
            endif
          endelse
        endelse
        psym = symbol(ioffset mod nelsymb)
        if abs(psym) gt 10 then begin
          tmpsym  = abs(psym) - 11 < (nel_symbols-1)
          mksymbol, tmpsym, fill = symfill((i) mod nsymfill)
          if psym lt 0 then psym = -8 else psym = 8
        endif 
        if linestyle(0) ge 0 $
            then lin = linestyle(ioffset mod nellines) else lin = 0
        if (n_elements(logrange) gt 0) && (size(/type, extrastuff) eq 8) then begin
          names = tag_names(extrastuff)
          ok = get_match('^YTY', names, reg = 0, nel = nel)
          if nel eq 0 then ok = get_match('^YL', names, reg = 0, nel = nel)
          if nel eq 0 then lclogran = 0 else lclogran = 1
          if logrange lt 0 or logrange ge 1 then begin
            print, '% PLO:  Bad value of LOGRANGE.  Setting value to 1.0E-4'
            logrange = 1.0e-4
          endif else if lclogran gt 0 then logrange = logrange[0]
        endif else lclogran = 0
        if nogrid eq 0 then begin
          lcrange = 0
          if n_elements (xr) lt 2 then $
            xrp= [wdfminx(ip-1), wdfmaxx(ip-1)] else begin
            xrp = [min(xr, max = mm), mm]
            eps = max([1.0e-5*(xrp[1]-xrp[0]), 3.0e-5*xrp])
            xrl = xr[0]-eps
            xrr = xr[1]+eps
            if n_elements (yr) lt 2 then begin
              yrp=[mn(wdfp(i),max=maxx,/quiet,left=xrl,right=xrr),maxx]
              lcrange  = 1
            endif
          endelse
          if((n_elements (yr) lt 2) and (lcrange eq 0)) then $
            yrp= [wdfmin(ip-1), wdfmax(ip-1)] $ 
          else if n_elements(yr) ge 2 then yrp=yr(0:1)
          if keyword_set(absolute) then begin
            if lcyrange_set eq 0 then begin
;             set the range to account for the absolute keyword
              junk = max(abs(yrp), ivalue)
              ivalue = (ivalue +1) mod 2
              if yrp[ivalue] ne 0 then $
                if yrp[1]/yrp[0] lt 0 then yrp[ivalue]=0.0
            endif
            if absolute eq -1 then begin
              array = abs(array)
              yrp = abs(yrp)
            endif   
            yrp = [min(yrp, max = mmx), mmx]
          endif
          if yrp(0) eq yrp(1) then yrp= (yrp[0] eq 0.0) ? [0.0, 1.0] : [0.999, 1] *yrp(1)
          if lclogran gt 0 then $
            if yrp[1] le 0 then yrp =[.1,1] else $
              yrp[0] = max([yrp[1]*logrange, yrp[0]])
          if psym lt 0 then psymtemp = -psym else psymtemp = 0
          if allsym ne 0 then psymtemp = -allsym* psym
          if (abs(psym) eq 10) then psymtemp = 10
          if ((over eq 0) or (i eq 0)) then begin
            if !p.multi(0) eq 0 then begin
              if header ne 0 then begin
                lcheader = 1
                thindate = thin_date()
                filename = strtrim(wdffile (ip-1), 2)
                if strlen(filename) eq 0 then header_text='Plot: '+thindate $
                else begin
                  if header eq -1 then $
                    header_text = ['Plot: '+thindate, 'File: ' +filename] $
                  else begin
                    iii = strpos(wdfcomment(ip-1), 'DATETIME=', 0)
;                   header equals -2, -3 or is positive 
;;                  field test 
;;                  shot date is after 'DATETIME='
                    if iii gt 0 then begin
                      ii = strpos(wdfcomment(ip-1), ';', iii+2)-iii-9
                      if ii gt 0 then cdate =strmid(wdfcomment(ip-1),iii+9,ii) $
                        else cdate = strmid(wdfcomment(ip-1),iii+9,20)
                      cdate = strtrim(cdate, 2)
                      ii = strlen(cdate)
                      ok = where (byte(cdate) eq 58B, nok)
                      if nok eq 2 then begin
                        ii = ii -3
                        if ii eq ok(1) then cdate = strmid(cdate, 0, ii) $
                          else cdate = strmid(cdate, 0, ok(1)) +  $
                            strmid(cdate, ok(1)+3, ii-ok(1))
                      endif
                    endif else ii = 0
                    if ii eq 0 then begin
;;                    end field test stuff
                      ss = file_info(filename)
                      cdate=(ss.exists eq 1) ? thin_date(ss.ctime) : thindate
                    endif
                    if header eq -2 then $
                      header_text = ['Plot: '+cdate, 'File: ' +filename] $ 
                    else header_text= ['Plot: '+thindate +'; Data: ' + cdate,  $
                      'File: ' +filename] 
                  endelse
                endelse
              endif
            endif
            if n_elements(title) gt 0 then titleout = title else begin
              if keyword_set (over) then titleout = '' else begin
                titleout=wdfcomment(ip-1)
                if keyword_set(truncate) then begin
                  st = strpos(titleout, ' - ')
                  if st gt 0 then titleout = strmid(titleout, 0, st)
                endif
              endelse
            endelse
            if keyword_set(engineering) then begin
              get_engr, max(abs(xrp)), xeng_scale, xeng_label
              if xeng_scale ne 1.00 then x = x*xeng_scale
              xrp = xrp*xeng_scale
              get_engr, max(abs(yrp)), yeng_scale, yeng_label
              if yeng_scale ne 1.00 then array = array*yeng_scale
              yrp = yrp*yeng_scale
              xtit= wdfxlabel(ip-1)+xeng_label
              ytit= wdfylabel(ip-1)+yeng_label
            endif else begin
              xtit=wdfxlabel(ip-1)
              ytit= wdfylabel(ip-1)
              xeng_scale = 1.00
              yeng_scale = 1.00
            endelse
            ntitle =  n_elements(titleout)
            if ntitle gt 1 or n_elements(start_offset) gt 0 then begin
              if n_elements(start_offset) eq 0 then start_offset = 0.5 
              toffset = start_offset-0.3
              mult_title = temporary(titleout)
              titleout = ''
              if n_elements (ymargin) lt 2 then $
                ymargin = [!y.margin[0], ceil(toffset+1.8*ntitle)] else $
                ymargin = [ymargin[0], ceil(toffset+1.8*ntitle)]
              ntitle = 2
            endif
            if keyword_set(left) then begin
                if n_elements(ystyle) eq 0 then ystyle = 8 else $
                  ystyle = ystyle or 8
                if keyword_set(aspect) then begin
                  if n_elements (xstyle) eq 0  then xstyle = 1 else $
                    xstyle = xstyle or 1
                  ystyle = ystyle or 1
                  pos = get_aspect(xrp, yrp, ymargin = ymargin, $
                    xmarg=[!x.margin(0), !x.margin(0)])
                  plot,x,array, xtitle=xtit, ytitle= ytit, position = pos,  $
                    xstyle =xstyle, nodata= nelcolor, $
                    thick = thick, title= titleout, xr = xrp, yr =yrp, $
                    lin = lin, psym= psymtemp, charsize=charsize, $
                    ystyle=ystyle , symsize= symsize(ioffset mod nsymsize), $
                    t3d = t3d, _extra = extrastuff
                endif else $
                  plot,x,array, xtitle=xtit, ytitle= ytit, $
                  thick = thick, title= titleout, xr = xrp, yr =yrp, $
                  lin = lin, psym= psymtemp, charsize=charsize,  $
                  nodata= nelcolor, xmargin=[!x.margin(0), !x.margin(0)], $
                  xstyle = xstyle, ystyle=ystyle, ymargin = ymargin, $
                  symsize= symsize(ioffset mod nsymsize), t3d = t3d, $
                  _extra = extrastuff
            endif else begin
                if keyword_set(aspect) then begin 
                  if n_elements (xstyle) eq 0  then xstyle = 1 else $
                    xstyle = xstyle or 1
                  if n_elements (ystyle) eq 0  then ystyle = 1 else $
                    ystyle = ystyle or 1

                    pos = get_aspect(xrp, yrp, ymargin = ymargin)
                  plot,x,array, xtitle=xtit, ytitle= ytit, position = pos, $
                    xstyle = xstyle, ystyle = ystyle, t3d = t3d, $
                    thick = thick, title= titleout, xr = xrp, yr =yrp, $
                    lin = lin, psym=psymtemp,  charsize = charsize, $
                    symsize= symsize(ioffset mod nsymsize), nodata = nelcolor, $
                     _extra = extrastuff
                endif else $
                  plot,x,array, xtitle=xtit, ytitle= ytit, ystyle = ystyle, $
                  thick = thick, title= titleout, xr = xrp, yr =yrp, $
                  lin = lin, ymargin = ymargin, $
                  psym=psymtemp,  charsize = charsize, xstyle = xstyle,  $
                  symsize= symsize(ioffset mod nsymsize), nodata = nelcolor, $
                  t3d = t3d, _extra = extrastuff
            endelse
            if ntitle gt 1 then header, /title, mult_title, start=start_offset
            if nelcolor gt 0 then oplot, x, array, color = colorcurve, $
              thick = thick, psym=psymtemp, t3d = t3d, $
              symsize= symsize(ioffset mod nsymsize) , lin = lin
            if lcheader ne 0 then begin
              lcheader = 0
              header, header_text
            endif
            if ((psym gt 0) and (nsymbol eq 0) and (allsym eq 0) and (psymtemp ne 10))then begin 
              if (keyword_set(absolute) && absolute eq -1) then begin
                ipp = get_maxwdf()
                i2w, x, array, ipp, /xydata
              endif else ipp = ip
              get_symbol_points,ipp, xsy, ysy, valid = valid
              if valid gt 0 then begin
                xsy = xsy*xeng_scale
                ysy = ysy*yeng_scale
                oplot, xsy, ysy, psym = psym, color = colorcurve, $
                  symsize= symsize(ioffset mod nsymsize) , thick = thick, $
                  t3d = t3d
              endif
            endif
            if over eq 0 and ((i ne ncurve-1 and !p.multi(0) eq 0 and $
              ncurve gt 1) or keyword_set(wait)) then hak, /mes
          endif else begin
            if psym lt 0 then psymtemp = -psym else psymtemp = 0
            if allsym ne 0 then psymtemp = -allsym* psym
            if xeng_scale ne 1.00 then x = x*xeng_scale
            if yeng_scale ne 1.00 then array = array*yeng_scale
;           if allsym eq 0 then psymtemp = 0 else psymtemp = - allsym*psym
            if (abs(psym) eq 10) then psymtemp = 10
            oplot, x,array, psym = psymtemp,lin=lin, color = colorcurve, $
              symsize= symsize(ioffset mod nsymsize) , thick = thick, t3d = t3d
            if (psym gt 0 and nsymbol eq  0 and allsym eq 0 and $
              psymtemp ne 10) then begin
              if (keyword_set(absolute) && absolute eq -1) then begin
                ipp = get_maxwdf()
                i2w, x, array, ipp, /xydata
              endif else ipp = ip
              get_symbol_points,ipp, xsy, ysy, valid = valid
              if valid gt 0 then begin
                xsy = xsy*xeng_scale
                ysy = ysy*yeng_scale
                oplot, xsy, ysy, psym = psym, color = colorcurve, $
                  symsize= symsize(ioffset mod nsymsize) , thick = thick, $
                  t3d = t3d
              endif
            endif
          endelse
          if (over ne 0 and lclname eq 0) then begin
            if keyword_set(truncate) then begin
              st = strpos( wdfcomment(ip-1), ' - ')
              if st gt 0 then $
                legendst(i) = strmid(wdfcomment(ip-1), 0, st< len_legend ) else $
                legendst(i) = strtrim (strmid(wdfcomment(ip-1),0,len_legend),2)
            endif else $
              legendst(i) = strtrim (strmid(wdfcomment(ip-1),0,len_legend),2)
          endif
        endif else begin
          if i eq 0 and keyword_set(right) then begin
            xrp = xr(0:1)*xeng_scale
            yrp = yr(0:1)
            if yrp(0) eq yrp(1) then yrp= (yrp[0] eq 0.0) ? [0.0, 1.0] : [0.999, 1] *yrp(1)
            if lclogran gt 0 then $
              if yrp[1] le 0 then yrp =[.1,1] else $
                yrp[0] = max([yrp[1]*logrange, yrp[0]])
            if lcyrange_set eq 0 and keyword_set(absolute) then begin
;             if n_elements(yrp) lt 2 then print, 'Contact Paul Mix, Error'
              junk = max(abs(yrp), ivalue)
              ivalue = (ivalue +1) mod 2
              if yrp[ivalue] ne 0 then $
                if yrp[1]/yrp[0] lt 0 then yrp[ivalue]=0.0
              if absolute eq -1 then begin
                array = abs(array)
                yrp = abs(yrp)
              endif
              yrp = [min(yrp, max = mmx), mmx]
            endif
            if yrp(0) eq yrp(1) then yrp(0)= 0.999*yrp(1)
            if keyword_set(zeroline) then begin
              if yrp(1) lt yrp(0) then yrp= [yrp(1), yrp(0)]
              if !y.type eq 0 and $
                ((!y.crange(0) le 0.0 and !y.crange(1) gt 0.0) or $
                (!y.crange(0) lt 0.0 and !y.crange(1) ge 0.0)) then begin
                if !y.crange (0) eq 0 then begin
                  if yrp(1) le 0 then begin
                    if lcyrange_set gt 0 then val = yrp[0] else begin
                      rtmp = get_nice_values([yrp[0], 0], /noswap)
                      val = rtmp[0]
                    endelse
                  endif else begin
                    if lcyrange_set gt 0 then val = yrp[1] else begin
                      rtmp = get_nice_values([0, yrp[1]], /noswap)
                      val = rtmp[1]
                    endelse
                  endelse 
                  yrp = [0.0, val] 
                endif else if !y.crange (1) eq 0 then begin
                  if yrp(0) ge 0 then begin
                    if lcyrange_set gt 0 then val = yrp[1] else begin
                      rtmp = get_nice_values([0, yrp[1]], /noswap)
                      val = rtmp[1]
                    endelse
                  endif else begin
                    if lcyrange_set gt 0 then val = yrp[0] else begin
                      rtmp = get_nice_values([yrp[0], 0], /noswap)
                      val = rtmp[0]
                    endelse
                  endelse 
                  yrp = [ val, 0.0]
                endif else begin
                  if abs(!y.crange[0]) gt abs(!y.crange[1]) then begin
                    if lcyrange_set gt 0 then begin
                      if abs(yrp[0]) gt abs(yrp[1]) then $ 
                         yrp =[yrp[0], !y.crange(1)/!y.crange(0)*yrp[0]] $
                       else $
                         yrp =[yrp[1], !y.crange(1)/!y.crange(0)*yrp[1]]
                    endif else begin
                      if abs(yrp[0]) gt abs(yrp[1]) then begin
                         rtmp = get_nice_values([yrp[0], 0], /noswap)
                         yrp =[rtmp[0], !y.crange(1)/!y.crange(0)*rtmp[0]] 
                      endif  else begin
                         rtmp = get_nice_values([yrp[1], 0], /noswap)
                         yrp =[ rtmp[0], !y.crange(1)/!y.crange(0)*rtmp[0]]
                      endelse
                    endelse
                  endif else begin
                    if lcyrange_set gt 0 then begin
                      if abs(yrp[0]) gt abs(yrp[1]) then $
                         yrp =[!y.crange(0)/!y.crange(1)*yrp[0], yrp[0]] $
                       else $
                         yrp =[!y.crange(0)/!y.crange(1)*yrp[1], yrp[1]]
                    endif else begin
                      if abs(yrp[0]) gt abs(yrp[1]) then begin
                         rtmp = get_nice_values([0, yrp[0]], /noswap)
                         yrp =[!y.crange(0)/!y.crange(1)*rtmp[1], rtmp[1]]
                      endif  else begin
                         rtmp = get_nice_values([0, yrp[1]], /noswap)
                         yrp =[!y.crange(0)/!y.crange(1)*rtmp[1], rtmp[1]]
                      endelse
                    endelse

                  endelse
                endelse
              endif else zeroline = 0
            endif
              
            if keyword_set(engineering) then begin
              get_engr, max(abs(yrp)), yeng_scale, yeng_label
              yrp = yrp*yeng_scale
              ytit= wdfylabel(ip-1)+yeng_label
            endif else begin
              ytit= wdfylabel(ip-1)
              yeng_scale = 1.00
            endelse
            if keyword_set (zeroline) then $
	    axis, /yaxis, /save, xrp(1), 0, yrange = yrp, ystyle =1, $
              ytitle= ytit,charsize = charsize, ylog =0, _extra= extrastuff $
	    else axis, /yaxis, /save, xrp(1), 0, yrange = yrp, ystyle = ystyle, $
              ytitle= ytit,charsize = charsize,ylog =0,  _extra= extrastuff
          endif
          if xeng_scale ne 1.00 then x = x*xeng_scale
          if yeng_scale ne 1.00 then array = array*yeng_scale
          if psym lt 0 then psymtemp = -psym else psymtemp = 0
          if allsym ne 0 then psymtemp = -allsym* psym
;         if psym lt 0 then psymtemp = -psym else psymtemp = 0
;         if allsym eq 0 then psymtemp = 0 else psymtemp = -allsym*psym
          if abs(psym) eq 10 then psymtemp = 10
          oplot, x,array, lin= lin, color = colorcurve, t3d = t3d, $
            psym = psymtemp, thick = thick,symsize=symsize(ioffset mod nsymsize)
          if (psym gt 0 and psymtemp ne 10 and nsymbol eq  0 and allsym eq 0) then begin
            if (keyword_set(absolute) && absolute eq -1) then begin
              ipp = get_maxwdf()
              i2w, x, array, ipp, /xydata
            endif else ipp = ip
            get_symbol_points,ipp, xsy, ysy, valid = valid
            if valid gt 0 then begin
              xsy = xsy*xeng_scale
              ysy = ysy*yeng_scale
              oplot, xsy, ysy,psym = psym, color = colorcurve, thick = thick, $
                symsize= symsize(ioffset mod nsymsize) , t3d = t3d
            endif
          endif 
          if (over ne 0 and lclname eq 0) then begin
            if keyword_set(truncate) then begin
              st = strpos( wdfcomment(ip-1), ' - ')
              if st gt 0 then $
                legendst(i) = strmid(wdfcomment(ip-1), 0, st< len_legend ) else $
                legendst(i) = strtrim (strmid(wdfcomment(ip-1),0,len_legend),2)
            endif else $
              legendst(i) = strtrim (strmid(wdfcomment(ip-1),0,len_legend),2)
          endif
        endelse
      endfor
      old_offset = over_offset
      over_offset = ncurve+ over_offset
    array= 0
    if lcgridswitch ne 0 then !p.multi = pmultisave
    if keyword_set(zeroline) then begin
      if !x.type eq 0 then xrp = !x.crange $
            else xrp = 10^(!x.crange)
      if n_elements(zeroline) eq 1 then begin
        if zeroline gt 0 then oplot, xrp , [0.0, 0.0] else $
        if zeroline lt 0 and zeroline ge -255 then  $
         oplot, xrp , [0.0, 0.0], color = -zeroline
      endif else begin
        zeroline = abs(zeroline(0:1))
        oplot, xrp, [0.0, 0.0], linestyle = zeroline(1), color = zeroline(0)
      endelse
    endif
;   reset colors if necessary
    if keyword_set(spectrum) then tvlct, rsave, gsave, bsave
    if over ne 0 then begin
      if n_elements (leg) eq 0 then begin
        legp = legend_save
        data = 0
      endif else begin
        if n_elements(leg) lt 2  then begin
          if leg eq -1 then return else begin
            data = 0
            legp=legend_save
          endelse
        endif else legp = leg(0:1)
      endelse
      if n_elements (nextpos) lt 2 then nextpos = 0
      if data eq 0 then rel = 1 else rel = 0
      if allsym lt 0 then linestyle = -1
      ok = where( abs(symbol) eq 10, nok)
      if nok gt 0 then symbol[ok] =0
      if max(symbol) lt 0 then linestyle = -1
      if n_elements(lsymsize) gt 0 then begin
        if lsymsize(0) gt 0.0 then symsize= lsymsize(0)* symsize
      endif
      if n_elements(lcharsize) gt 0 then legch = charsize*lcharsize $
         else legch = charsize*lcharsize_save
      
      if n_elements (blegend) eq 0 then blegend= blegend_save
      if keyword_set(blegend) then begin
        ss = size (blegend, /type)
        if ss ne 8 then blegend = {BOX:1}
      endif else undefine, blegend
      if n_elements (legendst) gt ncurve then legendst= legendst[0:ncurve-1]
      legendp, legendst, position=legp, relative=rel, data=data, $
        psym=symbol, color = pcolor, linestyle = linestyle, thick = thickin,  $
        symsize = symsize, fill= symfill, charsize = legch, lcskip = skip, left = left, $
        right = right, offset = old_offset, nextpos = nextpos, _extra= blegend
      if keyword_set(wait) then hak, /message
      over_legend = nextpos
    endif
;
;   return to caller
    end
