; ------------------------------------------------------------------------ 
; $Id: priar.pro,v 1.7 2010/09/17 21:20:35 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  priar, wdf, npoint = npoint, statistics = statistics, set_width = set_width
;+
;  Prints the parameters of a WDF array.  Maximum and minimum values, the 
;  number of points, the file from which it was read, and the character strings 
;  associated with each of the labels is printed with ordinate values.
;
;  Format: PRIAR, wd1 [, NPOINT = npoint] [, /STATISTICS] [, SET_WIDTH = set_width]
;  where:
;    wd1 - WDF array number
;    npoint - sspecifies the number of ordinate values to be printed.
;             If npoint is negative, all values will be printed.
;             If npoint is omitted, up to 72 values will be printed.
;    statistics - If set, print statistical data on the y-array
;             (SEE - STATS)
;    set_width - Maximum length of string variables printed to the screen
;                If SET_WIDTH LE 0 then the entire variable is written.
;                DEFAULT: SET_WIDTH = -1
;
;  Examples:
;    Print the parameters and up to 72 values of array 3
;      priar, 3
;    Print the parameters and the entire set of ordinates for array 4
;      priar, 4, np = -1
;
;    Set the default maximum character length to 50
;      PRIAR, set_width = 50
;-
;     print array values
;
;     include the common block specification
;
@com.dat
;
 common PRIAR_COMMON_BLOCK, prwidth
      on_error, 1
      if n_elements(prwidth) eq 0 then prwidth = -1
      if n_elements(set_width) gt 0 then begin
        prwidth = (set_width le 0) ? -1 : long(set_width[0])
        if n_elements(wdf) eq 0 then return
      endif
;
      if n_elements(wdf) eq 0 then begin
        print, 'WDF array must be specified'
        return
      end
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if wdfnp(wdf-1) eq 0 then begin
        print, 'No data in WDF array', wdf
        return
      endif
      if n_elements(npoint) eq 0 then npoint = 72
      if npoint lt 0 then npoint = wdfnp(wdf-1)
      print, 'Values for wdf array number', wdf
      if prwidth lt 0 then begin
        print, ' Comment: ',wdfcomment(wdf-1)
        print, '    File: ',wdffile(wdf-1)
        print, ' X-label: ',wdfxlabel(wdf-1)
        print, ' Y-label: ',wdfylabel(wdf-1)
      endif else begin
        print, ' Comment: ',strmid(wdfcomment(wdf-1), 0, prwidth)
        print, '    File: ',strmid(wdffile(wdf-1), 0, prwidth)
        print, ' X-label: ',strmid(wdfxlabel(wdf-1), 0, prwidth)
        print, ' Y-label: ',strmid(wdfylabel(wdf-1), 0, prwidth)
      endelse
      get_wdfarray, array, wdf, /xy
      nel = wdfnp(wdf-1)
      if n_elements (array) eq nel +2 then begin
        print, ' Start Time: ', array(0), ', Delta Time ',array(1)
        y = array(2:*)
      endif else begin
        print, ' Start Time: ', array(0), ', Non Uniform X-Axis'
        y = array(nel:*)
      endelse
      print, ' Number of Points:', nel
      print, ' Range of X-values: ', wdfminx(wdf-1), wdfmaxx(wdf-1)
      print, ' Range of Y-values: ', wdfmin(wdf-1), wdfmax(wdf-1)
      ok = where (~finite(y), nok)
      if nok gt 0 then $
        print, ' Y-Valuse have '+strtrim(nok, 2)+' NaN or Inf Values'
; 
      if keyword_set(statistics) then stats, getyf(wdf)
      np = min([nel, npoint])
      if np gt 0 then begin
        print, ' First', np, ' Y-Values are:'
        print, y(0: np-1)
      endif
      y =0 & array =0
      print, ' -----------------'
      end
