; ------------------------------------------------------------------------ 
; $Id: readimage.pro,v 1.4 2010/09/03 18:29:42 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro readimage, str, file, np = np, clabel = clab, ncol = ncol
;+
; Read an ascii image file
; 
; Format: readimage, str [, file] [, NP = np] [, NCOL = ncol]
;  where:
;        STR  - Structure array number
;        File - file name
;        NP   - Number of points read or to be read.
;               If more than 4096 points, set np to the value.
;        NCOL - Number of columns [Default = 4]
;                               
;   Example:
;        Read an image into structure 3. Pick the file from a widget.
;        READIMAGE, 3
;-
on_error, 1
@qsstrcom.dat
;
   if n_elements (structinit) eq 0 then qsstruct_init

   if n_params() lt 1 then message, ' Structure array number must be provided'
   if n_elements(file) eq 0 then file = 0
   str = get_save_str(str, 80)
   if n_elements(ncol) eq 0 then ncol=4 else ncol=ncol > 3
   readasc, file, ncol, x, yarr, np= np
   y = yarr(*, 0)
   image = yarr(*, 1)

   a = sort(y)
   x = x[a]
   y = y[a]
   image = image[a]
   dy = y[1:np-1]-y[0:np-2]
   ymax = max(y)
   ydel = ymax/1000.
   ok = where (dy  gt  ydel, nok)
   if nok lt 1 then message, 'No difference in Y values.'
   nx = ok[0]+1
   ny = nok+1
 print, nx, ny, np, nx*ny
   if np ne (nok+1)*(ok(0)+1) then begin
      Print, np, nx, ny
      message, 'Number of X and Y values not consistent with image'
   endif
   a = sort(x[0:ok[0]])
   yy = make_array(ny)
   xx = make_array(nx)
   xx = x[a]
   yy[0] = y[0]
   ok = [ok, np-1]
   image[0:ok[0]]= image[a]
   for i = 1, nok do begin
     a = sort(x(ok(i-1)+1:ok(i)))+ok(i-1)+1
     yy(i) = y(ok(i))
     image[ok[i-1]+1:ok[i]] = image[a]
   endfor
   image = reform (image, nx, ny, /overwrite)
   if n_elements(clab) eq 0 then clab = file
   arr2str, str,image, xx, yy, file = file, clabel = clab
   return
   end

   
