; ------------------------------------------------------------------------ 
; $Id: readrga.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readrga, fname, WDF, np = np, narray=narray, datetime = datetime 
;
;+
;    Read an ascii file into wdf arrays from a residual gas analyzer
;    format is:  readrga, fname, WDF, NARRAY = narray, NP = np, 
;                 DATETIME= datetime
;    where 
;      fname   - the file name or a unit number
;                enter 0 to use pickfile
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of 
;                narray values
;      NARRAY  - the number of arrays in the file 
;      NP      - Returned with the number of points read in each array
;      DATETIME- If set, the DATE and TIME are stored at the end of the 
;                dataset comment but are not plotted if truncate is set on PLO
;                
;      If file contains labels, they are stored in the dataset comment
;    Example:
;    Read waveform data from a file selected by pickfile and store it in
;    WDF array 10
;       READRGA, 0, 10
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READRGA, FNAME_UNIT, WDF [, NP = np]'
        print, '              [, NARRAY = narray] [, DATETIME = datetime]'
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, ' File name must be specified by name or unit number'
        return
      endif
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = ''
      if n_elements(wdf) eq 0 then begin
        print,'WDF array value must be provided'
        if lcfile eq 1 then  free_lun, unit
        return
      end
      wdfar = wdf(*,0,0,0)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then  free_lun, unit
        return
      endif
      str = ''
      if keyword_set (datetime) then begin
        while strlen(strtrim(str, 2)) eq 0 do readf, unit, str
        clab = strtrim(str, 2)
        clab = ' - '+';;DATETIME='+strtrim(clab, 2) + ' ;;'
      endif else clab = ''
      delim = (byte(','))(0)
      readf, unit, str
      while (strpos (str, 'Active channels in Scan,')) lt 0 do readf, unit, str
      parser, str, narray, np, /stringarray, delimiter = delim
      narray = fix(narray(1))
      readf, unit, str
      while (strpos (str, 'Units,')) lt 0 do readf, unit, str
      parser, str, ylabel, np, /stringarray, delimiter = delim
      ylabel = ylabel(1)
      readf, unit, str
      while (strpos (str, 'Sample Period,')) lt 0 do readf, unit, str
      parser, str, xscale, np, /stringarray, delimiter = delim
      xlabel = 'Time ('+xscale(2)+')'
      xscale = float(xscale(1))
      readf, unit, str
      while (strpos(str, 'Channel,') lt 0) do readf, unit, str
      clabel = make_array(narray, /string)
      calfactor = make_array(narray)
      readf, unit, str
      i = 0
      while (i lt narray) do begin
        while strlen(strtrim(str, 2)) eq 0 do readf, unit, str
        parser, str, newstr, nn, /stringarray, delimiter =9B
        ll = strlen(newstr(4))
        clabel(i)='Mass: '+strtrim(newstr(2),2)+' ('+  $
           strtrim(strmid(Newstr(4), 0, ll-1), 2)+')' +clab
        calfactor(i) = float (newstr(5))
        i = i+1
        readf, unit, str
      endwhile
       
      nel = n_elements(wdfar)
      if nel lt narray then begin
        wdfar = [wdfar, wdfar(nel-1)+1+indgen(narray-nel)]
        if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          print, wdfar
        if lcfile eq 1 then  free_lun, unit
          return
        endif 
      endif
      nn = 2048L
      i  = 0L
;     assume starts at 0 with a delta of 1
      time = make_array(nn)
      a = make_array(narray, nn)
      b = make_array(narray)
      readf, unit, str
      str2 = ''
      while (not eof(unit)) do begin
          while strlen(strtrim(str, 2)) eq 0 do readf, unit, str
          ok = where(byte(str) eq delim, nok) 
          if nok lt narray then begin
            readf, unit, str2
            str = str+str2
          endif
          reads,str,t, b
          if i eq nn then begin
            aa  =  make_array(narray, nn+nn)
            aa(*, 0:nn-1) = a
            time=[time, time]
            nn = nn + nn
            a = temporary(aa)
          endif
          time(i) = t
          a(*,i) = b
          i = i+1
          readf, unit, str
      endwhile
      if i eq 0 then begin
        print,  '% READRGA: NO data read.'
        return
      endif
      a = a(*, 0:i-1)
      dt = xscale
      t0 = time(0)*xscale

      if lcfile eq 1 then for i = 0, narray-1 do $
        put_wdfarray,[t0, dt, reform(a(i,*))] , $
           wdfar(i), clab = clabel(i), $
           file = fname , xlab=xlabel, ylab= ylabel $
      else for i = 0, narray-1 do $
        put_wdfarray,[t0, dt, reform(a(i,*))], $ 
          wdfar, clab = clabel+clab, xlab = xlabel, ylab = ylabel
      if lcfile eq 1 then  free_lun, unit
      return
    end
