; ------------------------------------------------------------------------ 
; $Id: remach.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro remach, fname, WDF, np = np, skip = skip, $
           xydata = xydata, endnumber = endnumber
;
;+
;    read a mach2 history file into wdf arrays
;    format is:  remach, fname, WDF, skip = skip, $
;                  xydata = xydata, endnumber = endnumber
;    where 
;      fname   - the file name or a unit numberr
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of 
;                narray values
;      np      - Number of points which are read
;      skip    - If the file has several lines of header information which
;                must be skipped, set skip equal to this number of lines.
;                (DEFAULT: SKIP = 0)
;      xydata  - If set to zero, the data will be left on a nonuniform grid.
;                Default: XYDATA = 1
;      endnumber - If defined, reading will stop when this number is reached.
;      If file contains labels, they are stored in the dataset comment
;    Example:
;    Read waveform data from a file selected by pickfile and store it in
;    WDF array 10.  Assume format = 2: label followed by data to EOF.
;       REMACH, 0, 10 
;    Read 3 arrays from file, 'history', into WDF arrays 3, 6, 9. 
;       REMACH, 'history', [3,6,9]
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: REMACH, FNAME_UNIT, WDF [, NP = np]'
        print, '     [, /xydata] [, SKIP = skip] [, ENDNUMBER = endnumber]'
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, '% REMACH: File name must be specified by name or unit number'
        return
      endif
      np = 0L
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = ''
      if n_elements(wdf) eq 0 then begin
        print,'WDF array value must be provided'
        return
      end
      wdfar = wdf(*,0,0,0)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      
      if n_elements (skip) eq 0 then skip = 0
      junkstr = ''
      if skip gt 0 then for i = 1, skip do readf, unit, junkstr
      RunName = ''
      readf, unit, runname
      runname = strtrim(runname,2)
      readf, unit, junkstr
      result = strsplit(junkstr, /extract) 
      narray = long(result[1])

      nel = n_elements(wdfar)
      if n_elements (xydata) eq 0 then xydata = 1
;
      if nel eq 1 and narray gt 1 then wdfar=indgen(narray)+wdfar(0) else begin
        if nel gt narray then print, $
            ' Warning: only', narray,' arrays will be filled'
        if nel lt narray then begin
          print, ' Only ', strtrim(string(nel), 2), $
          ' arrays have been provided for ', strtrim(string(narray), 2),' arrays'
          print, ' Please try again'
          if lcfile eq 1 then free_lun, unit
          return
        endif
      endelse
      if  max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be  </=', maxwdfarray
        return
      endif
      maxnp = 0L
      on_ioerror, badnp
      if maxnp le 0 then begin
        np = 4096L
        lcnp = 1
        maxnp = np 
      endif else lcnp = 0
; Time to start reading
      name = make_array (narray, /string)
      str = ''
      for i = 0, narray-1 DO begin
        readf, unit, str
        name[i] = str + ' - ' + runname
      endfor
      ax =make_array(maxnp)
      bx = make_array (narray-1, maxnp)
      y = make_array (narray-1)
      i=0L

      on_ioerror, badnum

      while (not eof(unit))  do begin
        readf,unit,x, y
        if keyword_set(lcend) then if (x eq endnumber) then goto, badnum
        if i eq maxnp then begin
          if lcnp eq 0 then goto, badnum
          bx = reform (bx, maxnp*narray,/overwrite)
          maxnp = maxnp+maxnp
          bx =[bx,bx]
          bx = reform (bx, narray, maxnp, /overwrite)
          ax =[ax, ax]
        endif
        ax(i) = x
        bx(*, i) = y
        i = i+1L
      endwhile
badnum:
      if lcfile eq 1 then free_lun, unit
      if i eq 0 then begin
        np = 0
        print, ' Error reading file -- No data found'
        return
      endif
      np = i
      npm= np-1
      ax= ax(0:npm)
      bx = temporary(bx(*, 0:npm))
      bx = transpose (bx)
      xlab = 'Time(s)' 

; convert to wdf array using the lowres option
      if lcfile eq 1 then for i = 0, narray-2 do $
         i2w, ax, bx(0:npm, i),wdfar(i), c = name(i+1), /lowres, $
           xydata = xydata, file = fname, xlab = xlab $
      else for i = 0, narray-1 do $
         i2w, ax, bx(0:npm, i) ,wdfar(i), c = name(i+1), /lowres, $
           xydata = xydata, xlab = xlaB

      return
badnp:
      if lcfile eq 1 then free_lun, unit
       print, ' Error reading number of points -- Check format of data '
      return

      end
