; ------------------------------------------------------------------------ 
; $Id: reufo.pro,v 1.2 2010/09/17 21:20:35 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:           REUFO
; PURPOSE:              
;       IDL routine to read to a WDF array an ASCII file in UFO format #111.
;       Format is assumed to consist of a 5 line header, followed by
;       N lines of x,y data pairs, concluded with a -999,-999 data pair.
;
; CALLING SEQUENCE:     REUFO, WDF [, FILENAME] [, FILENAME=fname ]
; INPUT:
;       WDF:            WDF array to save the data
;                       Default: WDF = GET_MAXWDF()
;       FILENAME:       Input filename or unit number.
;                       If zero or undefined, DIALOG_PICKFILE is called
;                         and the selected name is returned in FILENAME
; OUTPUTS:              X,Y DATA is stored in array, WDF
; KEYWORDS:
;       FILENAME:       Ignored if the parameter FILENAME is present
;                       If FILENAME is undefined, then FNAME is used.
;                       If zero or undefined, DIALOG_PICKFILE is called
;                         and the selected name is returned in FILENAME
; EXAMPLE:
;       Read to dataset 4, UFO file, data_4.ufo
;         REUFO, 4, 'data_4.ufo'
;       Read file, zdata1256_09_09_09_wave.ufo, to dataset 6
;         REUFO, 6, 'zdata1256_09_09_09_wave.ufo'
;
; MODIFICATION HISTORY:
;       Adapted from W E Nelson's routine and more recently RUFO.
;       Included in PFIDL, L. P. Mix, 9/10/2009
;
;-
;
PRO reufo, WDF, filename, file=fname

on_error, 1

NPAR = n_params()
if npar eq 0 then begin
  print, '% FORMAT:  REUFO, WDF [, FILENAME] [, FILENAME=filename ]'
  return
endif
maxwdf = get_maxwdf()
if n_elements(wdf) eq 0 then wdf = maxwdf else $
  if (wdf le 0) or (wdf gt maxwdf) then wdf = maxwdf
If (n_elements(filename) eq 0) && (n_elements(fname) ne 0) then begin
  filename = fname[0]
  lckey = 1
endif else lckey =0
filename = (n_elements(filename) eq 0) ? 0 : filename[0]
readasc,filename,2,xdata,ydata,skip=5, endnumber =-999
if lckey then fname = filename

I2W, Xdata, Ydata, WDF, file = filename, /xydata, clabel = get_bare_filename(filename)

xdata =0 & ydata =0

end
