; ------------------------------------------------------------------------ 
; $Id: revida.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro revida, fname,  WDF, xydata = xydata, npoints = np, skip = skip
;
;+
;    read a VIDA file into a wdf array
;    Format is :
;           xlabel ylabel
;            x1     y1
;           ...    ...
;            xn     yn
;          -999    -999
;    format is:  revida, fname, WDF, xydata = xydata
;    where 
;      fname   - the file name or a unit numberr
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - WDF array for storage
;      npoints - Returned with the number of points read.
;      xydata  - If set to zero, then the data will be put on a uniform grid.
;                Default is XYDATA = 1, ie save as read
;      skip    - If a number gt 0, then that many lines will be skipped
;                before reading the labels.
;    Example:
;    Read a VIDA array from unit 2 and store in wdf array 12.
;       REVIDA, 2, 1
;
;    Read VIDA data from a file selected by pickfile and store it in
;    WDF array 10.
;       REVIDA, 0, 10
;
;    Read a VIDA arrays from file, 'testdata.vida' into WDF arrays 3
;       of the following form:
;        111 = FORMAT #
;        2 = # TABLE COLUMNS
;        1  1 = ITYPE, IORDER
;        0.00E+00  0.00E+00 = ABSTOL, RELTOL
;                     X             Y
;               5033.39     0.0182046
;               5035.53     0.0194887
;
;       REVIDA, 'testdata.vida', 3, skip = 4
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: REVIDA, FNAME_UNIT, WDF ,[xydata=0]' 
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, ' File name must be specified by name or unit number'
        return
      endif
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, '% REVIDA: No File was Selected - RETURNING'
            return
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = 'VIDA data'
      if n_elements(wdf) eq 0 then begin
        print,'WDF array value must be provided'
        return
      end
      wdfar = wdf(0)
      if wdfar lt 1 or wdfar gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if n_elements(np) eq 0 then np = 4096L
      if np lt 4096 then np = 4096L
      maxnp = long(np)
      junkstr = ''
      on_ioerror, badnp
; Time to start reading
      if n_elements(skip) gt 0 then begin
        if skip gt 0 then for i = 1, skip do readf, unit, junkstr
      endif
      readf, unit, junkstr
      parser, junkstr, /string, names,nlab
      if nlab ne 2 then begin
        print, '% REVIDA: Two Axis labels not read'
        return
      endif
      xlab = names(0)
      Ylab = names(1)
      ax = make_array (maxnp )
      bx = ax
      i=0L
      x = 0.0
      y = 0.0
      while (not eof(unit))  do begin
        on_ioerror, badnum
        readf,unit,x, y
        if x eq -999. and y eq -999. then goto, badnum
        ax(i) = x
        bx(i) = y
        i = i+1
        if i eq maxnp then begin
          maxnp = maxnp+maxnp
          ax = [ax, ax]
          bx = [bx, bx]
        endif
      endwhile
badnum:
      if lcfile eq 1 then free_lun, unit
      if i eq 0 then begin
        np = 0
        print, ' Error reading file -- No data found'
        return
      endif
      np = i
      if n_elements (xydata) eq 0 then xydata = 1
      ax= ax(0:i-1)
      bx = bx(0:i-1)
; convert to wdf array using the lowres option
      i2w, ax, bx ,wdfar, c = fname, /lowres, xydata = xydata, $
          file = fname, xlab = xlab, ylab = ylab 

      return

badnp:
      if lcfile eq 1 then free_lun, unit
       print, ' Error reading number of points -- Check format of data '
      return
      end
