; ------------------------------------------------------------------------ 
; $Id: set_drawsize.pro,v 1.6 2008/04/09 14:27:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro set_drawsize, showdefault = showd, bigsize = big, ratio = rat, $
      smallsize = small, dualsize = dual, medsize = med, initialize = init
;+
;     Name:
;            SET_DRAWSIZE
;     Purpose:
;            Set the default size of draw widgets.
;     
;     Format: 
;            set_drawsize
;     
;     No required parameters.  If no keywords, help will be given.
;      
;     Keywords:
;       SHOWDEFAULT:  Show current values
;         NOTE:  Show is executed last; any changes are current.
;
;   The following are the size in pixels for the draw widgets.
;   If a single value then it is used for XSIZE and YSIZE = ratio * XSIZE
;   If two values then they are [XSIZE, YSIZE]
;
;       RATIO:        Ratio of YSIZE to XSIZE-- YSIZE = RATIO*XSIZE
;       BIGSIZE:      Set default size for TQEDIT, QSEDIT, WDEDIT
;       MEDSIZE:      Set default size for LINSTR, XCURW, WDSPLIT
;       DUALSIZE:     Set default size for ZOOMW
;       SMALLSIZE:    Set default size for PREVIEW in LINSTR
;
;   Example:
;     set the size for the preview frame to 450 in X and 400 in Y
;       SET_DRAWSIZE, small = [450, 400]
;
;     Show current defaults
;       SET_DRAWSIZE, /show
;
;   Original version: February 4, 1998, lpm
;
;-
      on_error, 1
      common pffcom, initpff, pff_exe
@draw_defaults.com
; common draw_defaults, rrratio, bigsize, medsize, dualsize, smallsize

;
;     start the process
;
      lcset = 0
;     have window a multiple of 16
      window_size = 16L
      window_size2 = window_size /2

      if n_elements(rrratio) eq 0 then begin
        rrratio = 0.9
        catch, TheError
          if ((!d.name eq 'X') or (!d.name eq 'WIN') or (!d.name eq 'MAC')) $
            And (TheError eq 0) then screen_size = my_get_screen_size()
        catch, /cancel
        if n_elements (screen_size) lt 2 then screen_size=[1000, 800]
;       added next line for dual monitors
;        if screen_size[0] gt 1800 then screen_size[0] = screen_size[0]/2
        bigsize = round(0.8*(screen_size))
        fudge = bigsize mod window_size
        ok = where(fudge ge window_size2, nok)
        if nok gt 0 then fudge[ok] = fudge[ok] - window_size
        bigsize = bigsize - fudge 
        medsize = round(0.7*bigsize)
        fudge = medsize mod window_size
        ok = where(fudge ge window_size2, nok)
        if nok gt 0 then fudge[ok] = fudge[ok] - window_size
        medsize = medsize - fudge
        dualsize = round([(screen_size[0]-50)/2, 0.7*bigsize[1]])
        fudge = dualsize mod window_size
        ok = where(fudge ge window_size2, nok)
        if nok gt 0 then fudge[ok] = fudge[ok] - window_size
        dualsize = dualsize - fudge
        smallsize = round(0.45*bigsize)
        fudge = smallsize mod window_size
        ok = where(fudge ge window_size2, nok)
        if nok gt 0 then fudge[ok] = fudge[ok] - window_size
        smallsize = smallsize - fudge
        if keyword_set(init) then return

        lcset = 1

; use bigsize for qsedit, tqedit
; use [bigsize[0]-225, bigsize[1]] for grid generation
; use medsize for lin6str, xcurw
; use dualsize for zoomw
; use small size for preview for linstr
      endif

      nel = n_elements(rat)
      if nel gt 0 then begin
        rrratio = rat(0)
        lcset = 1
      endif

      nel = n_elements(big)
      if nel gt 0 then begin
        if nel ge 2 then bigsize = big(0:1) else $
          bigsize= [big, rrratio*big]
        bigsize = round (bigsize)
        lcset = 1
      endif

      nel = n_elements(small)
      if nel gt 0 then begin
        if nel ge 2 then smallsize = small(0:1) else $
          smallsize= [small, rrratio*small]
        smallsize = round(smallsize)
        lcset = 1
      endif

      nel = n_elements(med)
      if nel gt 0 then begin
        if nel ge 2 then medsize = med(0:1) else $
          medsize= [med, rrratio*med]
        medsize = round(medsize)
        lcset = 1
      endif

      nel = n_elements(dual)
      if nel gt 0 then begin
        if nel ge 2 then dualsize = dual(0:1) else $
          dualsize= [dual, rrratio*dual]
        dualsize = round(dualsize)
        lcset = 1
      endif

      if keyword_set (showd) then begin
        lcset = 1
        f = '(a, i7, i7)'
        print, 'Default Screen Sizes:  Xsize  Ysize'
        print, '                 Big:'     , bigsize(0), bigsize(1), format=f
        print, '              Medium:'     , medsize(0), medsize(1), format=f
        print, '                Dual:'     , dualsize(0), dualsize(1), format=f
        print, '               Small:'     , smallsize(0), smallsize(1), format=f
      endif
      if lcset eq 0 then begin
Print,'% Format: SET_DRAWSIZE [, SHOWDEFAULT = showdefault] [, RATIO = ratio]'
print,'           [, BIGSIZE = bigsize] [, SMALLSIZE = smallsize]'
print,'           [, DUALSIZE = dualsize] [, MEDSIZE = medsize]'
      endif
      return
      end
