; ------------------------------------------------------------------------ 
; $Id: setdefault.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro setdefault, transplot = transplot, ccolor =ccolor, cthick = cthick, $
                 gcolor =gcolor, gthick = gthick, fcolor = fcolor, $
                 directoryhelp= dirhelp, directoryfont = dirfont,  $
                 directorypage= dirpage , path = path , filter = filter
;+
;  Set default path and filter for read and conductor/grid parameters
;  format; setdefault [, PATH = path] [, FILTER = filter]
;                     [, ccolor = ccolor] [, cthick = cthick]
;                     [, gcolor = gcolor] [, gthick = gthick]
;                     [, transplot = transplot]
;                [directoryhelp= dirhelp] [, directoryfont = dirfont] 
;                [directorypage= dirpage]
;    where: 
;         PATH   - Initial path for OPENPFF if no path is specified
;         FILTER - Initial filter for OPENPFF if no filter is specified
;    where c and g refer to conductor and grid and:
;         color - default color of the conductor must be greater than
;                 zero and less than !d.table_size
;                 1 to 7 are the basic colors 
;                 The colors are: red(1), green(2), blue(3),
;                 purple(4), orange(5), light blue(6), yellow(7)
;                 gcolor may be an array to sequence over colors.
;         thick - default line thickness, greater than 0, less than 20
;        fcolor - default color code for contour plots.(PLOTFLD)
;        directoryhelp - 0 no help, 1 print a help for DIRP
;        directoryfont - name of scroll directory font
;        directorypage - number of lines on a scroll directory
;  Example:
;      set the default path to '/extra/qs/qs5851/' and the filter to '*.pff'
;        setdefault, path = '/extra/qs/qs5851/' , filter = '*.pff'
;
;      set conductor color to red and thickness to 3 times normal
;        and set grid to green and 0.5
;        setdefault, ccolor = 1, cthick = 3, gcolor = 2, gt=0.5
;-
; insert a common block
;
@qscom.dat
COMMON savepicker, oldfilter, oldpath
;
        on_error, 1
        if n_elements (path) gt 0 then oldpath = path
        if n_elements (filter) gt 0 then oldfilter = filter
;
        if n_elements (transplot) gt 0 then transdefault = transplot
;
        if n_elements (dirhelp) gt 0 then directoryhelp = dirhelp
        if n_elements (dirfont) gt 0 then directoryfont = dirfont
        if n_elements (dirpage) gt 0 then directorypage = dirpage
        if n_elements (ccolor) gt 0 then begin
          if ccolor gt 0 and ccolor lt !d.table_size then concolor = ccolor $
          else print, ccolor, ' is not a valid conductor color number'
        endif
        if n_elements (cthick) gt 0 then begin
          if cthick gt 0 and cthick lt 20 then conthick = cthick $
          else print, ccolor, ' is not a valid conductor thickness'
        endif
        if n_elements (gcolor) gt 0 then begin
          gcolor = long(gcolor)
          if n_elements (gcolor) eq 1 then begin
            if gcolor ge 0 and gcolor le !d.table_size $
                then grdcolor = gcolor 
            if gcolor eq 8 then grdcolor = 0
          endif else if min(gcolor) gt 0 and max(gcolor) le !d.table_size $
                then grdcolor = gcolor $
          else print, gcolor, ' is not a valid grid color number'
        endif
        if n_elements (gthick) gt 0 then begin
          if gthick gt 0 and gthick lt 20 then grdthick  = gthick $
          else print, gthick, ' is not a valid grid thickness'
        endif
        if n_elements (fcolor) gt 0 then fldcolor = fcolor
        end
