; ------------------------------------------------------------------------ 
; $Id: showpff.pro,v 1.6 2010/08/24 21:53:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro showpff, current= p1, ufid = p2, number = number, $
              filename = filename, quiet = quiet
;
;+
;  Show all opened PFF files, 
;  their file id (fid) and indicate the location of the current file pointer.
;  The current file id, and arrays of the user file ids and filesnames
;    can be received as keywords.
;  Format: SHOWPFF [, current = current] [, ufid = ufid] [, /QUIET]
;              [, filename = filename]  [, NUMBER= number]
;  where
;         current - user file id of the current file
;         ufid    - array of user file id's
;         filename- Array of file names corresponding to the ufid's
;                   Note: longer filenames are returned with PFF_INFO
;         number  - number of files open
;                   Note: Number of datasets available with NDSPFF and PFF_INFO
;         quiet   - If present and not zero, send no output to terminal
;  Example:
;  Show the status, including file id, of all opened PFF files
;    showpff
;  
;  Get the filenames and user file id's for all open files
;  Do not send any output to the terminal
;  Note: File names will be truncated to 60 characters
;    showpff, c= cur, u=u, num= n, fil= fil, /q
;-
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      number = 0
      p1= 0L
      mxfil = 20
      p2 = make_array(mxfil, /long)
      filename = make_array(mxfil, /string, value = '')
      if (n_elements(initpff) eq 0) then initpff = 0
      if (initpff eq 0) then begin
        print, '% SHOWPFF: No PFF files are open'
        return
      endif
;     make the string equal to 256*(max # of files)= 5120
      lstr =256L
      p4 = lstr *mxfil
      p3 = make_array(p4,/byte, value = 32b)
      if n_elements (quiet) eq 0 then quiet = 0L else quiet = long(quiet)
      if initpff eq 1 then begin
; HP
        junk= call_external (PFF_EXE, 'pff_slshow',p1, p2, p3, p4, quiet)
      endif else if initpff eq 2 then begin
; Sun
        junk= call_external (PFF_EXE, 'pff_slshow_',p1, p2, p3, p4, quiet)
      endif else if initpff eq 4 then begin
; WIN
        junk= call_external (PFF_EXE, '_PFF_SLSHOW@8',p1, p2, p3, p4, quiet)
      endif
;     see if any files are open -- if not, return
;
      number = p4
      if p1 eq 0 then return 
      p3 = string(p3)
      for i = 0, number-1 do filename[i]= strtrim(strmid(p3,i*lstr,lstr), 2)
      return
      end
