; ------------------------------------------------------------------------ 
; $Id: sjpeg.pro,v 1.3 2010/08/18 16:00:14 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro sjpeg, image
;+
;  Save the current screen information to a motion JPEG2000 file.
;  File is determined by STARTJPEG
;  Movie can be seen using MJ2_Player or 
;
;  Format: SJPEG [, image]
;  Parameter:  IMAGE - If present, the image will be written to the 
;                      motion JPEG file.  Image must have more than 4 pixels.
;
;  Examples:
;  Send the nframe images to a motion JPEG2000 file called: target.mj2
;; Initialize JPEG2000
;    startJPEG, 'target.mj2'
;; Generate frames
;    for i = 1L, nframe do begin
;      <Make a plot>
;      sjpeg
;    endfor
;; Close the file
;    endjpeg
;; View the Movie
;    MJ2_Player
;
;; Similar if images are stored in true color in a series of files called
;    IMAG_FILES in the above example replace the for loop with:
;    for i = 1L, nframe do begin
;      read_jpeg, IMAG_FILES[i-1], image
;      sjpeg, image
;    endfor
;
;
;  MODIFICATION HISTORY:
;      Original version, L. P. Mix, April 12, 2006
;
;-
on_error, 1
   common switchjpeg, omj2_write, file_name, current_file, initial
 
   if not obj_valid(omj2_write) then begin
     print, '% SJPEG: Unable to save image.'
     Print, '        No valid object for motion JPEG2000'
     PRINT, '        USE: STARTJPEG [, FILENAME] '
     return
   endif
   if n_elements(image) gt 4 then result = oMJ2_write ->  SetData(image) $
   else begin
     device, get_visual_depth = depth
     if depth le 8 then result = oMJ2_write ->  SetData(tvrd()) $
          else  result = oMJ2_write -> SetData(tvrd(/true))
   endelse
   if result ne 1 then begin
      print, '% SJPEG: Possible error with SetData for IDLffMJPEG2000 object'
      help, result
   endif
   end
