; ------------------------------------------------------------------------ 
; $Id: sp2xyz.pro,v 1.3 2007/06/06 20:03:54 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
Pro sp2xyz, image, space, x, y, z
;+
;  Decompose a spatial array from the READPFF command into its component 
;  arrays for 1D, 3D and vector dataset types.
;  Format: SP2XYZ, image, space, x [, y] [, z ]
;  where:
;    image - image array from READPFF
;    space - space array from READPFF
;    x - one dimensional array for the first spatial dimension of the data
;    y - one dimensional array for the second spatial dimension of the data
;    z - one dimensional array for the third spatial dimension of the data
;  Examples:
;  Convert the spatial array, geom, of a series of image frames from the array, 
;  picture, into x, y, and time arrays.
;    sp2xyz, picture, geom, x, y, time
;-
      if (n_elements(image) eq 0 or n_elements(space) eq 0) then begin
        print, ' Image and space must be defined'
        return
      endif
;
      nparams = n_params()
      b = size (image)
      c = size (space)
;
    if (b(0) eq 3) then begin
       if (b(1)+b(2)+b(3) gt c(1) ) then begin
	 print,' not enough elements in the space array'
         return
       end
;
       x = space (0:b(1)-1)
       y = space (b(1): b(1)+b(2)-1)
       z = space (b(1)+b(2): b(1)+b(2)+b(3)-1)
;
       return
     endif
;
     if (b(0) eq 2) then begin
        if (b(1)+b(2) gt c(1) ) then begin
                print,' not enough elements in the space array'
        return
        end
;
        x = space (0:b(1)-1)
        y = space (b(1): b(1)+b(2)-1)
        if b(1)+b(2) lt c(1) then    z = space(b(1)+b(2)) else z=0.0
        return
     endif
;
     if (b(0) eq 1) then begin
        if (b(1) gt c(1) ) then begin
                print,' not enough elements in the space array'
        return
        end
;
        x = space (0:b(1)-1)
        if b(1)+1 ge c(1) then y = space (b(1)) else y = 0.0
        if b(1)+2 ge c(1) then z = space (b(1)+1) else z = 0.0
        return
     endif
;
end
