; ------------------------------------------------------------------------ 
; $Id: starteps.pro,v 1.3 2007/06/06 20:03:54 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro starteps, font= font, color= color, bold = bold, preview=preview,$
              copy=copy, interpolate = interpolate, file=file, noload =noload, $
                thick = thick, xythick = xythick, _extra = extrastuff
;+
;  Generate an encapsulated postscript file of the next graphics commands for 
;  incorporation into another document; no graphics is sent to the screen.
;  Helvetica font is used.  This may be changed by adding a extra keyword.
;  Note: ENDPOST must be used to close the file and return to terminal output
;  Format: STARTEPS  [, FONT = font]  [, COLOR= color]  [, BOLD = bold]  
;                  [, COPY = copy] [, INTERPOLATE = interpolate]
;                  [, THICK = thick]  [, XYTHICK = xythick]
;                  [, PREVIEW=preview] [, FILE=file]  [, BITS = bits]
;                  [, NOLOAD = noload] [, _extra = extrastuff]
;  where:
;    font - font size in points.  Default is 18 pt
;    color - black and white output is used if color is set to zero.
;            Default is color= 1
;    copy - See Set_plot - Copy color table from current device to postscript
;                          Default = 0
;    Interpolate - See Set_plot - Interpolate current color table to the new
;                  device. Default = 0
;         Note: If X-Y color table is used, these pure colors will be changed
;    bold - specifies bold print for the output if present and nonzero
;            Default is 0
;    thick - specifies the plot line thickness.  Default is 5
;    xythick - Specifies the axes thicknesses. Default is 5.
;    preview - generate a preview frame for the file if present and nonzero
;              Default is 0 (No preview frame and smaller file size)
;    file - output file name for the encapsulated postscript output.
;           Default is the default print file.
;           See "SET_PRINTER" or enter "SET_PRINTER, /SHOW"
;           If a directory is not specified then the file will be in the 
;             default directory.
;    bits - bits per pixel.  Default is 8
;    noload - if set, no color table will be loaded.
;             DEFAULT: LOADXYCT
;    _extra and extrastuff are valid keywords and values for the device command
;
;  Examples:
;  Send the next plot to a file called 'QSsummary.eps' with a preview frame 
;  and plot a graph.
;    starteps, /preview, file = 'QSsummary.eps
;    plo, 2, 3, /ov, title = 'Summary of Calculations' 
;    endpost
;
;  Same as above but plot in "Times" font. Change the first line as follows:
;    starteps, /preview, file = 'QSsummary.eps', /times
;-
@switchpost.com
@lprintsav.com
Common set_printer_common, printfile_save, file_path_save
on_error, 1

if n_elements (lpinit) eq 0 then set_printer
if n_elements(lcopen) eq 0 then lcopen = 0



   if !d.name eq 'X' or !d.name eq 'TEK' or !d.name eq 'WIN' $
        or !d.name eq 'MAC' then begin
     oldxythick = !x.thick
     OLDThick = !p.thick
     oldfont = !p.font
     olddevice =!d.name
   endif else if n_elements(olddevice) eq 0 then olddevice = 'X'

   if n_elements(thick) eq 0 then thick = 5
   if n_elements(xythick) eq 0 then xythick = 5
   if n_elements (color) eq 0 then color = 1 
   if n_elements (copy) eq 0 then copy = 0
   if n_elements (interpolate) eq 0 then interpolate = 0
   oldcopy = copy
   oldinterpolate = interpolate
   xythicksave = xythick
   thicksave = thick
   oldxythick = !x.thick
   oldthick = !p.thick
   oldfont = !p.font
   !x.thick = xythick
   !y.thick = xythick
   !z.thick = xythick
   !p.thick = thick
   !p.font = 0
   if n_elements (font) eq 0 then font = 18
   if n_elements (preview) eq 0 then preview = 0
   if n_elements (file) eq 0 then file = default_printfile else begin
     file = strtrim(file[0], 2)
     if strlen(file) eq 0 then file  = default_printfile else begin
       file = fulfnamf(file)
       name = strupcase(strmid(!version.os, 0, 3))
       if name eq 'WIN' then $
         lcfullname = (strmid(file, 1, 2) eq ':\') ?  1 : 0 $
       else lcfullname = (strmid(file, 0, 1) eq '/'  or $
            strmid(file, 0, 1) eq '.') ?  1 : 0
       file = lcfullname ? file : file_path_save+file
     endelse
   endelse

   if lcunique gt 0 then $
     file = find_newfile(file, initial = lcunique, /quiet)
   print_file_last= file
   if n_elements (bold) eq 0 then bold = 0
   if n_elements (bits) eq 0 then bits = 8
;
   lcopen = 1
   set_plot, 'ps', interpolate = interpolate, copy = copy
   device, /helvet, font_size = font, /enca, file= file, color=color, $
          preview=preview, /port, bold = bold, bits=bits, _extra=extrastuff
   if keyword_set(noload) then noloadsave = 1 else begin
     loadxyct
     noloadsave = 0
   endelse
   return
   end
