; ------------------------------------------------------------------------ 
; $Id: startjpeg.pro,v 1.3 2010/07/27 17:14:26 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro startjpeg,file, reversible = reversible, _extra = extrastuff
;+
;  Make the object to write a motion JPEG2000 file.
;  Note: ENDJPEG must be used to close the file.
;  Note: StartGif may be used to obtain TrueType fonts, wider lines, 
;        and white background.
;  Note: To save a single frame from a motion JPEG2000 file use MJPEG_FRAME
;
;  Format: STARTJPEG [, FILE] [, REVERSIBLE = reversible] [_EXTRA = extrastuff]
;
;  where:
;    file       - output file name for JPEG2000 output
;                 Default is 'idljpeg.mj2'
;                 A Unique file will be opened if none is specified.
;    REVERSIBLE - Specifies if the compression is lossy.
;                 Default: REVERSIBLE = 1 
;                 Typically is will be faster to compress and decompress lossy
;                 (ie, REVERSIBLE = 0) frames, but this depends on the image
;                 content.
;    extrastuff - Any valid keywords and values for the OBJ_NEW command for an
;                 IDLffMJPEG2000 object.
;
;  Examples:
;  Send the nframe images to a motion JPEG2000 file called: target.mj2
;; Initialize JPEG2000
;    startJPEG, 'target.mj2'
;; Generate frames
;    for i = 1L, nframe do begin
;      <Make a plot>
;      sjpeg
;    endfor
;; Close the file
;    endjpeg
;; View the Movie
;    MJ2_Player
;
; As noted above, MJPEG_FRAME will save individual frames to a PNG or other type 
; file.
;-

on_error, 1 
   common switchjpeg, omj2_write, file_name, current_file, initial
;
   if obj_valid(omj2write) then begin
     print, '% StartJPEG: Unable to start JPEG2000 file.'
     Print, '         File is already open: ', current_file
     PRINT, '         USE: EndJPEG'
     return
   endif
   if n_elements(reversible) eq 0 then reversible = 1
   if n_elements(file) eq 0 then begin
     if n_elements (file_name) eq 0 then file_name = 'idljpeg.mj2'
     file = file_name
     info = file_info(file)
     if info.exists gt 0 then file = find_newfile(file, /quiet, initial= initial)
   endif
   openw, unit, file, /get_lun, error = err
   if err ne 0 then begin
     Print, '% StartJPEG: Valid File Name must be provided'
     print, ' Directory must be one with write permission for user'
     return
   endif
   free_lun, unit
   oMJ2_write=OBJ_NEW('IDLffMJPEG2000', file, /WRITE, REVERSIBLE= reversible, $
              _extra = extrastuff)
   if obj_valid(oMJ2_write) then current_file = file else $
     print, '% StartJPEG: Unable to create Motion JPEG2000 object'
   end

