; ------------------------------------------------------------------------ 
; $Id: startpost.pro,v 1.7 2010/09/20 14:16:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro startpost,pfile, portrait = portrait, set_default = set_default, $
             font= font, color= color, bold = bold, noload = noload, $
             file = file, bits = bits, thick = thick, xythick = xythick, $
             copy = copy, interpolate = interpolate, dataplot = data, $
             xsize = xsize, ysize = ysize, xoffset = xoffset, $
             yoffset = yoffset, _extra= extrastuff
;+
;  Generate a postscript file of all graphics output. No graphics is sent 
;  to the screen.  Helvetica font is used in landscape orientation.
;  Font may be changed using the _extra keyword.
;  Note: ENDPOST must be used to close the file and return to terminal output
;  Format: STARTPOST [, FILE] [, FONT = font]  [, COLOR= color]  [, BOLD = bold]
;                  [, COPY = copy] [, INTERPOLATE = interpolate] [, FILE=file]
;                  [, BITS = bits] [, THICK = thick] [, XYTHICK = xythick]
;                  [, SET_DEFAULT = set_default] [, NOLOAD = noload]
;                  [, PORTRAIT = port] [, DATAPLOT = dataplot]
;                  [, XSIZE = xsize] [, YSIZE = ysize] [, XOFFSET = xoffset]
;                  [, YOFFSET = yoffset] [, _extra = extrastuff]
;  where:
;    file - output file name for the postscript output.
;           If the file parameter is used the keyword file is ignored.
;           Default is the default print file.
;           See "SET_PRINTER" or enter "SET_PRINTER, /SHOW"
;           If a directory is not specified then the file will be in the 
;             default directory.
;    portrait - put output into portrait mode with a full page
;              margin is 1/2 inch, size is 7.5x10 inches
;              DEFAULT is landscape
;    font - font size in points.  Default is 18 pt
;    color - black and white output is used if color is set to zero.
;            Default is color= 0
;    copy - See Set_plot - Copy color table from current device to postscript
;                          Default = 0
;    Interpolate - See Set_plot - Interpolate current color table to the new 
;                  device. Default = 0
;         Note: If X-Y color table is used, these pure colors will be changed
;    bold - specifies bold print for the output if present and nonzero
;            Default is bold = 0
;    thick - Plot line thickness.  Default is 5
;    xythick - Axis thickness.  Default is 5
;    bits - bits per pixel.  Default is 8
;    noload - if set, no color table will be loaded.
;             DEFAULT: LOADXYCT
;    set_default - If set the defaults for portrait, font, and color may 
;             be set. 
;            Note:  Default values are:
;                   Color = 0 & Portrait = 0 & font = 18
;            NOTE:  The command is ignored and only the default is changed.
;    DATAPLOT:  If set, assume a data plot.
;               Keyword defaults become:
;                 Thick = 2
;                 XYthick = 2
;                 Font = 10
;                 Xsize = 10.5
;                 Ysize = 7.75
;                 Inches = 1
;                 Yoffset = 10.75
;                 Xoffset = 0.25
;    _extra - any acceptable variables in the call to the "device" procedure
;             for example: /avant will use Avant Garde PostScript font
;  Note:  When using Postscript, Greek characters are accessed using "!9" and 
;         the norman font is "!3" for example distance in microns for an 
;         axis label would be written: xtitle='Distance (!9m!3m)'
;  Examples:
;  Send the next plots to a file called 'QSsummary.ps'
;    startpost, file = 'QSsummary.ps
;    { plot the stuff }
;    endpost
;
;
;  Same as above but plot in "Times" font. Change the first line as follows:
;    starteps, /preview, file = 'QSsummary.eps', /times
;-
   on_error, 1
;
;
@switchpost.com

common startpost_save, color_save, portrait_save, font_save
@lprintsav.com
;
Common set_printer_common, printfile_save, file_path_save
;
;common lprintsav,lpinit, destsave, filesave, default_printfile
;
   if n_elements (lpinit) eq 0 then set_printer
;
   if n_elements(lcopen) eq 0 then lcopen = 0
   if n_elements(color_save) eq 0 then begin
     color_save = 0
     portrait_save= 0
     font_save = 18
   endif
   if keyword_set (set_default) then begin
     if n_elements(color) gt 0 then color_save = color
     if n_elements(portrait) gt 0 then portrait_save = portrait
     if n_elements(font) gt 0 then font_save = font else $
     if keyword_set(data) then font_save = 10
     return
   endif
   if n_elements (portrait) eq 0 then portrait = portrait_save
   if n_elements (thick) eq 0 then thick= keyword_set(data) ? 2 : 5
   if n_elements (xythick) eq 0 then xythick= keyword_set(data) ? 2 : 5
   if keyword_set(data) then begin
     if keyword_set(portrait) then begin
       if n_elements(xsize) eq 0 then xsize = 8.0
       if n_elements(xoffset) eq 0 then xoffset = 0.25
       if n_elements(ysize) eq 0 then ysize = 10.5
       if n_elements(yoffset) eq 0 then yoffset = 0.25
     endif else begin
       if n_elements(xsize) eq 0 then xsize = 10.5
       if n_elements(xoffset) eq 0 then xoffset = 0.25
       if n_elements(ysize) eq 0 then ysize = 7.75
       if n_elements(yoffset) eq 0 then yoffset = 10.75
     endelse
     p_xoffset = 0.25
     p_yoffset = 0.25
     p_xsize = 8.0
     p_ysize = 10.5
     l_xoffset = 0.25
     l_yoffset = 10.75
     l_xsize = 10.5
     l_ysize = 7.75
   endif else begin
     if keyword_set(portrait) then begin
       if n_elements (ysize) eq 0 then ysize=10.0
       if n_elements (yoffset) eq 0 then yoffset=0.5
       if n_elements (xsize) eq 0 then xsize = 7.5
       if n_elements (xoffset) eq 0 then xoffset = 0.5
     endif 
     P_ysize=10.0
     P_yoffset=0.5
     P_xsize = 7.5
     P_xoffset = 0.5
     l_xoffset = 0.75 
     l_yoffset = 10.25 
     l_xsize = 9.5
     l_ysize = 7.0
   endelse
   if !d.name eq 'X' or !d.name eq 'TEK' or !d.name eq 'WIN' $
         or !d.name eq 'MAC' then begin
     oldxythick = !x.thick
     oldthick = !p.thick
     oldfont = !p.font
     olddevice =!d.name
   endif else if n_elements(olddevice) eq 0 then olddevice = 'X'
   if n_elements (pfile) gt 0 then file = strtrim(pfile(0), 2)
   if n_elements (file) eq 0 then file = default_printfile else begin
     if strlen(file[0]) eq 0 then file = default_printfile else begin
       file = fulfnamf(file[0])
       name = strupcase(strmid(!version.os, 0, 3))
       if name eq 'WIN' then $
         lcfullname = (strmid(file, 1, 2) eq ':\') ?  1 : 0 $
       else lcfullname = (strmid(file, 0, 1) eq '/'  or $
            strmid(file, 0, 1) eq '.') ?  1 : 0
       file = lcfullname ? file : file_path_save+file
     endelse
   endelse
   if lcunique gt 0 then $
     file = find_newfile(file, initial = lcunique, /quiet)
   print_file_last= file
   if n_elements (color) eq  0 then color = color_save
   if n_elements (copy) eq 0 then copy = 0
   if n_elements (interpolate) eq 0 then interpolate = 0
   oldcopy = copy
   oldinterpolate = interpolate
   xythicksave = xythick
   thicksave = thick
   !x.thick = xythick
   !y.thick = xythick
   !z.thick = xythick
   !p.thick = thick
   !p.font = 0
   if n_elements (font) eq 0 then font = keyword_set(data) ? 10 : font_save
   if n_elements (bold) eq 0 then bold = 0
   if n_elements (bits) eq 0 then bits = 8
;
   set_plot, 'ps', interpolate = interpolate, copy = copy
   if portrait eq 0 then $
   device,/helvet,font_s=font,file=file,color=color,bold=bold, /inches, $
     bits=bits,/landscape, encap= 0, preview = 0, xsize = xsize, $
     ysize = ysize, xoffset = xoffset, yoffset = yoffset,  _extra = extrastuff $
   else device,/helvet,font_s=font,file=file,color=color,bold=bold, $
     bits=bits,/portrait, encap= 0,/inches, ysize=ysize, yoffset=yoffset, $
      xsize = xsize, xoffset = xoffset, preview = 0,  _extra = extrastuff
   lcopen = 1

   if keyword_set(noload) then noloadsave = 1 else begin
     loadxyct
     noloadsave = 0
   endelse
;
   end
