; ------------------------------------------------------------------------ 
; $Id: svideo.pro,v 1.1 2011/05/05 18:38:59 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro svideo, image, verbose = verbose
;+
;  Save the current screen information to a AVI or MPEG-4 file.
;  File is determined by STARTVIDEO
;
;  Format: SVIDEO [, image]
;  Parameter:  IMAGE - If present, the image will be written to the 
;                      video file.  Image must be true color of the form
;                      [3, xsize, ysize]
;
;  Examples:
;  Send the nframe images to a motion VIDEO file called: target.mp4
;; Initialize VIDEO
;    startVIDEO, 'target.mp4'
;; Generate frames
;    for i = 1L, nframe do begin
;      <Make a plot>
;      svideo
;    endfor
;; Close the file
;    endvideo
;;
;  View with xine on Linux or QuickTime on Windows.
;
;  MODIFICATION HISTORY:
;      Original version, L. P. Mix, May 2, 2011
;
;-
on_error, 1

@switchvideo.dat
 
   if not obj_valid(ovid_write) then begin
     print, '% SVIDEO: Unable to save image.'
     Print, '        No valid object for video'
     PRINT, '        USE: STARTVIDEO [, FILENAME] '
     return
   endif
   if n_elements(video) eq 0 then begin
     device, get_visual_depth = depth
     if depth gt 8 then image = tvrd(/true) else begin
       print, '% SVIDEO: Only true color images may be written'
       return
     endelse
   endif
   ss = size(image)
   if ss[0] eq 3 then begin
     if ss[1] ne 3 then begin
       print, '% SVIDEO: True color image must be of the form [3, xsize, ysize]'
       return
     endif
     xsiz = ss[2] 
     ysiz = ss[3]
   endif else begin
     print, '% SVIDEO: Image must be true color of the form [3, xsize, ysize]'
     return
   endelse
 
   if n_elements(image) gt 4 then begin
     if init_video then begin
       xsize_vid = xsiz
       ysize_vid = ysiz
       video_stream = ovid_write.AddVideoStream(xsiz, ysiz, fr_rate, codec = codec_vid)
       init_video =0
     endif else if (xsiz ne xsize_vid) or (ysiz ne ysize_vid) then begin
       print, '% SVIDEO: All images should be the same size.'
       return
     endif
     time = ovid_write.put(video_stream, image)
   endif else begin
     print, '% SVIDEO: Image must have at least 4 elements.'
     return
   endelse
   if keyword_set(verbose) then print, ' Time = '+strtrim (time,2)

   end
