; ------------------------------------------------------------------------ 
; $Id: whelp.pro,v 1.6 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro whelp, wminin, wmaxin, npoint = npoint, full = full, set_width = set_width, $
       comment = comment , file = file, statistics = statistics
;
;+
;  Lists the dataset comments for each of the WDF arrays. 
;  
;
;  Format: WHELP [, wda_min] [, wda_max]  [, NPOINT = npoint] [,/FULL]
;                [, /COMMENT] [, /FILE] [, /STATISTICS] [, SET_WIDTH = set_width]
;  where:
;    WDA_MIN- Indicates the first wdf array to be listed
;             WDA_MIN may be an array 
;         or
;             If a single parameter, WDA_MIN can be a search string.
;             Those arrays containing the string in the dataset comment will
;               be listed. If the first character is '^' then the comment must
;               begin with the following letters letters.
;               If string length is 1 then '^' will be taken as a search string.
;       Note: Only WDF arrays containing data will be printed.
;       Note: FULL = 0 if WDA_MIN is a string
;    WDA_MAX- Indicates the last wdf array to be listed
;             If wmax is le 0 then wmax = maxwdfarray
;    NPOINT - Indicates the number of points to be printed with a full listing
;             Default is 0.  If NPOINT is less than zero, all points will
;             be listed.  If NPOINT is greater than zero, NPOINT points will
;             be listed.  If NPOINT is specified, then FULL = 1.
;    FULL   - Indicates the full array information is to be printed if
;             present and not zero.
;             If NPOINT is specified, then FULL = 1
;             If STATISTICS is specified, then FULL = 1
;    STATISTICS - Statistical values on the Y- Values
;             (see STATS)
;    SET_WIDTH - Set the maximum length for character variables.  This applies
;             only to FULL=1.  The value of SET_WIDTH is saved until changed.
;             If SET_WIDTH LE 0, then the entire variable is written.
;             Default:  SET_WIDTH = -1
;             Note:  To set width in IDL_STARTUP use
;                    PRIAR, SET_WIDTH = WIDTH
;                    where width is the desired width.
;    FILE   - If set, file information will be printed.
;    Comment- If set, full comment will be printed. ';' or ';;'will be taken as a
;             carriage return. If comment is set, then full = 0.
;             Note: If one instance of ';;' is found that will be the delimiter
;                   for the remainder of the line.
;             Note: Unless FULL is set, the comment is limited to 65 characters of 
;                   printing unless COMMENT is set.
;     Note: If wmax is missing, default is FULL = 1 unless COMMENT is set.
;           If wmax is present, default is FULL = 0
;  Examples: 
;  Print a directory of the dataset comments for all the WDF arrays
;      whelp 
;  Print a full listing of the WDF array values for arrays 1 thru 3
;      whelp, [1, 2, 3]
; (or) whelp, 1, 3, /fu
; (or) whelp, 1, 3, n=0 (prints no data values only the range of data)
;
;  List all datasets with 'MOCAM' in the dataset comments.
;      whelp, 'mocam'
;-
;
;     include the common block specification
;
@com.dat
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
;  sort the input
;
      if n_elements(npoint) eq 0 then npoint = 0 else full = 1
      if keyword_set (comment) then full = 0
      npar = n_params()
      if npar eq 0 then begin
        wmin = where ( wdfnp ne 0, nwdf)+1
        if nwdf eq 0 then begin
          Print, ' All WDF arrays are empty.'
          return
        endif
      endif else if npar eq 1 then begin
        ss = size(wminin)  
        if ss (ss(0)+1) eq 7 then begin
          wmin= wminin(0)
          full = 0
          lst = strlen(wmin)
          if lst eq 0  then wmin = '*'
          ok = get_match (wmin, /wdf, nelement=nwdf)
          if ok(0) eq -1 then begin
           print, '% WHELP: No COMMENTS contain the string: ', wmin
           return
          endif
          wmin = temporary(ok)
        endif else begin   
          if n_elements(full) eq 0 then full = 1
          nwdf = n_elements(wminin)
          if nwdf eq 0 then begin
            print, ' First parameter to WHELP is undefined'
            return
          endif else wmin = wminin
          ok = where(wdfnp(wmin-1) ne 0, nwdf)
          if nwdf eq 0 then begin
            print, ' Specified WDF arrays are empty'
            return
          endif else wmin = wmin(ok)
        endelse
      endif else begin
        nwdf  =min ([n_elements(wminin), n_elements(wmaxin)])
        if nwdf eq 0 then begin
          print, ' Parameters to WHELP are undefined'
          return
        endif else begin
          wmax = wmaxin(0)
          wmin = wminin(0)
        endelse
        if wmax le 0 then wmax = maxwdfarray
        wmin = min([wmin, wmax], max = wmax) >1
        wmax = wmax < maxwdfarray
        ok = where(wdfnp(wmin-1:wmax-1) ne 0, nwdf)
        if nwdf eq 0 then begin
          print, ' Specified WDF arrays are empty'
          return
        endif else begin
          wmin = lindgen(wmax-wmin+1)+wmin
          wmin = wmin(ok)
        endelse
      endelse
      if keyword_set(statistics) then full = 1
      if n_elements(full) eq 0 then full = 0
      wmin = long(wmin)
      if full eq 0 then begin
        print, ' WDF Arrays with Valid Data are Listed:'
        print, ' Array  Dataset Comment'
        for i = 0, nwdf-1 do begin
          if keyword_set(comment) then begin
            comm = wdfcomment(wmin(i)-1)
            ldelim = 1
            delimiter = ';'
            ok = strpos(comm, delimiter, 0)
            lcom = strlen(comm)
            if ok lt 0 or ok+1 ge lcom then  print,wmin(i),format='(i4,4x,a)', $
              strmid(wdfcomment(wmin(i)-1),0,66) else begin
              print, wmin(i),format='(i4,4x, a)', strmid(comm,0,ok)
              oko=ok+ldelim
              if strpos(comm, ';', oko) eq oko then begin
                delimiter = ';;'
                ldelim = 2
                oko = oko+1
              endif
              ok = strpos(comm, delimiter, oko)
              while ok gt 0 do begin
                print, format='(8x, a)', strmid (comm, oko, ok-oko)
                oko= ok+ldelim
                if ldelim eq 1 and strpos(comm, ';', oko) eq oko then begin
                  delimiter = ';;'
                  ldelim = 2
                  oko = oko+1
                endif
                if oko ge lcom+1-ldelim then ok = -1 else $
                  ok = strpos(comm, delimiter, oko)
              endwhile
              if oko le lcom+1-ldelim then $
                print, format='(8x, a)', strmid (comm, oko, lcom-oko)
            endelse
            if keyword_set(file) then print, format='(8x, a)', $
              wdffile(wmin(i)-1)
          endif else  begin
            if keyword_set(file) then begin
              if(strlen(wdffile(wmin(i)-1)) + strlen(wdfcomment(wmin(i)-1)) gt $
                70)  then begin
                print, wmin(i),format='(i5,4x, a)' ,$
                      strmid(wdfcomment(wmin(i)-1),0,66)
                print, format='(8x, a)', wdffile(wmin(i)-1)
              endif else print, format='(i5,4x, a, a, a)', wmin(i), $
                wdfcomment(wmin(i)-1), ': ', wdffile(wmin(i)-1)
            endif else print, wmin(i),format='(i5,4x, a)' ,$
              strmid(wdfcomment(wmin(i)-1),0,66)
          endelse
        endfor
      endif else for i = 0, nwdf-1 do $
        priar, wmin(i), npoint = npoint, statistics = statistics, set_width = set_width
      end
