; ------------------------------------------------------------------------ 
; $Id: win.pro,v 1.4 2010/09/20 14:29:06 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  win, wdf, start, stop
;
;+
;  Limit the extent of a dataset.  The cursor is used to indicate the 
;  desired initial and finals values of the abscissa;  therefore, a curve
;  with relevant abscissa values must be plotted before a window command 
;  can be issued.  This command is similar to the LIM command, except the
;  cursor is used to designate the domain of the array.
;  Format:  WIN, WD1 [, START]  [,STOP]
;  where:
;    wd1 - WDF array number
;    start - initial abscissa value
;    stop - final abscissa value
;  Example:
;  Window array 1  using the cursor.
;    win, 1
;  Window arrays 2 through 10 to the same domain (requires two commands.)
;    win, 2, beg, last 
;    lim, 3+indgen(8), beg, last
;-
;
;     include the common block specification
;
@com.dat
;
common plot_grid_last, yeng_scale, xeng_scale
;
      on_error, 1

      if n_elements(yeng_scale) eq 0 then begin
        xeng_scale = 1.00
        yeng_scale = 1.00
      endif

      if n_elements(initwdf) eq 0 then wdfinit
      if n_elements(wdf) eq 0 then begin
        print, '% Format: WIN, WDF, Start_Value, Stop_Value'
        return
      end
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '% WIN: wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      nel = wdfnp(wdf-1)
      if nel lt 2 then begin
        print, '% WIN: No valid data in WDF array: ' , strtrim (wdf)
        return
      endif
      get_wdfarray, array, wdf
      print, 'Position cursor at first X-location'
      cursor, start, y1, 1
      start = start/xeng_scale
      print,  ' Start Value: ', strtrim(start, 2)
      print, 'Position cursor at second X-location'
      repeat cursor, stop, y1, 3 until start ne stop
      stop = stop/xeng_scale
      print,  ' End Value: ', strtrim(stop, 2)
      if start gt stop then begin
        temp = stop & stop = start & start = temp
      endif
      if start lt wdfminx(wdf-1) then start = wdfminx(wdf-1)
      if stop gt wdfmaxx(wdf-1) then stop = wdfmaxx(wdf-1)
      get_wdfarray, array, wdf, /xy
      np  = n_elements(array)
      if np eq nel+2 then begin
        if start gt wdfminx(wdf-1) then $
          ist = long((start-array(0))/array(1)) else ist = 0
        if stop lt wdfmaxx(wdf-1) then $
          iend = long((stop-array(0))/array(1) +0.99) else iend = nel-1
        n = 2+iend -ist
        if n lt 4 then begin
          print, '% WIN: Less than 3 points in the indicated interval'
          print, start, stop
          return
        endif
        array (0) = array(0)+ist*array(1)
        array(2:n)  = array(ist+2:iend+2)
        array = temporary(array (0:n))
        put_wdfarray, array, wdf, /no_copy
      endif else begin
        ok = where ((array(0:nel-1) ge start) and (array(0:nel-1) le stop), nok)
        if nok lt 2 then begin
          print, '% WIN: Less than 2 points in the interval'
          return
        endif
        array = [array(ok), array(ok+nel)]
        put_wdfarray, array, wdf, /xy, /no_copy
      endelse
      end
