; ------------------------------------------------------------------------ 
; $Id: film_exp.pro,v 1.3 2007/06/06 20:09:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro film_exp, wdf1, wdf2, response = response
;+
; Name:
;       FILM_EXP
;
; PURPOSE:
;       Convert film density to relative exposure using a particular
;       film response curve.
;
; Format:
;       FILM_EXP, STR1 [, STR2], RESPONSE = response
;  Where:
;       STR1     - Structure array number of film image in density
;       STR2     - Structure array number of film image in relative exposure 
;                  If str2 is undefined then str2 = str1
;       RESPONSE - Ignored since we only have one response curve.
;                  (D7)
; Example:
;       Convert structure 1 to relative exposure and store in structure 2
;         FILM_EXP, 1, 2
;
;; MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 9, 1997
;-
;
@qscom.dat
;  Common block
@qsstrcom.dat
;     return on an error
      on_error, 1
      if n_elements (structinit) eq 0 then qsstruct_init

; 8th order least square fit of exposure vs density.

 data_d7 = [-19.099869d0, 227.6170d0, -433.72269d0, 627.93408d0, $
      -493.67853d0, 226.34076d0, -59.886211d0, 8.4488449d0, $
     -0.49147660d0]

 domain_d7 =[0.154508, 3.99009]
 range_d7 = [7.76857, 452.150]

 
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'FILM_EXP, STR1, STR2,  [, RESPONSE = response]'
        return
      endif
      if nparams lt 1 then begin
        print,'% FILM_EXP must have at least 1 parameter'
        return
      endif
      if n_elements(wdf1) eq 0 then begin
        print, '% FILM_EXP: One parameter must be defined'
        return
      endif
      if n_elements(wdf2) eq 0 then wdf2 = wdf1

;
      if wdf1 lt 1 or wdf1 gt maxstructure then begin
        print, 'Structure array must be greater than 0 and </='+ $
          strtrim(maxstructure, 2)
        return
      endif
      save = get_save_str(wdf2, 0)
      if save eq 0 then begin
        print, '% FILM_EXP: Valid save array must be provided'
        return
      endif
      wdf2 = save
      s2i, a, wdf1
      ok1 = where (a.v1 le domain_d7(0), nok1)
      ok2 = where (a.v1 ge domain_d7(1), nok2)
      ok = where (a.v1 ge domain_d7(0) and a.v1 le domain_d7(1), nok)
      if nok gt 0 then begin
        v = double(a.v1(ok))
        a.v1(ok)= (((((((data_d7(8) *v + data_d7(7))*v + data_d7(6))*v $
                 + data_d7(5))*v + data_d7(4))*v + data_d7(3))*v $
                 + data_d7(2))*v + data_d7(1))*v + data_d7(0)
      endif
      if nok1 gt 0 then a.v1(ok1)= range_d7(0)
      if nok2 gt 0 then a.v1(ok2)= range_d7(1)
      i2s, a, save, /no_copy
      return
      end
