; ------------------------------------------------------------------------ 
; $Id: plotgrd.pro,v 1.15 2009/09/02 16:23:23 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro plotgrd, ain, xin, yin, zvaluein,index=index, color=color, thick = thick,  $
   xarray = xarray, yarray= yarray, overplot = overplot, polarplot = polar, $
   xrange = xrange, yrange = yrange, header= header, aspect = aspect, $
   showdefault = showdefault, setdefault = setdefault, unset = unset,$
   skip = skip, block = block, ignore = ignore, frame = frame, quiet = quiet, $
   xtitle = xtitle, ytitle= ytitle , title = title, linestyle = linestyle, $
   label_block= labelblock, lcharsize= lcharsize, lcolor = lcolor, $
   conductor = conductor, printfile= printfile, lextra = lextra, $
   fill = fill, fcolor = fcolor, no_grid = no_grid, _extra = extrastuff
;
;+
; NAME:
;       PLOTGRD
;
; PURPOSE:
;       Plot conputational grids or block boundaries.
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTGRD, A [, X] [, Y] [,ZVALUE] [, XRANGE = XRANGE] [, YRANGE = YRANGE]
;              [, INDEX = index] [, POLARPLOT = polarplot] [, QUIET = quiet]
;            [, THICK = THICK] [, COLOR = COLOR] [, TITLE = TITLE]
;            [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, SKIP= SKIP]
;            [, LINESTYLE = LINESTYLE] [, BLOCK = BLOCK] [, IGNORE = ignore]
;            [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, OVERPLOT = overplot]
;            [, CONDUCTOR = CONDUCTOR] [, PRINTFILE = PRINTFILE] [, /ASPECT]
;            [, HEADER = header] [, SETDEFAULT = setdefault] [, UNSET = unset]
;            [, FRAME = frame] [, SHOWDEFAULT = showdefault] 
;            [, LABEL_BLOCK = labelblock] [, LCHARSIZE = LCHARSIZE]
;            [, FILL = fill] [, NO_GRID = no_grid] [, FColor = FColor]
;            [, LCOLOR = lcolor] [, LEXTRA = lextra] [, _EXTRA = extrastuff]
; INPUTS:
;       A     IDL grid structure (type = 3 or 10 or 11)
;       X     Abscissa value (1, 2, or 3 depending on desired spatial array)
;       Y     Ordinate value (1, 2, or 3 depending on desired spatial array)
;       ZVALUE  For three dimensional data, value of plane in the third 
;               dimension. Default is them mean of the minimum and maximum.
;               If only 1 parameter other than A is provided, it is assumed to
;               be ZVALUE
;       INDEX   If set, zvalue refers to a zvalue index (1, 2, 3, 4, 5, ...)
;       Note: In the case of an overplot, the x and y values will be determined
;             from the previous plot.
; KEYWORD PARAMETERS:
;       OVERPLOT - If not zero, indicates no new grid is to be drawn.
;       THICK    - Indicates line thickness for the grid default is 0.6
;       ASPECT   - If set, the ordinate and abscissa will have the same 
;                  relative scale
;       POLARPLOT - If set, a polar plot will be generated.
;                 For the SPARE[0] eq 1, no polar plot is generated
;                 The coordinates for cylindrical and spherical data are as follows:
;                   R     - Radius vector
;                   PHI   - Angle between the radius vector projected on the X-Y 
;                           plane and the X axis
;                   Z     - Axis normal to the X-Y plane
;                   THETA - Angle between the Z axis and the Radius vector.
;                 For the SPARE[0] eq 2, the cylindrical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Phi, Z)
;                   X and Y must be 1 and 2 in any order, but they will be
;                     set to 1 and 2 respectively.
;                   The conversions are:
;                     X = R * Cos(Phi)
;                     Y = R * Sin(Phi)
;                 For the SPARE[0] eq 3, the spherical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Theta, Phi)
;                   The conversions to cartesian are:
;                     X = R * Cos(Phi) * Sin(Theta)
;                     Y = R * Sin(Phi) * Sin(Theta)
;                     Z = R * Cos(Theta)
;                   For (X, Y) equal to (1,3), Theta will be set and the abscissa 
;                     and ordinates will be X and Y, respectively.
;                   For (X, Y) equal to (1,2), PHI will be set.  The plot will be
;                     rotated to the slice value, and the abscissa and ordinates
;                     will be X and Z, respectively.  Because of the rotation, 
;                     (PHI = 0) in the equation for X above.  
;                 
;                   ASPECT will be set unless it is explicitly unset.
;                   Current version is for type 3 data only. (QS data)
;       QUIET     - If set, informational messages will not be printed.
;       LINESTYLE - Line type for the grid. An array may be specified to 
;                 sequence over linestyles.  Lines specified by
;                 LINESTYLE are: solid(0), dot(1), dash(2), dot-dash(3),
;                 dot-dot-dot-dash(4), and long dash(5).
;       BLOCK    - The block of data to be plotted. Default is all blocks
;       COLOR    - Indicates the color of the grid, Color may be an array 
;                  to sequence over colors.  If colors is an array, the
;                  minimum value is 1 and the maximum is !d.table_size
;                   The colors are: black(0), red(1), green(2), blue(3),
;                   purple(4), orange(5), light blue(6), yellow(7) and white(8).
;                  If grid is type 11 (pdb dataset) or type 10 (Exodus), then 
;                  the default color is an array [1, 2, 3, 4, 5, 6, 7] and the
;                  grid will be determined by the region value
;            Note: Color = 0 may not be set.  Color = 0 is the default plot color.
;                  Color = 9 is usually very close to 0 and my be used.
;       SKIP     - The number of grid lines plotted is reduced by this value.
;                  Note: If skip is very large, the blocks will be outlined.
;                        (see FRAME below)
;                  Currently, the first grid in a block is plotted and then
;                  every lines = n*skip where n are integers.
;       FRAME     - If present and not zero, will outline each block.
;                 Same as skip = a very big number
;       CONDUCTOR - Specifies a conductor structure or a structure array 
;                   number specifying a conductor structure.  If valid, the
;                   conductors will be overplotted. Colors and thicknesses
;                   can be set with setdefault.
;       IGNORE   - If the keyword IGNORE is set, then all previous parameters 
;                  on overlay plots will be ignored.
;       TITLE, XTITLE and YTITLE If grid is not zero, these will be used for
;                 axis and the plot labels.  If these are not string arrays, 
;                 the title, xtitle and ytitle used for the plots will be used.
;       XARRAY, YARRAY, are the grid arrays for the first block plotted
;       (X,Y)RANGE, STYLE, TYPE have the standard IDL meaning.
;       PRINTFILE- Specifies a printfile for printing the x and y grid lines 
;                  in the plot.  If PRINTFILE is not a sting, the information
;                  will be sent to the screen.
;            NOTE: PRINTFILE has not been implemented for POLARPLOT.
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;     LABEL_BLOCK - Label the block using XYOUTS
;                 Valid values are 1, 0, -1, -2
;                 1 - Label with "B #, P ##" where B and P are the block and 
;                     processor numbers respectively
;                     If the keyword is set and not equal a valid number, then
;                     LABEL_BLOCK = 1
;                 0 - No labels - DEFAULT
;                -1 - Block numbers only
;                -2 - Processor numbes only
;                 If PROC is a scalar equal to zero, only blocks will be labeled
;     LCHARSIZE - Charsize arguement to XYOUTS for block labels
;     LCOLOR    - Color arguement for XYOUTS for block labels
;     LEXTRA    - Any other keywords for XYOUTS as a structure
;     SETDEFAULT- If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to PLO
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 HEADER       1
;                 XRANGE       0
;                 YRANGE       0
;                 ASPECT       0
;
;       FILL     - Used with a pdb type grid.
;                  If a structure this is passed to POLYFILL
;                  (LINE_FILL, ORIENTATION, etc)
;       NO_GRID  _ If set, no grid will be drawn if fill is set.
;                  This is ignored if FILL is not set.
;       FColor   - Fill Color
;                  Default = [1, 2, 3, 4, 5, 6, 7]
;                  Material region number mod n_elements(FColor) is used for 
;                  the fill color index.
;     SHOWDEFAULT-Show the present default plot values.
;       _EXTRA    These parameters are sent to PLOT to generate the grid and 
;                 can have any value appropriate for the plot command including
;                 the following:
;                 (X,Y)RANGE, (X,Y)STYLE, (X,Y)TYPE, POSITION , etc
;                 and all have the standard IDL meaning.
; OUTPUTS:
;       Designated datasets are plotted.
; EXAMPLE:
;       Plot from structure g, space 1 vs space 3 with blue dashed lines
;       plotgrd, g, 3, 1, color = 3, linestyle = 2
;
;       Plot from structure g, space 1 vs space 3 with the fisrt line green 
;       and the next 4 red. Print the values of the grid lines to the terminal.
;         plotgrd, g, 3,1, color = [2,1,1,1,1], /print
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 25, 1993
;-
;
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat
;
common plotgrd_defaults, header_save, xrange_save, yrange_save, aspect_save
      on_error, 1
;
common plot_grid_last, yeng_scale, xeng_scale
;

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
;
      if keyword_set(unset) or n_elements(header_save) eq 0 then begin
        HEADER_SAVE = 1
        XRANGE_SAVE = 0
        YRANGE_SAVE = 0
        ASPECT_SAVE = 0
      endif
      if keyword_set (setdefault) then begin
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = xrange
        if n_elements(yrange) gt 0 then yrange_save = yrange
        if n_elements(aspect) gt 0 then aspect_save = aspect
      endif
      if keyword_set(showdefault) then begin
         print, '% PLOTGRD: Default parameters for the PLOTGRD command are:'
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
         print, '    ASPECT     = ' , strtrim (ASPECT_SAVE, 2)
      endif
      npar = n_params()
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) ) $
        then return
;
      if npar eq 0 then begin
        print,'% Format:', $
 ' PLOTGRD, STR [, X] [, Y] [, ZVALUE] [, XRANGE = XRANGE] [, YRANGE = YRANGE] '
print,'          [, THICK = THICK] [, COLOR = COLOR] [, TITLE = TITLE]'
print,'          [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, SKIP = skip]'
print,'          [, LINESTYLE = LINESTYLE] [, PRINTFILE = PRINTFILE]'
print,'          [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, BLOCK = BLOCK]'
print,'          [, /POLARPLOT] [, /ASPECT ]'
print,'          [, SETDEFAULT = setdefault] [, UNSET = unset]'
print,'          [, SHOWDEFAULT = showdefault] [HEADER = header]'
print,'          [, CONDUCTOR = CONDUCTOR] [, OVERPLOT = overplot]'
print,'          [, FRAME = frame] [, IGNORE = ignore] [_EXTRA = extrastuff]'
        return
      endif
;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, '% PLOTGRD: Structure must be defined'
        return
      endif
      if n_elements(xin) gt 0 then x = xin
      if n_elements(yin) gt 0 then y = yin
      if n_elements(zvaluein) gt 0 then zvalue = zvaluein
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
          print, '% PLOTGRD: First parameter must be a structure or an array number'
            return
          endif
        endif  else begin
        print, '% PLOTGRD: First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if (a.type ne 3 and a.type ne 10 and a.type ne 11) then begin
        print, '% PLOTGRD: Only Grid type structures may be plotted with PLOTGRD'
         help, a.type
        return
      endif
      
      if n_elements (ignore) eq 0 then ignore = 0
      if keyword_set(ignore) then ignore = 1 else ignore = 0
      coordinate = a.spare[0]
      if coordinate eq  1 then begin
        if keyword_set(polar) then begin
           print, '% PLOTGRD: Cannot do a polar plot with a cartesian grid'
           return
        endif
      endif
      if n_elements(overplot) eq 0 then overplot = 0
      if n_elements (xoldvalue)  eq 0 then begin
        xoldvalue = 0
        yoldvalue = 0
      endif
      if npar eq 2 then begin
        zvalue = temporary(x)
        if not keyword_set (overplot) and a.ndim eq 2 then begin
          x = 1
          y = 2
        endif
      endif
      if overplot ne 0 then begin
          if n_elements (x) eq 0 then x = xoldvalue
          if n_elements (y) eq 0 then y = yoldvalue
      endif
      nx = n_elements (x)
      if nx eq 0 then begin
        Print, '% PLOTGRD: X array must be defined'
        return
      endif
      xt = x[0]
      if xt lt 1 or xt gt 3 then begin
        print, '% PLOTGRD:  Abscissa must be one of the three dimensions not =', xt
        return
      endif
      if overplot ne 0 and xt ne xoldvalue  and ignore eq 0 then begin
          Print, '% PLOTGRD: WARNING: This overplot requested with X-axis of ', x, $
              ', present value is', xoldvalue
          print, '           Requested data will be plotted'
          hak, mes = ' Hit any key to continue'
      endif
      ny = n_elements (y)
      if ny eq 0 then begin
        Print, '% PLOTGRD: Y array must be defined'
        return
      endif
      yt = y[0]
      if yt lt 1 or yt gt 3 then begin
        print, '% PLOTGRD:  Ordinate must be one of the three dimensions not =', yt
        return
      endif
      if overplot ne 0 and yt ne yoldvalue and ignore eq 0 then begin
         print, '% PLOTGRD: WARNING: This overplot requested with Y-axis of ', y, $
              ', present value is', yoldvalue
         print, '           Requested data will be plotted'
         hak, mes = ' Hit any key to continue'
      endif
      lcblock = 1
      if n_elements (block) eq 0 then block = 0 else lcblock = 0
      if n_elements (block) gt 1 then begin
        good = where (block gt 0 and block le a.block,i)
        if i lt 1 then begin
          print, '% PLOTGRD:  No valid blocks specified for PLOTGRD'
          return
        endif
        blockarray = block(good)
      endif else begin
        if block lt 0 or block gt a.block then block = 0
        if block gt 0 then ts = block else ts = 1
        if block gt 0 then tf = block else tf = a.block
        blockarray = indgen(tf-ts+1)+ts
      endelse
      if n_elements(polar) eq 0 then  begin
        if overplot ne 0 then polar = polarplot else polar=0
      endif
      if !x.crange(0) eq !x.crange(1) then overplot= 0
      xoldvalue = xt
      yoldvalue = yt
;
;     check for labels
;
      if keyword_set (labelblock) then begin
        if labelblock eq -1 then lclabel = 1 else $
          if labelblock eq -2 then lclabel = 2 else lclabel = 3
        if n_elements(a.proc) lt a.block then (lclabel= lclabel < 1)
        if n_elements(lcharsize) gt 0 then begin
          if !p.charsize eq 0.0 then lchars= lcharsize[0] else $
            lchars =  lcharsize[0]*!p.charsize
        endif
        if n_elements(lcolor) gt 0 then lcolor=lcolor[0] else lcolor=!p.color
      endif else lclabel = 0
      zplaneold = 6-xoldvalue-yoldvalue
      if a.type eq 3 then begin
        if overplot gt 0 and ignore eq 0 then begin
          if n_elements(zvalueold) eq 0 then begin
            zvalueold = total(a.range(0:1,zplaneold-1,0))/2.
            print, '% PLOTGRD:  Plane for the grid taken at dimension', zplaneold
            print, '    equal to', zvalueold
            print, '            Requested data will be plotted'
          endif 
        endif
        if n_elements (zvalue) eq 0 then begin
          if overplot gt 0 then zvalue = total(zvalueold)/n_elements(zvalueold) $
          else begin
            zvalueold = total(a.range(0:1,zplaneold-1,0))/2.
            print, '% PLOTGRD:  Plane for the grid taken at', zvalueold
            zvalue = zvalueold
          endelse
        endif
        if n_elements (zvalue) gt 1 then begin
          zvalue =float( [min(zvalue, max = mv), mv])
          zvalue(0) = max([zvalue(0), a.range(0, zplaneold-1, 0)])
          zvalue(1) = min([zvalue(1), a.range(1, zplaneold-1, 0)])
          zvalue=total(zvalue)/2.0
        endif
        if zvalue ne total(zvalueold)/n_elements(zvalueold) $
                and overplot ne 0 and ignore eq 0 then begin
          Print, '% PLOTGRD:  WARNING: This overplot requested with value of axis',$
                   ' of', zvalue
          Print, '  Old plot was on plane', zplaneold,' with value of', $
                   zvalueold
          print, ' Hit "q" to quit; hit any other key to continue'
          key =  (byte (get_kbrd(1)))(0)
          if ((key eq 81) or (key eq 113)) then return
        endif else zvalueold = zvalue
;
        ok = where (zvalue ge a.range(0, zplaneold-1, blockarray) and $
                  zvalue le a.range(1, zplaneold-1, blockarray), nblo)
        if nblo eq 0 then begin
          print, '% PLOTGRD:  None of the specifed blocks exist at a dimension',$
             zplaneold, ' value of' , zvalue
          return
        endif else blockarray = blockarray(ok)
      endif else begin
        if n_elements(zvalue) eq 0 then begin
            zvalue = zvalueold
            index = 0
        endif
      endelse
;
      if (a.type eq 10  or a.type eq 11) then polar = 0
      if keyword_set(polar) then begin
        if ((xt+yt) ne 3)  and (coordinate eq 2) then begin
          print, '% PLOTGRD: Cylindrical data:'
          print, '           X and Y values must be 1 and 2 for a polar plot'
          return
        endif else if  ((xt+yt) gt 4)  and (coordinate eq 3) then begin
          print, '% PLOTGRD: Spherical data:'
          print, '           X and Y values must be (1,2) or (1,3) for a polar plot'
          return
        endif 
        if coordinate eq 2 then begin
          xt = 1 & yt = 2
        endif else xt = min([xt, yt], max = yt)
        if n_elements(aspect) eq 0 then aspect = 1
      endif 
      polarplot = polar
      if overplot eq 0 or a.type eq 10 or a.type eq 11 then begin
        if a.type eq 3 then begin
          xx = get_array (a, xt, blockarray(0), lab = xlab)
          yy = get_array (a, yt, blockarray(0), lab = ylab)
        endif else if a.type eq 10 or a.type eq 11 then begin
          timlab = ''
          xlab = (xt eq 1) ? a.x1lab : a.x2lab
          ylab = (yt eq 1) ? a.x1lab : a.x2lab
          if a.ntime eq 1 then begin
;           All 3D data of type 10 has ntime = 1
            if a.ndim lt 3 then begin
              xx = (xt eq 1) ? a.x1 : a.x2
              yy = (yt eq 1) ? a.x1 : a.x2
              slice = a.x4[0]
;           We have only one time so assume labeled 
            endif else begin
              zvalueold = zvalue
              
              if xt eq 1 then xlab = a.x1lab else $
                if xt eq 2 then xlab = a.x2lab else $
                if xt eq 3 then xlab = a.x3lab
              if xt eq 1 then xx = a.x1 else $
                if xt eq 2 then xx = a.x2 else $
                if xt eq 3 then xx = a.x3
              if yt eq 1 then ylab = a.x1lab else $ 
                if yt eq 2 then ylab = a.x2lab else $
                if yt eq 3 then ylab = a.x3lab
              if yt eq 1 then yy = a.x1 else $ 
                if yt eq 2 then yy = a.x2 else $
                if yt eq 3 then yy = a.x3
              if zplaneold eq 1 then zz = a.x1 else $
                if zplaneold eq 2 then zz = a.x2 else $
                if zplaneold eq 3 then zz = a.x3
            endelse
          endif else begin
            if keyword_set (index) then begin
              time = round (zvalue)
              if time lt 1 or time gt a.ntime then begin
                print, '% PLOTGRD: Time index must be in the range: (1, ' + $
                  strtrim(a.ntime, 2)+')'
                return
              endif
              tim = time -1
              xx = (xt eq 1) ? reform(a.x1[*, tim]): reform(a.x2[*, tim])
              yy = (yt eq 1) ? reform(a.x1[*, tim]): reform(a.x2[*, tim])
              slice = a.x4(tim)
              timlab = '; Time: '+strtrim(slice, 2)
              zvalueold = slice
            endif else begin
              nt = a.ntime
              slice = zvalue
              time = zvalue
;           set a error of about 20*(point spacing) 
;           (should be nt-1 but who cares)
              eps = (a.x4(nt-1)-a.x4(0))/(nt * 20)
              if (time lt a.x4(0)-eps) or (time gt a.x4(nt-1)+eps) then begin
          print, '% PLOTGRD: Time outside the range of the data: ',strtrim(time,2)
          print,  nt, ' Data values in the range: ', a.x4(0), ' to', a.x4(nt-1)
                return
              endif
              if time ge a.x4(nt-1) then begin
                time = a.x4(nt-1)
                xx = (xt eq 1) ? reform(a.x1[*, nt-1]): reform(a.x2[*, nt-1])
                yy = (yt eq 1) ? reform(a.x1[*, nt-1]): reform(a.x2[*, nt-1])
              endif else if time le a.x4(0) then begin
                time = a.x4(0)
                xx = (xt eq 1) ? reform(a.x1[*, 0]): reform(a.x2[*, 0])
                yy = (yt eq 1) ? reform(a.x1[*, 0]): reform(a.x2[*, 0])
              endif else begin
                mint = min(abs(time-a.x4), ii)
                if ii eq nt -1 then  ii=ii-1
                if time ge a.x4(ii) then begin
                  tim = [ii, ii+1]
                  frac = [time-a.x4(ii), a.x4(ii+1)-time]/(a.x4(ii+1) - a.x4(ii))
                endif else begin
                  tim = [ii-1, ii]
                  frac = [time-a.x4(ii-1), a.x4(ii)-time]/(a.x4(ii) - a.x4(ii-1))
                endelse
                xx =(xt eq 1) ? reform(a.x1(*, tim(0))*frac(1)+a.x1(*, tim(1))*frac(0)): $
                   reform(a.x2[*, tim(0)]*frac(1)+a.x2(*, tim(1))*frac(0))
                yy =(yt eq 1) ? reform(a.x1[*, tim(0)]*frac(1)+a.x1[*, tim(1)]*frac(0)): $
                   reform(a.x2[*, tim(0)]*frac(1)+a.x2(*, tim(1))*frac(0))
              endelse
              timlab = '; Time: '+strtrim(time, 2)
            endelse
          endelse
          nblo  = a.block
          for i = 1, nblo do begin
            loc = a.locx(a.locv(i-1):a.locv(i))
;           nloc will be the index or one less than the number of values
;           this is only used as an index
            nloc = n_elements(loc)-1
            a.range(0, xt-1, i) = min(xx(a.connect(loc(0):loc(nloc)-1)), $
                    max = mm)
            a.range(1, xt-1, i) = mm
            a.range(0, yt-1, i) = min(yy(a.connect(loc(0):loc(nloc)-1)), $
                    max = mm)
            a.range(1, yt-1, i) = mm
          endfor
          a.range(0, xt-1, 0) = min(a.range(0, xt-1, 1:nblo))
          a.range(1, xt-1, 0) = max(a.range(1, xt-1, 1:nblo))
          a.range(0, yt-1, 0) = min(a.range(0, yt-1, 1:nblo))
          a.range(1, yt-1, 0) = max(a.range(1, yt-1, 1:nblo))
;          if xt eq 2 then begin
;            temp = temporary(xx)
;            xx = temporary(yy)
;            yy = temporary(temp)
;          endif
        endif
        tit = 'GRID ' + a.clab 
        if a.type eq 10  or a.type eq 11 then tit = tit+timlab
        
        if n_elements (xtitle) eq 0 then xtitle = 0
        b = size (xtitle)
        if b(b(0)+1) ne 7 then xtitle = xlab else xlab = xtitle
        if n_elements (ytitle) eq 0 then ytitle = 0
        b = size (ytitle)
        if b(b(0)+1) ne 7 then ytitle = ylab else ylab = ytitle
        if n_elements (title) eq 0 then title = 0
        b = size (title)
        if b(b(0)+1) ne 7 then title = tit  else tit = title
;
        if n_elements(header) eq 0 then header = header_save
        if overplot ne 0 then header = 0
        if !p.multi(0) ne 0 then header = 0
        if header ne 0 then begin
          filename = strtrim(a.file, 2)
          if (strlen(filename) eq 0) then header_text='Plot: ' + thin_date() $
          else  begin
            ss = file_info(filename)
            if ss.exists eq 1 then begin
              cdate = thin_date(ss.ctime)
              header_text= ['Plot: '+thin_date() +'; Data: ' + cdate,  $
                     'File: ' +filename]
            endif else header_text=['Plot: ' + thin_date(), 'File: ' +filename]
          endelse
        endif
        if n_elements(xrange) eq 0 then xrange = xrange_save
        if n_elements(yrange) eq 0 then yrange = yrange_save
        if keyword_set(polar) then begin
          ssize = make_array(2, nblo, /long)
          ssize(0,*)= a.size(0, blockarray-1)
          ssize(1,*)= a.size(yt-1, blockarray-1)
          nn = ssize(0, *)*ssize(1, *)
          totnn = long(total(nn))
          xgrid = make_array(totnn)
          ygrid = make_array(totnn)
          loc = make_array(/long, val=0L, nblo+1)
          px = 0
          for ii = 0, nblo-1 do begin
            iblo = blockarray(ii)
            xx = get_array(a,1,iblo) 
            if coordinate eq 2 then begin
;             Cylindrical
              yy = get_array(a,yt, iblo) 
              nxy = nn(ii)
              cy = cos(yy)
              sy = sin(yy)
            endif else begin
;             coordinate eq 3 - Spherical
              nxy = nn(ii)
              if yt eq 3 then begin
                yy = get_array(a, 3, iblo)
                sth = sin(zvalue)
                sy = sin(yy)*sth
                cy = cos(yy)*sth
              endif else begin
                zz = get_array(a, 2, iblo)
                sy = cos(zz)
                cy = sin(zz)
              endelse
            endelse
            ygrid(px:px+nxy-1) = reform(xx # sy, nxy, /overwrite)
            xgrid(px:px+nxy-1) = reform(xx # cy, nxy, /overwrite)
            px = px + nxy
            loc(ii+1) = loc(ii)+nxy
          endfor
          xmin = min(xgrid, max = xmax)
          ymin = min(ygrid, max = ymax)
        endif else begin
          xmin = min(a.range(0, xt-1, blockarray))
          xmax = max(a.range(1, xt-1, blockarray))
          ymin = min(a.range(0, yt-1, blockarray))
          ymax = max(a.range(1, yt-1, blockarray))
        endelse
        if n_elements(xrange) lt 2 then begin 
          xrange = [xmin, xmax]
          lcxrange = 0
        endif else begin
          xrange = xrange(0:1)
          lcxrange = 1
        endelse
        if n_elements(yrange) lt 2 then begin
          yrange = [ymin, ymax]
          lcyrange = 0
        endif else begin
          yrange = yrange(0:1)
          lcyrange = 1
        endelse
        
        if ((lcyrange  ne 0) xor (lcxrange ne 0)) then begin
          if   keyword_set(polar) then begin
            if lcyrange eq 0 then begin
              ok = where (xgrid ge xrange(0) and  xgrid le xrange(1), nok)
              if nok eq 0 then begin
                print, '% PLOTGRD: No plots exist form this range of X'
                return
              endif
              yrange(0) = min(ygrid(ok), max = ymax)
              yrange(1) = ymax
            endif else begin
              ok = where (ygrid ge yrange(0) and  ygrid le yrange(1), nok)
              if nok eq 0 then begin
                print, '% PLOTGRD: No plots exist form this range of Y'
                return
              endif
              xrange(0) = min(xgrid(ok), max = xmax)
              xrange(1) = xmax
            endelse
          endif else begin
            ok = where (a.range(0, xt-1, blockarray) le xrange(1) and $
                  a.range(1, xt-1, blockarray) ge xrange(0), nok)
            if nok gt 0 then begin
              iblo = blockarray(ok)
              ok = where (a.range(0, yt-1, iblo) le yrange(1) and $
                  a.range(1, yt-1, iblo) ge yrange(0), nok)
            endif else begin
              print, '% PLOTGRD:  No grid lines exist for the specified plot range'
              return
            endelse
            if nok eq 0 then begin
              print, '% PLOTGRD:  No grid lines exist for the specified plot range'
              return
            endif
            iblo = iblo(ok)
            if lcxrange eq 0 then $
              xrange=[ min(a.range(0, xt-1, iblo)), max(a.range(1, xt-1, iblo))]
            if lcyrange eq 0 then $
              yrange=[ min(a.range(0, yt-1, iblo)), max(a.range(1, yt-1, iblo))]
          endelse
        endif
;       Add a small delta to try to avoid precision problems with PFF which cause
;       strange axis scaling.
 
        if (lcxrange eq 0) and (not keyword_set(aspect)) then begin
          epsil = (xrange(1)-xrange(0))/32768.
          xrange=[xrange(0)+epsil , xrange(1)-epsil]
        endif
        if (lcyrange eq 0) and (not keyword_set(aspect)) then begin
          epsil = (yrange(1)-yrange(0))/32768.
          yrange=[yrange(0)+epsil , yrange(1)-epsil]
        endif
;       see if we have a horizontal or vertical line
        if xrange(1) le xrange (0) then xrange = [.98, 1.02]*xrange(0)
        if yrange(1) le yrange (0) then yrange = [.98, 1.02]*yrange(0)
;
        yeng_scale = 1.00  &  xeng_scale = 1.00
        if n_elements(xstyle) eq 0 then xstyle =0
        if n_elements(ystyle) eq 0 then ystyle =0
        if n_elements(xtype) eq 0 then xtype =0
        if n_elements(ytype) eq 0 then ytype =0
        if not keyword_set(overplot) then begin
          if keyword_set(aspect) then begin
            xstyle = (fix(xstyle) or 1)
            ystyle = (fix(ystyle) or 1)
            pos = get_aspect(xrange, yrange)
            plot, xx, yy, xrange = xrange, yrange = yrange, ystyle = ystyle, $
            xstyle = xstyle, ytype = ytype, xtype = xtype, title = title, $
            xtitle = xtitle, ytitle = ytitle, /nodata,position = pos, $
             _extra= extrastuff
          endif else $
          plot, xx, yy, xrange = xrange, yrange = yrange, ystyle = ystyle, $
            xstyle = xstyle, ytype = ytype, xtype = xtype, title = title, $
            xtitle = xtitle, ytitle = ytitle, /nodata, _extra= extrastuff
        endif
;
        if header ne 0 then header, header_text
        xarray = xx
        yarray = yy
        lctwodim = 0
      endif else if keyword_set(polar) then begin
          ssize = make_array(2, nblo, /long)
          ssize(0,*)= a.size(0, blockarray-1)
          ssize(1,*)= a.size(yt-1, blockarray-1)
          nn = ssize(0, *)*ssize(1, *)
          totnn = long(total(nn))
          xgrid = make_array(totnn)
          ygrid = make_array(totnn)
          loc = make_array(/long, val=0L, nblo+1)
          px = 0
          for ii = 0, nblo-1 do begin
            iblo = blockarray(ii)
            xx = get_array(a,1,iblo)
            if coordinate eq 2 then begin
;             Cylindrical
              yy = get_array(a,yt, iblo)
              nxy = nn(ii)
              cy = cos(yy)
              sy = sin(yy)
            endif else begin
;             coordinate eq 3 - Spherical
              nxy = nn(ii)
              if yt eq 3 then begin
                yy = get_array(a, 3, iblo)
                sth = sin(zvalue)
                sy = sin(yy)*sth
                cy = cos(yy)*sth
              endif else begin
                zz = get_array(a, 2, iblo)
                sy = cos(zz)
                cy = sin(zz)
              endelse
            endelse
            ygrid(px:px+nxy-1) = reform(xx # sy, nxy, /overwrite)
            xgrid(px:px+nxy-1) = reform(xx # cy, nxy, /overwrite)
            px = px + nxy
            loc(ii+1) = loc(ii)+nxy
          endfor
      endif
      xrang = !x.crange
      yrang = !y.crange
      if n_elements (color) eq 0 then begin
        if a.type eq 11 or a.type eq 10 then begin
          if keyword_set(fill) then color = !p.color $ 
             else color =[1, 2, 3, 4, 5, 6, 7] 
        endif else begin
          if n_elements (grdcolor) gt 0 then color = grdcolor else color =0
        endelse
      endif

      if min(color) lt 0 or max(color) gt !d.table_size then begin
        color =!p.color
        print, '% PLOTGRD: Color must be ge 0 and le ' , !d.table_size
      endif
      ok = where (color eq 0, nok)
      if nok gt 0 then color(ok) = !p.color
      if min(color) lt 8 then loadxyct, /quiet, /freeze
      ncolor = n_elements (color)
      if n_elements (linestyle) eq 0 then linestyle = 0
      if min(linestyle) lt 0 or max(linestyle) gt 5 then begin
        linestyle = 0
        print, '% PLOTGRD: Linestyle must be ge 0 and less than /equal to 5'
      endif
      nline = n_elements (linestyle)
      if n_elements (printfile) eq 0  or keyword_set (polar) then begin
        unit = -1
      endif else begin
        ps = size (printfile(0))
        if ps(1) eq 7 then openw, unit, printfile, /get $
        else unit = 0
      endelse
      if n_elements (thick) eq 0 then if n_elements(grdthick) gt 0 then $
         thick = (!p.thick>1) *grdthick else thick = (!p.thick>1)*0.6
      if n_elements (skip) eq 0 then skip = 1 else skip = long(abs(skip)>1)
      for ii = 0, n_elements (blockarray) -1 do begin
        iblo = blockarray(ii)
        if a.type eq 3 then begin
          errx = max (abs(xrang))*6e-5
          erry = max (abs(yrang))*6e-5
          if keyword_set (polar) then begin
            xx = reform(xgrid(loc(ii):loc(ii+1)-1), ssize(0, ii), ssize(yt-1, ii))
            yy = reform(ygrid(loc(ii):loc(ii+1)-1), ssize(0, ii), ssize(yt-1, ii))
            nx = ssize(0, ii)
            ny = ssize(yt-1, ii)
            if keyword_set(frame) then begin
              skipx = nx-1
              skipy = ny-1
            endif else begin
              skipx = skip < nx-1
              skipy = skip < ny-1
            endelse
            for i = 0, nx-1, (skipx >1) do begin
              col = color(i mod ncolor)
              lin = linestyle(i mod nline)
              if col eq 0 then oplot, xx(i,*), yy(i,*), linestyle = lin, $
                thick = thick else oplot, xx(i,*), yy(i,*), linestyle = lin, $
                thick = thick, color = col
            endfor
            for i = 0, ny-1, (skipy >1) do begin
              col = color(i mod ncolor)
              lin = linestyle(i mod nline)
              if col eq 0 then oplot, xx(*,i), yy(*,i), linestyle = lin, $
                thick = thick else oplot, xx(*,i), yy(*,i), linestyle = lin, $
                thick = thick, color = col
            endfor
            if keyword_set(lclabel) then begin
              lx = (max(xx, min = m)+m)*0.5
              ly = (max(yy, min = m)+m)*0.5
              lb = 'B '+strtrim(iblo, 2)
              if lclabel eq 1 then str = lb else begin 
                lp = 'P '+strtrim(a.proc[iblo-1], 2)
                if lclabel eq 2 then str = lp else str = lb+', '+lp
              endelse
              xyouts, lx, ly, str, align = 0.5, lcolor = lcolor,  $
                      charsize = lchars, _extra = lextra
            endif
          endif else begin
            if a.range(1,xt-1,iblo) lt xrang(0)+errx or $
              a.range(0,xt-1,iblo) gt xrang(1)-errx or $
              a.range(1,yt-1,iblo) lt yrang(0)+erry or $
              a.range(0,yt-1,iblo) gt yrang(1)-erry then begin
              if not keyword_set(quiet) then $
                print, '% PLOTGRD:  No GRIDS for block', iblo 
            endif else begin
              xx = get_array (a, xt, iblo, lab = xlab)
              nxx = n_elements (xx)
              if keyword_set(frame) or skip ge nxx then begin
                xx = [xx(0), xx(nxx-1)]
                nxx = 2
              endif
              ix = where (xx ge xrang(0)-errx and xx le xrang(1)+errx, nx)
              if nx eq 0 then begin
                ix = indgen(2) 
                if nxx gt 2 then xx = [xx(0), xx(nxx-1)]
                nxx = 2
                nx = 2
              endif
              skipx = skip < (nx-1)
              yy = get_array (a, yt, iblo, lab = ylab)
              nyy = n_elements (yy)
              if keyword_set(frame) or skip ge nyy then begin
                yy = [yy(0), yy(nyy-1)]
                nyy = 2
              endif
              iy = where (yy ge yrang(0)-erry and yy le yrang(1)+erry, ny)
              if ny eq 0 then begin
                iy = indgen(2)
                if nyy gt 2 then yy = [yy(0), yy(nyy-1)]
                nyy = 2
                ny = 2
              endif
              skipy = skip < (ny-1)
              if keyword_set(lclabel) then begin
                lx = max(xx, min = m) < xrang[1]
                m = m > xrang[0]
                lx= (m+lx)*0.5
              endif
              xx = xx(ix)
;           if keyword_set(frame) then skipx = nxx-1 else skipx = skip < (nx-1)
              skipx = skipx > 1
              ylim = a.range(*, yt-1, iblo)
              for i = 0L, nx-1, skipx do begin
                j= ix(i)
                col = color(j mod ncolor)
                lin = linestyle (j mod nline)
                if col eq 0 then oplot, [xx(i), xx(i)], ylim, $
                  linestyle = lin, thick = thick else oplot, [xx(i), xx(i)], $
                  ylim, linestyle = lin, thick = thick, color = col
              endfor
              if unit gt 0 then begin
                printf, unit, ' Block', fix(iblo)
                printf, unit, '  Grid Line for X axis - ', xlab
                xx = xx (indgen(nx/skipx)*skipx)
                jump = 5*skipx
                nrow = fix(nx/jump)
                for i = 0, nrow-1 do printf, unit, ix(i*jump), xx(i*5 : 5*i+4)
                j = fix (nx/skipx) mod 5
                if j ne 0 then printf, unit, ix(nrow*jump), xx(nrow*5: nrow*5+j-1)
              endif else if unit eq 0 then begin
                print, ' Block', fix(iblo)
                print,  '  Grid Line for X axis - ', xlab
                xx = xx (indgen(nx/skipx)*skipx)
                jump = 5*skipx
                nrow = fix(nx/jump)
                for i = 0, nrow-1 do print, ix(i*jump), xx(i*5 : 5*i+4)
                j = fix (nx/skipx) mod 5
                if j ne 0 then print, ix(nrow*jump), xx(nrow*5: nrow*5+j-1)
              endif
              if keyword_set(lclabel) then begin
                ly = max(yy, min = m) < yrang[1]
                m = m > yrang[0]
                ly= (m+ly)*0.5
              endif
              yy = yy(iy)
;           if keyword_set(frame) then skipy = nyy-1 else skipy = skip < (nyy-1)
              skipy = skipy > 1
              xlim = a.range(*, xt-1, iblo)
              for i = 0L, ny-1, skipy do begin
                j= iy(i)
                col = color(j mod ncolor)
                lin = linestyle (j mod nline)
                if col eq 0 then oplot, xlim, [yy(i), yy(i)], $
                  linestyle = lin, thick = thick else oplot, xlim, $
                  [yy(i), yy(i)], linestyle = lin, thick = thick, color = col
              endfor
              if unit gt 0 then begin
                printf, unit, ' Grid Line for Y axis - ', ylab
                yy = yy (indgen(ny/skipy)*skipy)
                jump = 5*skipy
                nrow = fix(ny/jump)
                for i = 0, nrow-1 do printf, unit, iy(i*jump), yy(i*5 : 5*i+4)
                j = fix (ny/skipy) mod 5
                if j ne 0 then printf, unit, iy(nrow*jump), yy(nrow*5: nrow*5+j-1)
              endif else if unit eq 0 then begin
                print,  ' Grid Line for Y axis - ', ylab
                yy = yy (indgen(ny/skipy)*skipy)
                jump = 5*skipy
                nrow = fix(ny/jump)
                for i = 0, nrow-1 do print, iy(i*jump), yy(i*5 : 5*i+4)
                j = fix (ny/skipy) mod 5
                if j ne 0 then print, iy(nrow*jump), yy(nrow*5: nrow*5+j-1)
              endif
              if keyword_set(lclabel) then begin
                lb = 'B '+strtrim(iblo, 2)
                if lclabel eq 1 then str = lb else begin
                  lp = 'P '+strtrim(a.proc[iblo-1], 2)
                  if lclabel eq 2 then str = lp else str = lb+', '+lp
                endelse
                xyouts, lx, ly, str, align = 0.5, color = lcolor,  $
                        charsize = lchars, _extra = lextra
              endif
            endelse
          endelse
        endif else begin
          loc = a.locx(a.locv(iblo-1):a.locv(iblo))
          nloc = n_elements(loc)-1
          if a.type eq 11 then begin
            if keyword_set (fill) then begin
              if n_elements (fcolor) eq 0 then fcolor = indgen(7)+1
              nfc = n_elements(fcolor)
              ok = where (fcolor eq 0, nok)
              if nok gt 0 then fcolor(ok) = !p.color
              if size(fill, /type) eq 8 then fill_extra = fill
              if nfc gt 1 then fc = fcolor[(a.region-1) mod nfc] $
              else fc = make_array(nloc, value = fcolor[0])

              for i =  0L, nloc-1 do begin
                xtmp = [xx(a.connect(loc(i):loc(i+1)-1)), xx(a.connect(loc(i)))]
                ytmp = [yy(a.connect(loc(i):loc(i+1)-1)), yy(a.connect(loc(i)))]
                polyfill, xtmp, ytmp, color= fc[i], noclip=0,  _extra = fill_extra
              endfor
              fc = 0
            endif 
            if not (keyword_set(fill) && keyword_set(no_grid)) then begin
              nc = n_elements(color)
              if nc gt 1 then fc = color[(a.region-1) mod nc] $
              else fc = make_array(nloc, value = color[0])
              
              for i = 0L, nloc-1 do begin
                xtmp = [xx(a.connect(loc(i):loc(i+1)-1)), xx(a.connect(loc(i)))]
                ytmp = [yy(a.connect(loc(i):loc(i+1)-1)), yy(a.connect(loc(i)))]
                lin = linestyle (ii  mod nline)
                oplot, xtmp, ytmp, linestyle = lin, thick = thick, color = fc[i]
              endfor
            endif

          endif else begin
;           type 10 
;            help, zplaneold , zvalueold
;            print, ' zz ', zvalueold, max(zz, min = m), m
;            print, ' xx', max(xx, min = m), m
;            print, ' yy', max(yy, min = m), m
            if a.ndim eq 3 then begin
              if (a.range[0, xt -1, iblo] le !x.crange[1]) && $
                 (a.range[1, xt -1, iblo] ge !x.crange[0]) && $
                 (a.range[0, yt -1, iblo] le !y.crange[1]) && $
                 (a.range[1, yt -1, iblo] ge !y.crange[0]) && $
                 (a.range[0, zplaneold -1, iblo] le zvalueold) && $
                 (a.range[1, zplaneold -1, iblo] ge zvalueold) then begin
                for j= a.locv[iblo-1], a.locv[iblo]-1 do begin
                  zzt = zz[a.connect[a.locx[j]:a.locx[j+1]-1]]
                  if ((min(zzt, max = m) le zvalueold) && (m ge zvalueold)) then begin
                    xxt = xx[a.connect[a.locx[j]:a.locx[j+1]-1]]
                    yyt = yy[a.connect[a.locx[j]:a.locx[j+1]-1]]
                    n = n_elements(xxt)
                    col = color[ii mod ncolor]
                    plots, total(xxt)/n, total(yyt)/n, noclip = 0, psym = 8, $
                      color= col, _extra = extrastuff
                  endif
                endfor
              endif
            endif else begin
              for i = 0L, nloc-1 do begin
                
                xtmp = [xx[a.connect[loc[i]:loc[i+1]-1]], xx[a.connect[loc[i]]]]
                ytmp = [yy[a.connect[loc[i]:loc[i+1]-1]], yy[a.connect[loc[i]]]]
                col = color(ii mod ncolor)
                lin = linestyle (ii  mod nline)
                if col eq 0 then oplot, xtmp, ytmp, $
                  linestyle = lin, thick = thick else oplot, xtmp, ytmp, $
                  linestyle = lin, thick = thick, color = col
              endfor
            endelse
          endelse
        endelse
      endfor
;
      if unit gt 0 then free_lun, unit
      if n_elements (conductor) gt 0 then $
        plotcon, conductor, /over
      end
