; ------------------------------------------------------------------------ 
; $Id: plotpar.pro,v 1.13 2011/07/12 16:55:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro plotpar, ain, x, y, wt, color =color, symsize=symsize, $
   xarray = xarray, yarray = yarray, farray = warray,w1 = window1, $
   xrange = xrange, yrange = yrange, xstyle = xstyle, xtype = xtype, $
   ystyle  = ystyle, ytype = ytype, fast=fast, skip = skipin,nparticle=npart, $
   changesign = change, psymbol =psymin, w2 = window2, w3 = window3, $
    w4 = window4, w5 = window5, w6 = window6, ignore = ignore, weight = weight, $
   conductor = conductor, grid = grid, bin = bin, follow= follow,  $
   xtitle = xtitle, ytitle= ytitle , title = title, overplot = overplot, $
   result = result,quiet = quiet, vector=vector, left = left, right = right, $
   showdefault = showdefault, setdefault = setdefault, unset = unset,$
   xbin = xbin, ybin = ybin, ngp = ngp, aspect = aspect, verbose = verbose, $
   zrange=zrange,thermometer= thermometer, polarplot = polar, zplane =zplane, $
   ptheta = theta, pphi = phi, ztype = ztype, c_range = c_range_in, $
   delta = delta, header= header, fill = fill, _extra = extrastuff
;
;+
; NAME:
;       PLOTPAR
;
; PURPOSE:
;       Plot particle arrays with color weighting
; CATEGORY:
;       Input/Output
; CALLING SEQUENCE:
;       PLOTPAR, A [, X] [, Y] [, WT] [, YRANGE = YRANGE] [, THICK = THICK]
;            [, XRANGE = XRANGE] [, COLOR = COLOR] [, TITLE = TITLE]
;            [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, CHANGESIGN = CHANGE]
;            [, FOLLOW = follow] [, DELTA = delta] [, SYMSIZE = SYMSIZE] 
;            [, PSYMBOL = psymbol] [, FAST = FAST] [, fill = fill] 
;            [, POLARPLOT = polarplot] [, PTHETA = theta] [, PPHI = phi]
;            [, SKIP= SKIP] [, NPARTICLE = nparticle] [, C_RANGE = crange]
;            [, W1= WINDOW1] [, W2= WINDOW2] [, W3= WINDOW3]
;            [, W4= WINDOW4] [, W5= WINDOW5] [, W6= WINDOW6] [, /ASPECT]
;            [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, FARRAY = FARRAY]
;            [, OVERPLOT = OVERPLOT]  [, IGNORE = ignore]
;            [,CONDUCTOR = CONDUCTOR] [, GRID = GRID]
;            [, RESULT = result] [, QUIET= quiet] [, VECTOR = vector]
;            [, HEADER = header] [, SETDEFAULT = setdefault] [, UNSET = unset]
;            [, BIN = bin] [, XBIN = xbin] [, YBIN = ybin] [, NGP = ngp]
;            [, SHOWDEFAULT = showdefault] [, THERMOMETER = thermometer]
;            [, ZRANGE = zrange] [, ZTYPE = ztype] [, /LEFT] [, /RIGHT]
;            [, ZPLANE = zplane] [,_EXTRA = extrastuff]
; INPUTS:
;       A     IDL particle structure (type = 2)
;       X     Abscissa values (positive = x array, negative = attribute array)
;       Y     Ordinate values (positive = x array, negative = attribute array)
;       WT    Weight array  (positive = x array, negative = attribute array)
;             Note:  If OVERPLOT is set and Y is undefined, then WT = X
;             ZRANGE is used to scale WT
;             If log scaling, the absolute value of zrange is used and the log taken.
;             ZTYPE is used to select log scaling for WT
;             Note:  Keyword WEIGHT may be used to set WT.  If WT is present, 
;                    the keyword WEIGHT is ignored.
;     NOTE: If x, y, or wt is an array, then the square root of the sum of 
;             the squares will be used unless the value is identical
;             For example:
;             If X = [1, 2, 3] for a cartesian system, then the abscissa will
;                be the radius.  Abscissa is SQRT(A.X1^2 + A.X2^2 + A.X3^2)
;             If X = [1] then the abscissa is ABS(A.X1) 
;             if X = [-4] then the abscissa is ABS(A.V4) (the 4th attribute)
;     NOTE: If overplot is not zero, then x and y can be determined from the
;           previous plot if they not specified. WT will be the second parameter.
;
; KEYWORD PARAMETERS:
;       WEIGHT   - If WT is undefined, then WT = WEIGHT.  WT is defined above.
;       PSYMBOL  - Symbol type - Default is is a filled circle unless 
;                  Fast is set. If Fast is set, a triangle will be used.
;                  The symbol is defined by PSYMBOL: (1)plus sign, 2(asterisk),
;                  3 (period), 4 (diamond), 5 (triangle), 6 (square), 7 ("X").
;                  (If the symbol value is greater than 10 then ll will be
;                  subtracted from the value and MKSYMBOL will be called.)
;                  (0),Square; (1),Circle; (2),Triangle; (3),Diamond;
;                  (4),Big X; (5),Delta; (6), Star; (7),Plus; (8),Hexagon;
;                  (9),Pentagon; (10), Clover; (11),PACMAN; (12),Spiral
;                  Note: Symbols may also be specified as names:
;                    Follow ='sq' or 'ci' or 'diamond' or 'clover']
;                    These will be converted to numbers: 11 or 12 or 14 or 21]
;       FAST     - Indicates a fast plot is desired.  If PSYMBOL is not
;                  specified and fast is not zero, a triangle will be used.
;                  FILL is set to zero.
;       FILL     - If set filled symbols will be used if available.
;                  (Applies to symbols 11 and above)
;       OVERPLOT - If not zero, indicates no new grid is to be drawn.
;                  If OVERPLOT is set, it will have a value of +/- 1 on return.
;                  If not set, OVERPLOT will be have a value of zero on return.
;                  If THERMOMETER is set, then OVERPLOT is set to zero if
;                  OVERPLOT is greater than or equal to zero.
;       IGNORE   - If set and an overplot, ignore the previous values.
;                  If IGNORE is set, it will have a value of 1 on return.
;                  If not set, IGNORE will have a value of zero on return.
;       SYMSIZE: - Symbol size multiplier - Default is 0.5
;       ASPECT   - If set, the ordinate and abscissa will have the same 
;                  relative scaling.
;       COLOR    - If wt has fewer elements than the x and y arrays, then
;                   the points will be drawn using color = color. 
;                   The colors are: black(0), red(1), green(2), blue(3),
;                   purple(4), orange(5), light blue(6), yellow(7) and white(8).
;       C_RANGE:  Two element vector specifying the range of the color table to
;                 use for the WT scaling.
;                 First number is the lower value, and second is the upper
;                 value expressed as a fraction of full scale.
;                 If C_RANGE LE 1:
;                   NCOL = !d.table_size -11
;                   (For true color devices, !d.table_size = 256)
;                 If C_RANGE GT 1:
;                 Elements of the color table can be specified.
;                 The smallest number should be greater than 8 and the larger
;                 number should be less than (!d.table_size-1)
;                 If not they are not in this this range, they will be changed.
;                 DEFAULT: C_RANGE=[0,1] = [9, !d.table_size-2]
;       CHANGESIGN - If change is present and not zero, wt = -wt
;       ZRANGE   - Range of WT scaling - This must be a two element vector.
;                  ZRANGE = [ WTmin, WTmax ] 
;       ZTYPE    - If set, use log scaling for WT.
;                  If negative, then 
;                    M = max(wt)
;                    wt = log (wt > abs(ztype)*M)
;                  If ZTYPE gt 0 then ZTYPE = -(1e-4)
;       THERMOMETER - If set a thermometer will be generated.
;                     If a structure, then it will be passed to thermometer 
;                      routine.
;                 Note: Used only for a weighted particle plot.
;                 Note: If THERMOMETER is set, then OVERPLOT is set to zero unless
;                       unless OVERPLOT = -1
;                       IF OVERPLOT is -1, then the thermometer may be unacceptable
;       SKIP     - The total number of points plotted is reduced by this value.
;                  unless follow is set.  If follow is set, skip is used as 
;                  a skip factor in a loop over trajectories.
;                  If skip less than zero then n/skip particles will be plotted.
;       W1, W2, W3, W4, W5, W6  - Each is an array of 3 or more elements. 
;                 Plot can be windowed in real space or attribute space
;                 Last two values in the array are the minimum and maximum 
;                 values of the window parameter; the first value(s) are the 
;                 spatial or attribute parameters to be used for the window.
;                 The criteria is: {min <= (value) <= max}
;
;                 Note: If W has 4 elements then:
;                     a.  If W[0] eq W[1] then the absolute value of W[0] will
;                           be windowed.
;                     b.  If W[0] ne W[1] then the square root of the sum of the
;                           squares will be windowed.
;       CONDUCTOR - Specifies a conductor structure or a structure array 
;                   number specifying a conductor structure.  If valid, the 
;                   conductors will be overplotted.  Colors and thicknesses
;                   can be set with setdefault.
;                   Use ZPLANE to set the normal plane value.
;       ZPLANE    - Value of the third dimension for the CONDUCTOR and GRID
;                   plots. Default: Middle of data range - for particles this
;                   may not be the middle of the problem range.
;       POLARPLOT - If set, a polar plot will be generated.
;                 For the SPARE[0] eq 1, no polar plot is generated
;                 The coordinates for cylindrical and spherical data are as follows:
;                   R     - Radius vector
;                   PHI   - Angle between the radius vector projected on the X-Y 
;                           plane and the X axis
;                   Z     - Axis normal to the X-Y plane
;                   THETA - Angle between the Z axis and the Radius vector.
;                 For the SPARE[0] eq 2, the cylindrical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Phi, Z)
;                   X and Y must be 1 and 2 in any order, but they will be
;                     set to 1 and 2 respectively.
;                   The conversions are:
;                     X = R * Cos(Phi)
;                     Y = R * Sin(Phi)
;                 For the SPARE[0] eq 3, the spherical data will be converted
;                   to cartesian and plotted; Coordinates=(R, Theta, Phi)
;                   The conversions to cartesian are:
;                     X = R * Cos(Phi) * Sin(Theta)
;                     Y = R * Sin(Phi) * Sin(Theta)
;                     Z = R * Cos(Theta)
;                   For (X, Y) equal to (1,3), X and Y will be calculated according
;                     to the equations above and will be plotted with THETA = (!pi/2)
;                     If THETA is defined, then the equation for X will use:
;                       X = R * COS(X3) * SIN(X2 - THETA)
;                       where X2 is the second dimension, theta.
;                   For (X, Y) equal to (1,2), PHI will be set depending on the 
;                     keyword PPHI.
;                   If PHI is undefined.  Each point will have a value of PHI =0. 
;                     The abscissa and ordinates will be X and Z, respectively.
;                     X = R * SIN(X2)
;                     Z = R * COS(X2)
;                   If PPHI is defined, the plot will use:
;                     X = R * COS(X3 - PPHI) * SIN(X2)
;                     where X2 and X3 are the second and third coordinates.
;                   ASPECT will be set unless it is explicitly unset.
;                   Current version is for type 2 data only.
;       PTHETA  - See POLARPLOT
;                 Used only if POLARPLOT is set and [X,Y] = [1,3]
;       PPHI    - See POLARPLOT
;                 Used only if POLARPLOT is set and [X,Y] = [1, 2]
;       NPARTICLE - Number of particles plotted in a standard particle plot
;                   is returned
;       Verbose   - If set, NPARTICLE is printed to the terminal
;       FOLLOW  - If a particle dataset represents the motion of a test particle,
;                 the follow command allows the orbit in one plane to be mapped.
;                 This keyword will be ignored if the number of attributes is
;                 less than 1.
;                 No symbols are plotted. The linestyle is given by 
;                 abs(follow -1) mod 6 (If /follow is used, linestyle is solid.)
;       DELTA   - Indicates a symbol is to be plotted on the curve at
;                 approximate increments given by delta. Ignored if FOLLOW=0
;                 If delta is less than zero, then all points will be plotted.
;                 If the number of attributes is 6, then attribute 4 will be
;                 used; otherwise the number of attributes -1 will be used.
;                 A maximum of 2 symbols per point will be used.
;       GRID    - Specifies a grid structure or a structure array number.
;                 If valid, the grid will be overplotted.  Color and 
;                 thickness can be set with setdefault.
;                 Use ZPLANE to set the normal plane value.
; (window parameter is positive = spatial array, negative = attribute array) 
; (in the case of multiple parameters, use the sq root of the sum of the squares)
;       TITLE, XTITLE and YTITLE If grid is not zero, these will be used for
;                 axis and the plot labels.  If these are not string arrays, 
;                 the title, xtitle and ytitle used for the plots will be used.
;       XARRAY, YARRAY, FARRAY are the arrays sent to PLOTWT for the actual 
;                 particle plots unless BIN ne 0
;       BIN       If present and not zero, then XARRAY, YARRAY and FARRAY become
;                 X and Y coordinates for a contour plot and the actual function.
;                 ie: "contour, farray, xarray, yarray" would produce a contour
;                 plot of the particle density.  If WT is the particle charge,
;                 then FARRAY is the charge density. If WT is absent, then FARRAY
;                 is the number density.
;                 NOTE:  If RESULT is a valid structure, then FARRAY, XARRAY, 
;                        and YARRAY will be put into structure = RESULT
;                 NOTE: FARRAY is normalized to the area of the bin.
;                       For Cartesian this is simply DX*DY
;                       R and R^2 sin(theta) scaling are included with 
;                       cylindrical and spherical coordinates.
;                 CAUTION: For a cylindrical plot with theta, Z, the radius in
;                          NOT included in the normalization.
;                 If BIN is less than 0 then XARRAY and YARRAY will be used for
;                   bins for the particles. (XARRAY and YARRAY may have arbitrary
;                   spacing.)
;                 Note: BIN less than zero is about 50% slower than a uniform
;                       bin.  Errors can be as large as over 1e-5 for a 64x64 
;                       bin but about 1e-7 for a 65x65 grid; that is  the
;                       spacing is not a reciprocal power of 2.
;                       Time is about 4 minutes/million particles for uniform, 
;                       ie, bin > 0 or 6 minutes/million for bin< 0.
;                 If BIN is less than 16 then a uniform 65x65 grid will be 
;                   established for FARRAY (64 cells, 65 grid lines).
;                 If BIN is 16 or larger then a BINxBIN grid will be established
;                   for FARRAY.
;       XBIN, YBIN  - These values if present will override the values in BIN
;                 nx = abs(xbin)> 3  
;                 ny = abs(ybin)> 3 
;                 If xbin and ybin are greater than zero, then bin will be set.
;                 Bin must be greater than zero if only xbin or ybin is set.
;       NGP       If set the number of X and Y BIN values is reduced by 1.
;                 The BIN values will be taken as boundaries of each cell.
;                 This is ignored if BIN =0, and either XBIN or YBIN is zero.
;                 Default is NGP =0
;                 If NGP = 0, values will be interpolated to the four closest 
;                   grid points.
;                 If NGP is not zero, values will be credited to the square in
;                   which they fall. And the X and Y values will be changed to
;                   the center of each grid after particles are sorted. This
;                   reduces the number of x and y values by 1.
;         Note: NGP is slightly faster than interpolation however the 
;                 errors can be much larger. Speed is about 100 sec/Million
;                 and 4 minutes /million for bin >0 and <0
;       Vector    If present and not zero then FARRAY will be used to store
;                 the specified quantity according to the following scheme:
;                 0 - Particle density or charge density
;                 1 - Px or charge weighted value of Px
;                 2 - Py or charge weighted value of Py
;                 3 - Pz or charge weighted value of Pz
;                 Note: Particle must have as many attributes as value of VECTOR
;                 Default is VECTOR = 0
;           NOTE: With a weight of -4 and a vector value of 1, the value 
;                 in farray will be proportional to the charge current density
;                 in the x direction; with a weight of 0 farray will be 
;                 proportional to the particle current density
;       RESULT    If present and a valid structure array number, the xarray, 
;                 yarray, and farray data from the bin command will be sent to 
;                 this structure array.
;       QUIET   - If present and not zero,level information will not be printed.
;                 If quiet has a value of 1 (/QUIET or QUIET =1), NO plots
;                 will be make.  If QUIET is not zero and not 1, then plots
;                 will be made, but no output to the terminal.
;                 Default: QUIET = 0
;       HEADER    If specified a plot header, consisting of the current date
;                 and the current file name, is displayed in the lower left
;                 corner of the plot.  See the HEADER command to set default
;                 HEADER parameters (character size, spacing, alignment).
;                 DEFAULT is HEADER = 1
;                 NOTE: For X-windows screen dumps, many will want to increase
;                      the character size with the command:
;                      HEADER, /set, charsize = 0.8
;                      (Default is 0.6 of current size)
;     SETDEFAULT- If present and not zero, then the following parameters will be
;                 saved and used as the default values in all future calls to PLO
;                 If a keyword is specified, it will always be used rather than
;                 the saved values.
;     UNSET     - If present and not zero, will set all save values of keywords
;                 to their default values.
;                 Normal default values are listed below.
;                 Keyword     Default value
;                 HEADER       1
;                 XRANGE       0
;                 YRANGE       0
;
;     SHOWDEFAULT-Show the present default plot values.
;       LEFT     Indicates a new plot using the left axis.
;                A single axis will be drawn on the left side of the grid
;                For this plot XMARGIN will be set to
;                   XMARGIN = [!X.margin(0), !X.margin(0)]
;                and YSTYLE = 8.
;                NOTE: If LEFT is set, OVERPLOT is set to 0.
;      RIGHT     Indicates a OVERPLOT making and using the right axis.
;                If RIGHT is present and not zero, then:
;                  A single axis will be drawn on the right side of the existing
;                    grid
;                  The particle data will be overplotted on the existing grid.
;                Before using this command, a plot should be created using
;                  the keyword: /LEFT
;                NOTE: If RIGHT is set then OVERPLOT is set to 1.
;                NOTE: After this command all memory of the original y-axis
;                      scaling is lost
;       _EXTRA    These parameters are sent to PLOTWT and can have any value 
;                 appropriate for the plot command including the following:
;                 (X,Y)RANGE, (X,Y)STYLE, (X,Y)TYPE, LINESTYLE, POSITION , etc
;                 have the standard IDL meaning.
; OUTPUTS:
;       Designated datasets are plotted.
; EXAMPLE:
;       Plot from structure a, space 1 vs space 3 with attribute 4 
;         as the weight and use a grid [0,0.064, 0, 0.14]
;       plotpar, a, 3, 1, -4, xrange = [0,0.064],yrange = [ 0, 0.14]
;
;       Plot from structure a, the sum of the squares of attributes 1 and 3 
;         vs space 3 with attribute 4 as the weight
;       plotpar, a, 3, [-1,-3], -4
;
;       Plot from structure a, space 1 vs space 3 and limit the plot to values
;         of space 2 in the range [0, 3]  with attribute 4 as the weight and 
;         overplot conductors in datasets 3 and 4
;       plotpar, a, 3, 1, -4, w1 =[3, 0, 3], cond= [3,4]
;
;       Plot a particle structure 1 with a thermometer. Label the thermometer Q
;       Plot z-r and color the particles with charge scaled by -1.
;         plotpar, 1, 3, 1, -4, /change, thermometer= {title:'Q'}
;
;       Same as before, but color the particles with the absolute value of the charge
;         plotpar, 1, 3, 1, [-4], thermometer= {title:'Q'}
;
;       Overplot a particle plot, with attribute 4 as a weight and the 
;       conductors in structure, c2.
;       plotpar, a, -4, /over, con = c2
;
;       Plot from structure a, space 1 vs space 3 with a window of energy of 
;         1e10 to 1e13 for the magnitude of the momentum in attributes [1,2,3]
;         with attribute no weight
;       plotpar, a, 3, 1, w1 =[-1, -2, -3, 1e10, 1e13]
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 5, 1993
;-
;
; insert a common block
;
@qscom.dat
;
;  Common block
@qsstrcom.dat

common plot_grid_last, yeng_scale, xeng_scale
common plotpar_common, iseed
;
   common plotpar_defaults, header_save, xrange_save, yrange_save
@symbol_list.pro
      common mkcircle, lcmkcircle, nel_symbols
      if n_elements(lcmkcircle) eq 0 then mkcircle
      if lcmkcircle eq 0 then mkcircle
      if n_elements(nel_symbols) eq 0 then nel_symbols = 13

      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      if keyword_set(unset) or n_elements(header_save) eq 0 then begin
        HEADER_SAVE = 1
        XRANGE_SAVE = 0
        YRANGE_SAVE = 0
      endif
      if keyword_set (setdefault) then begin
        if n_elements(header) gt 0 then header_save = header
        if n_elements(xrange) gt 0 then xrange_save = xrange
        if n_elements(yrange) gt 0 then yrange_save = yrange
      endif
      if keyword_set(showdefault) then begin
         print, '% PLOTPAR: Default parameters for the PLOTPAR command are:'
         print, '    XRANGE     = ' , strtrim (xrange_save, 2)
         print, '    YRANGE     = ' , strtrim (yrange_save, 2)
         print, '    HEADER     = ' , strtrim (HEADER_SAVE, 2)
      endif
      npar = n_params()
      if npar eq 0 and (keyword_set (setdefault) or keyword_set(unset) $
          or keyword_set(showdefault)) then return
;
      if npar eq 0 then begin
 print,'% Format:', $
 ' PLOTPAR, STR [, X] [, Y] [, WT] [, YRANGE= YRANGE] [, THICK= THICK]'
 print,'         [, XRANGE = XRANGE] [, COLOR = COLOR] [, TITLE = TITLE]'
 print,'         [, XTITLE = XTITLE] [, YTITLE = YTITLE] [, CHANGESIGN = CHANGE]'
 print,'         [, FOLLOW = follow] [, DELTA = delta] [, SYMSIZE = SYMSIZE]'
 print,'         [, FAST = FAST] [, SKIP= SKIP]'
 print,'         [, W1= WINDOW1] [, W2= WINDOW2] [, W3= WINDOW3]'
 print,'         [, W4= WINDOW4] [, W5= WINDOW5] [, W6= WINDOW6]'
 print,'         [, XARRAY = XARRAY] [, YARRAY = YARRAY] [, FARRAY = FARRAY]'
 print,'         [, OVERPLOT = OVERPLOT] [, IGNORE = ignore] [, VECTOR = vector]'
 print,'         [, CONDUCTOR = CONDUCTOR] [, GRID = GRID] [HEADER = header]'
 print,'         [, BIN = bin] [, XBIN = xbin] [, YBIN = ybin] [, NGP = ngp]'
 print,'         [, SETDEFAULT = setdefault] [, UNSET = unset]'
 print,'         [, SHOWDEFAULT = showdefault]'
 print,'         [, RESULT = result] [,QUIET = quiet] [,_EXTRA = extrastuff]'
        return
      endif
;    make sure we don't corrupt input data
;     now get the data  -- check that we have a structure of the proper type
      if n_elements(ain) eq 0 then begin
        print, 'Structure must be defined'
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
      print, '% PLOTPAR: First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
      print, '% PLOTPAR: First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 2 then begin
        print, '% PLOTPAR: Only Particle type structures may be plotted'
        return
      endif
      if keyword_set(overplot) then overplot = (overplot eq -1) ? -1:1 $
        else overplot = 0
      if keyword_set(ignore) then ignore = 1 else ignore = 0

      npar = n_params()
      if keyword_set(overplot) then begin
        xrange = !x.crange
        if !x.type eq 1 then xrange = 10.0^xrange
        yrange = !y.crange
        if !y.type eq 1 then yrange = 10.0^yrange
        
        if npar lt 3 then begin
          if n_elements(x) gt 0 then wt = x
          x = xoldvalue
          y = yoldvalue
        endif else if ignore eq 0 then begin
          if x ne xoldvalue then begin
            Print,'% PLOTPAR: WARNING: This overplot requested with X-axis of ', x, $
                  ', present value is', xoldvalue
            print,'          Requested data will be plotted'
            hak, mes = ' Hit any key to continue'
          endif
          if y ne yoldvalue then begin
            Print,'% PLOTPAR: WARNING: This overplot requested with Y-axis of ', y, $
                  ', present value is', yoldvalue
            print,'          Requested data will be plotted'
            hak, mes = ' Hit any key to continue'
          endif
        endif
        if n_elements(polar) eq 0 then polar = polarplot
      endif else if n_elements(polar) eq 0 then polar = 0
      if (n_elements(wt) eq 0) &&  (n_elements(weight)) gt 0 then wt = weight
      polarplot = polar
;
      if keyword_set(follow) then begin
        if a.natt lt 1 then follow=0
      endif else if n_elements(follow) eq 0 then follow = 0
;
      nx = n_elements (x)
      ny = n_elements (y)
      if (nx eq 0) || (ny eq 0) then begin
        Print, '% PLOTPAR: X and Y arrays must be defined: nx, ny'
        print, '        ', nx, ny
        return
      endif
;     Check on polar plot parameters
      coordinate = a.spare[0]
      if keyword_set (polar) and not keyword_set(ignore)  then begin
        if (coordinate eq  1) then begin
           print, '% PLOTPAR: Cannot do a polar plot with a cartesian grid'
           return
        endif
        if nx gt 1 or ny gt 1 then begin
          print,  '% PLOTPAR: X-value and Y-value must be a scalars for polar plot'
          help, nx, ny
          return
        endif
        xt = x[0]
        yt = y[0]
        if (xt le 0) || (xt gt a.ndim) then begin
          print, '% PLOTPAR: X-value must be in the range of [1, ', $
             strtrim(a.ndim, 2), ']'
          print, xt
          return
        endif
        if (yt le 0) || (yt gt a.ndim) then begin
          print, '% PLOTPAR: Y-value must be in the range of [1, ', $
             strtrim(a.ndim, 2), ']'
          print, yt
          return
        endif
        if xt eq yt then begin
           print, '% PLOTPAR: X and Y-values must different: X, Y', xt, yt
          return
        endif
        if ((xt+yt) ne 3)  and (coordinate eq 2) then begin
          print, '% PLOTPAR: Cylindrical data: (X, Y)', xt, yt
          print, '           X and Y values must be 1 and 2 for a polar plot'
          return
        endif else if  ((xt+yt) gt 4)  and (coordinate eq 3) then begin
          print, '% PLOTPAR: Spherical data: (X, Y)', xt, yt
          print, '           X and Y values must be (1,2) or (1,3) for a polar plot'
          return
        endif
        xt = min([xt, yt], max = yt)
        if n_elements(aspect) eq 0 then aspect = 1
      endif
      nvert = a.nvert 
      win_ind = lindgen(nvert)

      if n_elements(fast) eq 0 then fast = 0
      if n_elements(fill) eq 0 then fill = 0
      if keyword_set(fast) then fill = 0
      if n_elements (psymin) eq 0 then begin
        if keyword_set(follow) then psym =1 else $
        if keyword_set(fast) then psym = 5 else psym = 8
      endif else begin
        symbol = psymin(0)
        sz = size(symbol)
        if sz(sz(0)+1) eq 7 then begin
          s  = strupcase(strcompress(symbol, /remove_all))
          l = strlen(s)
          if l eq 1 then begin
            if s eq 'X' then s = 'BIGX' else $
            if s eq '+' then s = 'PLUS' else $
            if s eq 'O' then s = 'CIRCLE'
            l = strlen(s)
          endif
          if l gt 0 then begin
            templist = strmid(symbol_list, 0,l)
            ok = where (s eq templist, nok )
            if nok eq 0 then begin
              print, '%PLOTPAR: A valid symbol string must be specified - ', s
              print, '          A filled circle will be used'
              mkcircle
            endif else  mksymbol, ok(0), fill = fill
          endif else mkcircle
          psym = 8
        endif else begin
          if symbol gt 10 then begin
            mksymbol, (symbol-11) < (nel_symbols-1), fill = fill
            psym = 8
          endif else begin
            if symbol lt 1 or symbol gt 8 then psym = 8 else psym = symbol
          endelse
        endelse
      endelse
      if n_elements (symsize) eq 0 then symsize = 0.3

;     window the arrays
      nweight = n_elements (window1)
      if nweight gt 2 then begin
        minw = min([window1(nweight-2), window1(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window1[0:nw-1])
        wina = get_array(a,t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a, t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        aw = where (wina ge minw and wina le maxw,naw )
        if naw lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 1 filter result)'
          return
        endif
        if naw lt nvert then win_ind = win_ind[aw]
      endif else naw = nvert

;     second window
      nweight = n_elements (window2)
      if nweight gt 2 then begin
        minw = min([window2(nweight-2), window2(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window2[0:nw-1])
        wina = get_array(a, t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a, t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        if naw lt nvert then wina = wina(win_ind)
        bw = where (wina ge minw and wina le maxw, nbw)
        if nbw lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 2 filter result)'
          return
        endif
        if nbw lt naw then win_ind = win_ind[bw]
      endif else nbw = naw

;     third window
      nweight = n_elements (window3)
      if nweight gt 2 then begin
        minw = min([window3(nweight-2), window3(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window3[0:nw-1])
        wina = get_array(a, t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a,t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        if nbw lt nvert then wina = wina(win_ind)
        cw = where (wina ge minw and wina le maxw, ncw)
        if ncw lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 3 filter result)'
          return
        endif
        if ncw lt nbw then win_ind = win_ind[cw]
      endif else ncw = nbw

;     fourth window
      nweight = n_elements (window4)
      if nweight gt 2 then begin
        minw = min([window4(nweight-2), window4(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window4[0:nw-1])
        wina = get_array(a, t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a,t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        if ncw lt nvert then wina = wina(win_ind)
        dw = where (wina ge minw and wina le maxw, ndw)
        if ndw lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 4 filter result)'
          return
        endif
        if ndw lt ncw then win_ind = win_ind[dw]
      endif else ndw = ncw

;     fifth window
      nweight = n_elements (window5)
      if nweight gt 2 then begin
        minw = min([window5(nweight-2), window5(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window5[0:nw-1])
        wina = get_array(a, t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a,t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        if ndw lt nvert then wina = wina(win_ind)
        ew = where (wina ge minw and wina le maxw, nnew)
        if nnew lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 5 filter result)'
          return
        endif
        if nnew lt ndw then win_ind = win_ind[ew]
      endif else nnew = ndw

;     sixth window
      nweight = n_elements (window6)
      if nweight gt 2 then begin
        minw = min([window6(nweight-2), window6(nweight-1)], max = maxw)
        nw = nweight -2
        t = round(window6[0:nw-1])
        wina = get_array(a, t[0])
        if nw gt 1 then begin
          if nw eq 2 && t[0] eq t[1] then wina = abs(wina) else begin
            wina = wina*wina
            for i = 1L, nw-1 do wina = wina + get_array(a,t[i])^2
            wina = sqrt(wina)
          endelse
        endif
        if nnew lt nvert then wina = wina(win_ind)
        fw = where (wina ge minw and wina le maxw, nfw)
        if nfw lt 1  then begin
          print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
          print, '          (Window 6 filter result)'
          return
        endif
        if nfw lt nnew then win_ind = win_ind[fw]
      endif else nfw = nnew

      if keyword_set(polar) then begin
         xx = get_array(a,xt, lab = xlab)
         if nfw lt nvert then xx = xx[win_ind]
         yy = get_array(a,yt, lab = ylab)
         if nfw lt nvert then yy = yy[win_ind]
         if coordinate eq 2 then  begin
;          Cylindircal coordinates
           cy = cos(yy)
           yy = xx * sin(yy)
           xx = xx * temporary(cy)
         endif else begin
;          Spherical coordinates
           if yt eq 3 then begin
             cy = cos(yy)
             yy = xx * sin(yy)
             xx = xx * cy
             if n_elements(theta) gt 0 then begin
               theta = theta[0]
               zz = get_array(a,2, lab = lab)
               if nfw lt nvert then zz = zz[win_ind]
               sth = sin(zz - theta)
               xx = xx*sth
               yy = yy*sth
             endif
           endif else begin
;            YT equal 2
             sy = sin(yy)
             yy = xx * cos(yy)
             xx = xx *sy
             if n_elements(phi) gt 0 then begin
               phi = phi[0]
               zz = get_array(a,3, lab = lab)
               if nfw lt nvert then zz = zz[win_ind]
               xx = xx * cos(zz - phi)
             endif
           endelse
         endelse
      endif else begin
        xt = x[0]
        yt = y[0]
        sx = size(x)
        sy = size(y)
        if min(abs(x)) eq 0 then begin
          print, '%PLOTPAR: X = 0 is not a valid value'
          return
        endif
        if min(abs(y)) eq 0 then begin
          print, '%PLOTPAR: Y = 0 is not a valid value'
          return
        endif
        if (max(x, min = mm) gt a.ndim) || (mm lt -a.natt) then begin
          print, '%PLOTPAR: Invalid value for X. Natt, ndim:', a.natt, a.ndim
          print, '          X values:', x
          return
        endif
        if (max(y, min = mm) gt a.ndim) || (mm lt -a.natt) then begin
          print, '%PLOTPAR: Invalid value for Y. Natt, ndim:', a.natt, a.ndim
          print, '          Y values:', y
          return
        endif
        if nfw eq nvert then xx = get_array(a,xt, lab = xlab) $
          else xx = (get_array(a,xt, lab = xlab))[win_ind]
        if (sx[0] gt 0) then begin
          if nx eq 1 then xx = abs(xx) else begin
            xx = xx*xx
            for i = 1L, nx-1 do begin
              if nfw eq nvert then  xx = xx + get_array(a, x(i), lab = alab)^2 $
                else xx = xx + ((get_array(a, x(i), lab = alab))[win_ind]) ^2
              xlab = xlab+ ' + ' + alab
            endfor
            xx = sqrt(xx)
          endelse
        endif
        if nfw eq nvert then yy = get_array(a,yt, lab = ylab) $
          else yy = (get_array(a,yt, lab = ylab))[win_ind]
        if (sy[0] gt 0) then begin 
          if ny eq 1 then yy = abs(yy) else begin
            yy = yy*yy
            for i = 1L, ny-1 do begin
              if nfw eq nvert then  yy = yy + get_array(a, y(i), lab = alab)^2 $
                else yy = yy + ((get_array(a, y(i), lab = alab))[win_ind]) ^2
              ylab = ylab+ ' + ' + alab
            endfor
            yy = sqrt(yy)
          endelse
        endif
      endelse

          
      if keyword_set(follow) then begin
        nn = get_array(a, -a.natt)
        if nfw lt nvert then nn = nn[win_ind]
        if keyword_set(delta) then begin
          if a.natt eq 6 then ntime =  get_array(a, -4) $
            else  ntime =  get_array(a, -(a.natt-1))
          if nfw lt nvert then ntime = ntime[win_ind]
        endif
      endif
      if n_elements(xtitle) eq 0 then xtitle = xlab
      b = size (xtitle)
      if b(b(0)+1) ne 7 then xtitle = xlab
      if n_elements(ytitle) eq 0 then ytitle = ylab
      b = size (ytitle)
      if b(b(0)+1) ne 7 then ytitle = ylab
      if n_elements(title) eq 0 then title = a.clab
      b = size (title)
      if b(b(0)+1) ne 7 then title = a.clab
      nw = n_elements (wt)
      if nw eq 0 then wwt = 0.0 else wwt = round(wt(0))
      if wwt ne 0 then begin
        wwt = get_array(a,wwt, lab = lab)
        if nfw lt nvert then wwt = wwt[win_ind]
        sw = size(wt)
        if sw[0] gt 0 then begin
          if nw eq 1 then wwt = abs(wwt) else begin
            temp = wwt*wwt
            if nfw lt nvert then $
            for i = 1L, nw-1 do temp= temp + (get_array(a, wt[i]))[win_ind] ^2 $ 
            else for i = 1L, nw-1 do temp= temp + get_array(a, wt(i))^2
            wwt = sqrt(temp)
          endelse
        endif
      endif
      nwwt = n_elements(wwt)
      if n_elements (change) ne 0 then if change ne 0 then wwt = -wwt
      nx = nfw
      ny = nfw
      n = nfw
      if n lt 1  then begin
        print, '%PLOTPAR: X and Y arrays must be defined at at least 1 point'
        return
      endif
      if n_elements (vector) eq 0 then begin
        vect = make_array(n, value = 1.0)
        if nwwt gt 1 then vlab = 'Charge Density' $
            else vlab = 'Particle Density'
      endif else begin
        if vector eq 0 then begin
           vect = make_array(n, value = 1.0)
           if nwwt gt 1 then vlab = 'Charge Density' $
               else vlab = 'Charge Density'
        endif else if vector gt 0 then begin
          if vector gt a.natt then begin
   print, '%PLOTPAR: Value of VECTOR must be between 1 and ',strtrim(a.natt,2)
            return
          endif
          vect = get_array(a, -vector(0), lab = vlab)
          if nfw lt nvert then vect = vect[win_ind]
        endif else begin
   print, '%PLOTPAR: Value of VECTOR must be between 1 and ',strtrim(a.natt,2)
          return
        endelse
      endelse

      minx = min(xx, max = maxx)
      if (maxx-minx) lt abs(maxx+minx)/16384. then begin
        if not keyword_set(quiet) then $
        print, '%PLOTPAR: Range of x values is', minx,' to', maxx 
        if keyword_set (bin) then begin
          if bin lt 0 and n_elements(xarray) gt 2 and $
             n_elements(yarray) gt 2 then begin
             if n_elements (xrange) lt 2 then xrange = [min(xarray, max =m),m]
          endif
        endif
        if not keyword_set(overplot) and n_elements(xrange) lt 2 then begin
          print, '%PLOTPAR: No plot will be attempted'
          return
        endif
      endif
;
      if n_elements(header) eq 0 then header = header_save
      if overplot ne 0 then header = 0
      if !p.multi(0) ne 0 then header = 0
      if header ne 0 then begin
        filename = strtrim(a.file, 2)
        if (strlen(filename) eq 0) then header_text='Plot: ' + thin_date() $
        else  begin
          ss = file_info(filename)
          if ss.exists eq 1 then begin
            cdate = thin_date(ss.ctime)
            header_text= ['Plot: '+thin_date() +'; Data: ' + cdate,  $
                   'File: ' +filename]
          endif else header_text=['Plot: ' + thin_date(), 'File: ' +filename]
        endelse
      endif
      if n_elements(xrange) eq 0 then xrange = xrange_save
      if n_elements(yrange) eq 0 then yrange = yrange_save
;
      if n_elements (xrange) lt 2 then xrange = [minx, maxx] else $
        if not keyword_set(follow)  then begin
;       plotwt does not check for clipping so must do it now.
        xrange = xrange(0:1)
        xranmin = min(xrange, max = xranmax)
        ok  = where (xx ge xranmin and xx le xranmax,nel)
        if nel gt 0 then begin
          xx = xx(ok)
          yy = yy(ok)
          if keyword_set(follow) then begin
            nn = nn(ok)
            if keyword_set(delta) then ntime = ntime(ok)
          endif

          if nwwt gt 1 then wwt = wwt(ok)
        endif else begin
          print ,'%PLOTPAR: No particles in the designated spatial range'
          return
        endelse
      endif
      miny = min(yy, max = maxy)
      if (maxy-miny) lt abs(maxy+miny)/16384. then begin
        if not keyword_set(quiet) then $
        print, '%PLOTPAR: Range of y values is', miny,' to', maxy
        if keyword_set (bin) then begin
          if bin lt 0 and n_elements(xarray) gt 2 and $
             n_elements(yarray) gt 2 then begin
             if n_elements (yrange) lt 2 then yrange = [min(yarray, max =m),m]
          endif
        endif
        if not keyword_set(overplot) and n_elements(yrange) lt 2 then begin
          print, '%PLOTPAR: No plot will be attempted'
          return
        endif
      endif
      if n_elements (yrange) lt 2 then yrange = [miny, maxy] $
        else if not keyword_set(follow) then begin
;       plotwt does not check for clipping so must do it now.
        yrange = yrange(0:1)
        yranmin = min(yrange, max = yranmax)
        ok  = where (yy ge yranmin and yy le yranmax,nel)
        if nel gt 0 then begin
          xx = xx(ok)
          yy = yy(ok)
          if nwwt gt 1 then wwt = wwt(ok)
        endif else begin
          print ,'%PLOTPAR: No particles in the designated spatial range'
          return
        endelse
      endif
      n = n_elements (xx)
;
      if n_elements (skipin) eq 0 then skip = 1 else skip = skipin
      if skip gt 0 then lcrandom = 0 else begin
        skip = abs(skip)
        lcrandom = 1
      endelse
; the following 3 lines added because SKIP appears to use N and N was not 
; defined to be the number of points.

      if skip gt 1 and not keyword_set(follow) then begin
        if skip ge n then begin
          aa = [0, n-1]
        endif else if lcrandom eq 0 then begin
          np = long (n/skip) +1
          aa = long( indgen(np)*skip)
        endif else begin
          nok = 0
          while (nok eq 0 ) do begin
            aa = randomu(iseed, n)
            aa = where (aa lt 1.0/skip, nok)
          endwhile
        endelse
        xx = xx(aa)
        yy = yy(aa)
        if nwwt  gt 1 then wwt = wwt(aa)
      endif
      if n_elements(x) eq 1 then xoldvalue = x else xoldvalue = 0
      if n_elements(y) eq 1 then yoldvalue = y else yoldvalue = 0
      if overplot eq 0 then begin
        if xoldvalue gt 0 and yoldvalue gt 0 then begin
          zplaneold = 6-xoldvalue-yoldvalue
          if n_elements(zplane) gt 0 then zvalueold = zplane(0) else begin
            if zplaneold gt 0 and zplaneold le 3 then  begin
              zvalueold = (max(get_array(a, zplaneold), min= mm)+mm)*0.5
            endif else begin
              zplaneold = 0
              zvalueold = 0
            endelse
          endelse 
        endif else begin
          zplaneold = 0
          zvalueold = 0
        endelse
      endif
      if n_elements(color) eq 0 then begin
        if follow eq 0 then color=0 else color = !p.color
        if keyword_set(follow)  then color= !p.color else color = 0
      endif
      lctwodim = a.two
;
    if n_elements(left) eq 0 then left = 0
    if n_elements(right) eq 0 then right = 0
    if n_elements (quiet) eq 0 then quiet = 0
    if (quiet ne 1) then begin
      if not keyword_set(follow) then begin
        npart = n_elements(xx)
        if keyword_set(verbose) then print, ' Number of particles is ' , $
          strtrim(npart, 2)
        if n_elements(c_range_in) gt 1 then c_range = c_range_in
        plotwt, xx, yy, wwt, color =color, symsize=symsize, psym=psym, $
              xtitle= xtitle, ytitle = ytitle, title= title, over = overplot, $
              xrange = xrange, yrange = yrange, ystyle = ystyle, c_range= c_range, $
              xstyle = xstyle, xtype = xtype, ytype = ytype, aspect = aspect, $
              thermometer = thermometer, zrange = zrange, quiet = quiet, $
              left = left, right = right, ztype = ztype, _extra= extrastuff
        if header ne 0 and overplot eq 0 then header, header_text
      endif else begin
        if left ne 0 then begin
            ystyle = (ystyle or 8)
            xmargin = [!x.margin(0), !x.margin(0)]
        endif else xmargin = !x.margin
        if n_elements (xstyle) eq 0 then xstyle = 0
        if n_elements (ystyle) eq 0 then ystyle = 0
        if n_elements (xtype) eq 0 then xtype = 0
        if n_elements (ytype) eq 0 then ytype = 0
;
        if right ne 0 then begin
          overplot = 1
          if !x.type eq 0  then st = !x.crange(1) else st = 10.0^(!x.crange(1))
          axis, /yaxis, /save, st, 0, yrange = yrange, ytitle= ytitle
        endif
        if overplot eq 0 then begin
          yeng_scale= 1.00 &  xeng_scale = 1.00
          if keyword_set (aspect) then begin
            pos = get_aspect(xrange, yrange, xmargin = xmargin)
            xstyle = (xstyle or 1)
            ystyle = (ystyle or 1)
            plot, xx, yy, xtitle = xtitle, ytitle = ytitle, /nodata, $
             xrange = xrange, yrange = yrange, ystyle = ystyle, $
             title = title, xmargin=xmargin, position = pos, $
             xstyle = xstyle, xtype = xtype, ytype = ytype, _extra= extrastuff 
          endif else $  
          plot, xx, yy, xtitle = xtitle, ytitle = ytitle, /nodata, $
             xrange = xrange, yrange = yrange, ystyle = ystyle, $
             title = title, xmargin=xmargin, $
             xstyle = xstyle, xtype = xtype, ytype = ytype, _extra= extrastuff 
          if header ne 0 then header, header_text
        endif
        nx = n_elements(nn)
        if nx lt 2 then begin
          print, '%PLOTPAR: Must have more than one particle to plot'
          return
        endif

        dn = nn(1: nx-1)- nn(0:nx-2)
        ok = where (dn gt 0.5, nok)
        npart = nok+1
        if keyword_set(verbose) then print, ' Number of particles is ' , $
          strtrim(npart, 2)
        if nok eq 0 then begin
          if xeng_scale eq 1.0 and yeng_scale eq 1.0 then $
          oplot, xx, yy, color = color, linestyle = abs(follow(0)-1) mod 6, $
           _extra = extrastuff else $
          oplot, xx*xeng_scale, yy*yeng_scale, color = color, $
            linestyle = abs(follow(0)-1) mod 6, _extra = extrastuff
          ok = [0, nx]
        endif else begin
          ok = [0, ok+1, nx]
          if skip gt nok then skip = nok
          for i = 0L, nok, (skip>1) do begin
            sti = ok(i)
            stf = ok(i+1)-1
            if (stf-sti) gt 0 then oplot, xx(sti:stf), yy(sti:stf), $
               color = color, linestyle = abs(follow(0)-1) mod 6, $
               _extra = extrastuff
          endfor
        endelse
        if keyword_set(delta) then begin
           if xeng_scale ne 1.0 or yeng_scale ne 1.0 then $
            PRINT, 'ERROR:  PLOTPAR/DELTA set, Eng_SCALE ne 1.0'
          if delta lt 0 then oplot, xx, yy, psym = psym, syms = symsize $
          else begin
            for i = 0L, nok do begin
              sti =  ok(i)
              stf = ok(i+1)-1
              maxnp = 2*(stf-sti+1)
              time = ntime(stf)-ntime(sti)
              np = long (time/delta ) +1
              if np gt maxnp then begin
                np = maxnp
                dd = time/(np-1)
              endif else dd = delta
              if np gt 1 then begin
                par = ntime(sti)+dd * findgen(np)
                oplot, get_yarr(ntime(sti:stf), xx(sti:stf), par), $
                  get_yarr(ntime(sti:stf), yy(sti:stf), par), $
                  psym = psym, syms= symsize
              endif else plots, xx(sti), yy(sti), psym = psym, syms= symsize
            endfor
          endelse
        endif
      endelse
;
      if xoldvalue*yoldvalue*zplaneold gt 0 then begin
        if n_elements(conductor) gt 0 then begin
          plotcon, conductor, /over
        endif
        if n_elements(grid) gt 0 then begin
          plotgrd, grid, /over
        endif
      endif else if n_elements(conductor) gt 0 or n_elements(grid) gt 0 then $
 print, '% PLOTPAR: GRID and CONDUCTOR plots are not possible with these axes'
      xinit = min(!x.crange)
      if !x.type eq 1 then xinit = 10^xinit
      yinit = min(!y.crange)
      if !y.type eq 1 then yinit = 10^yinit
      dx = abs(!x.crange(1)-!x.crange(0))
      if !x.type eq 1 then dx = 10.0^dx
      dy = abs(!y.crange(1)-!y.crange(0))
      if !y.type eq 1 then dy = 10.0^dy
    endif else begin
      if n_elements (xrange) ne 2 then xrange= [min(xx, max = mm), mm]
      if n_elements (yrange) ne 2 then yrange= [min(yy, max = mm), mm]
      dx = abs(xrange(1)-xrange(0))
      xinit = min(xrange)
      dy = abs(yrange(1)-yrange(0))
      yinit = min(yrange)
    endelse
      dx = float (dx)
      dy = float (dy)
      if n_elements (bin) eq 0 then bin = 0
      if n_elements (xbin) gt 0 and n_elements(ybin) gt 0 then begin
        if xbin gt 0 and ybin gt 0 then bin = xbin
      endif
      if bin ne 0 then begin
        tstart = systime(1)
        np = n_elements (xx)
        if n_elements (wwt) lt np then wwt = make_array(np, value = 1.0)
        wwt = wwt*vect
        if bin lt 0 then begin
          nx = n_elements (xarray)
          if nx lt 3  then begin
            print, '%PLOTPAR: Unable to bin data (bin < 0) with nx=', nx
            return
          endif
          dx = xarray(1:nx-1) - xarray(0:nx-2)
          if min(dx lt 0.0) then begin
            aa = sort(xarray)
            xarray = xarray(aa)
            dx = xarray(1:nx-1) - xarray(0:nx-2)
            ok = where (dx gt 0.0, nok)
            if nok lt nx then begin
              if nok eq 0 then begin
                print, '%PLOTPAR: XARRAY has no valid array for BIN'
                return
              endif
              xarray = xarray(ok)
              nx = n_elements (xarray)
              dx = xarray(1:nx-1) - xarray(0:nx-2)
            endif
          endif

          ny = n_elements (yarray)
          dy = yarray(1:ny-1) - yarray(0:ny-2)
          if ny lt 3 then begin
            print, '%PLOTPAR: Unable to bin data (bin < 0) with ny=', ny
            return
          endif
          dy = yarray(1:ny-1) - yarray(0:ny-2)
          if min(dy lt 0.0) then begin
            aa = sort(yarray)
            yarray = yarray(aa)
            dy = yarray(1:ny-1) - yarray(0:ny-2)
            ok = where (dy gt 0.0, nok)
            if nok lt ny then begin
              if nok eq 0 then begin
                print, '%PLOTPAR: XARRAY has no valid array for BIN'
                return
              endif
              yarray = yarray(ok)
              ny = n_elements (yarray)
              dy = yarray(1:ny-1) - yarray(0:ny-2)
            endif
          endif
          if keyword_set (ngp) then begin
            warray = make_array (nx-1, ny-1, value = 0.0d0, /double)
          endif else begin
            dxa = [dx(0), (dx(0:nx-3)+dx(1:nx-2)), dx(nx-2)]*0.5
            dya = [dy(0), (dy(0:ny-3)+dy(1:ny-2)), dy(ny-2)]*0.5
            warray = make_array (nx, ny, value = 0.0d0, /double)
          endelse
          nx2 = nx/2 & ny2 = ny/2
          for k = 0L, np-1 do begin
            if ((xx(k) ge xarray(0)) and (yy(k) ge yarray(0))) then begin
              okx = where(xarray gt xx(k), nokx)
              oky = where(yarray gt yy(k), noky)
              if nokx*noky gt 0 then begin
                if keyword_set(ngp) then begin
                    warray(okx(0)-1,oky(0)-1)= warray(okx(0)-1,oky(0)-1)+wwt(k) 
                endif else begin
                  ii = okx(0)
                  if xx(k) eq xarray(ii) then dii = 0.0  else $
                    dii= (xarray(ii)- xx(k))/dx(ii-1)
                  jj = oky(0)
                  if yy(k) eq yarray(jj) then djj = 0.0  else $
                    djj= (yarray(jj)- yy(k))/dy(jj-1)
                  warray(ii, jj) = warray(ii, jj)+ wwt(k)*(1.0-dii)*(1.0-djj)
                  warray(ii-1, jj) = warray(ii-1, jj)+ wwt(k)*dii*(1.0-djj)
                  warray(ii, jj-1) = warray(ii, jj-1)+ wwt(k)*(1.0-dii)*(djj)
                  warray(ii-1, jj-1) = warray(ii-1, jj-1)+ wwt(k)*dii*djj
                endelse
              endif
            endif
          endfor
          if keyword_set (ngp) then begin
            warray = temporary(warray)/(dx # dy)
            xarray = xarray(0:nx-2)+dx*0.5
            yarray = yarray(0:ny-2)+dy*0.5
            nx = nx-1
            ny = ny-1
          endif else warray = temporary(warray)/(dxa # dya)
          
        endif else begin
          if bin lt 16 then nw = 64 else nw = long(bin)
          if n_elements (xbin) gt 0 then nx = abs (xbin) > 3 else nx = nw
          if n_elements (ybin) gt 0 then ny = abs (ybin) > 3 else ny = nw
          dx = dx/(nx-1)
          dy = dy/(ny-1)
          if keyword_set(ngp) then begin
            nx = nx-1
            ny = ny-1
            warray = make_array(nx, ny, value = 0.0d0, /double)
            xarray = xinit+0.5*dx+dx*findgen(nx)
            yarray = yinit+0.5*dy+dy*findgen(ny)
            for k = 0L, np-1 do begin
              if xx(k) ge xinit and yy(k) ge yinit then begin
                i = fix((xx(k)-xinit)/dx)
                j = fix((yy(k)-yinit)/dy)
                if (i ge 0 and i lt nx) and (j ge 0.0 and j lt ny) $
                  then warray(i, j) = warray(i, j)+ wwt(k)
              endif
            endfor
            warray= warray/(dx*dy)
            dx = make_array (nx, value = dx)
            dy = make_array (ny, value = dy)

          endif else begin
            warray = make_array(nx, ny, value = 0.0d0, /double)

            xarray = xinit+dx*findgen(nx)
            yarray = yinit+dy*findgen(ny)
            for k = 0L, np-1 do begin
              i = (xx(k)-xarray(0))/dx
              j = (yy(k)-yarray(0))/dy
              if (i ge 0.0 and i lt (nx-1)) and (j ge 0.0 and j lt (ny-1)) $
                then begin
                ii = fix(i)
                jj = fix(j)
                warray(ii, jj) = warray(ii, jj)+ wwt(k)*(ii+1-i)*(jj+1-j)
                warray(ii+1, jj) = warray(ii+1, jj)+ wwt(k)*(i-ii)*(jj+1-j)
                warray(ii, jj+1) = warray(ii, jj+1)+ wwt(k)*(ii+1-i)*(j-jj)
                warray(ii+1, jj+1) = warray(ii+1, jj+1)+ wwt(k)*(i-ii)*(j-jj)
              endif 
            endfor
            warray= warray/(dx*dy)
;         scale the border by 2
            warray(*, 0) = warray(*, 0) + warray(*, 0)
            warray(*, ny-1) = warray(*, ny-1) + warray(*, ny-1)
            warray(0, *) = warray(0, *) + warray(0, *)
            warray(nx-1, *) = warray(nx-1, *) + warray(nx-1, *)
            dx = make_array (nx, value = dx)
            dy = make_array (ny, value = dy)
            dx(0)= dx(0)*0.5
            dy(0)= dy(0)*0.5
            dx(nx-1) = dx(0)
            dy(ny-1) = dy(0)
          endelse
        endelse
        warray = float(warray)
;        print, 'time: ',systime(1)-tstart
        if a.spare(0) gt 1 then begin
          if xoldvalue eq 1 then begin
;           radius is the x axis
            if a.spare (0) eq 2 then begin
              for i = 1, nx-2 do $
              warray (i, *) = warray(i, *)/xarray(i)
              warray (0,*) = warray(0,*)/(xarray(0)+dx(0)*0.25)
              warray (nx-1,*) = warray(nx-1,*)/(xarray(nx-1)-dx(nx-2)*0.25)
            endif else begin
              for i = 1, nx-2 do $
              warray (i, *) = warray(i, *)/xarray(i)^2
              warray (0,*) = warray(0,*)/(xarray(0)+dx(0)*0.25)^2
              warray (nx-1,*) = warray(nx-1,*)/(xarray(nx-1)-dx(nx-2)*0.25)^2
            endelse
          endif else if a.spare(0) eq 3 and xoldvalue eq 2 then begin
              for i = 1, nx-2 do $
              warray (i, *) = warray(i, *)/sin(xarray(i))
              warray (0,*) = warray(0,*)/sin(xarray(0)+dx(0)*0.25)
              warray (nx-1,*) = warray(nx-1,*)/sin(xarray(nx-1)-dx(nx-2)*0.25)
          endif
          if yoldvalue eq 1 then begin
;           radius is the y axis
            if a.spare (0) eq 2 then begin
              for i = 1, ny-2 do $
              warray (*,i) = warray(*,i)/yarray(i)
              warray (*,0) = warray(*,0)/(yarray(0)+dy(0)*0.25)
              warray (*,ny-1) = warray(*,ny-1)/(yarray(ny-1)-dy(ny-2)*0.25)
            endif else begin
              for i = 1, ny-2 do $
              warray (*,i) = warray(*,i)/yarray(i)^2
              warray (*,0) = warray(*,0)/(yarray(0)+dy(0)*0.25)^2
              warray (*,ny-1) = warray(*,ny-1)/(yarray(ny-1)-dy(ny-2)*0.25)^2
            endelse
          endif else if a.spare(0) eq 3 and yoldvalue eq 2 then begin
              for i = 1, ny-2 do $
              warray (*,i) = warray(*,i)/sin(yarray(i))
              warray (*,0) = warray(*,0)/sin(yarray(0)+dy(0)*0.25)
              warray (*,ny-1) = warray(*,ny-1)/sin(yarray(ny-1)-dy(ny-2)*0.25)
          endif
        endif
        if n_elements(result) ne 0 then begin
          if result gt 0 and result le maxstructure then begin
            nx = n_elements (xarray)
            ny = n_elements (yarray)
            nf = n_elements (warray)
;
            locx = make_array(2, 2, /long)
            locx(0, *)= [0, nx]
            locx(1, *)= [0, ny]
            range = make_array(2, 2, 2)
            range(0, 0, *) = min(xarray, max = mx)
            range(1, 0, *) = mx
            range(0, 1, *) = min(yarray, max = mx)
            range(1, 1, *) = mx
            tlab = 'Contour'
            x1lab = make_array(1, /string, value = xtitle)
            x2lab = make_array(1, /string, value = ytitle)
            v1lab =  make_array(1, /string, value = title)
            tempstr = {type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
              size:[nx, ny],range: range, order: 1L, slice:0.0 , $
              connect: 0L,  locx:locx, $
              locv: [0L, nf], x1:xarray, x2:yarray, x3:0.0, x4:0.0, $
              v1:warray(0:nf-1), v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : x1lab, x2lab : x2lab, x3lab : '', x4lab :'', $
              v1lab : v1lab, v2lab :'', v3lab : '', v4lab : '',v5lab : '', $
              v6lab : '', file : a.file, spare : [0,0,0,0,0], tlab : tlab, $
              clab: vlab +' from particle plot: ' +a.clab }
            i2s, tempstr, result, /no_copy
          endif
        endif
;
      endif else begin
        xarray = xx
        yarray = yy
        warray = wwt
      endelse
      return
      end
