; ------------------------------------------------------------------------ 
; $Id: plotstr.pro,v 1.8 2010/07/07 20:21:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro plotstr,  ain,p1, p2, p3, p4, p5, xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, value_node= value_node, $
        title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           model = model,  _extra = extrastuff, front = front
;
;+
; NAME:
;       PLOTSTR
;
; PURPOSE:
;       Plot a structure array
;       WARNING:  The maximum structure array will be used as a working 
;                 structure array for PLOTFLD
;       For a description of parameters and keywords, see the commands
;          PLOTFLD
;          PLOTCON
;          PLOTGRD
;          PLOTPAR
;          PLOTFE12
;          PLOTFE9
;          PLOTSURF
; KEYWORD PARAMETERS:
;       FRONT:    Use WSHOW to pop current window to the front.
;                 This uses the same default value for FRONT as PLO
;                 DEFAULT: FRONT = 1
; EXAMPLE:
;       Contour plot of the charge density, in structure F, from a 2-D calc.
;         plotstr,  f
;
;       Contour plot of the charge density, in structure array 2, from a 3-D 
;       calculation taking a slice for the second dimension equal to 0.0
;         plotstr, 2,1, 2, 0.0
;  (or)   plotstr, 2,   2, 0.0 (since only one attribute, 1 can be omitted)
;
;       Plot from structure g, space 1 vs space 3 with blue dashed lines
;       plotstr, g, 3, 1, color = 3, c_line = 2
;
;       Plot from structure array 2, space 1 vs space 3 
;             use a grid [0,0.064, 0, 0.14]
;       plotstr, 2, 3, 1, xrange = [0,0.064],yrange = [ 0, 0.14]
;
;
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, March 15, 1993
;-
; insert a common block
;
@qscom.dat
;
@qsstrcom.dat
;
@plo_defaults.com
;
;       return on an error
      on_error, 1
;
      if n_elements (front_save) eq 0 then front_save =1
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
      front = (n_elements(front) eq 0) ? front_save : front[0]

      if keyword_set(front) && (!d.window ge 0) then my_wshow, !d.window, iconic =0
;
;
;     now get the data  -- check that we have a structure of the proper type
;
; change variables to insure we don't corrupt input data
      a =  get_valid_structure(ain, 'PLOTSTR', lcstr=lcstr, ierror=ier)
      if ier ne 0 then return
      structinit = -1
      if a.type eq 1  or a.type eq 8 then begin
        if n_elements(p5) gt 0 then $
          plotfld, a, p1, p2, p3, p4, p5,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else if n_elements(p4) gt 0 then $
          plotfld, a, p1, p2, p3, p4,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else if n_elements(p3) gt 0 then $
          plotfld, a, p1, p2, p3,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else if n_elements(p2) gt 0 then $
          plotfld, a, p1, p2,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotfld, a, p1,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else plotfld, a,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff

      endif else if a.type eq 9 then begin
        if n_elements(p1) gt 0 then $
          plotfe9, a, p1,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, value_node = value_node, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff $
        else plotfe9, a,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, value_node = value_node, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff 
;
      endif else if (a.type eq 14) || (a.type eq 15) then begin
        plotsurf, a, p1, model = model,  $
          xrange = xrange, yrange = yrange, zrange= zrange, $
           _extra = extrastuff


      endif else if a.type eq 12 then begin
        if n_elements(p2) gt 0 then $
          plotfe12, a, p1,p2, xtitle = xtitle, ytitle = ytitle, $
            grid = grid, value_node= value_node, $
            title = title, xarray= xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
            _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotfe12, a, p1, xtitle = xtitle, ytitle = ytitle, $
            grid = grid, value_node= value_node, $
            title = title, xarray= xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
            _extra = extrastuff $
        else plotfe12, a, xtitle = xtitle, ytitle = ytitle, $
            grid = grid, value_node= value_node, $
            title = title, xarray= xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, zrange= zrange, $
            _extra = extrastuff

      endif else if a.type eq 2 then begin
        if n_elements(p3) gt 0 then $
          plotpar, a, p1, p2, p3,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, zrange = zrange,  $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p2) gt 0 then $
          plotpar, a, p1, p2,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, zrange = zrange,  $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotpar, a, p1,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, zrange = zrange,  $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else plotpar, a,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, grid = grid, zrange = zrange, $
           title = title, xarray = xarray, yarray = yarray, farray = farray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff
      endif else if a.type eq 3 or a.type eq 10 or a.type eq 11 then begin
        if n_elements(p3) gt 0 then $
          plotgrd, a, p1, p2, p3,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, $
           title = title, xarray = xarray, yarray = yarray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p2) gt 0 then $
          plotgrd, a, p1, p2,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, $
           title = title, xarray = xarray, yarray = yarray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotgrd, a, p1,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, $
           title = title, xarray = xarray, yarray = yarray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff $
        else plotgrd, a,xtitle = xtitle, ytitle = ytitle, $
            conductor= conductor, $
           title = title, xarray = xarray, yarray = yarray, $
           xrange = xrange, yrange = yrange, _extra = extrastuff
      endif else if a.type eq 4 then begin
        if n_elements(p3) gt 0 then $
          plotcon, a, p1, p2, p3,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p2) gt 0 then $
          plotcon, a, p1, p2,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotcon, a, p1,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else plotcon, a,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff
      endif else if a.type eq 7 then begin
        if n_elements(p3) gt 0 then $
          plotacis, a, p1, p2, p3,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p2) gt 0 then $
          plotacis, a, p1, p2,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else if n_elements(p1) gt 0 then $
          plotacis, a, p1,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff $
        else plotacis, a,xtitle = xtitle, ytitle = ytitle, $
           title = title, xrange = xrange, yrange = yrange, _extra = extrastuff
      endif else print, ' This structure type is not supported by PLOTSTR'
      structinit = 1
      return
      end
