; ------------------------------------------------------------------------ 
; $Id: scanf.pro,v 1.3 2007/06/06 20:09:36 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro ScanF, fld, Norm, Normval, dsmin, dsmax, dsstp, block = block,  $
             con=con, scale=scale, index=index, winnum=winnum, $
             xsize = xsize, ysize = ysize,  _extra = extrastuff
;+
; PURPOSE:
;       Use XINTERANIMATE to build an animation of a DO-list of 
;       datasets of FIELD snapshots
; CALLING SEQUENCE:
;       ScanF, FLD, NORM, NORM_VAL, Start [, Stop] [, Skip]
;               [, CON=CON] [, SCALE=SCALE ] [, INDEX=INDEX ]
;               [, XSIZE = xsize] [, YSIZE = ysize] [, Block = block]
;               [, WINNUM=WINNUM ] [, _EXTRA = EXTRASTUFF]
; INPUTS:
;       FLD :     Field component
;       NORM:     Plane normal for PLOTFLD
;       NORM_VAL: Value of normal ordinate (PLOTFLD argument)
;       Start \  
;       Stop -  DO list for frames in file.
;       Skip  /  
;       Default Value for Stop is number of datasets in current PFF file
;       If Stop le start then stop = ndspff()
;       If Skip le 0 or undefined then skip = 1
; OPTIONAL INPUT:
;       -none-
; KEYWORDS:
;       CON:    Conductor structure
;       SCALE:  Scale factor for field (QHRE argument)
;       INDEX:  Index of plane (PLOTFLD option)
;       WINNUM: IDL window number
;       XSIZE:  X Size of the window; Default = 448 (7*64)
;       YSIZE:  Y Size of the window; Default = 384 (6*64)
;       BLOCK   Xinteranimate called with block = block
;
; Example
;       Make a movie using datasets 215 to 228, with levels from 0 to 
;          8e8 and put a thermometer on the plot in the theta plane at 
;          theta = 0.  Plot the square root of the sum of the squares
;          of the vector components.
;      ScanF, [1,2,3], 2, 0, 215,  228, 1,lev=[0, 8e8], /ther
;-

on_error, 1

nparams = n_params()
if nparams lt 4 then begin
  print, '% FORMAT: ScanF, FLD, Normal, Normval, Start [, Stop] [, Skip]'
  print, '         [, con=con] [, scale=scale ] [, index=index ]'
  print, '         [, winnum=winnum ] [, _extra=extra ]'
  return
endif

if n_elements(con)    eq 0 then con = 0
if n_elements(scale)  eq 0 then scale = 1.0
if n_elements(index)  eq 0 then index = 0
if n_elements(winnum) eq 0 then winnum = 1
if n_elements(dsmax) eq 0 then dsmax = ndspff()
if n_elements(dsstp) eq 0 then dsstp = 1

if n_elements(dsmin) eq 0 then begin
  print, '% ScanF: First dataset must be specified.'
  return
endif
if dsmax le dsmin then dsmax = ndspff()

nds   = (dsmax - dsmin) / dsstp + 1
if n_elements(xsize) eq 0 then xsize = 448
if n_elements(ysize) eq 0 then ysize = 384

print, nds, ' Frames'

window, winnum, xsiz=xsize, ysiz=ysize

xinteranimate, set = [xsize, ysize, nds]

; Loop over requested frames

frame_num = 0

for ds=dsmin,dsmax,dsstp do begin
  readfld, f, ds

  loctl  = strpos (f.clab, 't = ')
  if loctl lt 0 then loctl  = strpos (f.clab, 't=')
  len    = strlen(f.clab) - loctl + 1
  tlabel = strmid (f.clab, loctl, len)

  plotfld, f, fld, norm, normval, m= scale, con=con, index=index, $
           tit=tlabel, _extra = extrastuff
  xinteranimate, frame=frame_num, window=winnum
  frame_num = frame_num + 1
endfor
wdelete, winnum

if n_elements (block) eq 0 then block = 0
xinteranimate, block = block
;; xinteranimate, /close

return
end
