; ------------------------------------------------------------------------ 
; $Id: scans.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro ScanS, ain, fld, Norm,  dsmin, dsmax, dsstp, $
             con=con, scale=scale, index=index, winnum=winnum, $
             xsize = xsize, ysize = ysize,  _extra = extrastuff
;+
; PURPOSE:
;       Use XINTERANIMATE to build an animation of a DO-list of slices of 
;        a FIELD snapshot
; CALLING SEQUENCE:
;       ScanS, STR, FLD, NORM,  Start [, Stop] [, Skip]
;               [, CON=CON] [, SCALE=SCALE ] [, INDEX=INDEX ]'
;               [, XSIZE = xsize] [, YSIZE = ysize]
;               [, WINNUM=WINNUM ] [, _EXTRA = EXTRASTUFF]
; INPUTS:
;       STR :     Structure array value or field structure
;       FLD :     Field component
;       NORM:     Plane normal for PLOTFLD
;       Start \  
;       Stop -  DO list for frames in the normal direction.
;       Skip  /  
;       Default Value for Stop is number of datasets in current PFF file
;       If Stop le start then stop = number of indices in NORM direction
;       If Skip le 0 or undefined then skip = 1
; OPTIONAL INPUT:
;       -none-
; KEYWORDS:
;       CON:    Conductor structure
;       SCALE:  Scale factor for field (QHRE argument)
;       INDEX:  Index of plane (PLOTFLD option), Default =1
;               If index eq 0 then default then skip must be defined.
;       WINNUM: IDL window number
;       XSIZE:  X Size of the window; Default = 448
;       YSIZE:  Y Size of the window; Default = 384
;
; Example
;       Make a movie using structure array 4 with every other plane
;          and put a thermometer on the plot and use theta as the slice plane
;          Plot the square root of the sum of the squares
;          of the vector components. Scale the field between 0 and 8e8
;      ScanS,4,  [1,2,3],2,1,0,2,lev=[0, 8e8], /ther
;-

on_error, 1
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init

nparams = n_params()
if nparams lt 4 then begin
  print, '% FORMAT: ScanS, STR, FLD, Normal,  Start [, Stop] [, Skip]'
  print, '         [, con=con] [, scale=scale ] [, index=index ]'
  print, '         [, winnum=winnum ] [, _extra=extra ]'
  return
endif

;
      if n_elements(ain) eq 0 then begin
        print, '% ScanS: Structure must be defined'
        return
      endif
; change variables to insure we don't corrupt input data
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, $
   '% ScanS: First parameter must be a structure or a structure array number'
            return
          endif
        endif  else begin
          print, $
    '% ScanS: First parameter must be a structure or a structure array number'
          return
        endelse
      endif
structinit = -1
if n_elements(con)    eq 0 then con = 0
if n_elements(scale)  eq 0 then scale = 1.0
if n_elements(index)  eq 0 then index = 1 else begin
  if index eq 0 then begin
    if n_elements(dsstp) eq 0 then begin
      print, '% ScanS: STEP must be defined if index eq 0'
      return
    endif
    if n_elements(dsmax) eq 0 then dsmax = dsmin
    if dsmax le dsmin then dsmax = a.range(1, normal-1, 0)
  endif
endelse

if n_elements(winnum) eq 0 then winnum = 1
if n_elements(dsmax) eq 0 then dsmax = a.size(normal-1, 0)
if n_elements(dsstp) eq 0 then dsstp = 1

if n_elements(dsmin) eq 0 then begin
  print, '% ScanF: First dataset must be specified.'
  return
endif
if dsmax le dsmin then dsmax = a.size(normal-1, 0)

nds   = (dsmax - dsmin) / dsstp + 1
print, nds, ' Frames in this annimation'
 if nds le 0 then begin
   Print, '% ScanS: Number of frames must be >= 0'
   return
 endif
if n_elements(xsize) eq 0 then xsize = 448
if n_elements(ysize) eq 0 then ysize = 384


window, winnum, xsiz=xsize, ysiz=ysize

xinteranimate, set = [xsize, ysize, nds]

; Loop over requested frames
f = temporary (a)
frame_num = 0
  loctl  = strpos (f.clab, 't = ')
  if loctl lt 0 then loctl  = strpos (f.clab, 't=')
  len    = strlen(f.clab) - loctl + 1
  tlabel = strmid (f.clab, loctl, len)

for ds=dsmin,dsmax,dsstp do begin

  normval = ds
  plotfld, f, fld, norm, normval, m= scale, con=con, index=index, $
           tit=tlabel, _extra = extrastuff
  xinteranimate, frame=frame_num, window=winnum
  frame_num = frame_num + 1
endfor
wdelete, winnum

xinteranimate, /block 

;; xinteranimate, /close

return
end
