; ------------------------------------------------------------------------ 
; $Id: scanv.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro ScanV, fld, Norm, Normval, dsmin, dsmax, dsstp, work = work, $
             con=con, scale=scale, index=index, winnum=winnum, $
             xrange = xrange, yrange = yrange, $
             xsize = xsize, ysize = ysize,  _extra = extrastuff
;+
; PURPOSE:
;       Use XINTERANIMATE to build an animation of a DO-list of 
;       datasets of FIELD snapshots
; CALLING SEQUENCE:
;       ScanF, FLD, NORM, NORM_VAL, Start [, Stop] [, Skip]
;               [, CON=CON] [, INDEX=INDEX ]'
;               [, XSIZE = xsize] [, YSIZE = ysize] [, WORK = work]
;               [, WINNUM=WINNUM ] [, _EXTRA = EXTRASTUFF]
; INPUTS:
;       FLD :     Field components for the vector plot
;                 Must have two or more elements
;                 If 0 or undefined, value = [1,2]
;       NORM:     Plane normal for PLOTFLD
;       NORM_VAL: Value of normal ordinate (PLOTFLD argument)
;       Start \  
;       Stop -  DO list for frames in file.
;       Skip  /  
;       Default Value for Stop is number of datasets in current PFF file
;       If Stop le start then stop = ndspff()
;       If Skip le 0 or undefined then skip = 1
; OPTIONAL INPUT:
;       -none-
; KEYWORDS:
;       CON:    Conductor structure
;       INDEX:  Index of plane (PLOTFLD option)
;       WINNUM: IDL window number
;       WORK  : One or two element vector used to calculate the components.
;               If only one element then work= [work, work+1]
;               Default values are [maxstructure-4, maxstructure-3]
;       XSIZE:  X Size of the window; Default = 448
;       YSIZE:  Y Size of the window; Default = 384
;     _EXTRA :  Valid keyword for PLOTVEC
; 
; Example
;       Make a movie using datasets 215 to 228, with levels from 0 to 
;          8e8 and put a thermometer on the plot in the theta plane at 
;          theta = 0. 
;      ScanV, [3, 1], 2, 0, 215,  228, 1,zrange=[0, 8e8], /ther
;-

on_error, 1

nparams = n_params()
if nparams lt 4 then begin
  print, '% FORMAT: ScanF, FLD, Normal, Normval, Start [, Stop] [, Skip]'
  print, '         [, con=con] [, index=index ]'
  print, '         [, winnum=winnum ] [, WORK = work] [, _extra=extra ]'
  return
endif

if n_elements(con)    eq 0 then con = 0
if n_elements(index)  eq 0 then index = 0
if n_elements(winnum) eq 0 then winnum = 1
if n_elements(dsmax) eq 0 then dsmax = ndspff()
if n_elements(dsstp) eq 0 then dsstp = 1
maxstr = get_maxstr()
nelw = n_elements(work)
if nelw eq 0 then begin
  work = [maxstr-4, maxstr-3]
endif else begin
  if nelw eq 1 then work =[work, work+1] else work = work(0:1)
  if min (work, max = mm) lt 1 or mm gt maxstr then work = [maxstr-4, maxstr-3]
endelse
if n_elements(dsmin) eq 0 then begin
  print, '% ScanF: First dataset must be specified.'
  return
endif
if dsmax le dsmin then dsmax = ndspff()

if n_elements(fld) lt 2 then fld = [1,2]  
nds   = (dsmax - dsmin) / dsstp + 1
if n_elements(xsize) eq 0 then xsize = 448
if n_elements(ysize) eq 0 then ysize = 384
if n_elements(xrange) eq 0 then xrange = 0
if n_elements(yrange) eq 0 then yrange = 0

print, nds, ' Frames'

window, winnum, xsiz=xsize, ysiz=ysize

xinteranimate, set = [xsize, ysize, nds]

; Loop over requested frames

frame_num = 0

for ds=dsmin,dsmax,dsstp do begin
  readfld, f, ds

  loctl  = strpos (f.clab, 't = ')
  if loctl lt 0 then loctl  = strpos (f.clab, 't=')
  len    = strlen(f.clab) - loctl + 1
  tlabel = strmid (f.clab, loctl, len)
  plotfld, f, fld(0), norm, normval, work = work(0),  $
       cn=2, quiet =1, /nfar, index = index, xrange = xrange, yrange = yrange
  plotfld, f, fld(1), norm, normval, work = work(1),  $
       cn=2, quiet=1, /nfar, index = index, xrange = xrange, yrange = yrange
  plotvec, work(0), work(1), m= scale, con=con, index=index, $
           tit=tlabel, _extra = extrastuff, xrange = xrange, yrange = yrange
  if n_elements (con) gt 0 then plotcon, con, /over
  xinteranimate, frame=frame_num, window=winnum
  frame_num = frame_num + 1
endfor
wdelete, winnum

xinteranimate, /block
;; xinteranimate, /close

return
end
