; ------------------------------------------------------------------------ 
; $Id: set_maxstrarray.pro,v 1.4 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  set_maxstrarray, maxstr, quiet = quiet
;
;+
;    Set the maximum number of structure arrays
;
;    format:
;        set_maxstrarray, maxstr [, /quiet]
;        where
;          maxstr - an integer indicating the maximum number of structure arrays
;                   maxstr = abs(long(maxstr)) > 10
;                   Default maximum is 82
;
;          QUIET  - If set no warning will be issued if the number of
;                   datasets is decreased
;
;         Note: If the number of structure arrays is decreased, data can be lost.
;               Any data above maxstr will be deleted.
;
;    Example:
;        Set the number of structure arrays to 102.
;           set_maxstrarray, 102
;-
  on_error, 1
;
@qsstrcom.dat
      if n_elements(maxstr) eq 0 then begin
        print, '%SET_MAXSTRARRAY: No value set. Maximum is unchanged.'
        return
      endif
      if n_elements (maxstructure) gt 0 then oldmax = maxstructure
      maxstructure = abs(long(maxstr)) > 10
;
      if n_elements(structinit) eq 0 then return
      if oldmax eq maxstructure then return
;
      if oldmax gt maxstructure then begin
;
        if keyword_set(quiet) then lcquit=0L else $
          wok_notok, lcquit, ['Number of structure arrays is decreasing', $
          'Do you want to continue?'], 'Set Max Structure Array'
        if lcquit eq 1 then begin
          maxstructure = oldmax
          return
        endif 
        ok = where (strarraycom (maxstructure:oldmax-1) gt 0, nok)
        if nok gt 0 then for i = 0, nok-1 do $
          handle_free, strarraycom(maxstructure+ok(i))
        comment = comment(0:maxstructure-1)
        lcvalidstr = lcvalidstr(0:maxstructure-1)
        strarraycom = strarraycom(0:maxstructure-1)
        return
      endif
      m = maxstructure-oldmax
      comment =[comment, make_array(/string, m)]
      lcvalidstr =[lcvalidstr, make_array( m, value = 0)]
      strarraycom =[strarraycom, make_array( m, /long, value = 0L)]
      return
      end
