;
; ------------------------------------------------------------------------ 
; $Id: vida2str.pro,v 1.4 2010/12/08 16:33:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro vida2str, vida, str, structure = structure, clabel = clabel, $
        dx = dx, dy = dy, dz = dz
;+
;   Convert a vida image to a PFIDL structure
;   Format:
;     vida2str, vida, str [, /structure] [, CLABEL = clabel]
;           [dx = dx, dy = dy, dz = dz]
;
;     where:
;          VIDA   - Vida data type 
;                   1 = Film Intensity
;                   2 = Film Density
;                   3 = 
;          STR    - Variable name or a structure array number
;          STRUCTURE - If set then data is returned in STR.
;                      If not set then data is stored in a structure array.
;          CLABEL - Label used for dataset comment. 
;                   Default is Title_1
;          DX, DY, DZ - Axis spacing for each dimension
;      Example:
;          Make a structure array from the current intensity array in vida
;              and store in picture; use dataset description 'Filtered Image'
;            vida2str, 1, picture, clab = 'Filtered Image', /structure
;
;          Store in structure array, 3, the current density image in vida
;            vida2str, 2, 3
;
;-
; insert a common block
;
on_error, 1
@qsstrcom.dat
;@density_share.cmn

;
;+
; Common block for film density information.
;-
COMMON density_share, film_density, intensity, density_x_dim, density_y_dim, $
                      delta_x, delta_y, dens_min_value, dens_max_value, $
                      inten_min_value, inten_max_value, $
                      density_image, intensity_image
;@title_share.cmn

;
;+
; Common block for title information
;-
COMMON title_share, title_1, title_2, num_titles

;@analysis_share.cmn

;
;
;+
; Common block analysis information
;-
COMMON analysis_share, film_type, x_scale, x_0, y_scale, y_0, x_label, y_label
;
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'VIDA2STR, VIDA, STR  [, /STRUCTURE]'
        print, '    [, CLABEL = clabel] [, DX = dx] [, DY= dy]'
        return
      endif
;
      if nparams lt 2 then begin
        print,'% VIDA2STR: Procedure must have at least 2 parameters'
        return
      endif
;
;
      if n_elements(vida) eq 0 then begin
        print, '% VIDA2STR: First parameter to VIDA2STR must be defined'
        return
      endif

      if not keyword_set(structure) then begin
        if n_elements(str) eq 0 then begin
          print, '% VIDA2STR: Structure array to VIDA2STR must be defined'
          return
        endif
        if str lt 1 or str gt maxstructure then begin
          Print, '% VIDA2STR: Invalid Structure - ', strtrim(str,2)
          print, '  Structure array must be greater than 0 and </='+ $
            strtrim(maxstructure, 2)
          return
        endif
      endif
      if vida lt 1 or vida gt 2 then begin
        print,'% VIDA2STR: VIDA values must be in the range [1,2]'
        return
      endif
      if vida eq 1 or vida eq 2 then begin
        ndim = 2
        dz = 1.
        dx = delta_x*x_scale
        dy = delta_y*y_scale
        nx = density_x_dim
        ny = density_y_dim
        xarray = x_0+dx*findgen(nx)
        yarray = y_0+dy*findgen(ny)
        zarray = 0.0
        
        locx = make_array(ndim, 2)
        locx(0, *)= [0, nx]
        locx(1, *)= [0, ny]
        range = make_array(2, ndim, 2)
        range(0, 0, *) = min(xarray, max = mx)
        range(1, 0, *) = mx
        range(0, 1, *) = min(yarray, max = mx)
        range(1, 1, *) = mx
        two = 3L
        file = ''
        tlab = 'Vida Image'
        if n_elements(clabel) eq 0 then begin
          if vida eq 1 then clab = 'Film Intensity' + ': '+title_1 else $
          clab = 'Film Density' + ': '+title_1
        endif else clab = clabel(0)
        xlab=x_label(0)
        ylab=y_label(0)
        zlab = ''
        if ndim eq 2 then ssize = [nx, ny] else ssize = [nx, ny, nz]
        v1lab = ''
        nvect = 1L
        ntot = nx*ny
        if vida eq 1 then begin
          tempstr  = {type:1L, ndim:ndim, nvect: nvect, block :1L, two: two, $
              size:ssize,range: range, order: 1L, slice:0.0 , locx:locx, $
              dx1:dx, dx2:dy, dx3:0.0, dx4:0.0, $
              locv: [0L, ntot], x1:xarray, x2:yarray, x3:zarray, x4:0.0, $
              v1:intensity, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : xlab, x2lab : ylab, x3lab : zlab, x4lab :'', $
              v1lab : v1lab, v2lab :'', v3lab : '', v4lab : '',v5lab : '', $
              v6lab : '', file : file, spare : [0,0,0,0,0], tlab : tlab, $
              clab: clab }
        endif else $
          tempstr={type:1L, ndim:ndim, nvect:nvect, block:1L, two:two, $
              size:ssize,range: range, order: 1L, slice:0.0 , locx:locx, $
              dx1:dx, dx2:dy, dx3:0.0, dx4:0.0, $
              locv: [0L, ntot], x1:xarray, x2:yarray, x3:zarray, x4:0.0, $
              v1:film_density, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
              x1lab : xlab, x2lab : ylab, x3lab : zlab, x4lab :'', $
              v1lab : v1lab, v2lab :'', v3lab : '', v4lab : '',v5lab : '', $
              v6lab : '', file : file, spare : [0,0,0,0,0], tlab : tlab, $
              clab: clab }
        if keyword_set(structure) then str = temporary(tempstr) else $
          i2s, tempstr, str, /no_copy
      endif
      end
