; ------------------------------------------------------------------------ 
; $Id: vida_axis.pro,v 1.3 2007/06/06 20:09:38 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  vida_axis , wdf1, wdf2
;
;+
;  Modify the AXIS scaling of VIDA data
;    Vida stores offsets in mm and dx in microns.  This routine takes the
;    initial point and changes the dx by a factor of 1000.
;
;  Format:  VIDA_AXIS, STR1 [, STR2]
;  where:
;    STR1, STR2 - Structure array numbers
;    Data in STR1 is converted and stored in STR2.
;    If STR2 is missing, then data is stored in STR1.
;    Data must be type 1 field data.
;
;  Example:
;  Fix data in structure 4 and return to structure 4.
;    VIDA_AXIS, 4
;
;  Fix data in structure 5 and return to structure 6.
;    VIDA_AXIS, 5, 6
;-
;
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
;       return on an error
      on_error, 1
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: VIDA_ASIS, STR_IN [, STR_OUT]'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, '% VIDA_ASIS: Parameter must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxstructure then begin
        print, '% VIDA_ASIS: Structure array must be > 0 and </=', maxstructure
        return
      endif
;
      if n_elements(wdf2) eq 0 then wdf2 = wdf1 else begin
        if wdf2 lt 1 or wdf2 gt maxstructure then begin
        print, '% VIDA_ASIS: Save structure must be > 0 and </=', maxstructure
        return
        endif
      endelse
;
      if lcvalidstr(wdf1-1) eq 0  then begin
        print, '% VIDA_ASIS: No data in the first structure'
        return
      endif
;
      s2i, str, wdf1 
      if str.type ne 1 then begin
        print, '% VIDA_ASIS: Only type 1 structures can be scaled', str.type
        str = 0
        return
      endif

      if str.block ne 1 then begin
        print, '% VIDA_ASIS: Only 1 block structures can be scaled', str.block
        str = 0
        return
      endif

      ok = where(strpos(tag_names(str), 'DX') ge 0 , nok)
      if nok ne 0 then begin
        str.dx1 = str.dx1*1.0e-3
        str.dx2 = str.dx2*1.0e-3
        str.dx3 = str.dx3*1.0e-3
        str.dx4 = str.dx4*1.0e-3
        n = n_elements (str.x1)
        str.x1 = str.x1(0)+ findgen(n)*str.dx1(0)
        str.range(1, 0, *) = str.x1(n-1)
        n = n_elements (str.x2)
        str.x2 = str.x2(0)+ findgen(n)*str.dx2(0)
        str.range(1, 1, *) = str.x2(n-1)
        n = n_elements (str.x3)
        if n gt 1 then $
          str.x3 = str.x3(0)+ findgen(n)*str.dx3(0)
        if str.ndim gt 2 then str.range(1, 2, *) = str.x3(n-1)
      endif else begin
        str.x1 = (str.x1-str.x1(0))*1.0e-3 + str.x1(0)
        str.x2 = (str.x2-str.x2(0))*1.0e-3 + str.x2(0)
        n = n_elements (str.x3)
        if n gt 1 then $
           str.x3 = (str.x3-str.x3(0))*1.0e-3 + str.x3(0)
        str.range(1, 0, *) = max(str.x1)
        str.range(1, 1, *) = max(str.x2)
        if str.ndim gt 2 then str.range(1, 2, *) = max(str.x3)
      endelse  

      i2s, str, wdf2, /no_copy
      return
      end
