; ------------------------------------------------------------------------ 
; $Id: get_saf_dir.pro,v 1.3 2007/06/06 20:10:58 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro get_saf_dir, p1, p2, p3, dataset = dataset, type = type, filename = filename, $
    comment = comment, number = ndset, point = point, line = line, $
     no_trim = no_trim, sortit = sortit
          
;+
;  this routine returns up to four arrays of dataset numbers, dataset types,
;  dataset comments, and the entire dataset line.  The size of these arrays 
;  is returned in number.
;
;   Format: get_saf_dir [,p1] [, p2] [, p3] [, dataset=dataset]
;           [,type=type] [,comment=comment] [, number= ndset] [, POINT= point]
;           [, line = line] [, filename = filename] [, /no_trim] [, /SortIT]
;   where:
;          p1, p2, p2 have the same possibilities as dirpff
;          That is:
;            [, FILEID] [, LOW] [, HIGH]
;        or  [, FILEID] ,STRING
;            where:
;                fileid   - file id of the file
;                low      - first dataset number
;                           If low lt 0 then make a directory +/- low around
;                             current dataset
;                high     - last dataset number
;                string   - search string for the dataset comments
;
;          dataset - integer array of dataset numbers
;          type    - string array of dataset type
;          comment - string array of dataset comments
;          number  - number of elements in the above arrays
;          point   - dataset pointer
;          line    - string array of the entire dataset directory line
;          filename - name of the pff file
;          NO_trim - If set, leave comment as read
;          SortIT  - If set sort directory by dataset comments
;                    Dataset pointer refers to the dataset number.
;                    IF SORTIT is a string, then sorting will begin following 
;                    the string.
;     Example:
;          Get a directory around current dataset of 5 datasets
;            GET_DIR, -5, line = line
;          Get a directory of datasets 95 to 100
;            GET_DIR, 95, 100, line = line
;          Get a listing of all datasets with 'MOCAM2?'
;            GET_DIR, 'MOCAM2?', line = line
;-
      common safcom, initsaf, saf_exe
      common getdircom, junkfilename
;
       on_error, 1
;
      ndset = 0
      if (n_elements(initsaf) eq 0) then initsaf = 0 
      if initsaf eq 0 then begin
        print, '% GET_DIR: No PFF files are open'
        return
      endif
      if n_elements (junkfilename) lt 1 then begin
        spawn,  'echo ',ans, pid=pid
        junkfilename=  '/tmp/IDLdir'+strtrim (pid(0), 2)+'.tmp'
      endif
      file = junkfilename
      fileb = [byte(file), 0B]
      
      if initsaf lt 3 then begin
        junk = call_external (SAF_EXE, 'saf_direct', fileb)
      endif else begin
        print, '% GET_SAF_DIR: This must be a unix system.'
        return
      endelse
;
       c = ' '
       try = 0
tryagain:
       openr, unit, file, /get, /delete, error = err
       if err ne 0 then begin
         print, '% GET_DIR: Open error ', strtrim(err, 2),' - Try to Continue'
         if try eq 10 then message, ' Ten tries to open '+strtrim(file, 2) + $
                ', ERROR = '+strtrim(err, 2)
         try = try +1
         wait, 1
         goto, tryagain
       endif
readit:
       i = 0L
       ndata = 1024L
       dataset = make_array(ndata, /string)
       line = make_array(ndata, /string)
       while (not eof(unit) ) do begin
         readf, unit, c
         if i eq ndata then begin
           ndata = ndata +1024L
           dataset = [dataset, make_array(1024, value = '')]
           line = [line, make_array(1024, value = '')]
         endif
         dataset[i] =  strtrim(c, 2)
         line[i] = string(i+1, format ='(i4)')+'    '+dataset[i]
         i = i+1
       endwhile
       free_lun, unit
       ndset = i
       i = i > 1
       if i eq 0 then i = 1
       dataset = temporary(dataset(0:i-1))
       line = temporary(line(0:i-1))
       end
