; ------------------------------------------------------------------------ 
; $Id: make_one.pro,v 1.3 2007/06/06 20:11:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro make_one, strin, strout, grid = grid

on_error, 1
if n_elements(grid) eq 0 then begin
  print, '% MAKE_ONE: Grid not defined.
  return
endif
gr   = get_valid_structure(grid,'MAKE_ONE', lcstr=lcstr, ierror = ier)

if ier ne 0 then return

if n_elements(strin) eq 0 then begin
  print, '% MAKE_ONE: Input structure not defined.
  return
endif
str   = get_valid_structure(strin,'MAKE_ONE', lcstr=lcstr, ierror = ier)

if ier ne 0 then return

block = indgen(str.block)+1
sb = make_array(gr.block, /long)
for i = 0, gr.block-1 do $
    sb[i] = gr.size[0, i]*gr.size[1, i]*gr.size[2, i] 
locv =[0L, total(sb, /cum)]
vect= make_array(locv[gr.block])
  
for i = 1,gr.block do begin

  ok = where (block gt 0, nok)
  if nok eq 0 then begin
    print, '% MAKE_ONE: NOK eq 0 and we have more blocks
    print, i, gr.block
    return
  endif
  block = block[ok]
  range = reform(gr.range[*,*,i])
  v = make_array(gr.size[*, i-1])
  xg = gr.x1[gr.locx[0,i-1]:gr.locx[0, i]-1]
  yg = gr.x2[gr.locx[1,i-1]:gr.locx[1, i]-1]
  zg = gr.x3[gr.locx[2,i-1]:gr.locx[2, i]-1]
  
  for j = 0, nok-1 do begin
    blo = block[j]
    mid = (str.range[0, *, blo]+str.range[1, *, blo])*0.5

    if (total(mid gt range[0, *]) + total( mid lt range[1, *])) eq 6. then begin
      block[j] =0
      x = str.x1[str.locx[0,blo-1]]
      y = str.x2[str.locx[1,blo-1]]
      z = str.x3[str.locx[2,blo-1]]
      nx = str.size[0, blo-1]
      ny = str.size[1, blo-1]
      nz = str.size[2, blo-1]
      junk = min(abs(xg-x[0]), ix)
      junk = min(abs(yg-y[0]), iy)
      junk = min(abs(zg-z[0]), iz)
      v[ix:ix+nx-1, iy:iy+ny-1, iz:iz+nz-1] = $
          str.v1 [str.locv[blo-1]:str.locv[blo]-1]
    endif
    vect[locv[i-1]:locv[i]-1] = v
  endfor
endfor

  v1lab = str.v1lab[0:gr.block-1]
  tempstr = {type:1L, ndim:gr.ndim, nvect: 1L, block :gr.block, two: 0, $
     size:gr.size, range:gr.range, order:1L, locx: gr.locx, slice:0.0, $
     connect:0L, locv:locv, x1:gr.x1, x2:gr.x2, x3:gr.x3, x4:0.0, $
     v1:vect, v2:0.0, v3:0.0, v4:0.0, v5:0.0, v6:0.0, $
     x1lab : gr.x1lab, x2lab : gr.x2lab, x3lab : gr.x3lab, x4lab :'', $
     v1lab : v1lab , v2lab :'', v3lab : '', v4lab : '',v5lab : '', $
     v6lab : '', file : str.file, spare : [0,0,0,0,0], tlab : str.tlab, $
     clab:  str.clab}

  if n_elements (strout) eq 0 then strout= tempstr else $
    i2s, tempstr, get_save_str(strout, get_maxstr())
  end
