; ------------------------------------------------------------------------ 
; $Id: reduce_saf_array.pro,v 1.3 2007/06/06 20:11:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  reduce_saf_array, x, y, z, nx, ny, nz, v, type = type, $
           tolerance = tol, true_location = truelocation

if type eq 1 then begin
; node data
; delete 1 point the start
  x = x[1:*]
  y = y[1:*]
  z = z[1:*]
  v = v[1:*, 1:*, 1:*]
  return
endif

if not keyword_set(truelocation) then begin
  x = x[1:*]
  y = y[1:*]
  z = z[1:*]

  if n_elements(tol) eq 0 then tol = 0.0

  if type eq 2 then begin
; x edge data
; delete one point and average X
; divide into upper and lower
    vu = v[1:nx-1, 1:ny-1, 1:nz-1]
    vl = v[0:nx-2, 1:ny-1, 1:nz-1]
    vt = vu + vl
    del = tol*abs(vt)
    v = temporary(vt)/((abs(vu) ge del)+(abs(vl) ge del))
    vu = 0 & vl = 0
    return
  endif

  if type eq 3 then begin
; y edge data
; delete one point and average Y
    vu = v[1:nx-1, 1:ny-1, 1:nz-1] 
    vl = v[1:nx-1, 0:ny-2, 1:nz-1] 
    vt = vu + vl
    del = tol*abs(vt)
    v = temporary(vt)/((abs(vu) ge del)+(abs(vl) ge del))
    vu = 0 & vl = 0
    return
  endif

  if type eq 4 then begin 
; z edge data
; delete one point and average Z
    vu = v[1:nx-1, 1:ny-1, 1:nz-1] 
    vl = v[1:nx-1, 1:ny-1, 0:nz-2]
    vt = vu + vl
    del = tol*abs(vt)
    v = temporary(vt)/((abs(vu) ge del)+(abs(vl) ge del))
    vu = 0 & vl = 0
    return
  endif

  if type eq 5 then begin
; x normal face data
    vuu = v[1:nx-1, 1:ny-1, 1:nz-1]
    vlu = v[1:nx-1, 0:ny-2, 1:nz-1]
    vul = v[1:nx-1, 1:ny-1, 0:nz-2]
    vll = v[1:nx-1, 0:ny-2, 0:nz-2]
    vt = vuu + vlu + vul + vll
    del = tol/2.0*abs(vt)
    v = temporary(vt)/((abs(vuu) ge del)+(abs(vll) ge del) + $
                    (abs(vul) ge del)+(abs(vlu) ge del))
    vuu = 0 & vll = 0 & vul=0 & vlu=0
    return
  endif

  if type eq 6 then begin
; y normal face data
    vuu = v[1:nx-1, 1:ny-1, 1:nz-1]
    vlu = v[0:nx-2, 1:ny-1, 1:nz-1]
    vul = v[1:nx-1, 1:ny-1, 0:nz-2]
    vll = v[0:nx-2, 1:ny-1, 0:nz-2]
    vt = vuu + vlu + vul + vll
    del = tol/2.0*abs(vt)
    v = temporary(vt)/((abs(vuu) ge del)+(abs(vll) ge del) + $
                    (abs(vul) ge del)+(abs(vlu) ge del))
    vuu = 0 & vll = 0 & vul=0 & vlu=0
    return
  endif

  if type eq 7 then begin
; z normal face data
    vuu = v[1:nx-1, 1:ny-1, 1:nz-1]
    vlu = v[0:nx-2, 1:ny-1, 1:nz-1]
    vul = v[1:nx-1, 0:ny-2, 1:nz-1]
    vll = v[0:nx-2, 0:ny-2, 1:nz-1]
    vt = vuu + vlu + vul + vll
    del = tol/2.0*abs(vt)
    v = temporary(vt)/((abs(vuu) ge del)+(abs(vll) ge del) + $
                    (abs(vul) ge del)+(abs(vlu) ge del))
    vuu = 0 & vll = 0 & vul=0 & vlu=0
    return
  endif

  if type eq 8 then begin
; element data
    vuu = v[1:nx-1, 1:ny-1, 0:nz-2]
    vlu = v[0:nx-2, 1:ny-1, 0:nz-2]
    vul = v[1:nx-1, 0:ny-2, 0:nz-2]
    vll = v[0:nx-2, 0:ny-2, 0:nz-2]
    vuuu = v[1:nx-1, 1:ny-1, 1:nz-1]
    vluu = v[0:nx-2, 1:ny-1, 1:nz-1]
    vulu = v[1:nx-1, 0:ny-2, 1:nz-1]
    vllu = v[0:nx-2, 0:ny-2, 1:nz-1]
    vt = vuu + vlu + vul + vll + vuuu + vluu + vulu + vllu
    del = tol/4.0*abs(vt)
    v = temporary(vt)/((abs(vuu) ge del)+(abs(vll) ge del) + $
                    (abs(vul) ge del)+(abs(vlu) ge del) + $
                    (abs(vuuu) ge del)+(abs(vllu) ge del) + $
                    (abs(vulu) ge del)+(abs(vluu) ge del))
    vuu = 0 & vll = 0 & vul=0 & vlu=0
    vuuu = 0 & vllu = 0 & vulu=0 & vluu=0
  endif else begin
    print, '% REDUCE_SAF_ARRAY: This dataset type is not supported: ', $
        strtrim (type, 2)
  endelse
  return
endif
;
; Put the data at actual locations.
; Keep the data in the lower buffer cells 
;
; Node data is already done.
  if type eq 1 then return
  if type eq 2 then begin
; x edge data
; divide into upper and lower
    x = (x[1:nx-1] + x[0:nx-2])/2
    y = y[1:*]
    z = z[1:*]
    v = v[0:nx-2, 1:*, 1:*]

  endif else if type eq 3 then begin
; y edge data
; delete one point and average Y
    x = x[1:*]
    y = (y[1:ny-1] + y[0:ny-2])/2
    z = z[1:*]
    v = v[1:*, 0:ny-2, 1:*]

  endif else if type eq 4 then begin
; z edge data
; delete one point and average Z
    x = x[1:*]
    y = y[1:*]
    z = (z[1:nz-1] + z[0:nz-2])/2
    v = v[1:*, 1:*, 0:nz-2]

  endif else if type eq 5 then begin
; x normal face data
    x = x[1:*]
    y = (y[1:ny-1] + y[0:ny-2])/2
    z = (z[1:nz-1] + z[0:nz-2])/2
    v = v[1:*, 0:ny-2, 0:nz-2]

  endif else if type eq 6 then begin
; y normal face data
    x = (x[1:nx-1] + x[0:nx-2])/2
    y = y[1:*]
    z = (z[1:nz-1] + z[0:nz-2])/2
    v = v[0:nx-2, 1:*, 0:nz-2]

  endif else if type eq 7 then begin
; z normal face data
    x = (x[1:nx-1] + x[0:nx-2])/2
    y = (y[1:ny-1] + y[0:ny-2])/2
    z = z[1:*]
    v = v[0:nx-2, 0:ny-2, 1:*]
  endif else if type eq 8 then begin
; element data
    x = (x[1:nx-1] + x[0:nx-2])/2
    y = (y[1:ny-1] + y[0:ny-2])/2
    z = (z[1:nz-1] + z[0:nz-2])/2
    v = v[0:nx-2, 0:ny-2, 0:nz-2]
  endif else begin
    print, '% REDUCE_SAF_ARRAY: This dataset type is not supported: ', $
        strtrim (type, 2)
  endelse
end

