; ------------------------------------------------------------------------ 
; $Id: athwrite.pro,v 1.3 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro athwrite, grid = grid, transp = transp, gonly = gonly, file = file, $
  xonly = xonly, yonly = yonly, append = append, conly = conly
;
;+
;  Write a file to be included in a ATHETA geometry input deck
; format;
;     athwrite, grid = grid, transp = transp, gonly = gonly, file = file, $
;              xonly = xonly, yonly = yonly, append = append, conly = conly
;       if keyword (grid) is set then the conductor values fit to the grid 
;             will be printed.  Default is the actual spatial values.
;       if keyword (transp) is set then x and y will be interchanged for 
;             the output. Default is X = Z and Y = R
;       if keyword (gonly) is set then only grid parameters will be written
;       if file is specified, the data will be written to this file.
;          If no file is specified, a widget will inquire to determine the file
;       if APPEND is set then the data will be appended to the file
;       if XONLY is set then only X-grid data will be written
;       if YONLY is set then only Y-grid data will be written
;       if CONLY is set then only Conductor data will be written
;       if either XONLY or YONLY are set then GONLY is set.
;-
@pfidltq.com
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
 
 on_error, 1
    if n_elements(file) ne 0 then iq = 0 else $
      get_tqstring, iq, 'ENTER the ATHETA File Name:', file
    if n_elements(append) eq 0 then append =0
    if iq eq 0 then begin
      openw, unit, /get, file, error= err, append = append
      if err eq 0 then begin
        if not keyword_set(conly) then begin
          if keyword_set(yonly) then gonly = 1 else begin
            nel = n_elements (qsxgrid)
            if nel gt 0 then begin
              net = n_elements (x1grid)
              i = 0
              repeat begin
                printf, unit,'!', x1grid(i)
                i = i+1
                ok = strpos(x1grid(i), '#')
              endrep until ((i eq net-1) or (ok ne 0))
              if i le net-1 then begin
                ii = i+1
                if keyword_set(transp) then $
                temp = ' GRID    R '+ strmid( x1grid(i), 28, 40) $
                else temp = ' GRID    Z '+ strmid( x1grid(i), 28, 40)
                printf, unit, temp
                if ii le net-2 then $
                  for i = ii, net-2 do printf, unit, x1grid(i)
                printf, unit, x1grid(net-1), '  .'
              endif
            endif
          endelse
          if keyword_set(xonly) then gonly = 1 else begin
            nel = n_elements (qsygrid)
            if nel gt 0 then begin
              net = n_elements (x2grid)
              i = 0
              repeat begin
                printf, unit,'!', x2grid(i)
                i = i+1
                ok = strpos(x2grid(i), '#')
              endrep until ((i eq net-1) or (ok ne 0))
              if i le net-1 then begin
                ii = i+1
                if keyword_set(transp) then $
                temp = ' GRID    Z '+ strmid( x2grid(i), 28, 40) $
                else temp = ' GRID    R '+ strmid( x2grid(i), 28, 40)
                printf, unit, temp
                if ii le net-2 then $
                  for i = ii, net-2 do printf, unit, x2grid(i)
                printf, unit, x1grid(net-1), '  .'
              endif
            endif
          endelse
        endif
;
        if keyword_set (gonly) then free_lun, unit else begin
          athprint, unit, grid = grid, transp= transp
          free_lun, unit
        endelse
      endif else begin
        Print, ' Error = ', strtrim(err, 2)+' on opening'
        print, ' Check the directory file specification'
      endelse
    endif else print, ' Quitting without output'
    return
    end
