; ------------------------------------------------------------------------ 
; $Id: qs2icem.pro,v 1.4 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro qs2icem, fname, filename = filename, multiplier = multiplier,  $
             cm = cm, inch = inch, micron = micron, mil = mil, $
             gxout = gxout, gyout = gyout, gzout = gzout
;+ 
; Format:
;        QS2ICEM, fname [, filename = filename]
;         [, multiplier= multiplier] [, /CM] [, /INCH] [, /MICRON] [, /MIL]
;         [, GXOUT = gxout] [, GYOUT = gyout] [, GZOUT = gzout]
;    where
;      fname    - The name of the output file name.
;      filename - Output file name.  If outfile is present, the keyword is
;                ignored.
;                A widget is used to confirm the name if no name is specified.
;      multiplier - QuickSilver requires MKS units
;                   If ICEM uses other units a multiplier is required.
;            Note: If multiplier is set, the following are ignored.
;      CM      - If set, multiplier = 1e+2
;      Micron  - If set, multiplier = 1E+6
;      Inch    - If set, multiplier = 1/2.54E-2
;      Mil     - If set, multiplier = 1/2.54E-5
;
;      GXOUT      - actual X-grid locations - do not TQPFIDL generated points
;      GYOUT      - actual Y-grid locations - do not TQPFIDL generated points
;      GZOUT      - actual Z-grid locations - do not TQPFIDL generated points
;
;     Example:
; simple axis
; qsgrid, [-55,55],[2.5,2.5], gz, /x, axes =3
; qsgrid, [-55,55],[2.5,2.5], gx, /x, axes =1
; qsgrid, [-100,100],[5,5], gy, /y, axes =2
; qswrite, file ='data.out', /gonly
; qs2icem, file ='qs2icem.out'
;
;  MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 22, 2002
;      Modified: Addded gxout, gyout, gzout keywords - lpm, 10/05/2004
;-
on_error, 1
@pfidltq.com

if n_elements(fname) eq 0 then if n_elements(filename) gt 0 then $
  fname = filename[0]
iq = 0
if n_elements (fname) eq 0 then $
  get_tqstring, iq, 'ENTER the Output File Name:', fname, offset

if iq eq 1 then begin
  Print, '% QS2ICEM: Quitting.'
  return
endif

filename = fname

if n_elements(gxout) eq 0 then gxout = gxfull
if n_elements(gyout) eq 0 then gyout = gyfull
if n_elements(gzout) eq 0 then gzout = gzfull
nx = n_elements(gxout)
ny = n_elements(gyout)
nz = n_elements(gzout)

if nx*ny*nz eq 0 then begin
    print, '% QS2ICEM:  NX*NY*NZ must be greater than zero'
    print, nx, ny, nz
    return
endif

openw, unit, fname, /get, error = error
if error ne 0 then begin
  print, '% QS2ICEM:  Error opening file: ', fname
  print, !error_state.msg
  return
endif
printf, unit, '*Mailleur GLOBAL'
printf, unit, ' '
printf, unit, '*GEO_GRILLE'
printf, unit, nx, ny, nz
printf, unit, '.F. .F. .F.'

formatg ='(10(1x, g0.8))'
formatf ='(10(1x, f0.6))'
ff = formatg
      if keyword_set (multiplier) then mul=multiplier else $
        if keyword_set(cm) then mul = 1e-2 else $
          if keyword_set(micron) then mul = 1e-6 else $
            if keyword_set(inch) then mul = 2.54e-2 else $
              if keyword_set(mil) then mul = 2.54e-5 else mul =1.0

if mul eq 1 then begin
  printf, unit, gxout, format = ff
  printf, unit, gyout, format = ff
  printf, unit, gzout, format = ff
endif else begin
  printf, unit, gxout*mul, format = ff
  printf, unit, gyout*mul, format = ff
  printf, unit, gzout*mul, format = ff
endelse
free_lun, unit
end
