; ------------------------------------------------------------------------ 
; $Id: qsedit.pro,v 1.8 2010/09/21 13:59:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;   This is designed to edit a conductor data for QS input, generate a grid,
;     or edit the grid fitted points for tq input
;   format: qsedit, conductor = conductor, grid = grid, reset = reset, 
;                     fast = fast,xsize= xsize, ysize= ysize, tquick = tquick
;       conductor if nonzero indicates editing conductor data
;       if Keyword TQUICK is set then tqfit will be used.
;       All data must be in the TQ common block
;            reset then grid for conductors
;            default is fast = 0
;                  fast = 1 - unfilled diamonds
;                  fast = 0 - filled circles for the points
;       NOTE: XCUR will print out points to the terminal if the mouse
;             button is pressed.
;       Note: Diagnostic paths can be generated using the SHADOW Conductor button
;
;   Example:
;     Generate conductor data from scratch
;      qscond, [0, 10], [0, 1] ; define a region of interest
;      qsgrid, [0, 10], [.1, .1], /x ; define a reference grid
;      qsgrid, [0, 1], [.1, .1], /y
;      qsedit, /cond ;edit the conductor data
;      (if desired generate a new grid)
;       qsedit, /grid
;      qsfit  ; Fit the data to an existing grid 
;             ; ( or use fit button in qsedit, /grid
;      qsedit ; Edit the grid fitted conductors and add markers
;      qswrite ; write the output to a file
;-
;
 pro qsedit_outline, event
@pfidltq.com
;
@tqeditevent.com
;
on_error, 1
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        qsxgrid, qsygrid,  gxhalf, gyhalf, xyzmult, blocklimits, $
;        nxbin, nybin, xgridtype, ygridtype, $
;        cond, markx, marky, markerstring, lcfit, tqsystem, tqslice, syst_list
;
     npmax = 512
     xx = make_array(npmax)
     yy = make_array(npmax)
     loc = make_array(cond.ncon+1)
     ncondar = loc
     np = 0
     ncon = 0
     for i = 0, cond.ncon-1 do begin
       low = cond.locx(i)
       high = cond.locx(i+1)-1
       ok = where( cond.x(low:high) lt !x.crange(0) or $
                  cond.x(low:high) gt !x.crange(1) , nok)
       if nok eq 0 then begin
         ok = where( cond.y(low:high) lt !y.crange(0) or $
                  cond.y(low:high) gt !y.crange(1) , nok)
         if nok eq 0 then begin
           ncondar(ncon) = i+1
           ncon = ncon+1
           nnew = high-low+1
           np = np+nnew
           loc(ncon) = np
           if nnew+np gt npmax then begin
             npmax = npmax+npmax
             xx = [xx, xx]
             yy = [yy, yy]
           endif
           xx(loc(ncon-1):loc(ncon)-1) = cond.x(low:high)
           yy(loc(ncon-1):loc(ncon)-1) = cond.y(low:high)
         endif 
       endif 

     endfor
   
     if ncon gt 0 then begin
       loc = loc(0:ncon)
       ncondar = ncondar(0:ncon-1)
       xx = xx(0: np-1)
       yy = yy(0: np-1)
       widget_control, event.top,  /clear
       outline_ins, xx, yy, np, loc, lcquit, group = event.top, /modal
       if lcquit eq 0 then begin
         cond_sav = cond
         for i = 0, ncon-1 do tqdelete_cond, ncondar(i)-i
         old_cond = temporary(cond_sav)
         tqcond, xx, yy, loc
       endif
       if tqfast eq 1 then mksymbol, 2 
     endif else winformation, '  No Conductors are wholly within the area'

     plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
     tqplot,/over
     return
     end

 PRO qsedit_event, event
;
;  Common Blocks
;
@pfidltq.com
;
@tqeditevent.com
;  common tqeditevent, curoper, arr, narray, tempx, tempy,old_cond, tqfast, $
;    lcconductor, lcgrid, lcsavmode, lcmode, tqinstr, cerase, draw,  $
;    tempx2, tempy2, conductors, maxcond, cur_win, old_win, lcsegment, $
;    regionbut, lcregion, region_size, condstr, xcur_x, xcur_y, $
;    blocknumber, gridaxes, gridaxis, bgsnap, bghard, meshtype, doittoit, $
;    mult_list, funitmult, fdelta, fendpoint, fncells, lgridlist, $
;    lcfloat, lcsoft, lcmesh, bgaxes, fgeoratio, xgridsave, dxgridsave, $
;     ygridsave, dygridsave, xtypegridsave, ytypegridsave, x2gridsave, $
;    dx2gridsave, y2gridsave, dy2gridsave, xtype2gridsave, ytype2gridsave, $
;    lcgenerate, lcshift, lcaspect, narr
;
;  When a widget is manipulated,
;   put its User Value into the variable 'eventval':
;
      col_offset = 2
      symsize = 1.5
      WIDGET_CONTROL, event.id, GET_UVALUE = eventval
; Perform actions based on the User Value of the event:
;
  CASE eventval OF
   'DRAW_WIN_EVENT': BEGIN
      CASE curoper of
        'ERASE': qsedit_erase, event

        'SPLIT': qsedit_split, event

       'DELETE': BEGIN
;            calculate the nearest on a conductor and delete the conductor
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -2
             if event.press EQ 1 then begin
                ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
                widget_control, draw, /clear_events, sensitive = 0
                device, set_graphics = 3
                if lcregion ge 0 then begin
                  tqdelete_cond, lcsegment
                  tqplot, /over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
                endif
                plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                    yrange = !y.crange
                tqplot, /over
                if lcregion lt 0 then begin
                  arr = -1
                  device, set_graphics = 6
                  widget_control, draw, /clear_events, sensitive = 1
                endif
             endif else begin
                tempregion = 0
                ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                   lcmode, lcsegment, tempregion, region_size, condstr)
                if arr ge 0 then begin
                   if tempx2 ne lcsegment then begin
                     oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        psym =-8, thick =3, color = cerase
                     oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                   endif
                endif else begin
                   oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                endelse
                arr = ip
                tempx2= lcsegment
             endelse
             END

         'JOIN': qsedit_join_con, event

         'MOVE': qsedit_move_pt, event

       'MERASE': qsedit_merase, event
       'SHADOW': Begin
                 if (event.press eq 1) then begin
                   qsedit_shadow, event
                   curoper = 'WAIT'
                   widget_control, draw, /clear_events, sensitive = 0
                 endif
                END

          'ADD': qsedit_add_pt, event

         'ZOOM': BEGIN
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;            a point has been designated
                if narray lt 0 then begin
                   narray = 0
                   arr = coord
                  if n_elements (arr) gt 1 then begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endif
                   widget_control, tqinstr,set_value = $
                   'Indicate opposite corner of the ZOOM region'
                 endif else begin
                   tempx = [min([arr(0), coord(0)], max = maxx) , maxx]
                   tempy = [min([arr(1), coord(1)], max = maxx) , maxx]
                   plot, gx, gy, /nodata, yr = tempy, xr = tempx, /xst, /yst
                   tqplot, /over
;
                   curoper = 'WAIT'
                   tempx = 0
                   widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                  Clear events and make the draw widget insensitive 
                   if lcsavmode eq 'GRID' then   $
                     widget_control, draw, /clear_events, draw_motion_events=0 $
                   else  widget_control, draw, /clear_events, sensitive = 0
                 endelse
             endif else begin
;               we are moving the cursor over the plot
                if narray lt 0 then begin
;                 indicate the current point
                  if n_elements (arr) gt 1 then begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endif
                  arr = coord
                  oplot, [arr(0), arr(0)], !y.crange, color=7, lines = 2
                  oplot, !x.crange, [arr(1), arr(1)], color=7, lines = 2
                endif else begin
                  if (n_elements(tempx) gt 1) then $
                  oplot, [tempx(0), tempx(1), tempx(1), tempx(0), tempx(0)], $
                    [tempy(0), tempy(0), tempy(1), tempy(1), tempy(0)], $
                    color = 0, lines = 2  else begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endelse  
                  tempx = [min([arr(0), coord(0)], max = maxx) , maxx]
                  tempy = [min([arr(1), coord(1)], max = maxx) , maxx]
                  oplot, [tempx(0), tempx(1), tempx(1), tempx(0), tempx(0)], $
                    [tempy(0), tempy(0), tempy(1), tempy(1), tempy(0)], $
                    color = 7, lines = 2
                endelse
              endelse
           END

         'MARKER': BEGIN
;            calculate the nearest grid point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             ip = get_tqgrid(coord(0), coord(1))
             if event.press EQ 1 then begin
                 add_tqmrk , ip
                 device, set_graphics = 3
;                plot, gx, gy, /nodata, xrange = !x.crange, yrange = !y.crange
                 tqplot, /over
;
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                Clear events and make the draw widget insensitive 
                 widget_control, draw, /clear_events, sensitive = 0
              endif else begin
                 if arr(0) gt 0 then plots, gx(arr(0)), gy(arr(1)), $
                   color =cerase,psym = 8, syms =symsize
                 plots, gx(ip(0)),gy(ip(1)), color=cerase,psym=8,syms=symsize
                 arr = ip
              endelse
              END
           
         'D_MARKER': BEGIN
;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             cerase = !d.table_size-1
             ip = get_tqmark(coord(0), coord(1))
             if event.press EQ 1 then begin
                 delete_tqmrk, ip
                 device, set_graphics = 3
                 plot, gx, gy, /nodata, xrange = !x.crange, yrange = !y.crange, $
                    /xst, /yst
                 tqplot, /over
;
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                Clear events and make the draw widget insensitive 
                 widget_control, draw, /clear_events, sensitive = 0
              endif else begin
                 if arr ge 0 then plots, gx(markx(arr)), gy(marky(arr)), $
                    color = cerase, psym = 8, syms =symsize
                 plots, gx(markx(ip)), gy(marky(ip)), color = cerase, $
                     syms =symsize, psym = 8
                 arr = ip
              endelse
              END
      'XFIT' : BEGIN
;           if no button press, ignore
             if event.press EQ 1 then begin
              coord = convert_coord(event.x, event.y, /device, /to_data)
;               a point has been designated
                if n_elements(narray) eq 1 then begin
                   lcsegment = -1
                   narray  = get_tqconductor(coord, lcsegment)
                   plots, psym = 8, color = 5, narray(0), narray(1)
                   widget_control, tqinstr,set_value = $
                   'Indicate the other end point for the line to be fit'
                endif else begin
                  coord  = get_tqconductor(coord, lcsegment)
                  plots, psym = -8, color = 5, [narray(0),coord(0)], $
                    [narray(1),coord(1)]
                  ques =['Do you wish to fit this line?', $
                     'CANNOT UNDO']
                  wok_notok, lcnok, ques,'X Fit Grid', /modal, group_leader = event.top
                  if lcnok eq 0 then begin
                    if old_win ge 0 then begin
                      wset, old_win 
;                     wshow, old_win
                    endif else begin
                      window, 0
                      old_win=0
                    endelse
                    xran = !x.crange
                    yran = !y.crange
                    xfit_tqgrid, narray, coord
                    tqfit
                    wset, cur_win
                    old_cond = cond
                  endif else begin
                    xran = !x.crange
                    yran = !y.crange
                  endelse
                  plot, gx, gy,/nodata,xrange = xran, yrange = yran, /xst, /yst
                  tqplot,/over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                    widget_control, draw, /clear_events, sensitive = 0
                endelse
             endif
             END
      'YFIT' : BEGIN
;           if no button press, ignore
            if event.press EQ 1 then begin
              coord = convert_coord(event.x, event.y, /device, /to_data)
;               a point has been designated
                if n_elements(narray) eq 1 then begin
                   lcsegment = -1 
                   narray  = get_tqconductor(coord, lcsegment)
                   plots, psym = 8, color = 5, narray(0), narray(1)
                   widget_control, tqinstr,set_value = $
                   'Indicate the other end point for the line to be fit'
                endif else begin
                  coord  = get_tqconductor(coord, lcsegment)
                  plots, psym = -8, color = 5, [narray(0),coord(0)], $
                    [narray(1),coord(1)]
                  ques =['Do you wish to fit this line?', $
                     'CANNOT UNDO']
                  wok_notok, lcnok, ques,'Y Fit Grid', /modal, group_leader = event.top
                  if lcnok eq 0 then begin
                    if old_win ge 0 then begin
                      wset, old_win 
;                     wshow, old_win
                    endif else begin
                      window, 0
                      old_win=0
                    endelse
                    xran = !x.crange
                    yran = !y.crange
                    yfit_tqgrid, narray, coord
                    tqfit
                    old_cond= cond
                    wset, cur_win
                  endif else begin
                    xran = !x.crange
                    yran = !y.crange
                  endelse
                  plot, gx, gy,/nodata,xrange =xran, yrange = yran, /yst, /xst
                  tqplot, /over
; 
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                    widget_control, draw, /clear_events, sensitive = 0
                endelse
             endif
             END

           ELSE: begin
;                Clear events and make the draw widget insensitive 
                if lcsavmode eq 'GRID' then begin
                  widget_control, draw, /clear_events, draw_motion_events=0 
                  coord = convert_coord(event.x, event.y, /device, /to_data)
                  if gridaxis eq 0 then begin
                    if lcfloat eq 0 then begin
                      dist = (cond.x-coord(0))^2
                      junk = min(dist, j)
                      coord(0) = cond.x(j)
                    endif
                    widget_control, fendpoint, set_value = coord(0)
                    junk = min(abs(x-coord(0)), narr)
                  endif else begin
                    if lcfloat eq 0 then begin
                      dist = (cond.y-coord(1))^2
                      junk = min(dist, j)
                      coord(1) = cond.y(j)
                    endif
                    widget_control, fendpoint, set_value = coord(1)
                    junk = min(abs(y-coord(1)), narr)
                  endelse
                  widget_control, lgridlist, set_list_select = narr
                  plot, gx, gy,/nodata,xrange = !x.crange, yrange = !y.crange, $
                    /yst, /xst
                  tqplot, /over
                endif else begin
                   widget_control, draw, /clear_events, sensitive = 0
                   wmessage, $
                    'Something is wrong-Pick an activity'
                   curoper = 'WAIT'
                endelse
           END

        ENDCASE
        END

   'ERASE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'ERASE'
                device, set_graphics = 6
                arr = -1
                widget_control, tqinstr,set_value = $
                  'Indicate the point to be removed'
              endif
             END

   'MERASE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
	        WSET, cur_win
                curoper = 'MERASE'
                device, set_graphics = 6
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the points to be removed'
              endif
             END

   'SHADOW': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'SHADOW'
                device, set_graphics = 3
                arr = -1
                widget_control, tqinstr,set_value = $
                  'Indicate the conductor for the SHADOW conductor'
              endif
             END

   'MOVE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
;               Find the point
                  device, set_graphics = 6
                  arr = -1
                  curoper = 'MOVE'
                  narray = -1
                  widget_control, draw, /clear_events, sensitive = 1
                  widget_control, tqinstr,set_value = $
                 'Indicate the point you want to move'
              endif
             END
   'ADD': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
;               Find the point
                  device, set_graphics = 6
                  arr = -1
                  curoper = 'ADD'
                  narray = -1
                  widget_control, draw, /clear_events, sensitive = 1
                  widget_control, tqinstr,set_value = $
                 'Indicate the point immediately before the point to be added.'
              endif
             END

    'UNDO': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                if lcmode eq 'COND' then begin
                  cond =  old_cond
                endif else begin
                  cond = old_cond
                  get_tqbad
                  get_tqplotpnts
                endelse
                plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
                tqplot,/over
;
              endif
             END

   'REDRAW': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                ; Redraw the image
; 
                 maxx = max(gx, min= minx)
                 maxy = max(gy, min= miny)
                 plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
                 yrange = [miny, maxy]
                 tqplot,/over
;
              endif
             END

   'ZOOM': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                widget_control, draw, /clear_events, sensitive = 1, $
                  draw_motion_events = 1
                widget_control, tqinstr,set_value = $
                  'Indicate one corner of the ZOOM region'
                arr = -1
                narray = -1
                curoper = 'ZOOM'
              endif
             END

  'CANCEL': BEGIN
            widget_control, draw, /clear_events, sensitive = 0
            curoper = 'WAIT'
            device, set_graphics = 3
            if lcregion  lt 0 then begin
              cond= temporary(condstr)
              lcregion = -lcregion
            endif
;           Use PLOT to display an image in the draw widget.
;           Redraw the image
            curoper = 'WAIT'
            widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
            plot, gx, gy,/nodata, xran = !x.crange, yran = !y.crange,/xst,/yst
            tqplot,/over
            a = size(condstr)
            END

  'DELKILL': BEGIN
             widget_control, draw, /clear_events, sensitive = 0
             curoper = 'WAIT'
             ; Set the window number - important for multiple windows
             WSET, cur_win
;            
             wok_notok, lcnok, 'Delete ALL the Conductors?', 'Delete ALL', $
                group_leader = event.top, /modal

             if lcnok eq 0 then begin
               xx = cond.x
               yy = cond.y
               loc = cond.locx
               ncon= cond.ncon
               nncon = 0L
               for i = 1, ncon do begin
                 st = loc(i-1) & en = loc(i)-1
                 ok = where (xx(st:en) ge !x.crange(0) and $
                             xx(st:en) le !x.crange(1) and $
                             yy(st:en) ge !y.crange(0) and $
                             yy(st:en) le !y.crange(1), nok)
                 if nok eq 0 then begin
                   nel = en -st
                   nst = loc(nncon)
                   xx(nst:nst+nel) = xx(st:en)
                   yy(nst:nst+nel) = yy(st:en)
                   nncon = nncon+1
                   loc(nncon) = loc(nncon-1)+nel+1
                 endif
               endfor
               if nncon gt 0 then begin
                 xx = xx(0:loc(nncon)-1)
                 yy = yy(0:loc(nncon)-1)
                 loc = loc(0:nncon)
                 old_cond = temporary(cond)
                 lcfit = make_array(nncon+1, /int, value = 0)
                 loci = make_array(nncon+1, /long, value = 0L)
                 locp = make_array(nncon+1, /long, value = 0L)
                 locb = make_array(nncon+1, /long, value = 0L)
                 nbad = make_array(nncon+1, /long, value = 0L)
                 cond = {ncon:nncon, lcfit:lcfit, x:temporary (xx), $
                    y:temporary(yy), locx:temporary(loc), $
                   ix:0L, iy:0L, loci:loci, iplotx:0L, iploty:0L, locp:locp, $
                   badxy:0L, locb:locb, nbad:nbad }
                 maxx = max(gx, min= minx)
                 maxy = max(gy, min= miny)
                 plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
                 yrange = [miny, maxy]
                 tqplot,/over
               endif else winformation, ['Cannot delete all conductors', $
                   'Use tqcond, /reset to eliminate all conductors']
             endif
             END

   'LCOVER': BEGIN
             if curoper eq 'WAIT' then begin
               widget_control, event.id, get_value = lc
               lcgrid = lc(0)
               if n_elements (lc) ge 2 then lcconductor = lc(1)
               if n_elements (lc) eq 3 then lcaspect = lc(2)
               plot, gx, gy,/nodata, xran=!x.crange,/xst, yran=!y.crange, /yst
               tqplot,/over
             endif
           END

   'SELCON': BEGIN
             if curoper eq 'WAIT' then begin
                lcstat = conductors(0)
                widget_control, event.id, get_value = conductors
;
                if conductors(0) ne lcstat then begin
                  conductors = make_array(maxcond+1,value=conductors(0), /int)
                  if cond.ncon lt maxcond then conductors(cond.ncon:*) = 0
                endif else if total(conductors(1:*)) ne maxcond then begin
                  conductors(0) = 0 
                endif else conductors(0) = 1
                widget_control, event.id, set_value= conductors
                plot, gx, gy,/nodata, xran= !x.crange,yran=!y.crange, /xst, /yst
                tqplot,/over
             endif
           END
  'MARKER': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'MARKER'
                arr = -1
                narray = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the point to add the marker'
              endif
             END

  'JOIN': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'JOIN'
                arr = -1
                narray = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the first conductor to join'
              endif
             END

  'SPLIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'SPLIT'
                arr = -1
                widget_control, tqinstr,set_value = $
                  'Indicate the point to split the conductor'
              endif
             END

  'DELCON': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'DELETE'
                arr = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the conductor to delete'
              endif
             END

 'D_MARKER': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                if n_elements(markx) gt 0 then begin
                  if markx(0) ge 0 then begin
                    ; Find the point
                    widget_control, draw, /clear_events, sensitive = 1
                    curoper = 'D_MARKER'
                    arr = -1
                    narray = -1
                 device, set_graphics = 6
                    widget_control, tqinstr,set_value = $
                      'Indicate the point to be removed'
                  endif else print, ' No Markers to delete'
                endif else print, ' No Markers to delete'
              endif
             END

   'XFIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                curoper = 'XFIT'
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the line to be fit'
              endif
             END

   'YFIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                curoper = 'YFIT'
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the line to be fit'
              endif
             END

'LCREGION' : Widget_control, regionbut, get_value = lcregion

   'PRINT' : BEGIN
             if cond.nbad(0) gt 0 then print, $
                  'Bad conductor point must be resolved before printing' $
             else begin
               get_tqstring, iq, 'ENTER the File Name:', file, /modal, group = event.top
               if iq eq 0 then begin
                 openw, unit, /get, file, error= err
                 if err eq 0 then begin
                   tqprint, unit, /modal, group = event.top
                   free_lun, unit
                 endif else begin
                   Print, ' Error = ', strtrim(err, 2)+' on opening'
                   print, ' Check the directory file specification'
                 endelse
               endif
             endelse
             END
   'HELP': BEGIN
            XCUR, xx, yy,dxx, dyy, /modal, /snap, group = event.top
            nel = n_elements(xx)
            if nel gt 0 then begin
              if lcmode eq 'COND' then begin
                print, '    XCUR designated points: (x, y, dx, dy'
                print, xx(0), yy(0)
                if nel gt 1 then $
                  for i = 1, nel-1 do print, xx(i), yy(i), dxx(i-1), dyy(i-1)
              endif else begin
                print, $
               '    XCUR designated points:   Nearest Grid with offset = 2:'
                for i = 0, nel-1 do begin
                  ip = get_tqgrid(xx(i), yy(i))
                  print, xx(i), yy(i), ip(0)+2, ip(1)+2
                endfor
              endelse
              xcur_x = xx
              xcur_y = yy
            endif
            if tqfast eq 1 then mksymbol, 2 else mkcircle

            END

   'DONE': BEGIN
                if old_win ge 0 then wset, old_win
                WIDGET_CONTROL, event.top, /DESTROY
           END
;   multiplier field 
   'FIELD134':

   'UNITMULT': BEGIN
               if event.index lt 7 then begin
                 widget_control, funitmult, sensitive  = 0
                 widget_control, funitmult, set_value = mult_list(event.index)
               endif else widget_control, funitmult, sensitive = 1
               END
              
    'DOITTOIT': BEGIN
              doit = event.value
              xsav = x
              ysav = y
              dxsav = dx
              dysav = dy
              xgrsav = xgridtype
              ygrsav = ygridtype
              lcredraw = 0
              xran = !x.crange
              yran = !y.crange
              if doit lt 2 then begin
; add
                widget_control, fendpoint, get_value = endpoint
                widget_control, fdelta, get_value = delta
                widget_control, fncells, get_value = ncells
                widget_control, fgeoratio, get_value = georatio
                ier = 1
                if delta gt 0 or ncells gt 0 then begin
                  widget_control, funitmult, get_value = mult
                  if gridaxis eq 0 then begin
                    if narr ge 0  and doit eq 1 then begin
                      x(narr) = endpoint
                      if delta gt 0.0 then dx(narr) = delta
                    endif
                    qs_add_pnts, gridaxis, endpoint, delta, lcmesh, ier,$
                      ncells = ncells, soft = lcsoft
                    if ier eq 0 then begin
                      wset, old_win
;                     wshow
                      qsgrid, /x, mult = mult, ratio = georatio, $
                        axes = gridaxes+1
                      wset, cur_win
                      strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype)
                      widget_control, lgridlist, set_value = strgrid
                      junk = min(abs(x-endpoint), narr)
                      if endpoint gt xran(1) or endpoint lt xran(0) $
                        then lcredraw = 1
                    endif else wmessage, 'Cannot add this point'
                  endif else begin
                    if narr ge 0  and doit eq 1 then begin
                      y(narr) = endpoint
                      if delta gt 0.0 then dy(narr) = delta
                    endif
                    qs_add_pnts, gridaxis, endpoint, delta, lcmesh, ier,$
                      ncells = ncells, soft = lcsoft
                    if ier eq 0 then begin
                      wset, old_win
;                     wshow
                      qsgrid, /y, mult = mult, ratio = georatio, $
                        axes = gridaxes+1
                      wset, cur_win
                      strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
                      widget_control, lgridlist, set_value = strgrid
                      junk = min(abs(y-endpoint), narr)
                      if endpoint gt yran(1) or endpoint lt yran(0) $
                        then lcredraw = 1
                    endif else wmessage, 'Cannot add this point'
                  endelse
                endif else wmessage, 'Delta or #Cells must be greater than zero'
                
              endif else if doit eq 2 then begin
;  delete
                if gridaxis eq 0 then endpoint = x(narr) else endpoint = y(narr)
                qs_del_pnts, gridaxis, endpoint, ier
                if ier eq 0 then begin
                  narr = (narr -1)> 0
                  if gridaxis eq 0 then xtemp = x(narr) else xtemp = y(narr)
                  widget_control, fendpoint, set_value = xtemp
                  widget_control, fdelta, set_value = 0.0
                  widget_control, fncells, set_value = 0
                  widget_control, fgeoratio, get_value = georatio
                  widget_control, funitmult, get_value = mult
                  wset, old_win
;                 wshow
                  if gridaxis eq 0 then begin
                    qsgrid, /x, mult = mult, ratio = georatio, $
                      axes = gridaxes+1
                    wset, cur_win
                    strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype)
                    widget_control, lgridlist, set_value = strgrid
                  endif else begin
                    qsgrid, /y, mult = mult, ratio = georatio, $
                      axes = gridaxes+1
                    wset, cur_win
                    strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
                    widget_control, lgridlist, set_value = strgrid
                  endelse
                  widget_control, lgridlist, set_list_select = narr
                endif 
              endif else begin
; paste a list
                widget_control, fendpoint, get_value = endpoint
                widget_control, fgeoratio, get_value = georatio
                widget_control, funitmult, get_value = mult
                if gridaxis eq 0 then begin
                  if n_elements(x_cur) gt 0 then $
                  qs_add_pnts, gridaxis, xcur_x, 0.0, 0, ier, ncells = 5, $
                    soft =1 else ier = 1
                  if ier eq 0 then begin
                    narr = 0
                    wset, old_win
;                   wshow
                    qsgrid, /x, mult = mult, ratio = georatio, $
                      axes = gridaxes+1
                    wset, cur_win
                    strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype)
                    widget_control, lgridlist, set_value = strgrid
                    junk = min(x-x_cur(0), narr)
                  endif else begin
                    wmessage, 'No DATA pasted'
                  endelse
                endif else begin
                  if n_elements(y_cur) gt 0 then $
                    qs_add_pnts, gridaxis, xcur_y, 0.0, 0, ier, ncells = 5, $
                    soft =1 else ier = 1
                  if ier eq 0 then begin
                    narr = 0
                    wset, old_win
;                   wshow
                    qsgrid, /y, mult = mult, ratio = georatio, $
                      axes = gridaxes
                    wset, cur_win
                    strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
                    widget_control, lgridlist, set_value = strgrid
                    junk = min(y-y_cur(0), narr)
                  endif else begin
                    wmessage, 'No DATA to paste'
                  endelse
                endelse
              endelse
              widget_control, fendpoint, get_value = endpoint
              widget_control, fgeoratio, get_value = georatio
              widget_control, funitmult, get_value = mult
              if ier eq 0 then begin
                if  gridaxis eq 0 then begin
                  widget_control, fendpoint, set_value = x(narr)
;                 widget_control,  fdelta, set_value = abs(dx(narr))
                endif else begin
                  widget_control, fendpoint, set_value = y(narr)
;                 widget_control, fdelta, set_value = abs(dy(narr))
                endelse
                widget_control, lgridlist, set_list_select = narr
                if lcredraw gt 0 then begin
                  xran = [min(gx, max = mm), mm]
                  yran = [min(gy, max = mm), mm]
                endif
                lcmode = 'COND'
                plot, gx, gy,/nodata, xrange = xran, yrange = yran, $
                  /xst, /yst
                tqplot,/over
                lcgenerate = lcgenerate+1
                lcshift = lcshift +1
                if lcshift eq 2 then begin
                  x2gridsave = temporary(xgridsave)
                  y2gridsave = temporary(ygridsave)
                  dx2gridsave = temporary(dxgridsave)
                  dy2gridsave = temporary(dygridsave)
                  xtype2gridsave =temporary(xtypegridsave)
                  ytype2gridsave =temporary(ytypegridsave)
                endif
                xgridsave = temporary(xsav)
                ygridsave = temporary(ysav)
                dxgridsave = temporary(dxsav)
                dygridsave = temporary(dysav)
                xtypegridsave =temporary(xgrsav)
                ytypegridsave =temporary(ygrsav)
              endif
              END

    'BGSNAP' : BEGIN
              lcfloat = event.value
              END

    'BGHARD' : BEGIN
              lcsoft = event.value
              END

    'MESHTYPE': BEGIN
              lcmesh = event.value
              widget_control, fgeoratio, sensitive = lcmesh
              END

    'ENDPOINT': BEGIN
              endpoint = event.value
              if  gridaxis eq 0 then begin
                junk = min(abs(x-endpoint), narr) 
              endif else begin
                junk = min(abs(y-endpoint), narr) 
              endelse
              widget_control, lgridlist, set_list_select = narr
              plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
              tqplot,/over
              END

    'GRIDLIST': BEGIN
              j = event.index
              if gridaxis eq 0 then begin
                if j ge n_elements(x) then begin
                  print, 'Problem: j nel-x', j, n_elements(x)
                  j = n_elements(x)-1
                endif
                xx = x(j)
                dxx = dx(j)
                if j eq n_elements(x)-1 then gtyp = xgridtype(j-1) $
                  else gtyp = xgridtype(j)
              endif else begin
                if j ge n_elements(y) then begin
                  print, 'Problem: j nel-y', j, n_elements(y)
                  j = n_elements(y)-1
                endif
                xx = y(j)
                dxx = dy(j)
                if j eq n_elements(y)-1 then gtyp = ygridtype(j-1) $
                  else gtyp = ygridtype(j)
              endelse
              narr = j
              widget_control, fendpoint, set_value = xx
              widget_control, fdelta, set_value = abs(dxx)
              widget_control, lgridlist, set_list_select = narr
              if dxx lt 0 then lcsoft = 0 else lcsoft =1
              widget_control, bghard, set_value = lcsoft
              if gtyp le 0 then lcmesh = 0 else lcmesh = 1
              widget_control, meshtype, set_value = lcmesh
              widget_control, fgeoratio, sensitive = lcmesh
              plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
              tqplot,/over
              END

    'GRIDAXIS': BEGIN
              gridaxis = event.value
              if gridaxis eq 0 then begin
                strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype)
                narr = 0
                xx = x(0)
                dxx = dx(0)
              endif else begin
                strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
                narr = 0
                xx = y(0)
                dxx = dy(0)
              endelse
              widget_control, lgridlist, set_value = strgrid
              widget_control, lgridlist, set_list_select = narr
              widget_control, fendpoint, set_value = xx
              widget_control, fdelta, set_value = abs(dxx)
              gridaxes = gridaxis
              widget_control, bgaxes, set_value = gridaxes
              plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
              tqplot,/over
              END

    'GRIDAXES' : gridaxes = event.value
  
    'UNDO_GRID' : BEGIN
              temp = temporary(x)
              x = temporary(xgridsave)
              xgridsave = temporary(x2gridsave)
              x2gridsave = temporary(temp)
              temp = temporary(y)
              y = temporary(ygridsave)
              ygridsave = temporary(y2gridsave)
              y2gridsave = temporary(temp)
              temp = temporary(dx)
              dx = temporary(dxgridsave)
              dxgridsave = temporary(dx2gridsave)
              dx2gridsave = temporary(temp)
              temp = temporary(dy)
              dy = temporary(dygridsave)
              dygridsave = temporary(dy2gridsave)
              dy2gridsave = temporary(temp)
              temp = temporary(xgridtype)
              xgridtype = temporary(xtypegridsave)
              xtypegridsave = temporary(xtype2gridsave)
              xtype2gridsave = temporary(temp)
              temp = temporary(ygridtype)
              ygridtype = temporary(ytypegridsave)
              ytypegridsave = temporary(ytype2gridsave)
              ytype2gridsave = temporary(temp)
              generate_grid
              END
  
    'GENERATE' : BEGIN
              if lcgenerate gt 0 then begin
                generate_grid
                lcgenerate = 0
                if lcshift gt 1 then lcshift = 0
              endif
              END

    ELSE: Print, eventval,' is not supported at this time.'
    ENDCASE

      END
;
pro qsedit_shadow, event
 on_error, 1
@pfidltq.com
@tqeditevent.com

 coord = convert_coord(event.x, event.y, /device, /to_data)
 xrange = (!x.crange(1)-!x.crange(0))
 yrange = (!y.crange(1)-!y.crange(0))
 ratio = (xrange/yrange)^2
 dis =  (cond.x -coord[0])^2 + ratio* (cond.y -coord[1])^2
 m = min(dis, j)
 ok = where(j lt cond.locx, nok)
 if nok eq 0 then begin
   Print, '% QSEDIT_SHADOW: bad LOCX value'
   print, coord, j, cond.locx
   return
 endif
; conductor number - cc
 cc = ok[0]-1
 xc = cond.x [cond.locx[cc]: cond.locx[cc+1]-1]
 yc = cond.y [cond.locx[cc]: cond.locx[cc+1]-1]
; number of points in the conductor
 npc =  cond.locx[cc+1]-cond.locx[cc]
 if npc lt 3 then begin
   print, '% QSEDIT_SHADOW: Must have more than 2 conductor points.'
   return
 endif
 ngx = n_elements(gx)
 ngy = n_elements(gy)
; Nearest grid point for the conductors.
; IXC and IYC nearest grid point for conductors
; DXC and DYC are derivatives
 ixc = round(findex(gx, xc))
 iyc = round(findex(gy, yc))
 dxc = ixc[1:npc-1] -ixc[0:npc-2]
 dyc = iyc[1:npc-1] -iyc[0:npc-2]
 nix = 512L
 nix = 8L
 nixm = nix-3
 nseg = 0L
 jp = 0L
; IX and IY are the Shadow conductor coordinates
 ix = make_array(nix, /long)
 iy = ix
 x0 = total(xc[0:1])*0.5
 y0 = total(yc[0:1])*0.5
 leng = 5
 wset, cur_win
 plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst

 if dxc[0]*dyc[0] eq 0 then begin
   if dxc[0] ne 0 then y1 = gy[round(total(iyc[0:1])/2+leng) < (ngy-1)] else y1 = y0
   if dyc[0] ne 0 then x1 = gx[round(total(ixc[0:1])/2+leng) < (ngx-1)] else x1 = x0
 endif else begin
   y1 = gy[round(total(iyc[0:1])/2+leng)]
   x1 = (dxc[0]*dyc[0] gt 0) ? $
     gx[round(total(ixc[0:1])/2-leng)] : gx[round(total(ixc[0:1])/2+leng)]
 endelse
 oplot, xc, yc, thick = 3, color= 2
 arrow, x0, y0, x1, y1, /data, /solid, hthick=4,thick=5,col=7

 yes_no_ques, lcno, '% Shadow_Conductor: Is this the direction for the shadow?', $
    'QSEDIT', menu = ['   Y E S   ', '    N O    '], /modal, group= event.top
;
; Get the first two points
;

 if (dxc[jp]*dyc[jp]) eq 0 then begin
;  Horizontal or vertical line
   if dxc[jp] eq 0 then begin
;    Vertical beginning, dx= 0
     ix[0:1] = (lcno) ? ixc[0]-1 : ixc[0]+1
     iy[0:1] = iyc[0:1]
   endif else begin
;    Horizontal, Dy eq 0
     iy[0:1] =  (lcno) ? iyc[0]-1 : iyc[0]+1
     ix[0:1] = ixc[0:1]
   endelse
 endif else begin
;  diagonal
   if (dxc[0]*dyc[0] gt 0) then begin
;    Plus 45 degrees
     ix[0:1] =  (lcno) ? ixc[0:1]+1 : ixc[0:1]-1
     iy[0:1] = iyc[0:1]
   endif else begin
;    Minus 45 degrees
     ix[0:1] =  (lcno) ? ixc[0:1]-1 : ixc[0:1]+1
     iy[0:1] = iyc[0:1]
   endelse
 endelse
 jp = 1
 nseg = 1L
; sequence over the other segments
; JP is segment number starting at zero
; NSEG is the segment for the shadow conductor
;      NSEG will have one more x/y value than segments
 while jp lt (npc -1) do begin
; quit when jp = number of conductor points minus 1
;   since we have npc-1 segments and they start at zero
   if (dxc[jp]*dyc[jp]) eq 0 then begin
;    Horizontal or vertical segment.
     if (dyc[jp] ne 0) then begin
;      Vertical segment, dxc[jp] = 0
       if ixc[jp] ne ix[nseg] then begin
;        X values do not line up - Just continue the lines
         nseg += 1
         jp += 1
         ix[nseg] = ix[nseg-1]
         iy[nseg] = iyc[jp]
       endif else begin
;        X values are equal for shadow and conductor
;        Check on the direction for the last shadow conductor
;        This only occurs when we are doing a horizontal run.
;        After vertical and diagonal runs the Y values are equal.
         dx = ix[nseg] -ix[nseg-1]
;        calculate the position of the first point on this segment
;          versus the shadow conductor
;          DS > 0, conductor above the shadow
;        calculate if motion is toward the shadow or away
;        If motion is away from the conducor, simply go further
;        If motion is toward the conductor, back up one step
         ds = iyc[jp] - iy[nseg]
         isam = 0
         if  (ds*dyc[jp] gt 0) then begin
;          Motion is away from the shadow conductor
           nseg += 1
;          increment the x value to go around the conductor
           if dx eq 0 then begin
             isam = 1
             while ((jp-isam ge 0) && (dxc[jp-isam] eq 0)) do isam = isam+1
             sdx = dxc[jp-isam]/abs(dxc[jp-isam])
           endif else sdx = dx/abs(dx)
           ix[nseg] = ix[nseg-1] + sdx
           iy[nseg] = iy[nseg-1]
           jp += 1
           nseg += 1
           ix[nseg] = ix[nseg-1]
           iy[nseg] = iyc[jp]
           if ix[nseg] eq ix[nseg-1] && iy[nseg] eq iy[nseg-1] then nseg -= 1
         endif else begin
           while dx eq 0 do begin
             isam= isam+1
             dx = ix[nseg-isam] -ix[nseg-isam-1]
           endwhile
           sdx = dx/abs(dx)
;          motion is toward the shadow
           nseg = nseg-isam
;          go the the last time we moved in X to back up
           ix[nseg] = ix[nseg]-sdx
           iy[nseg] = iy[nseg]
           nseg += 1
           jp += 1
;          If the endpoint of the conductor segment is above the shadow
           if (iyc[jp]-iy[nseg-1])*ds gt 0 then begin
             if abs(iyc[jp]-iy[nseg-1]) gt 1 then begin
               ix[nseg] = ix[nseg-1] +sdx
               iy[nseg] = iy[nseg-1]
               nseg += 1
               ix[nseg] = ix[nseg-1]
               iy[nseg] = (ds lt 0) ? iyc[jp]+1 :  iyc[jp]-1
             endif else begin
               ix[nseg] = ix[nseg-1] + sdx
               iy[nseg] = iy[nseg-1]
             endelse
           endif else if (iyc[jp]-iy[nseg-1])*ds lt 0 then begin
             ix[nseg] = ix[nseg-1]
             iy[nseg] = iyc[jp]
           endif else nseg -= 1
         endelse
       endelse
     endif else if (dxc[jp] ne 0 ) then begin
;      Horizontal segment, dyc[jp] eq 0
       if iyc[jp] ne iy[nseg] then begin
;        Y values do not line up - just keep on going
         nseg += 1
         jp += 1
         iy[nseg] = iy[nseg-1]
         ix[nseg] = ixc[jp]
       endif else begin
;        Y values are equal for shadow and conductor
;        Check the direction for the last shadow conductor motion
         dx = ix[nseg] -ix[nseg-1]
         dy = iy[nseg] -iy[nseg-1]
         sdx = (dx eq 0) ? 0 : dx/abs(dx)
;        calculate if motion is toward the shadow or away
;        If ds is positive and direction is positive simply go further
         ds = ixc[jp] - ix[nseg]
         isam=0
         if  (ds*dxc[jp] gt 0) then begin
;          Motion is away from the shadow conductor
           if dy eq 0 then begin
             isam = 1
             while ((jp-isam ge 0) && (dyc[jp-isam] eq 0)) do isam = isam+1
             sdy = dyc[jp-isam]/abs(dyc[jp-isam])
             sdx = (dxc[jp-isam] eq 0) ? 0 : dxc[jp-isam]/abs(dxc[jp-isam])
           endif else sdy = dy/abs(dy)
           nseg += 1
           ix[nseg] = ix[nseg-1] + sdx
           iy[nseg] = iy[nseg-1] + sdy
           jp += 1
           nseg += 1
           iy[nseg] = iy[nseg-1]
           ix[nseg] = ixc[jp]
           if ix[nseg] eq ix[nseg-1] && iy[nseg] eq iy[nseg-1] then nseg -= 1
         endif else begin
;          Motion toward the shadow
           isam = 0
           while dy eq 0 do begin
             isam= isam+1
             dy = iy[nseg-isam] -iy[nseg-isam-1]
             dx = ix[nseg-isam] -ix[nseg-isam-1]
           endwhile
           sdy = dy/abs(dy)
           sdx = (dx eq 0) ? 0 : dx/abs(dx)
           nseg = nseg-isam
           ix[nseg] = ix[nseg]-sdx
           iy[nseg] = iy[nseg]-sdy
           nseg += 1
           jp += 1
           if (ixc[jp]-ix[nseg-1])*ds gt 0 then begin
             if abs(ixc[jp]-ix[nseg-1]) gt 1 then begin
               iy[nseg] = iy[nseg-1]+sdy
               ix[nseg] = ix[nseg-1]+sdx
               nseg += 1
               iy[nseg] = iy[nseg-1]
               ix[nseg] = (ds lt 0) ? ixc[jp]+1 :  ixc[jp]-1
             endif else begin
               iy[nseg] = iy[nseg-1] + sdy
               ix[nseg] = ix[nseg-1]
             endelse
           endif else if (ixc[jp]-ix[nseg-1])*ds lt 0 then begin
             iy[nseg] = iy[nseg-1]
             ix[nseg] = ixc[jp]
           endif else nseg-=1
         endelse
       endelse
;    no motion with this jp
     endif else jp+=1
   endif else begin
;    diagonal segment
;    Move only if none of the ends align
     if (ixc[jp+1] ne ix[nseg]) and (iyc[jp+1] ne iy[nseg]) then begin
       nseg += 1
       ix[nseg] = dxc[jp] + ix[nseg-1]
       iy[nseg] = dyc[jp] + iy[nseg-1]
     endif
     jp += 1
   endelse
;  get the next point
;  see if our array is full
   if nseg ge nixm then begin
     ix = [ix, ix]
     iy = [iy, iy]
     nix += nix
     nixm = nix - 3
   endif
 endwhile
; Look for fold backs
 again = 1
 ix = temporary(ix[0:nseg])
 iy = temporary(iy[0:nseg])
 nseg = nseg+1
 while again gt 0 do begin
;  eliminate double points
   ok = where (((ix[0:nseg-2] - ix[1:nseg-1]) eq 0) and $
          ((iy[0:nseg-2] - iy[1:nseg-1]) eq 0) ,nok, comp = comp, ncom= ns)
   if nok ne 0 then begin
     ix = [ix[comp], ix[nseg-1]]
     iy = [iy[comp], iy[nseg-1]]
     nseg = nseg-nok
   endif
; get folded points
   ok = where (((ix[0:nseg-3] - ix[2:nseg-1]) eq 0) or $
          ((iy[0:nseg-3] - iy[2:nseg-1]) eq 0) ,nok2)
;  double values to be eliminated next loop through the while loop
   if nok2 ne 0 then begin
     ix[ok+1] = ix[ok]
     iy[ok+1] = iy[ok]
   endif
   again = max([nok2, nok])
 endwhile
 tqcond, gx[ix], gy[iy]
 wset, cur_win
 erase
 plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
 tqplot,/over, cond = indgen(cond.ncon)+1
 end

   pro qsedit_move_pt, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5

;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;              a point has been designated
               if narray lt 0 then begin
                 lcsegment = -1
                 ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                    lcmode, lcsegment, lcregion, region_size, condstr)
                 device, set_graphics = 3
                 if lcregion ge 0 then begin
                   narray = ip
                   widget_control, tqinstr,set_value = $
                     'Indicate the new location'
                   if lcmode eq 'INPUT' then begin
                     ist = cond.loci(lcsegment-1)
                     iend = cond.loci(lcsegment)-1
                     if ip eq 0 then j1 = ist+1 else j1 = ist+ip-1
                     if ip+ist eq iend then j2 = iend-1 else j2 = ist + ip+1
                     tempx = [gx(cond.ix(j1)), tempx, gx(cond.ix(j2))]
                     tempy = [gy(cond.iy(j1)), tempy, gy(cond.iy(j2))]
                   endif else begin
                     ist = cond.locx(lcsegment-1)
                     iend = cond.locx(lcsegment)-1
                     if ip eq 0 then j1 = ist+1 else j1 = ist+ip-1
                     if ip+ist eq iend then j2 = iend-1 else j2 = ist + ip+1
                     tempx = [(cond.x(j1)), tempx, (cond.x(j2))]
                     tempy = [(cond.y(j1)), tempy, (cond.y(j2))]
                   endelse
                   arr = ip
                 endif 
                 plot, gx, gy, /nodata, xrange = !x.crange, $
                     yr = !y.crange, /xst, /yst
                 tqplot, /over
                 if narray ge 0 then $
                   oplot, tempx, tempy, psym = -8, thick = 3, color = 2 $
                 else  begin
                   device, set_graphics = 6
                   arr = -1
                 endelse
               endif else begin
                  if lcmode eq 'INPUT' then $
                    new = get_tqgrid(coord(0), coord(1)) else new = coord
                  move_tqpnts, narray, new
                  plot, gx, gy, /nodata, xrange = !x.crange, $
                     yr = !y.crange, /xst, /yst
                  tqplot, /over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                     'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                  widget_control, draw, /clear_events, sensitive = 0
               endelse
             endif else begin
               if narray lt 0 then begin
                 tempregion = 0
                 lcsegment = -1
                 ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                     lcmode, lcsegment, tempregion, region_size, condstr)
                 if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms =symsize,color=cerase
                 plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                 tempx2= tempx
                 tempy2= tempy
                 arr = ip
               endif else begin
                 if lcmode eq 'INPUT' then begin
                   new = get_tqgrid(coord(0), coord(1))
                   new= [gx(new(0)), gy(new(1))]
                 endif else new = coord
                 oplot, tempx, tempy, psym = -8 , thick = 3, color = 0
                   tempx(1)= new(0)
                   tempy(1)= new(1)
                 oplot, tempx, tempy, psym = -8, thick = 3, color = 2
               endelse
             endelse
             return
             END

   pro     qsedit_split, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5
;            calculate the nearest point on a conductor 
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -1
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive = 0
               if lcregion ge 0 then begin
                 if ip eq 0 then ip = 1
                 if ip eq cond.locx(lcsegment)-1-cond.locx(lcsegment-1) then $
                    ip= ip-1
                 if ip ne 0 then begin
                   tqsplit_cond, lcsegment, ip
                   maxcond = cond.ncon
                   conductors = make_array(cond.ncon+1, value=1, /int)
                 endif else  wwarning, $
                    ' Cannot split a two element conductor'
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
               endif
               plot, gx, gy, /nodata, xrange = !x.crange, $
                  yr = !y.crange, /xst, /yst
               tqplot, /over
               if lcregion lt 0 then begin
                  arr = -1
                  widget_control, draw, /clear_events, sensitive = 1
               endif 
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if ip eq 0 then ip = 1
               if ip eq cond.locx(lcsegment)-1-cond.locx(lcsegment-1) then $
                  ip= ip-1
               if arr ge 0 then begin
                  if tempx2 ne lcsegment then $
                    oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                      cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                      psym =-8, thick =3, $
                      color = ((tempx2+col_offset) mod 7  + 1)
               endif
               if ip ne 0 then begin
                  tempx2= lcsegment
                  arr = ip
                  oplot, cond.x(cond.locx(lcsegment-1):  $
                     cond.locx(lcsegment-1)+ip), $
                     cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment-1)+ip), $
                     psym =-8, thick =3, color = 2
                  oplot, cond.x(cond.locx(lcsegment-1)+ip: $
                     cond.locx(lcsegment)-1), $
                     cond.y(cond.locx(lcsegment-1)+ip:cond.locx(lcsegment)-1), $
                     psym =-8, thick =3, color = 5
               endif
             endelse
             return
             END

  pro qsedit_erase, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5
;            calculate the nearest point and delete the point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -1
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive  = 0
               device, set_graphics = 3
               if lcregion ge 0 then begin
                 delete_tqpnts , [ip, ip] 
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
               endif
               plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                    yrange = !y.crange
               tqplot, /over
               if lcregion lt 0 then begin
                 arr = -1
                 device, set_graphics = 6
                 widget_control, draw, /clear_events, sensitive = 1
               endif 
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms=symsize,color=cerase
               plots, tempx, tempy, color = cerase, psym = 8, syms = symsize
               tempx2= tempx
               tempy2= tempy
               arr = ip
             endelse
             return
             end

      pro qsedit_join_con, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5
;            calculate the nearest conductor to the point
;            join the two designated conductors
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -2
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive = 0
               if lcregion lt 0 then begin
                 arr = -1
                 device, set_graphics = 3
                 plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                     yrange = !y.crange
                 tqplot, /over
                 if narray lt 1 then device, set_graphics = 6 else $
                   oplot,cond.x(cond.locx(narray-1):cond.locx(narray)-1),$
                      cond.y(cond.locx(narray-1): $
                      cond.locx(narray)-1), $
                      psym =-8, thick =3, color = 2
               endif else if narray lt 0 then begin
;                first conductor has been picked
                 narray = lcsegment
                 widget_control, tqinstr,set_value = $
                    'Indicate the other conductor to be joined'
                 device, set_graphics = 3
                 oplot,cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                      cond.y(cond.locx(lcsegment-1): $
                      cond.locx(lcsegment)-1), $
                      psym =-8, thick =3, color = 2
                 arr = -1
               endif else begin
                 if lcsegment ne narray then begin
                   tqjoin_cond, lcsegment, narray
                   widget_control, tqinstr,set_value =  $
                     'Pick an Activity or press DONE '
                   curoper = 'WAIT'
                 endif else begin
                   winformation, ['Cannot join a conductor to itself', $
                      'Pick another conductor or CANCEL activity']
                 endelse
                 plot, gx, gy, /nodata, xrange = !x.crange, $
                       yr = !y.crange, /xst, /yst
                 tqplot, /over
                 if curoper eq 'JOIN' then begin
                   oplot,cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                      cond.y(cond.locx(lcsegment-1): $
                      cond.locx(lcsegment)-1), $
                      psym =-8, thick =3, color = 2
                   arr = -1
                 endif
               endelse
               if curoper eq 'JOIN' then $
                   widget_control, draw, /clear_events, sensitive = 1
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if narray lt 0 then begin
                 if arr gt 0 then begin
                    if tempx2 ne lcsegment then begin
                      oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        psym =-8, thick =3, color = cerase
                      oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                        psym =-8, thick =3, color = cerase
                      tempx2= lcsegment
                    endif
                  endif else begin
                    oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                    arr = lcsegment
                    tempx2 = lcsegment
                  endelse
               endif else begin
                 if lcsegment ne narray then begin 
                    if  arr gt 0 then  begin
                      if tempx2 ne lcsegment then begin
                        oplot,cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1),$
                          cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                          psym =-8, thick =3, color = (tempx2+col_offset) mod 7+1
                        oplot, cond.x(cond.locx(lcsegment-1): $
                          cond.locx(lcsegment)-1), $
                          cond.y(cond.locx(lcsegment-1): $
                           cond.locx(lcsegment)-1), $
                          psym =-8, thick =3, color = 5
                        tempx2= lcsegment
                      endif
                    endif else begin
                      oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = 5
                      arr = lcsegment
                      tempx2 = lcsegment
                    endelse
                  endif
               endelse
             endelse
             return
             end
;
    pro qsedit_add_pt, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5
;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;               a point has been designated
                if narray lt 0 then begin
                  device, set_graphics = 3
                  lcsegment = -1
                  narray = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                    lcmode, lcsegment, lcregion, region_size, condstr)
                  if lcregion ge 0 then begin
                    if narray eq 0 then begin
                      if lcmode eq 'INPUT' then begin
                        ist = cond.loci(lcsegment-1)
                        iend = cond.loci(lcsegment)-1
                        cond.ix (ist:iend) = rotate(cond.ix (ist:iend), 2)
                        cond.iy (ist:iend) = rotate(cond.iy (ist:iend), 2)
                        narray = iend-ist
                      endif else begin
                        ist = cond.locx(lcsegment-1)
                        iend = cond.locx(lcsegment)-1
                        cond.x (ist:iend) = rotate(cond.x (ist:iend), 2)
                        cond.y (ist:iend) = rotate(cond.y (ist:iend), 2)
                        narray = iend-ist
                      endelse
                      widget_control, tqinstr,set_value = $
                          'Indicate the new point to be added.'
                    endif
                  endif else narray = -1
                  arr= narray
                endif else begin
                  if lcmode eq 'INPUT' then $
                    new = get_tqgrid(coord(0), coord(1)) else new = coord
                  add_tqpnts , narray, new
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                  widget_control, draw, /clear_events, sensitive = 0
                endelse
                plot, gx, gy, /nodata, xrange = !x.crange, $
                  yr = !y.crange, /xst, /yst
                tqplot, /over
                arr = -1
                if narray lt 0 then device, set_graphics = 6
             endif else begin
;               we are moving the cursor over the plot
                if narray lt 0 then begin
                  lcsegment = -1
                  tempregion = 0
                  ip=   get_tq_conduct (coord(0), coord(1), tempx, tempy, $
                    lcmode, lcsegment, tempregion, region_size, condstr)
;                 indicate the nearest point
                  if arr ge 0 then $
                    plots, tempx2, tempy2, psym=8,syms=symsize,color=cerase
                  plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                  tempx2= tempx
                  tempy2= tempy
                  arr = ip
                endif else begin
                  t = narray+1
                  if lcmode eq 'COND' then begin
                    xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                    yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                    ip = coord
                  endif else begin
                    loc = cond.loci
                    xx = gx(cond.ix(cond.loci(lcsegment-1): $
                        cond.loci(lcsegment)-1))
                    yy = gy(cond.iy(cond.loci(lcsegment-1): $
                        cond.loci(lcsegment)-1))
                    ip = get_tqgrid (coord(0), coord(1))
                    coord(0) =gx(ip(0))
                    coord(1) =gy(ip(1))
                  endelse
                  np = n_elements(xx)
                  if t lt np then  begin 
                    if n_elements(arr) gt 1 then begin
                      oplot, [xx(narray),tempx2,xx(t)], $
                        [yy(narray),tempy2,yy(t)], psym = -8,$
                        color= 0
                    endif
                    oplot, [xx(0:narray), coord(0), xx(t:*)], $
                      [yy(0:narray),coord(1),yy(t:*)],psym= -8, $
                      color= 2
                  endif else begin
                    if n_elements(arr) gt 1 then oplot, $
                        [xx(narray), tempx2], [yy(narray), $
                        tempy2], psym = -8, color = 0
                    oplot, [xx(0:narray), coord(0)], $
                      [yy(0:narray), coord(1)], psym= -8, $
                      color= 2
                  endelse
                  tempx2 = coord(0)
                  tempy2 = coord(1)
                  arr = ip
                endelse
              endelse
              return
              end

;
    pro qsedit_merase, event
;
@pfidltq.com
;
@tqeditevent.com
;
      col_offset = 2
      symsize = 1.5

;            erase multiple points along a line calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;              a point has been designated
               device, set_graphics = 3
               if narray lt 0 then lcsegment = -1
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                 lcmode, lcsegment, lcregion, region_size, condstr)
               if lcregion ge 0 then begin
                 if narray lt 0 then begin
                   narray = ip
                   widget_control, tqinstr,set_value = $
                     'Indicate the other end point for the points to be removed'
                   arr = [ip, ip]
                 endif else begin
                   is = max([narray, ip], min = im)
                   delete_tqpnts , [im, is]
                   curoper = 'WAIT'
                   widget_control, tqinstr,set_value = $
                     'Pick an Activity or press DONE '
;                  Clear events and make the draw widget insensitive 
                   widget_control, draw, /clear_events, sensitive = 0
                 endelse
               endif
               plot, gx, gy, /nodata, xrange = !x.crange, $
                   yr = !y.crange, /xst, /yst
               tqplot, /over
               if narray lt 0 then begin
                 device, set_graphics = 6 
                 plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                 tempx2= tempx
                 tempy2= tempy
                 arr = ip
               endif else if curoper eq 'MERASE' then begin
                 is = max([narray, ip], min = im)
                 s = arr(0)-1
                 t = arr(1)+1
                 arr= [im, is]
                 if lcmode eq 'COND' then begin
                     xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                     yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                 endif else begin
                     xx = gx(cond.ix(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                     yy = gy(cond.iy(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                 endelse
                 np = n_elements(xx)
                 if s gt -1 and t lt np then  begin
                     plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)], color = 0, thick=2,psym=-8,syms=symsize
                 endif
                 s = (arr(0)-1) > 0
                 t = (arr(1)+1) < (np-1)
                 plots, xx, yy, thick=2, psym=-8, color = 2
                 plots, xx(arr(0):arr(1)), yy(arr(0):arr(1)),  $
                      psym=8, color = 1,syms =symsize
                 plots, xx(s:t), yy(s:t), color = 1, thick = 2
                 s = arr(0)-1
                 t = arr(1)+1
                 if s gt -1 and t lt np then $
                 plots, [xx(s),xx(t)], $
                     [yy(s), yy(t)],color=2,psym=-8, syms =symsize
               endif
             endif else begin
               tempregion = 0
               if narray lt 0 then lcsegment = -1
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                 lcmode, lcsegment, tempregion, region_size, condstr)
               if narray lt 0 then begin
                 if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms=symsize,color=cerase
                  plots, tempx, tempy, color = cerase, psym = 8, syms=symsize
                  tempx2= tempx
                  tempy2= tempy
                  arr = ip
               endif else begin
                   s = arr(0)-1 
                   t = arr(1)+1 
                   is = max([narray, ip], min = im)
                   arr= [im, is]
                   if lcmode eq 'COND' then begin
                     xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                     yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                   endif else begin
                     xx = gx(cond.ix(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                     yy = gy(cond.iy(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                   endelse
                   np = n_elements(xx)
                   if s gt -1 and t lt np then  begin 
                     plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)], color = 0,  thick = 2, psym = -8
                   endif
                   s = (arr(0)-1) > 0
                   t = (arr(1)+1) < (np-1)
                   plots, xx, yy, thick=2, psym=-8, color = 2, symsize=symsize
                   plots, xx(arr(0):arr(1)), yy(arr(0):arr(1)),  $
                      psym=8, color = 1,syms =symsize
                   plots, xx(s:t), yy(s:t), thick = 2, color = 1
                   s = arr(0)-1 
                   t = arr(1)+1 
                   if s gt -1 and t lt np then $
                   plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)],color=2,psym=-8, syms =symsize
               endelse
             endelse
             return
             end

 pro generate_grid
;
@pfidltq.com
;
@tqeditevent.com
;
              wset, old_win
;             wshow
              widget_control, funitmult, get_value = mult
              widget_control, fgeoratio, get_value = georatio
              widget_control, blocknumber, get_value = block
              xran = !x.crange(0:1)
              yran = !y.crange(0:1)
              if gridaxis eq 0 then begin
                qsgrid, /x, axes = gridaxes+1, block = block, mult = mult, $
                  ratio = georatio
                strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype)
                widget_control, lgridlist, set_value = strgrid
                if narr gt 0 then narr= narr < n_elements(x) -1
              endif else begin
                qsgrid, /y, axes = gridaxes+1, block = block, mult = mult, $
                  ratio = georatio
                strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
                widget_control, lgridlist, set_value = strgrid
                if narr gt 0 then narr= narr < n_elements(y) -1
              endelse
              wset, cur_win
              if fitsystem eq 0 then qsfit else tqfit
              lcmode = 'INPUT'
;                xran = [min(gx, max = mm), mm]
;                yran = [min(gy, max = mm), mm]
              plot, gx, gy,/nodata, xrange = xran, yrange = yran, $
                /xst, /yst
              tqplot,/over
              return
              end
;
 PRO   qsedit, XSIZE=x_size, YSIZE=y_size, conductor = conductor, $
        grid = grid, reset = reset, modal = modal, GROUP=GROUP, fast = fast, $
        noload = noload, regionsize_in = regionsize, tquick = tquick
;
;+ 
;       QSEDIT
;-
  common pffcom, initpff, pff_exe
@draw_defaults.com

@pfidltq.com
;
@tqeditevent.com
;
  if n_elements(initpff) eq 0 then init_pff

      on_error, 1
      if n_elements (regionsize) eq 0 then region_size = 0.01 else $
         region_size = abs(regionsize) < 1
      if n_elements (lcfit) eq 0 then begin
        if n_elements(gx) gt 1 and n_elements(gy) gt 1 and $ 
          keyword_set(conductor) then begin
          tqcond, [min(gx, max = mxx),mxx] , [ min(gy, max =mxy), mxy]
        endif else begin
          wwarning, ['  QSEDIT cannot be used until the  ', $
                   '  grid and conductors are defined.  ', $
                   '      Use TQGRID and TQCOND.     ']
          return
        endelse
      endif
      if n_elements (cond) eq 0 then begin
        if n_elements(gx) gt 1 and n_elements(gy) gt 1 and $
          keyword_set(conductor) then begin
          tqcond, [min(gx, max = mxx),mxx] , [ min(gy, max =mxy), mxy]
        endif else begin
          wwarning, '  Use TQCOND to input conductor values  '
          return
        endelse
      endif
      narr = -1
      lcaspect = 0
      si = size(cond)
      if si(si(0)+1) ne 8 then begin
        wwarning, '  Use TQCOND to input conductor values  '
        return
      endif
      if cond.ncon lt 1 then begin
        wwarning, '  Use TQCOND to input conductor values  '
        return
      endif
      if keyword_set (tquick) then fitsystem = 1 else fitsystem = 0
      if keyword_set(conductor) then begin
        lcmode= 'COND'
        title = 'Edit Theoretical Conductor Value'
      endif else if keyword_set(grid) then begin
        lcmode= 'GRID'
        title = 'Generate Calculational Grid'
      endif else begin
        lcmode = 'INPUT'
        if n_elements (gx) lt 2 or n_elements (gy) lt 2 then begin
          wwarning, '  Use TQGRID to generate a grid   '
          return
        endif
        if n_elements(cond.iy) lt 2 or n_elements(cond.ix) lt 2 then begin
          wwarning, '  Use TQFIT to fit data to the grid   '
          return
        endif
        if total(cond.lcfit) ne n_elements(cond.lcfit) then begin
          wwarning, ' Use TQFIT to fit data to the grid   '
          return
        endif
        title = 'Generate QS Input Deck Parameters'
      endelse
      lcsavmode = lcmode
      if lcmode eq 'GRID' then begin
        if n_elements (gx) lt 2 or n_elements (gy) lt 2 then begin
          lcmode = 'COND'
        endif else if $
          n_elements(cond.iy) lt 2 or n_elements(cond.ix) lt 2 then begin
          lcmode = 'COND'
        endif else if $
          total(cond.lcfit) ne n_elements(cond.lcfit) then begin
          lcmode = 'COND'
        endif else begin
          lcmode = 'INPUT'
        endelse
      endif
      if n_elements(fast) eq 0 then tqfast = 0 else tqfast = fast(0)
      if n_elements(lcconductor) eq 0 then lcconductor = 0
      if n_elements(lcgrid) eq 0 then lcgrid = 0
      if n_elements(lcaspect) eq 0 then lcaspect = 0
;  do the color table
      if (not keyword_set(noload)) then loadxyct
      device, set_graphics = 3
      cerase = !d.table_size -1
;
      old_win = !d.window
; The size of the draw area is one of the more important parameters
; for a draw widget.  This example uses keywords to define the size
; of the draw area.  An alternative would be to use a fixed size draw area. 
; 
  if n_elements(x_size) eq 0 then begin
    x_size = bigsize(0)
    if lcsavmode eq 'GRID' then begin
      x_size = x_size - 225
    endif
  endif
  if n_elements(y_size) eq 0 then begin
     y_size = bigsize(1)
;     if lcsavmode eq 'GRID' then y_size = y_size *5/6
  endif

;
; Save the initial points.
;   and save the current values in the old values for "REDO"
      if lcmode eq 'COND' then begin
        old_loc = cond.locx
        old_x = cond.x
        old_y = cond.y
      endif else begin
        old_loc = cond.loci
        old_x = cond.ix
        old_y = cond.iy
      endelse
;  Set the base widget
      base = WIDGET_BASE(TITLE = title, /COLUMN)
;
;  Now the Instructional widget base
;
      insbase =  WIDGET_BASE(base, /ROW, /FRAME)
;
;  Now a base with the draw on the left and a column of buttons on the right
;
      dbase =  WIDGET_BASE(base, /ROW, /FRAME)
      lcol =  WIDGET_BASE(dbase,  /FRAME)
      rcol =  WIDGET_BASE(dbase, /COLUMN, /FRAME)
;
; A widget for instructional text is created
;
      tqinstr = WIDGET_LABEL(insbase,  VALUE = 'Instructions:')
      tqinstr = WIDGET_Text(insbase, xsize = 50, edit= 0, $
          value = 'Pick an Activity or press DONE ')
      if lcsavmode eq 'INPUT' then begin
       lcover = cw_bgroup(insbase, ['Overlay Grid   ','Overlay Conductors  '], $
             /nonexclusive, /row, set_value=[lcgrid, lcconductor], $
               uvalue = 'LCOVER')
      endif else if lcsavmode eq 'GRID' then begin
       lcover = cw_bgroup(insbase, ['Overlay Grid ','Overlay Conductors ', $
              ' Aspect 1 : 1'], $
             /nonexclusive, /row, set_value=[lcgrid, lcconductor, lcaspect], $
               uvalue = 'LCOVER')
      endif else begin
        if n_elements (gx) gt 3 or n_elements (gy) gt 3 then $
        lcover = cw_bgroup(insbase, ['Overlay Grid   '], $
             /nonexclusive, /row, set_value=[lcgrid], $
             uvalue = 'LCOVER')
        regionbut =  cw_bgroup(insbase,/frame, ['Point', 'Region'], $
           /exclusive, /row, set_value = 0, $
           LABEL_LEFT='Selection Mode:', $
             uvalue = 'LCREGION')
      endelse
      lcregion = 0
;  A widget called 'draw' is created.
      draw = WIDGET_DRAW(lcol, $
	/BUTTON_EVENTS, $	;generate events when buttons pressed
        /MOTION_EVENTS, $	;generate events when cursor moves over window
	/FRAME, $
	UVALUE = 'DRAW_WIN_EVENT', $
	RETAIN = 2, $		;make sure draw is redrawn when covered
	XSIZE = x_size, $
	YSIZE = y_size)
        if n_elements (mult) eq 0 then mult = 1.0
        if n_elements(x) gt 1 and n_elements(dx) eq n_elements(x) then begin
          if n_elements(xinsav) eq 0 then xinsav= {x:x, dx:dx, gridtype:0}
          if n_elements(x) ne n_elements(nxbin) then qsgrid, /x, mult = mult
        endif
        if n_elements(y) gt 1 and n_elements(dy) eq n_elements(y) then begin
          if n_elements(yinsav) eq 0 then yinsav= {x:y, dx:dy, gridtype:0}
          if n_elements(y) ne n_elements(nybin) then qsgrid, /y, mult = mult
        endif
        if keyword_set(reset)  and keyword_set (conductor) then begin
          gx = [min(cond.x, max = mm), mm]
           x = gx
           dx = 0
          gy = [min(cond.y, max = mm), mm]
           y = gy
           dy = 0
        endif
        if n_elements  (gx ) eq 0 then $
          gx = [min(cond.x, max = mm), mm]
        if n_elements  (gy ) eq 0 then $
          gy = [min(cond.y, max = mm), mm]
        maxx = max(gx, min= minx)
        maxy = max(gy, min= miny)
        maxcond = cond.ncon
;
;  Now the buttons on the right
;
 if lcsavmode eq 'GRID' then begin
        conductors = make_array(maxcond+1, value=1, /int)
        if n_elements (blocklimits) eq 0 then begin
          blocklimits = make_array(2, 3, 1)
          blocklimits(*, 0, 0) = [minx, maxx]
          blocklimits(*, 1, 0) = [miny, maxy]
        endif
        strblock = mk_block_textstr(blocklimits)
        nel = n_elements(x)
        if nel eq 0 then begin
          x = gx
          nel = n_elements(x)
        endif
        neldx = n_elements(dx)
        if neldx ne nel then begin
          dx = x(1:*)-x(0:nel-2)
          dx = [dx(0), dx]*((nel-1)/100.0 < 0.5)
          xinsav= {x:x, dx:dx, gridtype:0}
          qsgrid, /x
        endif
        if nel ne n_elements(nxbin) then qsgrid, /x
        nel = n_elements(y)
        if nel eq 0 then begin
          y = gy
          nel = n_elements(y)
        endif
        neldy = n_elements(dy)
        if neldy ne nel then begin
          dy = y(1:*)-y(0:nel-2)
          dy = [dy(0), dy]*((nel-1)/100.0 < 0.5)
          yinsav= {x:y, dx:dy, gridtype:0}
          qsgrid, /y
        endif
        if nel ne n_elements(nybin) then qsgrid, /y
        if fitsystem eq 0 then qsfit else tqfit

        blklist=WIDGET_LIST( rcol,VALUE=strblock, $
          UVALUE='BLKLIST', $
          YSIZE=2)
        
        BASE7 = WIDGET_BASE(rcol, $
          row=1, $
          MAP=1, $
          UVALUE='BASE7')

        FieldVal126 = 1
        blocknumber = CW_FIELD( BASE7,VALUE=FieldVal126, $
          ROW=1, $
          INTEGER=1, $
          TITLE='Block:', $
          UVALUE='FIELD26', $
          XSIZE=3)

        if n_elements(gridaxis) eq 0 then gridaxis = 0

        gridbut = CW_BGROUP( BASE7, ['X-axis', 'Y-axis'], $
          set_value = gridaxis, ROW=1, $
          EXCLUSIVE=1, /NO_RELEASE, $
          FRAME=1, $
          LABEL_LEFT='Grid Direction:', $
          UVALUE='GRIDAXIS')

         Btns128 = [ $
           'i', $
           'j', $
           'k' ]
         if n_elements(gridaxes) eq  0 then gridaxes = gridaxis
         BGaxes = CW_BGROUP( rcol, Btns128, $
           ROW=1, $
           set_value = gridaxes, $
           EXCLUSIVE=1, /NO_RELEASE, $
           FRAME=1, $
           LABEL_LEFT='Coordinate Direction:', $
           UVALUE='GRIDAXES')

         BASE135 = WIDGET_BASE(rcol, $
           ROW=1, $
           MAP=1, $
           UVALUE='BASE135')

         LABEL132 = WIDGET_LABEL( BASE135, $
           UVALUE='LABEL132', $
           VALUE='Units:')

         ListVal132 = [ $
           'm', $
           'cm', $
           'mm', $
           'micron', $
           'ft', $
           'in', $
           'mil', $
           'user' ]
          LISTunits = WIDGET_LIST( BASE135,VALUE=ListVal132, $
            UVALUE='UNITMULT', YSIZE=3)

          mult_list=[1.0, .01, 0.001, 1.0e-6, .0254*12, 0.0254, 2.54e-5, 0]
  
          funitmult = CW_FIELD( BASE135,VALUE=mult, $
            COLUMN=1, $
            FLOAT=1, $
            TITLE='m/user units:', $
            UVALUE='FIELD134')

         BASE143 = WIDGET_BASE(rcol, $
           COLUMN=1, $
           FRAME=5, $
           MAP=1, $
           UVALUE='BASE143')

         LABEL141 = WIDGET_LABEL( BASE143, $
           UVALUE='LABEL141', $ 
           VALUE='End Point:      Delta:       # Cells:  Type:       ')
         if gridaxis eq 0 then $
           strgrid = mk_grid_textstr(x, dx, nxbin, xgridtype) else $
           strgrid = mk_grid_textstr(y, dy, nybin, ygridtype)
         Lgridlist = WIDGET_LIST( base143,VALUE=strgrid, $
           UVALUE='GRIDLIST', $
           YSIZE=10)

         BASE159 = WIDGET_BASE(BASE143, $
           ROW=1, $
           MAP=1, $
           UVALUE='BASE159')

         if gridaxis eq 0 then endpoint = x(0) else endpoint = y(0)
         narr = 0
         fendpoint = CW_FIELD( BASE159,VALUE=endpoint, $
           ROW=1, $
           FLOAT=1, $
           RETURN= 1, $
           TITLE='End Point:', $
           UVALUE='ENDPOINT', $
           XSIZE=15)

         Btns143 = [ $
           'Snap', 'Float' ]
         if n_elements(lcfloat) eq 0 then lcfloat = 0

         BGsnap = CW_BGROUP( BASE159, Btns143, $
           ROW=1, set_value = lcfloat, $
           EXCLUSIVE=1, /NO_RELEASE, $
           UVALUE='BGSNAP')


         BASE162 = WIDGET_BASE(BASE143, $
           ROW=1, $
           MAP=1, $
           UVALUE='BASE162')

         fdelta = CW_FIELD( BASE162,VALUE=0.0, $
           ROW=1, $
           FLOAT=1, $
           TITLE='Delta:        ', $
           UVALUE='DELTA', $
           XSIZE=15)

         Btns148 = [ 'Hard', 'Soft' ]
         if n_elements(lcsoft) eq 0 then lcsoft = 1

         BGHARD = CW_BGROUP( BASE162, Btns148, $
           ROW=1, $
           set_value = lcsoft, $
           EXCLUSIVE=1, /NO_RELEASE, $
           UVALUE='BGHARD')

         BASE168 = WIDGET_BASE(BASE143, $
           ROW=1, $
           MAP=1, $
           UVALUE='BASE168')

         fncells = CW_FIELD( BASE168,VALUE=0, $
           ROW=1, $
           INTEGER=1, $
           TITLE='# Cells:     ', $
           UVALUE='NCELLS', $
           XSIZE=6)

         fgeoratio = CW_FIELD( BASE168,VALUE=0.9, $
           ROW=1, $
           FLOAT=1, $
           TITLE='Min. Geo. Ratio:', $
           UVALUE='GEORATIO', $
           XSIZE=15)

         Btns152 = [ 'quadratic', 'geometric' ]
         if n_elements(lcmesh) eq 0 then lcmesh = 0

         meshtype = CW_BGROUP( BASE143, Btns152, $
           ROW=1, $
           EXCLUSIVE=1, /NO_RELEASE, $
           set_value = lcmesh, $
           FRAME=1, $
           LABEL_LEFT='Mesh Type:', $
           UVALUE='MESHTYPE')

         Btns154 = [ $
         '     Add     ', $
         '   Replace   ', $
         '   Delete   ', $
         '  Paste List  ' ]

         doittoit = CW_BGROUP( BASE143, Btns154, $
           ROW=1,  /NO_RELEASE, $
           UVALUE='DOITTOIT')

         BASE172 = WIDGET_BASE(rcol, $
           ROW=1, $
           MAP=1, $
           UVALUE='BASE172')

;  Make a button which will zoom the display
         button4 = WIDGET_BUTTON(base172, $
                UVALUE = 'ZOOM', $
                VALUE = '    ZOOM    ')

;  Make a button which will redisplay the image.
         button5 = WIDGET_BUTTON(base172, $
                UVALUE = 'REDRAW', $
                VALUE = '   REDRAW   ')

;  Make a button which will generate the grid and fit conductors 
         button5 = WIDGET_BUTTON(base172, $
                UVALUE = 'GENERATE', $
                VALUE = '   FIT Grid   ')
;
         button1 = WIDGET_BUTTON(base172, $
                UVALUE = 'UNDO_GRID', $
                VALUE = '     UNDO     ')
;
         button1 = WIDGET_BUTTON(base172, $
                UVALUE = 'DONE', $
                VALUE = '    EXIT   ')
;

endif else begin
;  The DONE button:
      button1 = WIDGET_BUTTON(rcol, $
		UVALUE = 'DONE', $
		VALUE = 'DONE - EXIT')
;
;
;  Make a button which will delete a point.
      button16 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MOVE', $
		VALUE = 'MOVE A POINT')

;  Make a button which will delete a point.
      button9 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ERASE', $
		VALUE = 'DELETE A POINT')

;  Make a button which will delete a several points.
      button2 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MERASE', $
		VALUE = 'DELETE a LINE')

;  Make a button which will add a point
      button8 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ADD', $
		VALUE = 'ADD a POINT')

;  Make a button which will zoom the display 
      button4 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ZOOM', $
		VALUE = 'ZOOM')

;  Make a button which will redisplay the image.
      button5 = WIDGET_BUTTON(rcol, $
		UVALUE = 'REDRAW', $
		VALUE = 'REDRAW')

;  Make a button which will cancel the current activity
;   button12 = WIDGET_BUTTON(rcol, UVALUE = 'CONDUC', VALUE = 'PLOT CONDUCTOR')

;  Make a button which will draw the grid
;    button6 = WIDGET_BUTTON(rcol, UVALUE = 'GRID', VALUE = 'PLOT GRID')

   if lcmode eq 'INPUT' then begin

;  Make a button set a marker
      button10 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MARKER', $
		VALUE = 'SET MARKER')

;  Make a button which will delete a marker
      button11 = WIDGET_BUTTON(rcol, $
		UVALUE = 'D_MARKER', $
		VALUE = 'DELETE MARKER')
; Make a buttor for a shadow Conductor
      button11a =  WIDGET_BUTTON(rcol, $
                UVALUE = 'SHADOW', $
                VALUE = 'SHADOW COND')

;  Make a button which will fit the X grid to a line 
      button12 = WIDGET_BUTTON(rcol, $
		UVALUE = 'XFIT', $
		VALUE = 'FIT X to LINE')

;  Make a button which will fit the Y grid to a line
      button13 = WIDGET_BUTTON(rcol, $
		UVALUE = 'YFIT', $
		VALUE = 'FIT Y to LINE')

;  Make a button which will print a TQ input file
      button14 = WIDGET_BUTTON(rcol, $
		UVALUE = 'PRINT', $
		VALUE = 'PRINT TQ FILE')
   endif else begin

;  Make a button which will join a conductor
      button12 = WIDGET_BUTTON(rcol, $
                UVALUE = 'JOIN', $
                VALUE = 'JOIN Conductors')

;  Make a button which will split a conductor
      button13 = WIDGET_BUTTON(rcol, $
                UVALUE = 'SPLIT', $
                VALUE = 'SPLIT Conductors')

; Make a buttor for a shadow Conductor
      button11a =  WIDGET_BUTTON(rcol, $
                UVALUE = 'SHADOW', $
                VALUE = 'SHADOW COND')

;  Make a button which will delete a conductor 
      button14 = WIDGET_BUTTON(rcol, $
                UVALUE = 'DELCON', $
                VALUE = 'DELETE Conductor')

;  Make a button which will delete all conductors
      buttonkill = WIDGET_BUTTON(rcol, $
                UVALUE = 'DELKILL', $
                VALUE = 'Delete ALL Cond.')

;  Make a button which will bring up the outline widget
      button15 = WIDGET_BUTTON(rcol, $
		UVALUE = 'OUTLINE', $
                Event_Pro = 'qsedit_outline', $
		VALUE = 'Outline Conductor')

   endelse
      names = strtrim(sindgen(cond.ncon+1))
      names = ['  ALL', names(1:*)]

   conductors = make_array(maxcond+1, value=1, /int)
;  Make a button which will produce a cursor
      button15 = WIDGET_BUTTON(rcol, $
		UVALUE = 'HELP', $
		VALUE = 'XCUR')

;  Make a button which will cancel the current activity
      button7 = WIDGET_BUTTON(rcol, $
		UVALUE = 'CANCEL', $
		VALUE = 'CANCEL ACTIVITY')

;  Make a button which will undo the last change
      button3 = WIDGET_BUTTON(rcol, $
		UVALUE = 'UNDO', $
		VALUE = ' UNDO Last Action ')

;  Make a compound button  which will select the conductors
      if maxcond lt 255 then $
      sel_cond = CW_BGROUP(rcol, /scrol,y_scrol = 260 , x_scrol = 125, $
		 names, /nonexclusive,set_value = conductors, $ 
               LABEL_TOP='Select Conductor', UVALUE = 'SELCON')

   
 endelse
;
;  Realize the widgets:
;
      WIDGET_CONTROL, base, /REALIZE
;
;  Get the window number from the draw widget.  This can only be done
;  after the widget has been realized.
;
      WIDGET_CONTROL, draw, GET_VALUE=cur_win
      WSET, cur_win
;
; Clear events and make the draw widget insensitive 
;
;
; 
        old_cond = cond
        if tqfast eq 1 then mksymbol, 2 else mkcircle
        widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
       if lcsavmode eq 'GRID' then  begin
         ok = where (abs (mult_list(0:6)-mult) lt 3.05e-5*mult, nok)
         if nok eq 0 then begin
           widget_control, funitmult, sensitive  = 1
           widget_control, listunits, set_list_select = 7
         endif else begin
           widget_control, funitmult, sensitive  = 0
           widget_control, listunits, set_list_select = ok(0)
         endelse
         widget_control, draw, /clear_events, draw_motion_events=0
         if lcmesh eq 0 then $
           widget_control, fgeoratio, sensitive = 0
         lcshift = 0
         lcgenerate = 1
         xgridsave = x
         dxgridsave = dx
         ygridsave = y
         dygridsave = dy
         xtypegridsave =xgridtype
         ytypegridsave =ygridtype
         x2gridsave = xgridsave
         y2gridsave = ygridsave
         dx2gridsave = dxgridsave
         dy2gridsave = dygridsave
         xtype2gridsave = xtypegridsave
         ytype2gridsave = ytypegridsave
       endif else  widget_control, draw, /clear_events, sensitive = 0
        curoper = 'WAIT'
;
        plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
          yrange = [miny, maxy]
;
        if (narr) ge 0 then $
              widget_control, lgridlist, set_list_select = narr
        tqplot, /over
;
;  Hand off control of the widget to the XMANAGER:
;
      XMANAGER, "qsedit", base, GROUP_LEADER=GROUP,  no_block=0

      Condstr = 0
      old_cond =0
;
      END
