; ------------------------------------------------------------------------ 
; $Id: qswrite.pro,v 1.4 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro qswrite, grid = grid, transp = transp, gonly = gonly, file = file, $
     path = path, $
     xonly = xonly, yonly = yonly, append = append, markers = markers
;
;+
;  Write a file to be included in a QS geometry input deck
; format;
;     qswrite, grid = grid, transp = transp, gonly = gonly, file = file, $
;              xonly = xonly, yonly = yonly, path = path, append = append
;       if keyword (GRID) is set then the  conductor values fit to the grid 
;             will be printed.  Default is the actual spatial values.
;       if keyword (TRANSP) is set then x and y will be interchanged
;       if keyword (gonly) is set then only grid parameters will be written
;       if file is specified, the data will be written to this file.
;          If no file is specified, a widget will inquire to determine the file
;       if APPEND is set then the data will be appended to the file
;       if XONLY is set then only X-grid data will be written
;       if YONLY is set then only Y-grid data will be written
;       if either XONLY or YONLY are set then GONLY is set.
;       if PATH is set, output in the form of line segments for a path integral
;         
;-
@pfidltq.com
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
 
 on_error, 1
      if n_elements(file) ne 0 then iq = 0 else $
        get_tqstring, iq, 'ENTER the File Name:', file
      if n_elements(append) eq 0 then append =0
      if iq eq 0 then begin
        openw, unit, /get, file, error= err, append = append
        if err eq 0 then begin
          if keyword_set(path) then begin
            qsprint, unit, transp = transp, path = path
            free_lun, unit
            return
          endif
          if keyword_set(yonly) then gonly = 1 else begin
            nel = n_elements (qsxgrid)
            if nel gt 0 then begin
              net = n_elements (x1grid)
              i = 0
              repeat begin
                printf, unit,'*', x1grid(i)
                i = i+1
                ok = strpos(x1grid(i), '#')
              endrep until ((i eq net-1) or (ok ne 0))
              for i = 0, nel-1 do printf, unit, qsxgrid(i)
            endif
          endelse
          if keyword_set(xonly) then gonly = 1 else begin
            nel = n_elements (qsygrid)
            if nel gt 0 then begin
              net = n_elements (x2grid)
              i = 0
              repeat begin
                printf, unit,'*', x2grid(i)
                i = i+1
                ok = strpos(x2grid(i), '#')
              endrep until ((i eq net-1) or (ok ne 0))
              for i = 0, nel-1 do printf, unit, qsygrid(i)
            endif
          endelse
;
          nel = n_elements (qszgrid)
          if nel gt 0 then begin
            net = n_elements (x3grid)
            i = 0
            repeat begin
                printf, unit,'*', x3grid(i)
                i = i+1
                ok = strpos(x3grid(i), '#')
            endrep until ((i eq net-1) or (ok ne 0))
            for i = 0, nel-1 do printf, unit, qszgrid(i)
          endif
          if keyword_set (gonly) then free_lun, unit else begin
            qsprint, unit, grid = grid, transp= transp
            if keyword_set (markers) then begin
              if n_elements (markx) gt 0 then begin
                if markx(0) ge 0 then begin
                  ns = n_elements(markerstring)
                  for i = 0, ns-1 do printf, unit, markerstring(i)
                endif
              endif
            endif
            free_lun, unit
          endelse
        endif else begin
          Print, ' Error = ', strtrim(err, 2)+' on opening'
          print, ' Check the directory file specification'
        endelse
      endif else print, ' Quitting without output'
      return
      end
