; ------------------------------------------------------------------------ 
; $Id: readdxf.pro,v 1.7 2011/01/21 17:29:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro readdxf, xx, yy,locg, fname, nosort = nosort, resolution = resolution, $
        layers= layervalue, plotall = plotall, insertblocks = insertblocks, $
         arc = nparc, circle = npcircle
;
;+
;  this program reads a dxf file most of the time!!
;  format: readdxf, x, y, loc, file, nosort = nosort, resolution = resolution
;             layers = layers, plotall = plotall, insertblocks = insertblocks
;              arc = arc, circle = circle, range = range
;     
;    where:
;       x are the x-values
;       y are the y-values
;       locg is a pointer to the start locations of each conductor
;           conductor i has values: x = xx(locg(i-1):locg(i)-1)
;                                   y = yy(locg(i-1):locg(i)-1)
;       file is the file name
;             If not specified, pickfile will be used.
;      nosort if present and not zero skip sorting data
;      layers - returns the layer value for each entity
;      plotall - if present and not zero, plots the insert blocks with
;                a value of !p.multi = [0,2,2] unless !p.multi has a nonzero
;                value on entry
;      insertblocks - returns a structure of the insert blocks. values are
;                   x - xvalues
;                   y - yvalues
;                   loc - pointer to each conductor segment
;                   layer - layer value
;                   point - pointer to loc for each insert block
;                   name - insert block names
;       resolution is optional parameter indicating how close
;          points must be to connect them into a single conductor.
;          Default is: resolution = max([xx, yy])/4.0e3
;       ARC and CIRCLE are keywords indicating the number of points to be used 
;                  to make a circle or an arc. Default is 31 points maximum 
;                  for both entities. The number of points is given by:
;                   NP = Circle (Default = 31)
;                   NP = Min(ARC, Angle * (ARC)/180)
;                        (Angle in degrees; Default = 31)
;  this is a prototype code - use with caution; contact paul mix 
;     if you have problems.
;  Splines are currently inserted as control points only!
;
;     Example;
;       Read data from a file, 'raised_cone.dxf'
;       readdxf, x, y, loc, 'raised_cone.dxf'
;-
  on_error, 1

;
      if n_elements(fname) eq 0 then fname = 0
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun
      endif else begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
            return
          endif
          openr,unit,fname, /get_lun
      endelse
;
      if n_elements (npcircle) eq 0 then npcircle = 31
      if n_elements (nparc) eq 0 then nparc = 31
;
str = ''
; set up structures for the insert
      nvertmax = 16384L 
      xvert = make_array(nvertmax)
      layervalue = make_array(nvertmax, /string)
      layernames = make_array(nvertmax, /string)
      yvert = xvert
      xxmax = 32768L
      xx = make_array(xxmax)
      yy = xx
      nstruct = 0
      maxstr = 512
      str_incl= replicate({incl_str, into:'', from:'', xoffset:0.0, $
        yoffset:0.0, zoffset:0.0, xscale:1.0, yscale:1.0, zscale:0.0, $
        rotation: 0.0}, maxstr)
      names = make_array(maxstr, /string)
      point_loc = make_array(2, maxstr, /long)
      point_locn = make_array(maxstr, /long)
      ang = make_array(2)
      x = 0.0
      tempcy = x 
      tempcx = x
      tempr = x
      tempx= make_array(2)
      tempy= tempx
      nvert = 0L
      degang = 0L
; use lcpoly to indicate if a closed polygon
;
      lcpoly = 0L
      locg = [0L, 0L]
      locnames = locg
      nnames = 0
startsect:
      lcentities = 0
      lcblocks = 0
;
      while (not eof(unit) and  strtrim(str, 2) ne 'BLOCKS' and $
        strtrim(str, 2) ne 'ENTITIES' and strtrim(str, 2) ne 'EOF') do $
        readf, unit, str 
      if strtrim(str, 2) eq 'BLOCKS' then lcblocks = 1 else $
      if strtrim(str, 2) eq 'ENTITIES' then lcentities = 1 else $
      if strtrim(str, 2) eq 'EOF' then goto, finished_read else begin
        print, 'End of file encountered - QUITTING'
        goto, finished_read
      endelse

startdata:
      while strtrim(str, 2) ne '0' and not eof(unit) do readf, unit, str
aftererror:
      readf, unit, str
      str = strtrim(str, 2)
      if strlen(str) lt  3   then goto, startdata
;
      case str of

'LINE': BEGIN
        nx = 0L
        ny = 0L
        layer = '0'
     processline:
        readf, unit, str
        if strtrim(str) eq '  0' then begin
          print, 'Error processing data'
          print, 'LINE:', nx, tempx, ny, tempy 
          goto, aftererror
        endif
        istr = fix(str)
        if istr eq 8 then begin
;         this is layer name 
          readf, unit, str
          layer = str
          goto, processline
        endif else if istr eq 6 then begin
;         this is line type name - accept anything
          readf, unit, str
        endif else if istr ge 20 and istr lt 29 then begin
          readf, unit,x
          if ny lt 2 then tempy(ny) = x else begin
            print, tempy , ' only 2 Y values per line segment'
            goto, startdata
          endelse
          ny = ny+1
        endif else if istr ge 10 and istr lt 19 then begin
          readf, unit,x
          if nx lt 2 then tempx(nx) = x else begin 
            print, tempx , ' only 2 X values per line segment'
            goto, startdata
          endelse
          nx = nx+1
        endif else readf, unit, str
        if nx eq 2 and ny eq 2 then begin
          nel = n_elements (locg)
          if locg(nel-1) ge xxmax -2  then begin
            xxmax = xxmax+xxmax
            xx = [xx, xx]
            yy = [yy, yy]
            layervalue = [layervalue, layervalue]
          endif
          if locg(1) eq 0 then begin
             xx(0:1) = tempx
             yy(0:1) = tempy
             locg(1) = 2
             layervalue(0) = layer
          endif else begin
             xx(locg(nel-1): locg(nel-1)+1) = tempx
             yy(locg(nel-1): locg(nel-1)+1) = tempy
             locg = [locg, locg(nel-1)+2]
             layervalue(nel-1) = layer 
          endelse
        goto, startdata
        endif
        goto, processline
        END

'SOLID': BEGIN
;       do a partial job with solids.
        nx = 0L
        ny = 0L
        xsolid = make_array(5)
        ysolid = make_array(5)
        layer = '0'
     processsolid:
        readf, unit, str
        if strtrim(str, 2) eq '0' then begin
          if nx ne ny then begin
            print, 'Error Processing Solid data'
            print, nx, ny, xsolid, ysolid
          endif else begin
            nel = n_elements (locg)
            xsolid(nx) = xsolid(0)
            ysolid(nx) = ysolid(0)
            if locg(nel-1) ge xxmax -nx  then begin
              xxmax = xxmax+xxmax
              xx = [xx, xx]
              yy = [yy, yy]
              layervalue = [layervalue, layervalue]
            endif
            if locg(1) eq 0 then begin
              xx(0:nx) = xsolid(0:nx)
              yy(0:nx) = ysolid(0:nx)
              locg(1) = nx+1
              layervalue(0) = layer
            endif else begin
              xx(locg(nel-1): locg(nel-1)+nx) = xsolid(0:nx)
              yy(locg(nel-1): locg(nel-1)+nx) = ysolid(0:nx)
              locg = [locg, locg(nel-1)+nx+1]
              layervalue(nel-1) = layer 
            endelse
          endelse
          goto, aftererror
        endif
        istr = fix(str)
        if istr eq 8 then begin
;         this is layer name 
          readf, unit, str
          layer = str
          goto, processsolid
        endif else if istr eq 6 then begin
;         this is line type name - accept anything
          readf, unit, str
        endif else if istr ge 10 and istr lt 19 then begin
          readf, unit,x
          xsolid(istr-10) = x
          nx = nx+1
        endif else if istr ge 20 and istr lt 29 then begin
          readf, unit,x
          ysolid(istr-20) = x
          ny = ny+1
        endif else readf, unit, str
        goto, processsolid
        END

 'ARC': BEGIN
        nx = 0
        ny = 0
        nr = 0
        layer = '0'
        nang = 0l
      processarc:
        readf, unit, str
        if strtrim(str) eq '  0' then begin
          print, 'Error processing data'
          print, 'ARC:', nx, tempcx, ny, tempcy , nr, nang, ang, tempr
          goto, aftererror
        endif
        istr = fix(str)
        if istr eq 8 then begin
;         this is layer name
          readf, unit, str
          layer = str
          goto, processarc
        endif else if istr eq 6 then begin
;         this is line type name - ignore
          readf, unit, str
;         if strtrim(str, 2) ne 'CONTINUOUS' then goto, startdata
        endif else if istr ge 20 and istr lt 29 then begin
          if ny lt 1 then readf, unit, tempcy else begin 
            print, tempcy , ' only 1 Y value per arc segment'
            goto, startdata
          endelse
          ny = ny+1
        endif else if istr ge 10 and istr lt 19 then begin
          if nx lt 1 then readf, unit, tempcx else begin 
            print, tempcx , ' only 1 X value per arc segment'
            goto, startdata
          endelse
          nx = nx+1
        endif else if istr ge 40 and istr lt 49 then begin
          if nr lt 1 then readf, unit, tempr else begin 
            print, tempr , ' only 1 RAD value per arc segment'
            goto, startdata
          endelse
          nr = nr+1
        endif else if istr ge 50 and istr lt 59 then begin
          degang = 0d0
          readf, unit,degang
          if nang lt 2 then ang(nang) = degang else begin 
            print, ang , ' only 2 ANGLE values per arc segment'
            goto, startdata
          endelse
          nang = nang+1
        endif else readf, unit, str
        if nx eq 1 and ny eq 1 and nr eq 1 and nang eq 2 then begin
          if ang (0) gt ang(1) then ang(0)= ang(0)-360
          n = round ((ang(1)-ang(0)+1)*nparc/180) < nparc 
          n = n >5
          thet = findgen(n)/(n-1)*(ang(1)-ang(0)) +ang(0)
          thet = thet*!dtoR
          xarc= tempcx+cos(thet)*tempr
          yarc= tempcy+sin(thet)*tempr
          nel = n_elements (locg)
          if locg(nel-1) ge xxmax-n  then begin
            xxmax = xxmax+xxmax
            xx = [xx, xx]
            yy = [yy, yy]
          endif
          if locg(1) eq 0 then begin
              xx(0:n-1) = xarc
              yy(0:n-1) = yarc
              locg(1) = n
              layervalue(0) = layer
          endif else begin
              xx(locg(nel-1): locg(nel-1)+n-1) = xarc
              yy(locg(nel-1): locg(nel-1)+n-1) = yarc
              locg = [locg, locg(nel-1)+n]
              layervalue(nel-1) = layer
          endelse
          goto, startdata
        endif
        goto, processarc
        END

 'CIRCLE': BEGIN
        nx = 0
        ny = 0
        nr = 0
        layer = '0'
        processcir:
          readf, unit, str
          if strtrim(str) eq '  0' then begin
            print, 'Error processing data'
            print, 'CIRCLE:', nx, tempcx, ny, tempcy , tempr
            goto, aftererror
          endif
          istr = fix(str)
          if istr eq 8 then begin
;           this is layer name - ignore
            readf, unit, str
            layer = str
            goto, processcir
          endif else if istr eq 6 then begin
;           this is line type name - Ignore
            readf, unit, str
;           if strtrim(str, 2) ne 'CONTINUOUS' then goto, startdata
          endif else if istr ge 20 and istr lt 29 then begin
            if ny lt 1 then readf, unit, tempcy else begin 
              print, tempcy , ' only 1 Y value per arc segment'
              goto, startdata
            endelse
            ny = ny+1
          endif else if istr ge 10 and istr lt 19 then begin
            if nx lt 1 then readf, unit, tempcx else begin 
              print, tempcx , ' only 1 X value per arc segment'
              goto, startdata
            endelse
            nx = nx+1
          endif else if istr ge 40 and istr lt 49 then begin
            if nr lt 1 then readf, unit, tempr else begin 
              print, tempr , ' only 1 RAD value per arc segment'
              goto, startdata
            endelse
            nr = nr+1
          endif else readf, unit, str
          if nx eq 1 and ny eq 1 and nr eq 1  then begin
            n = npcircle
            thet = findgen(n)*(360./(n-1))
            thet = thet*!dtoR
            xarc= tempcx+cos(thet)*tempr
            yarc= tempcy+sin(thet)*tempr
            nel = n_elements (locg)
            if locg(nel-1) ge xxmax-n  then begin
              xxmax = xxmax+xxmax
              xx = [xx, xx]
              yy = [yy, yy]
            endif
            if locg(1) eq 0 then begin
                xx(0:n-1) = xarc
                yy(0:n-1) = yarc
                locg(1) = n
                layervalue(0) = layer
            endif else begin
                xx(locg(nel-1): locg(nel-1)+n-1) = xarc
                yy(locg(nel-1): locg(nel-1)+n-1) = yarc
                locg = [locg, locg(nel-1)+n]
                layervalue(nel-1) = layer
            endelse
            goto, startdata
          endif
        goto, processcir
        END

'SPLINE': Begin
          lcspline = 0
          npoint = 0
          max_np = 0
          xinit = 0.
          yinit = 0.
          nx = 0
          ny = 0
          layer = '0'
          knot = 0.0
          processspline:
            readf, unit, str
            if strtrim(str) eq '  0' then begin
              print, 'Error processing data'
              print, 'SPLINE:', nx, ny, max_np, npoint
              goto, aftererror
            endif
            istr = fix(str)
            if istr eq 8 then begin
;               this is layer name - ignore
                readf, unit, str
;               layer = str
                goto, processspline
            endif else  if istr eq 40 or istr eq 41 then begin
;               this is the knot value or a weight, we ignore
                readf, unit, knot
            endif else if istr eq 73 then begin
                readf, unit, max_np
                xsp = make_array(max_np)
                ysp = xsp
            endif else if istr eq 10 then begin
                readf, unit, xinit
                if nx eq max_np then begin
                  print, 'SPLINE: NX gt number of control points'
                  goto, aftererror
                endif
                xsp[nx] = xinit
                nx = nx+1
            endif else if istr eq  20  then begin
                readf, unit, yinit
                if ny eq max_np then begin
                  print, 'SPLINE: NY gt number of control points'
                  goto, aftererror
                endif
                ysp[ny] = yinit
                ny = ny+1
            endif else readf, unit, str
            if ny gt 1 && ny eq max_np && nx eq max_np then begin
              nel = n_elements (locg)
              if locg(nel-1) ge xxmax-n  then begin
                xxmax = xxmax+xxmax
                xx = [xx, xx]
                yy = [yy, yy]
              endif
              if n_elements(xsp) lt nx then begin
                xsp = xsp[0:nx-1]
                ysp = ysp[0:nx-1]
              endif
              if locg(1) eq 0 then begin
                  xx(0:nx-1) = xsp
                  yy(0:nx1) = ysp
                  locg(1) = nx
                  layervalue(0) = layer
              endif else begin
                  help, xx, nx, ny, xsp, ysp
                  xx(locg(nel-1): locg(nel-1)+nx-1) = xsp
                  yy(locg(nel-1): locg(nel-1)+ny-1) = ysp
                  locg = [locg, locg(nel-1)+nx]
                  layervalue(nel-1) = layer
              endelse
              goto, startdata
           endif
           goto, processspline
         END

'POLYLINE': BEGIN
            lcpoly = 0
            xinit = 0.
            yinit = 0.
            layer = '0'
            zinit = 0.
            polywidth = 0.0
            processpoly:
              readf, unit, str
              if strtrim(str) eq '  0' then begin
                goto, aftererror
              endif
              istr = fix(str)
              if istr eq 8 then begin
;               this is layer name - ignore
                readf, unit, str
;               layer = str
                goto, processpoly
              endif else  if istr eq 40 or istr eq 41 then begin
;               this is the line width
                readf, unit, polywidth
              endif else if istr eq 70 then begin
                iflag = 0
;               if iflag bit 1 is set, it is a closed polygon; lcpoly is flag
                readf, unit, iflag
                lcpoly = iflag - (iflag/2)*2
              endif else if istr eq 10 then begin
                readf, unit, xinit
              endif else if istr eq  20  then begin
                readf, unit, yinit
              endif else if istr eq  30  then begin
                readf, unit, zinit
              endif else readf, unit, str
              goto, processpoly
            END

'LWPOLYLINE': Begin
          lclwpolyline = 0
          npoint = 0
          max_np = 0
          xinit = 0.
          yinit = 0.
          nx = 0
          ny = 0
          lc_closed = 0
          layer = '0'
          processlwpoly:
            readf, unit, str
            if strtrim(str) eq '  0' then begin
              print, 'Error processing data'
              print, 'LWPOLYLINE:', nx, ny, max_np, npoint
              goto, aftererror
            endif
            istr = fix(str)
            if istr eq 8 then begin
;               this is layer name
                readf, unit, str
                layer = str
                goto, processlwpoly
            endif else if istr eq 70 then begin
                readf, unit, lc_closed
            endif else if istr eq 90 then begin
                readf, unit, max_np
                xsp = make_array(max_np+1)
                ysp = xsp
            endif else if istr eq 10 then begin
                readf, unit, xinit
                if nx eq max_np then begin
                  print, 'LWPOLYLINE: NX gt number of control points'
                  goto, startdata
                endif
                xsp[nx] = xinit
                nx = nx+1
            endif else if istr eq  20  then begin
                readf, unit, yinit
                if ny eq max_np then begin
                  print, 'LWPOLYLINE: NY gt number of control points'
                  goto, startdata
                endif
                ysp[ny] = yinit
                ny = ny+1
            endif else readf, unit, str
            if ny gt 1 && ny eq max_np && nx eq max_np then begin
              if lc_closed  eq 1 then begin
                ysp[ny] = ysp[0]
                xsp[nx] = xsp[0]
                ny +=1
                nx +=1
              endif
             
              nel = n_elements (locg)
              if locg(nel-1) ge xxmax-n  then begin
                xxmax = xxmax+xxmax
                xx = [xx, xx]
                yy = [yy, yy]
              endif
              if n_elements(xsp) gt nx then begin
                xsp = xsp[0:nx-1]
                ysp = ysp[0:nx-1]
              endif
              if locg(1) eq 0 then begin
                  xx(0:nx-1) = xsp
                  yy(0:nx-1) = ysp
                  locg(1) = nx
                  layervalue(0) = layer
              endif else begin
                  xx(locg(nel-1): locg(nel-1)+nx-1) = xsp
                  yy(locg(nel-1): locg(nel-1)+ny-1) = ysp
                  locg = [locg, locg(nel-1)+nx]
                  layervalue(nel-1) = layer
              endelse
              goto, startdata
           endif
           goto, processlwpoly
         END

'SEQEND' : BEGIN
           
           if lcpoly eq 1 then begin
             if xvert(0) ne xvert(nvert-1) or $
                yvert(0) ne yvert(nvert-1) then begin
                xvert(nvert)= xvert(0)
                yvert(nvert)= yvert(0)
                nvert = nvert+1
             endif
           endif
           xtemp = xvert(0:nvert-1)
           ytemp = yvert(0:nvert-1)
           if polywidth ne 0.0 then begin
             thick_line, xtemp, ytemp, polywidth
             nvert = n_elements(xtemp)
           endif
           nel = n_elements (locg)
           if locg(nel-1) ge xxmax-nvert  then begin
             xxmax = xxmax+xxmax
             xx = [xx, xx]
             yy = [yy, yy]
           endif
           if locg(1) eq 0 then begin
               xx(0:nvert-1) = xinit+ xtemp
               yy(0:nvert-1) = yinit+ ytemp
               locg(1) = nvert
               layervalue(0) = layer
           endif else begin
               xx(locg(nel-1): locg(nel-1)+nvert-1) = xinit+xtemp
               yy(locg(nel-1): locg(nel-1)+nvert-1) = yinit+ytemp
               locg = [locg, locg(nel-1)+nvert]
               layervalue(nel-1) = layer
           endelse
           nvert = 0
           goto, startdata
           END

'VERTEX': BEGIN
        nx = 0L
        ny = 0L
        processvert:
          str = ' '
          readf, unit, str
          if strtrim(str) eq '  0' then begin
            print, 'Error processing data'
            print, 'vert:', nx, tempcx, ny, tempcy 
            goto, aftererror
          endif
          istr = -99
          istr = fix(str)
          if istr le  0 then begin
            print, str, tempcx, tempcy
            hak, /mes
            str = ' '
            for i= 1, 50 do begin
              readf, unit, str
              print, str
            endfor
            return
          endif

          if istr eq 8 then begin
;           this is layer name - ignore
            readf, unit, str
;           layer = str
            goto, processvert
          endif else if istr eq 6 then begin
;           this is line type name - accept only CONTINUOUS
            readf, unit, str
;           if strtrim(str, 2) ne 'CONTINUOUS' then goto, startdata
          endif else if istr eq 20 then begin
            if ny lt 1 then readf, unit, tempcy else begin
              print, tempcy , ' only 1 Y values per vertex segment'
              goto, startdata
            endelse
            ny = ny+1
          endif else if istr eq 10 then begin
            if nx lt 1 then readf, unit, tempcx else begin 
              print, tempcx , ' only 1 X values per vertex segment'
              goto, startdata
            endelse
            nx = nx+1
          endif else readf, unit, str
          if nx eq 1 and ny eq 1 then begin
            if nvert eq nvertmax  then begin
              nvertmax = nvertmax*2 
              xvert= [xvert, xvert]
              yvert= [yvert, yvert]
            endif
            xvert (nvert) = tempcx
            yvert (nvert) = tempcy
            nvert = nvert+1
            goto, startdata
          endif
          goto, processvert
        END

'BLOCK'  : BEGIN
           nel = n_elements(locg)
           if nel gt 2 then point_loc(0, nnames) = nel-1 else begin
             if locg(1) eq 0 then point_loc(0, nnames) = 0 else $
                point_loc(0, nnames) = 1
           endelse
           processblock:
             readf, unit, str
             if strtrim(str, 2) eq '0' then goto, aftererror
             istr = fix(str)
             if istr eq 2 or istr eq 3  then begin
;              this is block name 
               readf, unit, str
               names(nnames) = strtrim(str, 2)
             endif else if istr eq 8 then begin
               readf, unit, str
;              layer data - ignored
             endif else if istr eq 10 then begin
               readf, unit, x
               if x ne 0.0 then print, 'ERROR-X: Assumed to be zero, value =', x
             endif else if istr eq  20  then begin
               readf, unit, x
               if x ne 0.0 then print, 'ERROR-Y: Assumed to be zero, value =', x
             endif else if istr eq  30  then begin
               readf, unit, x
               if x ne 0.0 then print, 'ERROR-Z: Assumed to be zero, value =', x
             endif else if istr eq 70 then begin
               readf, unit, istr
               if istr lt 0 then print, ' WE have PROBLEMs'
;           this is the block type flag and is not used.
             endif else readf, unit, str
             goto, processblock
            END

'ENDBLK' : BEGIN
           nel = n_elements(locg)
           if nel gt 2 then point_loc(1, nnames) = nel-1 else begin
             if locg(1) eq 0 then point_loc(1, nnames) = 0 else $
                point_loc(1, nnames) = 1
           endelse
           if point_loc(1, nnames)-point_loc(0, nnames) le 0 then $
             print, 'Warning: No new data for block ' , names(nnames)
           nnames = nnames +1
           goto, startdata
           END
 'INSERT' : BEGIN
            if nstruct eq maxstr then begin
               Print,'Cannot continue with read until MAXSTR has been increased'
               return
            endif
            if lcblocks eq 1 then str_incl(nstruct).into = names(nnames) $
              else str_incl(nstruct).from = 'null'
       processins:
             readf, unit, str
             if strtrim(str, 2) eq 0 then begin
               if nstruct ge maxstr-5 then print, $
                  'WARNING: increase the size of MAXSTR in READDXF.'
               if lcentities eq 1 then begin
                 insname = strtrim(str_incl(nstruct).from, 2)
                 if insname ne 'null' then begin
                   ok = where ( insname eq namesn, nok)
                   if nok eq 0  then $
                     print,'INSERT block is not found - ', insname else begin
                     if nok gt 1 then begin
                       print, 'Multiple INSERTS found first one used '
                       print, ' Insert value: ',insname
                       print, ' Values matching: ', names(ok)
                     endif
                     ok = ok(0)
                     if locnames(point_locn(ok+1))-1 gt $
                       locnames(point_locn(ok)) then begin
                     xtemp = xnames(locnames(point_locn(ok)): $
                            locnames(point_locn(ok+1))-1)
                     ytemp = ynames(locnames(point_locn(ok)): $
                            locnames(point_locn(ok+1))-1)
                     if str_incl(nstruct).rotation ne 0.0 then $
                         rotate_xy, xtemp, ytemp, str_incl(nstruct).rotation
                     if str_incl(nstruct).xscale ne 1.0 then $
                         xtemp = xtemp * str_incl(nstruct).xscale
                     if str_incl(nstruct).yscale ne 1.0 then $
                         ytemp = ytemp * str_incl(nstruct).yscale
                     if str_incl(nstruct).xoffset ne 0.0 then $
                         xtemp = xtemp + str_incl(nstruct).xoffset
                     if str_incl(nstruct).yoffset ne 0.0 then $
                         ytemp = ytemp + str_incl(nstruct).yoffset
                     loctemp= locnames(point_locn( ok): point_locn( ok+1))
                     loctemp = loctemp-loctemp(0)
                     layertemp = layervalue(point_locn( ok): point_locn( ok+1)-1)
                     nel = n_elements(locg)
                     npt = n_elements(xtemp)
                     if npt ne max(loctemp)  then begin
                       print, ' Error on insert: npt ne max loctemp'
                       print, '  ***********Terminating READDXF**********'
                       return
                     endif
                     if locg(nel-1) gt xxmax - npt then begin
                       xxmax = xxmax+xxmax
                       xx = [xx, xx]
                       yy = [yy, yy]
                     endif
                     if locnames(1) eq 0 then begin
                       locg(1) = npt
                       xx(0: npt-1) = xtemp
                       yy(0: npt-1) = ytemp
                       locg =  loctemp
                       layervalue(0:nel-2) = layertemp
                     endif else begin
                       xx(locg(nel-1):locg(nel-1)+npt-1) = xtemp
                       yy(locg(nel-1):locg(nel-1)+npt-1) = ytemp
                       locg = [locg(0:nel-2), loctemp +locg(nel-1)]
                       layervalue (nel-1 :nel-2+n_elements(layertemp))=layertemp
                     endelse
                     endif
                   endelse
                 endif else $
                   Print, 'Insert block is not defined - skipping insert'
               endif
               nstruct = nstruct+1
               goto, aftererror
             endif
             istr = fix(str)
             if istr eq 2 or istr eq 3  then begin
;              this is block name
               readf, unit, str
               str_incl(nstruct).from= strtrim(str, 2)
             endif else if istr eq 8 then begin
               readf, unit, str
;              layer data - ignored
             endif else if istr eq 10 then begin
               readf, unit, x
               str_incl(nstruct).xoffset = x
             endif else if istr eq  20  then begin
               readf, unit, x
               str_incl(nstruct).yoffset = x
             endif else if istr eq  30  then begin
               readf, unit, x
               str_incl(nstruct).zoffset = x
             endif else if istr eq 41 then begin
               readf, unit, x
               str_incl(nstruct).xscale = x
             endif else if istr eq 42 then begin
               readf, unit, x
               str_incl(nstruct).yscale = x
             endif else if istr eq 43 then begin
               readf, unit, x
               str_incl(nstruct).zscale = x
             endif else if istr eq 50 then begin
               readf, unit, x
               str_incl(nstruct).rotation = x
             endif else readf, unit, str
             goto, processins
            END

 'ENDSEC' : BEGIN
           if lcblocks ne 0 then begin
             nnamesn = 0L
             if nnames gt 0 then namesn = make_array(nnames, /string)
             trip = 0
tripagain:
             trip = trip + 1
             if trip gt 20 then begin
               Print, ' More than 20 trips through the ENDSEC loop for blocks'
               return
             endif
help, nnames, nstruct
             for i = 0, nnames-1 do begin
               if names(i) ne 'null'  then begin
                 if nstruct gt 0 then begin
                 ok = where (names(i) eq str_incl(0:nstruct-1).into, nok)
                 if nok gt 0 then begin
                   if nnamesn eq 0 then goto, nexti
                   for j = 0, nok-1 do begin
                     s = where( str_incl(ok(j)).from eq namesn(0:nnamesn-1), ns)
                     if ns eq 0 then goto, nexti 
                   endfor
                 endif
                 endif
                 namesn(nnamesn) = names(i)
                 npt  = locg(point_loc(1,i)) - locg(point_loc(0, i))
                 if npt gt 0 then begin
                 if locnames(1) eq 0 then begin
                   point_locn (nnamesn) = 0
                   locnames = locg(point_loc(0,i): point_loc(1, i))- $
                      locg(point_loc(0,i))
                   layernames(0:point_loc(1, i)-point_loc(0,i)-1)= layervalue $
                     (point_loc(0,i): point_loc(1, i)-1)
                   xnames = xx
                   ynames = yy
                   xxnmax = xxmax
                   xnames(0:npt-1) = xx(locg(point_loc(0,i)): $
                      locg(point_loc(1, i))-1)
                   ynames(0:npt-1) = yy(locg(point_loc(0,i)): $
                      locg(point_loc(1, i))-1)
                   point_locn(nnamesn) = 0
                 endif else begin
                   nel = n_elements(locnames)
                   point_locn (nnamesn) = nel-1
                   locnames = [locnames(0:nel-2) , locg(point_loc(0,i): $
                     point_loc(1, i))- locg(point_loc(0,i)) +locnames(nel-1)]
                   layernames(nel-1:nel-2+ point_loc(1, i)-point_loc(0,i))= $
                     layervalue (point_loc(0,i): point_loc(1, i)-1)
                   if locnames(nel-1) ge xxnmax - npt then begin
                     xxnmax = xxnmax+xxnmax
                     xnames = [xnames, xnames]
                     ynames = [ynames, ynames]
                   endif
                   xnames(locnames(nel-1): locnames(nel-1)+npt-1) = $
                     xx(locg(point_loc(0,i)):locg(point_loc(1, i))-1)
                   ynames(locnames(nel-1): locnames(nel-1)+npt-1) = $
                     yy(locg(point_loc(0,i)):locg(point_loc(1, i))-1)
                 endelse
                 endif
                 if nstruct eq 0 then nok = 0 else $
                   inc = where(names(i) eq str_incl(0:nstruct-1).into, nok)
                 if nok ne 0 then begin
; now get the include stuff
                   for iok = 0, nok-1 do begin
                     insname = str_incl(inc(iok)).from
                     ookk = where ( insname eq namesn, nook)
                     if nook gt 0  then begin
                       if nook gt 1 then print, $
                       'Multiple INSERTS found first one used ',names(ok),insname
                       ookk = ookk(0)
                       if locnames(point_locn(ookk+1))-1 gt $
                        locnames(point_locn(ookk)) then begin
                       xtemp = xnames(locnames(point_locn(ookk)): $
                              locnames(point_locn(ookk+1))-1)
                       ytemp = ynames(locnames(point_locn(ookk)): $
                              locnames(point_locn(ookk+1))-1)
                       if str_incl(inc(iok)).rotation ne 0.0 then $
                           rotate_xy, xtemp, ytemp, str_incl(inc(iok)).rotation
                       if str_incl(inc(iok)).xscale ne 1.0 then $
                           xtemp = xtemp * str_incl(inc(iok)).xscale
                       if str_incl(inc(iok)).yscale ne 1.0 then $
                           ytemp = ytemp * str_incl(inc(iok)).yscale
                       if str_incl(inc(iok)).xoffset ne 0.0 then $
                           xtemp = xtemp + str_incl(inc(iok)).xoffset
                       if str_incl(inc(iok)).yoffset ne 0.0 then $
                           ytemp = ytemp + str_incl(inc(iok)).yoffset
                       loctemp= locnames(point_locn( ookk):point_locn( ookk+1))
                       loctemp = loctemp-loctemp(0)
                       nel = n_elements(locnames)
                       npt = n_elements(xtemp)
                       if npt ne max(loctemp)  then begin
                         print, ' Error on insert: npt ne max loctemp'
                         print, '  ***********Terminating READDXF**********'
                         return
                       endif
                       if locnames(nel-1) ge xxnmax - npt then begin
                         xxnmax = xxnmax+xxnmax
                         xnames = [xnames, xnames]
                         ynames = [ynames, ynames]
                       endif
                       locnames = [locnames(0:nel-2), loctemp+locnames(nel-1)]
                       xnames(locnames(nel-1): locnames(nel-1)+npt-1) = xtemp
                       ynames(locnames(nel-1): locnames(nel-1)+npt-1) = ytemp
                       if (point_loc(1, i) gt point_loc(0,i)) then $
                         layernames(nel-1:nel-2+point_loc(1, i)-point_loc(0,i)) $
                         = layervalue (point_loc(0,i): point_loc(1, i)-1)
                       endif
                     endif else $
                       Print, 'Insert block is not defined - skipping insert'
                     str_incl(inc(iok)).from = 'null'
                     str_incl(inc(iok)).into = 'null'
                   endfor
                 endif
                 names(i) = 'null'
                 nnamesn = nnamesn+1
                 point_locn(nnamesn) = n_elements(locnames)-1
      nexti:
               endif
             endfor
           if nnamesn lt nnames  and nstruct gt 0 then goto, tripagain
;    if keyword_set(outline) then begin
;      j = 0
;      nj = 0
;      for ii = 0, nnames-1 do begin
;        xtemp = xnames(locnames(point_locn(ii)):locnames(point_locn(ii+1))-1)
;        ytemp = ynames(locnames(point_locn(ii)):locnames(point_locn(ii+1))-1)
;        np0 = n_elements(xtemp)
;        np = np0
;        loc_ins = locnames(point_locn(ii):point_locn(ii+1)) - $
;                  locnames(point_locn(ii))
;        point_locn(ii) = nj
;        outline_ins, xtemp, ytemp, np, loc_ins, lcskip
;        if lcskip eq 0 then begin
;          if np gt np0 then print, 'Warning: insert array reduced by ', $
;            strtrim(np-np0, 2), ' points'
;          np = np < np0
;          nj = j+1
;          locnames(nj) = locnames(j) + np
;          xnames(locnames(j):locnames(nj)-1) = xtemp
;          ynames(locnames(j):locnames(nj)-1) = ytemp
;          j = nj
;        endif else begin
;          nj = j+point_locn(ii+1)-pointlocn(ii)
;          locnames(j:nj) = locnames(j) + loc_ins
;          xnames(locnames(j):locnames(nj)-1) = xtemp
;          ynames(locnames(j):locnames(nj)-1) = ytemp
;          j = nj
;        endelse
;      endfor
;      point_locn(nnames) = nj
;    endif
;     if keyword_set(plotall)  then begin
     if keyword_set(plotall)  and n_elements (xnames) gt 0 then begin
       if max(!p.multi) eq 0 then begin
          !p.multi = [0, 2, 2] 
          lcreset = 1
       endif else lcreset = 0
       for ii = 0, nnames-1 do begin
          loctemp = locnames(point_locn(ii):point_locn(ii+1))
          nell = n_elements (loctemp)
          xtemp = xnames(locnames(point_locn(ii)):locnames(point_locn(ii+1))-1)
          ytemp = ynames(locnames(point_locn(ii)):locnames(point_locn(ii+1))-1)
          plot, xtemp, ytemp, /nodata, tit= namesn(ii)
          for jj = 0, nell-2 do oplot, xnames(loctemp(jj):loctemp(jj+1)-1), $
                      ynames (loctemp(jj):loctemp(jj+1)-1)
          if !p.multi(0) eq 0 then hak, /mes
       endfor
       point_locn= point_locn(0:nnames)
       nel = n_elements(locnames)
       xnames = xnames(0:locnames(nel-1)-1)
       ynames = ynames(0:locnames(nel-1)-1)
       insertblocks = {x: xnames, y:ynames, loc:locnames, layer: layernames, $
                        point:point_locn, name: namesn}
       if lcreset ne 0 then !p.multi = 0
     endif
             locg = [0L, 0L]
           endif
           goto, startsect
           END

 'EOF' : print, ' Finished reading input'
 
  ELSE : BEGIN
        PRINT, str, ' is not supported in this version of READDXF'
        goto, startdata
        END

     ENDCASE

finished_read:

  free_lun, unit    
    if n_elements(resolution) eq 0 then resolution = max([xx, yy])/4.0e3
     resol2 = resolution*resolution/10.
;
      rm_dupes, xx, yy, locg
      nline = n_elements (locg)-1
       if keyword_set(nosort) then goto, finished
        change = 0
        i = 0
newline:
        i = i+1
        if i eq nline then goto, returnhere
again3:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i-1)) - xx(locg(check))) lt resolution and $
            abs( yy(locg(i-1))- yy(locg(check))) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i-1)) - xx(locg(check(ok))))^2+ $
                 (yy(locg(i-1))-yy(locg(check(ok))))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, rotate(xx(locg(n-1):locg(n)-1),2), $
               xx(locg(i-1):locg(i)-1), rotate(yy(locg(n-1):locg(n)-1), 2), $
               yy(locg(i-1):locg(i)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again3
          endif
again4:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i-1)) - xx(locg(check+1)-1)) lt resolution $
            and abs( yy(locg(i-1))- yy(locg(check+1)-1)) lt resolution, ns)
          if ns gt 0 then begin
            if ns gt 1 then begin
              change = 1
              junk = min((xx(locg(i-1)) - xx(locg(check(ok)+1)-1))^2+ $
                 (yy(locg(i-1))-yy(locg(check(ok)+1)-1))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(n-1):locg(n)-1), $
               xx(locg(i-1):locg(i)-1), yy(locg(n-1):locg(n)-1), $
               yy(locg(i-1):locg(i)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again4
          endif
again5:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i)-1) - xx(locg(check))) lt resolution and $
            abs( yy(locg(i)-1)- yy(locg(check))) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i)-1) - xx(locg(check(ok))))^2+ $
                 (yy(locg(i)-1)-yy(locg(check(ok))))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(i-1):locg(i)-1), $
               xx(locg(n-1):locg(n)-1), yy(locg(i-1):locg(i)-1), $
               yy(locg(n-1):locg(n)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again5
          endif
again6:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i)-1) - xx(locg(check+1)-1)) lt resolution $
            and abs( yy(locg(i)-1)- yy(locg(check+1)-1)) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i)-1) - xx(locg(check(ok)+1)-1))^2+ $
                 (yy(locg(i)-1)-yy(locg(check(ok)+1)-1))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(i-1):locg(i)-1), $
               rotate(xx(locg(n-1):locg(n)-1),2) , yy(locg(i-1):locg(i)-1), $
               rotate(yy(locg(n-1):locg(n)-1),2), i, nline, locg, xx, yy, $
                resol= resol2
;
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if nline eq i then goto, returnhere
            goto, again6
          endif
          goto, newline
returnhere:
          if change eq 1 then begin
            change = 0
            i = 0
            goto, newline
          endif
finished:
loadxyct

 locg = locg(0:nline)
 layervalue = layervalue(0:nline-1)
 xx = xx(0:locg(nline)-1)
 yy = yy(0:locg(nline)-1)
     plot, xx, yy, /nodata
 for i =1,nline do  if locg(i) gt locg(i-1) then $
   oplot,xx(locg(i-1):locg(i)-1),yy(locg(i-1):locg(i)-1),thi= 2, $
   color = i mod 7 +1

return
end
