; ------------------------------------------------------------------------ 
; $Id: save_markers.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro save_markers, fname, space = space
 on_error, 1
;+
; Write markers to an ascii file
; SAVE_MARKERS can be used to save the markers when problem has a new grid
; file format:
;         name
;         x, y
;         comment (May be blank)
;  Name must be have one or more ascii characters.
;  # indicates a comment and is ignored.
;  a blank line is ignored except for the comment line
;
; Command format:
;   SAVE_MARKERS [, FILENAME] [, /SPACE]
;   where
;      FILENAME  - Name of the file containing the marker information
;      SPACE     - If set put a comment space between markers.
;-
 if n_elements(fname) eq 0 then fname = '' else fname = strtrim(fname, 2)
 i = strlen(fname)
 tqhelp, markers = mark 
 nmark = n_elements(mark) 
 if nmark eq 0 then begin
   print, '% SAVE_MARKERS:  Markers must be defined'
   return
 endif
 openw, unit, fname,/get_lun, error = error
 if error ne 0 then begin
   print, '% SAVE_MARKERS: Error opening file'
   return
 endif
 printf, unit, '# Marker file format'
 printf, unit, '#    Name of the Marker'
 printf, unit, '#    Location of the marker'
 printf, unit, '#    Comment for the marker'


 n = nmark/3
 ii = strpos(mark(0), '(' )
 for i = 0, n-1 do begin
   if keyword_set(space) then printf, unit, '# Marker ', strtrim(i+1, 2)
   j = 3*i
   jj = strpos(mark(j+2), "'")
   printf, unit, strmid(mark(j+2), 9, jj-10) 
   jj = strpos(mark(j), ')', ii)
   printf, unit, strmid(mark(j), ii+1, jj-ii-1) 
   printf, unit,strmid( mark(j+1), 2, 80)
 endfor
 free_lun, unit
 end
