; ------------------------------------------------------------------------ 
; $Id: slice2cond.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro slice2cond, ain, xval, yval, reset = reset
;+
; Routine: Slice2Cond
; 
; Format: Slice2Cond, str, xval, yval [, RESET = reset]
; 
; where:
;       str      - Structure or structure array number with a conductor slice
;       Xval     - Dimension to be used for the X axis
;       Yval     - Dimension to be used for the Y axis
;       reset    - Optional keyword to be used in call to tqcond.
;                  Default = 1; All old values will be eliminated and these
;                               values added.
;                  Reset = 0 will add new conductors to existing ones.
; Example:
;      Put a slice from structure 5 into the conductor structure mapping 
;         Z values to x and X values to Y.
;      SLICE2COND, 5, 3, 1
; 
;      Original version April 14, 1997. lpm
;-
on_error, 1
@pfidltq.com
@qsstrcom.dat
      npar = n_params()
      if npar eq 0 then begin
        print,'% Format:', $
                   ' Slice2Cond, STR, Xval, Yval [, RESET = reset]'
        return
      endif
      if n_elements(ain) eq 0 then begin
        print, '% Slice2Cond: Structure must be defined'
        return
      endif
      if n_elements(xval) eq 0 or n_elements(yval) eq 0 then begin
        print, '% Slice2Cond: Axis quantities must be defined'
        return
      endif
      mm = max([xval, yval], min = m)
      if mm gt 3 or m lt 1 then begin
        print, '% Slice2Cond: Axis quantities must be in the range [1, 3]'
        return
      endif
      a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 7 then begin
        print, ' Only ACIS type structures may be converted to Conductors'
        return
      end
      
      if xval eq 1 then begin
        xx = a.x1
        if n_elements(xinsav) gt 0 then begin
          x= xinsav.x
          dx= xinsav.dx 
        endif
      endif else if xval eq 2 then begin
        xx = a.x2
        if n_elements(yinsav) gt 0 then begin
          x= yinsav.x
          dx= yinsav.dx 
        endif
      endif else begin
        xx = a.x3
        if n_elements(zinsav) gt 0 then begin
          x= zinsav.x
          dx= zinsav.dx 
        endif
      endelse
      if yval eq 2 then begin
        yy = a.x2
        if n_elements(yinsav) gt 0 then begin
          y= yinsav.x
          dy= yinsav.dx 
        endif
      endif else if yval eq 1 then begin
        yy = a.x1
        if n_elements(xinsav) gt 0 then begin
          y= xinsav.x
          dy= xinsav.dx 
        endif
      endif else begin
        yy = a.x3
        if n_elements(zinsav) gt 0 then begin
          y= zinsav.x
          dy= zinsav.dx 
        endif
      endelse
      if n_elements(reset) eq 0 then reset = 1

      tqcond, xx, yy, a.locx, reset = reset
      end
