; ------------------------------------------------------------------------ 
; $Id: tqedit.pro,v 1.4 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;   This is designed to edit a conductor data for TQ input or 
;    edit the grid fitted points for tq input
;   format: tqedit, conductor = conductor, reset = reset, 
;                     fast = fast,xsize= xsize, ysize= ysize
;       conductor if nonzero indicates editing conductor data
;       All data must be in the TQ common block
;            reset then grid for conductors
;            default is fast = 0
;                  fast = 1 - unfilled diamonds
;                  fast = 0 - filled circles for the points
;       NOTE: XCUR will print out points to the terminal if the mouse
;             button is pressed.
;
;   Example:
;     Generate conductor data from scratch
;      tqcond, [0, 10], [0, 1] ; define a region of interest
;      tqgrid, [0, 10], [.1, .1], /x ; define a reference grid
;      tqgrid, [0, 1], [.1, .1], /y
;      tqedit, /cond ;edit the conductor data
;      (if desired generate a new grid)
;      tqfit  ; Fit the data to an existing grid 
;      tqedit ; Edit the grid fitted conductors and add markers
;-
;
 PRO tqedit_event, event
;
;  Common Blocks
;
@pfidltq.com
;
@tqeditevent.com
;
;  When a widget is manipulated,
;   put its User Value into the variable 'eventval':
;
      col_offset = 2
      symsize = 1.5
      WIDGET_CONTROL, event.id, GET_UVALUE = eventval
; Perform actions based on the User Value of the event:
;
  CASE eventval OF
   'DRAW_WIN_EVENT': BEGIN
      CASE curoper of
         'ERASE': BEGIN
;            calculate the nearest point and delete the point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -1
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive = 0
               device, set_graphics = 3
               if lcregion ge 0 then begin
                 delete_tqpnts , [ip, ip] 
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
               endif
               plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                    yrange = !y.crange
               tqplot, /over
               if lcregion lt 0 then begin
                 arr = -1
                 device, set_graphics = 6
                 widget_control, draw, /clear_events, sensitive = 1
               endif 
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms=symsize,color=cerase
               plots, tempx, tempy, color = cerase, psym = 8, syms = symsize
               tempx2= tempx
               tempy2= tempy
               arr = ip
             endelse
             END

         'SPLIT': BEGIN
;            calculate the nearest point on a conductor 
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -1
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive = 0
               if lcregion ge 0 then begin
                 if ip eq 0 then ip = 1
                 if ip eq cond.locx(lcsegment)-1-cond.locx(lcsegment-1) then $
                    ip= ip-1
                 if ip ne 0 then begin
                   tqsplit_cond, lcsegment, ip
                   maxcond = cond.ncon
                   conductors = make_array(cond.ncon+1, value=1, /int)
                 endif else  wwarning, $
                    ' Cannot split a two element conductor'
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
               endif
               plot, gx, gy, /nodata, xrange = !x.crange, $
                  yr = !y.crange, /xst, /yst
               tqplot, /over
               if lcregion lt 0 then begin
                  arr = -1
                  widget_control, draw, /clear_events, sensitive = 1
               endif 
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if ip eq 0 then ip = 1
               if ip eq cond.locx(lcsegment)-1-cond.locx(lcsegment-1) then $
                  ip= ip-1
               if arr ge 0 then begin
                  if tempx2 ne lcsegment then $
                    oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                      cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                      psym =-8, thick =3, $
                      color = ((tempx2+col_offset) mod 7  + 1)
               endif
               if ip ne 0 then begin
                  tempx2= lcsegment
                  arr = ip
                  oplot, cond.x(cond.locx(lcsegment-1):  $
                     cond.locx(lcsegment-1)+ip), $
                     cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment-1)+ip), $
                     psym =-8, thick =3, color = 2
                  oplot, cond.x(cond.locx(lcsegment-1)+ip: $
                     cond.locx(lcsegment)-1), $
                     cond.y(cond.locx(lcsegment-1)+ip:cond.locx(lcsegment)-1), $
                     psym =-8, thick =3, color = 5
               endif
             endelse
             END

         'DELETE': BEGIN
;            calculate the nearest on a conductor and delete the conductor
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -2
             if event.press EQ 1 then begin
                ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
                widget_control, draw, /clear_events, sensitive = 0
                device, set_graphics = 3
                if lcregion ge 0 then begin
                  tqdelete_cond, lcsegment
                  tqplot, /over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
                endif
                plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                    yrange = !y.crange
                tqplot, /over
                if lcregion lt 0 then begin
                  arr = -1
                  device, set_graphics = 6
                  widget_control, draw, /clear_events, sensitive = 1
                endif
             endif else begin
                tempregion = 0
                ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                   lcmode, lcsegment, tempregion, region_size, condstr)
                if arr ge 0 then begin
                   if tempx2 ne lcsegment then begin
                     oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        psym =-8, thick =3, color = cerase
                     oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                   endif
                endif else begin
                   oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                endelse
                arr = ip
                tempx2= lcsegment
             endelse
             END

         'JOIN': BEGIN
;            calculate the nearest conductor to the point
;            join the two designated conductors
             coord = convert_coord(event.x, event.y, /device, /to_data)
             lcsegment = -2
             if event.press EQ 1 then begin
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, lcregion, region_size, condstr)
               widget_control, draw, /clear_events, sensitive = 0
               if lcregion lt 0 then begin
                 arr = -1
                 device, set_graphics = 3
                 plot, gx, gy, /nodata, /xst, /yst, xrange= !x.crange, $
                     yrange = !y.crange
                 tqplot, /over
                 if narray lt 1 then device, set_graphics = 6 else $
                   oplot,cond.x(cond.locx(narray-1):cond.locx(narray)-1),$
                      cond.y(cond.locx(narray-1): $
                      cond.locx(narray)-1), $
                      psym =-8, thick =3, color = 2
               endif else if narray lt 0 then begin
;                first conductor has been picked
                 narray = lcsegment
                 widget_control, tqinstr,set_value = $
                    'Indicate the other conductor to be joined'
                 device, set_graphics = 3
                 oplot,cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                      cond.y(cond.locx(lcsegment-1): $
                      cond.locx(lcsegment)-1), $
                      psym =-8, thick =3, color = 2
                 arr = -1
               endif else begin
                 if lcsegment ne narray then begin
                   tqjoin_cond, lcsegment, narray
                   widget_control, tqinstr,set_value =  $
                     'Pick an Activity or press DONE '
                   curoper = 'WAIT'
                 endif else begin
                   winformation, ['Cannot join a conductor to itself', $
                      'Pick another conductor or CANCEL activity']
                 endelse
                 plot, gx, gy, /nodata, xrange = !x.crange, $
                       yr = !y.crange, /xst, /yst
                 tqplot, /over
                 if curoper eq 'JOIN' then begin
                   oplot,cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                      cond.y(cond.locx(lcsegment-1): $
                      cond.locx(lcsegment)-1), $
                      psym =-8, thick =3, color = 2
                   arr = -1
                 endif
               endelse
               if curoper eq 'JOIN' then $
                   widget_control, draw, /clear_events, sensitive = 1
             endif else begin
               tempregion = 0
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                  lcmode, lcsegment, tempregion, region_size, condstr)
               if narray lt 0 then begin
                 if arr gt 0 then begin
                    if tempx2 ne lcsegment then begin
                      oplot, cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                        psym =-8, thick =3, color = cerase
                      oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1),$
                        psym =-8, thick =3, color = cerase
                      tempx2= lcsegment
                    endif
                  endif else begin
                    oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = cerase
                    arr = lcsegment
                    tempx2 = lcsegment
                  endelse
               endif else begin
                 if lcsegment ne narray then begin 
                    if  arr gt 0 then  begin
                      if tempx2 ne lcsegment then begin
                        oplot,cond.x(cond.locx(tempx2-1):cond.locx(tempx2)-1),$
                          cond.y(cond.locx(tempx2-1):cond.locx(tempx2)-1), $
                          psym =-8, thick =3, color = (tempx2+col_offset) mod 7+1
                        oplot, cond.x(cond.locx(lcsegment-1): $
                          cond.locx(lcsegment)-1), $
                          cond.y(cond.locx(lcsegment-1): $
                           cond.locx(lcsegment)-1), $
                          psym =-8, thick =3, color = 5
                        tempx2= lcsegment
                      endif
                    endif else begin
                      oplot, cond.x(cond.locx(lcsegment-1): $
                        cond.locx(lcsegment)-1), $
                        cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1), $
                        psym =-8, thick =3, color = 5
                      arr = lcsegment
                      tempx2 = lcsegment
                    endelse
                  endif
               endelse
             endelse
             END

        'MOVE': BEGIN
;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;              a point has been designated
               if narray lt 0 then begin
                 lcsegment = -1
                 ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                    lcmode, lcsegment, lcregion, region_size, condstr)
                 device, set_graphics = 3
                 if lcregion ge 0 then begin
                   narray = ip
                   widget_control, tqinstr,set_value = $
                     'Indicate the new location'
                   if lcmode eq 'INPUT' then begin
                     ist = cond.loci(lcsegment-1)
                     iend = cond.loci(lcsegment)-1
                     if ip eq 0 then j1 = ist+1 else j1 = ist+ip-1
                     if ip+ist eq iend then j2 = iend-1 else j2 = ist + ip+1
                     tempx = [gx(cond.ix(j1)), tempx, gx(cond.ix(j2))]
                     tempy = [gy(cond.iy(j1)), tempy, gy(cond.iy(j2))]
                   endif else begin
                     ist = cond.locx(lcsegment-1)
                     iend = cond.locx(lcsegment)-1
                     if ip eq 0 then j1 = ist+1 else j1 = ist+ip-1
                     if ip+ist eq iend then j2 = iend-1 else j2 = ist + ip+1
                     tempx = [(cond.x(j1)), tempx, (cond.x(j2))]
                     tempy = [(cond.y(j1)), tempy, (cond.y(j2))]
                   endelse
                   arr = ip
                 endif 
                 plot, gx, gy, /nodata, xrange = !x.crange, $
                     yr = !y.crange, /xst, /yst
                 tqplot, /over
                 if narray ge 0 then $
                   oplot, tempx, tempy, psym = -8, thick = 3, color = 2 $
                 else  begin
                   device, set_graphics = 6
                   arr = -1
                 endelse
               endif else begin
                  if lcmode eq 'INPUT' then $
                    new = get_tqgrid(coord(0), coord(1)) else new = coord
                  move_tqpnts, narray, new
                  plot, gx, gy, /nodata, xrange = !x.crange, $
                     yr = !y.crange, /xst, /yst
                  tqplot, /over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                     'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                  widget_control, draw, /clear_events, sensitive = 0
               endelse
             endif else begin
               if narray lt 0 then begin
                 tempregion = 0
                 lcsegment = -1
                 ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                     lcmode, lcsegment, tempregion, region_size, condstr)
                 if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms =symsize,color=cerase
                 plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                 tempx2= tempx
                 tempy2= tempy
                 arr = ip
               endif else begin
                 if lcmode eq 'INPUT' then begin
                   new = get_tqgrid(coord(0), coord(1))
                   new= [gx(new(0)), gy(new(1))]
                 endif else new = coord
                 oplot, tempx, tempy, psym = -8 , thick = 3, color = 0
                   tempx(1)= new(0)
                   tempy(1)= new(1)
                 oplot, tempx, tempy, psym = -8, thick = 3, color = 2
               endelse
             endelse
             END

        'MERASE': BEGIN
;            erase multiple points along a line calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;              a point has been designated
               device, set_graphics = 3
               if narray lt 0 then lcsegment = -1
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                 lcmode, lcsegment, lcregion, region_size, condstr)
               if lcregion ge 0 then begin
                 if narray lt 0 then begin
                   narray = ip
                   widget_control, tqinstr,set_value = $
                     'Indicate the other end point for the points to be removed'
                   arr = [ip, ip]
                 endif else begin
                   is = max([narray, ip], min = im)
                   delete_tqpnts , [im, is]
                   curoper = 'WAIT'
                   widget_control, tqinstr,set_value = $
                     'Pick an Activity or press DONE '
;                  Clear events and make the draw widget insensitive 
                   widget_control, draw, /clear_events, sensitive = 0
                 endelse
               endif
               plot, gx, gy, /nodata, xrange = !x.crange, $
                   yr = !y.crange, /xst, /yst
               tqplot, /over
               if narray lt 0 then begin
                 device, set_graphics = 6 
                 plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                 tempx2= tempx
                 tempy2= tempy
                 arr = ip
               endif else if curoper eq 'MERASE' then begin
                 is = max([narray, ip], min = im)
                 s = arr(0)-1
                 t = arr(1)+1
                 arr= [im, is]
                 if lcmode eq 'COND' then begin
                     xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                     yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                 endif else begin
                     xx = gx(cond.ix(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                     yy = gy(cond.iy(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                 endelse
                 np = n_elements(xx)
                 if s gt -1 and t lt np then  begin
                     plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)], color = 0, thick=2,psym=-8,syms=symsize
                 endif
                 s = (arr(0)-1) > 0
                 t = (arr(1)+1) < (np-1)
                 plots, xx, yy, thick=2, psym=-8, color = 2
                 plots, xx(arr(0):arr(1)), yy(arr(0):arr(1)),  $
                      psym=8, color = 1,syms =symsize
                 plots, xx(s:t), yy(s:t), color = 1, thick = 2
                 s = arr(0)-1
                 t = arr(1)+1
                 if s gt -1 and t lt np then $
                 plots, [xx(s),xx(t)], $
                     [yy(s), yy(t)],color=2,psym=-8, syms =symsize
               endif
             endif else begin
               tempregion = 0
               if narray lt 0 then lcsegment = -1
               ip = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                 lcmode, lcsegment, tempregion, region_size, condstr)
               if narray lt 0 then begin
                 if arr ge 0 then $
                    plots, tempx2, tempy2, psym = 8, syms=symsize,color=cerase
                  plots, tempx, tempy, color = cerase, psym = 8, syms=symsize
                  tempx2= tempx
                  tempy2= tempy
                  arr = ip
               endif else begin
                   s = arr(0)-1 
                   t = arr(1)+1 
                   is = max([narray, ip], min = im)
                   arr= [im, is]
                   if lcmode eq 'COND' then begin
                     xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                     yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                   endif else begin
                     xx = gx(cond.ix(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                     yy = gy(cond.iy(cond.loci(lcsegment-1):  $
                          cond.loci(lcsegment)-1))
                   endelse
                   np = n_elements(xx)
                   if s gt -1 and t lt np then  begin 
                     plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)], color = 0,  thick = 2, psym = -8
                   endif
                   s = (arr(0)-1) > 0
                   t = (arr(1)+1) < (np-1)
                   plots, xx, yy, thick=2, psym=-8, color = 2, symsize=symsize
                   plots, xx(arr(0):arr(1)), yy(arr(0):arr(1)),  $
                      psym=8, color = 1,syms =symsize
                   plots, xx(s:t), yy(s:t), thick = 2, color = 1
                   s = arr(0)-1 
                   t = arr(1)+1 
                   if s gt -1 and t lt np then $
                   plots, [xx(s),xx(t)], $
                       [yy(s), yy(t)],color=2,psym=-8, syms =symsize
               endelse
             endelse
             END

       'ADD': BEGIN
;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;               a point has been designated
                if narray lt 0 then begin
                  device, set_graphics = 3
                  lcsegment = -1
                  narray = get_tq_conduct(coord(0), coord(1),tempx, tempy, $
                    lcmode, lcsegment, lcregion, region_size, condstr)
                  if lcregion ge 0 then begin
                    if narray eq 0 then begin
                      if lcmode eq 'INPUT' then begin
                        ist = cond.loci(lcsegment-1)
                        iend = cond.loci(lcsegment)-1
                        cond.ix (ist:iend) = rotate(cond.ix (ist:iend), 2)
                        cond.iy (ist:iend) = rotate(cond.iy (ist:iend), 2)
                        narray = iend-ist
                      endif else begin
                        ist = cond.locx(lcsegment-1)
                        iend = cond.locx(lcsegment)-1
                        cond.x (ist:iend) = rotate(cond.x (ist:iend), 2)
                        cond.y (ist:iend) = rotate(cond.y (ist:iend), 2)
                        narray = iend-ist
                      endelse
                      widget_control, tqinstr,set_value = $
                          'Indicate the new point to be added.'
                    endif
                  endif else narray = -1
                  arr= narray
                endif else begin
                  if lcmode eq 'INPUT' then $
                    new = get_tqgrid(coord(0), coord(1)) else new = coord
                  add_tqpnts , narray, new
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                  widget_control, draw, /clear_events, sensitive = 0
                endelse
                plot, gx, gy, /nodata, xrange = !x.crange, $
                  yr = !y.crange, /xst, /yst
                tqplot, /over
                arr = -1
                if narray lt 0 then device, set_graphics = 6
             endif else begin
;               we are moving the cursor over the plot
                if narray lt 0 then begin
                  lcsegment = -1
                  tempregion = 0
                  ip=   get_tq_conduct (coord(0), coord(1), tempx, tempy, $
                    lcmode, lcsegment, tempregion, region_size, condstr)
;                 indicate the nearest point
                  if arr ge 0 then $
                    plots, tempx2, tempy2, psym=8,syms=symsize,color=cerase
                  plots, tempx, tempy, color = cerase, psym = 8, syms =symsize
                  tempx2= tempx
                  tempy2= tempy
                  arr = ip
                endif else begin
                  t = narray+1
                  if lcmode eq 'COND' then begin
                    xx = cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                    yy = cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)
                    ip = coord
                  endif else begin
                    loc = cond.loci
                    xx = gx(cond.ix(cond.loci(lcsegment-1): $
                        cond.loci(lcsegment)-1))
                    yy = gy(cond.iy(cond.loci(lcsegment-1): $
                        cond.loci(lcsegment)-1))
                    ip = get_tqgrid (coord(0), coord(1))
                    coord(0) =gx(ip(0))
                    coord(1) =gy(ip(1))
                  endelse
                  np = n_elements(xx)
                  if t lt np then  begin 
                    if n_elements(arr) gt 1 then begin
                      oplot, [xx(narray),tempx2,xx(t)], $
                        [yy(narray),tempy2,yy(t)], psym = -8,$
                        color= 0
                    endif
                    oplot, [xx(0:narray), coord(0), xx(t:*)], $
                      [yy(0:narray),coord(1),yy(t:*)],psym= -8, $
                      color= 2
                  endif else begin
                    if n_elements(arr) gt 1 then oplot, $
                        [xx(narray), tempx2], [yy(narray), $
                        tempy2], psym = -8, color = 0
                    oplot, [xx(0:narray), coord(0)], $
                      [yy(0:narray), coord(1)], psym= -8, $
                      color= 2
                  endelse
                  tempx2 = coord(0)
                  tempy2 = coord(1)
                  arr = ip
                endelse
              endelse
           END

         'ZOOM': BEGIN
             coord = convert_coord(event.x, event.y, /device, /to_data)
             if event.press EQ 1 then begin
;            a point has been designated
                if narray lt 0 then begin
                   narray = 0
                   arr = coord
                  if n_elements (arr) gt 1 then begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endif
                   widget_control, tqinstr,set_value = $
                   'Indicate opposite corner of the ZOOM region'
                 endif else begin
                   tempx = [min([arr(0), coord(0)], max = maxx) , maxx]
                   tempy = [min([arr(1), coord(1)], max = maxx) , maxx]
                   plot, gx, gy, /nodata, yr = tempy, xr = tempx, /xst, /yst
                   tqplot, /over
;
                   curoper = 'WAIT'
                   tempx = 0
                   widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                  Clear events and make the draw widget insensitive 
                    widget_control, draw, /clear_events, sensitive = 0
                 endelse
             endif else begin
;               we are moving the cursor over the plot
                if narray lt 0 then begin
;                 indicate the current point
                  if n_elements (arr) gt 1 then begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endif
                  arr = coord
                  oplot, [arr(0), arr(0)], !y.crange, color=7, lines = 2
                  oplot, !x.crange, [arr(1), arr(1)], color=7, lines = 2
                endif else begin
                  if (n_elements(tempx) gt 1) then $
                  oplot, [tempx(0), tempx(1), tempx(1), tempx(0), tempx(0)], $
                    [tempy(0), tempy(0), tempy(1), tempy(1), tempy(0)], $
                    color = 0, lines = 2  else begin
                    oplot, [arr(0), arr(0)], !y.crange, color= 0, lines = 2
                    oplot, !x.crange, [arr(1), arr(1)], color= 0, lines = 2 
                  endelse  
                  tempx = [min([arr(0), coord(0)], max = maxx) , maxx]
                  tempy = [min([arr(1), coord(1)], max = maxx) , maxx]
                  oplot, [tempx(0), tempx(1), tempx(1), tempx(0), tempx(0)], $
                    [tempy(0), tempy(0), tempy(1), tempy(1), tempy(0)], $
                    color = 7, lines = 2
                endelse
              endelse
           END

         'MARKER': BEGIN
;            calculate the nearest grid point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             ip = get_tqgrid(coord(0), coord(1))
             if event.press EQ 1 then begin
                 add_tqmrk , ip
                 device, set_graphics = 3
;                plot, gx, gy, /nodata, xrange = !x.crange, yrange = !y.crange
                 tqplot, /over
;
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                Clear events and make the draw widget insensitive 
                 widget_control, draw, /clear_events, sensitive = 0
              endif else begin
                 if arr(0) gt 0 then plots, gx(arr(0)), gy(arr(1)), $
                   color =cerase,psym = 8, syms =symsize
                 plots, gx(ip(0)),gy(ip(1)), color=cerase,psym=8,syms=symsize
                 arr = ip
              endelse
              END
           
         'D_MARKER': BEGIN
;            calculate the nearest point
             coord = convert_coord(event.x, event.y, /device, /to_data)
             cerase = !d.table_size-1
             ip = get_tqmark(coord(0), coord(1))
             if event.press EQ 1 then begin
                 delete_tqmrk, ip
                 device, set_graphics = 3
                 plot, gx, gy, /nodata, xrange = !x.crange, yrange = !y.crange, $
                    /xst, /yst
                 tqplot, /over
;
                 curoper = 'WAIT'
                 widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
;                Clear events and make the draw widget insensitive 
                 widget_control, draw, /clear_events, sensitive = 0
              endif else begin
                 if arr ge 0 then plots, gx(markx(arr)), gy(marky(arr)), $
                    color = cerase, psym = 8, syms =symsize
                 plots, gx(markx(ip)), gy(marky(ip)), color = cerase, $
                     syms =symsize, psym = 8
                 arr = ip
              endelse
              END
      'XFIT' : BEGIN
;           if no button press, ignore
             if event.press EQ 1 then begin
              coord = convert_coord(event.x, event.y, /device, /to_data)
;               a point has been designated
                if n_elements(narray) eq 1 then begin
                   lcsegment = -1
                   narray  = get_tqconductor(coord, lcsegment)
                   plots, psym = 8, color = 5, narray(0), narray(1)
                   widget_control, tqinstr,set_value = $
                   'Indicate the other end point for the line to be fit'
                endif else begin
                  coord  = get_tqconductor(coord, lcsegment)
                  plots, psym = -8, color = 5, [narray(0),coord(0)], $
                    [narray(1),coord(1)]
                  ques =['Do you wish to fit this line?', $
                     'CANNOT UNDO']
                  wok_notok, lcnok, ques,'X Fit Grid', /modal, group_leader = event.top
                  if lcnok eq 0 then begin
                    if old_win ge 0 then begin
                      wset, old_win 
                      wshow, old_win
                    endif else begin
                      window, 0
                      old_win=0
                    endelse
                    xran = !x.crange
                    yran = !y.crange
                    xfit_tqgrid, narray, coord
                    tqfit
                    wset, cur_win
                    old_cond = cond
                  endif else begin
                    xran = !x.crange
                    yran = !y.crange
                  endelse
                  plot, gx, gy,/nodata,xrange = xran, yrange = yran, /xst, /yst
                  tqplot,/over
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                    widget_control, draw, /clear_events, sensitive = 0
                endelse
             endif
             END
      'YFIT' : BEGIN
;           if no button press, ignore
            if event.press EQ 1 then begin
              coord = convert_coord(event.x, event.y, /device, /to_data)
;               a point has been designated
                if n_elements(narray) eq 1 then begin
                   lcsegment = -1 
                   narray  = get_tqconductor(coord, lcsegment)
                   plots, psym = 8, color = 5, narray(0), narray(1)
                   widget_control, tqinstr,set_value = $
                   'Indicate the other end point for the line to be fit'
                endif else begin
                  coord  = get_tqconductor(coord, lcsegment)
                  plots, psym = -8, color = 5, [narray(0),coord(0)], $
                    [narray(1),coord(1)]
                  ques =['Do you wish to fit this line?', $
                     'CANNOT UNDO']
                  wok_notok, lcnok, ques,'Y Fit Grid', /modal, group_leader = event.top
                  if lcnok eq 0 then begin
                    if old_win ge 0 then begin
                      wset, old_win 
                      wshow, old_win
                    endif else begin
                      window, 0
                      old_win=0
                    endelse
                    xran = !x.crange
                    yran = !y.crange
                    yfit_tqgrid, narray, coord
                    tqfit
                    old_cond= cond
                    wset, cur_win
                  endif else begin
                    xran = !x.crange
                    yran = !y.crange
                  endelse
                  plot, gx, gy,/nodata,xrange =xran, yrange = yran, /yst, /xst
                  tqplot, /over
; 
                  curoper = 'WAIT'
                  widget_control, tqinstr,set_value = $
                   'Pick an Activity or press DONE '
;                 Clear events and make the draw widget insensitive 
                    widget_control, draw, /clear_events, sensitive = 0
                endelse
             endif
             END

           ELSE: begin
                 wmessage, $
                  'Something is wrong-Pick another activity'
;                Clear events and make the draw widget insensitive 
                 widget_control, draw, /clear_events, sensitive = 0
                 curoper = 'WAIT'
           END
        ENDCASE
        END

   'ERASE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'ERASE'
                device, set_graphics = 6
                arr = -1
                widget_control, tqinstr,set_value = $
                  'Indicate the point to be removed'
              endif
             END

   'MERASE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
	        WSET, cur_win
                curoper = 'MERASE'
                device, set_graphics = 6
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the points to be removed'
              endif
             END


   'MOVE': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
;               Find the point
                  device, set_graphics = 6
                  arr = -1
                  curoper = 'MOVE'
                  narray = -1
                  widget_control, draw, /clear_events, sensitive = 1
                  widget_control, tqinstr,set_value = $
                 'Indicate the point you want to move'
              endif
             END
   'ADD': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
;               Find the point
                  device, set_graphics = 6
                  arr = -1
                  curoper = 'ADD'
                  narray = -1
                  widget_control, draw, /clear_events, sensitive = 1
                  widget_control, tqinstr,set_value = $
                 'Indicate the point immediately before the point to be added.'
              endif
             END

    'UNDO': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                if lcmode eq 'COND' then begin
                  cond =  old_cond
                endif else begin
                  cond = old_cond
                  get_tqbad
                  get_tqplotpnts
                endelse
                plot, gx, gy,/nodata, xrange = !x.crange, yrange = !y.crange, $
                /xst, /yst
                tqplot,/over
;
              endif
             END

   'REDRAW': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                ; Redraw the image
; 
                 maxx = max(gx, min= minx)
                 maxy = max(gy, min= miny)
                 plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
                 yrange = [miny, maxy]
                 tqplot,/over
;
              endif
             END

   'ZOOM': BEGIN
              if curoper eq 'WAIT' then begin
		; Set the window number - important for multiple windows
		WSET, cur_win
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate one corner of the ZOOM region'
                arr = -1
                narray = -1
                curoper = 'ZOOM'
              endif
             END

  'CANCEL': BEGIN
            widget_control, draw, /clear_events, sensitive = 0
            curoper = 'WAIT'
            device, set_graphics = 3
            if lcregion  lt 0 then begin
              cond= temporary(condstr)
              lcregion = -lcregion
            endif
;           Use PLOT to display an image in the draw widget.
;           Redraw the image
            curoper = 'WAIT'
            widget_control, tqinstr,set_value = $
                    'Pick an Activity or press DONE '
            plot, gx, gy,/nodata, xran = !x.crange, yran = !y.crange,/xst,/yst
            tqplot,/over
            a = size(condstr)
            END

  'DELKILL': BEGIN
             widget_control, draw, /clear_events, sensitive = 0
             curoper = 'WAIT'
             ; Set the window number - important for multiple windows
             WSET, cur_win
;            
             wok_notok, lcnok, 'Delete ALL the Conductors?', 'Delete ALL', $
                group_leader = event.top, /modal

             if lcnok eq 0 then begin
               xx = cond.x
               yy = cond.y
               loc = cond.locx
               ncon= cond.ncon
               nncon = 0L
               for i = 1, ncon do begin
                 st = loc(i-1) & en = loc(i)-1
                 ok = where (xx(st:en) ge !x.crange(0) and $
                             xx(st:en) le !x.crange(1) and $
                             yy(st:en) ge !y.crange(0) and $
                             yy(st:en) le !y.crange(1), nok)
                 if nok eq 0 then begin
                   nel = en -st
                   nst = loc(nncon)
                   xx(nst:nst+nel) = xx(st:en)
                   yy(nst:nst+nel) = yy(st:en)
                   nncon = nncon+1
                   loc(nncon) = loc(nncon-1)+nel+1
                 endif
               endfor
               if nncon gt 0 then begin
                 xx = xx(0:loc(nncon)-1)
                 yy = yy(0:loc(nncon)-1)
                 loc = loc(0:nncon)
                 old_cond = temporary(cond)
                 lcfit = make_array(nncon+1, /int, value = 0)
                 loci = make_array(nncon+1, /long, value = 0L)
                 locp = make_array(nncon+1, /long, value = 0L)
                 locb = make_array(nncon+1, /long, value = 0L)
                 nbad = make_array(nncon+1, /long, value = 0L)
                 cond = {ncon:nncon, lcfit:lcfit, x:temporary (xx), $
                    y:temporary(yy), locx:temporary(loc), $
                   ix:0L, iy:0L, loci:loci, iplotx:0L, iploty:0L, locp:locp, $
                   badxy:0L, locb:locb, nbad:nbad }
                 maxx = max(gx, min= minx)
                 maxy = max(gy, min= miny)
                 plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
                 yrange = [miny, maxy]
                 tqplot,/over
               endif else winformation, ['Cannot delete all conductors', $
                   'Use tqcond, /reset to eliminate all conductors']
             endif
             END

   'LCOVER': BEGIN
             if curoper eq 'WAIT' then begin
               widget_control, event.id, get_value = lc
               lcgrid = lc(0)
               if n_elements (lc) eq 2 then lcconductor = lc(1)
               plot, gx, gy,/nodata, xran=!x.crange,/xst, yran=!y.crange, /yst
               tqplot,/over
             endif
           END

   'SELCON': BEGIN
             if curoper eq 'WAIT' then begin
                lcstat = conductors(0)
                widget_control, event.id, get_value = conductors
;
                if conductors(0) ne lcstat then begin
                  conductors = make_array(maxcond+1,value=conductors(0), /int)
                  if cond.ncon lt maxcond then conductors(cond.ncon:*) = 0
                endif else if total(conductors(1:*)) ne maxcond then begin
                  conductors(0) = 0 
                endif else conductors(0) = 1
                widget_control, event.id, set_value= conductors
                plot, gx, gy,/nodata, xran= !x.crange,yran=!y.crange, /xst, /yst
                tqplot,/over
             endif
           END
  'MARKER': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'MARKER'
                arr = -1
                narray = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the point to add the marker'
              endif
             END

  'JOIN': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'JOIN'
                arr = -1
                narray = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the first conductor to join'
              endif
             END

  'SPLIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'SPLIT'
                arr = -1
                widget_control, tqinstr,set_value = $
                  'Indicate the point to split the conductor'
              endif
             END

  'DELCON': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                ; Find the point
                widget_control, draw, /clear_events, sensitive = 1
                curoper = 'DELETE'
                arr = -1
                 device, set_graphics = 6
                widget_control, tqinstr,set_value = $
                  'Indicate the conductor to delete'
              endif
             END

 'D_MARKER': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                if n_elements(markx) gt 0 then begin
                  if markx(0) ge 0 then begin
                    ; Find the point
                    widget_control, draw, /clear_events, sensitive = 1
                    curoper = 'D_MARKER'
                    arr = -1
                    narray = -1
                 device, set_graphics = 6
                    widget_control, tqinstr,set_value = $
                      'Indicate the point to be removed'
                  endif else print, ' No Markers to delete'
                endif else print, ' No Markers to delete'
              endif
             END

   'XFIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                curoper = 'XFIT'
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the line to be fit'
              endif
             END

   'YFIT': BEGIN
              if curoper eq 'WAIT' then begin
                ; Set the window number - important for multiple windows
                WSET, cur_win
                curoper = 'YFIT'
                arr = -1
                narray = -1
                widget_control, draw, /clear_events, sensitive = 1
                widget_control, tqinstr,set_value = $
                  'Indicate the first end point for the line to be fit'
              endif
             END

'LCREGION' : Widget_control, regionbut, get_value = lcregion

   'PRINT' : BEGIN
             if cond.nbad(0) gt 0 then print, $
                  'Bad conductor point must be resolved before printing' $
             else begin
               get_tqstring, iq, 'ENTER the File Name:', file, /modal, $
                  group = event.top
               if iq eq 0 then begin
                 openw, unit, /get, file, error= err
                 if err eq 0 then begin
                   tqprint, unit, group = event.top, /modal
                   free_lun, unit
                 endif else begin
                   Print, ' Error = ', strtrim(err, 2)+' on opening'
                   print, ' Check the directory file specification'
                 endelse
               endif
             endelse
             END
   'HELP': BEGIN
            XCUR, xx, yy,dx, dy, /modal, /snap, group = event.top
            nel = n_elements(xx)
            if nel gt 0 then begin
              if lcmode eq 'COND' then begin
                print, '    XCUR designated points: (x, y, dx, dy'
                print, xx(0), yy(0)
                if nel gt 1 then $
                  for i = 1, nel-1 do print, xx(i), yy(i), dx(i-1), dy(i-1)
              endif else begin
                print, $
               '    XCUR designated points:   Nearest Grid with offset = 2:'
                for i = 0, nel-1 do begin
                  ip = get_tqgrid(xx(i), yy(i))
                  print, xx(i), yy(i), ip(0)+2, ip(1)+2
                endfor
              endelse
              xcur_x = xx
              xcur_y = yy
            endif
            if tqfast eq 1 then mksymbol, 2 else mkcircle

            END

   'DONE': BEGIN
                if old_win ge 0 then wset, old_win
                WIDGET_CONTROL, event.top, /DESTROY
           END
    ELSE: Print, eventval,' is not supported at this time.'
      ENDCASE

      END
;
 PRO   tqedit, XSIZE=x_size, YSIZE=y_size, conductor = conductor, $
        reset = reset, modal = modal, GROUP=GROUP, fast = fast, $
        regionsize_in = regionsize
;
;+ 
;       TQEDIT
;-
@pfidltq.com
;
@tqeditevent.com
  common pffcom, initpff, pff_exe
@draw_defaults.com
  if n_elements(initpff) eq 0 then init_pff
  if n_elements(group) eq 0 then group = 0
;
      if n_elements (regionsize) eq 0 then region_size = 0.01 else $
         region_size = abs(regionsize) < 1
      if n_elements (lcfit) eq 0 then begin
        if n_elements(gx) gt 1 and n_elements(gy) gt 1 and $ 
          keyword_set(conductor) then begin
          tqcond, [min(gx, max = mxx),mxx] , [ min(gy, max =mxy), mxy]
        endif else begin
          wwarning, ['  TQEDIT cannot be used until the  ', $
                   '  grid and conductors are defined.  ', $
                   '      Use TQGRID and TQCOND.     ']
          return
        endelse
      endif
      if n_elements (cond) eq 0 then begin
        if n_elements(gx) gt 1 and n_elements(gy) gt 1 and $
          keyword_set(conductor) then begin
          tqcond, [min(gx, max = mxx),mxx] , [ min(gy, max =mxy), mxy]
        endif else begin
          wwarning, '  Use TQCOND to input conductor values  '
          return
        endelse
      endif
      si = size(cond)
      if si(si(0)+1) ne 8 then begin
        wwarning, '  Use TQCOND to input conductor values  '
        return
      endif
      if cond.ncon lt 1 then begin
        wwarning, '  Use TQCOND to input conductor values  '
        return
      endif
      if keyword_set(conductor) then begin
        lcmode= 'COND'
        title = 'Edit Theoretical Conductor Value'
      endif else begin
        lcmode = 'INPUT'
        if n_elements (gx) lt 2 or n_elements (gy) lt 2 then begin
          wwarning, '  Use TQGRID to generate a grid   '
          return
        endif
        if n_elements(cond.iy) lt 2 or n_elements(cond.ix) lt 2 then begin
          wwarning, '  Use TQFIT to fit data to the grid   '
          return
        endif
        if total(cond.lcfit) ne n_elements(cond.lcfit) then begin
          wwarning, ' Use TQFIT to fit data to the grid   '
          return
        endif
        title = 'Generate TQ Input Deck Parameters'
      endelse
      if n_elements(fast) eq 0 then tqfast = 0 else tqfast = fast(0)
      if n_elements(lcconductor) eq 0 then lcconductor = 0
      if n_elements(lcgrid) eq 0 then lcgrid = 0
      lcaspect = 0
;  do the color table
      loadxyct
      device, set_graphics = 3
      cerase = !d.table_size -1
      lcsavmode = lcmode
;
      old_win = !d.window
; The size of the draw area is one of the more important parameters
; for a draw widget.  This example uses keywords to define the size
; of the draw area.  An alternative would be to use a fixed size draw area. 
;
  if n_elements(x_size) eq 0 then x_size = bigsize(0)
  if n_elements(y_size) eq 0 then y_size = bigsize(1)
;
; Save the initial points.
;   and save the current values in the old values for "REDO"
      if lcmode eq 'COND' then begin
        old_loc = cond.locx
        old_x = cond.x
        old_y = cond.y
      endif else begin
        old_loc = cond.loci
        old_x = cond.ix
        old_y = cond.iy
      endelse
;  Set the base widget
      base = WIDGET_BASE(TITLE = title, /COLUMN)
;
;  Now the Instructional widget base
;
      insbase =  WIDGET_BASE(base, /ROW, /FRAME)
;
;  Now a base with the draw on the left and a column of buttons on the right
;
      dbase =  WIDGET_BASE(base, /ROW, /FRAME)
      lcol =  WIDGET_BASE(dbase,  /FRAME)
      rcol =  WIDGET_BASE(dbase, /COLUMN, /FRAME)
;
; A widget for instructional text is created
;
      tqinstr = WIDGET_LABEL(insbase,  VALUE = 'Instructions:')
      tqinstr = WIDGET_Text(insbase, xsize = 50, edit= 0, $
          value = 'Pick an Activity or press DONE ')
      if lcmode eq 'INPUT' then begin
       lcover = cw_bgroup(insbase, ['Overlay Grid   ','Overlay Conductors  '], $
             /nonexclusive, /row, set_value=[lcgrid, lcconductor], $
             uvalue = 'LCOVER')
      endif else begin
        if n_elements (gx) gt 3 or n_elements (gy) gt 3 then $
        lcover = cw_bgroup(insbase, ['Overlay Grid   '], $
             /nonexclusive, /row, set_value=[lcgrid], $
             uvalue = 'LCOVER')
        regionbut =  cw_bgroup(insbase,/frame, ['Point', 'Region'], $
           /exclusive, /row, set_value = 0, $
           LABEL_LEFT='Selection Mode:', $
             uvalue = 'LCREGION')
      endelse
      lcregion = 0
;  A widget called 'draw' is created.
      draw = WIDGET_DRAW(lcol, $
	/BUTTON_EVENTS, $	;generate events when buttons pressed
        /MOTION_EVENTS, $	;generate events when cursor moves over window
	/FRAME, $
	UVALUE = 'DRAW_WIN_EVENT', $
	RETAIN = 2, $		;make sure draw is redrawn when covered
	XSIZE = x_size, $
	YSIZE = y_size)
;
;  Now the buttons on the right
;
;  The DONE button:
      button1 = WIDGET_BUTTON(rcol, $
		UVALUE = 'DONE', $
		VALUE = 'DONE - EXIT')
;
;
;  Make a button which will delete a point.
      button16 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MOVE', $
		VALUE = 'MOVE A POINT')

;  Make a button which will delete a point.
      button9 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ERASE', $
		VALUE = 'DELETE A POINT')

;  Make a button which will delete a several points.
      button2 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MERASE', $
		VALUE = 'DELETE a LINE')

;  Make a button which will add a point
      button8 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ADD', $
		VALUE = 'ADD a POINT')

;  Make a button which will zoom the display 
      button4 = WIDGET_BUTTON(rcol, $
		UVALUE = 'ZOOM', $
		VALUE = 'ZOOM')

;  Make a button which will redisplay the image.
      button5 = WIDGET_BUTTON(rcol, $
		UVALUE = 'REDRAW', $
		VALUE = 'REDRAW')

;  Make a button which will cancel the current activity
;   button12 = WIDGET_BUTTON(rcol, UVALUE = 'CONDUC', VALUE = 'PLOT CONDUCTOR')

;  Make a button which will draw the grid
;    button6 = WIDGET_BUTTON(rcol, UVALUE = 'GRID', VALUE = 'PLOT GRID')

   if lcmode eq 'INPUT' then begin

;  Make a button set a marker
      button10 = WIDGET_BUTTON(rcol, $
		UVALUE = 'MARKER', $
		VALUE = 'SET MARKER')

;  Make a button which will delete a marker
      button11 = WIDGET_BUTTON(rcol, $
		UVALUE = 'D_MARKER', $
		VALUE = 'DELETE MARKER')

;  Make a button which will fit the X grid to a line 
      button12 = WIDGET_BUTTON(rcol, $
		UVALUE = 'XFIT', $
		VALUE = 'FIT X to LINE')

;  Make a button which will fit the Y grid to a line
      button13 = WIDGET_BUTTON(rcol, $
		UVALUE = 'YFIT', $
		VALUE = 'FIT Y to LINE')

;  Make a button which will print a TQ input file
      button14 = WIDGET_BUTTON(rcol, $
		UVALUE = 'PRINT', $
		VALUE = 'PRINT TQ FILE')
   endif else begin

;  Make a button which will join a conductor
      button12 = WIDGET_BUTTON(rcol, $
                UVALUE = 'JOIN', $
                VALUE = 'JOIN Conductors')

;  Make a button which will split a conductor
      button13 = WIDGET_BUTTON(rcol, $
                UVALUE = 'SPLIT', $
                VALUE = 'SPLIT Conductors')

;  Make a button which will delete a conductor 
      button14 = WIDGET_BUTTON(rcol, $
                UVALUE = 'DELCON', $
                VALUE = 'DELETE Conductor')

;  Make a button which will delete all conductors
      buttonkill = WIDGET_BUTTON(rcol, $
                UVALUE = 'DELKILL', $
                VALUE = 'Delete ALL Cond.')

   endelse
      maxcond = cond.ncon
      names = strtrim(sindgen(cond.ncon+1))
      names = ['  ALL', names(1:*)]
   conductors = make_array(maxcond+1, value=1, /int)

;  Make a button which will produce a cursor
      button15 = WIDGET_BUTTON(rcol, $
		UVALUE = 'HELP', $
		VALUE = 'XCUR')

;  Make a button which will cancel the current activity
      button7 = WIDGET_BUTTON(rcol, $
		UVALUE = 'CANCEL', $
		VALUE = 'CANCEL ACTIVITY')

;  Make a button which will undo the last change
      button3 = WIDGET_BUTTON(rcol, $
		UVALUE = 'UNDO', $
		VALUE = ' UNDO Last Action ')

;  Make a compound button  which will select the conductors
      sel_cond = CW_BGROUP(rcol, /scrol,y_scrol = 260 , x_scrol = 125, $
		 names, /nonexclusive,set_value = conductors, $ 
               LABEL_TOP='Select Conductor', UVALUE = 'SELCON')

;  Realize the widgets:
;
      WIDGET_CONTROL, base, /REALIZE
;
;  Get the window number from the draw widget.  This can only be done
;  after the widget has been realized.
;
      WIDGET_CONTROL, draw, GET_VALUE=cur_win
      WSET, cur_win
;
; Clear events and make the draw widget insensitive 
;
;
; 
        old_cond = cond
        if tqfast eq 1 then mksymbol, 2 else mkcircle
        widget_control, tqinstr,set_value = $
                  'Pick an Activity or press DONE '
        widget_control, draw, /clear_events, sensitive = 0
        curoper = 'WAIT'
        if keyword_set(reset)  and keyword_set (conductor) then begin
          gx = [min(cond.x, max = mm), mm]
          gy = [min(cond.y, max = mm), mm]
        endif
        if n_elements  (gx ) eq 0 then begin
          gx = [min(cond.x, max = mm), mm]
          gy = [min(cond.y, max = mm), mm]
        endif
        maxx = max(gx, min= minx)
        maxy = max(gy, min= miny)
        plot, gx, gy, /nodata, /xst, /yst, xrange = [minx, maxx], $
          yrange = [miny, maxy]
;
        arr = -1
        tqplot, /over
;
;  Hand off control of the widget to the XMANAGER:
;
      XMANAGER, "tqedit", base, GROUP_LEADER=GROUP, modal = modal
      Condstr = 0
      old_cond =0
;
      END
