; ------------------------------------------------------------------------ 
; $Id: tqfilter.pro,v 1.4 2010/09/21 13:59:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro tqfilter, x, y, loc, levels, layers = keeplevels, xrange = xrange, $
           yrange = yrange, xzero = xzero, yzero = yzero, zero = zero, $
           undo=undo, quiet = quiet
;
;+
;    format: tqfilter, x, y, loc, layers, layers = keeplayers, xrange = xrange, 
;          yrange = yrange, xzero = xzero, yzero = yzero, zero = zero,
;          undo=undo, quiet = quiet
;   where
;           x - X conductor values
;           y - Y conductor values
;          loc - pointer to the conductor arrays
;          layers - optional array for the layers of each conductor element
;         keeplayers - optional array of desired layers.
;          xrange - desired X-range of conductors 
;          yrange - desired Y-range of conductors
;             Note - If a single value is provided, the cursor will be used.
;             Note - Any conductors not entirely within xrange/yrange will
;                    be eliminated.
;           xzero - set minimum x to xzero value
;           yzero - set minimum y to yzero value
;           zero - set minimum x and y to zero value
;           undo - revert to the last x, y, loc, levels values passed to tqfilter
;           quiet - if present and not zero, no plot will be make of the final 
;                   result
;       examples
;             Restore previous values
;               tqfilter, a, b, c, lev, /undo
;             Take only conductors between (0, 0) and (6, 4)
;               tqfilter, x, y, loc, xr = [0, 6],yr = [0, 4]
;             Set minimum x and y values to zero
;               tqfilter, x, y, loc, zero = 0.0
;             Take only levels [0, 1]
;               tqfilter, x, y, loc, lev, layers = [0, 1]
;             Take the set of x, y, loc and layer values from previous call to 
;             tqfilter and filter for levels [0, 1, 6]
;               tqfilter, x, y, loc, lev, lay = [0, 1, 6], /undo
;-
common tqfilter_save, xxsave, yysave, locsave, levelsave
on_error , 1
          if keyword_set(undo) then begin
            if n_elements(xxsave) gt 0 then x = xxsave
            if n_elements(yysave) gt 0 then y = yysave
            if n_elements(locsave) gt 0 then loc = locsave
            if n_elements(levelsave) gt 0 then levels = levelsave
          endif else begin
            if n_elements(x) lt 2  or n_elements(y) lt 2 or $
              n_elements(loc) lt 2 then begin
              tqhelp, con= con
              if n_elements (con) eq 0 then begin
              print, ' X, Y and LOC must be defined with at least 2 elements'
              return
              endif else begin
              x = con.x
              y = con.y
              loc = con.locx
              xxsave = x
              yysave = y
              locsave = loc
              endelse
            endif else begin
              xxsave = x
              yysave = y
              locsave = loc
            endelse
            if n_elements(levels) gt 0 then levelsave = levels 
          endelse
          nlev =  n_elements(keeplevels)
          if nlev ne 0 then begin
            if n_elements (levels) lt 2  then begin
              print, ' Cannot sort levels if level information is not provided'
              print, '     or if threre is only 1 level.'
              return
            endif
            ok = -1
            nok = 0
            for i = 0, nlev-1 do begin
              ok = [ok, where (levels eq keeplevels(i), noki)]
              nok = nok+noki
            endfor
            if nok gt 0 then begin
              sok = sort (ok)
              ok = ok(sok)
              nul = where (ok eq -1, nokm)
              if nokm gt 0 then ok = ok(nokm:*)
              ind = 0
              np = loc(ok(0)+1)-loc(ok(0))
              x(ind:ind+np-1) = x(loc(ok(0)):loc(ok(0)+1)-1)
              y(ind:ind+np-1) = y(loc(ok(0)):loc(ok(0)+1)-1)
              loc(1) = np
              levels(0) = levels(ok(0))
              if nok gt 1 then for i = 1, nok-1 do begin
                ind = ind+np
                np = loc(ok(i)+1)-loc(ok(i))
                x(ind:ind+np-1) = x(loc(ok(i)):loc(ok(i)+1)-1)
                y(ind:ind+np-1) = y(loc(ok(i)):loc(ok(i)+1)-1)
                loc(i+1) = loc(i) + np
                levels(i) = levels(ok(i))
              endfor
            endif else begin
               print, ' NO levels with the specified values'
               return
            endelse
            x = x(0: loc(nok)-1)
            y = y(0: loc(nok)-1)
            levels = levels(0:nok-1)
            loc = loc(0:nok)
          endif
          nel = n_elements (loc)
          nelx = n_elements(xrange)
          nely = n_elements(yrange)
          minx =  min(x, max = maxx)
          miny = min (y, max = maxy)
          if nelx gt 1 then xrange = [min(xrange, max = mm),mm] $
            else xrange= [minx, maxx] 
          if nely gt 1 then yrange = [min(yrange, max = mm),mm] $
            else yrange= [miny, maxy] 
          if nelx eq 1 or nely eq 1 then begin
            plot, x, y, /nodata, xrang= xrange, yr = yrange
            for i = 0, nel-2 do $
              oplot, x(loc(i):loc(i+1)-1), y(loc(i):loc(i+1)-1) 
            if nelx eq 1 then begin
              print, ' Indicate with the cursor first X-location'
              cursor, start, y1, 3
              print, 'Position cursor at second X-location'
              repeat cursor, sstop, y1, 3 until start ne sstop
              xrange = [min([start, sstop], max = mm), mm]
            endif
            if nely eq 1 then begin
              print, ' Indicate with the cursor first Y-location'
              if nelx ne 1 then cursor, x1, start, 3 $
                  else repeat cursor, x1, start,1 until y1  ne start
              print, 'Position cursor at second Y-location'
              repeat cursor, x1, sstop,3 until start ne sstop
              yrange = [min([start, sstop], max = mm), mm]
            endif
          endif
          if minx lt xrange(0) or maxx gt xrange(1) or $
          miny lt yrange(0) or  maxy gt yrange (1) then begin
             ind = 0
             iloc = 0
            for i = 0, nel-2 do begin
              if min(x(loc(i):loc(i+1)-1) , max = mm) ge xrange(0) and $
                mm le xrange(1) and min(y(loc(i):loc(i+1)-1), max = mmm) $
                ge yrange(0) and mmm le yrange(1) then begin
                np = loc(i+1)-loc(i)
                x(ind:ind+np-1) = x(loc(i):loc(i+1)-1)
                y(ind:ind+np-1) = y(loc(i):loc(i+1)-1)
                if n_elements(levels) gt 1 then levels(iloc) = levels(i)
                loc(iloc+1) = loc(iloc)+np
                ind = ind+np
                iloc = iloc+1
              endif
            endfor
            if iloc eq 0 then begin
              print, 'No data in the specified range'
              return
            endif
            x = x(0:ind-1)
            y = y(0:ind-1)
            loc = loc(0: iloc)
            if n_elements(levels) gt 0 then levels = levels(0:iloc-1)
          endif
          if n_elements (zero) ne 0 then begin
            xzero = zero
            yzero = zero
          endif
          if n_elements(xzero) ne 0 then x = x- (min(x)-xzero)
          if n_elements(yzero) ne 0 then y = y- (min(y)-yzero)
          nel = n_elements (loc)
          if not(keyword_set (quiet)) then begin
            plot, x, y, /nodata
            for i = 0, nel-2 do $
              oplot, x(loc(i):loc(i+1)-1), y(loc(i):loc(i+1)-1) 
          endif
          return
          end

