; ------------------------------------------------------------------------ 
; $Id: tqfit.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqfit, conductors, close = close
;
;+
;   purpose:  Fit conductors to the grid
;   format:  tqfit, conductor, close = close
;     where:
;      conductor is an optional array specifying the conductors to be fit
;                  default is all conductors
;      close is an optional keyword to close all conductors
;     NOTE: this will be with a straight line!
;     NOTE: Close will not affect 2 point conductors
;
;    example: 
;        Fit conductors in the TQ common block to the grid
;            tqfit
;        Fit conductors for future use in TQPOTENTIAL
;            tqfit, /close
;-
@pfidltq.com
;
    on_error, 1
;   see if conductors are defined
    if n_elements(cond) eq 0 then message, $
      'Conductors must be defined to fit to a grid'
    if n_elements(gx) eq 0 then message, $
      'X-grid must be defined'
    if n_elements(gy) eq 0 then message, $
      'Y-grid must be defined'
;
    nct = cond.ncon
    if nct gt 0 then begin
      if n_elements(conductors) eq 0 then $
        conductors = indgen(nct)+1
    endif else message, 'No conductors have been defined.'
;
    if nct +1 ne n_elements(lcfit) then $
        lcfit = make_array(nct+1, /int, value = 0)
    if nct + 1 eq n_elements(cond.nbad) then nbadxy = cond.nbad else $
      nbadxy = make_array(nct+1, /int, value = 0)
    ok = where (conductors gt 0 and conductors le nct, nc)
    if nc eq 0 then message, 'No valid conductors for the fit' else $
      conductors= conductors(ok)
    location = cond.loci
    if n_elements(location) ne nct+1 then location = make_array(nct+1, /int)
    nix = cond.ix
    niy = cond.iy
    locaplot = cond.locp
    if n_elements(locaplot) ne nct+1 then locaplot = make_array(nct+1, /int)
    pix = cond.iplotx
    piy = cond.iploty
    locx = cond.locx
    locabad = cond.locb
    if n_elements(locabad) ne nct+1 then locabad = make_array(nct+1, /int)
    bxy = cond.badxy
    nbadxy = cond.nbad
    for i = 0, nc-1 do begin
      j = conductors(i)
      cx = cond.x(locx(j-1):locx(j)-1)
      cy = cond.y(locx(j-1):locx(j)-1)
      np = n_elements (cx)
      con2ind, gx, gy, cx, cy, ixx, iyy, np, close = close
      if np eq 0 then begin
        print, 'The following conductor must be deleted or moved: ', $
          strtrim(j+1, 2)
        return
      endif
      array2pack, j, nct, location,np,nix, ixx, niy, iyy
      lcfit(j) = 1
    endfor
    fit = where (lcfit(1:*)  eq 0, nbad)
    if nbad eq 0  then lcfit(0) = 1 else lcfit(0) = 0
;
    cond = {ncon:cond.ncon, lcfit:lcfit, x:cond.x, y:cond.y, locx:cond.locx, $
             ix:nix, iy:niy, loci:location, iplotx:pix, iploty:piy, $
             locp:locaplot, badxy:bxy, locb:locabad, nbad:nbadxy }
;
     get_tqplotpnts 
     get_tqbad
    return
    end
