; ------------------------------------------------------------------------ 
; $Id: tqplotcon.pro,v 1.4 2007/06/06 20:12:26 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqplotcon, color = color, fill = fill, orientation= orientation , $
              grid = grid,  _extra = extrastuff
;
;+
;   Purpose:  Plot a tq conductor 
;             NOTE:  This is an overplot which requires a grid.
;   format:  tqplotcon  [, COLOR = color] [, FILL = fill] [, GRID = grid]
;              [, ORIENTATION = orientation] [, _EXTRA = extrastuff]
;       Color:   Color to draw the conductor
;                Multiple values are cycled. If nc is the number of colors and
;                nd is the dataset number, then the dielectric color is:
;                   dielectric_color = color[nd mod nc]
;       GRID:    Draw a grid.
;                If GRID is a structure, pass the structure to the plot command.
;                Default parameters are: XTITLE = 'X-Axis', YTITLE = 'Y-Axis'
;       FILL:    If set, POLYFILL rather than OPLOT is used
;       ORIENTATION: IF set, this is passed to POLYFILL for LINE FILL.
;                    If ORIENTATION eq 1, then ORIENTATION = [-45., 45.]
;       _EXTRA:  Passed to PLOT or POLYFILL.
;
;    example: 
;        Plot the current conductors with fill and cycle through first 3 colors
;            TQPLOTCON , /fill, color = [1, 2, 3]
;
;        Plot the current conductors and cycle through colors
;        Plot a grid with the following parameters
;        {title:'Anode and Cathode',xtitle:'Radius',ytitle:'Z-Axis',xrange:[0, 3.5],xstyle:1}
;           grid = {title:'Anode and Cathode',xtitle:'Radius',ytitle:'Z-Axis',xrange:[0,.35],xstyle:1}
;           tqplotcon, /fill, color=1+indgen(3), grid = grid
;-

@pfidltq.com
;
    on_error, 1
;   see if conductors are defined
    if n_elements(cond) eq 0 then begin
      print, '% TQPLOTCON: Conductors must be defined'
      return
    endif
    if size(cond, /type) ne 8 then begin
      print, '% TQPLOTCON: Conductor structure not initialized'
      return
    endif
    if cond.ncon le 0 then begin
      print, '% TQPLOTCON: Number of conductors is zero.'
      return
    endif
;
    ncon = cond.ncon
;   See if we need a grid
    if keyword_set(grid) then begin
      lc_extra = (size(grid, /type) eq 8 ) ? 1 : 0
      xrange = [min(cond.x, max = m), m]
      yrange = [min(cond.y, max = m), m]
      xtitle = 'X-Axis'
      ytitle = 'Y-Axis'
      if lc_extra then plot, xrange, yrange, /nodata, xrange= xrange, $
        yrange = yrange, xtitle = xtitle, ytitle = ytitle, _extra = grid $
      else plot, xrange, yrange, /nodata, xrange= xrange, $
        yrange = yrange, xtitle = xtitle, ytitle = ytitle
    endif
;
    if n_elements(color) eq 0 then color = !p.color
    ncol = n_elements(color)
    if keyword_set(fill) then begin
      if keyword_set(orientation) then begin
        line_fill = 1
        if n_elements(orientation) eq 1 && orientation[0] eq 1 then $
          orient = [-45., 45.] else orient = orientation
        norient = n_elements(orient)
      endif
    endif
    for i = 0, ncon-1 do begin
      if keyword_set(fill) then begin
        if keyword_set(line_fill) then $
          for j = 0, norient-1 do polyfill, cond.x[cond.locx[i]:cond.locx[i+1]-1], $
            cond.y[cond.locx[i]:cond.locx[i+1]-1], color = color[i mod ncol],  $
            orientation = orient[j], noclip = 0, _extra = extrastuff $
        else $
          polyfill, cond.x[cond.locx[i]:cond.locx[i+1]-1], cond.y[cond.locx[i]:cond.locx[i+1]-1], $
            color = color[i mod ncol], noclip = 0, _extra = extrastuff

      endif else oplot, cond.x[cond.locx[i]:cond.locx[i+1]-1], cond.y[cond.locx[i]:cond.locx[i+1]-1], $
        color = color[i mod ncol], _extra = extrastuff
    endfor
    end
