; ------------------------------------------------------------------------ 
; $Id: tqset_system.pro,v 1.4 2010/09/21 14:00:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  Name: tqset_system
;  Purpose: Set the TQ geometry coordinate system.
;           Normally this will be done automatically by other routines.
;
;  Format: tqset_system, lcquit, offset = offset
;    offset - optional parameter to set the position of the widget
;    lcquit - 0 = OK pressed
;             1 = QUIT pressed
;-

 Pro set_system_event, Event
  on_error, 1
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

; value = {lcquit:0, hand:lcs, val:tqslice, system:tqsystem, offset:offset}
; info={xval:field11, hand :bgroup5, system:bgroup6, ptr:ptr}
  CASE Ev OF

  'EXIT': BEGIN
      but = event.value
      widget_control, event.top, get_uvalue = info, /no_copy, $
              tlb_get_offset = offset

      case but of
 'QUIT' : lcquit = 1
 'OK' :   lcquit = 0
       ELSE: BEGIN
              lcquit = 1
              print, ' Unknown event - QUIT'
             END
       endcase
          widget_control,info.xval, get_value = xvalue
          widget_control,info.system, get_value = system
          widget_control, info.hand, get_value = lcs
          value = {lcquit:lcquit, val:xvalue, hand:lcs, system:system, $
                  offset:offset}
          widget_control, info.ptr, set_uvalue = value, /no_copy
          widget_control, event.top, /destroy 

      END
   ELSE:
  ENDCASE
END


 Pro tqset_system, lcquit, offset = offset, group = group

on_error, 1
  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

; common block
;
@pfidltq.com
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit, tqsystem, tqslice, syst_list

  Base = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Coordinate System and Slice Value', $
      UVALUE='MAIN13')
  BASE1 = WIDGET_BASE(BASE, $
      COLUMN=1, $
      FRAME = 2, $
      MAP=1, $
      UVALUE='BASE1')
  Btns39 = [ $
    'Right Hand System', $
    'Left Hand System']

  if n_elements (tqsystem) eq 0 then lcs = 0 else begin
      if tqsystem lt 0 then lcs = 1 else lcs = 0
  endelse

  BGROUP5 = CW_BGROUP( BASE1, Btns39, $
      Row=1, set_value = lcs, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='BGROUP5')

  BASE2 = WIDGET_BASE(BASE, $
      COLUMN=1, $
      FRAME = 2, $
      MAP=1, $
      UVALUE='BASE2')

  syst_list = [ $
    'Cartesian (X, Y)', $
    'Cartesian (Y, Z)', $
    'Cartesian (Z, X)', $
    'Cylindrical (Z, R)', $
    'Cylindrical (R, Phi)', $
    'Spherical (Theta, Phi)', $
    'Spherical (R, Theta)', $
    'Spherical (Phi, R)' ]

  if n_elements(tqsystem) eq 0 then tqsystem = 0 else $
   tqsystem = abs (tqsystem)-1 > 0
  print, tqsystem
  BGROUP6 = CW_BGROUP( BASE2, syst_list, $
      COLUMN=1, set_value = tqsystem, $
      EXCLUSIVE=1, /NO_RELEASE, $
      UVALUE='BGROUP6')

  BASE8 = WIDGET_BASE(BASE, $
      COLUMN=1, $
      FRAME = 2, $
      MAP=1, $
      UVALUE='BASE8')

  Label10 = WIDGET_LABEL( BASE8, $
      UVALUE='LABEL10', $
      VALUE= 'Enter Value of Missing Coordinate:')

  if n_elements(tqslice) eq 0 then tqslice = 0.0

  FIELD11 = CW_FIELD( BASE8,VALUE=tqslice, $
      ROW=1, $
      FLOAT=1, $
      TITLE='Value:', $
      UVALUE='FIELD11')


      menu = ['     OK     ',  '  Quit  ']
  but = cw_bgroup(base, menu,/row, /NO_RELEASE, $
        uvalue = 'EXIT', button_uvalue =['OK','QUIT'])

  if n_elements (offset) lt 2  then begin
    offset = [0, 0]
    WIDGET_CONTROL, BASE, /REALIZE
  endif else $
  WIDGET_CONTROL, BASE, /REALIZE, tlb_set_xoffset = offset(0)> 10, $
       tlb_set_yoffset= offset(1) > 10
  ptr = widget_base ()
  value = {lcquit:0, hand:lcs, val:tqslice, system:tqsystem, offset:offset}
  info={xval:field11, hand :bgroup5, system:bgroup6, ptr:ptr}

  widget_control, ptr, set_uvalue = value, /no_copy
  Widget_control, base, set_uvalue = info

  XMANAGER, 'set_system', base
  widget_control, ptr, get_uvalue = value, /no_copy
  widget_control, ptr, /destroy
  tqsystem = value.system+1
  if value.hand eq 1 then tqsystem = -tqsystem
  tqslice = value.val
  offset = value.offset
END

