; ------------------------------------------------------------------------ 
; $Id: qsprint.pro,v 1.4 2007/06/06 20:12:42 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  qsprint, unit, grid = grid, transp= transp, path = path
;+
; print tq common block to a file for QS input
; format;
;     qsprint, unit
;        where unit is the logical unit 
;-
@pfidltq.com
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
;
on_error, 1
  if n_elements(unit) eq 0 then message, $
   ' Unit for print must be defined'
if keyword_set(grid) then begin
  if cond.lcfit(0) eq 0 then begin
    print , ' Data must be fit to the grid'
    return
  endif
  xx = gx(cond.ix)
  yy = gy(cond.iy)
  loc = cond.loci
endif else begin
  xx = cond.x
  yy = cond.y
  loc = cond.locx
endelse
  nx = n_elements (xx)
  ny = n_elements (yy)
  dimension = 3
  zbeg = 0.0000
  zend = 4.0
  normal = 2
  offset = [20, 20]
  dimension = 0
  xunits = [1.0, 0.01, 0.001, 1.0e-6, 2.54e-2, 2.54e-5]
  units = ['Meters', 'Centimeters', 'Millimeters', 'Microns', 'Inches', 'Mils']
  normalvalue = ['i' , 'j', 'k']
  ncon = n_elements(cond.locx)-1
  xran = [min(gx, max = mm), mm]
  yran = [min(gy, max = mm), mm]
  nx = n_elements(gx)-1
  xranc= [xran(0)-gx(1)+gx(0), xran(1)+gx(nx)-gx(nx-1)]
  ny = n_elements(gy)-1
  yranc= [yran(0)-gy(1)+gy(0), yran(1)+gy(ny)-gy(ny-1)]
  for i = 1, ncon do begin
    tx = xx(loc(i-1):loc(i)-1)
    ty = yy(loc(i-1):loc(i)-1)
    nc= n_elements (tx)
    tranx = [min(tx, max = mm), mm]
    trany = [min(ty, max = mm), mm]
    if tranx(0) gt xranc(1) or tranx(1) lt xranc(0) or $
      trany(0) gt yranc(1) or trany(1) lt yranc(0) then begin
      print, 'No data for conductor ', strtrim(i, 2)
      endif else begin
      plot, xx, yy, /nodata, xrange = xran, yrange = yran
      for ii = 1, ncon do oplot, xx(loc(ii-1):loc(ii)-1), $
        yy(loc(ii-1):loc(ii)-1), line = 2
      if nc gt 0 then begin
        oplot, tx, ty, color = 4
        if keyword_set(transp) then begin
          temptx = temporary(tx)
          tx = temporary(ty)
          ty = temporary(temptx)
          endif
        conductor = 'strip'+ strtrim(i, 2)
        get_qscond_info, lc,units, conductor,type, normal, zbeg, zend, $
           dimension, offset, path = path
        if lc eq 0 then begin
          printf, unit, '*'
          printf, unit, '**** START - ', conductor
          printf, unit, '*'
          if keyword_set(path) then begin
;           Diagnostic stuff
            np = n_elements(tx)
            norm_val= make_array(np, value = float(zbeg))
            if normal eq 0 then con= [norm_val, tx, ty] $
              else if normal eq 1 then con= [ty, norm_val, tx] $
              else con = [tx, ty, norm_val]
            con = transpose(reform(temporary(con)*mult, np, 3))
            for i = 0, np-2 do printf, unit,  format='(2(e16.5, e13.5 , e13.5), ''   & '')', $
               con[*, i], con[*, i+1]
          endif else begin
            if (nc mod 2) eq 0 then begin
              if type lt 3 then printf, unit, ' conductor   ' , ' poly  ',  '   &' $
                  else  printf, unit, ' dielectric   ' , ' poly  ',  '   &'
              con = [tx, ty] * mult
              con = reform (temporary(con), nc, 2)
              con = transpose(temporary(con))
              printf, unit,  format='(2(e16.5, e13.5 ), ''   & '')', con
                endif else begin
              if type lt 3 then printf, unit, $
                  ' conductor   ' , ' poly ', tx(0)*mult, $
                  ty(0)*mult,  '   &' else printf, unit, $
                 ' dielectric   ' , ' poly ', tx(0)*mult, $
                  ty(0)*mult,  '   &'
              con = [tx(1:*), ty(1:*)] * mult
              con = reform (temporary(con), nc-1, 2)
              con = transpose(temporary(con))
              printf, unit,  format='(2(e16.5, e13.5 ), ''   & '')', con
            endelse
            if type eq 1 then last = '   -1' else if type eq 2 then $
                last = '    0' else last = ''
            printf, unit, '  ', normalvalue(normal), zbeg *xunits(dimension), $
              zend*xunits(dimension),'   ', conductor, last
          endelse
        printf, unit, '*'
        printf, unit, '**** END - ', conductor
        printf, unit, '*'
      endif
    endif
    endelse
  endfor
  return
  end
