; ------------------------------------------------------------------------ 
; $Id: xfit_tqgrid.pro,v 1.3 2007/06/06 20:12:43 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  xfit_tqgrid, p1, p2
;+
;   Purpose: Fit the X-grid to a line between two points P1, and P2
;   format:
;       xfit_tqgrid, p1, p2
;       where P1 and P2 are 2-element vectors
;   Example:
;       fit the grid between the  points, [0, 2], [2, -3]
;       xfit_tqgrid, [0, 2], [2, -3]
;
;-
@pfidltq.com
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
;
      on_error, 1
      if p2(0) eq p1(0) then begin
       print, ' Cannot fit a verticle line'
       return
      endif
      if p2(0) lt p1(0) then begin
        temp = p1
        p1 = p2
        p2 = temp
      endif
;
      slope = (p2(1)-p1(1))/(p2(0)-p1(0))
      if slope eq 0 then begin
        print, ' Current gridding is adequate'
        return
      endif
      if slope lt 0 then begin
        aslope = -slope
      endif else aslope = slope
;     clip the line to the grid
      nx = n_elements (gx)
      ny = n_elements (gy)
      xmin =  gx(0)
      xmax = max(gx)
      ymin = gy(0)
      ymax = max(gy)
;     clip to the grid
      print, xmin, xmax, ymin, ymax
      if p1(0) lt xmin then begin
        p1(1) = p1(1) + (xmin-p1(0))*slope
        p1(0) = xmin
      endif
      if p1(1) gt ymax then begin
        p1(0) = p1(0) +(ymax-p1(1))/slope
        p1(1) = ymax
      endif
      if p1(1) lt ymin then begin
        p1(0) = p1(0) +(ymin-p1(1))/slope
        p1(1) = ymin
      endif
      if p2(0) gt xmax then begin
        p2(1) = p2(1) + (xmax-p2(0))*slope
        p2(0) = xmax
      endif
      if p2(1) gt ymax then begin
        p2(0) = p2(0) +(ymax-p2(1))/slope
        p2(1) = ymax
      endif
      if p2(1) lt ymin then begin
        p2(0) = p2(0) +(ymin-p2(1))/slope
        p2(1) = ymin
      endif
      x1 = p1(0)
      x2 = p2(0)
      if p2(1) lt p1(1) then begin
        y1 = p2(1)
        y2 = p1(1)
      endif else begin
        y1 = p1(1)
        y2 = p2(1)
      endelse
;
      ist = get_tqgrid (x1, y1)
      iend = get_tqgrid (x2, y2)
;
      if ist(1) lt 2 then yd = ymin else yd = gy(ist(1)-1)
      if iend(1) +2 ge ny then yu = ymax else yu = gy(iend(1)+1)
;     find the x and y values which are within 4 grid lines of the region
      oky =where (y ge yd-gdy(ist(1)) and y le yu+gdy(iend(1)<ny-2), noky)
;     fix the x values
      if noky gt 0 then begin
        xnew = make_array(noky)
        dxnew = xnew
        for i = 0, noky-1 do begin
          j = oky(i)
          yy =  y(j)
          xnew(i) = p1(0) + (yy - p1(1))/slope
          del = min (abs(yy-gy), n)
          dxnew(i) = -gdy(n < ny-2)/aslope
        endfor
        ok = sort(xnew)
        xnew = xnew(ok)
        dxnew = dxnew(ok)
      endif
;
      if ist(0) le 3 then xd = xmin else xd = gx(ist(0)-3)
      if iend(0) +4 ge nx then xu = xmax else xu = gx(iend(0)+3)
;
      okx =where (x ge xd-gdx(ist(0)) and  x le xu+gdx(iend(0)<nx-2), nokx)
      if nokx gt 0 then begin 
        if okx(0) eq 0 then begin
          yy = p1(1) + (x(0)-p1(0))*slope
          del =  min (abs(yy-gy), n)
          dx(0)= -gdy(n< ny-2)/aslope
          nokx = nokx -1
          if nokx gt 0 then okx = okx(1:*)
        endif
        nxa = n_elements(x)-1
        if nokx gt 0 then begin
          if okx(nokx-1) eq nxa then begin
            yy = p2(1) + (x(nxa)-p2(0))*slope
            del =  min (abs(yy-gy), n)
            dx(nxa)= -gdy(n< ny-2)/aslope
            nokx = nokx -1
            if nokx gt 0 then okx = okx(0:nokx-1)
          endif
        endif
        if nokx gt 0 then x(okx) = x(0)-10.*gdx(0)
      endif
;
      if n_elements (xnew) gt 0 then begin
        x = [x, xnew]
        dx = [dx, dxnew]
      endif
;
      del = min(abs(x-x1), n)
      if del gt 2.0*(dx(n)) then begin
        x = [x, x1]
        del = min(abs(gy-p1(1)), j)
        dx  = [dx, -gdy(j< ny-2)/aslope]
      endif 
      del = min(abs(x-x2), n)
      if del gt 2.0*abs(dx(n)) then begin
        x = [x, x2]
        del = min(abs(gy-p2(1)), j)
        dx  = [dx, -gdy(j< ny-2)/aslope]
      endif 
      tqgrid, /xaxis
      return
      end
