; ------------------------------------------------------------------------ 
; $Id: check_identical.pro,v 1.3 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro check_identical, array1, array2, lcident
; 
;+
;   Check to see if two structures have identical spatial components
;   format: check_identical, st1, st2, lcident
;      where: st1 is the first structure
;             st2 is the second structure
;             lcident is returned 0 or neg if they are not zero
;                                 1 if they are identical to an error
;                                   of 6.1e-5
;-
      on_error, 2
      lcident = 0
      a = size (array1)
      b = size (array2)
;
      if a(a(0)+1) ne 8 or b(b(0)+1) ne 8 then return
;
      if array1.ndim ne array2.ndim then begin
        if array1.two eq 0 or  array2.two eq 0 then return
        if array1.two ne array2.two then return
      endif
;
      if array1.nvect  ne array2.nvect  then return
;
      s1 = n_elements(array1.v1)
      s2 = n_elements(array2.v1)
;
      if s1 ne s2 then return
;
      if max(abs(array1.size - array2.size)) gt 0 then return
;
      if array1.two ne 1 then begin
        d = max(abs(array1.range(*,0,0)))
        if d eq 0.0 then d = 1.
        if max(abs(array1.x1-array2.x1))/d gt 6.1e-5 then return
      endif
;
      if array1.ndim ge 2 and array1.two ne 2 then begin
        d = max(abs(array1.range(*,1,0)))
        if d eq 0.0 then d = 1.
        if max(abs(array1.x2-array2.x2))/d gt 6.1e-5 then return
      endif
;
      if array1.ndim ge 3 and array1.two ne 3 then begin
        d = max(abs(array1.range(*,2,0)))
        if d eq 0.0 then d = 1.
        if max(abs(array1.x3-array2.x3))/d gt 6.1e-5 then return
      endif
;
      if array1.ndim ge 4 and array1.two ne 4 then begin
        d = max(abs(array1.range(*,3,0)))
        if d eq 0.0 then d = 1.0
        if max(abs(array1.x4-array2.x4))/d gt 6.1e-5 then return
      endif
;
      lcident = 1
      return
      end
