; ------------------------------------------------------------------------ 
; $Id: cyl2cartesian.pro,v 1.4 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro cyl2cartesian, a, kvalue, smooth = smoo
;+
; Convert a field structure from a cylindrical to to a cartesian mesh
; or 
; Convert a field structure from a spherical to to a cartesian mesh
; The resultant structure is a single block finite element structure.
; 
; Store the result in maxstructure-1
;-
      type = 8L
      ndim = 3L
      coordinate = a.spare[0]
      if coordinate eq 1 then begin
        print, '% CYL2CART: Structure must have cylindrical or'
        print, '            spherical geometry.'
        a=0
        return
      endif
      x3 = a.slice
      two = a.two
      if coordinate eq 2 then begin
;       cylindrical coordinates
        if ((a.ndim eq 3) and (two ne 3)) then begin
          print, '% CYL2CART: Structure must have a degenerate 3 dimension'
          a = 0
          return
        endif
        nblo = a.block
        x3 = a.slice
        ntot = n_elements(a.v1)
        triangles = make_array(3, 2*ntot, /long)
        pt = 0L
        xx = make_array(ntot)
        yy = xx
        px = 0L
        for iblo = 0, nblo-1 do begin
          x = get_array(a,1, iblo+1)
          ok = where (x eq 0.0, nok)
          if nok gt 0 then x(ok) = x(ok) + 1e-4*max(x)
          y = get_array(a,2, iblo+1)
          nx = n_elements(x)
          ny = n_elements(y)
          nxy = nx*ny
          cy = cos(y)
          sy = sin(y)
          yy(px:px+nxy-1) = reform(x # sy, nxy, /overwrite)
          xx(px:px+nxy-1) = reform(x # cy, nxy, /overwrite)
          tri = make_triangles(0, nx, ny)+px
          ntri = (size(tri))(2)
          triangles(*, pt:pt+ntri-1) = tri
          pt = pt+ntri
          px = px+nxy
        endfor
        zz = x3
        clab = a.clab + ', Sliced at Z = '+strtrim(x3,2)
        xlab = 'X'
        ylab = 'Y'
      endif else begin
;     Spherical coordinates
;     coordinate = 3
        if two eq 1 then begin
          PRINT, '% CYL2CARTESIAN: Radius may not be degenerate dimension'
          a = 0
          return
        endif
        nblo = a.block
        zval = 5-two
        ntot = n_elements(a.v1)
        triangles = make_array(3, 2*ntot, /long)
        pt = 0L
        xx = make_array(ntot)
        yy = xx
        px = 0L
        if two eq 2 then stheta = sin(x3)
        for iblo = 0, nblo-1 do begin
          x = get_array(a,1, iblo+1)
          ok = where (x eq 0.0, nok)
          if nok gt 0 then x(ok) = x(ok) + 1e-4*max(x)
          y = get_array(a, zval, iblo+1)
          nx = n_elements(x)
          ny = n_elements(y)
          nxy = nx*ny
          cy = cos(y)
          sy = sin(y)
          if two eq 2 then begin
            yy(px:px+nxy-1) = reform(x # sy*stheta, nxy, /overwrite)
            xx(px:px+nxy-1) = reform(x # cy*stheta, nxy, /overwrite)
          endif else begin
            yy(px:px+nxy-1) = reform(x # cy, nxy, /overwrite)
            xx(px:px+nxy-1) = reform(x # sy, nxy, /overwrite)
          endelse
          xlab = 'X'
          if two eq 2 then ylab = 'Y' else ylab = 'Z'
          a.spare[0] = 0
          zz = a.slice
          tri = make_triangles(0, nx, ny)+px
          ntri = (size(tri))(2)
          triangles(*, pt:pt+ntri-1) = tri
          pt = pt+ntri
          px = px+nxy
        endfor
        if two eq 2 then $
          clab = a.clab +', Slice at Theta = ' + strtrim(x3, 2) $
        else $
          clab = a.clab +', Slice at Phi = ' + strtrim(x3, 2)
        two = 3
      endelse
      kvalue = 3
      locx = [0L, px]
      locv = locx
      loctri = [0L, pt]
      range = make_array(2, 3, 2) 
      range(*, 2, *)=a.slice
      range(0, 0, *)= min(xx, max = mm)
      range(1, 0, *)= mm
      range(0, 1, *)= min(yy, max = mm)
      range(1, 1, *)= mm

      ssize = [px, px, 1]
      if n_elements(smoo) eq 0 then smoo = 0
      if smoo gt 2 then begin
        for i = 1, a.nvect do begin
          for j = 1, nblo do begin
            v = reform(get_array(a, -i, j))
            v = smooth(v, smoo)
            put_array, a, v, -i, j
          endfor
        endfor
      endif
  
      a = {type: type, ndim: ndim, nvect: a.nvect, block:1L, $
          two: 3L, slice: a.slice, size: ssize, range: range, order: 0L, $
            locx:locx, locv: locv, triangles: triangles(*,0:pt-1), $
            loctri: loctri, x1:xx, x2:yy, x3:a.slice, x4:a.x4, $
            v1:a.v1, v2:a.v2, v3:a.v3, v4:a.v4, v5:a.v5, v6:a.v6, $
            x1lab : xlab, x2lab : ylab, x3lab : 'Z', $
            x4lab :a.x4lab, v1lab : a.v1lab , v2lab : a.v2lab, $
            v3lab : a.v3lab, v4lab : a.v4lab, v5lab : a.v5lab, $
            v6lab : a.v6lab, file : a.file, spare : a.spare, tlab : a.tlab, $
                clab: clab  }
;       save the slice
        i2s, a, get_maxstr()-1
        smoo = 0
      end  

