; ------------------------------------------------------------------------ 
; $Id: fillmitlstr.pro,v 1.3 2007/06/06 19:34:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro fillmitlstr, limit
;+
; calculate the plot parameters
; data is already sorted with respect to x and duplicate values removed
; setup up plot arrays 
; limit is the default change in amplitude between points
;    if not specified limit = 1%
;-
@mitl_common.com
;
on_error, 1
  if str.np eq 0 then message, ' NO points in MITL structure.'
  if str.np lt 2 then message, ' Less than 2 points in MITL structure.'
  nplot = 1024L
  pi2 = !pi*0.5
  zvac = make_array(nplot)
  zflow = zvac
  pos = zvac
  zvac(0) = str.zv(0)
  if min(str.zv) le 0.0 then begin
     message, ' Vacuum Impedance Must be GE Zero.'
  endif
  uratio = (zvac(0)+zvac(0))*umax/(zvac(0)+zsource)
  if str.zf(0) le 0.0 or lcexpert eq 0 then begin
       zflow(0)= zvac(0)*uratio/(uratio+1.022)
       str.zf(0) = zflow(0)
  endif else zflow(0) = str.zf(0)
  pos(0) = str.x(0)
  totrange = str.x(str.np-1) - pos(0)
  epsilon = 1e-5*totrange
  if n_elements (limit) eq 0 then limit = 0.0075
  limit2 = limit*0.5
  delz = (str.zv(1:str.np-1)-str.zv(0:str.np-2))
  delx = (str.x(1:str.np-1)-str.x(0:str.np-2))
  ok = where (delx le 0.0 , nok)
  if nok gt 0 then begin
    print, 'Multiple x values at ', (str.x)(ok)
    message, ' Cannot continue, Delta X LE zero.'
  endif
  delzz = delz/str.zv(0:str.np-2)
  npos = 1
  if str.zfcon(0) gt 0.0 and str.zfdwn(0) gt 0.0 then begin
    winformation, ['First Point may NOT be a convolute', $
                  'Convolute impedances set to Zero']
    str.zfcon(0)=0.0
    str.zfdwn(0)=0.0
  endif
  total_lc = str.lcemit(0)
  for i = 1, str.np-1 do begin
    del = abs(delzz(i-1))
    if  str.zf(i) le 0.0 or lcexpert eq 0 then begin
        str.zf(i)= str.zv(i)*uratio/(uratio+1.022)
        if total_lc ne 0 then $
             str.zf(i)= str.zf(i)  < zflow(npos-1)
    endif
    if str.lcemit(i) ne 0 then total_lc = 1
    delzf = abs(zflow(npos-1)-str.zf(i))/(zflow(npos-1)+str.zf(i))
    if del gt limit2 or delzf gt limit2 then begin
      dnp = 1+round(max([del, delzf])/limit)> 3L
      dnpm = dnp -1
      while ((npos +dnp) ge nplot) do begin
        nplot= nplot+nplot
        zvac = [zvac, zvac]
        zflow = [zflow, zflow]
        pos = [pos, pos]
      endwhile
      smoother = (findgen(dnp)+1.0)/dnp
      pos(npos:npos+dnpm) = pos(npos-1) + smoother *(str.x(i) - pos(npos-1))
      if str.fit(i-1) eq 1 then $
           smoother = sin(pi2 *smoother)^2 $
        else if str.fit(i-1) eq 2 then begin
          del = (str.zv(i)/str.zv(i-1))^smoother
          delf = (str.zf(i)/zflow(npos-1))^smoother
      endif
      if str.fit(i-1) eq 2 then $
          zvac(npos:npos+dnpm) = str.zv(i-1)*del else $
          zvac(npos:npos+dnpm) = str.zv(i-1)*(1.0 + smoother*delzz(i-1))
      if str.fit(i-1) eq 2 then $
          zflow(npos:npos+dnpm) = zflow(npos-1)*delf else $
          zflow(npos:npos+dnpm) = zflow(npos-1)+ smoother* $
                  (str.zf(i)-zflow(npos-1))
;
; if a convolute set the downstream flow impedance
;
      if str.zfcon(i) ne 0.0 and str.zfdwn(i) ne 0.0 then begin
          zflow(npos+dnp+findgen(2)) = str.zfdwn(i)
          zvac (npos+dnp+findgen(2)) = zvac (npos+dnpm)
          pos (npos+dnp+findgen(2)) = pos (npos+dnpm) + findgen(2)*epsilon
          pos (npos+dnpm)= pos (npos+dnpm) -epsilon
          dnp= dnp+2
      endif else begin
;
; if not a convolute limit zflow - make sure last point was not a convolute
;
         if ((str.zfcon(i-1) le 0.0) or (str.zfdwn(i-1) le 0.0)) and $
           lcexpert eq 0 then begin
           ok = where (2*zflow(npos:npos+dnpm) lt zvac(npos:npos+dnpm), nok)
           if nok gt 0 then zflow(npos+ok) = 0.5*zvac(npos+ok)
         endif
       endelse   
    endif else begin
      dnp = 3
      dnpm = 2
      if npos +dnp ge nplot then begin
        nplot= nplot+nplot
        zvac = [zvac, zvac]
        zflow = [zflow, zflow]
        pos = [pos, pos]
      endif
      del = epsilon*delz(i-1)/(str.x(i)-pos(npos-1))
      zvac(npos:npos+2) = [str.zv(i-1)+del, str.zv(i)-del, str.zv(i)]
      del = epsilon*(str.zf(i)-zflow(npos-1))/(str.x(i)-pos(npos-1))
      zflow(npos:npos+2) = [zflow(npos-1)+del, str.zf(i)-del, str.zf(i)]
      pos(npos:npos+2) = [pos(npos-1)+epsilon, str.x(i)-epsilon, str.x(i)]
      if str.zfcon(i) ne 0.0 and str.zfdwn(i) ne 0.0 then begin
          zflow(npos+dnp+findgen(2)) = str.zfdwn(i)
          zvac (npos+dnp+findgen(2)) = zvac (npos+dnpm)
          pos (npos+dnp+findgen(2)) = pos (npos+dnpm)+(1.0+findgen(2))*epsilon
          dnp= dnp+2
      endif
    endelse
    npos = npos+dnp
  endfor
  zvac= zvac(0: npos-1)
  zflow = zflow(0:npos-1)
  pos = pos(0:npos-1)
  str = {np:str.np, x:str.x, zv:str.zv, zf: str.zf, fit:str.fit, $
        zfcon:str.zfcon, zfdwn:str.zfdwn,lcemit:str.lcemit, npos:npos, $
        zvac: zvac, zflow:zflow, pos:pos}
  return
  end
