; ------------------------------------------------------------------------ 
; $Id: get_common_time.pro,v 1.4 2007/06/06 19:34:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  get_common_time, array1, array2, max1, max2, np1, np2, xydata, $
         junk, match_only = match_only, sortx = sortx
;
;      put two WDF arrays on a common time base
;
      on_error, 1
;  
      nel1 = n_elements (array1)
      nel2 = n_elements (array2)
      if nel1 eq np1+ np1 then begin
        xy1 = 1 
        min1 = min(array1[0:np1-1])
      endif else begin
        xy1 = 0
        min1 = array1[0]
      endelse
      if nel2 eq np2+ np2 then begin
        xy2 = 1 
        min2 = min(array2[0:np2-1])
      endif else begin
        xy2 = 0
        min2 = array2[0]
      endelse
;     check the mean against machine accuracy and the range against pff.
      start_err = max([abs([min1, max1, min2, max2])/32, $
              (max1-min1), (max2- min2)]) /32768.
      start_err2 = start_err+start_err
;     See if we are nearly equal already
      if np1 eq np2 and (abs(min1 - min2) le start_err) then begin
;       First check XY data
        if  xy1*xy2 eq 1 then begin
;         we have xydata, and the number of points and start values are equal
          if max((array1(0:np1-1) - array2(0:np1-1)), min=mm)- mm le $
            start_err2 then begin
            xydata = 1
            return
          endif
        endif else if (xy1 ne xy2) then begin
          if xy1 eq 0 then begin
            if max((array1[0]+findgen(np1)*array1[1]-array2(0:np1-1)),min=mm) $
               -mm le start_err2 then begin
              xydata = 1
              array1 = [array2[0:np1-1], array1[2:np1+1]]
              return
            endif
          endif else begin
            if max((array2[0]+findgen(np2)*array2[1]-array1(0:np1-1)),min=mm) $
               -mm le start_err2 then begin
              xydata = 1
              array2 = [array1[0:np1-1], array2[2:np1+1]]
              return
            endif 
          endelse
        endif
      endif 
      if xy1 +xy2 gt 0 then begin
        if xy1 eq 0 then del1 = array1[1] else $
           del1 = array1[1:np1-1] - array1[0:np1-2]
        if xy2 eq 0 then del2 = array2[1] else $
           del2 = array2[1:np2-1] - array2[0:np2-2]
        mb = max([del1, del2], min = ms)

        if ms eq 0.0 then ms = 1e-6*mb
        if(not keyword_set(match_only) and mb/ms gt 100) $
          or  keyword_set(sortx) then begin
          if xy1 eq 1 then begin
            x1 = array1[0:np1-1]
            y1 = array1[np1:np1+np1-1]
            i = sort(x1)
            x1 = x1[i]
            y1 = y1[i]
          endif else begin
            x1 = array1[0]+findgen(np1)* array1[1]
            y1 = array1[2:np1+1]
          endelse
          if xy2 eq 1 then begin
            x2 = array2[0:np2-1]
            y2 = array2[np2:np2+np2-1]
            i = sort(x2)
            x2 = x2[i]
            y2 = y2[i]
          endif else begin
            x2 = array2[0]+findgen(np2)* array2[1]
            y2 = array2[2:np2+1]
          endelse
          xx =  [x1, x2]
          xx = xx[uniq (xx, sort(xx))]
          lower = max([x1[0], x2[0]])
          upper = min([max1, max2])
  
          ok = where (xx ge lower and xx le upper, nok)
          if nok gt 2 then xx = xx(ok) else begin
            array1 = 0
            np1 = nok
            return
          endelse
          array1 = make_array(2*nok)
          array2 = make_array(2*nok)
          array1[0:nok-1] = xx
;          xtemp = get_yarr(x1, findgen(np1), xx)
;          array1[nok:*] = interpolate(float(y1), xtemp)
          array1[nok:*] = interpolate(float(y1), findex(x1, xx))
  
          array2[0:nok-1] = xx
;          xtemp = get_yarr(x2, findgen(np2), xx)
;          array2[nok:*] = interpolate(float(y2), xtemp)
          array2[nok:*] = interpolate(float(y2), findex(x2, xx))
          np1 = nok
          xydata = 1
          return
        endif
        if keyword_set(match_only) then begin
          xydata = 1
          if xy1 eq 1 and ((np1 lt np2) or (xy2 eq 0)) then begin
;       begin match to dataset 1
            x1 = array1(0:np1-1)
            y1 = array1(np1:np1+np1-1)
            if (match_only lt 0.00) then err = -(match_only) else $
              err = (max(x1, min = mm) -mm)/100./np1
            if xy2 eq 0 then begin
              offset = 2
              x2 = array2(0)+findgen(np2)*array2(1)
            endif else begin
              offset = np2
              x2 = array2(0:np2-1)
            endelse
            x2d = x2-err
            x2u = x2+err
            y2 = y1
            j = 0
            for i = 0, np1-1 do begin
              ok = where(x1(i) ge x2d and x1(i) le x2u, nok)
              if nok gt 0 then begin
                x1(j) = x1(i)
                y1(j) = y1(i)
                y2(j) = total(array2(ok+offset))/nok
                j = j+1
              endif
            endfor
            if j lt 2 then begin
              print, '% GET_COMMON_TIME: Sorry, less than 2 point overlap '
              print, '                   with an exact match in the data'
              np1 = 0
              array1 = 0
              array2 = 0
            endif else begin
              array1= [x1(0:j-1), y1(0:j-1)]
              array2= [x1(0:j-1), y2(0:j-1)]
              np1 = j
              np2 = j
            endelse
          endif else begin
;       begin match to dataset 2
            x1 = array2(0:np2-1)
            y1 = array2(np2:np2+np2-1)
            if xy1 eq 0 then begin
              offset = 2
              x2 = array1(0)+findgen(np1)*array1(1)
            endif else begin
              offset = np1
              x2 = array1(0:np1-1)
            endelse
            if match_only lt 0.00 then err = -(match_only) else $
              err = (max(x2, min = mm) -mm)/100./np1
            x2d = x2-err
            x2u = x2+err
            y2 = y1
            j = 0
            for i = 0, np2-1 do begin
              ok = where(x1(i) ge x2d and x1(i) le x2u, nok)
              if nok gt 0 then  begin
                x1(j) = x1(i)
                y1(j) = y1(i)
                y2(j) = total(array1(ok+offset))/nok
                j = j+1
              endif
            endfor
            if j lt 2 then begin
              print, '% GET_COMMON_TIME: Sorry, less than 2 point overlap '
              print, '                   with an exact match in the data'
              np1 = 0
              array1 = 0
              array2 = 0
            endif else begin
              array2= [x1(0:j-1), y1(0:j-1)]
              array1= [x1(0:j-1), y2(0:j-1)]
              np1 = j
              np2 = j
            endelse
          endelse
          return
;       end matches 
        endif
      endif
;     put datasets on a uniform time base
      xydata = 0
      if xy1 ne 0 then begin
        space1 = array1(0:np1-1)
        image1 = array1(np1:*)
        i2w, space1, image1,delta1, /get_delta, endpoint = endpoint
        nel1 = round ((endpoint-space1(0))/delta1) +3
        nels1 = n_elements(space1)
      endif else begin
        delta1 = array1(1)
        space1 = array1(0)
      endelse
      if xy2 ne 0 then begin
        space2 = array2(0:np2-1)
        image2 = array2(np2:*)
        i2w, space2, image2,delta2, /get_delta, endpoint = endpoint
        nel2 = round ((endpoint-space2(0))/delta2) +3
        nels2 = n_elements(space2)
      endif else begin
        delta2 = array2(1)
        space2 = array2(0)
      endelse
       
;     find the start time
;     allow an error 
;
      delta_err = (delta1+delta2)/2097152.
;
      if (abs(space1(0) - space2(0)) le start_err) and $
        (abs(delta1 - delta2) le delta_err) then begin
        if ((nel1 ne nel2) or (xy1+xy2 gt 0)) then begin
          n = min([nel1, nel2])-1
          start = (space1(0)+space2(0))*0.5
          delta = (delta1+delta2)*0.5
          stopp = 1.000002*(n-2)*delta1 + start
          if xy1 eq 0 then array1= temporary(array1(0:n)) else $
            put2uniform, space1, image1, nels1, start, delta, stopp, array1
          if xy2 eq 0 then array2= temporary(array2(0:n)) else $
            put2uniform, space2, image2, nels2, start, delta, stopp, array2
        endif
      endif else begin
        start = max( [space1(0), space2(0)])
        stopp  = min([max1, max2])
        if start ge stopp  then begin
          print, '% GET_COMMON_TIME: Sorry, there is no overlap in the data'
          np1 = 0
          array1 = 0
          array2 = 0
          return
        endif
;       find the minimum delta
;
        delta = min([delta1, delta2])
        delta_err = delta/2097152.0
        start_err = max([abs(start)/2097152.0, delta_err])
        n = ceil((stopp-start)/delta -.125)>2
        delta = (stopp -start)/n
        ndelta = n+3
        if xy1 eq 1 then $ 
          put2uniform, space1, image1, nels1, start, delta, stopp, array1 $
        else begin
        if ((abs(start-array1(0)) lt start_err) and $
          (abs(delta-array1(1)) lt delta_err)) then begin
          if nel1 lt ndelta then ndelta = ndelta-1
          array1 = temporary(array1(0:ndelta-1))
        endif else begin
          sp = (start-array1(0))/array1(1) + $
             findgen(ndelta-2)*(delta/array1(1))
          spi = long(sp)
          spx = sp -spi
          delarr = [array1(1:nel1-1)-array1(0:nel1-2), 0.0]
          spi = spi+2
          array1= [start, delta, array1(spi)+spx*delarr(spi)]
        endelse
        endelse
        if xy2 eq 1 then $
          put2uniform, space2, image2, nels2, start, delta, stopp, array2 $
        else begin
        if ((abs(start-array2(0)) lt start_err) and $
          (abs(delta-array2(1)) lt delta_err)) then begin
          if nel2 lt ndelta then begin
            ndelta = ndelta-1
            array1 = temporary(array1(0:ndelta-1))
            array2 = temporary(array2(0:ndelta-1))
          endif else array2 = temporary(array2(0:ndelta-1))
        endif else begin
          sp = (start-array2(0))/array2(1) + $
             findgen(ndelta-2)*(delta/array2(1))
          spi = long(sp)
          spx = sp -spi
          delarr = [array2(1:nel2-1)-array2(0:nel2-2), 0.0]
          spi = spi+2
          array2= [start, delta, array2(spi)+spx*delarr(spi)]
        endelse
        endelse
      endelse
      return
      end
