; ------------------------------------------------------------------------ 
; $Id: get_fast_fft.pro,v 1.5 2007/06/06 19:34:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_fast_fft, array, mode, limit

;+ 
; Name: GET_FAST_FFT
; 
; Purpose: Modify an array to increase the speed of an fft.
; 
; Format: 
;      GET_FAST_FFT, ARRAY, MODE, LIMIT
; where 
;        ARRAY     - Array to be modified
;        MODE      - Type of array modification desired
;                    1  - Resample points to a new spacing
;                         Sample delta is now = (np-1)*delta1/(new_n-1)
;                    2  - Linear extrapolation from the last point toward
;                         the magnitude of the first point
;        LIMIT     - Maximum modification allowed always at least 1 point
;                    If limit <= 0, then modify to the next power of 2.
;                    If limit > 1 then it is in number of points.
;                    if limit in [0, 1] then it is a fraction of the total.
;
;-
compile_opt strictarr
 COMPILE_OPT hidden

on_error, 1
nel = n_elements(array)-2
if n_elements(mode) eq 0 then mode = 0
if mode eq 0 or mode gt 2 then begin
  print, '% GET_FAST_FFT: Bad mode value'
  return
endif

if limit le 0 then begin
; go to powers of 2 or slightly better, 3*(powers of 2)
  nmax = 2L^(ceil(alog(nel)/alog(2))) > nel
  if nmax lt 64 then np = nmax else begin
    if nel lt nmax/4*3 then np =  nmax/4*3 else np = nmax
  endelse
endif else begin
  if limit lt 1 then limit = limit*nel
  nmax = nel + (long(limit) > 1)
  nopt = 2L^(ceil(alog(nel)/alog(2))) > nel
  if nmax gt nopt then np = nopt else begin
    fudge = nmax-nel
    if fudge lt 129 then begin
      a = make_array(fudge+1, value = 0L)
      for i = 0, fudge do a[i]=total(factors(nel+i))
      nt = min(a, i)
      np = nel+i
      del = 1
    endif else begin
      del = 2L^((ceil(alog(fudge)/alog(2))-6)> 1) 
      n = nel - (nel mod del)
      del = 2L^((ceil(alog(nmax-n)/alog(2))-6)> 1) 
      nn = round ((nmax-n)/del)
      a = make_array(nn+1, value = 0L)
      if n eq nel then a[0]= total(factors(nel)) else a[0]= 9999
      for i = 1, nn do a[i]=total(factors(n+i*del))
      nt = min(a, i)
      np = n+i*del
    endelse
  endelse
endelse
if mode eq 1 then begin
  delta = (nel-1)*array[1]/(np-1)
   put2uniform, findgen(nel)*array[1]+array[0], array[2:*], nel, $
    array[0], delta, (np-.999)*delta+array[0], array
endif else  if np gt nel then begin
  nadd = np-nel
  array = [array, (findgen(nadd)+1.0)*((array[2]-array[nel+1])/(nadd+1))+ $
            array[nel+1]]
endif

end
