; ------------------------------------------------------------------------ 
; $Id: get_max_min.pro,v 1.3 2007/06/06 19:34:28 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_max_min, x, y, xmax, ymax, xmin, ymin, quiet = quiet, smooth = smoo
;
;+
;    get_max_min, x, y, xmax, ymax, xmin, ymin, quiet= quiet, smooth = smooth
;    
;    this routine calculates them local maximum and minimum of an input 
;      waveform (x, y)
;      (Envelope calculation)
;    the derivative is calculated and filtered with a box car filter 
;    the points at which the derivative changes are then recorded for 
;       the maximums (xmax, ymax) and the minimums (xmin, ymin)
;    Quiet - If present and not zero, no plot of the derivative is made.
;    Smooth - Specifies the number of points to be used in the smoothing.
;             Default is 5, minimum is 0.  Any number over zero is rounded to 
;             an odd number greater than 3
;-
     if n_elements (quiet) eq 0 then quiet = 0
     if n_elements (smoo) eq 0  then smoo = 5
     if smoo gt 0 then smoo = smoo >3
     seed = make_array(512)
     xmax = seed
     ymax = seed
     xmin = seed
     ymin = seed
     nseed = 512
     ixtot = 512
     intot = 512
;    
     np = n_elements(y)
     d = make_array(np-1)
     t = d
     t = 0.5*(x(0:np-2)+x(1:np-1))
     ydel = y(1:np-1)-y(0:np-2)
     xdel = x(1:np-1)-x(0:np-2)
     d = ydel/xdel
     if smoo gt 0 then d = smooth (d,smoo)
     if quiet eq 0 then plot, t, d, title= 'Derivative of input data'
     ix = 0
     in = 0
     for i = 0, np-3 do begin
       if d(i) gt 0.0 then begin
         if d(i+1) le 0.0 then begin
           xx = t(i)+(t(i+1)-t(i))*d(i)/(d(i)-d(i+1))
           ymax(ix) = get_y(x(i:i+2),y(i:i+2),xx)
           xmax(ix) = xx
           ix = ix+1
           if ix eq ixtot then begin
             ixtot = ixtot + nseed
             ymax = [ymax, seed]
             xmax = [xmax, seed]
           endif
         endif
       endif else if d(i) lt 0.0 then begin
         if d(i+1) ge 0.0 then begin
           xx = t(i)+(t(i+1)-t(i))*d(i)/(d(i)-d(i+1))
           ymin(in) = get_y(x(i:i+2),y(i:i+2),xx)
           xmin(in) = xx
           in = in+1
           if in eq intot then begin
             intot = intot + nseed
             ymin = [ymin, seed]
             xmin = [xmin, seed]
           endif
         endif
       endif
     endfor
     if in gt 0 then begin
       ymin = ymin(0:in-1)
       xmin = xmin(0:in-1)
     endif else print , 'NO minimums were found'
     if ix gt 0 then begin
       ymax = ymax(0:ix-1)
       xmax = xmax(0:ix-1)
     endif  else print , 'NO maximums were found'
     return
     end
