; ------------------------------------------------------------------------ 
; $Id: get_rect_xyz.pro,v 1.3 2007/06/06 19:34:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_rect_xyz,  x, nx, y, ny, z, nz, xcoord, ycoord, zcoord, order, ier, $
       delta = delta, quiet = quiet
;+
; convert xcoord, ycoord, zcoord to 3 arrays of x, y, and z assuming
; If n_elements(Zcoord) eq 1 then do only 2 dimensional data
; that the data is on a rectilinear grid.
;  nx*ny*nz should equal n_elements(xcoord)
;  values assumed equal if they differ by less than 1/2^16 =(1.55e-5)
;-
 on_error, 2
  if n_elements(delta) eq 0 then delta =  1.55e-5

  ntot = n_elements(xcoord)
  order = make_array(ntot, /long, value = 0L)
  temp = make_array(ntot, /long)

  if delta gt 0 then $
    del = (max([xcoord, ycoord, zcoord], min = mm) -mm)*delta

  if n_elements (zcoord) eq 1 then begin
    nz = 1L
    z = make_array(1, value = float(zcoord(0)))
  endif else begin
    i = sort (zcoord)
    z = zcoord(i)
    rr = max(z, min = m)-m
    dmax =0.000
    jf = 0L
    nz =0L
    if delta lt 0 then $
      del = -delta*(z(ntot-1)-z(0))
    for j =1L, ntot-1 do begin
      if z(j)-z(j-1) gt del then begin
        z(nz) = total (z(jf:j-1))/(j-jf)
        dnew = z[j-1]-z[jf]
        if dnew gt dmax then dmax = dnew
        order (i(jf:j-1)) = nz
        jf = j
        nz = nz +1L
      endif
    endfor
    z(nz) = total (z(jf:ntot-1))/(ntot-jf)
    order (i(jf:ntot-1)) = nz
    z = z(0:nz)
    nz = nz +1L
    dmax = dmax/rr
    if dmax gt 0.0001 then print, '% GET_RECT_XYZ: Z coordinates vary by ', $
       strtrim(dmax, 2), ' of their range'
  endelse

  i = sort (ycoord)
  y = ycoord(i)
  rr = max(y, min = m)-m
  dmax =0
  jf = 0L
  ny =0L
  if delta lt 0 then $
    del = -delta*(y(ntot-1)-y(0))
  for j =1L, ntot-1 do begin
    if y(j)-y(j-1) gt del then begin
      y(ny) = total (y(jf:j-1))/(j-jf)
      dnew = y[j-1]-y[jf]
      if dnew gt dmax then dmax = dnew
      temp (i(jf:j-1)) = ny
      jf = j
      ny = ny +1L
    endif
  endfor
  y(ny) = total (y(jf:ntot-1))/(ntot-jf)
  temp(i(jf:ntot-1)) = ny
  y = y(0:ny)
  ny = ny +1L
  order = ny*order+temp
  dmax = dmax/rr
  if dmax gt 0.0001 then print, '% GET_RECT_XYZ: Y coordinates vary by ', $
       strtrim(dmax, 2), ' of their range'
  i = sort (xcoord)
  x = xcoord(i)
  rr = max(x, min = m)-m
  dmax =0
  jf = 0L
  nx =0L
  if delta lt 0 then $
    del = -delta*(x(ntot-1)-x(0))
  for j =1L, ntot-1 do begin
    if x(j)-x(j-1) gt del then begin
      x(nx) = total (x(jf:j-1))/(j-jf)
      dnew = x[j-1]-x[jf]
      if dnew gt dmax then dmax = dnew
      temp (i(jf:j-1)) = nx
      jf = j
      nx = nx +1L
    endif
  endfor
  x(nx) = total (x(jf:ntot-1))/(ntot-jf)
  temp(i(jf:ntot-1)) = nx
  x = x(0:nx)
  nx = nx +1L
  dmax = dmax/rr
  if dmax gt 0.0001 then print, '% GET_RECT_XYZ: X coordinates vary by ', $
       strtrim(dmax, 2), ' of their range'
  order = nx*order+temp
  temp = 0
  ier = 0
  nel = nx*ny*nz

  if nel ne ntot  or nel lt 4 then begin
    if not keyword_set (quiet) then begin
      print, '% GET_RECT_XYZ: Number of elements is incorrect or lt 4. '
      print, nx, ny, nz, nx*ny*nz, ntot
    endif
    ier = 1
  endif else ier = 0
  return
  end
