; ------------------------------------------------------------------------ 
; $Id: get_yarr.pro,v 1.3 2007/06/06 19:34:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
function get_yarr, xarray, yarray, x, error = err, no_truncate = no_truncate, $
           sortit = sortit
;+
;     Format: y = get_Yarr(xarray, yarray,x,error = err,no_truncate=no_truncate)
;     xarray and x are sorted
;     this function returns the y value of a function for a passed x array
;     xarray is assumed to have at least as many elements as yarray
;     The routine looks form the minimum of (xarray -x) and interpolates
;     No value is returned if x is outside the values of xarray  to within
;        err = 2^(-14) of the maximum or the specified value
;     no_truncate - if set, return zeros for all values outside the range of x
;                   if not set, The x array is modified to reflect the number
;                               of values returned
;     sortit      - if set, xarray will be sorted if necessary
;-
 on_error, 1
      nel = n_elements(yarray)
      if nel le 1 then return, 0.0
      if (nel gt n_elements(xarray)) then begin
        print, '% GET_YARR: XArray must have as many elements as the Y array.'
        return,0.0
      endif
      dxx = xarray(1:nel-1) - xarray(0:nel-2)
      ok = where (dxx le 0,nok)
      if nok gt 0 then begin
        if keyword_set(sortit) then begin
          j = sort(xarray)
          xarray = temporary(xarray(j))
          yarray = temporary(yarray(j))
          dxx = xarray(1:nel-1) - xarray(0:nel-2)
          ok = where (dxx eq 0,nok)
          while nok gt 0 do begin
            yarray[ok] = (yarray[ok] + yarray[ok+1])*0.5
            j = indgen(nel)
            j[ok] = -1
            j = j [where (j ge 0)]
            xarray= xarray[j]
            yarray= yarray[j]
            nel = n_elements (xarray)
            dxx = xarray(1:nel-1) - xarray(0:nel-2)
            ok = where (dxx eq 0,nok)
          endwhile
        endif else begin
          if min(dxx) lt 0.0 then begin
            print, '% GET_YARR: XArray must be sorted (derivative ge 0)'
            print, xarray[ok], xarray[ok+1]
            return , 0.0
          endif
        endelse
      end
; add some fudge room
      if size(x, /type) lt 4 then x = float(x)
      nel_x = n_elements(x)
      if n_elements (err) eq 0 then err = 6.10352e-05*max(abs(x))
      if err le  0 then  err = 6.10352e-05*max(abs(x))
      ok = where (x ge xarray(0)-err and x le xarray(nel-1)+err, nok)
      if nok eq 0 then begin
        print, '% GET_YARR: X array has no overlap with the function'
        return , 0.0
      end
      if keyword_set(no_truncate) then oksave = ok
      x = x(ok)
      if nok gt 1 then begin
        dx = x(1:nok-1) - x(0:nok-2) 
        ok = where (dx lt 0,nok2)
        if nok2 gt 0 then begin
          print, '% GET_YARR: X array must be sorted'
          help, ok, nok2
          return , 0.0
        endif
      endif
      nelx = n_elements(x)
      retarr = make_array(nelx)
      ok = where (x ge xarray(nel-1), nok)
      if nok gt 0 then begin
        retarr(nelx-nok:nelx-1) = yarray(nel-1)
        nelx = nelx-nok
      endif 
      ok = where (x le xarray(0), nok)
      if nok gt 0 then begin
        retarr(0:nok-1) = yarray(0)
      endif 
;      i = 1L
;      for j = nok, nelx-1 do begin
;        while x(j) ge xarray(i) do i = i+1L 
;        retarr(j) = (x(j)-xarray(i))/dxx(i-1)* $
;               (yarray(i)-yarray(i-1)) + yarray(i)
;      endfor
      retarr = interpolate(yarray, findex(xarray, x))
      if keyword_set(no_truncate) and nel_x gt n_elements(retarr) then begin
        ret = make_array(nel_x, value= 0.0)
        ret(oksave) = retarr
        return, ret
      endif else return, retarr
      end
