; ------------------------------------------------------------------------ 
; $Id: inside_poly.pro,v 1.3 2007/06/06 19:34:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
FUNCTION Inside_poly, x, y, px, py
;+
; from http://www.dfanning.com/tips/point_in_polygon.html
; This solution is provided by Baard Krane (bard.krane@fys.uio.no) at the
; University of Oslo. He writes: 
;
;      The code is based on an algorithm which takes the dot and cross product
; of vectors from the point itself to neighbouring points in the polygon. If
; we write the expressions for the dot and cross product, we see that by
; dividing the two we get the tangent of the angle between two vectors
; extending from the point to two neighbouring points in the polygon. A sum
; over all angles should then give a very small number if the point is outside
; the polygon. Rather than choosing an arbitrarily small number to check, I
; just choose to compare the total of the
; Format:
;    is_it = inside_poly(x, y, px, py)
; isit - The return value of the function is 1 if the point is inside the
;        polygon and 0 if it is outside the polygon.
;  x   - The x coordinate of the point.
;  y   - The y coordinate of the point.
; px   - The x coordinates of the polygon.
; py   - The y coordinates of the polygon.
;-

    sx = Size(px)
    sy = Size(py)
    IF (sx[0] EQ 1) THEN NX=sx[1] ELSE RETURN, -1    ; Error if px not a vector
    IF (sy[0] EQ 1) THEN NY=sy[1] ELSE RETURN, -1    ; Error if py not a vector
    IF (NX EQ NY) THEN N = NX ELSE RETURN, -1        ; Incompatible dimensions
    
    tmp_px = [px, px[0]]                             ; Close Polygon in x
    tmp_py = [py, py[0]]                             ; Close Polygon in y
     
    i = indgen(N)                                    ; Counter (0:NX-1)
    ip = indgen(N)+1                                 ; Counter (1:nx)
     
    X1 = tmp_px(i)  - x 
    Y1 = tmp_py(i)  - y
    X2 = tmp_px(ip) - x 
    Y2 = tmp_py(ip) - y
    
    dp = X1*X2 + Y1*Y2                               ; Dot-product
    cp = X1*Y2 - Y1*X2                               ; Cross-product
    theta = Atan(cp,dp)
    IF (Abs(Total(theta)) GT !PI) THEN RETURN, 1 ELSE RETURN, 0
END
