; ------------------------------------------------------------------------ 
; $Id: minmaxstructure.pro,v 1.3 2007/06/06 19:34:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro MINMAXSTRUCTURE, strin, vector, xrange=xrange, yrange=yrange, $
    zrange=zrange, maximum = maxval, minimum = minval, $
    pmaximum = pmaximum, pminimum = pminimum, ier = ier

;+
; calculate the maximum and minimum of a structure
;-
on_error, 1
@qsstrcom.dat

maxval = 0
minval = 0
pminimum = 0
pmaximum = 0

If n_elements (strin) eq 0 then begin
print, '% MINMAXSTRUCTURE: Structure or structure array number must be provided'
  return
endif

  arr = get_valid_structure(strin, 'MINMAXSTRUCTURE', lcstr=lcstr, ierror=ier)
  if ier ne 0 then return
  ier = 1

  if arr.type ne 1 then begin
    Print, '% MINMAXSTRUCTURE: Structure must be a Type 1, field structure'
    return
  endif
if n_elements (vector) gt 0 then fld = vector else begin
    if arr.nvect gt 1 then fld = 1+indgen(arr.nvect) else fld = 1
endelse
if n_elements (xrange) lt 2 then xrange = 0 else $
    xrange =  [min(xrange, max = mm), mm]
if n_elements (yrange) lt 2 then yrange = 0 else $
    yrange =  [min(yrange, max = mm), mm]
if n_elements (zrange) lt 2 then zrange = 0 else $
    zrange =  [min(zrange, max = mm), mm]
;
;  initial value logical
  if (max(fld) gt max([arr.nvect, arr.ndim])) then fld = 1+indgen(arr.nvect)
  fld = (abs(fld) > 1)< arr.nvect
  if n_elements (fld) gt 1 then fld = fld[uniq(fld, sort(fld))]
  b = size(fld) 
  if b(0) eq 0 then lc_abs = 0 else lc_abs = 1
  nfld = n_elements(fld)

for nblo = 1, arr.block do begin
  x = get_array(arr, 1, nblo)
  y = get_array(arr, 2, nblo)

  nokx = N_elements(x)
  noky = N_elements(y)
  if n_elements (xrange) gt 1 then begin
    nx = where (x ge xrange[0] and x le xrange[1], nokx)
    if nokx gt 0 then nx = [nx[0], nx[nokx-1]]
  endif else nx =[0, (nokx-1)]
  if nokx gt 0 then begin
    if n_elements (yrange) gt 1 then begin
      ny = where (y ge yrange[0] and y le yrange[1], noky)
      if noky gt 0 then ny = [ny[0], ny[noky-1]]
    endif else ny = [0, (noky-1)]
    if noky gt 0 then begin
      if  arr.ndim gt 2 then begin
        z = get_array(arr, 3, nblo)
        nokz = N_elements(z)
        if n_elements (zrange) gt 1 then begin
          nz = where (z ge zrange[0] and z le zrange[1], nokz)
          if nokz gt 0 then nz = [nz[0], nz[nokz-1]]
        endif else nz = [0, (nokz-1)]
      endif else begin
        nz = [0, 0]
        nokz = 1
        z = 0
      endelse
      if nokz gt 0 then begin
  v1 =(get_array(arr,-fld[0],nblo))[nx(0):nx[1],ny(0):ny[1],nz(0):nz[1]]
        if nfld eq 1 then begin
          if lc_abs gt 0 then v1 = abs(v1)
        endif else begin
          v1 = v1*v1
          for i = 1, nfld-1 do begin
  v2 = (get_array(arr,-fld[i],nblo))[nx(0):nx[1],ny(0):ny[1],nz(0):nz[1]]
            v1 = temporary(v1)+v2*v2
          endfor
          v1 = sqrt(v1)
        endelse
        mmin = min(v1, j)
        dz = ((j) / (nokx*noky)) > 0
        j = j-dz*nokx*noky
        dy = ((j) / nokx) > 0
        dx = j-dy*nokx
        locmin = [x(nx[0]+dx), y(ny[0]+dy), z(nz[0]+dz)]
        mmax = max(v1, j)
        dz = ((j) / (nokx*noky)) > 0
        j = j-dz*nokx*noky
        dy = ((j) / nokx) > 0
        dx = j-dy*nokx
        locmax = [x(nx[0]+dx), y(ny[0]+dy), z(nz[0]+dz)]
        if ier gt 0 then begin
          ier = 0
          pminimum =  locmin
          pmaximum =  locmax
          minval = mmin
          maxval = mmax
        endif else begin
          if minval gt mmin then begin
            pminimum =  locmin
            minval = mmin
          endif 
          if maxval lt mmax then begin
            pmaximum =  locmax
            maxval = mmax
          endif
        endelse
      endif
    endif
  endif 
endfor
  return
  end

