; ------------------------------------------------------------------------ 
; $Id: mk_poly_surface_object.pro,v 1.3 2007/06/06 19:34:31 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; Make a surface object from an image.
; Image is a two dimensional image and x, y, and z are coordinates.
;
; Written: March 8, 2000 lpmix
;-
 function mk_poly_surface_object,image, x, y, z, lev = lev, palette = palette, $
   min_color = min_color, zrange = zrange, normal = normal

 on_error, 2

 ss = size(image)
 if ((ss[0] lt 2) or (ss[0] gt 3)) then begin
   print, '% MK_POLY_SURFACE_OBJECT: Image must have two or three dimensions"
   return, 0
 endif
 if n_elements (z) eq 0 then z = 0.0
 nx = ss[1] 
 ny = ss[2] 
 
 if n_elements(zrange) eq 0 then begin
   Print, '% MK_POLY_SURFACE_OBJECT:

 if n_elements(lev) lt 2 then lev = [min(image, max = mm), mm] else $
   lev= [min(lev, max= mm), mm]

 if ss[0] eq 3 then nz = ss[3]  else nz = n_elements(z)
 if nx eq 1 then begin
   nxx = ny
   nyy = nz
   xx = image
   yy = y # make_array(nz, value = 1.0)
   zz = make_array ( ny, value = 1.0) # z
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_SURFACE_OBJECT: Bad data NX = 1
     print, nx, ny, nz, ss
     return, 0
   endif
   conn = fill_poly_arrays(yy, zz, y, z)
   if n_elements(conn) lt 4 then return, 0
 endif else if ny eq 1 then begin
   nxx = nx
   nyy = nz
   xx = x # make_array ( nyy, value = 1.0)
   yy = make_array (nxx, nyy, value = float(y[0]))
   zz = make_array (nxx, value = 1.0) # z
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_SURFACE_OBJECT: Bad data NY = 1
     print, nx, ny, nz, ss
     return, 0
   endif
 endif else if nz eq 1 then begin
   nxx = nx
   nyy = ny
   xx = x# make_array ( nyy, value = 1.0)
   yy = make_array (nxx, value = 1.0) # y
   zz = image
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_SURFACE_OBJECT: Bad data NZ = 1
     print, nx, ny, nz, ss
     return, 0
   endif
 endif else begin
   print, '% MK_POLY_SURFACE_OBJECT: Bad data - period
   print, nx, ny, nz
   return, 0
 endelse

 nxm = nx-1
 nym = ny-1
 npoly = nxm*nym
 conn = lonarr(5*npoly)
 conn [5*lindgen(npoly)] = 4L
 
 row = [ 0L, 1, nx+1,nx]
 elem = [1,2,3,4l]
 for j = 0, nym-1 do begin
   offset = j*nxm
   offset2 = j*nx
   for i = 0, nxm-1 do conn[(i+offset)*5+ elem] = (offset2+i)+row
 endfor
 if n_elements(palette) ne 0 then begin
  if size(palette, /type) eq 11 then opal = palette
 endif
 if n_elements (opal) eq 0 then begin
   loadct, 5
   tvlct, /get, r, g, b
   opal = obj_new ('IDLgrPalette', r, g, b)
 endif
 xx = reform (xx, nx*ny, /over)
 yy = reform (yy, nx*ny, /over)
 if n_elments (min_color) eq 0 then $
 vv = bytscl(reform (ima, nx*ny, /over), min = lev[0], max = lev[1]) else $
 vv = bytscl(reform (ima, nx*ny, /over), min = lev[0], max = lev[1], $
       top = 255-min_color)+byte (min_color > 0)

 omodel = obj_new('idlgrpolygon', xx, yy, zz, polygons = conn, $
    vert_colors = vv, shading =1, palette = opal)

 return, omodel
 end

