 function my_read_tiff, fname 
 on_error, 1
 openr, unit, /get, fname
 ft = bytarr(2)
 readu, unit, ft
 free_lun, unit
 if string(ft) eq 'MM' then $
   openr, unit, /get, fname,  /swap_if_little_endian $
 else $
   openr, unit, /get, fname,  /swap_if_big_endian
 tiff = 0
 start =0L
 readu, unit, ft, tiff, start
 if tiff ne 42 then begin
   print, '% READTIFF: This does not appear to be a TIFF file'
   help, tiff
   print, '            Value of 42 expected'
   return, 0
 endif
; go to the Image File Header
 point_lun, unit, start
; number of image file directory entries
 nifde = 0
 readu, unit, nifde
 dtag = 0
 dtype = 0      
 nval = 0L
 v2 = uintarr(2)
 v1 = ulong(1)
 for i = 1, nifde do begin
   readu, unit, dtag, dtype, nval
   if dtag eq 256 then begin
     xsize = 0L
;    Image Width
     if dtype eq 3 then begin
       readu, unit, v2
       xsize = long(v2[0])
     endif else readu, unit, xsize
   endif else if dtag eq 257 then begin
;    Image Length
     ysize = 0L
     if dtype eq 3 then begin
       readu, unit, v2
       ysize = long(v2[0])
     endif else readu, unit, ysize
   endif else if dtag eq 258 then begin
;    Bits per Sample
     if nval eq 1 then begin
       if dtype eq 3 then begin
         readu, unit, v2
         nbits = long(v2[0])
       endif else begin
         readu, unit, v1
         nbits = long(v1)
       endelse
     endif else begin
       readu, unit, start 
       point_lun, -unit, pos
       point_lun, unit, start
       if dtype eq 3 then begin
         nbits = intarr(nval)
         readu, unit, nbits
       endif else begin
         nbits = lonarr(nval)
         readu, unit, nbits
       endelse
       point_lun, unit, pos
     endelse
   endif else if dtag eq 273 then begin
;    Stirp Offsets
     if nval eq 1 then begin
       if dtype eq 3 then begin
         readu, unit, v2
         offset = long(v2[0])
       endif else begin
         readu, unit, v1
         offset = long(v1)
       endelse
     endif else begin
       readu, unit, start 
       point_lun, -unit, pos
       point_lun, unit, start
       if dtype eq 3 then begin
         offset = intarr(nval)
         readu, unit, offset
       endif else begin
         offset = lonarr(nval)
         readu, unit, offset
       endelse 
       point_lun, unit, pos
     endelse
   endif else if dtag eq 277 then begin
;    Samples per Pixel
     if dtype eq 3 then begin
       readu, unit, v2
       samples = long(v2[0])
     endif else begin
       samples = 0L
       readu, unit, samples
     endelse
   endif else if dtag eq 278 then begin
;    Rows per strip
     if dtype eq 3 then begin
       readu, unit, v2
       rows = long(v2[0])
     endif else begin
       rows = 0L
       readu, unit, rows
     endelse
   endif else if dtag eq 279 then begin
;    Strip byte counts
     if nval eq 1 then begin
       if dtype eq 3 then begin
         readu, unit, v2
         byt_strip = long(v2[0])
       endif else begin
         readu, unit, v1
         byt_strip = long(v1)
       endelse
     endif else begin
       readu, unit, start
       point_lun, -unit, pos
       point_lun, unit, start
       if dtype eq 3 then begin
         byt_strip = intarr(nval)
         readu, unit, byt_strip
       endif else begin
         byt_strip = lonarr(nval)
         readu, unit, byt_strip
       endelse
       point_lun, unit, pos
     endelse
   endif else readu, unit, v1
 endfor 
;  Now read the image
   if n_elements (nbits) eq 0 then nbits = -1
   if nbits[0] eq 8 then type = 1 else type = 12
   if n_elements (nbits) gt 1 or nbits[0] lt 0 then begin
     if min(nbits) ne max(nbits) or nbits[0] lt 0 then begin
       print, '% MY_READ_TIFF: Bad value for Number of Bits per Sample'
       return, 0
     endif
   endif
   if n_elements (samples) eq 0 then samples = 1
   if n_elements (xsize) eq 0 or n_elements (ysize) eq 0 then begin
     print, '% MY_READ_TIFF: No values for Xsize or Ysize'
     help, xsize, ysize
     return, 0
   endif
   str = make_array(samples, xsize, ysize, type = type)
   if n_elements (byt_strip) eq 0 then begin
     print, '% MY_READ_TIFF: No value for StripByteCount'
     return, 0
   endif
   if n_elements (rows) eq 0 then begin
     print, '% MY_READ_TIFF: No value for Rows per Strip'
     return, 0
   endif
   stripbuf = make_array(samples* xsize* rows, type= type)
   bstrip = n_elements(stripbuf)

;   if type eq 12 then tt = bstrip*2L else tt= bstrip
;   help, type, tt, bstrip
   tt=bstrip
   if tt ne byt_strip[0] then begin
     print, '% MY_READ_TIFF: Calculate strip size does not agree with file'
     help, tt, byt_strip
;     return, 0
   endif
   nstrip = n_elements (offset)
   if nstrip * bstrip ne n_elements(str) then begin
     print, '% MY_READ_TIFF: Calculate strip size does not agree with file'
     help, nstrip, bstrip, offset
     return, 0
   endif
   for ii = 0, nstrip-1  do begin
     point_lun, unit, offset[ii]
     readu, unit, stripbuf
     str[ii*bstrip] = stripbuf
   endfor
   stripbuf = 0
   str = reform(str)
   return,  str
   end
