; ------------------------------------------------------------------------ 
; $Id: plotcon_solid.pro,v 1.7 2010/02/23 18:35:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro plotcon_solid, str, oData, color= color, back_color = back_color, $
      xrange= xrange, yrange = yrange, zrange = zrange, shininess = shininess, $
      style = style, shading = shading, decimate = decimate, normal = normal
; str is the structure or the array of structures to be plotted.
; range is the actual data range 
; odata is the object dataset model for the planes.
; Normal are the normal planes, Default = [0, 1, 2, 3] ie all planes
;      (Plane 0 are slants)
; 
;

 COMPILE_OPT hidden

   on_error , 1
   oData = 0

   if n_elements (str) eq 0 then begin
     print, '% PLOTCON_SOLID: Structure must be defined '
     return
   endif
   
   ss = str[0]
   b = size(ss)
   if b(b(0)+1) ne 8 then begin
     xss = strvalid(ss, /valid, np = np)
     if np ne 1 then begin
       print, '% PLOTCON_SOLID: Invalid structure', ss
       return
     endif
     s2i, ss, xss[0]
   endif

   if n_elements(xrange) gt 1 then xr= [min(xrange[0:1], max = mm), mm] $
        else xr =0
   if n_elements(yrange) gt 1 then yr= [min(yrange[0:1], max = mm), mm] $
        else yr =0
   if n_elements(zrange) gt 1 then zr= [min(zrange[0:1], max = mm), mm] $
        else zr =0

   if ss.type ne 4 then begin
     print, '% PLOTCON_SOLID: Only type 4 structures may be plotted'
     print, '                 Type =', ss.type
     return
   endif

   if ((n_elements(ss.space) lt 9) and (n_elements(ss.connectivity) lt 5)) $
       then begin
     print, '% PLOTCON_SOLID: Structure does not have plane information.'
     print, '                 To save the plane information please use:'
     print, '                 READSTR, N, DataSet, /solid_model'
     return
   endif
   space_save = ss.space
   ssp = size(space_save)
   conn = ss.connectivity
   if n_elements (normal) gt 0 then begin
     norm = normal[uniq(normal, sort(normal))]
     ok = where (((norm ge 0) and (norm le 3)), nok)
     if nok eq 0 then $
       print, '% PLOTCON_SOLID: No valid normal direction.  All planes taken.' $
     else begin
       cc = conn[ss.loc_con[norm[ok[0]]]: ss.loc_con[norm[ok[0]]+1]-1]
       if nok gt 1 then for i = 1L, nok-1 do $
       cc =[cc, conn[ss.loc_con[norm[ok[i]]]: ss.loc_con[norm[ok[i]]+1]-1]]
       conn = temporary(cc)
     endelse
   endif
   if n_elements (decimate) eq 0 then decimate = 0
   if keyword_set(decimate) then begin
     i =  mesh_decimate(space_save, conn, connout)
     conn= temporary(connout)
   endif
   ncon = n_elements(conn)
   scon = ncon
  
   i = 0L
   inew = 0L

   if n_elements(xr) gt 1 then begin
     while (i lt ncon) do begin
       minx = min(space_save[0, conn[i+1:i+conn[i]]], max = maxx)
       if ((minx ge xr[0]) and  (minx le xr[1])) or $
         ((maxx ge xr[0]) and  (maxx le xr[1])) then begin
         if inew lt i then conn[inew:inew+conn[i]] = conn[i:i+conn[i]]
         inew = inew+conn[i]+1
       endif
       i= i+conn[i]+1
     endwhile
     ncon= inew
     space_save[0, *] = (space_save[0, *] > xr[0]) < xr[1]
     i = 0L
     inew = 0L
   endif
  
   if n_elements(yr) gt 1 then begin
     while (i lt ncon) do begin
       minx = min(space_save[1, conn[i+1:i+conn[i]]], max= maxx)
       if ((minx ge yr[0]) and (minx le yr[1])) or $
        ((maxx ge yr[0]) and (maxx le yr[1])) then begin
         if inew lt i then conn[inew:inew+conn[i]] = conn[i:i+conn[i]]
         inew = inew+conn[i]+1
       endif
       i= i+conn[i]+1
     endwhile
     ncon= inew
     space_save[1, *] = (space_save[1, *] > yr[0]) < yr[1]
     i = 0L
     inew = 0L
   endif

   if n_elements(zr) gt 1 then begin
     while (i lt ncon) do begin
       minx = min(space_save[2, conn[i+1:i+conn[i]]], max= maxx)
       if ((minx ge zr[0]) and (minx le zr[1])) or $
       ((maxx ge zr[0]) and (maxx le zr[1])) then begin
         if inew lt i then conn[inew:inew+conn[i]] = conn[i:i+conn[i]]
         inew = inew+conn[i]+1
       endif
       i= i+conn[i]+1
     endwhile
     ncon= inew
     space_save[2, *] = (space_save[2, *] > zr[0]) < zr[1]
   endif
   if scon gt ncon then conn= temporary(conn[0:ncon-1])
   spar =  ss.spare
  
   if spar[0] eq 2 then begin
      ctheta = reform(cos(space_save[1, *]))
      stheta = reform(sin(space_save[1, *]))
      space_save[1, *] = space_save[0, *]*stheta
      space_save[0, *] = space_save[0, *]*ctheta
      ctheta =0
      stheta =0
   endif else if spar[0] eq 3 then begin
      ctheta = reform(cos(space_save[1, *]))
      stheta = reform(sin(space_save[1, *]))
      cphi = reform(cos(space_save[2, *]))
      sphi = reform(sin(space_save[2, *]))
      r  = reform(space_save[0, *] )
      space_save[2, *] = r * ctheta
      r =temporary(r)*stheta
      space_save[1, *] = r*sphi
      space_save[0, *] = r*cphi
      ctheta =0
      stheta =0
      cphi =0
      sphi =0
      r=0
   endif

   nc = n_elements(color)
   if nc eq 0 then col = getcolor(((spar[1]-1) mod 8)+1) else begin
     if nc ge 3 then col = color[0:2] else col = getcolor(color[0])
   endelse
   nb = n_elements(back_color)
   if nb eq 0 then bcol= 0.6*col else begin
     if nb ge 3 then bcol = back_color[0:2] else bcol = getcolor(back_color[0])
   endelse

   col = byte (col)
   bcol = byte (bcol)
   if n_elements(style) eq 0 then style = 2
   if n_elements(shading) eq 0 then shading = 1
   if n_elements(shininess) eq 0 then shininess = 25

   oData = obj_new('IDLgrPolygon', space_save, polygons= conn, $
      thick = 1.0, name = ss.clab, color = col, bottom = bcol, $
      shininess = shininess, shading = shading, style = style)

   return
   end
