; ------------------------------------------------------------------------ 
; $Id: put_wdfarray.pro,v 1.6 2010/03/15 16:05:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro put_wdfarray, array, id, xydata = xydata, no_copy = no_copyin, $
              clabel = clab, file = file, xlabel = xlab, ylabel = ylab
;+
;
;     this procedure moves an array to the common area indicated by id
;      Format: 
;         put_wdfarray, array, id [, xydata = xydata] [, no_copy = no_copy]
;              [,clabel = clabel] [, file = file] [, xlabel = xlabel] 
;              [, ylabel = ylabel]
;        where:
;          array    - data array of the form:
;                     [t0, dt, y-values] if xydata is omitted or zero
;                     [x-values, y-values] if xydata is not zero
;               Note: If dt is negative, the Y-values will be swapped
;                     and t0 and dt will be modified to reflect the change.
;                     Array will be modified if dt is negative!
;          id       - WDF array number
;          xydata   - Flag to indicate array format.
;                     Default is zero ; format= [t0, dt, y-values]
;                     Not zero indicates format= [x-values, y-values]
;          no_copy  - If set, ARRAY will be undefined on return.
;                     For a million element array, a few tenths of a second
;                     will be saved.
;          clabel, xlabel, ylabel - Comment, x and y labels for the data.
;                     Note: If not specified whatever was there will remain!!
;                           For new data specify:' ' to clear old data.
;          File     - File associated with the data.
;                     Note: If not specified whatever was there will remain!!
;                           For new data specify:' ' to clear old data.
;
;        Example:
;          Read sig data from data1 into wdfarray 3.
;          readsig, 'data1',to, dt, y 
;          put_wdfarray, [to, dt, y],3, /no_copy,f=' ',c=' ', x=' ',y= ' '  
;    or    readsig, 'data1', 3
;-
;     include the common block
 COMPILE_OPT hidden
@com.dat
;
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
;     start
      if n_elements(array) lt 4 or n_elements(id) eq 0 then begin
        print,'Bad parameters to PUT_WDFARRAY'
        return
      endif
      wdf = round(id)
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print,'%PUT_WDFARRAY: Bad WDF array number = ',strtrim (wdf, 2)
        print,'     (WDF array number must be >0 and </=',strtrim(maxwdfarray,2)
        return
      end

      wdf1 = wdf-1
      if keyword_set(xydata) then begin
        npt = n_elements(array)
        if  ((npt mod 2) ne 0) then begin
          print, '% PUT_WDFARRAY: An even number of points must be present to store XY data.'
          print, '                This dataset will not be written'
          return
        endif
        np = npt/2
        if np lt 3 then begin
          print, '% PUT_WDFARRAY: Data must have 3 datapoints'
          print, '                Number of points: '+strtrim(np, 2)
          print, '                This dataset will not be written'
          return
        endif
        wdfnp(wdf1) = np
        wdfmax(wdf1) = float(max(array(np:*), min = mindata, /nan))
        wdfmin(wdf1) = float(mindata)
        wdfminx(wdf1) = min(array(0:np-1), max = mx)
        wdfmaxx(wdf1) = mx
      endif else begin
        np = n_elements(array)-2
        if np lt 3 then begin
          print, '% PUT_WDFARRAY: Data must have 3 datapoints'
          print, '                Number of points: '+strtrim(np, 2)
          print, '                This dataset will not be written'
          return
        endif
        wdfnp(wdf1) = np
        wdfmax(wdf1) = max(array(2:*), min = mindata, /nan)
        wdfmin(wdf1) = mindata
        if array(1) gt 0 then wdfminx(wdf1) = array(0) else begin
          wdfminx(wdf1) = array(0) + (wdfnp(wdf1)-1)*array(1)
          array(0) = wdfminx(wdf1)
          array(1) = -array(1)
          array(2:*) = rotate(temporary(array(2:*)), 2)
        endelse
        wdfmaxx(wdf1) = array(0) + (wdfnp(wdf1)-1)*array(1)
      endelse
      if n_elements(clab) gt 0 then wdfcomment(wdf1) = clab(0)
      if n_elements(xlab) gt 0 then wdfxlabel(wdf1) = xlab(0)
      if n_elements(ylab) gt 0 then wdfylabel(wdf1) = ylab(0)
      if n_elements(file) gt 0 then wdffile(wdf1) = file(0)
      if wdfarraycom(wdf1) eq 0 then wdfarraycom(wdf1) = handle_create()
      no_copy =  keyword_set(no_copyin) ? 1 : 0
      handle_value, wdfarraycom(wdf1), array, /set, no_copy = no_copy
      end
