; ------------------------------------------------------------------------ 
; $Id: read_ascii_columns.pro,v 1.3 2007/06/06 19:34:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro read_ascii_columns, SourceFile, out, npoints = np, ncols=ncols
compile_opt IDL2, obsolete
on_error, 2
        undefine, out
        if n_elements(ncols) eq 0 then begin
          print, '% READ_ASCII_COLUMNS: Number of columns must be specified.'
          return
        endif
        if ncols lt 1 then begin
          print, '% READ_ASCII_COLUMNS: Number of columns must be > 0'
          return
        endif  
        if n_elements(np) eq 0 then np=0
        lc_onetrip = 1
        if np gt 0 then buff_size = np else begin
          buff_size = 4096/ncols > 256
          if (buff_size gt 256) then buff_size = (buff_size/256 +1)*256L
          lc_onetrip =0
        endelse
        buffer = fltarr(ncols, buff_size)
        
; If Buffer was longer than file, reading will fail.
; Reduce buffer if read fails

        FileTooShort = 0b ; Just to skip buffer reduction the first time.

        on_ioError, reduce_buffer

        reduce_buffer : if FileTooShort eq 1b then begin
           unread_rows = (fstat(SourceFile)).Transfer_Count / ncols
           if unread_rows eq 0 then goto, finish
           Buffer=fltarr(ncols, unread_rows)
           point_lun, SourceFile, LastPos
           lc_onetrip =1
        endif
        FileTooShort = 1b ; In case of ioError, this will be set to 1b
        
        while not EOF(SourceFile) do begin
;          Memorize start in case it fails.
           point_lun, (-SourceFile), LastPos 
           readf, SourceFile, Buffer
           if n_elements(out) eq 0 then out = Buffer $
           else out = [[out], [Buffer]]
           if lc_onetrip eq 1 then goto, finish
        endwhile
        
        finish:
        if n_elements (out) gt 0 then begin
          out = transpose(out)
          np = (size(out))[1]
        endif else np =0
end
