; ------------------------------------------------------------------------ 
; $Id: spec_unknown.pro,v 1.3 2007/06/06 19:34:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
PRO Spec_unk_event, Event
@spec_common.dat

  widget_control, event.id, get_uval = uval
  if uval eq 'QUIT' then begin
    lcquit = 1
    widget_control, event.top, /destroy
  endif else if uval eq 'EXIT' then begin
  widget_control, event.top, get_uvalue = values
  widget_control, values.ion_z, get_value = ion_z
  if ion_z le 0 or ion_z gt 92 then begin
    str= [' ', 'Atomic number must be in the range [1, 92]']
    lcquit = 0
  endif else begin
    str= ' '
    lcquit = 1
  endelse
  widget_control, values.ion_a, get_value = ion_a
  if ion_a le 0.then begin
    str= [str, 'Atomic weight must be greater than 0']
    lcquit = 0
  endif 
  widget_control, values.ion_q, get_value = ion_q
  widget_control, values.anode_dist, get_value = anode_dist
  if anode_dist le 0.0 then begin
    str= [str, 'Anode distance must be greater than 0']
    lcquit = 0
  endif 
  widget_control, values.anode_y, get_value = anode_y
  widget_control, values.anode_z, get_value = anode_z
  widget_control, values.gcell_1_dist, get_value = gcell_1_dist
  widget_control, values.gcell_1_gas, get_value = gcell_1_gas
  widget_control, values.gcell_1_th, get_value = gcell_1_th
  widget_control, values.gcell_1_press, get_value = gcell_1_press
  if gcell_1_dist lt 0.0 or gcell_1_dist gt anode_dist then begin
    str= [str, 'Front gas cell must be between anode and scatterer']
    lcquit = 0
  endif 
  if gcell_1_th eq 0 then begin
    if gcell_1_press ne 0.0 then begin
      gcell_1_press = 0.0
      widget_control, values.gcell_1_press, set_value = 0.0
      print, 'Gas Cell #1 pressure set to zero'
    endif
  endif
  widget_control, values.scat_angle, get_value = scat_angle
  if scat_angle gt 90.0 or scat_angle lt -90.0 then begin
    str= [str, 'Scattering foil normal must be in the range (-90, 90)']
    lcquit = 0
  endif
  widget_control, values.scat_th_1, get_value = scat_th_1
  widget_control, values.scat_mat_1, get_value = scat_mat_1
  widget_control, values.scat_th_2, get_value = scat_th_2
  widget_control, values.scat_mat_2, get_value = scat_mat_2
  widget_control, values.scat_th_3, get_value = scat_th_3
  widget_control, values.scat_mat_3, get_value = scat_mat_3
  widget_control, values.gcell_2_dist, get_value = gcell_2_dist
  widget_control, values.gcell_2_gas, get_value = gcell_2_gas
  widget_control, values.gcell_2_th, get_value = gcell_2_th
  widget_control, values.gcell_2_press, get_value = gcell_2_press
  widget_control, values.spect_angle, get_value = spect_angle
  widget_control, values.lc90, get_value = lc90
  widget_control, values.ph_dist_1, get_value = ph_dist_1
  ph_dist_1 = abs(ph_dist_1)
  widget_control, values.ph_thick, get_value = ph_thick
  widget_control, values.ph_mat, get_value = ph_mat
  widget_control, values.magnet_dist, get_value = magnet_dist
  magnet_dist = abs(magnet_dist)
  widget_control, values.magnet_dia, get_value = magnet_dia
  widget_control, values.magnet_b, get_value = magnet_b
  widget_control, values.cr39_radius, get_value = cr39_radius
  if scat_th_2 le 0.0 then begin
    lcquit = 0
    str = [str, 'Scatterer Thickness must be greater than zero']
  endif
  if gcell_2_th eq 0 then begin
    if gcell_2_press ne 0.0 then begin
      gcell_2_press = 0.0
      widget_control, values.gcell_2_press, set_value = 0.0
      print, 'Gas Cell #2 pressure set to zero'
    endif
  endif

  if gcell_2_dist gt ph_dist_1 then print, $
    '******** caution Spectrometer pinhole is in the rear gas cell ********'

  if magnet_dist lt ph_dist_1 then begin
    lcquit = 0
    str = [str, 'Magnet distance must be greater than the pinhole distance']
  endif
  if magnet_dia*0.5 gt cr39_radius then begin
    lcquit = 0
    str = [str, 'Magnet radius must be less than the CR-39 radius']
  endif
  
  if lcquit gt 0 then begin
    lcquit = 0
    widget_control, event.top, /destroy
  endif else winformation, str(1:*)
  endif
END
 
pro angle_event_handler, event
@spec_common.dat
  widget_control, event.id, get_uvalue = uval
  widget_control, event.handler, get_uvalue = value, /no_copy
  case uval of
   'LIST': widget_control, value.edit, set_value = magnet_angle(event.index)
    'ADD': BEGIN
         widget_control, value.edit, get_value= angle
         magnet_angle = [magnet_angle, angle]
         ok = sort(magnet_angle)
         magnet_angle = magnet_angle(ok)
         nang = n_elements(magnet_angle)
         if nang gt 1 then begin
           delta = magnet_angle(1:nang-1)-magnet_angle(0:nang-2)
           ok = where(delta le 0.0, nok)
           if nok gt 0 then begin
             magnet_angle(ok) = -1.0
             ok = where (magnet_angle gt 0.0)
             magnet_angle = magnet_angle(ok)
           endif
         endif
         widget_control, value.magnet_angle, set_value = string(magnet_angle)
         END
   'DEL':  BEGIN
         nang = n_elements(magnet_angle)
         if nang eq 1 then begin
           winformation, 'Cannot delete last magnet value'
         endif else begin
           widget_control, value.edit, get_value= angle
           minv = min(abs(angle-magnet_angle), i)
           magnet_angle(i) = -1
           ok = where (magnet_angle gt 0.0)
           magnet_angle = magnet_angle(ok)
           widget_control, value.magnet_angle, set_value = string(magnet_angle)
         endelse
         END
    ELSE: print, 'Unknown Value: ', uval
  endcase
  widget_control, event.handler, set_uvalue = value, /no_copy
end

PRO spec_unknown, GROUP=Group


@spec_common.dat
  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  superbase = WIDGET_BASE(GROUP_LEADER=Group, $
      Title = 'Spectrometer Transport Parameters', $
      row=1, MAP=1, UVALUE='MAIN13')
  mainbase = widget_base(superbase, /column, /map)
  helpbase = widget_base(superbase, /column, /map )
  materials = widget_base(helpbase, /column, /map, frame = 2)
  nm = n_elements(matname)
  name = matname
  for i = 0, nm-1 do name(i)=string(i+1, format='(i4)') +' = '+matname(i)
  thelper = widget_label(materials, value ='Material/Ion Numbers', /align_center)
  helper = widget_list(materials, value = name, xsize = 12, ysize = 18)
  anglebase = widget_base(helpbase, /column, /map, frame = 2, $
          event_pro='angle_event_handler')
  tmagang = widget_label(anglebase, value ='Magnet Exit Angles', /align_center)
  if n_elements(magnet_angle) eq 0 then magnet_angle = 30+findgen(20)
  fmagnet_angle =  widget_list(anglebase, value = string(magnet_angle), $
          xsize = 12, ysize=16, uval = 'LIST')
  fedit = CW_FIELD(anglebase ,VALUE=0 , ROW=1, $
      FLOAT=1, TITLE='Ion Exit Angle ', XSIZE=8)
  buttonbase = widget_base (anglebase, /row)
  addbut = widget_button(value = '    ADD Value   ', buttonbase, uval='ADD')
  delbut = widget_button(value = '  Delete Value  ', buttonbase, uval='DEL')
  
  angle_value = {magnet_angle:fmagnet_angle, edit:fedit}
  ions = WIDGET_BASE(mainbase, column=1, FRAME=2, MAP=1)
  
  iontext = widget_label(ions, /align_center, value = 'Ion Parameters')

  ionbase = WIDGET_BASE(ions, ROW=1, MAP=1)

  if n_elements(ion_z) eq 0 then ion_z = 3

  Fion_z = CW_FIELD( ionbase,VALUE= ion_z, ROW=1, $
      INTEGER=1, TITLE='Ion Atomic Number', XSIZE=3)

  if n_elements(ion_a) eq 0 then ion_a = 7

  Fion_a = CW_FIELD( ionbase,VALUE= ion_a, ROW=1, $
      FLOAT=1, TITLE='Ion Atomic Weight', XSIZE=8)

  if n_elements(ion_q) eq 0 then ion_q = 1

  Fion_q = CW_FIELD( ionbase,VALUE= ion_q, ROW=1, $
      INTEGER=1, TITLE='Ion Charge', XSIZE=3)

  anode = WIDGET_BASE(mainbase, column=1, FRAME=2, MAP=1)

  anodetext = widget_label(anode, /align_center, $
        value = 'Anode Emission Parameters')

  anodebase = WIDGET_BASE(anode, ROW=1, MAP=1)

  if n_elements(anode_dist) eq 0 then anode_dist = 3.0 

  Fanode_dist = CW_FIELD( anodebase,VALUE= anode_dist, ROW=1, $
      FLOAT=1, XSIZE=8, TITLE = 'Anode Distance (cm)')

  if n_elements(anode_y) eq 0 then anode_y = 0.0

  Fanode_y = CW_FIELD( anodebase,VALUE= anode_y, ROW=1, $
      FLOAT=1, XSIZE=8,  TITLE='Y (cm)')

  if n_elements(anode_z) eq 0 then anode_z = 0.0

  Fanode_z = CW_FIELD( anodebase, Value = anode_z, $
      ROW=1, FLOAT=1, TITLE='Z (cm)', XSIZE=8)

  gascella = WIDGET_BASE(mainbase, column=1, FRAME=2, MAP=1)

  gcellatext = widget_label(gascella, /align_center, $
        value = 'Anode Side Gas Cell Parameters')

  gcellabase = WIDGET_BASE(gascella, ROW=2, MAP=1)

  if n_elements(gcell_1_dist) eq 0 then gcell_1_dist = 12.54

  Fgcell_1_dist = CW_FIELD( gcellabase,VALUE=gcell_1_dist, ROW= 1, $
      FLOAT=1, TITLE='Front Gas Cell Distance (cm)', XSIZE=8)

  if n_elements(gcell_1_gas) eq 0 then gcell_1_gas = 18

  Fgcell_1_gas = CW_FIELD( gcellabase,VALUE= gcell_1_gas, ROW=1, $
      INTEGER=1, TITLE='Fill Gas Atomic Number', XSIZE=3)

  if n_elements(gcell_1_th) eq 0 then gcell_1_th =2.0

  Fgcell_1_th = CW_FIELD( gcellabase,VALUE= gcell_1_th, ROW=1, $
      FLOAT=1, TITLE='Front Gas Cell Thickness (Micron)', XSIZE=8)

  if n_elements(gcell_1_press) eq  0 then gcell_1_press = 0.000

  Fgcell_1_press = CW_FIELD( gcellabase,VALUE=gcell_1_press, ROW=1, $
      FLOAT=1, TITLE='Gas Fill Pressure (torr)', XSIZE=8)


  scat= WIDGET_BASE(mainbase, COLUMN=1, FRAME=2, MAP=1)

  scattext =  widget_label(scat, /align_center, $
        value = 'Scattering Foil Parameters')

  if n_elements(scat_angle) eq 0 then scat_angle = 0.0
  scatbase= WIDGET_BASE(scat, row=4,  MAP=1)

; Fscat_angle = CW_FIELD( scat,VALUE=scat_angle, ROW=1, FLOAT=1, $
  Fscat_angle = CW_FIELD( scatbase,VALUE=scat_angle, ROW=1, FLOAT=1, $
      TITLE='Scatterer Normal Angle (deg) (up = +)', XSIZE=10)
  if n_elements (lc90) eq 0 then lc90 = 0

  flc90 = cw_bgroup( scatbase, ['Yes', 'No'], /row, /frame, /exclusive, $
    /NO_RELEASE, set_value = lc90, LABEL_LEFT='Extraction Diode', uval = 'yesno')

; scatbase= WIDGET_BASE(scat, row=3,  MAP=1)

  if n_elements(scat_th_1) eq 0 then scat_th_1 = 0.0

  Fscat_th_1 = CW_FIELD( scatbase,VALUE=scat_th_1, ROW=1, FLOAT = 1, $
      TITLE='Scatterer Front Thickness (micron)', XSIZE=8)

  if n_elements(scat_mat_1) eq 0 then scat_mat_1 = 102

  Fscat_mat_1 = CW_FIELD( scatbase,VALUE=scat_mat_1, ROW=1, $
      INTEGER=1, TITLE='Scatterer Front Material (At.#)', XSIZE=3)

  if n_elements(scat_th_2) eq 0 then scat_th_2 = 0.5

  Fscat_th_2 = CW_FIELD( scatbase,VALUE=scat_th_2, ROW=1, FLOAT=1, $
      TITLE='Scatterer Thichness (micron)', XSIZE=8)

  if n_elements(scat_mat_2) eq 0 then scat_mat_2 = 73

  Fscat_mat_2 = CW_FIELD( scatbase,VALUE=scat_mat_2, ROW=1, INTEGER=1, $
      TITLE='Scatterer Material (At.#)', XSIZE=3)

  if n_elements(scat_th_3) eq 0 then scat_th_3 = 0.0

  Fscat_th_3 = CW_FIELD( scatbase,VALUE= scat_th_3, ROW=1, FLOAT=1, $
      TITLE='Scatterer back thickness (micron)', XSIZE=8)

  if n_elements(scat_mat_3) eq 0 then scat_mat_3 = 102

  Fscat_mat_3 = CW_FIELD( scatbase,VALUE=scat_mat_3, ROW=1, INTEGER=1, $
      TITLE='Scatterer Back Material (At.#)', XSIZE=3)

  gascellb = WIDGET_BASE(mainbase, column=1, FRAME=2, MAP=1)

  gcellbtext = widget_label(gascellb, /align_center, $
        value = 'Exit Side Gas Cell Parameters')

  gcellbbase = WIDGET_BASE(gascellb, ROW=2, MAP=1)

  if n_elements(gcell_2_dist) eq 0 then gcell_2_dist = 12.54

  Fgcell_2_dist = CW_FIELD( gcellbbase,VALUE=gcell_2_dist, ROW= 1, $
      FLOAT=1, TITLE='Rear Gas Cell Distance (cm)', XSIZE=8)

  if n_elements(gcell_2_gas) eq 0 then gcell_2_gas = 18

  Fgcell_2_gas = CW_FIELD( gcellbbase,VALUE= gcell_2_gas, ROW=1, $
      INTEGER=1, TITLE='Fill Gas Atomic Number', XSIZE=3)

  if n_elements(gcell_2_th) eq 0 then gcell_2_th =2.0

  Fgcell_2_th = CW_FIELD( gcellbbase,VALUE= gcell_2_th, ROW=1, $
      FLOAT=1, TITLE='Rear Gas Cell Thickness (Micron)', XSIZE=8)

  if n_elements(gcell_2_press) eq  0 then gcell_2_press = 0.000

  Fgcell_2_press = CW_FIELD( gcellbbase,VALUE=gcell_2_press, ROW=1, $
      FLOAT=1, TITLE='Gas Fill Pressure (torr)', XSIZE=8)

  spec = WIDGET_BASE(mainbase, column=1, FRAME=2, MAP=1)

  spectext = widget_label(spec, /align_center, $
        value = 'Magnetic Spectrometer Parameters')

  specbase = WIDGET_BASE(spec, ROW=4, MAP=1)

  if n_elements(spect_angle) eq 0 then spect_angle = 25.0

  Fspect_angle = CW_FIELD( specbase,VALUE=spect_angle, ROW= 1, $
      FLOAT=1, TITLE='Spectrometer Angle (up=+)(deg)', XSIZE=8)

  if n_elements(ph_dist_1) eq 0 then ph_dist_1 = 18.

  Fph_dist_1 = CW_FIELD( specbase,VALUE= ph_dist_1, ROW=1, $
      FLOAT=1, TITLE='Spect. Ent. Distance (cm)', XSIZE=8)

  if n_elements(ph_thick) eq 0 then ph_thick = 0.00

  Fph_thick = CW_FIELD( specbase,VALUE= ph_thick, ROW=1, $
      FLOAT=1, TITLE='Spect. Entrance Filter (Micron)', XSIZE=8)
  if n_elements(ph_mat)eq  0 then ph_mat = 13

  Fph_mat = CW_FIELD( specbase,VALUE=ph_mat, ROW=1, $
      INTEGER=1, TITLE='Filter Material (At.#)', XSIZE=3)

  if n_elements(magnet_dist)eq  0 then magnet_dist = 30. 

  Fmagnet_dist = CW_FIELD( specbase,VALUE=magnet_dist, ROW=1, $
      FLOAT=1, TITLE='Magnet Distance (cm)', XSIZE=8)

  if n_elements(magnet_dia)eq  0 then magnet_dia = 30. 

  Fmagnet_dia = CW_FIELD( specbase,VALUE=magnet_dia, ROW=1, $
      FLOAT=1, TITLE='Magnet Diameter (cm)', XSIZE=8)

  if n_elements(magnet_B)eq  0 then magnet_B = 7140.

  Fmagnet_B = CW_FIELD( specbase,VALUE=magnet_b, ROW=1, $
      FLOAT=1, TITLE='Magnet Field (gauss)', XSIZE=8)

  if n_elements(cr39_radius)eq  0 then cr39_radius = 16.

  Fcr39_radius = CW_FIELD( specbase,VALUE=cr39_radius, ROW=1, $
      FLOAT=1, TITLE='CR39 Radius (cm)', XSIZE=8)
values = {ion_z:fion_z, ion_a:fion_a, ion_q:fion_q, anode_dist:fanode_dist, $
          anode_y:fanode_y, anode_z:fanode_z, gcell_1_dist:Fgcell_1_dist, $
          gcell_1_gas:fgcell_1_gas, gcell_1_th:Fgcell_1_th, $
          gcell_1_press:fgcell_1_press, scat_angle:fscat_angle, $
          scat_th_1:Fscat_th_1, scat_mat_1:fscat_mat_1, scat_th_2:fscat_th_2, $
          scat_mat_2:fscat_mat_2, scat_th_3:fscat_th_3, scat_mat_3:fscat_mat_3, $
          lc90:flc90, gcell_2_dist:fgcell_2_dist, gcell_2_gas:fgcell_2_gas, $
          gcell_2_th:fgcell_2_th, gcell_2_press:fgcell_2_press, $
          spect_angle:fspect_angle, ph_dist_1:fph_dist_1, ph_thick:fph_thick, $
          ph_mat:fph_mat, magnet_dist:fmagnet_dist, magnet_dia:fmagnet_dia, $
          magnet_b:fmagnet_b, cr39_radius:fcr39_radius, $
          magnet_angle:fmagnet_angle}

  butbas = widget_base(mainbase, row = 1)
  butq = widget_button(value = '  QUIT without Calculation  ', butbas, $
      uval = 'QUIT')

  bute = widget_button(value = '  EXIT to do Calculation  ', butbas, $
      uval = 'EXIT')

  WIDGET_CONTROL, superbase, /REALIZE
  widget_control, superbase, set_uvalue = values, /no_copy
  widget_control, anglebase, set_uvalue = angle_value, /no_copy

  XMANAGER, 'spec_unk', superbase

END
