; ------------------------------------------------------------------------ 
; $Id: z_gridinteg.pro,v 1.4 2007/06/06 19:34:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function z_gridinteg, array, x, flag
;
;+
;
;        integrate an array in the x direction an return the integral
;        assume that the array is a grid array with values of the function
;        at the end points 
;          Integral is defined according to the following:
;          integ(0) = init
;          integ(i) = integ(i-1) + (A(i)+A(i-1))(x(i)-x(i-1))/2.0
;            where A(i) is the value of the function at x(i)
;        format: integ = z_gridinteg(array, x, init, flag)
;        where;
;          Array is the three dimensional array to be integrated
;          x     is the array of z values for array
;          flag  is an integration - default is 0
;               integ = integral(array * dx)             flag = 0 
;               integ = integral(array * x dx)           flag = 1 
;               integ = integral(array * x*x dx)         flag = 2 
;-
 COMPILE_OPT hidden
 on_error, 2

         if n_elements (array) eq 0 or n_elements (x) eq 0 then begin
           print, ' Z_GRIDinteg - Array and abscissa must be defined'
           return, 0.0
         endif
         a = size (array)
         if a(0) ne 3 or a(3) ne n_elements (x) then begin
           Print, ' Z_GRIDinteg ERROR Array must be three dimensional with same'
           print, ' number of elements in third dimension as x'
           return, 0.0
         endif
         nx = a(3)
;   calculate dx/2
         dx2 = 0.5*(x(1:nx-1)-x(0:nx-2))
;    calculate the average value of the function between x values
         aarray= array(*, *, 0:nx-2) + array(*,*,1:nx-1)
;    check on initial value
;        if n_elements (init) ne a(2) then array(0,*)=0 else array(0,*)=init
         array(*,*,0) = 0.0*array(*,*,0)
         if n_elements (flag) eq 0 then flag = 0
         if flag gt 2 or flag lt 0 then begin
           PRINT, ' INTEG flag for Z_GRIDinteg must be in the range 0, 2'
           print, ' Flag set to zero'
           flag = 0
         endif
         if flag eq 0 then begin
           for ix = 0, nx-2  do $
               array(*,*,ix+1)=array (*,*,ix)+aarray(*,*,ix)*dx2(ix)
         endif else begin
            tempx = x(0:nx-2)+dx2
            tempx = tempx^flag
            for ix=0,nx-2 do $
              array(*,*,ix+1)=array(*,*,ix)+aarray(*,*,ix)*dx2(ix)*tempx(ix)
         endelse
         return, array
         end
