; ------------------------------------------------------------------------ 
; $Id: init_xrd.pro,v 1.4 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
 pro init_xrd, headerin

on_error, 1
@xrd_common_data.dat
common pffcom, initpff, pff_exe
;     check to see if pff has been initialized
;
  if n_elements(initxrd) eq 0 then initxrd = 0L

  if (n_elements(initpff) eq 0) then init_pff else $
    if (initpff eq 0) then init_pff

  if initpff eq 0 then begin
    print, '% XRD_INIT: PFF libraries are not supported by this System'
    print, '% XRD_INIT: XRD analysis not supported by this operating system'
    initxrd=-1
    return
  endif

  showpff, filename = fname, ufid = fid, numb = nfile, /quiet
  if n_elements(headerin) eq 0 then header='' else header = strtrim(headerin, 2)
  z_data_file = '/pff_data/pbfa2z/'
  valid_ufo_computer =['sonic', 'luigi']

  hfid=0
  hndset = 0
  if strlen(header) gt 0 and nfile gt 0 then begin
    ok = where (strpos (fname, header) ge 0, nok)
    if nok eq 1 then begin 
      header = fname[ok[0]]
      hfid = fid[ok[0]]
      error =0
      hndset = ndspff(hfid)
    endif else nok = 0
  endif else nok = 0

  pdir = get_pfidl_directory()
  if initpff lt 3 then head_path = pdir+'../xrd/' $
    else  head_path = pdir+'\..\xrd\'
  sep = path_sep()
  if strlen(header) eq 0 then header = head_path+'shot_config.pff'
  cfid=0
  ffid=0
  dfid=0
  
  if hfid eq 0 then begin
    openpff, header, fileid = hfid, ndset = hndset, error = error 
    if error ne 0 then begin
      print, '% XRD_INIT: Unable to open header file'
      print, '      file: ', header
      print, "  Enter: XRD_INIT, 'header_file' to continue"
      return
    endif
  endif
  header_file = header

  pos = strpos(header, sep, /reverse_search)+1
  if pos gt 0 then datapath = strmid(header, 0, pos)

  cathode_file = 'cathode_resp.pff'
  ok = where(strpos (fname,cathode_file) ge 0, nok)
  if nok eq 1 then begin
    cfid =  fid[ok[0]]
    error = 0 
    cndset = ndspff(cfid)
  endif else begin
    info = file_info(cathode_file)
    if (not info.exists) then begin
      if pos gt 0 then info = file_info(datapath+cathode_file)
      if (not info.exists) then begin
        info = file_info(head_path+cathode_file)
        if (not info.exists) then begin
          print, '% XRD_INIT: Unable to find cathode response file'
          print, '      files: ', cathode_file
          if pos gt 0 then print, '           : ', datapath +cathode_file
          print, '           : ', head_path+cathode_file
          return
        endif
        cathode_file = head_path+cathode_file
      endif else cathode_file = datapath+cathode_file
    endif
    openpff, cathode_file, fileid = cfid, ndset = cndset, error = error
    if error ne 0 then begin
      print, '% XRD_INIT: Unable to open cathode response file'
      print, '      file: ', cathode_file
      return
    endif
  endelse


  filter_file = 'filter_resp.pff'
  ok = where(strpos (fname,filter_file) ge 0, nok)
  if nok eq 1 then begin
    ffid =  fid[ok[0]]
    error = 0 
    fndset = ndspff(ffid)
  endif else begin
    info = file_info(filter_file)
    if (not info.exists) then begin
      if pos gt 0 then info = file_info(datapath+filter_file)
      if (not info.exists) then begin
        info = file_info(head_path+filter_file)
        if (not info.exists) then begin
          print, '% XRD_INIT: Unable to find filter response file'
          print, '      files: ', filter_file
          if pos gt 0 then print, '           : ', datapath +filter_file
          print, '           : ', head_path+filter_file
          return
        endif
        filter_file = head_path+filter_file
      endif else filter_file = datapath+filter_file
    endif
    openpff, filter_file, fileid = ffid, ndset = fndset, error = error
    if error ne 0 then begin
      print, '% XRD_INIT: Unable to open cathode response file'
      print, '      file: ', filter_file
      return
    endif
  endelse

  detector_file = 'detector_resp.pff'

; help, cfid
;  openpff, detector_file, fileid = dfid, ndset = dndset, error = error
;  if error ne 0 then begin
;    print, '% XRD_INIT: WARNING: Unable to open detector response file'
;    print, '      file: ', detector_file
;    print, '  Detector_resp.pff must be in the same directory as Shot_Config'
;    print, '  Unfold can continue.'
;  endif


  
  initxrd = initpff
  hcomments  = make_array(hndset, /string)
  hcomm = hcomments
  setpff, hfid
  for i =0, hndset-1 do begin
    readhdr, (i+1), clab = c, /nostrip, /quiet
    hcomments[i] =c
    l = strpos(c, ' - ;;')
    if l gt 0 then hcomm[i]= strmid(c, 0, l) else  hcomm[i] = c
  endfor
end

; thindate = systime(0)
; thindate = strmid(thindate, 4, 12)+ strmid(thindate, 19, 5)
; common xrd_common_data, initxrd, hndset, hfid, cndset, cfid, fndset, ffid, $
;          dndset, dfid
;
; z_data_file - directory for zdata
; initxrd - 1 for hp, 2 for sun and 4 for Windows
; hndset  - number of datasets in the shot configuration file
; hfid    - file id for the shot configuration file
; cndset  - number of datasets in the cathode response file
; cfid    - file id for the cathode response file
; fndset  - number of datasets in the filter response file
; ffid    - file id for the filter response file
; dndset  - number of datasets in the total detector response file
; dfid    - file id for the total detector response file
; hcomm   - comments for the headers truncated to 64 characters
; hcomments - full length comments
;

