; ------------------------------------------------------------------------ 
; $Id: setxrd.pro,v 1.3 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 

; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  setxrd, fileid, Cathode = cathode, filter = filter, detector = detector, $
       header = header
;+
; NAME:
;       SETXRD
;
; PURPOSE:
;       Set the default XRD analysis file to a the file id of an open PFF file
;
; CALLING SEQUENCE:
;       SETXRD,  FILEID
;
; INPUTS:
;       FILEID   - User file id for a current PFF file
;                  If zero, the current PFF file is used.
;
; KEYWORD PARAMETERS:
;       
;       Cathode  - Indicates FILEID is to be used for cathode response
;       Filter   - Indicates FILEID is to be used for filter response
;       Detector - Indicates FILEID is to be used for detector response
;       Header   - Indicates FILEID is to be used for shot configuration header
;
;    Example:
;    Set the header configuration the file with File ID =3 
;      setxrd, 3, /header
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, February 12, 2001
;-
;
on_error, 1
;
@xrd_common_data.dat
common pffcom, initpff, pff_exe

      if n_params() eq 0 then begin
        print, '% Format: SETXRD,  fileid [/header] [, /cathode] [, /filter]'
        print, '                    [, /detector]'
        return
      endif
;     check to see if xrd has been initialized

      if n_elements(fileid) eq 0 then begin
        print, '% SETXRD: User file ID must be set.'
        return
      endif 
 
      showpff, ufid = ufid, curr= curr, filename= filename, $
                number = number, /quiet

      if number eq 0 then begin
        print, '% SETXRD: No pff files are opened. No files can be set.'
        return
      endif
      if fileid [0] eq 0 then fileid = current
;
      if (n_elements(initxrd) eq 0) then initxrd=0
      if initxrd eq 0 then begin
        hfid=0
        cfid=0
        ffid=0
        dfid=0
        hndset = 0
      endif

      ok = where (fileid[0] eq  ufid, nok)
      if nok eq 0 then begin
        print, '% SETXRD: '+strtrim (fileid[0], 2)+ ' is not a valid file ID>'
        return
      endif 

      if keyword_set(header) then begin
        hfid = ufid[ok[0]]
        header_file = filename[ok[0]]
        if initxrd eq 0 then initxrd = initpff
        hndxet = ndspff(hfid)
        hcomments = make_array(hndset, /string)
        hcomm = hcomments
        if initxrd eq 0 then initxrd = initpff
        setpff, hfid
        for i =0, hndset-1 do begin
          readhdr, (i+1), clab = c, /nostrip, /quiet
          hcomments[i] =c
          l = strpos(c, ' - ;;')
          if l gt 0 then hcomm[i]= strmid(c, 0, l) else  hcomm[i] = c
        endfor
        return
      endif

      if keyword_set(cathode) then begin
        cfid = ufid[ok[0]]
        cndset = ndspff(cfid)
        cathode_file = filename[ok[0]]
        if initxrd eq 0 then initxrd = initpff
        return
      end

      if keyword_set(filter) then begin
        ffid = ufid[ok[0]]
        fndset = ndspff(cfid)
        filter_file = filename[ok[0]]
        if initxrd eq 0 then initxrd = initpff
        return
      end

      if keyword_set(detector) then begin
        dfid = ufid[ok[0]]
        dndset = ndspff(cfid)
        detector_file = filename[ok[0]]
        if initxrd eq 0 then initxrd = initpff
        return
      end
      print, '% SETXRD: One keyword must be set'
      print, '% Format: SETXRD,  fileid [/header] [, /cathode] [, /filter]'
      print, '                    [, /detector]'

      end
