pro make_diode_sensitivity, beginning, ending, delta, wdf = wdf, $
            reduce = reduce, loginter= loginter

;
; make diode sensitivity ev vs energy calculations using technique of 
;   Larry Ruggles.
;
; Parameters:
;
;  SiO2 density = 2.2 g/cc	
rho_sio2 = 2.2
;  silicon density = 2.33 g/cc	
rho_si = 2.33
;  theoretical sensitivity = 0.272 amps/watt
sensi = 0.272
;  diode dead layer = 80 angstroms	
x_dead = 80
;  active region thickness = 12.535 microns	
x_pin = 12.535
;  actual active area =	228.6 microns square	
act_area = 228.6
;  assumed active area = 228.6 microns square	
assumed_area = 228.6
;
;
if n_elements(delta) eq 0 then delta = 1. 
if size(delta, /type) lt 4 then delta = float(delta)
if n_elements(beginning) eq 0 then start = 50 else start = beginning
if n_elements(ending) eq 0 then eend = 5000 else eend = ending
if n_elements(wdf) eq 0 then wdf = get_maxwdf() 
if keyword_set(reduce) then begin
if n_elements(loginter) eq 0 then loginter=1
  if reduce gt 0.99 then reduce = 0.001
endif
; make energies
nener = round((eend-start)/delta+1.0)
ener = start +findgen(nener)*delta

pdir = get_pfidl_directory()
ddir = pdir+'pfidl/xrd/data/filter_material.pff'
re, wdf, 'silica'
; Limit data to the desired range (it goes to 10 MeV)
; Add a little in case no data at the endpoint
lim, wdf, 0, 1.2*ener[nener-1]
if keyword_set(loginter) then begin
  puty, alog(getyf(wdf)), wdf
  asio2 = exp(get_value(wdf, ener))
endif else asio2 = get_value(wdf, ener)

;
re, wdf, 'silicon(si)'
; Limit data to the desired range (it goes to 10 MeV)
;
lim, wdf, 0, 1.2*ener[nener-1]
if keyword_set(loginter) then begin
  puty, alog(getyf(wdf)), wdf
  asi = exp(get_value(wdf, ener))
endif else asi = get_value(wdf, ener)

; SiO2 transmission
arg = (-(rho_sio2*1e-8*x_dead)*asio2) > (-80)
tran = exp (arg)

; Si absorption
arg = (-(rho_si*x_pin*.0001)*asi) > (-80)

Absorption = 1.0d0-exp(arg)

R= act_area/assumed_area
sensitivity = sensi*r*r*tran*absorption 

if keyword_set(reduce) then begin
  i = 0
  j = 1
  while j lt nener do begin
    if abs(sensitivity[j]-sensitivity[i])/sensitivity[i] gt reduce $
      or j eq nener-1 then begin
; take the last point
      i = i+1
      sensitivity[i] = sensitivity[j]
      ener[i]= ener[j]
    endif
    j = j+1
  endwhile
endif else i = nener-1
help, i, j, nener
i2w,ener[0:i], sensitivity[0:i], wdf, /xydata, $
   clab='Detector Sensitivity', xlab= 'Energy(eV)', $
   Ylab='Sensitivity(A/W)'
end
