function read_grating_eff,name, unique = unique, norder = norder, verbose = verbose, $
         ierror = ierror
;
; NAME:     Name of the grating efficiency file
; UNIQUE:   If set multiple values of energy are eliminated.  Only the first is retained.
;           !always set!!
;
on_error, 2
  ierror = 1
  if n_elements(name) eq 0 then begin
    print, '% READ_GRATING_EFF: File name must be provided'
    return, 0
  endif
  openr, unit, /get, name, error = error
  if error ne 0 then begin
    print, '% READ_GRATING_EFF: Error opening file'
    print, !error_state.msg
    ierror = error
    return, 0
  endif 
  ss = ''
  readf, unit, ss
  if keyword_set (verbose) then print, ss
  readf, unit, ss
  if keyword_set (verbose) then print, ss
  readf, unit, ss
  if keyword_set (verbose) then print, ss
  s = ss
  readf, unit, ss
  if keyword_set (verbose) then print, ss
  if strlen(ss) gt 0 then begin
    s = s+ss
;   Read the last blank line unless it was already read
    readf, unit, ss
    if keyword_set (verbose) then print, ss
  endif
  sb = byte(s)
  ok = where (sb eq 37B, norder)
  if norder eq 0 then begin 
    Print, '% READ_GRATING_EFF: No orders found'
    free_lun, unit
    return, 0
  endif
  if norder ne 9 or keyword_set(verbose) then $
     print, 'Number of orders found is: ', strtrim(norder, 2)
  ok = where (sb eq 32, nok)
  if nok lt 5 then begin
    nok = strtrim(nok, 2)
    Print, '% READ_GRATING_EFF: Only ', nok, ' spaces found in the header'
    free_lun, unit
    return, 0
  endif
  diff = ok[1: nok-1] - ok[0: nok-2]
  d0 = diff[0:nok-3]
  d1 = diff[1:nok-2]
; get the number of spaces with more than 2 spaces
  ok = where ((d0-d1 eq 0) and d0 eq 1, nok_double)
; get the number of double spaces
  ok = where(diff eq 1, n1)
; the number of columns is one more than the net number of double spaces
  ncol = 1+n1-nok_double
  mxarr = 1024
  arr = make_array(ncol, mxarr)
  ix = 0L
  if ncol ne 12 then print, '% Warning: Different file format for grating efficiency'
  x = make_array(ncol) 
  readf, unit, x
  arr[0,0]= x
  while not (eof(unit)) do begin
    on_ioerror, baddata
    readf, unit, x
    if x[0] gt arr[0, ix] then begin
      ix +=1
      if ix eq mxarr then begin
        arr = [reform (arr, ncol*mxarr, /overwrite), make_array(128*ncol)]
        mxarr = mxarr +128
        arr= reform(arr, ncol, mxarr, /overwrite)
      endif
      arr[0, ix]= x
    endif
  endwhile
baddata:
  free_lun, unit
  if ix eq 0 then return, 0
  if ix lt 10 then print, $
    '% READ_GRATING_EFF:  Only '+strtrim(ix,2)+ ' energy values in the file'
  ierror = 0  
  return, arr[*, 0:ix]
  end
  
