;Original author, G.A. Rochau, spring 2010
;Modified by R.E. Falcon, summer 2012

;***************************************************************
; Analyze the step wedge and convert the data to exposure
;***************************************************************

;fit the step wedge file
ALT_WEDGE,PATH=1

;apply the step wedge to the data
REGRID

;***************************************************************
; Read in the data and correct for rotation / flipping
;***************************************************************

;Specify shot and instrument
Shot='z2409';(e.g.)
SVS='SVS1';(e.g.)

;open the pff containing the exposure corrected data
OPENPFF;*_unp.pff
READSTR,1
CLOSEPFF

;guess level range and adjust as necessary
LevRange=[0.,0.3]
PLOTSTR,1,LEV=LevRange,/ASPECT

;straighten the data using a laser pulse
ALIGNSTR,1,2,LEV=LevRange

;***************************************************************
; Convert the data to time
;***************************************************************

;take a lineout over the comb and save in wdf 201
LINSTR,2,LEV=LevRange,/VERTICAL

;analyze the comb spacing by taking the FFT
PLO,201
XCUR,xpp  ;select the boundaries of the comb
LIM,201,xpp(0),xpp(1)

APPLYCOMB,2,3,201,COMB=5.,INT=10,SWEEP=430.

;take a lineout over the Impulse time and save in wdf 202
LINSTR,3,LEV=LevRange,/VERTICAL

;Determine the impulse time
ZOOMW,202,XC=xp

;apply the impulse time correction
ImpTime=1000;get VISSPECIMP value for specific shot
CHASTR,3,AYD=ImpTime-xp(0)

;***************************************************************
; Convert the data to wavelength
;***************************************************************

;Method A: Image Approach
;========================

;take a lineout over the HeNe pulses and save in wdf 203
LINSTR,3,LEV=LevRange,/HORIZONTAL

;analyze the wavelength spacing
ZOOMW,203,XC=xp
;wdisp=(6328.2-5433.7)/(xp(1)-xp(0))
;wdisp=(5433.7-4861.323)/(xp(1)-xp(0))

;5435.161+/-0.06AA (Amir Marashi, CVI Melles Griot, private communication)
;4579.0  +/-0.5 AA (implied assumption, should be conservative overestimate)
WDisp=(5435.161-4579.)/(xp(1)-xp(0));using green/blue lasers

;apply the wavelength dispersion
XFRSTR,3,4
CHASTR,4,MXD=WDisp
;chastr,4,axd=5433.7-xp(0)*wdisp,xlab='Wavelength (Angstroms)'
;chastr,4,axd=4861.323-xp(0)*wdisp,xlab='Wavelength (Angstroms)'
CHASTR,4,AXD=4579.-xp(0)*WDisp,XLAB='Wavelength (Angstroms)';using green/blue

;***************************************************************
; Save the image now in units of energy vs. time and wavelength (kinda)
;***************************************************************

FileName=Shot+SVS+'_cor.pff'
CREATEPFF,FileName
WRISTR,4
CLOSEPFF


;Method B: Lineout Approach
;==========================

;Save the time-corrected image
FileName=Shot+SVS+'_tcor.pff'
CREATEPFF,FileName
WRISTR,3
CLOSEPFF

;Determine Guess values
LevRange=[0.,0.3]
LINSTR,3,LEV=LevRange,/HORIZONTAL

;Arguments for LineByLine.pro
StartTime  = 950. ; ns
EndTime    = 1295. ; ns
TimeRange  = [StartTime,EndTime]
TimeWidth  = 5. ; ns (no point in using anything <2ns)
GuessBlue  = 2.360e4 ; x position (microns)
GuessGreen = 3.665e4 ; x position (microns)
Guess      = [GuessBlue,GuessGreen]
GuessWidth = [0.5e3,0.5e3] ; x position (microns)
W_Lasers   = [4579.D,5435.161D] ; Angstroms
W_Lasers_E = [0.5D,0.06D] ; Angstroms

;Extract wavelength-corrected lineouts
;Also get back Wave, Time, WaveE
SPowArr=LineByLine(3, TimeRange, TimeWidth, Guess, W_Lasers, W_Lasers_E, $
                  Wave, Time, WAVEE=WaveE)

;Extract instrumental profile from laser fiducials
Result=EXTRACT_INST(Wave, SPowArr, W_Lasers, Time, FWHM_0, FWHM_1)

;Rename variables
FWHM_Blue  = FWHM_0;Guassian width, uncertainty, Lorentzian width, uncertainty, Voigt width
FWHM_Green = FWHM_1

;Save variables to do work later
SAVE, Wave, WaveE, SPowArr, Time, FWHM_Blue, FWHM_Green, $
      FILENAME='work\'+Shot+SVS+'.var'

