; ------------------------------------------------------------------------ 
; $Id: bbox.pro,v 1.2 2005/03/09 22:20:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; Name: BBOX
;       Calculates the bounding box for all the current ACIS objects.
;
; format: bbox, xmin, xmax, ymin, ymax, zmin, zmax, /floatvalues
;    
;     where:
;        xmin, xmax  - the minimum and maximum X coordinates
;        ymin, ymax  - the minimum and maximum Y coordinates
;        zmin, zmax  - the minimum and maximum Z coordinates
;        Floatvalues - If set, the values are floats.
;                      Default - doubles
; Example:
;     Obtain the extent of the current acis values
;      bbox, x0, x1, y0, y1, z0, z1
;
;-

 pro bbox, xmin, xmax, ymin, ymax, zmin, zmax, floatvalues = fv
on_error, 1
  common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save
;
; check to see if ACIS has been initialized
;
  if (n_elements(initacis) eq 0) then init_acis else $
    if (initacis eq 0) then init_acis
  if initacis eq 0 then begin
    print, '% BBOX: ACIS libraries are not supported by this System'
    return
  endif

  xmin = 0.0d0
  xmax = 0.0d0
  ymin = 0.0d0
  ymax = 0.0d0
  zmin = 0.0d0
  zmax = 0.0d0


      if n_elements(npart_save) eq 0 then npart_save = 0
      if npart_save eq 0 then begin
        Print, '% BBOX: No acis parts have been saved'
        return
      endif

;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_bounding_box',  $
          xmin, ymin, zmin, xmax, ymax, zmax)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_bounding_box',  $
          xmin, ymin, zmin, xmax, ymax, zmax)
      endif else if initacis eq 3 then begin
; VMS
        junk = call_external(ACIS_EXE, 'acis_bounding_box',  $
          xmin, ymin, zmin, xmax, ymax, zmax)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_BOUNDING_BOX@8',  $
          xmin, ymin, zmin, xmax, ymax, zmax)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_bounding_box',  $
          xmin, ymin, zmin, xmax, ymax, zmax)
      endif
  if keyword_set(fv) then  begin
    xmin = float(xmin)
    xmax = float(xmax)
    ymin = float(ymin)
    ymax = float(ymax)
    zmin = float(zmin)
    zmax = float(zmax)
  endif
  return
  end


