; ------------------------------------------------------------------------
; $Id: list_acis_parts.pro,v 1.1 2009/04/10 17:24:02 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro list_acis_parts, names= pname, material = pmat , update = update

;+
; ; NAME:
;       list_acis_parts
;
; PURPOSE:
;       List the ACIS parts that are currently loaded and available.
;       This is now identical to AHELP
;
; CALLING SEQUENCE:
;       LIST_ACIS_PARTS
;
; INPUTS:
;       none
;  
; KEYWORD PARAMETERS:
;    NAMES    - Part names
;    MATERIAL - Part material
;    UPDATE   - Reat the names and materials from the ACIS database
;
; COMMON BLOCKS:
;       ACISCOM
;       ACIS_PARTS_COM
;
; SEE ALSO:
;      AHELP, READ_ACIS, ACIS2CIM
;
; MODIFICATION HISTORY:
;      Original version, D. B. Seidel, Sept. 26, 2008
;      Added keywords from AHELP, April 10, 2009, LPM
;      
;-
;
on_error, 1

common aciscom, initacis, acis_exe
common acis_parts_com, part_name, material_name, npart_save
;
;     check to see if ACIS has been initialized
;
if (n_elements(initacis) eq 0) then init_acis $
else if (initacis eq 0) then init_acis
if initacis eq 0 then begin
  print, '% LIST_ACIS_PARTS: ACIS libraries are not supported by this System'
  return
endif

if npart_save eq 0 then begin
  print, '% LIST_ACIS_PARTS: No ACIS parts have been read'
  return
endif
if keyword_set(update) then readacis, /update

pchrs = max(strlen(part_name))
mchrs = max(strlen(material_name))
ptit = 'Part Name'
mtit = 'Material Name'
npt = strlen(ptit)
nmt = strlen(mtit)
pwin = pchrs > npt
mwin = mchrs > nmt
form='(' + strtrim(string(pwin - npt + 2),2) + 'x,a)'

print,form='($,2a)','Part Number  ',ptit
print,form=form,mtit
dash = string(make_array((pwin > mwin),/byte,val=byte('-')))
print,'-----------  ',strmid(dash,0,pwin),'  ',strmid(dash,0,mwin)
for i=0,npart_save-1 do begin
  nsp = pwin - strlen(part_name[i]) + 2
  form = '(i7,6x,a,' + strtrim(string(nsp),2) + 'x,a)'
  print,form=form,i+1,part_name[i],material_name[i]
endfor
if arg_present(pname) then pname = part_name
if arg_present(pmat) then pmat = material_name
end
