; ------------------------------------------------------------------------ 
; $Id: wriacis.pro,v 1.4 2009/04/10 17:56:57 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Write an acis set of parts
;
; Format:
;     WRIACIS [, filename] 
; where:
;     filename - the acis assembly file
;                (.sat will be added to the name)
;
; Example:
;     Write the file 'myparts.sat'
;       WRIACIS, 'myparts'
;-
 pro wriacis, filename
on_error, 1
 common aciscom, initacis, acis_exe
  common acis_parts_com, part_name, material_name, npart_save

  npar = n_params()
;
;     check to see if ACIS has been initialized
;
      if (n_elements(initacis) eq 0) then begin
        print,'% WRIACIS: ACIS has not been initialized'
        return
      endif 
      if initacis eq 0 then begin
        print, '% WRIACIS: ACIS objects have not been stored.'
        return
      endif
    if n_elements(npart_save) eq 0 then npart_save = 0

    if npart_save  eq 0 then begin
      print, '% WRIACIS: No valid parts to write'
      return
    endif
     
    if n_elements(filename) eq 0 then begin
      get_tqstring, lcquit, 'Enter the output filename', $
      'acis_out_default', title='ACIS Geometry Output File   (.sat)'
      if lcquit ne 0 then return
    endif
    filename = strtrim(filename, 2)
    len = strlen(filename) 
    if len eq 0 then begin
      Print, '% WRIACIS: A valid file name must be provided'
      return
    endif
;   eliminate .sat if it exists
    i = strpos(strlowcase(filename), '.sat')
    if ((i gt 0) and ((i+4) eq len)) then filename = strmid(filename, 0, i)

    bfile = [byte(filename), 0B]
    ;
      if initacis eq 1 then begin
; HP
        junk = call_external(ACIS_EXE, 'acis_write', bfile)
      endif else if initacis eq 2 then begin
; Sun
        junk = call_external(ACIS_EXE, 'acis_write', bfile)
      endif else if initacis eq 3 then begin
; VMS
        file = string(bfile)
        junk = call_external(ACIS_EXE, 'acis_write', file)
      endif else if initacis eq 4 then begin
; WIN
       junk = call_external(ACIS_EXE, 'ACIS_WRITE@8', bfile)
      endif else if initacis eq 5 then begin
; MAC
         junk = call_external(ACIS_EXE, 'acis_write', bfile)
      endif

    return
  end
