; ------------------------------------------------------------------------ 
; $Id: ataxes.pro,v 1.2 2005/03/09 22:21:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATAXES
;
; PURPOSE:
;	This procedure sets up the axes for an ATHETA geometry plot,
;	with no data actually plotted.
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATAXES, Xr, Yr, Xlab, Ylab, Titl [, FLIP=Flip] [, CLEAN=Clean] 
;       [, THERMOMETER=Thermometer] [, OTO=oto] [, _EXTRA=Extra]
;
; INPUTS:
;	Xr:	range for x axis; 2 element array
;	Yr:	range for y axis; 2 element array
;	Xlab:	x axis label
;	Ylab:	y axis label
;	Titl:	plot title
;
; KEYWORD PARAMETERS:
;	FLIP:	Set this keyword to flip the plot orientation to y-x,
;		Default is x-y.
;	CLEAN:	Set this keyword to set up plot with no axes, labels
;		or tick marks.  Default is draw axes, labels and ticks.
;	THERMOMETER:	
;		Set this keyword to leave room for a thermometer of 
;               contour levels on the plot.  Default is no thermometer.
;	OTO:	Set this keyword for a one-to-one aspect ratio
;	_EXTRA:	Extra keywords to be passed to PLOT procedure
;
; RESTRICTIONS:
;	"Extra" keywords must be valid keywords for PLOT procedure.
;
; EXAMPLE:
;	Set up axes for ATHETA geometry plot with specified ranges 
;       and titles.
;		ataxes, [0,.2], [0,.2], 'z (cm)', 'r (cm)', 'My plot'
;
;	Set up axes for geometry plot.  Omit axes and labels and leave
;       room for thermometer
;		ataxes, [0,.2], [0,.2], 'z (cm)', 'r (cm)', 'Clean plot' $
;               /clean, /thermometer
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro ataxes, xr, yr, xlab, ylab, titl, flip=flip, clean=clean, $
            thermometer=thermometer, oto=oto, _extra=extra
;
if (n_elements(xr) eq 0) or (n_elements(yr) eq 0) then return
;
@qscom.dat
if n_elements(qsplotinit) eq 0 then qsinit
zplaneold = 2
zvalueold = 0.0
;
if not keyword_set(flip) then flip=0
if keyword_set(thermometer) then xmargin = [10,10] else xmargin = !X.MARGIN
if keyword_set(clean) then begin
  xstyle = 21
  ystyle = 21
endif else begin
  xstyle = 17
  ystyle = 17
endelse
;
if flip then begin
  xoldvalue = 1
  yoldvalue = 3
  if keyword_set(oto) then begin
    plot, yr, xr, position = get_aspect(yr, xr, xmargin=xmargin), $
          xtitle=ylab, ytitle=xlab, title=titl, xmargin=xmargin, $
          xstyle=xstyle, ystyle=ystyle, _extra=extra, /nodata
  endif else begin
    plot, yr, xr, xtitle=ylab, ytitle=xlab, title=titl, xmargin=xmargin, $
          xstyle=xstyle, ystyle=ystyle, _extra=extra, /nodata
  endelse
endif else begin
  xoldvalue = 3
  yoldvalue = 1
  if keyword_set(oto) then begin
    plot, xr, yr, position = get_aspect(xr, yr, xmargin=xmargin), $
          xtitle=xlab, ytitle=ylab, title=titl, xmargin=xmargin, $
          xstyle=xstyle, ystyle=ystyle, _extra=extra, /nodata
  endif else begin
    plot, xr, yr, xtitle=xlab, ytitle=ylab, title=titl, xmargin=xmargin, $
          xstyle=xstyle, ystyle=ystyle, _extra=extra, /nodata
  endelse
endelse
;
ex = size(extra)
if ex[ex[0]+1] eq 8 then begin
  tag = tag_names(extra)
  ok = where (strmid(tag, 0, 3) eq 'HEA', nok)
  head = 1
  if nok gt 0 then begin
    junk = execute('head = extra.'+tag[ok[0]])
    head = head[0]
  endif 
endif else head = 1 
if ((not keyword_set(clean)) and (head eq 1)) then begin
  header, charsize=0.8
  header,'ATHETA  ', charsize=1.5, align=1
endif
;
return
end
