; ------------------------------------------------------------------------ 
; $Id: atdraw.pro,v 1.2 2005/03/09 22:21:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATDRAW
;
; PURPOSE:
;	This procedure sets up the axes and draws the ATHETA geometry 
;	including parts, coilsets and curves.
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATDRAW [, FLIP=Flip] [, FILL=Fill] [, GRID=Grid] [, CLEAN=Clean] $
;       [, THERMOMETER=Thermometer] [, OTO=oto] [, RADIUS=radius] $
;       [, OVERPLOT=overplot] [, _EXTRA=Extra]
;
; KEYWORD PARAMETERS:
;	FLIP:	Set this keyword to flip the plot orientation to r: x-axis,
;		z: y-axis.  Default is z: x-axis, r: y-axis.
;	FILL:	Set this keyword to draw parts as filled polygons.
;		Default is parts drawn as outlines.
;	GRID:	Set this keyword to draw ATHETA grid on plot.
;		Default is no grid drawn.
;	CLEAN:	Set this keyword to draw geometry with no axes, labels
;		or tick marks.  Default is draw axes, labels and ticks.
;	THERMOMETER:	
;		Set this keyword to leave space for a thermometer of 
;		contour levels on the plot.  Default is no thermometer.
;	OTO:	Set this keyword for a one-to-one aspect ratio
;	RADIUS: Draw coils as wires with radius "radius"
;	OVERPLOT:
;		Set this keyword to overplot geometry on current plot
;	_EXTRA:	Extra keywords to be passed to PLOT procedure
;
; RESTRICTIONS:
;	ATHETA geometry data must already be loaded in IDL arrays by 
;	previous ATHETA DRAW command.
;	"Extra" keywords must be valid keywords for PLOT procedure.
;
; EXAMPLE:
;	Draw ATHETA geometry with parts as filled polygons:
;		atdraw, /fill
;
;	Draw ATHETA geometry with the title specified:
;		atdraw, title='This is my title'
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro atdraw, flip=flip, fill=fill, grid=grid, clean=clean, $
  thermometer=thermometer, oto=oto, radius=radius, overplot=overplot, $
  _extra=extra
;
common at_colr
;
if not keyword_set(overplot) then begin
atgeom,'axes', flip=flip, clean=clean, thermometer=thermometer, oto=oto, $
               _extra=extra
endif
;
if keyword_set(overplot) then begin
  atgeom,'part', flip=flip, fill=fill, cl=at_fgc
endif else begin
  atgeom,'part', flip=flip, fill=fill
endelse
atgeom,'coil', flip=flip, radius=radius
atgeom,'curv', flip=flip
if keyword_set(grid) then atgrid, flip=flip
return
end
