; ------------------------------------------------------------------------ 
; $Id: atplot.pro,v 1.2 2005/03/09 22:21:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	ATPLOT
;
; PURPOSE:
;	This procedure draws the ATHETA geometry and plots contours
;	of the stream function field.
;
; CATEGORY:
;	ATHETA
;
; CALLING SEQUENCE:
;	ATPLOT [, Struct] [, FLIP=Flip] [, FILL=Fill] [, GRID=Grid] $
;       [, CLEAN=Clean] [, NLEVEL=Nlevel] [, LRANGE=Lrange] $
;       [, THERMOMETER=Thermometer] [, OTO=oto] [, _EXTRA=Extra]
;
; OPTIONAL INPUTS:
;	Struct:	Structure number of field to be plotted.  Default is 
;		structure 1.
;
; KEYWORD PARAMETERS:
;	FLIP:	Set this keyword to flip the plot orientation to r: x-axis,
;		z: y-axis.  Default is z: x-axis, r: y-axis.
;	FILL:	Set this keyword to draw parts as filled polygons.
;		Default is parts drawn as outlines.
;	GRID:	Set this keyword to draw ATHETA grid on plot.
;		Default is no grid drawn.
;	CLEAN:	Set this keyword to draw geometry with no axes, labels
;		or tick marks.  Default is draw axes, labels and ticks.
;	NLEVEL:	Number of contour levels to draw.  Default number of
;		levels is 10.
;	LRANGE:	Two element array giving contour range, [lowest, highest],
;		for the plot.  Defaults are field minimum and maximum.
;	THERMOMETER:	
;		Set this keyword to draw a thermometer of contour levels
;               on the plot.  Default is no thermometer.
;	OTO:	Set this keyword for a one-to-one aspect ratio
;	_EXTRA:	Extra keywords to be passed to PLOT procedure.
;
; RESTRICTIONS:
;	ATHETA geometry and field data must be already be loaded in 
;	IDL arrays by previous ATHETA PLOT command.
;	"Extra" keywords must be valid keywords for PLOT procedure.
;
; EXAMPLE:
;	Draw ATHETA contour plot. Field is in structure 2:
;		atplot, 2
;
;	Draw ATHETA contour plot with the title specified; include 
;       a thermometer:
;		atplot, title='My ATHETA field', /thermometer
;
; MODIFICATION HISTORY:
; 	Written by:	Rebecca Coats, 9/96.
;-
pro atplot, struct, flip=flip, fill=fill, grid=grid, clean=clean, $
            nlevel=nlevel, lrange=lrange, thermometer=thermometer, $
            oto=oto, _extra=extra
;
if n_params() eq 0 then struct = 1
atdraw, flip=flip, fill=fill, grid=grid, clean=clean, $
  thermometer=thermometer, oto=oto, _extra=extra
atfld, struct, nlevel=nlevel, lrange=lrange, flip=flip, clean=clean, $
  thermometer=thermometer
return
end
