; ------------------------------------------------------------------------ 
; $Id: closecdf.pro,v 1.4 2007/06/06 19:39:20 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro closecdf, p1
;
;+
;   Close a cdf file. If the current file is closed, the current file pointer
;   will be set the last file in the showcdf file listing.
;   format:   CLOSECDF, [, fileid]
;   where:
;      fileid    - file id of the file to be closed
;                  If it is omitted, the current file is closed.
;                  If it is zero or 'all', then all files will be closed.
;   Example:
;      Close the current file.
;         closecdf
;
;      Close the file with a file id of 3
;         closecdf, 3
;
;      Close all files
;         closecdf, 0
;    <or> closepff, 'all'
;-
@netcdfcom.dat
;     check to see if cdf has been initialized
;
      if n_elements(initcdf) eq 0 then begin
        print, '% CLOSECDF: No netCDF files are open'
        return
      endif
;
;     check on the number of parameters
;
      nparams = long( n_params())
;
;     get integers
;
      if (n_elements(p1) eq 0) then p1= -1L
      j = size(p1(0))
      if (j(1) eq 7) then begin
        p1 = strtrim(p1(0), 2)
        p1 = strupcase(p1)
        if (p1 eq 'ALL') then begin
          p1 = 0L 
        endif else begin
          print, ' Invalid parameter sent to to CLOSEPFF - ', p1
          return
        endelse
      endif
      fid = long(p1)
;
      if p1 eq 0 then begin
;       close all the files
        for i = 0, max_cdffile-1 do begin
          if array_cdfid(i, 1) ne 0 then ncdf_close, fcdf_id[i]
          array_cdfid(i, 1) = 0
        endfor
        fcdf_id[*] = -1
        cdf_id = -1
      endif else begin
        nel = n_elements (p1)
        if nel eq 1 and ( p1 eq -1 or p1 eq cdf_id) then begin
          if cdf_id ge 0 then begin
            ncdf_close, fcdf_id[cdf_id]
            fcdf_id[cdf_id] = -1
            array_cdfid(cdf_id, 1) = 0
            ok = where (array_cdfid(*, 1) ne 0, nok)
            if nok eq 0 then cdf_id = -1 else  cdf_id= ok(nok-1)
          endif else begin
            print, '% CLOSECDF: Not a valid current netCDF file id'
          endelse 
        endif else begin
          for i = 0, nel-1 do begin
            ok = where (array_cdfid(*, 0) eq p1(i), nok)
            if nok gt 0 then begin
              if array_cdfid(ok(0), 1) ne 0 then ncdf_close, fcdf_id[ok(0)]
              fcdf_id[ok(0)] = -1L
              array_cdfid(ok(0), 1) = 0
            endif else print, $
               '% CLOSECDF: '+strtrim(p1(i), 2)+' is not a valid file id'
             
          endfor
          if array_cdfid(cdf_id, 1) eq 0 then begin
            ok = where (array_cdfid(*, 1) ne 0, nok)
            if nok eq 0 then cdf_id = -1 else  cdf_id= ok(nok-1)
          endif
        endelse
      endelse
      return
      end
