; ------------------------------------------------------------------------ 
; $Id: setcdf.pro,v 1.4 2007/06/06 19:39:21 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  setcdf, p1, noload = noload, ierror = ierror, verbose = verbose
;+
;   Set the current file pointer to indicated file.
;   Format: SETCDF,  fileid [, /NOLOAD] [, /VERBOSE] [, IERROR = ierror]
;   where:
;      fileid - file id of desired file
;      NOLOAD - Keyword, if set, data will not be read into the common block
;      VERBOSE- Keyword, if set, information about data will be printed.
;               This keyword is ignored if NOLOAD is set
;   Note:  If fileid is undefined or is zero, nothing is done.
;
;    Example:
;   Set the current file pointer to the file with a file id (fid) of 3
;      setcdf, 3
;-
;
      on_error, 1
@netcdfcom.dat
;

      if n_elements(initcdf) eq 0 then init_cdf
;
;     check on the number of parameters
;
      ierror = 1
;     if no fid is entered then return
      if (n_elements(p1) eq 0) then begin
        print,'% Format: SETCDF,  File_id, [, /NOLOAD] [, /VERBOSE] [, IERROR = ierror]'
        return
      endif
;     if p1 is zero then quit
      if p1 eq 0 then begin
        print ,'% SETCDF: File id is zero - nothing changed'
        ierror = 0
        return
      endif
;
;     get integers for the write
;
      ok = where (array_cdfid(*, 0) eq p1, nok)
      if nok eq 0 then begin
        Print, '% SETCDF: ', strtrim(p1, 2), ' is not a valid file id.'
        return 
      endif
      ierror = 0
      cdf_id = ok(0)
      if keyword_set(noload) then return
      load_cdf_data, verbose = verbose
      end
