; ------------------------------------------------------------------------ 
; $Id: get_rectilinear.pro,v 1.4 2007/06/06 19:36:41 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro get_rectilinear, global, ier, nelements = nelements, delta= deltaval
;
; Get the rectilinear parameters for a dataset
;
on_error, 1
@netcdfcom.dat

      ier = 1
      nblo = num_el_blk
      ndim = num_dim
      ntime = n_elements(time_whole)

      if keyword_set (global) then begin
        nglo = 1
        loc_x = make_array(ndim+1, nglo+1, value = 0L, /long)
        loc_xc = make_array(ndim+1, nglo+1, value = 0L, /long)
        loc_farray = make_array(nglo+1, value = 0L, /long)
        loc_fcarray = make_array(nglo+1, value = 0L, /long)
        rec_size = make_array(ndim+1, nglo, /long)
        rec_csize = make_array(ndim+1, nglo, /long)
        rec_range = make_array(2, ndim+1, nglo+1)
        rec_crange = make_array(2, ndim+1, nglo+1)
        xcoord  = reform(coord_node(*, 0, 0))
        ycoord  = reform(coord_node(*, 0, 1))
        if ndim gt 2 then zcoord  = reform(coord_node(*, 0, 2)) else $
          zcoord =0.0
        get_rect_xyz, x1, nx, x2, ny, x3, nz, xcoord, ycoord, zcoord, $
           norder, ier, delta= deltaval
        nelements = [nx, ny, nz]
        xcoord = 0 & ycoord = 0 & zcoord = 0
        if ier ne 0  then begin
          print, '% GET_RECTILINEAR: Error in obtaining global nodes.'
          print, '           Nx, ny, nz = ',  nx, ny, nz
          return
        endif
        loc_farray(1) = +nx*ny*nz*ntime
        if ndim eq 2 then begin
          loc_x(*, 1) = [nx, ny, ntime]
          rec_size (*, 0) = [nx, ny, ntime]
          rec_range(0, *, 1) = [x1(0), x2(0), time_whole(0)]
          rec_range(1, *, 1) = [x1(nx-1), x2(ny-1), time_whole(ntime-1)]
        endif else begin
          loc_x(*, 1) = [nx, ny, nz, ntime]
          rec_size (*, 0) = [nx, ny, nz, ntime]
          rec_range(0, *, 1) = [x1(0), x2(0), x3(0), time_whole(0)]
          rec_range(1, *, 1) = $
            [x1(nx-1), x2(ny-1), x3(nz-1), time_whole(ntime-1)]
        endelse
        node_order = sort(norder)
        norder = 0
        x1_array =temporary(x1)
        x2_array =temporary(x2)
        x3_array =temporary(x3)

;       now the cell data 
        xcoord  = reform(coord_cell(*, 0, 0))
        ycoord  = reform(coord_cell(*, 0, 1))
        if ndim gt 2 then zcoord  = reform(coord_cell(*, 0, 2)) else $
          zcoord =0.0
        get_rect_xyz, x1, nx, x2, ny, x3, nz, xcoord, ycoord, zcoord, $
           corder, ier, delta= deltaval
        nelements = [nx, ny, nz]
        xcoord = 0 & ycoord = 0 & zcoord = 0

        if ier ne 0 then begin
          print, '% GET_RECTILINEAR: Error in obtaining cell values.'
          print, '           Block, nx, ny, nz = ', i, nx, ny, nz
          return
        endif

        loc_fcarray(1) = nx*ny*nz*ntime
        if ndim eq 2 then begin
          loc_xc(*, 1) =  [nx, ny, ntime ]
          rec_csize (*, 0) = [nx, ny, ntime]
          rec_crange(0, *, 1) = [x1(0), x2(0), time_whole(0)]
          rec_crange(1, *, 1) = [x1(nx-1), x2(ny-1), time_whole(ntime-1)]
        endif else begin
          loc_xc(*, 1) =  [nx, ny, nz, ntime]
          rec_csize (*, 0) = [nx, ny, nz, ntime]
          rec_crange(0, *, 1) = [x1(0), x2(0), x3(0), time_whole(0)]
          rec_crange(1, *, 1) = $
            [x1(nx-1), x2(ny-1), x3(nz-1), time_whole(ntime-1)]
        endelse
        cell_order = sort(corder)
        x1_carray =temporary(x1)
        x2_carray =temporary(x2)
        x3_carray =temporary(x3)
        corder = 0
        rec_range(*,*,0) = rec_range(*,*,1)
        rec_crange(*,*,0) = rec_crange(*,*,1)

      endif else begin
        loc_x = make_array(ndim+1, nblo+1, value = 0L, /long)
        loc_xc = make_array(ndim+1, nblo+1, value = 0L, /long)
        loc_farray = make_array(nblo+1, value = 0L, /long)
        loc_fcarray = make_array(nblo+1, value = 0L, /long)
        rec_size = make_array(ndim+1, nblo)
        rec_csize = make_array(ndim+1, nblo)
        rec_range = make_array(2, ndim+1, nblo+1)
        rec_crange = make_array(2, ndim+1, nblo+1)
          for i = 1, nblo do begin
            varid = ncdf_varid( fcdf_id[cdf_id], 'connect'+strtrim(i, 2))
            if varid ge 0 then begin
              ncdf_varget,  fcdf_id[cdf_id], varid, connect
              connect = connect-1
              ss = size(connect)
              tot = n_elements(connect)
              if nblo gt 1 then begin
                connect = reform (connect, tot, /overwrite)
                nod = connect(uniq (connect, sort(connect)))
                xcoord  = reform(coord_node(nod, 0, 0))
                ycoord  = reform(coord_node(nod, 0, 1))
                if ndim gt 2 then zcoord  = reform(coord_node(nod, 0, 2)) else $
                  zcoord =0.0
              endif else begin
                xcoord  = reform(coord_node(*, 0, 0))
                ycoord  = reform(coord_node(*, 0, 1))
                nod = lindgen(n_elements(xcoord))
                if ndim eq 2 then  zcoord =0.0 else $
                  zcoord  = reform(coord_node(*, 0, 2))
              endelse
              get_rect_xyz, x1, nx, x2, ny, x3, nz, $
                xcoord, ycoord, zcoord, norder, ier, delta= deltaval
              nelements = [nx, ny, nz]
              xcoord = 0 & ycoord = 0 & zcoord = 0
              if ier ne 0 then begin
                print, '% GET_RECTILINEAR: Error in obtaining node values.'
                print, '           Block, nx, ny, nz = ', i, nx, ny, nz
                return
              endif
              loc_farray(i) = loc_farray(i-1) +nx*ny*nz*ntime
              if ndim eq 2 then begin
                loc_x(*, i) = loc_x(*, i-1)+[nx, ny, ntime]
                rec_size (*, i-1) = [nx, ny, ntime]
                rec_range(0, *, i) = [x1(0), x2(0), time_whole(0)]
                rec_range(1, *, i) = [x1(nx-1), x2(ny-1), time_whole(ntime-1)]
              endif else begin
                loc_x(*, i) = loc_x(*, i-1)+[nx, ny, nz, ntime]
                rec_size (*, i-1) = [nx, ny, nz, ntime]
                rec_range(0, *, i) = [x1(0), x2(0), x3(0), time_whole(0)]
                rec_range(1, *, i) = $
                  [x1(nx-1), x2(ny-1), x3(nz-1), time_whole(ntime-1)]
              endelse

              if i eq 1 then begin
                if nblo eq 1 then node_order = sort(norder) else $
                  node_order = nod(sort(norder))
                x1_array = temporary(x1)
                x2_array = temporary(x2)
                x3_array = temporary(x3)
              endif else begin
                node_order = [node_order,nod((norder))]
                x1_array = [x1_array, x1]
                x2_array = [x2_array, x2]
                x3_array = [x3_array, x3]
              endelse
              norder = 0
              xcoord  = reform(coord_cell(loc_f(i-1):loc_f(i)-1, 0, 0))
              ycoord  = reform(coord_cell(loc_f(i-1):loc_f(i)-1, 0, 1))
              if ndim eq 2 then zcoord = 0.0 else $
                zcoord  = reform(coord_cell(loc_f(i-1):loc_f(i)-1, 0, 2))
              get_rect_xyz, x1, nx, x2, ny, x3, nz, $
                xcoord, ycoord, zcoord, corder, ier, delta= deltaval
              nelements = [nx, ny, nz]
              xcoord = 0 & ycoord = 0 & zcoord = 0
              if ier ne 0 then begin
                print, '% GET_RECTILINEAR: Error in obtaining cell values.'
                print, '           Block, nx, ny, nz = ', i, nx, ny, nz
                return
              endif
              loc_fcarray(i) = loc_fcarray(i-1) +nx*ny*nz*ntime
              if ndim eq 2 then begin
                loc_xc(*, i) = loc_xc(*, i-1) + [nx, ny, ntime ]
                rec_csize (*, i-1) = [nx, ny, ntime]
                rec_crange(0, *, i) = [x1(0), x2(0), time_whole(0)]
                rec_crange(1, *, i) = [x1(nx-1), x2(ny-1), time_whole(ntime-1)]
              endif else begin
                loc_xc(*, i) = loc_xc(*, i-1) + [nx, ny, nz, ntime]
                rec_csize (*, i-1) = [nx, ny, nz, ntime]
                rec_crange(0, *, i) = [x1(0), x2(0), x3(0), time_whole(0)]
                rec_crange(1, *, i) = $
                  [x1(nx-1), x2(ny-1), x3(nz-1), time_whole(ntime-1)]
              endelse

              if i eq 1 then begin
                cell_order = sort(corder)
                x1_carray = temporary(x1)
                x2_carray = temporary(x2)
                x3_carray = temporary(x3)
              endif else begin
                cell_order = [cell_order,sort(corder)+loc_f(i-1)]
                x1_carray = [x1_carray, x1]
                x2_carray = [x2_carray, x2]
                x3_carray = [x3_carray, x3]
              endelse
              corder = 0
              rec_range(0, 0, 0) = min(rec_range(0, 0, 1:*))
              rec_range(1, 0, 0) = max(rec_range(1, 0, 1:*))
              rec_range(0, 1, 0) = min(rec_range(0, 1, 1:*))
              rec_range(1, 1, 0) = max(rec_range(1, 1, 1:*))
              rec_range(0, 2, 0) = min(rec_range(0, 2, 1:*))
              rec_range(1, 2, 0) = max(rec_range(1, 2, 1:*))
              rec_crange(0, 0, 0) = min(rec_crange(0, 0, 1:*))
              rec_crange(1, 0, 0) = max(rec_crange(1, 0, 1:*))
              rec_crange(0, 1, 0) = min(rec_crange(0, 1, 1:*))
              rec_crange(1, 1, 0) = max(rec_crange(1, 1, 1:*))
              rec_crange(0, 2, 0) = min(rec_crange(0, 2, 1:*))
              rec_crange(1, 2, 0) = max(rec_crange(1, 2, 1:*))

              if ndim eq 3 then begin
                rec_range(0, 3, 0) = min(rec_range(0, 3, 1:*))
                rec_range(1, 3, 0) = max(rec_range(1, 3, 1:*))
                rec_crange(0, 3, 0) = min(rec_crange(0, 3, 1:*))
                rec_crange(1, 3, 0) = max(rec_crange(1, 3, 1:*))
              endif
            endif else begin
              print, '% READCDF: No Cell data in the file, NO data read'
              return
            endelse
          endfor
      endelse
      ier = 0
      return
      end
