; ------------------------------------------------------------------------ 
; $Id: closeexo.pro,v 1.2 2007/06/06 19:40:41 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro closeexo, p1
;
;+
;   Close a Exodus file. If the current file is closed, the current file pointer
;   will be set the last file in the showexo file listing.
;   format:   CLOSEEXO [, fileid]
;   where:
;      fileid    - file id of the file to be closed
;                  If it is omitted, the current file is closed.
;                  If it is zero or 'all', then all files will be closed.
;   Example:
;      Close the current file.
;         closeexo
;
;      Close the file with a file id of 3
;         closeexo, 3
;
;      Close all files
;         closeexo, 0
;    <or> closepff, 'all'
;
;       Written by:     L. P. Mix, October 10, 2005
;-
;
on_error, 1
;
@exoduscom.dat
;     check to see if Exodus has been initialized
;
 if n_elements(init_exo) eq 0 then begin
    print, '% CLOSEEXO: No Exodus files have been opened.'
    return
  endif
 if exo_id lt 0 then begin
    print, '% CLOSEEXO: No Exodus files are open.'
    return
  endif

;
; check on the number of parameters
;
;
; get integers
;
  if (n_elements(p1) eq 0) then p1= exo_id
  j = size(p1(0))
  if (j(1) eq 7) then begin
    p1 = strtrim(p1(0), 2)
    p1 = strupcase(p1)
    if (p1 eq 'ALL') then p1 = 0L else begin
      print, ' Invalid parameter sent to to CLOSEEXO - ', p1
      return
    endelse
  endif
  fid = long(p1)
;
  if p1 eq 0 then begin
;   close all the files
    for i = 0, max_exofile-1 do begin
      if array_exoid(i, 1) ne 0 then ncdf_close, fexo_id[i]
      array_exoid(i, 1) = 0
    endfor
    fexo_id[*] = -1
    exo_id = -1
  endif else begin
    nel = n_elements (p1)
    if nel eq 1 and ( p1 eq exo_id) then begin
      if exo_id ge 0 then begin
        ncdf_close, fexo_id[exo_id]
        fexo_id[exo_id] = -1
        array_exoid(exo_id, 1) = 0
        ok = where (array_exoid(*, 1) ne 0, nok)
        if nok eq 0 then exo_id = -1 else  exo_id= ok(nok-1)
      endif else $
        print, '% CLOSEEXO: Not a valid current Exodus file id'
    endif else begin
      for i = 0, nel-1 do begin
        ok = where (array_exoid(*, 0) eq p1(i), nok)
        if nok gt 0 then begin
          if array_exoid(ok(0), 1) ne 0 then ncdf_close, fexo_id[ok(0)]
          fexo_id[ok(0)] = -1L
          array_exoid(ok(0), 1) = 0
        endif else print, $
             '% CLOSEEXO: '+strtrim(p1(i), 2)+' is not a valid file id'
      endfor
      if array_exoid(exo_id, 1) eq 0 then begin
        ok = where (array_exoid(*, 1) ne 0, nok)
        if nok eq 0 then exo_id = -1 else  exo_id= ok(nok-1)
        if exo_id gt 0 then load_exo_data
      endif
    endelse
  endelse
  end
