; ------------------------------------------------------------------------ 
; $Id: load_exo_data.pro,v 1.3 2008/02/27 22:48:12 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro load_exo_data, dimname, dimvalue, verbose = verbose
;+ 
;  Load the exo common block
;-
;
      on_error, 1
@exoduscom.dat
      
      id = exo_id
      fid =  fexo_id[exo_id]
      lexovalid = -1
      wdf_array = 0
      node_array = 0
      t = systime(1)

      if n_elements(dimname) eq 0 then begin
        if (exo_id  lt 0) or (exo_id  ge max_exofile) then begin
          print,'% LOAD_EXO_DATA: Invalid EXODUS ID.'
          return
        endif
        if array_exoid(exo_id, 1) eq 0 then begin
          print,'% LOAD_EXO_DATA: Invalid EXODUS ID, Ident = 0.'
          return
        endif
        glob = ncdf_inquire(fid)
        dimname = make_array(glob.ndims, /string)
        dimvalue = make_array(glob.ndims, /long)
;       get the names of the dimensional variables
        for i = 0, glob.ndims-1  do begin
          ncdf_diminq, fid, i, name, dimval
          dimname(i) = strlowcase(strtrim(name, 2))
          dimvalue(i) = dimval
        endfor
      endif
      if exo_type(id) eq 1 then begin
;     EXODUS II parameters:
;       
;   name_elem_var  - String array with Cell array names
        apiversion = api_version[id]
        if num_elem_var(id) gt 0 then begin
          varid = ncdf_varid(fid, 'name_elem_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_elem_var= make_array(num_elem_var[id], /str)
            for i = 0, num_elem_var[id]-1 do  $
                name_elem_var(i)= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Cell Variables: ', $
              name_elem_var
          endif else begin
            print, '% LOAD_EXO_DATA: Cell variable names not found.'
            print, '                 Number of Cell Variables set to zero.'
            num_elem_var[id] = 0
          endelse
        endif

;   name_nod_var   - String array with Node array names
        if num_nod_var(id) gt 0 then begin
          varid = ncdf_varid(fid, 'name_nod_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_nod_var= make_array(num_nod_var(id), /str)
            for i = 0, num_nod_var(id)-1 do  $
              name_nod_var(i)= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Node Variables: ', $
              name_nod_var
          endif else begin
            print, '% LOAD_EXO_DATA: Node variable names not found.'
            print, '                 Number of Node Variables set to zero.'
            num_nod_var(id) = 0
          endelse
        endif

;   name_glo_var   - String array with WDF array names
        if num_glo_var(id) gt 0 then begin
          varid = ncdf_varid(fid, 'name_glo_var')
          if varid ge 0 then begin
            ncdf_varget, fid, varid, value
            name_glo_var= make_array(num_glo_var(id), /str)
            for i = 0, num_glo_var(id)-1 do  $
              name_glo_var(i)= strtrim(value(*,i), 2)
            if keyword_set(verbose) then print, ' Global Variables: ', $
              name_glo_var
          endif else begin
            print, '% LOAD_EXO_DATA: Global variable names not found.'
            print, '                 Number of Global Variables set to zero.'
            num_glo_var(id) = 0
          endelse
        endif

;   Number of time steps
        ok = where(dimname eq 'time_step', nok)
        if nok eq 0 then nt = 0 else nt = dimvalue(ok(0))
        if keyword_set(verbose) then print, ' Number of time steps = ', $
            strtrim(dimvalue(ok(0)), 2)

;   Number of nodes
        ok = where(dimname eq 'num_nodes', nok)
        if nok eq 0 then num_nodes = 0 else num_nodes = dimvalue(ok(0))
        if keyword_set(verbose) then print, ' Number of nodes = ', $
            strtrim(dimvalue(ok(0)), 2)

;   Number of elements
        ok = where(dimname eq 'num_elem', nok)
        if nok eq 0 then num_elem = 0 else num_elem = dimvalue(ok(0))
        if keyword_set(verbose) then print, ' Number of elements = ', $
            strtrim(dimvalue(ok(0)), 2)

        if nt gt 0 then begin
;   time_whole     - Time values
          varid = ncdf_varid(fid, 'time_whole')
          if varid ge 0 then ncdf_varget, fid, varid, time_whole else begin
            print, '% LOAD_EXO_DATA: No time values. Check file type'
            return
          endelse
        endif else begin
           nt = 1
           time_whole = 0.0
        endelse

         ndim = num_dim[id]

;   X1_Label       - Coordinate name(s) for first dimension
;   X2_Label       - Coordinate name(s) for second dimension
        if ndim gt 0 then begin
          varid = ncdf_varid(fid, 'coor_names')
          if varid ge 0 then begin

            ncdf_varget, fid, varid, value
            x1_label = strtrim(value(*, 0), 2)
            if ndim gt 1 then begin
              x2_label = strtrim(value(*, 1), 2)
              if ndim gt 2 then x3_label = strtrim(value(*, 2), 2) else $
                x3_label = ''
            endif else begin
              x2_label = ''
              x3_label = ''
            endelse
          endif else begin
            x1_label = 'X'
            x2_label = 'Y'
            x3_label = 'Z'
          endelse
          lc_2d =  num_nod_var(id) +num_elem_var (id)
        endif else begin
          print, '% LOAD_EXO_DATA: Number of dimensions is zero'
          print, '               Setting number of node and cell variables to 0'
          num_nod_var(id) = 0
          num_elem_var(id) = 0
          lc_2d = 0
        endelse
;       lc_2d is the total number of node and element(cell) variables
        
        coord_node = 0L
        lc_vesey = 0
        if lc_2d gt 0 then begin
          if apiversion ge api_change then begin
            varid = ncdf_varid(fid, 'coordx')
            if varid ge 0 then ncdf_varget, fid, varid, coordx else begin
              print, '% LOAD_CDF_DATA: Unable to read COORDX'
              return
            endelse
            num_nodes = n_elements(coordx)
            varid  = ncdf_varid(fid, 'coordy')
            if varid ge 0 then ncdf_varget, fid, varid, coordy else begin
              print, '% LOAD_CDF_DATA: Unable to read COORDY'
              return
            endelse
            if ndim eq 3 then begin
              varid  = ncdf_varid(fid, 'coordz')
              if varid ge 0 then ncdf_varget, fid, varid, coordz else begin
                print, '% LOAD_CDF_DATA: Unable to read COORDZ'
                return
              endelse
              coord_node = reform([coordx, coordy, coordz], num_nodes, 3)
            endif else coord_node = reform ([coordx, coordy], num_nodes, 2)
          endif else begin
            varid = ncdf_varid(fid, 'coord')
            if varid ge 0 then ncdf_varget, fid, varid, coord_node else begin
              print, '% LOAD_CDF_DATA: Unable to read COORD'
              return
            endelse
          endelse
        endif
        if num_nod_var[id] gt 0 then begin
          DISPLX = where (name_nod_var eq 'DISPLX', nokx)
          DISPLY = where (name_nod_var eq 'DISPLY', noky)
          DISPLZ = where (name_nod_var eq 'DISPLZ', nokz)
          if nokx eq 0 then begin
; Fix for Roger Vesey! 
            displx = where (name_nod_var eq 'NODE_R(t)', nokx)
            if nokx gt 0 then lc_vesey = 1
            if apiversion ge api_change then $
              displx = 'vals_nod_var'+strtrim(displx[0]+1,2)
          endif

          if ((ndim eq 3) && (nokx gt 0) && (noky gt 0) && (nokz gt 0)) || $
             ((ndim eq 2) && (nokx gt 0) && (noky gt 0)) then begin
             displx = displx[0]
             disply = disply[0]
             displz = displz[0]
             if apiversion ge api_change then begin
               displx = 'vals_nod_var'+strtrim(displx+1,2)
               disply = 'vals_nod_var'+strtrim(disply+1,2)
               displz = 'vals_nod_var'+strtrim(displz+1,2)
             endif
             lc_displace = 1
          endif else lc_displace = 0
        endif else lc_displace = 0
        nblo = num_elem_blk[id]
        if nblo gt 0 then begin
          loc_v = make_array(nblo+1, /long)
          size_v = make_array(nblo, /long)
          for i = 1, nblo do begin
            nn = 'num_el_in_blk'+strtrim(i, 2)
            ok = where(dimname eq  nn, nok)
            if nok eq 0 then $
                  n = 0 else n= dimvalue(ok(0))
            loc_v[i]= n+loc_v[i-1]
            size_v[i-1] = n
          endfor
          if loc_v[nblo] ne num_elem then print, $
          'WARNING - Problem with loc_V and num_elem', loc_v[nblo], num_elem
        endif else begin
          loc_v = [0, 0]
          size_V = 0
        endelse
        
      endif else if exo_type(exo_id) eq 2 then begin
;   X1_Label       - Coordinate name(s) for first dimension
;   X2_Label       - Coordinate name(s) for second dimension
;   wdf_array      - Packed array of WDF arrays
;   Coord_node     - Maximum number of datasets.
;   Coord_cell     - Number of points
;   num_el_blk     - Number of datasets (WDF arrays)
;   Loc_f          - Pointer Array for each dataset
;   name_glo_var   - String array with WDF array names(clabel)
        ndset_id = ncdf_varid(fid, 'numb_dataset')
        locf_id = ncdf_varid(fid, 'pointer_array')
        wdf_id = ncdf_varid(fid, 'wdf_array')
        np_id = ncdf_varid(fid, 'numb_of_pnts')
        x1lab_id = ncdf_varid(fid, 'x1_label')
        x2lab_id = ncdf_varid(fid, 'x2_label')
        clab_id = ncdf_varid(fid, 'comment_label')
        ncdf_varget, fid, ndset_id, num_el_blk
        ncdf_varget, fid, locf_id, loc_f
        if num_el_blk gt 0 then $
           ncdf_varget, fid, wdf_id, wdf_array else wdf_array=0
        ncdf_varget, fid, np_id, coord_cell
        ncdf_varget, fid, x1lab_id,  label
        num_label = (size(label))(1)
        x1_label= strtrim(label, 2)
        ncdf_varget, fid, x2lab_id,  label
        x2_label= strtrim(label, 2)
        ncdf_varget, fid, clab_id,  label
        num_comment = (size(label))(1)
        name_glo_var = strtrim(label, 2)
        coord_node = n_elements(coord_cell)
        label = ''
      endif 
      elem_connect = 0
      lexo_valid = array_exoid(id, 1)
      end
