; ------------------------------------------------------------------------ 
; $Id: bigwin.pro,v 1.4 2010/09/29 18:01:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;
;+
; NAME:
;       BIGWIN
;
; PURPOSE:
;       Make a large PixMap Window
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;       BIGWIN,  WinSize, XSIZE = XSIZE, YSIZE = YSIZE, _extra = extrastuff
;
; INPUTS:
;       WinSize: Two element vector with window size
;                If only a singe element, this is ignored.
;                If WINSIZE has 2 or more elements, XSIZE and YSIZE are ignored.
;                  XSIZE = ROUND(WINSIZE[0]) > 128
;                  YSIZE = ROUND(WINSIZE[1]) > 128
;
; OPTIONAL KEYWORD PARAMETERS:
;       XSIZE  - Horizontal size of the window > 128
;                DEFAULT: XSIZE = 4*640 = 2560
;       YSIZE  - Vertical size of the window > 128
;                DEFAULT: YSIZE = 4*512 = 2048
;       _extra - Any other valid keyword for the WINDOW command.
;
; PROCEDURE:
;      The following command is issued:
;      WINDOW, /PIXMAP, /FREE,  XSIZE = XSIZE, YSIZE = YSIZE, _extra = extrastuff
;
; Example:
;       Make a big pixmap window
;         BIGWIN
; MODIFICATION HISTORY:
;      Original version:   L. P. Mix, May 18, 1992
;      September 29, 2010  Added WINSIZE parameter, L. P. Mix
;
;-

pro bigwin, winsize, xsize = xsize, ysize = ysize, _extra= extrastuff
on_error, 1
if n_elements(winsize) gt 1 then begin
  xsize = round(winsize[0]) > 128L
  ysize = round(winsize[1]) > 128L
endif else begin
  xsize = (n_elements(xsize) eq 0) ? 2560 : (Xsize[0] > 128)
  ysize = (n_elements(ysize) eq 0) ? 2048 : (Ysize[0] > 128)
endelse
window, /pix, xsize = xsize, ysize = ysize, /free,  _extra= extrastuff
end
