; ------------------------------------------------------------------------
; $Id$
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

pro comp_files, filea, fileb, WDF = WDF

;+
; NAME:
;       COMP_FILES
;       Compare two PFF files, dataset by dataset
;
; CALLING SEQUENCE:
;
;       COMP_FILES, FILE1, FILEB
; Where:
; File1:    Name of first file 
; File2:    Name of second file
; Current version limited to 300 datasets
; Maximum and minimum differences are stored in the following WDF arrays
; 601   max x difference
; 602   min x difference
; 603   max y difference
; 604   min y difference
; 
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, Jr., October 22, 2008
;-
on_error, 1

if get_maxwdf() lt 1000 then set_maxwdfarray, 1000

openpff, filea, ndset = ndseta
re, 1, ''
closepff

openpff, fileb, ndset = ndset
if ndset ne ndseta then begin
  print, 'Number of datasets is different - quit'
  print, ndseta, ndset
  return
endif else if ndset eq 0 then begin
  print, 'Number of datasets is ZERO'
  return
endif
re, 301, ''
closepff

; make arrays to hold the differences
maxy = make_array(ndset)
miny = maxy
maxx = maxy
minx = maxy
numb = make_array(ndset, /long)

for i = 0, ndset-1 do begin
  xa = getxf(i+1)
  xb = getxf(i+301)
; range of data
  rang =max(xa, min = m) - m
  minx[i] = min(xa-xb, max = m)/rang
  maxx[i] = m/rang
  ya = getyf(i+1)
  yb = getyf(i+301)
  numb[i]= n_elements(ya) - n_elements(yb)
  rang =max(ya, min = m) - m
  miny[i] = min(ya-yb, max = m)/rang
  maxy[i] = m/rang
endfor
if ndset lt 3 then begin
  print, 'MIN X =', minx
  print, 'MAX X =', maxx
  print, 'MIN Y =', miny
  print, 'MAX Y =', maxy
  print, 'NUMB Dif =', numb
  return
endif

i2w, [1., 1], maxx, 601, clab= 'Max X'
i2w, [1., 1], minx, 602, clab= 'Min X'
i2w, [1., 1], maxy, 603, clab= 'Max Y'
i2w, [1., 1], miny, 604, clab= 'Min Y'
i2w, [1., 1], miny, 605, clab= 'Number Diff'

plo, 601, 5, /color, /over
end
