;+
; NAME:
;   Function readinfpcb, filename, noconvert=noconvert, $
;       saturationlevel=saturationlevel ,silent=silent, $
;       lut=lut ,header_data =header_data
;
; PURPOSE:
;
;   Thisfunction reads a FUJI format file for the IP plate data and outputs a
;   psl (photo stimulable level) dose equivelent file.
;   The data for the conversion is read from the Fuji .PCB or .INF  header file
;   the data  itself must be in a .IMG file of the same name as the header file
;   this is standard for the fuji system.
;
; CATEGORY:
;   Put a category (or categories) here.  For example:
;   Widgets.
;
; CALLING SEQUENCE:
;
;
;  image=readinfpcb('filename')
;or:-
;  image=readinfpcb( )    if no filename is specified then dialog_pickfile is called to select a file
;
; INPUTS:
;       Arrayname     variable in which to store the pcb array
;
;
;
; KEYWORD PARAMETERS:
;      setting lut to a variable name returns the integer to floating  look up table conversion. This will be a 65536 length vector
;          containing the conversion factors for the unsigned  integers 0 to 2^16 -1 . will enable use of the no-convert keyword to
;         return a smaller uint array with the psl conversion done on the fly as requred to save memory.
;  setting  noconvert   inhibits the conversion to PSL units ie. gives the raw stored data as viewed witht he TINA sotfware
;   saturationlevel    returns the maximum value that the machine settings can return
;  setting the silent keyword inhibits the writing of info to the screen
;  setting header_data  to a variable name returns the .inf file headwer information as a structure


; OUTPUTS:
;   This procedure returns the dose equivelent .PSL file from the fuji scanner data.
;
; OPTIONAL OUTPUTS:
;
;
; COMMON BLOCKS:
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; PROCEDURE:
;
;
;
; EXAMPLE:
;       a=readinfpcb('I:\fe55stuff\helen1.inf')
;
;
;
; MODIFICATION HISTORY:
;   Written by: Steve Quillin     June 23 1998
;     june 2003,  SJQ edited to reverse width and height for PCB files only
;-


FUNCTION readinfpcb  ,pcbfilename , noconvert=noconvert ,saturationlevel=saturationlevel  ,silent=silent, lut=lut,header_data =header_data

IF N_Params() eq 0 THEN   pcbfilename=dialog_pickfile(filter=['*.pcb','*.inf'])



checkonfilename=file_info( pcbfilename)
if checkonfilename.exists eq 1 then begin


OPENR,1,pcbfilename
h=1
a=''

if strmid(pcbfilename,  STRLEN(pcbfilename)-3,3) eq 'pcb'  then begin



WHILE NOT EOF(1)   DO BEGIN
 READF,1,a        ;& PRINT,a
 sl=strpos(a,'=')
 IF sl GT -1 THEN BEGIN
    firstpart=STRMID(a,0,sl)


    CASE firstpart OF
        'height '     :  reads,STRMID(a,sl+1,strlen(a)-sl-1),height
        'width '      :  reads,STRMID(a,sl+1,strlen(a)-sl-1),width
; above lines changed to form the srray correctly in .pcb files only
 ;      'height '     :  reads,STRMID(a,sl+1,strlen(a)-sl-1),width
 ;       'width '      :  reads,STRMID(a,sl+1,strlen(a)-sl-1),height
        'xResolution ': reads,STRMID(a,sl+1,strlen(a)-sl-1),xResolution
        'yResolution ': reads,STRMID(a,sl+1,strlen(a)-sl-1),yResolution
        'GradLevels ' : reads,STRMID(a,sl+1,strlen(a)-sl-1),GradLevels
        'Rotation '   : reads,STRMID(a,sl+1,strlen(a)-sl-1),Rotation
        'flipped '    : reads,STRMID(a,sl+1,strlen(a)-sl-1),flipped
        'Latitude '   : reads,STRMID(a,sl+1,strlen(a)-sl-1),Latitude
        'Sensitivity ': reads,STRMID(a,sl+1,strlen(a)-sl-1),Sensitivity
        'IPType '     : reads,STRMID(a,sl+1,strlen(a)-sl-1),IPType
        ELSE          :Q=1
    ENDCASE
 ENDIF
ENDWHILE
close,1
if (rotation eq 0.) or (rotation eq 180.) then begin
   htemp=height
   height=width
   width=htemp
end

endif  else  if strmid(pcbfilename,  STRLEN(pcbfilename)-3,3) eq 'inf'  then begin
a1=' '
a2=a1
a3=a2

READF,1,a1          & IF ~Keyword_set(silent) then  PRINT,a1
READF,1,a2          & IF ~Keyword_set(silent) then  PRINT,a2
READF,1,a3          & IF ~Keyword_set(silent) then  PRINT,a3
READF,1,xResolution & IF ~Keyword_set(silent) then  PRINT,'xResolution  = ',xResolution
READF,1,yResolution & IF ~Keyword_set(silent) then  PRINT,'yResolution  = ',yResolution
READF,1,bitdepth    & IF ~Keyword_set(silent) then  PRINT,'bitdepth  = ',bitdepth
GradLevels = 2^bitdepth & IF ~Keyword_set(silent) then  PRINT,'GradLevels  = ',GradLevels
READF,1,height      & IF ~Keyword_set(silent) then  PRINT,'height   = ',height
READF,1,width       & IF ~Keyword_set(silent) then  PRINT,'width    = ',width
READF,1, Sensitivity & IF ~Keyword_set(silent) then  PRINT,'Sensitivity    = ',Sensitivity
READF,1,Latitude     & IF ~Keyword_set(silent) then  PRINT,'Latitude     = ',Latitude

header_data = CREATE_STRUCT('filetype', a1, 'filetitle', a2 ,'scan_dimensions',a3   , $
              'xResolution',xResolution  ,  'yResolution',yResolution , $
              'bitdepth' ,bitdepth,  'height', height ,  'width'  , width , $
              'Sensitivity' , Sensitivity,  'Latitude' , Latitude)
xResolution=xResolution/100.

close,1

endif
imgfilename=strmid(pcbfilename,0,  STRLEN(pcbfilename)-3)+'img'
img=UINTARR(height,width)

IF  Keyword_set(silent) eq 0 then  PRINT,  'OPENING FILE  :- ',imgfilename,'  FOR READING'
OPENR,1,imgfilename
readu,1,img
close,1
byteorder,img


IF Keyword_set(noconvert) eq 0 then begin
    img=float(img)
 IF ~Keyword_set(silent) then PRINT,'PROCESSING FILE  ',imgfilename
    zeropoints=where(img eq 0.)
    scalefactor=(xresolution)^2  *  (4000./Sensitivity)
        img=  scalefactor*  10^(latitude*(TEMPORARY(img)/gradLevels-0.5))
    img(zeropoints)=0.
    saturationlevel=scalefactor *  10^(latitude*((gradLevels-1)/gradLevels-0.5))
IF  ~Keyword_set(silent) then PRINT,'Saturation Level  = ',saturationlevel

endif

 IF ~Keyword_set(silent) then print,'Finished'


IF  Arg_present(lut) then begin
lut=findgen(2L^16)
 scalefactor=(xresolution)^2  *  (4000./Sensitivity)
lut=  scalefactor*  10^(latitude*(TEMPORARY(lut)/gradLevels-0.5))
lut[0]=0.
endif
RETURN ,img
endif else print,'Invalid Filename'

end
