; ------------------------------------------------------------------------ 
; $Id$
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;
;  Save the data stored in WDF arrays and in structure arrays
;
;  Format:  SAVE_NTOF, FILE, FILE = filename, _EXTRA = extrastuff, /QUIET
;
;  where:
;      file    -  File in which the data is stored
;                 Default is 'ntof_<shot number>_save.dat'
;                 A unique file will be determined using
;      Filename- Alternate method of specifying file name.
;      NOTE:  FIND_NEWFILE will be called to insure the filename is unique.
;             If filename is not unique, then:
;             If file contains a . then add _# before the period
;             If file has a leading . the add _# to the end
;             If file has no . then add _#
;    NOTE: If FILE is specified, FILENAM is ignored.
;      _EXTRA  -  Any additional keywords recognized by SAVE
;      QUIET   -  If set, no information regarding the output file name or the 
;                 number of arrays processed will be printed.
;  Example:
;    Save the data from the current session
;      save_ntof
;
;    Save the data from the current session in file 'Z5581.sav'
;      save_ntof, file = 'Z5581.sav'
;-
 PRO save_ntof,file,  file = filnam, _extra= extrastuff, quiet = quiet

 on_error, 1 
;
  get_ntof_data,sntof_init, sshot_number, snumb_los, sname_los, snumb_detect, $
              smax_detect, sdd_time, sdt_time, sdetect_dist, sdetect_name, $
              sdetect_resp, sbang_time, sfit_velocity, sfit_energy, slsfit_param
 
  if n_elements(sntof_init) eq 0 then begin
    print, '% SAVE_NTOF_FILE:  No data to save.'
    return
  endif
;

  if n_elements (file) eq 0 then if n_elements(filnam) gt 0 then file = filnam
;
  if n_elements (file) eq 0 then file = 'ntof_'+strtrim(sshot_number, 2)+'_save.dat'
  file = find_newfile(file, /quiet)
; set up waveform arrays
; save the data
  save, file = file, _extra= extrastuff
if not keyword_set(quiet) then $
  print, '% SAVE_NTOF: Data saved to file, ', file
;
; clean up the memory
 sntof_init = 0.0
 sshot_number = 0.0
 snumb_los = 0.0
 sname_los = 0.0
 snumb_detect = 0.0
 smax_detect = 0.0
 sdd_time = 0.0
 sdt_time = 0.0
 sdetect_dist = 0.0
 sdetect_name = 0.0
 sdetect_resp = 0.0
 sbang_time = 0.0
 sfit_velocity = 0.0
 sfit_energy = 0.0
 slsfit_param = 0.0
end
