; ------------------------------------------------------------------------ 
; $Id: bc_vertex.pro,v 1.1 2006/07/19 15:57:12 rscoats Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
function bc_vertex, pdi_bc, imax, jmax, kmax, xi, xj, xk, $
                    system = systemin, z_exact = z_exact

@qs_enums.com
                 
 on_error , 2

  if n_elements(systemin) gt 0 then sys = systemin else sys = CARTESIAN

  add_chunk = 4000L
  max_v = add_chunk
  vertices = make_array(3, add_chunk, /float)
  size_v = 0L

  if pdi_bc[5] eq I_DIR then begin

    if n_elements(z_exact) gt 0 then begin 
      if z_exact ge xi[1] and z_exact le xi[imax] then x_norm = z_exact $
      else return, 0
    endif else x_norm = xi[pdi_bc[4]] 

    if sys eq CYLINDRICAL or sys eq SPHERICAL then jstep = 1 $
    else jstep = pdi_bc[2] - pdi_bc[0]
    if sys eq SPHERICAL then kstep = 1 $
    else kstep = pdi_bc[3] - pdi_bc[1]
    
    for k=pdi_bc[1], pdi_bc[3]-1, kstep do begin
      for j=pdi_bc[0], pdi_bc[2]-1, jstep do begin
        if size_v + 4 gt  max_v then begin
          vertices = [[vertices], [make_array(3, add_chunk, /float)]]
          max_v = max_v + add_chunk
        endif
        if size_v + 4 le max_v then begin
          vertices[I_DIR-1, size_v:size_v+4-1] = x_norm
          vertices[J_DIR-1, size_v] = xj[j]
          vertices[K_DIR-1, size_v] = xk[k]
          vertices[J_DIR-1, size_v+2] = xj[j+jstep]
          vertices[K_DIR-1, size_v+2] = xk[k+kstep]
          if pdi_bc[4] eq 1 then begin
            vertices[J_DIR-1, size_v+1] = xj[j+jstep]
            vertices[K_DIR-1, size_v+1] = xk[k]
            vertices[J_DIR-1, size_v+3] = xj[j]
            vertices[K_DIR-1, size_v+3] = xk[k+kstep]
          endif else begin
            vertices[J_DIR-1, size_v+1] = xj[j]
            vertices[K_DIR-1, size_v+1] = xk[k+kstep]
            vertices[J_DIR-1, size_v+3] = xj[j+jstep]
            vertices[K_DIR-1, size_v+3] = xk[k]
          endelse
          size_v = size_v + 4
        endif
      endfor
    endfor

  endif else if pdi_bc[5] eq J_DIR then begin

    if n_elements(z_exact) gt 0 then begin
      if z_exact ge xj[1] and z_exact le xj[jmax] then x_norm = z_exact $
      else return, 0
    endif else x_norm = xj[pdi_bc[4]] 

    if sys eq SPHERICAL then kstep = 1 $
    else kstep = pdi_bc[2] - pdi_bc[0]
    istep = pdi_bc[3] - pdi_bc[1]
    
    for i=pdi_bc[1], pdi_bc[3]-1, istep do begin
      for k=pdi_bc[0], pdi_bc[2]-1, kstep do begin
        if size_v + 4 gt  max_v then begin
          vertices = [[vertices], [make_array(3, add_chunk, /float)]]
          max_v = max_v + add_chunk
        endif
        if size_v + 4 le max_v then begin
          vertices[J_DIR-1, size_v:size_v+4-1] = x_norm
          vertices[K_DIR-1, size_v] = xk[k]
          vertices[I_DIR-1, size_v] = xi[i]
          vertices[K_DIR-1, size_v+2] = xk[k+kstep]
          vertices[I_DIR-1, size_v+2] = xi[i+istep]
          if pdi_bc[4] eq 1 then begin
            vertices[K_DIR-1, size_v+1] = xk[k+kstep]
            vertices[I_DIR-1, size_v+1] = xi[i]
            vertices[K_DIR-1, size_v+3] = xk[k]
            vertices[I_DIR-1, size_v+3] = xi[i+istep]
          endif else begin
            vertices[K_DIR-1, size_v+1] = xk[k]
            vertices[I_DIR-1, size_v+1] = xi[i+istep]
            vertices[K_DIR-1, size_v+3] = xk[k+kstep]
            vertices[I_DIR-1, size_v+3] = xi[i]
          endelse
          size_v = size_v + 4
        endif
      endfor
    endfor

  endif else if pdi_bc[5] eq K_DIR then begin

    if n_elements(z_exact) gt 0 then begin 
      if z_exact ge xk[1] and z_exact le xk[kmax] then x_norm = z_exact $
      else return, 0
    endif else x_norm = xk[pdi_bc[4]] 

    istep = pdi_bc[2] - pdi_bc[0]
    if sys eq CYLINDRICAL or sys eq SPHERICAL then jstep = 1 $
    else jstep = pdi_bc[3] - pdi_bc[1]
    
    for j=pdi_bc[1], pdi_bc[3]-1, jstep do begin
      for i=pdi_bc[0], pdi_bc[2]-1, istep do begin
        if size_v + 4 gt  max_v then begin
          vertices = [[vertices], [make_array(3, add_chunk, /float)]]
          max_v = max_v + add_chunk
        endif
        if size_v + 4 le max_v then begin
          vertices[K_DIR-1, size_v:size_v+4-1] = x_norm
          vertices[I_DIR-1, size_v] = xi[i]
          vertices[J_DIR-1, size_v] = xj[j]
          vertices[I_DIR-1, size_v+2] = xi[i+istep]
          vertices[J_DIR-1, size_v+2] = xj[j+jstep]
          if pdi_bc[4] eq 1 then begin
            vertices[I_DIR-1, size_v+1] = xi[i+istep]
            vertices[J_DIR-1, size_v+1] = xj[j]
            vertices[I_DIR-1, size_v+3] = xi[i]
            vertices[J_DIR-1, size_v+3] = xj[j+jstep]
          endif else begin
            vertices[I_DIR-1, size_v+1] = xi[i]
            vertices[J_DIR-1, size_v+1] = xj[j+jstep]
            vertices[I_DIR-1, size_v+3] = xi[i+istep]
            vertices[J_DIR-1, size_v+3] = xj[j]
          endelse
          size_v = size_v + 4
        endif
      endfor
    endfor

  endif

  if size_v gt 0 then return, vertices[*,0:size_v-1] else return, 0
  end
