; ------------------------------------------------------------------------ 
; $Id: qs_bc_in.pro,v 1.3 2007/05/16 17:09:21 rscoats Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro qs_bc_in, type, blk, pdi_in, $
              locib, locjb, lockb, bgxif, bgxjf, bgxkf

@qs_enums.com
@qs_data.com

num = n_elements(blk)
if num eq 0 then return

;  Start structure numbering after conductors
if numbr_bc eq 0 then first_bc = $
    max([first_diel+numbr_diel, first_cond+numbr_cond])

if type eq 'pmc' then bc_index = PMC $
else if type eq 'periodic' then bc_index = PERIODIC $
else if type eq 'inlet' then bc_index = INLET $
else if type eq 'outlet' then bc_index = OUTLET $
else if type eq 'dbc' then bc_index = DBC $
else print, 'Error - Unknown boundary condition type'

;  Build vertex structure

all_v = 0L

pdi = reform(pdi_in, 6, num)
imax = qs_grid.size[0,*]
jmax = qs_grid.size[1,*]
kmax = qs_grid.size[2,*]
crdflg = qs_grid.spare[0]
for nbc=0, num-1 do begin
  n = blk[nbc]
  verts = bc_vertex(pdi[*,nbc], $
                    imax[n], jmax[n], kmax[n], $
                    bgxif[locib[n]:locib[n]+imax[n]], $
                    bgxjf[locjb[n]:locjb[n]+jmax[n]], $
                    bgxkf[lockb[n]:lockb[n]+kmax[n]], $
                    system=crdflg)
  size_v = n_elements(verts)/3
  if size_v gt 0 then begin
    if all_v eq 0 then vertex = verts $
    else vertex = [[vertex], [verts]]
    all_v = all_v + size_v
  endif
endfor

current = first_bc + numbr_bc
build_vertex_struct, current, bc_index, type, crdflg, all_v, vertex
numbr_bc = numbr_bc + 1

return
end
