; ------------------------------------------------------------------------ 
; $Id: qs_his_in.pro,v 1.2 2008/02/05 20:51:53 rscoats Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2005 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro qs_his_in, blk, vdi_in, $
              locib, locjb, lockb, bgxif, bgxjf, bgxkf

@qs_enums.com
@qs_data.com

num = n_elements(blk)
if num eq 0 then return

;  Start structure numbering after bcs
if numbr_diag eq 0 then first_diag = $
  max([first_bc+numbr_bc,first_diel+numbr_diel,first_cond+numbr_cond])

;  Build vertex structure

all_v = 0L

vdi = reform(vdi_in, 6, num)
imax = qs_grid.size[0,*]
jmax = qs_grid.size[1,*]
kmax = qs_grid.size[2,*]
crdflg = qs_grid.spare[0]
pdi = make_array(6, /long)

for nseg=0, num-1 do begin
  n = blk[nseg]

  ndims = 3
  for i=0, 2 do $
    if vdi[i,nseg] eq vdi[i+3,nseg] then ndims = ndims - 1

  if ndims eq 0 then begin

  endif else if ndims eq 1 then begin
;  line - convert vdi to line & direction
    for i=0, 2 do $
      if vdi[i,nseg] ne vdi[i+3,nseg] then dir = i+1
    ny = (dir mod 3) + 1
    nz = (ny  mod 3) + 1
    iy = vdi[ny-1, nseg]
    iz = vdi[nz-1, nseg]
    ixb = min([vdi[dir-1, nseg],  vdi[dir+2, nseg]], max = ixe)
    pdi = [ixb, ixe, iy, iz, dir, 1]

;  Build narrow vertices on 4 sides centered around line
    verts = line_vertex(pdi, $
                      imax[n], jmax[n], kmax[n], $
                      bgxif[locib[n]:locib[n]+imax[n]], $
                      bgxjf[locjb[n]:locjb[n]+jmax[n]], $
                      bgxkf[lockb[n]:lockb[n]+kmax[n]], $
                      system=crdflg)

  endif else if ndims eq 2 then begin
;  surface - convert vdi to pdi
    for i=0, 2 do $
      if vdi[i,nseg] eq vdi[i+3,nseg] then norm = i+1
    ix = (norm mod 3) + 1
    iy = (ix   mod 3) + 1
    iz = vdi[norm-1,nseg]
    ixb = min([vdi[ix-1,nseg], vdi[ix+2,nseg]], max = ixe)
    iyb = min([vdi[iy-1,nseg], vdi[iy+2,nseg]], max = iye)
    pdi = [ixb, iyb, ixe, iye, iz, norm]

;  Build vertex on surface
    verts = bc_vertex(pdi, $
                      imax[n], jmax[n], kmax[n], $
                      bgxif[locib[n]:locib[n]+imax[n]], $
                      bgxjf[locjb[n]:locjb[n]+jmax[n]], $
                      bgxkf[lockb[n]:lockb[n]+kmax[n]], $
                      system=crdflg)
  endif else begin
;  volume - make sure indices are ordered low to high
    for i=0, 2 do begin
      ib = min([vdi[i,nseg], vdi[i+3,nseg]], max = ie)
      pdi[i]   = ib 
      pdi[i+3] = ie 
    endfor

;  Build vertices on 6 faces
;;    verts = vol_vertex(pdi, $
;;                      imax[n], jmax[n], kmax[n], $
;;                      bgxif[locib[n]:locib[n]+imax[n]], $
;;                      bgxjf[locjb[n]:locjb[n]+jmax[n]], $
;;                      bgxkf[lockb[n]:lockb[n]+kmax[n]], $
;;                      system=crdflg)
  endelse

 
  size_v = n_elements(verts)/3
  if size_v gt 0 then begin
    if all_v eq 0 then vertex = verts $
    else vertex = [[vertex], [verts]]
    all_v = all_v + size_v
  endif
endfor

current = first_diag + numbr_diag
maxx = get_maxstr()
if current gt maxx then set_maxstrarray, maxx+100
build_vertex_struct, current, DBC+1, 'history', crdflg, all_v, vertex
numbr_diag = numbr_diag + 1

return
end
