; ------------------------------------------------------------------------ 
; $Id: inqvar.pro,v 1.4 2007/06/06 19:44:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro inqvar, p1, p2, type = type, ndset = ndset, truncate = truncate, $
       ndim = ndim, dim = dim, printit = printit, variable = variable, $
       fileid = fileid
;+
;
;     This routine returns information about a variable in a pdb file
;     This routine calls dirpdb, with quiet set.
;
; Format:
;     INQVAR,  P1 [, P2] [, NDSET = ndset] [, TYPE = type]
;            [, NDIM = ndim]  [, DIM = dim] [, PRINTIT = printit]
;            [, VARIABLE = variable] [, FILEID = fileid]
;
; where:
;
;   P1 and P2 have several possible configurations
;       INQVAR,  start [, NUMBER]
;       INQVAR,  <Search_string>
;      where:
;        start    - first dataset number or an array of dataset numbers 
;        number   - if start is a vector, then number is ignored
;                   if start is a scalar, then variables are 
;                     variables = variables(start+ indgen(number)
;        SEARCH_STRING - Search string for the variable name
;                        See GET_MATCH for details of the search.
;   FILEID    - File id for the variable information
;               Default is the current file
;   VARIABLE  - Variable or dataset names
;   NDSET     - Number of datasets 
;   TYPE      - Dataset type 
;   NDIM      - Number of dimensions
;   DIM       - Minimum and maximum values for each dimension 
;               array [2, 3, ndset]
;   PRINTIT   - If set, the variable data wil be printed.
;               Default:  PRINTIT = 1
;   TRUNCATE  - If set, Delete source information.
;               Default: Truncate = 1
; 
; Example:
;    Print the variable information for variable 5 for the current dataset.
;       inqvar, 5
;
;    Print the variable information for variable 5 to 8 for the current dataset.
;       inqvar, 5, 8
;
;    Print the variable information for all variables beginning with src
;       inqvar, '^src'
;
; Original version:  July 29, 1999, lpm
;-
;
@pdbcom.dat

;     start the process
      on_error, 1
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: INQVAR,  P1 [, P2] [, NDSET = ndset] [, TYPE = type]'
        print,'          [, NDIM = ndim]  [, DIM = dim] [, PRINTIT = printit]'
        print,'          [, VARIABLE = variable] [, FILEID = fileid]'
        return
      endif
      if n_elements(initpdb) eq 0 then initpdb = 0
      if initpdb eq 0 then begin
        print, '% INQVAR:  No PDB files are open.'
        return
      endif
      if curfid lt 0 then begin
        print, '% INQVAR:  Current file id is not defined.  Open a pdb file.'
        return
      endif
      if n_elements(p1) eq 0 then begin
        print, '% INQVAR:  Variable(s) for INQVAR must be defined.'
        return
      endif
      dirpdb, p1, p2, type = type, ndset = ndset, truncat = truncate, $
        ndim = ndim, dim = dim, variable = variable, fileid = fileid, /quiet
      if ndset eq 0 then return

      if n_elements(printit) eq 0 then printit =1
      If keyword_set(Printit) then begin
        for i = 0, ndset-1 do begin
          print, 'Variable - '+ strtrim (variable[i], 2)
          print, 'Type: ', type[i]
          print, 'Number of Dimensions :' + strtrim (ndim[i], 2)
          if ndim[i] gt 0 then begin
            s= dim[1, 0:ndim[i]-1, i] - dim[0, 0:ndim[i]-1, i]
            sdim = 'Dimensions: '+ strtrim(s[0], 2)
            for j = 1, ndim[i]-1 do sdim = sdim+', '+strtrim(s[j], 2)
            print, sdim
            print, ''
          endif
        endfor
      endif
      end
