; ------------------------------------------------------------------------ 
; $Id: showpdb.pro,v 1.4 2007/06/06 19:44:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 pro showpdb, current = current, ufid = userfid, filename = filename, $
               number = number, quiet = quiet
;
;+
;  Show all opened PDB files, 
;  their file id (fid) and indicate the location of the current file pointer.
;  The current file id, and arrays of the user file ids and filesnames
;    can be received as keywords.
;  Format: SHOWPDB [, current = current] [, ufid = ufid]
;              [, filename = filename]  [, NUMBER= number]
;  where
;         current - user file id of the current file
;         ufid    - array of user file id's
;         filename- Array of file names corresponding to the ufid's
;         number  - number of files open
;                   Note: Number of datasets available with NDSPFF and PFF_INFO
;         quiet   - If present and not zero, send no output to terminal
;  Example:
;  Show the status, including file id, of all opened PDB files
;    showpdb
;  
;  Get the filenames and user file id's for all open pdb files
;  Do not send any output to the terminal
;    showpdb, c= cur, u=u, num= n, fil= fil, /q
;-
;
;
@pdbcom.dat

on_error, 1

;
;     check to see if pdb has been initialized
;
      number = 0
      p1= 0L
      userfid = 0
      filename = ''
      current = 0
      if (n_elements(initpdb) eq 0) then initpdb = 0
      if (initpdb eq 0) then begin
        print, '% SHOWPDB: No PDB files are open'
        return
      endif
      current = curfid
      ok = where (ufid ne 0, number)
      if number gt 0 then begin
        userfid = make_array(/long, number)
        filename = make_array(/string, number)
        for i = 0, number-1 do begin
          userfid[i] = ufid[ok[i]]
          filename[i] = (*family_info[ok[i]]).file
        endfor
        current = ufid[curfid]
        if not keyword_set(quiet) then begin
          print, ' Open PDB files: (current file: -->)
          print, '     File id     File Name'
          
          for i = 0, number-1 do $
            if (current eq userfid[i]) then $
              print, userfid[i], ' --> ' , filename[i] else $
              print, userfid[i], '     ' , filename[i]
        endif
       
      endif
      return
      end
