; ------------------------------------------------------------------------ 
; $Id: tindex.pro,v 1.3 2007/06/06 19:44:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
FUNCTION TINDEX, strin, time, maximum = maximum
;+
; NAME:
;       TINDEX 
;
; PURPOSE:
;       Function to return the index for the closest specified time.
;
; CALLING SEQUENCE:
;	IND = TIndex (STR, TIME, /Maximum)
;   where:
;       IND   - Index for output time closest to TIME
;               IND = -1 indicates an error.
;               valid values are [1, ntime]
;       STR   - Structure array number of a type 9 FE structure
;       MAXIMUM - If set time is ignored and the maximum time index is returned
;       TIME  - Time for desired INDEX.
;         
;
; EXAMPLE:
;       Determine the index closest to 14.5 in structure 4
;           ind = tindex(4, 14.5)
;       Plot the output at the time closest to 15.2 for structure 3
;           plotstr, 3, tindex(3, 15.2), /index
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, December 22, 1999
;-
;
;       return on an error
      on_error, 1
@qscom.dat
;  Common block
@qsstrcom.dat
;
      common pffcom, initpff, pff_exe
      common draw_defaults, rratio, bigsize, medsize, dualsize, smallsize

      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams eq 0 then begin
        print,'% Format: ', $
       'IND = TIndex( STR , Time)
        return, -1
      endif
      if n_elements(strin) eq 0 then begin
        print, '% TIndex: Structure must be defined'
        return, -1
      endif else arr= strin
;
      b = size(arr)
      if b(b(0)+1) ne 8 then begin
        if arr gt 0 and arr le maxstructure then begin
          s2i, arr, arr
          b = size (arr)
          if b(b(0)+1) ne 8 then begin
            print, '% TIndex:', $
   ' First parameter must be a structure or a structure array number'
            return, -1
          endif
        endif  else begin
          print, '% TINDEX:', $
         ' Structure array number must be greater than 0 and </='+ $
          strtrim(maxstructure, 2)
          return, -1
        endelse
      endif
      
      if arr.type ne 9  then begin
        Print, '%TIndex: Structure array must be a type 9 FE structure'
        return, -1
      endif
      if keyword_set (maximum) then return, arr.ntime
  
      if n_elements(time) eq 0 then begin
        print, '% TIndex: TIME must be defined'
        return, -1
      endif
      tt = min(abs(arr.x4 -time), j)
      return, j+1

END
