; ------------------------------------------------------------------------
; $Id: read_pdb_open.pro,v 1.4 2008/11/11 17:05:12 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO read_pdb_open , array, buffer, pid, type, len, ndim, dim, ier

;+
;  Reads a PDB dataset to an IDL array.
;  This is intended as a utility function for OPENPDB
;
;  Format:
;     READ_PDB_OPEN, Name, Buffer, PID, TYPE, LENGTH, NDIM, DIM, IERROR
;  where:
;      Name     - Dataset Name
;      Buffer   - IDL array of the stored values
;                 The following types are supported:
;                   CHAR, DOUBLE, FLOAT, INTEGER
;      PID      - PDB File ID (Long integer returned on open)
;      TYPE     - String for the dataset type being returned
;                 valid types are integer, float, double, and char
;      LENGTH   - Length of the dataset
;      NDIM     - Number of dimensions
;      DIM      - Size of each dimension
;      IER      - If zero, read failed.
;-
;
@pdbcom.dat

;     start the process
      on_error, 1
;
      nparams = n_params()
      ier =0L
      if nparams eq 0 then begin
      print,'% Format: '
      print,'   READ_PDB_OPEN, Name, Buffer, PID, TYPE, LENGTH, NDIM, DIM, IERROR'
        return
      endif
      if n_elements(initpdb) eq 0 then initpdb = 0
      if initpdb eq 0 then begin
        print, '% READ_PDB_OPEN:  No PDB files are open.'
        return
      endif
      

      if n_elements(array) eq 0 then begin
        print, '% READ_PDB_OPEN:  Dataset name must be presented.'
        return
      endif
      
;     Let on_error kick us out if any of the parameters are undefined.

      pid = long(pid[0])
      vartype = type[0] 
      len = len[0]
      ndim = long(ndim)
      dim = long(dim)
      lname = 256L
      ok = where (vartype eq ['char', 'double', 'float', 'integer', 'int'], nok)
      if nok eq 0 then begin
        print, '% READ_PDB_OPEN: Only the following dataset types are supported:'
        print, '   [char, double, float, integer], Dataset type: ', vartype
        return
      endif

      if ok[0] eq 0 then buffer = bytarr(len+1) $
        else if  ok[0]  eq 1 then buffer = dblarr(len) $
        else if  ok[0]  eq 2 then buffer = fltarr(len) $
        else if  ok[0]  gt 2 then buffer = lonarr(len)
      
      str2byt, array, varb, 256
      
                if initpdb eq 1 then begin
; HP
            junk = call_external(PDB_EXE, 'pdb_read', $
              pid, varb, lname, buffer, ier)
          endif else if initpdb eq 2 then begin
; Sun
            junk = call_external(PDB_EXE, 'pdb_read_', $
              pid, varb, lname, buffer, ier)
          endif else if initpdb eq 4 then begin
; WIN
           junk = call_external(PDB_EXE, '_PDB_READ@8', $
              pid, varb, lname, buffer, ier)
          endif else if initpdb eq 5 then begin
; MAC
           junk = call_external(PDB_EXE, 'pdb_read', $
              pid, varb, lname, buffer, ier)
          endif
       if ok(0) eq 0 then buffer = strmid(buffer, 0, dim[1, 0]) $
       else if ok[0] ne 0 and ndim gt 0 then begin
         if max(dim(0, *)) gt 1 then begin
           print, '% READ_PDB_OPEN: Some dimensions do not begin at 1'
           print, dim(0, *)
         endif
         del = (dim(1, *)-dim(0, *)) + 1
         buffer = reform(reform(buffer, del(0), del(1), del(2), /overwrite))
       endif else if n_elements(buffer) eq 1 then buffer = buffer[0]
       end
