; ------------------------------------------------------------------------ 
; $Id: set_pds_var.pro,v 1.3 2007/06/06 19:50:17 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro set_pds_var, par1, par2, file_handle = filehandle,  $
          type = vartype, ier = ier
;
; Copyright (c) 1998, Sandia National Laboratories  All rights reserved.
;       Unauthorized reproduction prohibited.
;
;+
; NAME:
;      SET_PDS_VAR
;
; PURPOSE:
;      Set a PDS variable from a specified segment
;      Also set the type of data returned.
;         Default is to return the type passed during the write process.
;      Uses: PDS_RESET_VAR
;
; CALLING SEQUENCE:
;
;       SET_PDS_VAR ([FILE_HANDLE,] VARIABLE [, FILE_HANDLE = file_handle]
;             [, TYPE  = type] [, IER = ier])
;
; WHERE:
;        FILE_HANDLE  - PDS file ID
;                       Default: File_Handle = current file
;                       Note: If two parameters are present then the keyword
;                             file_handle is ignored.
;        VARIABLE     - Variable ID or a string.
;        TYPE         - Integer or string indicating the type of output desired
;                       from this variable. Default Value = 0
;                       (These are the same codes returned by SIZE.)
;                         0 - No change <Default Value>
;                         1 - Byte data (Character data)
;                         2 - Integer data (2 byte integer)
;                         3 - Long integer data (4 byte integer)
;                         4 - Float data (4 byte real)
;                         5 - Double precision real data (8 byte real)
;                         7 - Character data
;        IER          - Error Parameter
;                       Returns 0 if successful
;                               -1 if unsuccessful or if help is selected
;
; EXAMPLE:
;       Set the variable 'veloc' and request the data to be returned as a float
;          set_pds_var('veloc', type =4)
;         
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, September 11, 1998
;-

@pds_datacom.dat

@pds_info_types.dat

      on_error, 1
 
 ier = -1L
 npar = n_params()
 if npar eq 0 then begin
   print, '% FORMAT: SET_PDS_VAR ([FILE_HANDLE,] VARIABLE 
   print, '          [, FILE_HANDLE = file_handle] [, TYPE = type] [, IER = ier]'
   return
 endif

 if n_elements(initpds) eq 0 then initpds = 0
 if initpds eq 0 then begin
   Print, '% SET_PDS_VAR: No PDS files are open'
   return
 endif

 if initpds lt 0 then begin
   print, '% SET_PDS_VAR: This installation does not support PDS'
   return
 endif

 if n_elements (par1) eq 0 then begin
   print, '% SET_PDS_VAR: First parameter must be defined'
   return
 endif
 if npar eq 2 then begin
   if n_elements (par2) eq 0 then begin
     print, '% SET_PDS_VAR: Second parameter must be defined'
     return
   endif
 endif

 if npar eq 1 then begin
   if keyword_set(filehandle) then fh = long(filehandle[0]) $
       else fh = long(pds_id)
   name  = par1[0]
 endif else begin
   fh = long(par1[0])
   name = par2[0]
 endelse

 if size(name, /type) eq 7 then begin
    name=get_pds_varid(name, ier = ier)
    if ier ne 0 then begin
      print,  '% SET_PDS_VAR: Error getting variable ID.'
      return
    endif
 endif else name=long(name)

 varT = 0L
 putTC = 0L
 if n_elements (type) then getTC = 0L else begin
   if type eq 1 or type eq 7 then getTC = PIO_CHAR else $
     if type eq 2 then getTC = PIO_SHORT else $
     if type eq 3 then getTC = PIO_INT else $
     if type eq 4 then getTC = PIO_FLOAT else $
     if type eq 5 then getTC = PIO_DOUBLE else getTC = 0L
 endif
    
 ier = 1L
 junk= call_external(PDS_EXE, 'pdsp_reset_var',fh, name, varT, putTC, getTC, ier)

 
 if ier ne 0 then $
    print, '% SET_PDS_VAR: Error setting PDS variable', name
 return
 end
