; ------------------------------------------------------------------------ 
; $Id: dirp.pro,v 1.4 2010/04/13 16:53:10 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dirp, p1, fileid = fileid, page = page, help = help, luout = luout
;+
;
;     this routine prints a directory of pff file to a file or allows 
;     the user to look at one "page" of directory information at a tiem
;     
;
;     DIRP  [, INIT] [, FILEID = FILEID] [, PAGE= PAGE] [,/HELP]
;          [, LUOUT = luout]
;        where init - initial dataset number
;                   - default is current dataset
;              page - number of datasets per page
;              fileid - file id of the file
;              help - puts a widget on the screen to guide the user in looking
;                     at pages of data on the screen 
;                     HELP is ignored if LUOUT is specified
;              luout - writes the directory to a file 
;                      LUOUT must be a string and is the name of 
;                      the file to which the directory is printed.
;-
      common pffcom, initpff, pff_exe
;
@qscom.dat
;
      if n_elements(qsplotinit) eq 0 then qsinit
;
      on_error, 1
      if n_elements (page) eq 0 then page = directorypage
;     check to see if pff has been initialized
;
      if n_elements(fileid) eq 0 then fileid = 0L
;
      if (n_elements(initpff) eq 0) then initpff = 0
      if initpff eq 0 then begin
        print, '% DIRP: No PFF files are open'
        return
      endif
;
;     check on the number of parameters
      if n_elements(p1) eq 0 then p1 = 0L
      curnum  = long(p1)
      pid = long(fileid)
      page = long (page)
      lcforward = page
      if n_elements (luout) ne 0 then begin
        if size(/type, luout) ne 7 then begin
          print, '% DIRP: LUOUT must be a character string'
          return
        endif
        m = strlen(luout)
        if (m eq 0) || (m ge 256) then begin
          print, '% DIRP: Maximum file name length is 255 characters'
          return
        endif
        ldst = ((m+31L)/32L) * 32L
        str2byt, luout, stringbyt, ldst
      endif
;
      if initpff eq 1 then begin
; HP
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldirp',curnum, pid, lcforward) $
        else begin
          junk = call_external (PFF_EXE, $
            'pff_sldirp',curnum, pid, lcforward, stringbyt, ldst)
          return
        endelse
      endif else if initpff eq 2 then begin
; Sun
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldirp_',curnum, pid, lcforward) $
        else begin
          junk = call_external (PFF_EXE, $
            'pff_sldirp_',curnum, pid, lcforward, stringbyt, ldst)
          return
        endelse
      endif  else if initpff eq 3 then begin
; VAX
        if n_elements (luout) eq 0 then begin
          lcfile = 0L
          file  = string (make_array(60, /byte, value = 32b))
          junk = call_external (PFF_EXE, $
          'pff_vaxdirp',curnum, pid, lcforward, file, lcfile) 
        endif else begin
          file = string (stringbyt)
          junk = call_external (PFF_EXE, $
            'pff_vaxdirp',curnum, pid, lcforward, file, ldst)
          return
        endelse
      endif else if initpff eq 4 then begin
; WIN
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
            '_PFF_SLDIRP@8',curnum, pid, lcforward) $
        else begin
          junk = call_external (PFF_EXE, $
            '_PFF_SLDIRP@8',curnum, pid, lcforward, stringbyt, ldst)
          return
        endelse
      endif else if initpff eq 5 then begin
; MAC
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldirp',curnum, pid, lcforward) $
        else begin
          junk = call_external (PFF_EXE, $
            'pff_sldirp',curnum, pid, lcforward, stringbyt, ldst)
          return
        endelse
      endif
;     see if the dataset is smaller than page
;
      maxdata = abs(lcforward)
      if maxdata le page then return
;     now we do the work of scrolling 
      if n_elements (help) eq 0 then help = directoryhelp
 if help ne 0 then begin
    base = WIDGET_BASE(TITLE = 'PFF File Directory', /column)
    l = WIDGET_LABEL(base, VALUE ='Use the arrow keys to view the directory')
    l = WIDGET_LABEL(base, VALUE =' -- or --')
    l = WIDGET_LABEL(base, VALUE ='Enter F to go forward 1 page')
    l = WIDGET_LABEL(base, VALUE ='Enter B to go backward 1 page')
    l = WIDGET_LABEL(base, VALUE ='Enter N to continue in same direction')
    l = WIDGET_LABEL(base, VALUE ='Enter G to go to the end of the file')
    l = WIDGET_LABEL(base, $
         VALUE ='Enter 1G or 1<return> to go to the beginning of the file')
    l = WIDGET_LABEL(base, VALUE ='Enter a number to go to that dataset number')
    l = WIDGET_LABEL(base, VALUE ='   ( #G or #<return> to go to dataset #)')
    l = WIDGET_LABEL(base, VALUE ='Enter X or Q to stop')
    WIDGET_CONTROL, base, /REALIZE
 endif
      page2 = page/2
      pagem = page-1
      curnumold = curnum
      curnummax = maxdata-page2
      if lcforward gt 0 then lcforward = 1 else lcforward = -1
scroll:
   a = (byte (get_kbrd(1)))(0)
;     see if we have a q or an x
      j = where (a eq [81, 88, 113, 120], nel)
      if nel gt 0 then begin
        if n_elements(help) gt 0 then WIDGET_CONTROL, base, /DESTROY
        return
      endif
;     see if we have a F
      if a eq 70 or a eq 102 then begin
        curnum = curnum +pagem
        lcforward = 1
;     see if we have a B
      endif else if a eq 66 or a eq 98 then begin
        curnum = curnum - pagem
        lcforward = -1
;     see if we have a N
      endif else if a eq 78 or a eq 110 then begin
        curnum = curnum +lcforward*pagem
;     see if we have an arrow key
      endif else if a eq 27 then begin
        a = get_kbrd(1) 
        if a eq '[' then begin
          a = get_kbrd(1)
          if a eq 'A' or a eq 'D' then begin
            curnum = curnum -pagem
            lcforward = -1
          endif else if a eq 'B' or a eq 'C' then begin
            curnum = curnum +pagem
            lcforward = 1
          endif
        endif
;     see if we have a G
      endif else if a eq 71 or a eq 103 then begin
        curnum = maxdata
;     see if we have a number
      endif else begin
        j = where(a eq [48, 49, 50, 51, 52, 53, 54, 55, 56, 57], nel)
        if nel gt 0 then begin
          num = a
again:
          a =  (byte (get_kbrd(1)))(0)
          if a eq 10 or a eq  71 or a eq 103 then begin
            curnum = long(string(num))
          endif else begin
            j = where(a eq [48, 49, 50, 51, 52, 53, 54, 55, 56, 57], nel)
            if nel gt 0 then begin
              num = [num, a]
              goto, again
            endif
          endelse
        endif
      endelse
      if curnum lt page2 then begin
        lcforward = 1
        first = 1L
        last = page
        curnum = page2
      endif else if curnum gt curnummax then begin
        curnum = curnummax
        lcforward = -1
        first = maxdata-pagem
        last = maxdata
      endif else begin
        first = curnum -page2
        last = first+pagem
      endelse
    if curnum ne curnumold then begin
      if initpff eq 1  then begin
; HP
        junk = call_external (PFF_EXE, 'pff_dscrol',pid, first, last)
      endif else if initpff eq 2 then begin
; SUN
        junk = call_external (PFF_EXE, 'pff_dscrol_',pid, first, last)
      endif  else if initpff eq 3 then begin
; VAX
        junk = call_external (PFF_EXE, 'pff_vaxdscrol',pid, first, last)
      endif else if initpff eq 4 then begin
; WIN
        junk = call_external (PFF_EXE, '_PFF_DSCROL@8',pid, first, last)
      endif else if initpff eq 5 then begin
; MAC
        junk = call_external (PFF_EXE, 'pff_dscrol',pid, first, last)
      endif
      curnumold = curnum
    endif else print, string(7b)
    goto, scroll
;
      end
