; ------------------------------------------------------------------------ 
; $Id: dirpff.pro,v 1.4 2010/04/13 16:54:22 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dirpff, p1, p2, p3, point = point, luout = luout, $
        filename = filename, width = width, number = number
;+
;
;     this routine prints a directory of pff files
;     two forms are possible:
;     DIRPFF  [, fileid ]  [, low] [, high ] 
;             [, LUOUT = luout] [, FILENAME = filename] [, WIDTH = width]
;             [, NUMBER = ndset] [, POINT = point]
;     DIRPFF  [, fileid ] , string  [, LUOUT = luout]
;             [, LUOUT = luout] [, FILENAME = filename] [, WIDTH = width]
;             [, NUMBER = ndset] [, POINT = point]
;
;      where: 
;        fileid   - file id of the file
;        low      - first dataset number
;                   If LOW is negative, a listing of +/- LOW around current
;                   dataset pointer will be returned.
;        high     - last dataset number
;        string   - search string for the dataset comments
;        point    - dataset pointer
;        luout    - name of a file if the directory is to a file
;        filename - name of the file
;        width    - desired width of the output
;                   default is 80 if luout is not specified
;                   default is 76 if luout is not specified
;                         because of a feature in IDL.
;                   default is 98 if luout is specified
;        number   - number of datasets in the directory
;
;     Example:
;          Get a directory around current dataset of 5 datasets
;            DIRPFF, -5
;          Get a directory of datasets 95 to 100
;            DIRPFF, 95, 100
;          Get a listing of all datasets with 'MOCAM2?'
;            DIRPFF, 'MOCAM2?'
;
;-
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      if n_elements(initpff) eq 0 then initpff = 0
      if initpff eq 0  then begin
        print, '% DIRPFF: No PFF files are open'
        return
      endif 
; 
;     check on the number of parameters
;
      nparams = long(n_params())
;
;     get integers for the write
;
      dstring = ' '
      slen = 0L
      indx = -1
      inpt= make_array(3,/long)
      if (nparams gt 0) then begin
        if (n_elements(p1) eq 0) then return
        j = size(p1)
        if (j(0) ne 0) then begin
          p1 = p1(0)
          j = size(p1)
        endif
        if (j(1) eq 7) then begin
          dstring = p1
          slen = long(strlen(dstring))
        endif else begin
          indx = indx+1
          inpt(indx) = long(p1)
        endelse
      endif
      if (nparams gt 1) then begin
        if (n_elements(p2) eq 0) then return
        j = size(p2)
        if (j(0) ne 0) then begin
          p2 = p2(0)
          j = size(p2)
        endif
        if (j(1) eq 7) then begin
          dstring = p2
          slen = long(strlen(dstring))
        endif else begin
          indx = indx+1
          inpt(indx) = long(p2)
        endelse
      endif
      if (nparams gt 2) then begin
        if (n_elements(p3) eq 0) then return
        j = size(p3)
        if (j(0) ne 0) then begin
          p3 = p3(0)
          j = size(p3)
        endif
        if (j(1) eq 7) then begin
          print ,' Third parameter cannot be a string'
          return
        endif else begin
          indx = indx+1
          inpt(indx) = long(p3)
        endelse
      endif
      indx = long(indx+1)
;
      ldstring = 64L
      lfstring = 256L
      number = 0L
      str2byt, dstring, dstringb, ldstring
      filename = make_array(lfstring, /byte, value = 32B)
      if n_elements (luout) ne 0 then begin
        test = size(luout)
        if test(1) ne 7 then begin
          print, ' LUOUT must be a character string'
          return
        endif
        str2byt, luout, stringlu, lfstring
        if n_elements(width) eq 0 then width = 98L else width = long(width)
      endif else begin
;       if n_elements(width) eq 0 then width = 80L else width = long(width)
        if n_elements(width) eq 0 then width = 76L else width = long(width)
      endelse

;
      point = 0L
      if initpff eq 1 then begin
; HP
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldir',indx, inpt, slen, dstringb, ldstring, point,  $
           number, width, filename, lfstring) $
        else junk = call_external (PFF_EXE, $
          'pff_sldir',indx, inpt, slen, dstringb, ldstring, stringlu, point, $
           number, width, filename, lfstring) 
      endif else if initpff eq 2 then begin
; Sun
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldir_',indx, inpt, slen, dstringb, ldstring, point, $
           number, width, filename, lfstring) $
        else junk = call_external (PFF_EXE, $
          'pff_sldir_',indx, inpt, slen, dstringb, ldstring, stringlu, point, $
           number, width, filename, lfstring) 
      endif else if initpff eq 3 then begin
; VMS
        dstringb = string(dstringb)
        stringlu = string(stringlu)
        filename = string(filename)
        
        if n_elements (luout) eq 0 then begin
          stringlu= string (make_array(60, /byte, value = 32b))
          ldstring = 0L
          junk = call_external (PFF_EXE, 'pff_vaxsldir',indx, inpt, $
             slen, dstringb, ldstring,stringlu, point, $
           number, width, filename)
        endif else begin 
          stringlu = string(stringlu)
          junk = call_external (PFF_EXE, 'pff_vaxsldir',indx, inpt, $
             slen, dstringb, ldstring, stringlu, point, $
             number, width, filename)
        endelse
      endif else if initpff eq 4 then begin
; WIN
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          '_PFF_SLDIR@8',indx, inpt, slen, dstringb, ldstring, point, $
           number, width, filename, lfstring) $
        else junk = call_external (PFF_EXE, $
          '_PFF_SLDIR@8',indx, inpt, slen, dstringb, ldstring, stringlu, point, $
           number, width, filename, lfstring) 
      endif else if initpff eq 5 then begin
; MAC
        if n_elements (luout) eq 0 then $
          junk = call_external (PFF_EXE, $
          'pff_sldir',indx, inpt, slen, dstringb, ldstring, point,  $
           number, width, filename, lfstring) $
        else junk = call_external (PFF_EXE, $
          'pff_sldir',indx, inpt, slen, dstringb, ldstring, stringlu, point, $
           number, width, filename, lfstring)
      endif
      filename = strtrim(filename, 2)
      inpt= 0
      return
      end
