; ------------------------------------------------------------------------ 
; $Id: openzz.pro,v 1.5 2007/06/25 16:33:24 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro OpenZZ, shot, scratch = scratch, fileid = fileid, ierror = ierror
;
;+
; Name: OPENZZ
; 
; Description:
;       Open a Z shot file or a Z scratch file
;       NOTE:  This only works on machines that mount the Z data.
;       The following files can be opened
;         Default: "/pff_data/pbfa2z/pbfa2z_"+shot_number+".pff"
;         Scratch: "/pff_data/scratch/z"+shot_number+"*.pff"
;
; Format: OPENZZ [, SHOT] [, /SCRATCH] [, IERROR = IERROR] [, FILEID = fileid]
;    where:
;       SHOT    - Z shot number
;                 If missing or zero, then Dialog_Pickfile will be called
;       SCRATCH - If set, open a STYGAR scratch file.
;       IERROR  - Error parameter
;                 If no file specified for open then error = -13
;                 If Error = -99, file not found
;                 If Error = n > -99, error is a pff library error code defined
;                   by the particular routine where the error occurred.
;                   For example, file has no PFF header.
;       FILEID  - File ID for file.  Passed to OPENPFF
; Example:
;    Open shot 848
;        OPENZZ, 848
;
;    Open a Z shot from a list
;        OPENZZ
;
; MODIFICATION HISTORY:
;    Modified Greg Dunham "openzz" by:       Paul Mix, May 8, 2002 
;-
 on_error, 1
 ierror = -13
 if n_elements(shot) eq 0 then shot = 0
 sep = path_sep()
 server = (strupcase(!version.os_family) eq 'WINDOWS') ? '\\sasn898' : ''
 path = (keyword_set(scratch)) ? server+sep+'pff_data'+sep+'scratch'+sep : $
                                 server+sep+'pff_data'+sep+'pbfa2z'+sep 

 if shot le 0 then file = $
   dialog_pickfile(title= 'Select PFF file to read', path = path, $
     /must_exist, filter = '*.pff') else begin
   if keyword_set(scratch) then file = path+'z'+strtrim(shot, 2)+'*.pff' $
     else file = path+'pbfa2z_'+strtrim(shot, 2)+'.pff'
 endelse
 if strlen(file) lt 1 then begin
   print, '% OPENZZ: No file selected'
   return
 endif  
 result = file_search(file, count = count)
 if count eq 0 then begin
   print, '% OPENZZ: File does not exist'
   print, '    File: ', file
   return
 endif 
 val = 0
 if count gt 1 then begin
   please_select, lcquit, val, result
   if lcquit ne 0 then return
 endif
 result = result[val]
 openpff, result, fileid, error = ierror
end
