; ------------------------------------------------------------------------ 
; $Id: get_dset_string.pro,v 1.4 2010/09/20 18:02:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro get_dset_string, dataset, fid, map, nmap, nmax, ier, lcstring, mmap = maxmap
;
;+
;
;  Gets a dataset from a string
;  Returns ier = 1 if not successful
;  returns ier = 0 if successful
;  fid is the file id
;  dataset is the dataset specifier and may be a string or a number
;  map is the dataset list 
;  nmap is the number in map
;  nmax is the maximum number of datasets desired
;       if nmax is less than 0 then all datasets are desired.
;       Default is 1
;  lcstring is 0 if dataset is not a string
;           is +/- 1 if dataset is a string
;  
;-
; add common block for call_external
;
      common pffcom, initpff, pff_exe
;
      on_error, 1
      ier = 1
      if (n_elements(initpff) eq 0) then initpff = 0
      if initpff eq 0 then message, 'No PFF files are open'
      if n_elements (dataset) eq 0 then dataset = 0
      if n_elements (fid) eq 0 then fid = 0
      if n_elements (nmax) eq 0 then nmax = 1
      b = size (dataset)
      pid = long(fid)
      if n_elements(maxmap) gt 0 then mmap = maxmap else mmap = ndspff(fid)
      if mmap eq 0 then begin
        print, '% GET_DSET_STRING: Maximum number of datasets is zero'
        map = 0
        nmap = 0
        return
      endif
      mmap = long(mmap) > 1
      map = make_array(mmap, /long)
      nmap = 0L
      if b(0) eq 0 then begin
; string for dataset
        if b(1)  eq 7 then begin
;         dataset = strtrim(dataset, 2)
          slen = long(strlen (dataset))
          lcstring = 1
          if slen le 1 then begin
            if dataset eq '*' or dataset eq '' then begin
              nmap = ndspff(pid)
              if nmap le 0 then begin
                print, '% GET_DSET_STRING: No datasets in this file'
                nmap = 0
                map = 0
                return
              endif
              map = 1+indgen(nmap)
              lcstring = -1
              slen = 0
            endif
          endif
          if slen gt 0 then begin
;           length of the dstring string and the label strings
            ldst = 128L
            str2byt, dataset, dstringbyt, ldst
; 
            if initpff eq 1 then begin
; HP
              junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 2 then begin
; Sun
              junk = call_external(PFF_EXE,'slgetdset_', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 3 then begin
; VMS
              dstringbyt = string (dstringbyt)
              junk = call_external(PFF_EXE,'get_dset', pid, slen, dstringbyt, $
                 nmap, map, mmap)
            endif else if initpff eq 4 then begin
; WIN
              junk = call_external(PFF_EXE,'_SLGETDSET@8', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif else if initpff eq 5 then begin
; MAC
              junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                 dstringbyt, nmap, map, mmap, ldst)
            endif
            if nmap eq 0 then begin
              print, '% GET_DSET_STRING: No match for string: ', dataset
              return
            endif
;           limit the string to defining nmax datasets
;           if we have more try to find an exact match
            if nmap gt nmax then begin
              if (strmid(dataset, 0, 1) ne '^' ) then newdata = '^'+dataset $
                else newdata = dataset
              if (strmid(dataset, slen-1, 1) ne '$') then $
                newdata = newdata+'$'
              slen2 = strlen(newdata)
              if slen2 gt slen then begin
                slen = slen2
                str2byt, newdata, dstringbyt, ldst
                if initpff eq 1 then begin
; HP
                  junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif else if initpff eq 2 then begin
; Sun
                  junk = call_external(PFF_EXE,'slgetdset_', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif  else if initpff eq 3 then begin
; VMS
                  dstringbyt = string (dstringbyt)
                  junk = call_external(PFF_EXE,'get_dset',pid,slen,dstringbyt, $
                     nmap, map, mmap)
                endif else if initpff eq 4 then begin
; WIN
                  junk = call_external(PFF_EXE, '_SLGETDSET@8', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif else if initpff eq 5 then begin
; MAC
                  junk = call_external(PFF_EXE,'slgetdset', pid, slen, $
                     dstringbyt, nmap, map, mmap, ldst)
                endif
              endif
              if nmap eq 0 or nmap gt nmax then begin
                print, ' A unique dataset was not found with the string '
                print, '             ' , dataset, ' -- Current datasets are'
                dirpff, dataset
                return
              endif
            endif
          endif else if lcstring ne -1 then begin 
            print, ' Cannot search for a null string'
            return
          endif
        endif else begin
          nmap = n_elements(dataset)
          map = dataset
          lcstring = 0
        endelse
        if nmap gt 0 then map = map(0:nmap-1) else map = 0
      endif else begin
        lcstring = 0
        map = dataset
      endelse
      nmap = n_elements(map)
      if nmap gt 0 then ier = 0L
      end
