; ------------------------------------------------------------------------ 
; $Id: readpff.pro,v 1.3 2010/09/20 18:02:18 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro readpff, fid, image,space, directin,block,tlabel = tlab, clabel = clab, $
      xlabel = xlab, ylabel = ylab, zlabel = zlab, wlabel = wlab, $
      blabel = blab, atlabel = atlab, label = lab, apptyp=apptyp, pff = pfftyp, $
      spare = ispare, mxblck = maxbl, npoints = nxyzt, file = file
;+
;  Read a PFF dataset.  This command allows most dataset types to be read
;  into IDL.  
;  
;                                        Following a READ command the dataset
;  pointer will be moved to the next array so that reads of sequential arrays
;  will require no value for the directory entry.
;      
;  This routine is OBSOLETE.
;
;  Format: READPFF [, fileid] [, image] [, space] [, dataset] [,  block]  
;            [, TLABEL = tlabel] [, CLABEL = clabel] [, BLABEL = blabel]
;            [,  XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel] 
;            [, WLABEL = wlabel] [,  ATLABEL = atlabel] [,  LABEL = label] 
;            [, AP = apptype] [, PFF = pfftype] [, SPARE = ispare(5)]  
;            [, MXBLCK = mxblck] [, NPOINTS = npoints] [, FILE = file]
;  where:
;    fileid - file id. If it is omitted or zero, the current file is used.
;             If present, the current file pointer will be set to this file.
;    image, space - arrays containing the dataset information defined below.
;    dataset - dataset directory entry.  If it is omitted or zero, the 
;              dataset indicated by the current dataset pointer is read.
;              Dataset may also be a string or string variable of two or more
;              characters or a single character.  See GET_DSET for a more
;              complete description of the string options.
;    block - integer indicating the block number.  If it is omitted 
;            or zero, the first block is read.
;    tlabel - dataset type label
;             the first 16 characters are stored in the directory
;    clabel - dataset comment
;             the first 64 characters are stored in the directory
;    blabel - title or label for this block
;    xlabel - first dimension label for this block
;    ylabel - second dimension label for this block
;    zlabel - third dimension label for this block
;    wlabel - fourth dimension label for this block
;    atlabel - attribute labels for vertex data only
;    label - array of all the above labels in the order given 
;            (label(0) = tlabel, label(1) = clabel, etc.)
;    ap - application dataset type (User parameter)
;    pff - PFF raw dataset type  ( see section 1.x)
;    spare - spare integer array for the application
;            (User specified interger iarray(5))
;    mxblck - maximum number of blocks in the dataset
;    npoints - array of  number of points for each dimension
;    file - name of file
;
;    Note:  Text strings are limited to 240 characters for the read command
;
;    Definition of the IMAGE and SPACE arrays in the READPFF command
;    Note: - nx, ny, nz are the number of points in each spatial dimension
;          - m is the number of spatial dimensions
;          - natt is the number of attributes
;          - nvert are the number of vertex values
; Dataset Type        Image                       Space
; 1-D Data            Amplitude (nx)              Grid Array (0:nx-1)
; 3-D Data            Amplitude (nx, ny, nz)      Grid Array (0:nx+ny+nz-1)
; Vector Data         Amplitude (nx, ny, nz, 3)   Grid Array (0:nx+ny+nz-1)
;                 (Vx. Vy, Vz  at each location)
; Vertex Data         Amplitudes (nvert, natt)    Grid Array (0:m-1, 0:nvert-1
; Integer/Float List  Integer List (nx)           Float List and Array (nfl+nz)
;                                                Float list = space (0:ny-1)
;                                                Float array = space (ny:ny+nz-1)
; N-Dimensional Vect  Amplitude                   Grid Array (0:nx+ny+nz+nt-1)
;  on M-Spatial Dim.    V(nx, ny, nz, nt, natt)
;                       natt = Vector dimensions
;                       nvert = Space dimensions
; Nonuniform 3-D Grid Integer array [nx, ny, nz]  Grid Array (0:nx+ny+nz-1)
;
;  Examples:
;  Read a data array into image from the current dataset of the current file
;     readpff, 0, image
;  Read a data array into picture from block 2 of dataset 6 in the current file
;  The space array is not returned.
;    readpff, 0, picture,  0, 6, 2
;  Read from the currrent file the dataset having \`mspin\' in the first 64 
;  characters of the dataset comment and put the amplitude and spatial arrays 
;  into pin and time.  Put the x-axis label into the character variable, xlab.
;    readpff, 0, pin, time, `mspin', x = xlab
;  Read from the file with file id  of 34, the current dataset and put the 
;  amplitude and spatial arrays into pinsig and time.  Put all the labels
;  into a character array(13), pinlab.
;    readpff, 34, pinsig, time, lab = pinlab
;  Read from the current file the dataset having `pin1' as the first
;  4 characters in the dataset comment and put the amplitude and spatial
;  arrays into pin and time.  Put all the labels into a character
;  array(13), LAB.
;    readpff, 0, pin, time, `^pin1', L = lab
;-
;     this routine reads a pff file
;
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      on_error, 1
      if (n_elements(initpff) eq 0) then initpff=0
      if (initpff eq 0) then begin
        print, '% READPFF: File must be opened before reading data'
        return
      endif
;
;     check on the number of parameters
;
      nparams =  n_params()
;
;     get integers for the write
;
      if nparams eq 0 then begin
 print,'% Format: READPFF [, fileid] [, image] [, space] [, dataset] [,  block]'
print,'            [, TLABEL = tlabel] [, CLABEL = clabel] [, BLABEL = blabel]'
print,'            [,  XLABEL = xlabel] [, YLABEL = ylabel] [, ZLABEL = zlabel]'
print,'            [, WLABEL = wlabel] [,  ATLABEL = atlabel] [,  LABEL = label]'
print,'            [, AP = apptype] [, PFF = pfftype] [, SPARE = ispare(5)]'
print,'            [, MXBLCK = mxblck] [, NPOINTS = npoints] [, FILE = file]'
        return
      endif
      print, '% READPFF:  This routine is obsolete.'
      Print, '            Please call Paul Mix, (505) 845-7493'
      print, '            if you are still using this routine.'
      if(nparams gt 0 and n_elements(fid) gt 0)then $
             get_pffinteger,fid else fid= 0L
      if (nparams gt 1) then image = 1L else image = 0L
      if (nparams gt 2) then space = 1L else space = 0L
      if (nparams gt 3) then begin
;       if directin is undefined - set equal to the current position (0)
        if (n_elements(directin) eq 0) then direct=0L else $
           direct = directin(0)
        b = size (direct)
        if (b(1) eq 7) then begin
          dstring = direct
          direct = 0L
        endif else begin
          dstring = '    '
          direct = long(direct)
        endelse
      endif else begin
        direct = 0L
        dstring = '    '
      endelse
      if(nparams gt 4 and n_elements(block) gt 0)then get_pffinteger, block $
        else block = 1L
      ier = 0L
;
;     set up the variables
;
      spare = make_array(5, /long)
      nxyzt = make_array(4, /long)
      apptyp = 0L
      pfftyp = 0L
      maxbl = 0L
      ndim  = 0L
      natt  = 0L
      nvert = 0L
      blank = make_array(240, /byte, value = 32b)
;     blank = string(replicate(32b,240))
      if initpff eq 3 then blank = string(blank)
      tlab = blank
      clab = blank
      blab = blank
      xlab = blank
      ylab = blank
      zlab = blank
      wlab = blank
      atlab0 = blank
      atlab1 = blank
      atlab2 = blank
      atlab3 = blank
      atlab4 = blank
      atlab5 = blank
      file = blank
;
;     length of the dstring string and the label strings
      ldst = 60L
      llabstr = 240L
      str2byt, dstring, dstringbyt, ldst
;
;     read the data
;
      if  initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE,'pff_slread',fid,direct,block,dstringbyt, $
          tlab, clab, blab, xlab, ylab, zlab, wlab, atlab0, atlab1, $
          atlab2, atlab3, atlab4, atlab5, file, $
          apptyp, pfftyp, spare, maxbl, nxyzt, ndim, natt, nvert, ier, $
          ldst, llabstr)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE,'pff_slread_',fid,direct,block,dstringbyt, $
          tlab, clab, blab, xlab, ylab, zlab, wlab, atlab0, atlab1, $
          atlab2, atlab3, atlab4, atlab5, file, $
          apptyp, pfftyp, spare, maxbl, nxyzt, ndim, natt, nvert, ier, $
          ldst, llabstr)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLREAD@8',fid,direct,block,dstringbyt, $
          tlab, clab, blab, xlab, ylab, zlab, wlab, atlab0, atlab1, $
          atlab2, atlab3, atlab4, atlab5, file, $
          apptyp, pfftyp, spare, maxbl, nxyzt, ndim, natt, nvert, ier, $
          ldst, llabstr)
      endif else if initpff eq 5 then begin
; MAC
        junk = call_external(PFF_EXE,'pff_slread',fid,direct,block,dstringbyt, $
          tlab, clab, blab, xlab, ylab, zlab, wlab, atlab0, atlab1, $
          atlab2, atlab3, atlab4, atlab5, file, $
          apptyp, pfftyp, spare, maxbl, nxyzt, ndim, natt, nvert, ier, $
          ldst, llabstr)
      endif else if initpff eq 3 then begin
; VMS
        dstringbyt = string(dstringbyt)
        junk = call_external(PFF_EXE,'pff_read',fid,direct,block,dstringbyt, $
          tlab, clab, blab, xlab, ylab, zlab, wlab, atlab0, atlab1, $
          atlab2, atlab3, atlab4, atlab5, file, $
          apptyp, pfftyp, spare, maxbl, nxyzt, ndim, natt, nvert, ier )
      endif
;
;     see if there was an error reading the header
;
      ispare = spare
      if (ier ne 0) then begin
        print,' Error reading PFF dataset header'
        return
      endif
;
      lab = make_array(13,/string)
;
      lab(0) = strtrim(tlab,2)
      lab(1) = strtrim(clab,2)
      lab(2) = strtrim(blab,2)
      lab(3) = strtrim(xlab,2)
      lab(4) = strtrim(ylab,2)
      lab(5) = strtrim(zlab,2)
      lab(6) = strtrim(wlab,2)
      lab(7) = strtrim(atlab0,2)
      lab(8) = strtrim(atlab1,2)
      lab(9) = strtrim(atlab2,2)
      lab(10) = strtrim(atlab3,2)
      lab(11) = strtrim(atlab4,2)
      lab(12) = strtrim(atlab5,2)
      for i = 0, 12 do lab(i) = strip_spaces(lab(i))
      file = strtrim(file,2)
;
;
      tlab = lab[0]
      clab = lab[1]
      blab = lab[2]
      xlab = lab[3]
      ylab = lab[4]
      zlab = lab[5]
      wlab = lab[6]
      atlab = make_array(6,/string)
      for i = 0, 5 do  atlab(i) = lab(7+i)
;
;     see if there was an error
;
      if (nxyzt(0) eq 0)  then begin
          print, '% READPFF:  Error reading PFF dataset'
          return
      endif
      nx = nxyzt(0)
      ny = nxyzt(1)
      nz = nxyzt(2)
      nt = nxyzt(3)
;
      image = 0
      space = 0
;
;     see if the datatype is supported
;
      if (pfftyp gt 8) then goto, endread
;     get the image data
;       parameter ( PFTUF3 = 1 , PFTUF1 = 2 , PFTNF3 = 3 )
;       parameter ( PFTNV3 = 4 , PFTVTX = 5 , PFTIFL = 6 )
;       parameter ( PFTNGD = 7 , PFTNG3 = 8 )
        if (pfftyp lt 4) then begin
          if (pfftyp eq 2) then image = make_array(nx, /float) $
            else image = make_array(nx, ny, nz, /float)
          np = nx*ny*nz
        endif else if (pfftyp eq 4) then begin
          image = make_array(nx, ny, nz, 3, /float)
          np = nx*ny*nz*3
        endif else if (pfftyp eq 5) then begin
          nattr = natt > 1
          image = make_array(nvert, nattr, /float)
          np = nvert*nattr
        endif else if (pfftyp eq 6) then begin
          if nx eq 0 then image = 0L else $
          image = make_array(nx, /long)
          np = nx
        endif else if (pfftyp eq 7) then begin
          if natt lt 1 then begin
            print, ' Problem reading NGD dataset'
            print, ' Vector dimensionality is less than 1
            return
          endif
          if nvert eq 1 then begin
            image = make_array(nx, natt, /float)
            np = nx
          endif else if nvert eq 2 then begin
            image = make_array(nx, ny, natt, /float) 
            np = nx*ny
          endif else if nvert eq 3 then begin
            image = make_array(nx, ny, nz, natt, /float)
            np = nx*ny*nz
          endif else if nvert eq 4 then begin
            image = make_array(nx, ny, nz, nt, natt, /float)
            np = nx*ny*nz*nt
          endif
          np = np*natt  
        endif else if (pfftyp eq 8) then begin
          nattr = natt > 1
          image = make_array(3, /float)
          np = 3L
        end
        if (pfftyp lt 5) then begin
          if (pfftyp eq 2) then begin
            space = make_array(nx, /float)
            nps = nx
          endif else begin
            space = make_array(nx+ ny+ nz, /float)
            nps = nx+ny+nz
          endelse
        endif else if pfftyp eq 6 then begin
          space = make_array(ny+nz, /float)
          nps = ny+nz
        endif else if pfftyp eq 5 then begin
          space = make_array(ndim,nvert, /float)
          nps = nvert*ndim
        endif else begin
          nps = nx+ny+nz+nt
          space = make_array(nps, /float)
        endelse
;
      if initpff eq 1 then begin
; HP
        junk = call_external(PFF_EXE, 'read_slarrays', image, space)
      endif else if initpff eq 2 then begin
; Sun
        junk = call_external(PFF_EXE, 'read_slarrays_', image, space)
      endif else if initpff eq 4 then begin
; WIN
        junk = call_external(PFF_EXE, '_READ_SLARRAYS@8', image, space)
      endif else if initpff eq 5 then begin
; MAC
        junk = call_external(PFF_EXE, 'read_slarrays', image, space)
      endif else if initpff eq 3 then begin
; VMS
        junk = call_external(PFF_EXE, 'read_arrays', image, space)
      endif
;
;     set image to zero if we have no attributes with vertex data
;
      if pfftyp eq 5 and natt eq 0 then image= 0L
endread:  return
      end
