; ------------------------------------------------------------------------ 
; $Id: sort_field.pro,v 1.2 2007/06/25 19:58:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro   sort_field, image, space, pfftyp, ndim, natt, np, lv, $
        aa, bb, cc, dd, ee, ff, gg, hh, ii, jj, ier
;
;+
;      SORT_FIELD 
;
;      Sorts field data from image and space
;      Format:   SORT_FIELD, image, space, pfftyp, ndim, natt, np, lv $
;       aa, bb, cc, dd, ee, ff, gg, hh, ii, jj, ier
;      where: image - first array returned by the READPFF command
;             space - second array returned by the READPFF command
;             pfftyp - PFF dataset type returned with the keywork PFF in
;                      the READPFF command
;             ndim  - number of spatial dimensions
;             natt - number of vector dimensions
;             np  - (ndim) number of elements in each spatial dimension
;             lv  - size of the field array 
;             aa, bb, ... - arrays containing the spatial parameters in the 
;                           first ndim and vectors in the next 4+natt
;-
;      zero the arrays - must have 2 spatial and 1 vector array
       ier = 0 & cc = 0.  &  dd = 0.
       ff = 0 & gg = 0 & hh = 0 & ii = 0 & jj = 0
; 
       b = size (image)
       if pfftyp eq 1 or pfftyp eq 3 or pfftyp eq 4 then begin
         ndim = b(0)
         if ndim lt 2 then begin
           print, 'Error in SORT_FIELD - data must have at least 2 dimensions'
           ier = 1
           return
         endif
         if (pfftyp eq 4) then begin
           if (ndim ne 4) then begin
             Print, ' NV3 data must have a 4 dimensional image file'
             ier = 1
             return
           endif
           ndim = 3
           np = [b(1), b(2), b(3)]
         endif else np = b(1:b(0))
         aa = space(0:b(1)-1)
         bb = space(b(1): b(1)+b(2)-1)
         if ndim gt 2 then cc = space(b(1)+b(2): *)
         space = 0
         if pfftyp eq 4 then begin
           natt = 3
           lv = b(6)/3
           ee = image(0:lv-1)
           ff = image(lv  :2*lv-1)
           gg = image(2*lv:3*lv-1)
           image = 0
         endif else begin
           lv = b(b(0)+2)
           natt = 1
           ee = temporary(image(0:lv-1))
         endelse
       endif else if pfftyp eq 7 then begin
         ndim = b(0) 
         natt = 1
         if total (b(1: ndim)) gt n_elements(space) then begin
           ndim = b(0) -1
           natt = b(ndim+1)
         endif
         if ndim lt 2 then begin
           ier = 1
           print, ' NGD dataset has only 1 spatial dimension'
           return
         endif
         aa = space(0:b(1)-1)
         bb = space(b(1): b(1)+b(2)-1)
         if ndim gt 2 then cc = space(b(1)+b(2): b(1)+b(2)+b(3)-1)
         if ndim gt 3 then dd = space(b(1)+b(2)+b(3): b(1)+b(2)+b(3)+b(4)-1)
         space = 0
         np = b(1: ndim)
         lv = b(b(0)+2)/natt
         ee = image(0:lv-1)
         if natt gt 1 then ff = image(lv  :2*lv-1)
         if natt gt 2 then gg = image(2*lv:3*lv-1)
         if natt gt 3 then hh = image(3*lv:4*lv-1)
         if natt gt 4 then ii = image(4*lv:5*lv-1)
         if natt gt 5 then jj = image(5*lv:6*lv-1)
         image = 0
       endif else begin
         ier = 1
         Print,' PFF dataset type ', pfftyp, ' is not supported by SORT_FIELD'
       endelse
       return
       end
