; ------------------------------------------------------------------------ 
; $Id: box.pro,v 1.3 2007/06/06 20:03:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro box, x1range = x1ran, y1range = y1ran, x2range = x2ran, y2range = y2ran, $
     x1title = x1tit, x2title= x2tit, y1title = y1tit, y2title = y2tit , $
     save = save, _extra= extrastuff
;+
; NAME:
;       BOX
; PURPOSE:
;       Draw a box grid with different labels on all four sides.
;       Scaling will be according to the x1 and y1 values unless save is set.
; 
; CALLING SEQUENCE:
;       BOX
;
; KEYWORD PARAMETERS:
;       X1RANGE  - Xrange for bottom axis.
;       X2RANGE  - Xrange for top axis.
;       Y1RANGE  - Yrange for left axis.
;       Y2RANGE  - Yrange for right axis.
;       X1TITLE  - Xtitle for bottom axis.
;       X2TITLE  - Xtitle for top axis.
;       Y1TITLE  - Ytitle for left axis.
;       Y2TITLE  - Ytitle for right axis.
;       SAVE     - If set, plot parameters will be set to the second set of 
;                  axes. (X2range, Y2range)
;       _extra   - Any valid keywords for the plot command.
;                  (ticklen, margin, etc)
;                  Parameters set by box command are:
;                     NoData = 1
;                     xstyle, ystyle = 9
;                     xmargin, ymargin = !x.margin(0), !y.margin(0)
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 2, 1998
;-
  xmargin= [!x.margin(0), !x.margin(0)]
  ymargin= [!y.margin(0), !y.margin(0)]
  if n_elements(save) eq 0 then save = 0
  plot, x1ran, y1ran, /nodata, xstyle=9, ystyle=9, xtit=x1tit, ytit=y1tit, $
     xmargin = xmargin, ymargin= ymargin, _extra = extrastuff
  axis, /xaxis, xstyle = 1, xtit= x2tit, xrange = x2ran,save = save
  axis, /yaxis, ystyle = 1, ytit= y2tit, yrange = y2ran, save = save
end
