; ------------------------------------------------------------------------ 
; $Id: delwdf.pro,v 1.6 2007/06/06 20:03:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
Pro  delwdf, wdfain, quiet = quiet
;+
;  format: delwdf, WDFA [, /QUIET]
;  This routines clears WDF arrays
;  If WDFA is zero or undefined; all arrays are cleared.
;  Otherwise, all valid WDF arrays specified by WDFA are cleared.
;  example:
;  clear all waveform arrays
;      delwdf
; or   delwdf, 0
;
;  QUIET:  If set, no information is print to the screen
;
; Examples:
;  clear waveform array number 15 and 18
;      delwdf, [15, 18]
;
;  Modified: July 28, 2003, L. P. Mix
;            Bug fix and improve efficiency
;         
;-
;
@com.dat
;
on_error, 1

     if n_elements(initwdf) eq 0 then return
;
     nel =  n_elements (wdfain)
     if nel eq 0 then wdfa = 1+lindgen(maxwdfarray) else wdfa = wdfain
     if nel eq 1 and wdfa[0] eq 0 then wdfa = 1+lindgen(maxwdfarray)
     ok = where (wdfa gt 0 and wdfa le maxwdfarray, nok)
     if nok eq 0 then return
;      the following is CLEAR_WDFARRAY from the util directory.
     wm = wdfa[ok]-1
     oks = where ((wdfnp[wm]) gt 2, nok)
     if nok eq 0 then return
     wm = wm[oks]
     wdfnp(wm)= 0L
     wdfcomment(wm) = ''
     wdffile(wm) = ''
     wdfxlabel(wm) = ''
     wdfylabel(wm) = ''

     for i = 0, nok-1 do $
       handle_value, wdfarraycom[wm[i]], 0.0, /set
     if not keyword_set(quiet) then $
         print, String(nok), ' Waveform Arrays have been Cleared'
     end
