; ------------------------------------------------------------------------ 
; $Id: dirw.pro,v 1.3 2007/06/06 20:03:45 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro dirw, wminin, wmaxin, xsize = xsize, truncate = truncate, $
          reset = reset, page = page, file = kfile , base = tlb
;+
;
; NAME:
;      DIRW
; PURPOSE:
;      Show the WDF arrays in the common area which contain data
; CALLING SEQUENCE:
;      DIRW  [, WMIN] [, WMAX] [, TRUNCATE = truncate] [, Xsize = xsize]
;            [, PAGE = page] [, RESET = reset]
; INPUTS:
;      WMIN   - First waveform array number to be listed or an array of 
;               WDF array numbers 
;         or
;             If a single parameter, WMIN can be a search string.
;             Those arrays containing the string in the dataset comment will
;               be listed. The full set of wild card options is available.
;               See GET_MATCH
;      WMAX   - Last WDF array number
;      TRUNCATE - If set, the dataset comments will be truncated at ';'
;               This is a toggle; If not specified the last value will
;               be used.  Default value =1 -- truncate comments at first ";".
;      FILE   - If set, file will be printed. If file< 0 file name will be on 
;               a new line.   This is a toggle; if not specified the last value
;               will be used. Default value = 0
;      PAGE   - Number of lines on the directory
;      RESET  - If set, a new window will be opened.
;      XSIZE  - Width of the window in pixels (generally not set)
; Outputs:
;      Waveform array parameters in a widget 
; Example:
;      See the contents of all structure arrays
;         DIRW
;      See the structures with E-field in the comment
;         DIRW, 'E-field'
;      See the full contents of structure arrays 1 through 13
;         DIRW, 1, 13
; (or)    DIRW, 1+indgen(13)
;      See all the datasets which begin with v and the associated files
;         DIRW,  '^v', /file

;-
;  Common block
@com.dat
@qscom.dat
;
      common dirsave, dirbase, dirlist, oldpage,  text1, ttruncate_save
;
      common pffcom, initpff, pff_exe
      common dirw_common, base, list, file_save, truncate_save
;
      on_error, 1
;
      if n_elements(initpff) eq 0 then initpff = 0
      if initpff eq 0 then init_pff
;

      if n_elements(qsplotinit) eq 0 then qsinit
      if n_elements(initwdf) eq 0 then wdfinit

      if n_elements (truncate_save) eq 0 then truncate_save =1
      if n_elements (truncate) gt 0 then truncate_save = truncate

      if n_elements (file_save) eq 0 then file_save =0
      if n_elements (kfile) gt 0 then file_save = kfile
      if keyword_set (reset) then base = 0

      lcpar = 0
      npar = n_params()
      if npar eq 0 then begin
        wmin = where (wdfnp gt 0 , nwdf)+1
      endif else if npar eq 1 then begin
        ss = size(wminin)
        if ss (ss(0)+1) eq 7 then begin
          wmin= wminin(0)
          full = 0
          lst = strlen(wmin)
          if lst eq 0  then wmin = '*'
          ok = get_match (wmin, /wdf, nelement=nwdf)
          if ok(0) eq -1 then begin
           print, '% DIRW: No COMMENTS contain the string: ', wmin
           return
          endif
          wmin = temporary(ok)
        endif else begin
          nwdf = n_elements(wminin)
          if nwdf eq 0 then begin
            print, '% DIRW: First parameter is undefined'
            return
          endif else wmin = wminin
        endelse
        if nwdf eq 1 then begin
          b = size(wmin)
          if  b(b(0)+1) gt 3 then begin
            print,'% DIRW: First parameter to DIRW ', $
                 'must be a search string or a structure array number'
            return
          endif
        endif  else begin
          ok = where (wdfnp(wmin-1) ne 0, nwdf)
          if nwdf eq 0 then begin
            print, '% DIRW: Specified waveform arrays are empty'
            return
          endif else wmin = wmin(ok)
        endelse
      endif else begin
        nwdf = min ([n_elements(wminin), n_elements(wmaxin)])
        if nwdf eq 0 then begin
          print, '% DIRW: Parameters are undefined'
          return
        endif else begin
          wmax = wmaxin(0)
          wmin = wminin(0)
        endelse
        wmin = min([wmin, wmax], max = wmax) >1
        if wmax le 0 then wmax = maxwdfarray
        wmax = wmax < maxwdfarray
        ok = where(wdfnp(wmin-1:wmax-1) ne 0, nwdf)
        if nwdf eq 0 then begin
          print, '% DIRW: Specified structure arrays are empty'
          return
        endif else begin
          wmin = lindgen(wmax-wmin+1)+wmin
          wmin = wmin(ok)
        endelse
      endelse
      wmin = long(wmin)
      if file_save ge 0 then m = 1L else m=2L
      line = make_array(nwdf*m> 1L, /string)
      if nwdf eq 0 then line(0) = 'All Waveform Arrays are empty' else begin
        for i = 0, nwdf-1 do begin
          str = wdfcomment(wmin(i)-1)
          if keyword_set(truncate_save) then begin
            ii = strpos(str, ';')
            if ii gt 0 then str = strmid(str, 0, ii)
          endif
          if file_save le 0 then line(i*m) = strtrim(wmin(i), 2) + ' - '+str 
          if file_save lt 0 then line(i*m+1) = '        '+wdffile(wmin(i)-1) $ 
            else if file_save gt 0 then line(i) = $
             strtrim(wmin(i), 2) + ' - '+str+' - '+wdffile(wmin(i)-1)
        endfor
      endelse
      if n_elements (oldpage) eq 0 then oldpage = directorypage
      if n_elements (page) eq 0 then page = oldpage
      page = page >4
;
      if n_elements (base) eq 0 then begin
        base = 0
        list = 0
      endif
      text = 0
      if oldpage ne page then page=-page
;
      if n_elements(xsize) eq 0 then xsize = 300
      widget_directory, base, page, list, line, xsize = xsize, $
        base_title = 'Waveform Array Directory', text1 = text, $
        dirtype=11, value1 = 'WDF #          WDF Array Comment'
      tlb = base
      oldpage = page
      return
      end
