; ------------------------------------------------------------------------ 
; $Id: fftf.pro,v 1.5 2010/09/20 14:10:32 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  FFTF, wdf1, y,  time , double = double ,fast = fast, limit = limit, $
       inverse = inverse
;+
;     format: fftf, wda, y, freq, /double, fast = fast, limit = limit,
;                    inverse = inverse
;     Find the FFT of an wdf array
;     Return the fft as a complex variable, y, and the frequencies in freq
;       
;        or
;
;     Find the real part of an inverse fft of a complex array
;     Return the real part of the inverse in WDA, Y is the complex array
;       and freq should be delta frequency or an array of frequencies.
;       If n_elements (freq) gt 1 then delta freq = freq[1]-freq[0]
;
;    The following are used only if inverse = 0 or undefined
;     If double is set, calculation is double precision
;      FAST     To avoid problems with long FFT's the number of points
;                will be changed to speed the fft.
;                Fast = 0 Do not modify the data
;                Fast = 1 Modify the data by changing the spacing
;                Fast = 2 Modify the data by adding extra points to connect to
;                         the first value in the waveform
;                Default: FAST = 0
;                If FAST lt 0 or FAST gt 2, then FAST = 1
;      LIMIT    Indicates the number of points which may be added
;                This is approximate.
;                If NP = total number of points and NEL is the original number:
;                LIMIT <= 0  Change NP to a power of 2 or 3 times a power of 2.
;                LIMIT [0,1] Change NP such than (NP-NEL)< LIMIT*NEL
;                LIMIT > 1   Change NP such than (NP-NEL)< LIMIT
;                Default: LIMIT = 0.03
;-
@com.dat
on_error, 1

;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print, '% FFTF:  Routine must have 2 parameter'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameter to FFTF must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
;     Check for inverse.
; 
      if keyword_set(inverse) then begin
        ny = n_elements(y)
        nt = n_elements(time)
        if ny eq 0 then begin
          print, '% FFTF:  FFT must be defined'
          return
        endif
        if nt eq 0 then time =0
        yy = float(fft(y, 1, double = double))
        if nt eq 1 then df = time[0] else df = time[1]-time[0]
        if df eq 0 then begin
          print, '% FFTF: Invalid frequency array.  Delta Time =1'
          dt = 1.0
        endif else dt = 1.0/(ny*df)

        put_wdfarray, [0., dt, yy], wdf1, clabel = 'Inverse FFT'
        return
      endif
      np = wdfnp(wdf1-1)
      if np eq 0 then begin
        print, 'No data in the array'
        return
      endif
;
      get_wdfarray, array, wdf1
      if keyword_set (fast) then begin
        if fast lt 0 or fast gt 2 then fast = 1
        if n_elements (limit) eq 0 then limit = 0.03
        get_fast_fft, array, fast, limit
      endif
      np = n_elements(array)-2
      time = findgen(np)
      time = time < (np-time)
;      total = np *array(1)
      time = time/ (np*array[1])
      time (np/2+1:*) =- time (np/2+1:*)
      y  = fft(array(2:*), -1, double = double)
      array = 0
      end
