; ------------------------------------------------------------------------ 
; $Id: fulfnam.pro,v 1.5 2009/02/17 17:11:11 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro FULFNAM, fnamein, fulnam
; FulFnamF is a utility.
;+
;  Return the full file name from input string FNameIn. 
;  
;  Format: FULFNAM, FNameIn, Full_Name
; or 
;  Format:  Full_Name = FULFNAMF( FNameIn)
;    where:
;      FNameIn  - Input Name
;      Full_Name- Output Name
;
;
;    This routine handles the following file specifications:-
;
;    $ddd/file, $(ddd)/file  Where "ddd" is an environment variable
;    ${ddd}/file             pointing to a directory
;
;    $fff, $(fff), ${fff}    Where "fff" is an environment variable
;                            pointing to a file
;
;    $(xxx)yyy ${xxx}yyy     Where "xxx" is an environment variable
;                            and entire string points to a file
;
;    ~/file                  For file specification relative to user's
;                            HOME directory
;
;  Contributed by T. D. Pointon, 2/1/94
;-
;
vers = strupcase(strmid(!version.os, 0, 3))
if (vers eq 'WIN') then return
if n_elements (fnamein) gt 0 then fname = fnamein[0] else return
fulnam = ' '

fnamlen   = strlen(fname)
firstchar = strmid(fname, 0, 1)

; PROCESS ENVIRONMENT VARIABLE

if firstchar eq '$' then begin
  case strmid(fname, 1, 1) of
    '(':  endchar = ')'
    '{':  endchar = '}'
    else: endchar = ' '
  endcase

; Get 'bare' environment variable, and rest of string

  if endchar eq ' ' then begin
    i = strpos (fname, '/')
    if i ge 2 then begin
      envvar = strmid (fname, 1, i-1)
      rest   = strmid (fname, i, fnamlen-i+1)
    endif else begin
      envvar = strmid (fname, 1, fnamlen-1)
      rest   = ' '
    endelse

; Get environment variable enclosed in () or {}, and rest of string

  endif else begin
    i = strpos(fname, endchar)
    if i ge 3 then begin
      envvar = strmid (fname, 2, i-2)
      if i lt fnamlen then begin
        rest = strmid (fname, i+1, fnamlen-i)
      endif else begin
        rest = ' '
      endelse
    endif else begin
      print, '%FULFNAM: Syntax error in file name'
      return
    endelse        
  endelse

; Get equivalence string of env. variable, and build full file name

  if rest eq ' ' then begin
    fulnam = getenv(envvar)
  endif else begin 
    fulnam = getenv(envvar) + rest
  endelse

; PROCESS ~/file or ~file

endif else if firstchar eq '~' then begin

  a = file_info(fname)
  if strlen(a.name) gt 0 then fulnam = a.name else fulnam = fname

endif else begin
  fulnam = fname
endelse
end
