; ------------------------------------------------------------------------ 
; $Id: get_time.pro,v 1.7 2010/09/20 14:16:19 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function  Get_time, wdf, yin , npoints = np
;+
;     this procedure returns the x values of an wdf array at a given y value
;     format time = get_time (wdf, y, npoints = npoints)
;       where wdf is a wdf array number and y is the ordinate value
;       npoints are the number of values found
;       zero is returned if y is outside the range of the wdf array
;-
@com.dat
;
      on_error, 1
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      np = 0L
      if nparams lt 2 then begin
        print,'% GET_TIME: Function must have at least 2 parameters'
        return, 0.0
      endif
;
      ny = n_elements(yin)
      if ny eq 0 then begin
        print, '% GET_TIME: Y-values must be defined'
        return, 0.0
      endif else  y = yin[0]
      if ny gt 1 then print, $
        '% GET_TIME: *** WARNING *** Only first Y-value will be used'

      if n_elements(wdf) eq 0 then begin
        print, '% GET_TIME: WDF array must be defined'
        return, 0.0
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, '% GET_TIME: WDF array must be greater than 0 and </=', maxwdfarray
        return, 0.0
      endif
;
      nel = wdfnp(wdf-1)
      if nel lt 3 then begin
        print, '% GET_TIME: No valid data in the wdf array'
        return, 0.0
      endif
      if y lt wdfmin(wdf-1) or y gt wdfmax(wdf-1) then time=0.0 else begin
        get_wdfarray, array, wdf, /xydata
        nel = wdfnp(wdf-1)
        if n_elements(array) eq nel+nel then begin
          xx = array(0:nel-1)
          yy = array(nel:*)
        endif else begin
          xx = array(0)+findgen(nel)*array(1)
          yy = array(2:*)
        endelse
        ok = where ( (yy(0:nel-2) le y and yy(1:nel-1) gt y) or $
              (yy(0:nel-2) ge y and yy(1:nel-1) lt y), nok)
        okp = ok+1
        if nok eq 0 then time = 0.0 else $
           time = xx(ok) + (y- yy(ok)) *(xx(okp)-xx(ok))/(yy(okp)-yy(ok))
        array=0
        if yy(nel-1) eq y then begin
          nok = nok+1
          if nok eq 1 then time = xx(nel-1) else time =[time, xx(nel-1)]
        endif
        np = nok
      endelse
      return, time
      end
