; ------------------------------------------------------------------------ 
; $Id: get_wdf.pro,v 1.4 2007/06/06 20:03:48 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function get_wdf , name, casesen = casesen, NELEMENTS =nelements
;
;+
;    Return the wdf number(s) of all wdf arrays having a certain string
;      in the the comment.
;    Similar to wdvalid
;
;    format:
;        array_numbers = get_wdf(string)
;        where
;          array_number - an array of integers representing the WDF arrays
;                         with STRING in the wdf comment 
;                         If no matches are found then -1 is returned.
;          String       - search string
;                         See GET_MATCH for options, wild cards, etc.
;    Keywords:
;          CASESEN      - If present and not zero, then search is case
;                         sensitive.  Default is CASESEN=0 (not case sensitive)
;          NELEMENTS    - Number of elements found
;    Example:
;        List all wdfarrays with data.
;           print, get_wdf()
;        List the wdf arrays having 'mocam' in the dataset commment.
;           print, get_wdf('mocam')
;        Plot in an overlay all wdf arrays with 'MOCAM2' in the dataset comment.
;           plo, get_wdf('MOCAM2'), /over
;
;-
    on_error, 1
;
@com.dat
;
    if n_elements(initwdf) eq 0 then wdfinit
;
    if n_elements(name) eq 0 then name = '*'
    if n_elements(casesen) eq 0 then casesen = 0 else $
        casesen = fix(casesen(0))
    str = strtrim(name, 2)
    return, get_match(str, /wdf, casesen= casesen, NELEMENTS =nelements)
    end
