; ------------------------------------------------------------------------ 
; $Id: invertct.pro,v 1.8 2010/09/17 21:20:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro invertct, xydata = xydata, gifdata = gifdata, reset = reset

;+ 
; Name:         invertct
; Format:       invertct [, /RESET] [, /XYDATA] [, /GIFDATA]
;         where: 
;                RESET    If set, reset the invertct flag
;                GIFDATA  If set, the colors between 9 and !d.table_size-1
;                           will be inverted and the foreground and background.
;                XYDATA   if set, will only invert colors between 
;                           9 and !d.table_size-1.  Foreground and 
;                           background are not changed.
;                  NOTE:  If both xydata and gif are set, then the colors 
;                         between 9 and !p.color-1 will not be inverted.
;                  Note: invertct, /xy, /gif
;                        is the same as: swapct
; Purpose:      Reverses the color table
;               Note: Existing plots are modified, but loading a new color 
;                     table will change everything back to the original colors.
;               See swapct.pro to modify the only the foreground
;                   and background colors.
; Format: invertct
; Technique: switches the color table entries between the minimum and the 
;            Maximum ([!p.color, !p.background]) < (!d.table_size -1)
;            Minimum is zero unless xydata is set
;- 
; Loadxyct common block
; common loadxyct_common, loadct_init, max_color ,invertct_flag, $
;     color_maximum, color_minimum
@loadxyct_common.dat
common swap_common, lcswap

on_error, 1

  if n_elements(lcswap) eq 0 then lcswap = 0
  if keyword_set(reset) then begin
    invertct_flag = 0
    loadxyct, /quiet
    return
  endif
  tvlct, r, g, b, /get
  nm = !d.table_size-1
  if nm lt 10 then begin
    print, '% INVERTCT:  Insufficient number of colors', !d.table_size
    return
  endif
;
; NM = number of colors -1
  top= nm-1
  if keyword_set(gifdata) then begin
;    c = r[0] & r[0] = r[nm] & r[nm] = c
;    c = g[0] & g[0] = g[nm] & g[nm] = c
;    c = b[0] & b[0] = b[nm] & b[nm] = c
    if lcswap eq 0 then begin
       temp = !p.color
       !p.color = !p.background
       !p.background = temp
    endif
    invertct_flag = 1
    if keyword_set(xydata) then invertct_flag = -1 else begin
      r(9:top) = reverse(r(9:top))
      g(9:top) = reverse(g(9:top))
      b(9:top) = reverse(b(9:top))
    endelse
    tvlct, r, g, b
    return
  endif 
  if keyword_set(xydata) then mm = 9 else mm = 0
  r(mm:top) = reverse(r(mm:top))
  g(mm:top) = reverse(g(mm:top))
  b(mm:top) = reverse(b(mm:top))
  tvlct, r, g, b
  end
