; ------------------------------------------------------------------------ 
; $Id: lhelp.pro,v 1.7 2011/05/05 18:31:16 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;	LHELP
;
; PURPOSE:
;	Provides a graphical user interface to the user help procedures
;         mk_library_help can be used to generate the help file
;         IDL_HELP_PATH can be used to add user routines to the PFIDL routines
;
; CATEGORY:
;	Help documentation
; FORMAT:
;       LHELP [, /HTML] [, /REMOTE ] [, /SET_default ] [, /SHOW_default]
;             [, /PDF]
;
;       HTML   - If set, an HTML version of help will be used.
;                Default = 0
;       REMOTE - If set, a Netscape browser must be running.
;                Default = 0
;       PDF    - If set a FrameMaker version of help will be displayed using the
;                Acrobat reader.
;                This version will not be current with all commands.
;
;  NOTE: If either or both of the following keywords are set, then help
;        will not be invoked.
;       SET    - If set, the values of HTML and REMOTE if present, will
;                will be set as the default.
;       SHOW   - If set, show the current defaults.
; Example:
;       Display an old style IDL help widget
;         lhelp
;
;       Display the PDF version of help.
;         lhelp, /pdf
;
;       Display a help menu in the current Netscape session.
;         lhelp, /remote, /html
;
; OUTPUTS:
;	None. A widget interface is used to allow reading the help text.
; COMMON BLOCKS:
;	None.
; MODIFICATION HISTORY:
;       Original version May 17, 1995, lpmix
;-
;
   Pro lhelp, routine, html = html, remote = remote, set_default = set_defaut, $
              show_default = show_default, pdf = pdf
;
 common lhelp_common, html_save, remote_save, lhelp_init
on_error, 1
       if n_elements(lhelp_init) eq 0 then begin
         b = get_pfidl_directory()
         sep =  path_sep()
         search = path_sep(/search_path)
         if !version.os_family eq 'unix' or (!version.os_family eq 'Windows' $
           and strpos(!version.release, 5.5) lt 0 ) then $
           !help_path= b+'help'+sep+'help_files'+search+!help_path $
         else !help_path= b+'help'+sep+'win_help_files'+search+!help_path
         lhelp_init = 1
         html_save = 0
         remote_save = 0
       endif
       if keyword_set(set_default) then begin
         if n_elements(remote) gt 0 then remote_save = remote(0)
         if n_elements(html) gt 0 then html_save = html(0)
       endif

       if keyword_set(show_default) then begin
         print, ' HTML   = ' , strtrim(html_save, 2)
         print, ' REMOTE = ' , strtrim(remote_save, 2)
       endif
       if keyword_set(set_default) or keyword_set(show_default) then return
       if keyword_set (pdf) then begin
         b  = get_pfidl_directory()
         fam = strmid(strupcase(!version.os_family), 0, 3)
         if fam eq 'UNI' then $
           spawn, 'acroread '+ b+'help/help_files/pfidl_1d.pdf &'  $
             else $
           spawn, b+'help\help_files\pfidl_1d.pdf', /hide, /nowait
         return
       endif
       if n_elements (html) eq 0 then html = html_save
       if n_elements (remote) eq 0 then remote = remote_save
;
       if keyword_set(html) then begin
         b =  get_pfidl_directory()
         sep = path_sep()
         html_file= b+'help'+sep + 'help_files'+sep+'idl_html_help.html'
         if strupcase(strmid(!version.os, 0, 3)) eq 'WIN' then begin
           spawn, '"c:\Program Files\Internet Explorer\Iexplore"  '+ $
             html_file, /nowait, /noshell
           return
         endif

         a = getenv('IDL_HTML_HELP')

         if strlen(a) lt 2 then begin
           a = html_file
         endif
         if keyword_set(remote) then $
           spawn, 'firefox  -remote ''openFile( ' + a + ')''  &' $
            else spawn, '/usr/bin/firefox -mono ' + a + ' &'
           return
       endif
       if n_elements (routine) eq 0 then begin
         widget_olh, 'pfidl'
       endif else widget_olh, routine
       return
       end
