; ------------------------------------------------------------------------ 
; $Id: limits.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  limits, wdf, xrange = xrange, yrange = yrange, keep = keep, quiet = quiet
;+
;     This procedure returns the range of abscissa and ordinate values
;
;     Format: LIMITS, WDF [, /KEEP] [, XRANGE = xrange] [, YRANGE = yrange]
;     where:
;       WDF    - WDF array number or array of numbers
;                This array may be modified if it contains invalid WDF arrays.
;       KEEP   - If set, WDF will not be modified and xrange and yrange 
;                will contain (0,0) for the range of invalid WDF.
;                Default is KEEP = 0
;                WDF will return only with valid WDF arrays or -1 if 
;                none of the arrays are valid.
;       XRANGE - XRANGE[2, *] has the minimum and maximum abscissa values
;                for each wdf array
;       YRANGE - YRANGE[2, *] has the minimum and maximum ordinate values
;                for each wdf array
;       QUIET  - Do not send messages if datasets are not valid.
;       
;
;-
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: LIMITS, WDF [, /KEEP] 
        print,'          [, XRANGE = xrange] [, YRANGE = yrange]
        return
      endif
;
      nelwdf = n_elements(wdf)
      xrange = make_array(2, nelwdf)
      yrange = make_array(2, nelwdf)
      if nelwdf eq 0 then begin
        print, '% LIMITS: WDF must be defined
        return
      endif
;  
      ok = where((wdf gt 0) and (wdf le maxwdfarray), nok, comp = notok, $
         ncomp = nnotok)
      if nnotok gt 0 and not keyword_set(quiet) then print, $
        '% LIMITS: Some WDF arrays are not valid: ', fix(wdf[notok] )
      if nok eq 0 then begin
        print, '% LIMITS: No valid WDF arrays'
        if keyword_set (keep) then return
        wdf = -1
        xrange = [0,0]
        yrange = [0,0]
        return
      endif
      if not keyword_set(keep) and nnotok gt 0 then begin
        wdf = wdf[ok]
        ok = indgen(nok)
        xrange = make_array(2, nok)
        yrange = make_array(2, nok)
      endif
      oknp = where(wdfnp[wdf[ok]-1] gt 2, nok, comp= notok, ncomp=nnotok)
      if nnotok gt 0 and not keyword_set(quiet) then print, $
        '% LIMITS: Some WDF arrays do not have valid data: ', $
          fix(wdf[ok[notok]] )
      if nok eq 0 then begin
        print, '% LIMITS: No valid WDF arrays with data'
        if keyword_set (keep) then return
        wdf = -1
        xrange = [0,0]
        yrange = [0,0]
        return
      endif
      if not keyword_set(keep) and nnotok gt 0 then begin
        wdf = wdf[ok[oknp]]
        ok = indgen(nok)
        xrange = make_array(2, nok)
        yrange = make_array(2, nok)
      endif else ok = ok[oknp]


      for i = 0L, nok-1 do $
        xrange[*, ok[i]] = [wdfminx(wdf[ok[i]]-1),wdfmaxx(wdf[ok[i]]-1)]
      for i = 0L, nok-1 do $
        yrange[*, ok[i]] = [wdfmin(wdf[ok[i]]-1),wdfmax(wdf[ok[i]]-1)]

      return
      end
