; ------------------------------------------------------------------------ 
; $Id: lprint.pro,v 1.3 2007/06/06 20:03:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro lprint, file, destination = destination, nosave = nosave, $
        color= color, filein = filein
;+
;  Procedure to send a postscript file to a printer
;    (LPRINT = Local Print command)
;
;  format: LPRINT [, file] [, DESTINATION = destination] [, /NOSAVE]
;                  [, FILEIN = filein] [, COLOR = color] 
;  where:
;    file   - file to be printed - If file is specified, then that value
;             will become the default in future calls to LPRINT
;             To return to the default value, use ' ' for the file
;             DEFAULT = default_printfile (SEE "SET_PRINTER")
;             Note: IF file is specified, then keyword file is ignored.
;    DESTINATION - site specific
;             If DESTINATION is specified, COLOR is ignored.
;             If DESTINATION is specified, that value will be used on all
;               future calls to LPRINT unless NOSAVE is set. 
;             To return to the default value of the destination use D= ' '
;             Use SET_PRINTER, /show to see current printer defaults.
;    NOSAVE - If DESTINATION is specified and NOSAVE is set, value will not
;             be saved. 
;             If FILE is specified and NOSAVE is set, value will not
;             be saved.
;    COLOR  - if present and not zero, indicates the default 
;             color printer
;             This parameter is ignored if DESTINATION is specified
;             DEFAULT is COLOR = 0
;    FILEIN - Alternate method of specifying file.
;
;
;  Examples:
;    Print idl.ps on the default printer
;      lprint
;
;    Print /users/lpuser/idl/junk.ps on the default printer
;      lprint, '/users/lpuser/idl/junk.ps'
;    NOTE: to print the same file again use the command: lprint
;
;    Print idl.ps on the default color printer
;      lprint, /c  
;    or if all prints are to go to tek_paper use
;      lprint, d = 'tek_paper'
;    Print idl.ps to laser4si but send all future plots to current values.
;      lprint, d = 'laser4si', /nosave
;-
; note for the pc spawn, cmd
; where cmd =string(filename, format='("copy ', a, " prn:")') 
;
on_error, 1
common pffcom, initpff, pff_exe
common lprintdefault, bw_default, color_default
@lprintsav.com
;
 if n_elements (lpinit) eq 0 then set_printer
;
; get the file - if unspecified use the saved file or the default

  if n_elements(file) eq 0 then begin
    if n_elements(filein) gt 0 then file = filein
  endif

  if n_elements(file) gt 0 then begin
    file = strtrim(file[0], 2)
    if strlen(file) gt 0 and not keyword_set (nosave) then filesave = file
    if strlen(file) eq 0 then junk= temporary(filesave)
  endif else file =''
  
  if strlen (file) eq 0 then begin
    if n_elements (filesave) gt 0 then file = filesave  $
       else file = print_file_last
  endif

; get the destination
  if strlen(destsave) eq 0 then begin
    if keyword_set(color) then dest = color_default else $
      dest = bw_default
  endif else dest = destsave

  osver = strupcase(strmid(!version.os, 0, 3))

  if osver eq 'WIN' then begin
    spawn, 'copy '+file + ' '+dest
    return
  endif

; enter '   ' to reset the destinations to bw_default and color_default
  if n_elements(destination) ne 0 then begin
    dest = strtrim(destination, 2)
    if strlen(dest) gt 0 then begin
      if not keyword_set(nosave) then destsave = dest
      spawn, 'lp -d'+dest+'  '+ file
      return
    endif else destsave = dest
  endif
; debug
; print,  'lp -d'+dest+'  '+ file
  spawn, 'lp -d'+dest+'  '+ file
  end
