; ------------------------------------------------------------------------ 
; $Id: putx.pro,v 1.4 2008/07/23 14:52:00 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Putx, x, wdf
;
;+
;  Store an array of abscissas (x-values) into a WDF array.
;
;  Format: PUTX, xarray,  wd1
;  where:
;    xarray - Values to be use as Abscissa values for WD1
;             X-array must have the same number of values as wd1.
;    NOTE:  Regardless of the X array, the data will be changed to XY data.
;           If the data is uniform and you are only changing the initial point
;           of the delta, use CHA.
;    wd1 - WDF array number
;  Example:
;    Put the IDL array time into WDF array 4
;      putx, time, 4
;-
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 2 then begin
        print,' PUTX must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf) eq 0 or n_elements(x) eq 0 then begin
        print, 'Parameters to PUTX must be defined'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      nely = n_elements(x)
      nel = wdfnp(wdf-1)
      if nel eq 0 then begin
        print, 'No valid data in the wdf array'
        return
      endif
      if nely ne nel then begin
        print, 'X-Array must have the same number of elements as Y array'
        return
      endif

      get_wdfarray, array1, wdf, /no_copy, /xydata
      if n_elements(array1) eq nel+nel then array1[0:nel-1] = x else $
        array1= [x, array1[2:nel+1]]
      put_wdfarray, array1, wdf, /no_copy, /xydata
      return
      end
