; ------------------------------------------------------------------------ 
; $Id: puty.pro,v 1.3 2007/06/06 20:03:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Puty, y, wdf
;+
;  Store an array of ordinates (y-values)  into a WDF array.
;  Note:  The array must have the same number of elements as the original array.
;  Note:   PUTY can be used with GETY to perform IDL functions on an array. 
;
;  Format: PUTY, yarray,  wd1
;  where:
;    yarray - array to be stored as ordinates in a WDF array
;    wd1 - WDF array number
;  Examples:
;    Put the IDL array  voltage into WDF array 4
;      puty, voltage, 4
;    Calculate the square root of the contents of array 4
;      puty, sqrt ( getyf(4) ), 4
;-
;     This routine will replace and wdf array
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: PUTY, YArray, WDF'
        return
      endif
      if nparams lt 2 then begin
        print,' PUTY must have at least 2 parameters'
        return
      endif
;
      if n_elements(wdf) eq 0 or n_elements(y) eq 0 then begin
        print, 'Parameters to PUTY must be defined'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      nely = n_elements(y)
      nel = wdfnp(wdf-1)
      if nel eq 0 then begin
        print, 'No valid data in the wdf array'
        return
      endif
      if nely ne nel then begin
        print, 'Array must have the same number of elements'
        return
      endif
      y = reform (y)
      get_wdfarray, array1, wdf, /no_copy, /xydata
      if n_elements(array1) eq nel+2 then begin
        xydata = 0
        array1(2:nel+1) = y 
      endif else begin
        array1(nel:nel+nel-1) = y
        xydata = 1
      endelse
      put_wdfarray, array1, wdf, /no_copy, xydata= xydata
      return
      end
