; ------------------------------------------------------------------------ 
; $Id: rainbow.pro,v 1.3 2007/06/06 20:03:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro rainbow, ncolors, invert_ct = invert_ct, $
     set_spectrum = set_spectrum, get_spectrum = get_spectrum, $
     red= red, blue = blue, green = green, RESET_SPECTRUM = RESET_spectrum
;+
;    loads a RAINBOW specturm of colors for an x,y plot starting in color 10
;
; format: RAINBOW, NCOLORS, /SET_SPECTRUM, /GET_SPECTRUM, /RESET_SPECTRUM
;            RED = red, BLUE = blue, GREEN = green, /INVERT_CT
;
;  where: 
;      NCOLORS      - Number of colors required
;                     If Red, Green and Blue have NC values then the colors
;                     given by:
;                     val = round(nc*findgen(ncolors)/(ncolors-1))
;                     are stored in color table values 10, 11, ..., (9+ncolors)
;      SET_SPECTRUM - If set, reset the values of red, green and blue for the 
;                     spectrum.  NCOLORS is optional.  RED, GREEN and Blue are
;                     arrays with the same number of elements.  Values must be
;                     in the range of [0, 255] 
;      GET_SPECTRUM - If set, return the values of red, green and blue for the 
;                     current spectrum.  NCOLORS is optional.
;      RESET_SPECTRUM - If set, reset the values of red, green and blue for
;                     the current spectrum to default values.
;                     NCOLORS is optional.
;      RED, GREEN, BLUE  - Current spectrum color values.
;      INVERT_CT    - If set, reverse the order of R, G, and B
; 
;      Written 1/18/00 - lpm
;-
@loadxyct_common.dat
common SPECTRUM_COMMON, nsp, rsp, gsp, bsp
;     
on_error, 1
      neln = n_elements(ncolors)
      lcquit = 0
      if keyword_set(set_spectrum) then begin
        nr = n_elements(red)
        ng = n_elements(green)
        nb = n_elements(blue)
        mx = max([nr, ng, nb], min = m)
        if (mx - m) gt 0 or m le 0 then begin
  print, '% SPECTRUM: R, G, B arrays must have the same number of elements'
          return
        endif
        nsp = mx
        rsp = byte(round((red > 0)< 255))
        gsp = byte(round((green > 0)< 255))
        bsp = byte(round((blue > 0)< 255))
        if neln eq 0 then lcquit = 1
      endif
      if n_elements(rsp) eq 0  or keyword_set(reset_spectrum) then begin
        loadxyct, /quiet, /freeze
        nsp = 256
        rsp = make_array(nsp, /byte, value =0B)
        gsp = make_array(nsp, /byte, value =0B)
        bsp = make_array(nsp, /byte, value =0B)
        rsp[0:85]=255B
        rsp[85:170]= byte(round(255*(1.0-findgen(86)/85.)))
        gsp[0:85]= byte(round(255*(findgen(86)/85.)))
        gsp[85:170] = 255B
        gsp[170:255]= byte(round(255*(1.0-findgen(86)/85.)))
        bsp[170:255]= byte(round(255*(findgen(86)/85.)))
        rsp = reverse(rsp)
        gsp = reverse(gsp)
        bsp = reverse(bsp)
        if keyword_set(reset_spectrum) and neln eq 0 then lcquit = 1
      endif
      if keyword_set (get_spectrum) then begin
        red = rsp
        green = gsp
        blue = bsp
        if neln eq 0 then lcquit = 1
      endif
      if keyword_set(invert_ct) then begin
        rsp = reverse(rsp)
        gsp = reverse(gsp)
        bsp = reverse(bsp)
        if neln eq 0 then lcquit = 1
      endif
      if lcquit ne 0 then return
      if neln eq 0 then begin
        print, '% SPECTRUM: Number of color values must be provided.
        return
      endif
      ncol = ncolors[0] < 240
      if ncol le 0 then begin
        print, '% SPECTRUM: Number of color values must be greater than 0
        return
      endif
      if ncol eq 1 then i = 0 else begin
        konst = (nsp-1.)/(ncol-1)
        i = round(konst *findgen(ncol))
      endelse

      tvlct, rsp[i], gsp[i], bsp[i], 10
      end
