; ------------------------------------------------------------------------ 
; $Id: readic.pro,v 1.3 2007/06/06 20:03:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readic, fname, WDF, np = np, narray=narray 
;
;+
;    Read an ascii file into wdf arrays from a Inbedded Controler
;    Format:  readic, fname, WDF,narray = narray,np = np
;    where: 
;      fname   - the file name or a unit number
;                enter 0 to use pickfile
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of 
;                narray values
;      narray  - the number of arrays in the file 
;      np      - Returned with the number of points read in each array
;                
;      If file contains labels, they are stored in the dataset comment
;    Example:
;    Read waveform data from a file selected by pickfile and store it in
;    WDF array 10
;       READIC, 0, 10
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READNIF, FNAME_UNIT, WDF [, NP = np]'
        print, '              [, NARRAY = narray]'
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, ' File name must be specified by name or unit number'
        return
      endif
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = ''
      if n_elements(wdf) eq 0 then begin
        print,'% READIC: WDF array value must be provided'
        if lcfile eq 1 then  free_lun, unit
        return
      end
      wdfar = wdf(*,0,0,0)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then  free_lun, unit
        return
      endif
      str = ''
      while strlen(strtrim(str, 2)) eq 0 do readf, unit, str
      clab = strtrim(str, 2)
      readf, unit, str
;     while (strpos(str, '/') lt 0) do readf, unit, str
;     date
      clab = ' - '+clab+';;DATETIME='+strtrim(str, 2) + ' ;;'
      readf, unit, str
      while (strpos (str, 'Channel Label')) lt 0 do readf, unit, str
      readf, unit, str
      parser, str, clabel, np, /stringarray, delimiter =9b
      narray = np
      readf, unit, str
      while (strpos(str, 'Channel units') lt 0) do readf, unit, str
      readf, unit, str
      parser, str, ylabel, npy, /stringarray, delimiter =9b
      if np ne npy then Begin
        print, '% READIC: Number of Labels and units are different:', np, npy
        return
      endif
      readf, unit, str
      while(strpos(str, 'Attenuators') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, att, np,  delimiter=9b
      readf, unit, str
      while(strpos(str, 'Gage Factor') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, gag, np,  delimiter=9b
      att = att*gag
      readf, unit, str
      while(strpos(str, 'T0') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, t0, np,  delimiter=9b
      readf, unit, str
      while(strpos(str, 'dt') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, dt, np,  delimiter=9b
      readf, unit, str
      while(strpos(str, 'Offset in ADC') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, offset, np,  delimiter=9b
      readf, unit, str
      while(strpos(str, 'ADC SF') lt 0 ) do readf, unit, str
      readf, unit, str
      parser, str, scale, np,  delimiter=9b
      scale = att*scale
      nel = n_elements(wdfar)
      if nel lt narray then begin
        wdfar = [wdfar, wdfar(nel-1)+1+indgen(narray-nel)]
        if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          print, wdfar
        if lcfile eq 1 then  free_lun, unit
          return
        endif 
      endif
      nn = 8192L
      i  = 0L
      a = make_array(narray, nn)
      b = make_array(narray)
      while (not eof(unit)) do begin
          readf,unit,b
          if i eq nn then begin
            aa  =  make_array(narray, nn+nn)
            aa(*, 0:nn-1) = a
            nn = nn + nn
            a = temporary(aa)
          endif
          a(*,i) = b
          i = i+1
      endwhile
      if i eq 0 then begin
        print,  '% READIC: NO data read.'
        return
      endif
      a = a(*, 0:i-1)
      np=i
      dt = dt *1e-6
      t0 = t0 * 1e-6
      if lcfile eq 1 then for i = 0, narray-1 do $
        put_wdfarray,[t0(i), dt(i), reform(a(i,*)-offset(i))*scale(i)] , $
           wdfar(i), clab = clabel(i) + clab, $
           file = fname , xlab='Time(sec)', ylab= ylabel(i) $
      else for i = 0, narray-1 do $
        put_wdfarray,[t0(i), dt(i), reform(a(i,*)-offset(i))*scale(i)], $ 
          wdfar(i), clab = clabel(i)+clab, xlab = 'Time(sec)', ylab = ylabel(i)
      if lcfile eq 1 then  free_lun, unit
      return
    end
