; ------------------------------------------------------------------------ 
; $Id: readnif.pro,v 1.3 2007/06/06 20:03:51 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readnif, fname, WDF, np = np, narray=narray 
;
;+
;    Read an ascii file into wdf arrays from a hp54111D digitizer
;    format is:  readnif, fname, WDF,narray = narray,np = np
;    where 
;      fname   - the file name or a unit number
;                enter 0 to use pickfile
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of 
;                narray values
;      narray  - the number of arrays in the file 
;      np      - Returned with the number of points read in each array
;                
;      If file contains labels, they are stored in the dataset comment
;    Example:
;    Read waveform data from a file selected by pickfile and store it in
;    WDF array 10
;       READNIF, 0, 10
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READNIF, FNAME_UNIT, WDF [, NP = np]'
        print, '              [, NARRAY = narray]'
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, ' File name must be specified by name or unit number'
        return
      endif
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = ''
      if n_elements(wdf) eq 0 then begin
        print,'WDF array value must be provided'
        if lcfile eq 1 then  free_lun, unit
        return
      end
      wdfar = wdf(*,0,0,0)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then  free_lun, unit
        return
      endif
      str = ''
      while strlen(strtrim(str, 2)) eq 0 do readf, unit, str
      clab = strtrim(str, 2)
      readf, unit, str
      while (strpos(str, '/') lt 0) do readf, unit, str
;     date
      clab = ' - '+clab+';;DATETIME='+strtrim(str, 2)
      readf, unit, str
;     time
      str = strtrim(str, 2)
      clab = clab+'; '+strtrim(str, 2)
;     Determine if HP or DSP
      readf, unit, str
      while (strpos (str, 'gpib')) lt 0 do readf, unit, str
    if strmid(str, 0, 7) eq 'HP54111' then begin
;     HP data is always 2 channels
      narray = 2

      readf, unit, str
      while (strpos(str, 'Number of Data Points') lt 0) and $
        (strpos(str, 'Channel Labels:')lt 0) do readf, unit, str
      if (strpos(str, 'Channel Labels:')ge 0) then begin
        str  = strmid(str, 15 )
        parser, str, label, np, /stringarray, delimiter =9b
        if np lt 2   then begin
          clabels = make_array(/str, 2)
          if np gt 0 then clabels (0:np-1) = label
          for i = np, 1 do clabels(i) = 'Ch.'+strtrim(i+1,2)
        endif else  clabels = temporary(label)
        for i = 0, 1 do clabels(i) = strtrim(clabels(i),2)+clab
        readf, unit, str
        while(strpos(str, 'Channel Units:') lt 0 ) do readf, unit, str
        str = strmid(str, 14)
        parser, str, label, np, /stringarray, delimiter=9b
        if np lt narray  then begin
          ylabel = make_array(/str, narray)
          if np gt 0 then ylabel (0:np-1) = strtrim(label,2)
          for i = np, 1 do ylabel(i) = 'Amplitude'
        endif else ylabel = strtrim(label, 2)
        readf, unit, str
        while(strpos(str, 'External Attenuators') lt 0 ) do readf, unit, str
        str = strmid(str, 21)
        parser, str, label, np,  delimiter=9b
        np = np < narray
        scale = make_array( narray, value = 1.0)
        if np gt 0 then scale (0:np-1) = label(0:np-1)
        readf, unit, str
        while(strpos(str, 'Gage Factors') lt 0 ) do readf, unit, str
        str = strmid(str, 13)
        parser, str, label, np,  delimiter=9b
        np = np < narray
        if np gt 0 then scale (0:np-1) = scale(0:np-1)*label(0:np-1)
      endif else begin
        ylabel = make_array(2, value = 'Amplitude')
        clabels(0) = 'Ch.1'+clab
        clabels(1) = 'Ch.2'+clab
        scale = make_array(2, value = 1.0)
      endelse
      while (strpos(str, 'Number') lt 0) do readf, unit, str
      n = strpos(str, '=')
      np = strmid(str, n+1, 10)
      np = fix(np)

      readf, unit, str
      str = strtrim(str, 2)
      if strpos(str, 't0') lt 0 then begin
        print, ' Invalid Header, t0', str
        if lcfile eq 1 then  free_lun, unit
        return
      endif
      n = strpos(str, '=')
      t0 = strmid(str, n+1, 20)
      t0 = float(t0)
      readf, unit, str

      str = strtrim(str, 2)
      if strpos(str, 'Delta') lt 0 then begin
        print, ' Invalid Header, Delta', str
        if lcfile eq 1 then  free_lun, unit
        return
      endif
      n = strpos(str, '=')
      dt = strmid(str, n+1, 20)
      dt = float(dt)
      a = make_array(narray, np)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then  free_lun, unit
        return
      endif 
      nel = n_elements(wdfar)
      if nel lt narray then begin
        wdfar = [wdfar, wdfar(nel-1)+1+indgen(narray-nel)]
        if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          print, wdfar
        if lcfile eq 1 then  free_lun, unit
          return
        endif 
      endif
      readf, unit, a
      if lcfile eq 1 then for i = 0, narray-1 do $
         put_wdfarray,[t0, dt, reform(a(i,*))*scale(i)]  ,wdfar(i), $
          clab = clabels(i),  file = fname, xlab='Time(sec)', ylab= ylabel(i) $
      else for i = 0, narray-1 do $
        put_wdfarray,[t0, dt, reform(a(i,*))*scale(i)]  ,wdfar(i), $
         clab = clabels(i), xlab='Time(sec)', ylab= ylabel(i)
         if lcfile eq 1 then  free_lun, unit
      return
    endif
;     Get number of channels
      readf, unit, str
      while strpos(str, 'Channel') lt 0 do readf, unit, str
      n = strpos(str, '=')
      first = fix (strmid(str, n+1, 6))
      n = strpos(str, '=', n+6)
      last = fix (strmid(str, n+1, 6))
      narray = last-first+1

      readf, unit, str
      while ((strpos(str, 'X0') lt 0) and $
         (strpos(str, 'Channel Definition Labels') lt 0 )) do readf, unit, str
      if strpos(str, 'Labels') gt 0 then begin
;       New version with labels
        readf, unit, str 
        parser, str, label,np, /stringarray, delimiter = 9B
        if np lt narray then begin
          clabels = make_array(/str, narray)
          if np gt 0 then clabels (0:np-1) = label
          for i = np, narray-1 do clabels(i) = 'Ch.'+strtrim(i,2)
        endif else clabels = temporary(label)
        for i = 0, narray-1 do clabels(i) = strtrim(clabels(i),2)+clab
        readf, unit, str
        while(strpos(str, 'Channel Units') lt 0 ) do readf, unit, str
        readf, unit, str
        parser, str, label, np, /stringarray, delimiter=9b
        if np lt narray then begin
          ylabel = make_array(/str, narray)
          if np gt 0 then ylabel (0:np-1) = strtrim(label,2)
          for i = np, narray-1 do ylabel(i) = 'Amplitude'
        endif else ylabel = strtrim(label, 2)
        readf, unit, str
        while(strpos(str, 'External Attenuators') lt 0 ) do readf, unit, str
        readf, unit, str
        parser, str, label, np,  delimiter=9b
        np = np < narray
        scale = make_array( narray, value = 1.0)
        scale (0:np-1) = label(0:np-1)
        readf, unit, str
        while(strpos(str, 'Gage Factor') lt 0 ) do readf, unit, str
        readf, unit, str
        parser, str, label, np,  delimiter=9b
        np = np < narray
        scale (0:np-1) = scale(0:np-1)*label(0:np-1)
      endif else begin
        clabels = make_array(/str, narray)
        for i = 0, narray-1 do clabels(i) = 'Ch.'+strtrim(i,2)+clab
        scale = make_array(narray, value = 1.0)
        ylabel = make_array(narray, value = 'Amplitude')
      endelse
      while(strpos(str, 'X0') lt 0 ) do readf, unit, str

      n = strpos(str, '=')

      t0 = float(strmid(str, n+1, 20))

      readf, unit, str
      while (strpos(str, 'Delta') lt 0) do readf, unit, str
      n = strpos(str, '=')
      dt = float(strmid(str, n+1, 10))

      readf, unit, str
      while (strpos(str, 'Number') lt 0) do readf, unit, str
      n = strpos(str, '=')
      np = fix(strmid(str, n+1, 20))

      a = make_array(narray, np)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        if lcfile eq 1 then  free_lun, unit
        return
      endif 
      nel = n_elements(wdfar)
      if nel lt narray then begin
        wdfar = [wdfar, wdfar(nel-1)+1+indgen(narray-nel)]
        if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          print, wdfar
          if lcfile eq 1 then  free_lun, unit
          return
        endif 
      endif
      readf, unit, a
      if lcfile eq 1 then for i = 0, narray-1 do $
        put_wdfarray,[t0, dt, reform(a(i,*))*scale(i)]  ,wdfar(i), $
          clab = clabels(i), $
           file = fname , xlab='Time(sec)', ylab= ylabel(i) $
      else for i = 0, narray-1 do $
        put_wdfarray,[t0, dt, reform(a(i,*))*scale(i)]  ,wdfar(i), $
          clab = clabels(i), xlab = 'Time(sec)', ylab = ylabel(i)
      if lcfile eq 1 then  free_lun, unit
      return
    end
