; ------------------------------------------------------------------------ 
; $Id: readpop.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro readpop, fname, str
;
;+
;    Name: READPOP
; 
;    read a ASCII POP  file
;    format is:  readpop, fname, str
;    where 
;      fname   - the file name or a unit number
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used 
;                  to determine a file
;      str     - Structure Array Number to store the image
;                Default is Maxstructure
;    Assumes file of the form:
;$-----------------------------------------$
;$ time=    7.40000 sh.
;$ (KIMAX,LIMAX) = (  60, 152) ;  (R,Z) direction.
;$ Response used is sfcamrsp taken from file sfcamrsp        .
;inputexp/vlb/gs  7.40/ 152/  60
;$  LI=   1,       (1<=KI<=KIMAX)
; 0.00000E+00/ 1.04872E-97/ 1.04872E-97/ 1.04872E-97/ 1.04872E-97/
; 1.04872E-97/ 1.04872E-97/ 1.04872E-97/ 1.04872E-97/ 1.04872E-97/
; 
;    Stores the data in str
;
;    Example:
;    Read an array from unit 2 and store in structure x.
;       READPOP, 2, x
;
;    Read the image from file 'IMMG74' and store in structure 3
;       READPOP, 'IMMG74', 3
;
;    Read from a file determined by PICKFILE and store in structure 4
;       READPOP, 0, 4
;-
;
     on_error, 1
;
;  Common blocks
;
@qscom.dat
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
;
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: READPOP, FNAME_UNIT, STR'
        return
      endif
      if nparams lt 2 then Message, 'Structure array must be specified'
      str = get_save_str(str, maxstructure)
      str = str(0)
      if n_elements (fname) eq 0 then message, $
        ' File must be specified by name or unit number, use 0 for PICKFILE'
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun
        lcfile = 1
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
            return
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      junkstr = ' ' 
      readf, unit, junkstr
      bjunk = byte(strtrim(junkstr, 2))
      nel = n_elements(bjunk)
      ok = where (bjunk eq 45B, nok)
      if nok eq nel-2 then clabel = 'Image' else $
          clabel = strtrim(bjunk(1:nel-2), 2)
      clabel = clabel +' - '
      readf, unit, junkstr
      bjunk = byte(strtrim(junkstr, 2))
      nel = n_elements(bjunk)
      ok = where (bjunk eq 45B, nok)
      if nok ne nel-2 then clabel = clabel + '; ' + strtrim(bjunk(1:nel-2), 2)
      readf, unit, junkstr
      ok = strpos(junkstr, 'KIMAX')
      if ok gt 0 then begin
        ok1 = strpos(junkstr, '(',ok)
        ok2 = strpos(junkstr, ')', ok1)
        if ok1 gt 0 and ok2 gt 0 then begin
          ind = strmid(junkstr, ok1+1, ok2-ok1-1)
          reads, ind, nx, ny
        endif
      endif else message, ' Contact Paul Mix, 845-7492 or lpmix@sandia.gov'
      array = make_array(nx, ny)
      ok = strpos(junkstr, ';', ok2) 
      if ok gt 0 then clabel = clabel + '; ' + strtrim(strmid(junkstr, ok+1, 99))
      readf, unit, junkstr
      bjunk = byte(strtrim(junkstr, 2))
      nel = n_elements(bjunk)
      ok = where (bjunk eq 45B, nok)
      if nok ne nel-2 then clabel = clabel + '; ' + strtrim(bjunk(1:nel-2), 2)
      readf, unit, junkstr
      bjunk = byte(strtrim(junkstr, 2))
      if bjunk(0) ne 36B then clabel = clabel + '; ' + strtrim(junkstr, 2)
      lcstop = 0
      l = 0
      while ((not eof(unit)) and l lt ny and lcstop eq 0) do begin
        readf, unit, junkstr
        ok = strpos(junkstr, 'LI')
        if ok gt 0 then lcstop = 0 else lcstop = 1
        k = 0
        while((not eof(unit)) and k lt nx and lcstop eq 0) do begin
          readf,unit,junkstr
          ok = strpos(junkstr, 'end')
          if ok gt 0 then begin
            lcstop = 1
            junkstr = strmid(junkstr, 0, ok(0))
          endif
          parser, junkstr, arr, np, ignore = '/'
          array(k:k+np-1, l) = arr
          k = k+np
        endwhile
        l = l+1
      endwhile
      if lcfile eq 1 then  free_lun, unit
      if l eq 0 then begin
        np = 0
        print, ' Error reading file -- No data found'
        return
      endif
      if lcfile eq 1 then arr2str, str, array, clab = clabel, file = fname $
        else arr2str, str, array, clab = clabel
      return
      end
