; ------------------------------------------------------------------------ 
; $Id: reresp.pro,v 1.3 2007/06/06 20:03:52 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro reresp, fname, WDF, np = np , npatch = npatch, nfreq = nfreq
;
;+
;    read an ascii frequency response file into wdf arrays
;    Store the amplitude and the complex values.
;    format is:  reresp, fname, WDF, np = np , npatch = npatch, nfreq = nfreq
;    where
;      fname   - the file name or a unit numberr
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                  if unit is equal to zero then pickfile will be used
;                  to determine a file
;      WDF     - First WDF array or an array of NARRAY WDF arrays
;                WDF must be either a single value or an array of values
;                If no value is provided, WDF is the first unused array number
;                or 1 if all arrays are full.
;                Additional patches are stored in successive arrays.
;      np      - Number of points which are read
;      Npatch  - If number of patches is greater than 512, this must be se.
;                to a number larger than the number of patches.
;      Nfreq   - If more than 512 frequencies, this must be set to a number
;                larger than the number of frequencies.
;    Example:
;    Read an array from unit 2 and store in wdf array 12
;       RERESP, 2,12
;    Read 3 patches from file, 'testdata', into WDF arrays 3, 6, 9.
;       RERESP, 'testdata', 3, [3,6,9]
;-
;
     on_error, 1
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit
;    check format
      nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: RERESP, FNAME_UNIT, WDF [, NP = np]'
        return
      endif
      if n_elements(fname) eq 0 then begin
        print, '% RERESP: File name must be specified by name or unit number'
        return
      endif
      np = 0L
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        lcfile = 1
        openr,unit,fname, /get_lun
      endif else begin
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
          endif
          lcfile = 1
          openr,unit,fname, /get_lun
        endif else lcfile = 0
      endelse
      if lcfile eq 1 then filename = fname else filename = ''
      wdf = get_save_array(wdf, 1)
      wdfar = wdf(*,0,0,0)
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        print, 'WDF values: ', wdfar
        return
      endif
      junkstr = ''
      if n_elements(nfreq) eq 0 then nfreq = 512
      if n_elements(npatch) eq 0 then npatch = 512
      resp = make_array(nfreq, npatch,/dcomplex)
      Freq = make_array(nfreq, /double)
      patch = make_array (npatch, /long)
      ff = 0d0
      ip = 0L
      res = dcomplex(0d0, 0d0)
      ifreq = 0L
      ipatch = 0L
      on_ioerror, badnp
      readf, unit, junkstr

      if strpos(junkstr, '--') ge 0 then readf, unit, junkstr
      lcfreq = strpos(junkstr, 'Frequency')
      if lcfreq lt 0 then reads, junkstr, ff else begin
        lcfreq = lcfreq+9
        reads, strmid(junkstr, lcfreq), ff
      endelse
      freq[0] = ff
;      print, ff, format='(e24.14)'
      on_ioerror, badnp
      readf, unit, junkstr
      lcelem = strpos(junkstr, 'element #')
      lcsum = strpos(junkstr, 'sum')
      if lcelem lt 0 and lcsum lt 0 then $
        reads, junkstr, ipatch, res else begin
        lcsum = lcsum+3
        lcelem= lcelem+9
        reads, strmid(junkstr, lcelem), ipatch
        reads, strmid(junkstr, lcsum), res
      endelse
      patch[ip] = ipatch
      resp [ifreq, ip] = res
;      print, res, format='(2e24.14)'
      readf, unit, junkstr
      ip = ip+1
      while strpos(junkstr, '--')lt 0 and not eof(unit) do begin
        if lcelem lt 0 then $
          reads, junkstr, ipatch, res else begin
          reads, strmid(junkstr, lcelem), ipatch
          reads, strmid(junkstr, lcsum), res
        endelse
        patch[ip] = ipatch
        resp[ifreq, ip] = res
;        print,res, format='(2e24.14)'
        ip = ip+1
        readf, unit, junkstr
      endwhile
      ifreq = ifreq+1
      patch = patch[0:ip-1]
      on_ioerror, finish
    while not eof(unit) do begin
      readf, unit, junkstr
      if lcfreq lt 0 then reads, junkstr, ff else $
        reads, strmid(junkstr, lcfreq), ff
      freq [ifreq] = ff
      i =0
      while i lt ip and not eof(unit) do begin
        if lcelem lt 0 then  readf, unit, ipatch, res else begin
          readf, unit, junkstr
          reads, strmid(junkstr, lcelem), ipatch
          reads, strmid(junkstr, lcsum), res
        endelse
        resp[ifreq, i] = res
        i = i+1
      endwhile
      if i eq ip then ifreq = ifreq+1
      if not eof(unit) then readf, unit, junkstr else goto, finish
    endwhile

finish:  np = ifreq
      if ifreq lt 3 then begin
        print, '% RERESP: Must have more than 2 frequencies'
        return
      endif
      freq = freq[0:ifreq-1]
      freqc = dcomplex(freq)
      resp = resp[0:ifreq-1, 0:ip-1]

; convert to wdf array using the lowres option
      nwdf = n_elements (wdfar)
      if nwdf  lt 2*ip then wdfar = [wdfar, wdfar[nwdf-1]+1+lindgen(2*ip)]
      for i = 0 , ip-1 do begin
        clabel = 'Patch ' + strtrim (patch[i], 2)
        i2w, freq, reform(abs(resp[*, i])), wdfar[i], $
          clab =clabel+' - Amplitude', file= filename, /xydata
        i2w, freq, reform(resp[*, i]), wdfar[i+ip], $
          clab =clabel+' - Values', file= filename, /xydata
      endfor
;
      if lcfile eq 1 then free_lun, unit
      return

badnp:
      if lcfile eq 1 then free_lun, unit
       print, '% RERESP: Error reading first frequency data'
      return
      end
