; ------------------------------------------------------------------------ 
; $Id: restorepfidl.pro,v 1.8 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO restorepfidl, file, keep = keepin, _extra= extrastuff, filename = filnam, $
         sstart = sstart, quiet = quiet
;+
;
;  Restore the data stored in WDF arrays and in structure arrays
;
;  Format:  RESTOREPFIDL, FILE, KEEP = keep, _extra = extrastuff, $
;          FILENAME = filename, SSTART = sstart, QUIET = quiet
;
;  where:
;      FILE    -  File in which the data is stored
;                 If FILE is not a string or FILE does not exist,
;                    DIALOG_PICKFILE is called.
;                 Default is 'pfidlsave.dat'
;      FILENAME-  Alternate method of specifying file name.
;           Note: If FILE is specified, FILENAME is ignored.
;           Note: If FILE equals ZERO, then DIALOG_PICKFILE will be called.
;
;      KEEP    -  if positive and valid, all wdfarrays greater than or equal 
;                   to keep will be unaffected by the restore command.
;                   ( same as keep = [keep, maxwdfarray] )
;                 if negative and valid, all wdfarrays less than or equal
;                   to keep will be unaffected by the restore command.
;                   ( same as keep = [1, -keep] )
;                 if keep has two or more elements then elements from keep(0) to 
;                   keep(1) will be saved where keep is redefined to:
;                   keep = round([(min(keep, max = m) > 1) , (m < maxwdfarray)])
;           Note: If keep is zero, no waveforms will be saved.
;           Note: If there is not enough WDF arrays available to store all saved arrays,
;                 then MAXWDFARRAY will be increased.
;      SSTART   -  Structure arrays which are restored will be saved beginning
;                  with structure array sstart or 1 if sstart is not valid or
;                  not specified
;           Note: If there is not enough structure arrays available to store all saved 
;                 structures then MAXSTRARRAY will be increased.
;      QUIET   -  If set, no information regarding the file or the number 
;                 of arrays and structures will be printed.
;      _EXTRA  -  Any additional keywords recognized by RESTORE
;  Example:
;    Restore  the data from a previous session
;      restorepfidl
;
;    Restore the data from the session in file 'qs5581.sav'
;      restorepfidl,  'qs5581.sav'
;
;    Restore the data from a file to be determined and begin storing the
;    waveform data in wdfarray 101
;      restorepfidl, 0, keep = -100
;-
      on_error, 1
;  Common block
;
  maxwdf= get_maxwdf()
  maxwdfarray = maxwdf
  if n_elements(keepin) eq 0 then keep = maxwdfarray+1 else keep = keepin
  if n_elements(keep) eq 1 then begin
    if keep ge 0 then keep = [keep, maxwdfarray] else keep = [1, (-keep < maxwdfarray)]
  endif else $
    keep = round([(min(keep, max = m) > 1) , (m < maxwdfarray)]) 
;
  if n_elements(sstart) eq 0 then sstart = 1
  smaxstructure = get_maxstr()
  if sstart lt 1 or sstart gt smaxstructure then sstart= 1

;
  if n_elements (file) eq 0 then if n_elements(filnam) gt 0 then $
        file = filnam else file = 'pfidlsave.dat'
  if (size(/type, file) ne 7) then lcdp = 1 else begin
    info = file_info(strtrim(file, 2))
    lcdp = ((info.exists eq 0) || (info.read eq 0)) ?  1 : 0
  endelse
  if lcdp ne 0 then begin
    file = dialog_pickfile (title = 'Please pick a file to restore', /must_exist)
    if strlen(strtrim(file, 2)) lt 1 then begin
      print, '% RESTOREPFIDL: No file specified'
      return
    endif
  endif
          
  pathsave = !path
  dirsave = !dir
  restore, file, _extra= extrastuff
  !dir = dirsave
  !path = pathsave
;
  okw = where(wdfnp gt 2 , nwdf)
  if ((keep[0] gt 0)  && (nwdf gt 0))then begin
    nkeep = (keep[1] -keep[0]+1) > 0
    if (nkeep+ nwdf) ge maxwdf then begin
      maxwdf = (((nkeep+nwdf)/10)+2)*10
      set_maxwdfarray, maxwdf
    endif
    get_com_data, swdfcomment, swdffile, swdfnp, swdfmax, swdfmin, swdfminx, $
       swdfmaxx, swdfxlabel, swdfylabel, swdfarraycom,  maxwdfarray

    if keep[0] gt keep[1] then keep[0] = nwdf+1
    nw = (keep[0]-1) < nwdf
    if nw gt 0 then begin
      swdfcomment[0:nw-1] = wdfcomment[0:nw-1]
      swdffile[0:nw-1]  = wdffile[0:nw-1]
      swdfnp[0:nw-1]  = wdfnp[0:nw-1]
      swdfmax[0:nw-1]  = wdfmax[0:nw-1]
      swdfmin[0:nw-1]  = wdfmin[0:nw-1]
      swdfminx[0:nw-1]  = wdfminx[0:nw-1]
      swdfmaxx[0:nw-1]  = wdfmaxx[0:nw-1]
      swdfxlabel[0:nw-1]  = wdfxlabel[0:nw-1]
      swdfylabel[0:nw-1]  = wdfylabel[0:nw-1]
      for i = 0, nw-1 do begin
        junk = execute('array = wdfarray'+strtrim(i+1, 2))
        if swdfarraycom(i) eq 0 then swdfarraycom(i)= handle_create()
        handle_value, swdfarraycom(i), array, /set, /no_copy
      endfor
    endif
    if nw lt nwdf then begin
      nws= keep[1]
      nwe= keep[1]+nwdf-nw-1
      swdfcomment[nws:nwe] = wdfcomment[nw:nwdf-1]
      swdffile[nws:nwe]  = wdffile[nw:nwdf-1]
      swdfnp[nws:nwe]  = wdfnp[nw:nwdf-1]
      swdfmax[nws:nwe]  = wdfmax[nw:nwdf-1]
      swdfmin[nws:nwe]  = wdfmin[nw:nwdf-1]
      swdfminx[nws:nwe]  = wdfminx[nw:nwdf-1]
      swdfmaxx[nws:nwe]  = wdfmaxx[nw:nwdf-1]
      swdfxlabel[nws:nwe]  = wdfxlabel[nw:nwdf-1]
      swdfylabel[nws:nwe]  = wdfylabel[nw:nwdf-1]
      j= nw-nws+1
      for i = nws, nwe do begin
        junk = execute('array = temporary(wdfarray'+strtrim(i+j, 2)+')')
        if swdfarraycom(i) eq 0 then swdfarraycom(i)= handle_create()
        handle_value, swdfarraycom(i), array, /set, /no_copy
      endfor
    endif 
    put_com_data, swdfcomment, swdffile, swdfnp, swdfmax, swdfmin, swdfminx, $
        swdfmaxx, swdfxlabel, swdfylabel, swdfarraycom
  endif
; check for old format 
    if n_elements(lcvalidstr) gt 0 then $
      ok = where (lcvalidstr gt 0, nok) else begin
      ok = where(strlen(comment) ge 2, nok)
;     in case we have some blank comments
      if (nok gt 0) then nok = ok[nok-1]+1
    endelse
    nstr = nok
    if nok gt 0 then begin
      maxstr = smaxstructure
      if maxstr lt sstart-1+nok then begin
        maxstr = (((sstart+nok)/10)+1)*10
        set_maxstrarray, maxstr
      endif
;
      get_str_data, sstrarraycom, scomment, slcvalidstr, smaxstructure
      nokm = nok-1  
      sstm = sstart-1
      scomment[sstm:sstm+nokm] = comment[0:nokm]
      slcvalidstr[sstm:sstm+nokm]  = 1
      for i = 0, nokm do begin
        junk = execute('str = temporary(structure'+strtrim(i+1, 2)+')')
        if sstrarraycom(i+sstm) eq 0 then sstrarraycom[i+sstm]= handle_create()
        handle_value, sstrarraycom[i+sstm], str, /set, /no_copy
      endfor
      put_str_data, sstrarraycom, scomment, slcvalidstr
    endif
    if not keyword_set(quiet) then begin
      print, '% RESTOREPFIDL: Data restored from file, ', file
      print, '             '+strtrim(nwdf, 2)+' Waveform arrays'
      print, '             '+strtrim(nstr, 2)+' Structure arrays'
    endif

    end
