; ------------------------------------------------------------------------ 
; $Id: rufo.pro,v 1.3 2010/09/17 21:20:35 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:                 RUFO
; PURPOSE:              
;       IDL routine to read in a simple x,y file. UFO format #111.
;       Format is assumed to consist of a 5 line header, followed by
;       N lines of x,y data pairs, concluded with a -999,-999 data pair.
;
; CALLING SEQUENCE:     RUFO,X,Y [, FILENAME] [, FILENAME=fname ]
; INPUT:
;       FILENAME:       Input filename or unit number.
;                       If zero or undefined, DIALOG_PICKFILE is called
;                         and the selected name is returned in FILENAME
; OUTPUTS:              X,Y 
;                       X and Y data from the UFO file
; KEYWORDS:
;       FILENAME:       Ignored if the parameter FILENAME is present
;                       If FILENAME is undefined, then FNAME is used.
;                       If zero or undefined, DIALOG_PICKFILE is called
;                         and the selected name is returned in FILENAME
; EXAMPLE:
;       Read to X and Y, UFO file, data_4.ufo
;         RUFO, X, Y, 'data_4.ufo'
;       Read file, zdata1256_09_09_09_wave.ufo, to XVAL and YVAL
;         REUFO, XVAL, YVAL, 'zdata1256_09_09_09_wave.ufo'
;
; MODIFICATION HISTORY:
;       Adapted from W E Nelson's routine
;       Included in PFIDL, L. P. Mix, 9/10/2009
;-


PRO rufo,xdata,ydata, filename, file=fname

on_error, 1

NPAR = n_params()
if npar eq 0 then begin
  print, '% FORMAT:  RUFO,X,Y [, FILENAME] [, FILENAME=filename ]'
  return
endif
If (n_elements(filename) eq 0) && (n_elements(fname) ne 0) then begin
  filename = fname[0]
  lckey = 1
endif else lckey =0
filename = (n_elements(filename) eq 0) ? 0 : filename[0]
readasc,filename,2,xdata,ydata,skip=5, endnumber =-999
if lckey then fname = filename

end
