; ------------------------------------------------------------------------ 
; $Id: savepfidl.pro,v 1.5 2007/10/05 17:45:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 PRO savepfidl,file,  file = filnam, _extra= extrastuff, exact = exact, quiet = quiet
;+
;
;  Save the data stored in WDF arrays and in structure arrays
;
;  Format:  SAVEPFIDL, FILE, FILE = filename, EXACT = exact, _EXTRA = extrastuff, /QUIET
;
;  where:
;      file    -  File in which the data is stored
;                 Default is 'pfidlsave.dat'
;      Filename- Alternate method of specifying file name.
;      EXACT   - If set, the exact filename is used.
;      NOTE:  FIND_NEWFILE will be called to insure the filename is unique
;             UNLESS EXACT is SET.
;             If filename is not unique, then:
;             If file contains a . then add _# before the period
;             If file has a leading . the add _# to the end
;             If file has no . then add _#
;    NOTE: If FILE is specified, FILENAM is ignored.
;
;      _EXTRA  -  Any additional keywords recognized by SAVE
;      QUIET   -  If set, no information regarding the output file name or the 
;                 number of arrays processed will be printed.
;  Example:
;    Save the data from the current session
;      savepfidl
;
;    Save the data from the current session in file 'qs5581.sav'
;      savepfidl, file = 'qs5581.sav'
;-
      on_error, 1
;
  get_com_data, wdfcomment, wdffile, wdfnp, wdfmax, wdfmin, wdfminx, $
                wdfmaxx, wdfxlabel, wdfylabel, wdfarraycom,  maxwdfarray
;
  if n_elements (file) eq 0 then if n_elements(filnam) gt 0 then file = filnam
;
  if n_elements (file) eq 0 then file = 'pfidlsave.dat'
  if not keyword_set(exact) then file = find_newfile(file, /quiet)
  j = 0
  i = 0
; set up waveform arrays
  ok = where((wdfnp gt 2) and (wdfarraycom gt 0), nok)
  if nok gt 0 then begin
    jj = indgen(nok)
    j = nok
    wdfcomment[jj] = wdfcomment[ok]
    wdffile[jj] = wdffile[ok]
    wdfnp[jj] = wdfnp[ok]
    wdfmax[jj] = wdfmax[ok]
    wdfmin[jj] = wdfmin[ok]
    wdfminx[jj] = wdfminx[ok]
    wdfmaxx[jj] = wdfmaxx[ok]
    wdfxlabel[jj] = wdfxlabel[ok]
    wdfylabel[jj] = wdfylabel[ok]
    for i = 0, nok-1 do begin
      handle_value, wdfarraycom[ok[i]], array
      junk = execute('wdfarray'+strtrim(i+1, 2)+' = array')
    endfor
  endif
  if j lt maxwdfarray then begin
    jm = (j-1) > 0
    wdfcomment = wdfcomment[0:jm] 
    wdffile = wdffile[0:jm]
    wdfxlabel =wdfxlabel[0:jm]
    wdfylabel = wdfylabel[0:jm]
    wdfnp = wdfnp[0:jm]
    wdfmax = wdfmax[0:jm]
    wdfmin = wdfmin[0:jm]
    wdfminx = wdfminx[0:jm]
    wdfmaxx = wdfmaxx[0:jm]
  endif
  nwdf = j
;
  get_str_data, strarraycom, comment, lcvalidstr, maxstructure
  j = 0
  i = 0
  ok = where((lcvalidstr ne 0) and (strarraycom gt 0), nok)
  if nok gt 0 then begin
    jj = indgen(nok)
    j = nok
    comment[jj] = comment[ok]
    lcvalidstr[jj] = 1
    for i = 0, nok-1 do begin
      handle_value, strarraycom[ok[i]], str
      junk = execute('structure'+strtrim(i+1, 2)+' = str')
    endfor
  endif
  nstr = j
  if j lt maxstructure then begin
      jm = (j-1)>0
      comment = comment[0:jm]
      lcvalidstr =lcvalidstr[0:jm]  
  endif
  str = 0
  array = 0
;
; Save the data
  save, file = file, _extra= extrastuff
if not keyword_set(quiet) then begin
  print, '% SAVEPFIDL: Data saved to file, ', file
  print, '             '+strtrim(nwdf, 2)+' Waveform arrays'
  print, '             '+strtrim(nstr, 2)+' Structure arrays'
endif
;
;
; clean up the memory
  if nwdf gt 0 then $
    for i = 1, nwdf do junk = execute('array = temporary(wdfarray'+strtrim(i, 2)+')')
  if nstr gt 0 then $
    for i = 1, nstr do junk = execute('array = temporary(structure'+strtrim(i, 2)+')')
   array = 0
   comment = 0
   lcvalidstr = 0
   wdfcomment = 0
   wdffile = 0
   wdfxlabel = 0
   wdfylabel = 0
   wdfnp = 0
   wdfmax = 0
   wdfmin = 0
   wdfminx = 0
   wdfmaxx = 0
;
  return
  end
