; ------------------------------------------------------------------------ 
; $Id: set_maxwdfarray.pro,v 1.5 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  set_maxwdfarray, maxwdf, quiet = quiet
;
;+
;    Set the maximum number of wdf arrays
;
;    format:
;        set_maxwdfarray, maxwdf [, /quiet]
;        where
;          maxwdf - an integer indicating the maximum number of WDF arrays
;                   maxwdfarray = abs(long(maxwdf)) > 10
;                   Default maximum is 82
;
;          QUIET  - If set no warning will be issued if the number of
;                   datasets is decreased
;         Note: If the number of WDFarrays is decreased, data can be lost.
;               Any data above maxwdf will be deleted.
;
;    Example:
;        Set the number of WDF arrays to 102.
;           set_maxwdfarray, 102
;-
  on_error, 1
;
@com.dat
      if n_elements(maxwdf) eq 0 then begin
        print, '%SET_MAXWDFARRAY: No value set. Maximum is unchanged.'
        return
      endif
      if n_elements(initwdf) eq 0 then begin
        maxwdfarray = long(abs(max(maxwdf)) > 10L)
        return
      endif
;
      oldmax = maxwdfarray
      maxwdfarray = long(abs(max(maxwdf)) > 10L)
      if oldmax eq maxwdfarray then return
;
      if oldmax gt maxwdfarray then begin
;
        if keyword_set(quiet) then lcquit=0L else $
          wok_notok, lcquit, ['Number of WDF arrays is decreasing', $
          'Do you want to continue?'], 'Set Max WDF array'
        if lcquit eq 1 then begin
          maxwdfarray = oldmax
          return
        endif 
        ok = where (wdfarraycom (maxwdfarray:oldmax-1) gt 0, nok)
        if nok gt 0 then for i = 0, nok-1 do $
          handle_free, wdfarraycom(maxwdfarray+ok(i))

        wdfarraycom = wdfarraycom(0:maxwdfarray-1)
        wdfcomment = wdfcomment(0:maxwdfarray-1)
        wdfnp =wdfnp(0:maxwdfarray-1)
        wdfmax =wdfmax(0:maxwdfarray-1)
        wdfmin =wdfmin(0:maxwdfarray-1)
        wdfmaxx =wdfmaxx(0:maxwdfarray-1)
        wdfminx =wdfminx(0:maxwdfarray-1)
        wdffile =wdffile(0:maxwdfarray-1)
        wdfxlabel =wdfxlabel(0:maxwdfarray-1)
        wdfylabel =wdfylabel(0:maxwdfarray-1)
        return
      endif
      m = maxwdfarray-oldmax
      wdfarraycom = [wdfarraycom, make_array(m, /long)]
      wdfcomment = [wdfcomment, make_array(m,/string,value = '')]
      wdfnp = [wdfnp,  make_array(m, /long, value = 0L)]
      wdfmax = [wdfmax, make_array(m, /float, value = 0.0)]
      wdfmin = [wdfmin, make_array(m, /float, value = 0.0)]
      wdfmaxx = [wdfmaxx, make_array(m, /float, value = 0.0)]
      wdfminx = [wdfminx, make_array(m, /float, value = 0.0)]
      wdffile = [wdffile, make_array(m, /string, value = '')]
      wdfxlabel = [wdfxlabel, make_array(m, /string, value = '')]
      wdfylabel = [wdfylabel, make_array(m, /string, value = '')]
      return
      end
