; ------------------------------------------------------------------------
; $Id: set_pfidl_directory.pro,v 1.1 2009/03/27 19:11:56 lpmix Exp $
; ------------------------------------------------------------------------
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       SET_PFIDL_DIRECTORY
;
; PURPOSE:
;       Set the pfidl directory location (this can be put into IDL_STARTUP)
;       With standard installations, PFIDL will determine the location
;       With other installations, PFIDL requires that either a system variable, 
;         PFIDL_DIRECTORY or this routine be used to allow IDL to know where 
;         the shared libraries and help files are located.
;
; CALLING SEQUENCE:
;       SET_PFIDL_DIRECTORY, DIRECTORY
;
; INPUTS:
;       DIRECTORY: Location of the top PFIDL directory.
;               This directory contains the directories: HELP, PFIDL, and LIB
;               (Generally the directory names are lower case)
;
; EXAMPLE:
;       Set the pfidl directory to a non_standard location, '/home/lpmix/pfidl'
;         SET_PFIDL_DIRECTORY, '/home/lpmix/pfidl'
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, Jr., March 27, 2009
;			something!
;-

pro set_pfidl_directory, direct
common pfidl_directory_common, pdirect

on_error, 1

if n_elements (pdirect) eq 0 then begin
  print, '% SET_PFIDL_DIRECTORY: Directory must be defined'
  print, 'Format: SET_PFIDL_DIRECTORY, DIRECTORY'
  return
endif

if size(/type, pdirect) ne 7 then begin
  print, '% SET_PFIDL_DIRECTORY: Directory must be string'
  return
endif
;
sep = path_sep()
pdirect = direct
if strlen (pdirect) gt 1 then $
  if (get_match(sep+'$', pdirect, casesen=0, regexp=0) lt 0) then pdirect = pdirect+sep

end
