; ------------------------------------------------------------------------ 
; $Id: set_printer.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro set_printer, show = show, destination = destination,  $
        color_default= color_def, bw_default = bw_def, $
        printfile = printfile, file_path = file_path, $
        unique_printfile = unique_printfile
;+
;  Procedure to show or set the destination for LPRINT
;
;  format: SET_PRINTER [, /SHOW] [, DESTINATION = destination]
;              [, COLOR_DEFAULT = color_def] [, BW_DEFAULT = bw_def]
;              [, PRINTFILE = printfile] [, FILE_PATH = file_path]
;              [, /UNIQUE_PRINTFILE]
;  where:
;    SHOW     - If set, show current defaults
;    DESTINATION - If not blank, send all plots to this destination.
;             Set DESTINATION to '' to return to the color and bw_defaults.
;    COLOR_DEFAULT - Set the color default to this destination
;    BW_DEFAULT - Set the black and white default to this destination
;    PRINTFILE  - Default printfile for postscript, pcl output and lprint.
;                 Default: PRINTFILE = 'idl.ps' 
;                 Note:  If the first character is '/' or '.' for unix or the 
;                        second and third characters are ':\' for Windows then 
;                        printfile is assumed to be a fully qualified file name
;                        and file_path is set to '' (NULL string).
;                 Note:  FulFnam will be used to resolve the printfile name
;                        on Unix/Linux systems.
;    FILE_PATH  - If set, PRINTFILE will be placed in this directory.
;                 Default: FILE_PATH = '.' for unix
;                 Default: FILE_PATH = 'c:\temp' for windows
;    UNIQUE_PRINTFILE - If set each printfile will be unique. 
;                 For example: If printfile ='idl.ps' then the second time
;                              the actual file will be 'idl_001.ps'
;                       The first additional number will be given by the 
;                       value of (UNIQUE_PRINTFILE > 1)
;
;  Examples:
;    Determine the current default printer destination
;      SET_PRINTER, /show
;
;    Set the color default to 962_1_hp8550
;      SET_PRINTER, col='962_1_hp8550'
;
;    Send all plots to 962_1_hp8550
;      SET_PRINTER, dest='962_1_hp8550'
;
;    Put all plotfiles into the home directory on UNIX
;      SET_PRINTER, file_path =  getenv('HOME')
;    or 
;      SET_PRINTER, file_path = '~/'
;
;    Set the plotfile to 'idl.ps' in the printer directory under home
;      SET_PRINTER, printfile = '~/printer/idl.ps'
;
;-
;
on_error, 1
 common pffcom, initpff, pff_exe
 common lprintdefault, bw_default, color_default
@lprintsav.com
;common lprintsav,lpinit, destsave, filesave, default_printfile,
;  lcunique, print_file_last
;
common copyright_common, copyright_init
;
Common set_printer_common, printfile_save, file_path_save
;
 if n_elements (copyright_init) eq 0 then copyright_notice
 name = strupcase(strmid(!version.os, 0, 3))

 if n_elements (lpinit) eq 0 then begin
     destsave = ''
     lpinit = 1
     lcunique = 0
     printfile_save = 'idl.ps'
     if (name  eq 'WIN') then begin
       file_path_save = 'c:\temp\'
       bw_default = 'lpt1:'
       color_default = 'lpt1:'
     endif else begin
       file_path_save = './'
       bw_default  = 'C962_1_lj'
       color_default = 'C962_1_clj'
     endelse
     default_printfile = file_path_save+printfile_save
     print_file_last= default_printfile
 endif
 if n_elements(bw_def) gt 0 then bw_default= strtrim(bw_def[0], 2)
 if n_elements(color_def) gt 0 then color_default= strtrim(color_def[0], 2)
 if n_elements(destination) gt 0 then destsave= strtrim(destination[0], 2)
 if n_elements(unique_printfile) gt 0 then lcunique = $
    (unique_printfile) ? (unique_printfile > 1) : 0

 if n_elements(file_path) gt 0 then begin
   file_path_save = strtrim(file_path[0], 2)
   if strlen(file_path_save) lt 2 then file_path_save=file_path_save+path_sep()
   file_path_save = fulfnamf(file_path_save)
   l = strlen(file_path_save)
   if strmid(file_path_save, l-1, 1 ) ne path_sep() then $
     file_path_save = file_path_save+path_sep()
   if n_elements(printfile) eq 0 then printfile = printfile_save
 endif
 if n_elements(printfile) gt 0 then begin
   printfile_save = fulfnamf(printfile[0])
   if name eq 'WIN' then $
     lcfullname = (strmid(printfile_save, 1, 2) eq ':\') ?  1 : 0 $
   else lcfullname = (strmid(printfile_save, 0, 1) eq '/' or $
            strmid(printfile_save, 0, 1) eq '.') ?  1 : 0
   default_printfile = $
     lcfullname ? printfile_save : file_path_save+printfile_save

 endif

 if keyword_set (show) then begin
   print, 'SET_PRINTER default values:'
   IF strlen(destsave) gt 0 then print, $
     '   All printer output goes to ', destsave
   print, '  Black/White default is ', bw_default
   print, '  Color default is ', color_default
   print, '  Print File is ', default_printfile
   print, '  Default file directory is ', file_path_save
   if keyword_set(lcunique) then print, '  Each file has a unique name' else $
     print, '  Each file will have the same name'
 endif
 end
