; ------------------------------------------------------------------------ 
; $Id: setdirpff.pro,v 1.3 2007/06/06 20:03:53 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  setdirpff, p1, fileid
;
;+
;     This routine sets directory pointer of the current pff file.  
;         or alternatively, will return the total number of datasets.
;
;     format: setdirpff, dataset, fileid
; 
;     where
;         dataset - desired dataset pointer value
;         fileid  - file id of the PFF file containing the data
; 
;     if dataset is undefined or is less than or equal to zero then
;         the total number of datasets is returned in dataset.
;           
;
;     Example
;       set the dataset pointer to dataset 6
;          setdirpff, 6
;
;       store in n, the number of datasets in the current pff file
;          n = -1 & setdirpff, n
;       or
;          n = ndspff( )
;-
 on_error, 1
      common pffcom, initpff, pff_exe
;
;     check to see if pff has been initialized
;
      if (n_elements(initpff) eq 0) then initpff=0
      if (initpff eq 0) then begin
        print, '% SETDIRPFF: No PFF files are open'
        return
      endif
;
;     check if the dataset pointer is defined
;
;     if no dataset number is entered then return
      if (n_elements(p1) eq 0) then p1 = -1L
;
;     get integers for the fortran call
;
      if n_elements(fileid) eq 0 then fid = 0L else fid = long(fileid) 
      dirpt = p1
      get_pffinteger, dirpt
      ier = 0L
      if initpff eq 1 then begin
; HP
        junk=call_external(PFF_EXE,'pff_slsetdir',dirpt, fid, ier)
      endif else if initpff eq 2 then begin
; Sun
        junk=call_external(PFF_EXE,'pff_slsetdir_',dirpt, fid, ier)
      endif else if initpff eq 4 then begin
; WIN
        junk=call_external(PFF_EXE,'_PFF_SLSETDIR@8',dirpt, fid, ier)
      endif else if initpff eq 3 then begin
; VMS
        junk=call_external(PFF_EXE,'pff_setdir',dirpt, fid, ier)
      endif else if initpff eq 5 then begin
; MAC
        junk=call_external(PFF_EXE,'pff_slsetdir',dirpt, fid, ier)
      endif
      if ier ne 0 then print, ' ERROR in SETDIRPFF'
      if p1 le 0 then p1 = dirpt
      return
      end
