; ------------------------------------------------------------------------ 
; $Id: startgif.pro,v 1.9 2010/11/18 17:59:41 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro startgif,file, font= font, size_char = size_char, noswap = noswap, $
     filename = filename, thick = thick, xythick = xythick, $
     character_size = char_size, bigwin = bigwin, $
     inquire = inquire, number = number,  _extra = extrastuff
;+
;  Set the screen to generate a plot suitable for generating a GIF file or 
;    a PICT file or a TIFF file for inclusion in a word document.
;  Note: ENDGIF must be used to restore the old screen parameters.
;  True Type fonts are used.
;
;  Format: STARTGIF [, FILE] [, FONT = font]  [, SIZE_CHAR = size_char]
;                  [, BIGWIN = bigwin] [, CHARACTER_SIZE = character_size ]
;                  [, /NOSWAP] [, THICK = thick]  [, XYTHICK = xythick]
;                  [, INQUIRE = inquire] [, FILENAME =file] 
;                  [, NUMBER = number]  [, _extra = extrastuff]
;  where:
;    file       - output file name for the GIF or PICT output.
;                 Default is 'idl000n.gif' or 'idl000n.pict'
;                  (that is, File = 'idl')
;    font       - Font Name (any True Type font)
;                 Default = 'Helvetica Bold'
;                 Default font will be changed to font if font is specified.
;                 Note: Use the following to set a different true type font.
;                    device, set_font = font, /tt_font
;    NOSWAP     - If set color map will not be switched.
;                 If NOSWAP lt 0 then background and drawing colors will be
;                   switched.
;                 Default is to issue the command:
;                   INVERTCT, /GIF
;                 and when endgif is entered:
;                   INVERTCT, /RESET
;    BIGWIN     - If set, BIGWIN will be called.  This will make a large PIXMAP
;                 window for the plot.  If BIGWIN has two elements, then
;                 BIGWIN = [ XSIZE, YSIZE] in the call to BIGWIN.
;                 Default values of XSIZE and YSIZE are in LHELP --> LOCAL --> BIGWIN
;    Character_size - Multiplier for the default character size.
;                  Default character size is defined in "Size_char" below
;    Size_char   - Width and height for character.
;                  If no values given - Default = [1, 1.4]*0.022*!d.y_size
;                  If 1 value given   - Default = [1, 1.4]*size_char
;                  
;                  Note 1: The current font size is stored in !d.x_ch_size and
;                          !d.y_ch_size.  These are read only.
;                  Note 2: Use the following to set another character size.
;                          device, set_character_size = [x_ch_size, y_ch_size]
;                
;    thick       - specifies the plot line thickness.
;                  Default is 0.004*!d.y_size
;    xythick     - Specifies the axes thicknesses.
;                  Default is 0.004*!d.y_size
;    inquire     - Used to set the font to a hardware font.
;                  If set to a number, then the command a = xfont() will be
;                  executed and the result set to the value.
;                  If set to a string, then that hardware font will be used.
;                  One font on the HP is
;     "-adobe-helvetica-bold-r-normal--34-240-100-100-p-182-iso8859-1"
;    FILENAME    - Output file name.  This is ignored if FILE is present.
;    NUMBER      - Initial number for the screen capture.  
;                  Default = 1
;            <or>  Number for the next screen capture if STARTGIF has been 
;                  called.
;
;                  Number must be greater than or equal to zero.
;                  
;    extrastuff  - Any valid keywords and values for the device command
;
;  Examples:
;
;  Send the plots to gif files of the form: movie0001.gif, movie0002.gif, ...
;  NOTE:
;   For a PICT file replace SGIF with SPICT. For TIFF replace SGIF with STIFF.
;    Startgif, file =  'movie'
;    < do a plot >
;    sgif
;    < do a plot >
;    sgif
;    < do a plot >
;    sgif    ; etc
;    endgif
;-

on_error, 1 
@loadxyct_common.dat
   common swap_common, lcswap
   common big_window_common, big_window
   common switchgif,xythicksave,thicksave,oldthick,oldxythick,oldfont, $
      gif_file, font_default, no_swap, character_size, file_number, $
      lc_gif_output, old_character_size

   if n_elements(loadct_init) eq 0 then loadxyct
   if n_elements(lcswap) eq 0 then lcswap = 0

   if keyword_set(bigwin) then begin
     if n_elements (bigwin) lt 2 then bigwin else $
       bigwin, xsize = bigwin[0], ysize = bigwin[1]
     big_window = 1
   endif else big_window = 0

   if n_elements(lc_gif_output) eq 0 then lc_gif_output= 0
   if lc_gif_output eq 1 then begin
print, '% STARTGIF: Device is already in GIF output mode. STARTGIF is ignored.'
print, '            Use ENDGIF to return to normal output mode'
     return
   endif
   if !d.name eq 'X' or !d.name eq 'TEK' or !d.name eq 'WIN' $
        or !d.name eq 'MAC' then begin
     oldxythick = !x.thick
     oldthick = !p.thick
     oldfont = !p.font
     if !d.window lt 0 then begin
       window
       loadxyct 
     endif
     old_character_size = [!d.X_CH_SIZE, !d.Y_CH_SIZE]
;     if lcswap then begin
;       gif = 0
;       xydata = 1      
;     endif else begin
       gif = 1
       xydata = 0
;     endelse
   endif else begin
     print, '% STARTGIF: Window device must be active for GIF'
     return
   endelse
   if keyword_set(noswap) then begin
     if noswap lt 0 then swap
     no_swap = noswap
   endif else begin
     no_swap = 0
     invertct, gif = gif, xydata = xydata
   endelse
   if n_elements(font_default) eq 0 then font_default = 'Helvetica Bold'
   
   if n_elements (number) eq 0 then numb = 1L else begin
     numb = abs(number)
     file_number = numb
   endelse
   if n_elements(file) gt 0 then begin
      gif_file = strtrim(file(0), 2)
      file_number = numb
   endif else if n_elements(filename) gt 0 then begin
      gif_file = strtrim(filename(0), 2)
      file_number = numb
   endif else if n_elements(gif_file) eq 0 then begin
      gif_file = 'idl'
      file_number = numb
   endif
 
   if n_elements(thick) eq 0 then thick = 0.004*!d.y_size
   if n_elements(xythick) eq 0 then xythick = 0.004*!d.y_size
   xythicksave = xythick
   thicksave = thick
   !x.thick = xythick
   !y.thick = xythick
   !z.thick = xythick
   !p.thick = thick
   lcopen = 1
   if keyword_set (inquire) then begin
     inq = size(inquire, /type)
     if inq ne 7 then inquire = xfont()
     !p.font = -1
     device, set_font = inquire, _extra=extrastuff
     !p.font = 0
     lc_gif_output = 1
     return
   endif
   !p.font = 1
   if n_elements (font) eq 0 then font = font_default else $
        font_default = font
;
   if n_elements(size_char) eq 0 then size_ch= .022*!d.y_size* [1.0, 1.4] $
     else begin
     if  n_elements(size_char) eq 1 then size_ch = size_char*[1.0, 1.4] else $
       size_ch = size_char(0:1)
   endelse
   if n_elements (char_size) gt 0 then size_ch=char_size[0]*size_ch
   device, set_font = font, set_character_size  = size_ch, /tt_font, $
           _extra=extrastuff
   lc_gif_output = 1
   end
