; ------------------------------------------------------------------------ 
; $Id: startp.pro,v 1.4 2009/11/03 17:04:20 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  Pro startp,pfile, portrait = portrait, charsize= charsize, $
             font= font, color= color, bold = bold, noload = noload, $
              file = file, bits = bits, thick = thick, xythick = xythick, $
              copy = copy, interpolate = interpolate, _extra= extrastuff
;+
;  Generate a plot for the default printer
;  Note: ENDP must be used to close the device and return to terminal output
;  Format: STARTP  [, FONT = font]  [, COLOR= color]  [, BOLD = bold]
;                  [, COPY = copy] [, INTERPOLATE = interpolate] [, FILE=file]
;                  [, BITS = bits] [, THICK = thick] [, XYTHICK = xythick]
;                  [, NOLOAD = noload] [, PORTRAIT = port] [, _extra = extrastuff]
;  where:
;    portrait - put output into portrait mode with a full page
;              margin is 1/2 inch, size is 7.5x10 inches
;              DEFAULT is landscape
;    charsize - Multiplier for character size
;               If set, !p.charsize = charsize
;    font - font thickness Default = 3
;           !p.charthick = font
;    color - black and white output is used if color is set to zero.
;            Default is color= 0
;    copy - See Set_plot - Copy color table from current device to postscript
;                          Default = 0
;    Interpolate - See Set_plot - Interpolate current color table to the new
;                  device. Default = 0
;         Note: If X-Y color table is used, these pure colors will be changed
;    bold - specifies bold print for the output if present and nonzero
;            Default is !p.charthick = 2*font
;    thick - Plot line thickness.  Default is 5
;    xythick - Axis thickness.  Default is 5
;    bits - bits per pixel.  Default is 8
;    noload - if set, no color table will be loaded.
;             DEFAULT: LOADXYCT
;    _extra - any acceptable variables in the call to the "device" procedure
;             for example: /avant will use Avant Garde PostScript font
;  Note:  When using Postscript, Greek characters are accessed using "!9" and
;         the norman font is "!3" for example distance in microns for an
;         axis label would be written: xtitle='Distance (!9m!3m)'
;  Examples:
;  Send the next plots to a file called 'QSsummary.ps'
;    startpost, file = 'QSsummary.ps
;    { plot the stuff }
;    endpost
;-
   on_error, 1
;
;

@switchpost.com

   if n_elements (portrait) eq 0 then portrait = 0
   if n_elements (thick) eq 0 then thick = 5
   if n_elements (xythick) eq 0 then xythick = 5
   if !d.name eq 'X' or !d.name eq 'TEK' or !d.name eq 'WIN' $
         or !d.name eq 'MAC' then begin
     oldxythick = !x.thick
     oldthick = !p.thick
     oldpclfont = [!p.charsize, !p.charthick]
     olddevice =!d.name
   endif else if n_elements(olddevice) eq 0 then olddevice = 'WIN'
   if n_elements (pfile) gt 0 then file = pfile(0)
   if n_elements (color) eq  0 then color = 0
   if n_elements (copy) eq 0 then copy = 0
   if n_elements (interpolate) eq 0 then interpolate = 0
   oldcopy = copy
   oldinterpolate = interpolate
   xythicksave = xythick
   thicksave = thick
   !x.thick = xythick
   !y.thick = xythick
   !z.thick = xythick
   !p.thick = thick
   if n_elements (font) eq 0 then font = 3
   if keyword_set (bold) then font = 2*font
   !p.charthick = font
   if  n_elements (charsize) ne 0 then !p.charsize = charsize
;
  set_plot, 'printer', interpolate = interpolate, copy = copy
  lcopen = 1
  if portrait eq 0 then $
  device, set_font=font, $
    /landscape, _extra = extrastuff $
  else device,set_font=font, $
    bits=bits,/portrait, /inches, ysize=10.0, yoffset=0.5, $
     xsize = 7.5, xoff = 0.5,  _extra = extrastuff

   if keyword_set(noload) then noloadsave = 1 else begin
     if !d.table_size gt 9 then loadxyct, /quiet
     noloadsave = 0
   endelse
;
   end
