; ------------------------------------------------------------------------ 
; $Id: wdedit.pro,v 1.3 2007/06/06 20:03:55 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  wdedit, wdf1, wdf2, over = wdf3, xsize = xsizein, ysize= ysizein, $
 clabel = clabel, xlabel = xlabel, ylabel = ylabel, file =file, $
 _extra = extra_stuff
; 
;+
;  Edit a WDF array shape by adding or deleting points.  A reference
;  dataset can be plotted over the curve being edited.  The editing is
;  interactive using the cursor.
;  Format:  WDEDIT, wd1 [, wd2]  [, OVER = wd3] 
;  where:
;    wd1, wd2 - WDF array numbers
;               if wd2 is present, the final curve is stored in wd2
;               otherwise the result is stored in wd1
;    wd3 - WDF array number of the curve to be plotted over wd1 
;  KEYWORDS:
;    X/Ysize  - size of the graphics screen
;    CLABEL   - Comment label -  Default = WDF1 comment label
;    XLABEL   - X label -  Default = WDF1 X label
;    YLABEL   - Y label -  Default = WDF1 Y label
;    FILE     - File -  Default = WDF1 file
;    XLOG, YLOG - for a log X or Y axis.
;               If less than 1 then the range of x or y will be limited
;               to xlog*max(x) and/or ylog*max(y)
;               If /ylog or /xlog is used then minimum is 1e-4* max(value)
;  Examples:
;    Edit data points in array 2 and return the data to array 2
;      WDedit, 2
;    Edit data points in array 2 and return the data to array 3;
;    use array 1 as a reference
;      WDedit, 2, 3, 1
;
;    Edit Waveform 6 and store the result in 6 but append " - Modified"
;      to the dataset comment.
;      WDEDIT, 6, c = wdcomment(6)+ ' - Modified'
;-
;
@com.dat
;
      on_error, 2
      if n_elements(initwdf) eq 0 then wdfinit
      nparams = n_params()
      if nparams lt 1 then begin
        print,'% Format: WDEDIT, wd1 [, wd2]  [, OVER = wd3]'
        return
      endif
;
      if n_elements(wdf1) eq 0 then begin
        print, 'Parameters to WDFedit must be defined'
        return
      endif
      if wdf1 lt 1 or wdf1 gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
;
      if nparams gt 1 then save = get_save_array(wdf2, wdf1) else save = wdf1
      if save lt 1 or save gt maxwdfarray then save = wdf1
      nel = wdfnp(wdf1-1)
      if nel eq 0 then begin
        print, 'No valid data in the array'
        return
      endif
      get_wdfarray, array, wdf1, /xy
      if nel +2 ne n_elements(array) then begin
        nel = n_elements(array)/2
        a = make_array (nel + 256)
        b = a
        a(0:nel-1) = array(nel:nel+nel-1)
        b(0:nel-1) = array(0:nel-1)
      endif else begin
        a = make_array (nel + 256)
        b = a
        a(0:nel-1) = array(2:*)
        b(0:nel-1)=findgen(nel)*array(1)+array(0)
      endelse
      lcover = 0
      if n_elements(wdf3) gt 0 then begin
        if wdf3 lt 1 or wdf3 gt maxwdfarray then begin
          print, 'wdf array must be greater than 0 and </=', maxwdfarray
          print, ' No overlay will be used.'
        endif else begin
          n3 = wdfnp(wdf3-1)
          if n3 gt 1 then begin
            get_wdfarray, array, wdf3
            if n_elements (array) eq n3+2 then begin
              yover = array(2:*)
              xover = findgen(n3)*array(1)+array(0)
            endif else begin
              xover = array[0:n3-1]
              yover = array[n3:n3+n3-1]
            endelse
            lcover = 1
          endif else print, ' No data in array', wdf3, '-No overlay '
        endelse
      endif
      edit_curve, b, a, nel, xover = xover, yover= yover, $
           xsize = xsizein, ysize= ysizein, _extra= extra_stuff
      if save ne wdf1 then xfr, wdf1, save
      put_wdfarray, [b(0:nel-1), a(0:nel-1)], save, /xydata, /no_copy, $
        clabel = clabel, xlabel = xlabel, ylabel = ylabel, file =file
      wdf2 = save
      return
      end
