; ------------------------------------------------------------------------ 
; $Id: wdinfo.pro,v 1.7 2011/07/20 14:05:56 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro wdinfo, wdfin,  WDF = WDF, NWDF = nwdf, COMMENT = comment, $
             NPOINTS = npoint, XLABEL = xlabel, YLABEL = ylabel, $
             XMIN = xmin, XMAX = xmax, YMIN = ymin, YMAX = ymax, $
             DX = dx, FILE = file
;
;+
;  This function returns the stored infomation about a WDF array. 
;
;  Format:  WDINFO, WDFin [, WDF = WDF] [, NWDF = nwdf]
;           [, COMMENT = comment] [, NPOINTS = npoint]
;           [, XLABEL = xlabel] [, YLABEL = ylabel]
;           [, XMIN = xmin] [, YMIN = ymin]
;           [, XMAX = xmax] [, YMAX = ymax]
;           [, DX = dx] [, FILE = file]
;  where:
;    WDFin - Valid WDF array number or an array numbers, or a search string.
;            Note:  WDF = wdvalid(WDFIN, /valid)
;    WDF   - Waveform array numbers for the data
;            Note:  WDF = wdvalid(WDFIN, /valid)
;    NWDF  - Number of waveform array numbers
;    COMMENT - Dataset Comments for the waveform arrays
;    NPOINTS - Number of data values in the WDF array
;    XLABEL- Abscissa label
;    YLABEL- Ordinate label
;    XMIN  - Minimum abscissa value
;    XMAX  - Maximum abscissa value
;    YMIN  - Minimum ordinate value
;    YMAX  - Maximum ordinate value
;    DX    - DX value for uniform data.
;            DX = 0, for Nonuniform Data
;            The function GET_DX(WDFin) is used.
;    FILE  - File from which the data came
;    
;
;  Example:
;    Obtain the file name for wdf 5 and store in variable: FILENAME.
;      WDINFO, 5, FILE = FILENAME
;
;-
;
;
@com.dat
;
      if n_elements(initwdf) eq 0 then wdfinit
;
      wdf = wdvalid(wdfin, /valid, Npoint = nwdf)
      if nwdf eq 0 then begin
        print, '% WDINFO: Valid waveform array must be provided'
        return 
      endif
;
      wd = wdf-1
      
      if arg_present(comment) then comment =  wdfcomment[wd]
      if arg_present(file) then file = wdffile[wd]
      if arg_present(ymax) then ymax = wdfmax[wd]
      if arg_present(ymin) then ymin = wdfmin[wd]
      if arg_present(xmax) then xmax = wdfmaxx[wd]
      if arg_present(xmin) then xmin = wdfminx[wd]
      if arg_present(xlabel) then xlabel = wdfxlabel[wd]
      if arg_present(ylabel) then ylabel = wdfylabel[wd]
      if arg_present(npoint) then npoint = wdfnp[wd]
      if arg_present(dx) then begin
        dx = (nwdf gt 1) ? make_array(nwdf) : 0.0
        for i = 0, nwdf-1 do dx[i] = get_dx(wdf[i])
      endif
      end
