; ------------------------------------------------------------------------ 
; $Id: which.pro,v 1.5 2007/06/06 20:03:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  which, spec, output = current, noutput = noutput, full = full, quiet = quiet
;
;+
; NAME:
;       WHICH
;
; PURPOSE:
;       Shows source for a compiled routine.
;       If routine is not compiled, a message will be printed.
; CATEGORY:
;       UTILITY 
; CALLING SEQUENCE:
;       WHICH [, SPEC][, OUTPUT = output] [, NOUTPUT = noutput] [/FULL] [, /QUIET]
; OPTIONAL PARMAETERS
;       SPEC    - Name of a compiled procedure or an array of names.
;                 If SPEC is missing, a blank or a null string, $
;                    the source of all routines will be shown
;                 (See Get_Match for search techniques.)
; KEYWORDS:
;       OUTPUT  - Variable to receive the output from the WHICH command.
;       NOUTPUT - Number of lines of output.
;         Note: This is more than the number of routines if SPEC is not defined.
;       FULL    - Passed to IDL HELP to obtain hidden procedures
;       QUIET   - If set, no output.  Keyword OUTPUT must be used.
; OUTPUTS:
;       Source file name for routines which match SPEC.
; EXAMPLES:
;       Show the source files for currently compiled routines
;             WHICH
;        <or> WHICH, ''
;
;       Show all source files files which begin with PLO.
;             WHICH , '^PLO'
;       Show all source files files which end with PLO.
;             WHICH , 'PLO$'
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, December 3, 2001
;-
;
;
      on_error, 1
;
       help, /source, output = output, full = full

       if n_elements (spec) eq 0 then begin
         if keyword_set (quiet) then return
         print, output, format ='(a)'
         noutput = n_elements(output)
         return
       endif
       if n_elements(current) gt 0 then $
         tempvar = size(temporary(current), /type)
       nel = n_elements(spec)
       noutput = 0
       for i = 0, nel-1 do begin
         nam = strtrim(spec[i], 2)
         l = strlen(nam)
         if l eq 0 then begin
           print, output, format ='(a)'
           return
         endif
         ok = strpos(nam, '$')
;  add a space to insure that we don't get the name of the file.
         if ok  eq l-1 then nam = strmid(strtrim(spec[i], 2), 0, l-1) +' ' $
             else nam = strtrim(spec[i], 2) +'* '
         ok = get_match(strupcase(nam), output, nelements = nok, /casesen)
         if nok eq 0 then begin
;          fix the case where data is on two lines
           nam = strtrim(spec[i], 2)
           ok = get_match(strupcase(nam), output, nelements = nok, /casesen)
           nn = 0
           if nok gt 0 then begin
             for i = 0, nok-1 do begin
               if strpos(output[ok[i]+1], ' ') eq 0 then begin
                 ok[nn] = ok[i]
                 nn = nn +1
                 output[ok[i]] =  output[ok[i]] + output[ok[i]+1]
               endif
             endfor
             nok = nn
             ok = ok[0:nn-1]
           endif
         endif
         if nok eq 0 then begin
           if not keyword_set(quiet) then $
             print, 'No routines compiled with string: "',nam,'"'
         endif else begin
           if not keyword_set(quiet) then print, output[ok], format ='(a)'
           noutput = noutput+nok
           if n_elements(current) eq 0 then $
             current = output[ok] else current = [current, output[ok]]
         endelse
       endfor
       end
