; ------------------------------------------------------------------------ 
; $Id: writetk.pro,v 1.3 2007/06/06 20:03:56 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro writetk, fnamein, wdfin, narray
;
;+
;    write a tkdas data file
;     Format consists of tab delimited columns of the form:
;     Note: Each Title may not have spaces in this version.
;  title
;  t0
;  delta_t
;  Npoints
;  y1  (at t0)
;  y2
;  y3
;  .
;  .
;  .
;  yN (at t0+(N-1)*delta_t)
;
;   format is:  writetk , fname , WDF [, narray]
;    where 
;      fname   - the file name
;                if a file name then the file is opened and closed at the end
;      NOTE: If only one parameter is given and the parameter is a number, 
;            then ReadTK assumes that it is a WDF number.
;      WDF     - WDF array number or an array of numbers.
;                Default is 1  
;                If WDF is less than 0 and n_elements(data) > 0 then no WDF
;                  arrays will be stored.
;      Narray  - If WDF is a single number then Narray total arrays will be 
;                written beginning with WDF.
; EXAMPLES:
;    Write a TKDAS file with arrays [2, 3, 4, 5, 6] to data.dat
;       writeTK, 'data.dat', 2, 5 
;   or  WRITETK, 'data.dat', [2, 3, 4, 5, 6]
;
;       ORIGINAL version: April 8, 1997
;-
;
     on_error, 1
;    check format
@com.dat
     if n_elements(initwdf) eq 0 then wdfinit

     nparams = n_params()
      if nparams eq 0 then begin
        print, '% Format: WRITETK, FNAME, WDF [, NARRAY]
        return
      endif
      if n_elements (fnamein) eq 0 then begin
        print, '% WRITETK: Output file must be specified.'
        return
      endif
      fname = fnamein(0)
      b = size(fname)
      if b(1) ne 7 then begin
        print, '% WRITETK: Output file must be a string.'
        return
      endif
      delimiter = string(9B)
      ndelim = strlen(delimiter)
      nel = n_elements(wdfin)
      if nel eq 0 then begin
        print,'% WRITETK: WDF array value must be provided'
        return
      endif
      wdf = wdfin
      if nel eq 1 then if n_elements (narray) gt 0 then begin
         wdf = wdf(0)+ indgen(narray)
      endif
      wdfar = wdf
      if min(wdfar) lt 1 or max(wdfar) gt maxwdfarray then begin
        print, 'wdf array must be greater than 0 and </=', maxwdfarray
        return
      endif
      nel = n_elements(wdfar)
      ok = where (wdfnp(wdfar-1) gt 0, nok)
      if nok eq 0 then begin
        print, '% WRITETK: No valid data in specified arrays'
        return
      endif
      wdfar = wdfar(ok)

      xydata = 0
      nmax = max(wdfnp(wdfar-1))
      array = make_array(nmax+2, nok)
      tit= make_array(nok, /string)
      np = make_array(nok, /long)

      for i = 0, nok-1 do begin
        get_wdfarray, arr, wdfar(i), clab = clab
        nelar =  n_elements (arr)-2
        if nelar eq wdfnp(wdfar(i)) or nelar le nmax then  $
          array(0:nelar+1, i) = arr else xydata = nelar
        ss = strtrim(clab, 2)
        if strlen(ss) gt 0 then tit(i) = ss else tit(i) = 'Array '+strtrim(i, 2)
        np(i) = nelar
      endfor
      if xydata gt nmax then begin
        array  = make_array(xydata+2, nok)
        for i = 0, nok-1 do begin
          get_wdfarray, arr, wdfar(i)
          nelar =  n_elements (arr)-2
          array(0:nelar+1, i)= arr
        endfor
      endif
; = make_array(nok)
      groups = np(uniq(np, sort(np)))
      ngroup = n_elements(groups)
      delta = groups
      if ngroup gt 1 then delta(1:*) = groups(1:ngroup-1)-groups(0:ngroup-2)
      str = tit(0)
      if nok gt 1 then for i = 1, nok-1 do str = str+delimiter+tit(i)
      recl = max([ nok*(14+ndelim)+18, strlen(str)])
      openw,unit,fname, /get_lun, width = recl
      printf, unit, str     
      ax = make_array(nok)
      fmstr = '(g14.7, '+strtrim(nok, 2)+'("'+delimiter+'", g14.7))'
      for i = 0, 1 do $
        printf, unit, format = fmstr, array(i, *)
      printf, unit, format = fmstr, np


      np = round(np)
      groups = np(uniq(np, sort(np)))
      ngroup = n_elements(groups)
      delta = groups 
      if ngroup gt 1 then delta(1:*) = groups(1:ngroup-1)-groups(0:ngroup-2)
      maxnp = max(np, min = minnp)+2
      groups2 = groups+2
      i = 2
      for  ng = 0, (ngroup-1) do begin
        ok = where (np ge groups(ng), nok)
        ax = temporary(ax(0:nok-1))
        while (i lt groups2(ng)) do begin
          ax = array(i, ok)

          printf,unit, format = fmstr, ax
          i = i+1
        endwhile
      endfor
badeof:
      free_lun, unit
      return
      end
