; ------------------------------------------------------------------------ 
; $Id: chastr.pro,v 1.5 2009/02/13 16:52:43 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  Chastr, strarray, clabel = clab, ylabel= ylab, xlabel = xlab, $
        zlabel= zlab, tlabel = tlab, alabel = alab, $
        v1label = v1lab, v2label = v2lab, v3label = v3lab, $
        v4label = v4lab, v5label = v5lab, v6label = v6lab, valabel = valab, $
        mxdim = mxd, mydim = myd, mzdim = mzd, mtdim = mtd, madim = mad, $
        axdim = axd, aydim = ayd, azdim = azd, atdim = atd, aadim = aad, $
        s1v = s1v, s2v = s2v, s3v = s3v, s4v = s4v, s5v = s5v, s6v = s6v, $
        scale = scale, space = space, file = file, _EXTRA = extrastuff
;
;+
; NAME:
;       CHASTR
;
; PURPOSE:
;       Change the prameters of a structure array 
;       The following types of structures arrays are supported 
;         valid_typ = [1, 2, 3, 4, 7, 8, 9, 10]
;       Changes which do not apply to a particular structure type are ignored 
;       
; CATEGORY: 
;       Utility 
; CALLING SEQUENCE: 
;       CHASTR, STR 
; INPUTS: 
;       STR:      Structure array number or an array of structure array 
;                 numbers to be changed.  If only one structure is specified,
;                 then it may be an IDL structure.
; KEYWORD PARAMETERS:
;       CLABEL    Replace the dataset comment for the structures specified by
;                 STR with the string or string variable specified
;       XLABEL    Replace the first spatial dimension label for the specified
;                 structures with the string or string variable specified
;       YLABEL    Replace the second spatial dimension label for the specified
;                 structures with the string or string variable specified
;       ZLABEL    Replace the third spatial dimension label for the specified
;                 structures with the string or string variable specified
;       TLABEL    Replace the fourth spatial dimension label for the specified
;                 structures with the string or string variable specified
;       ALABEL    Replace the all spatial dimension labels for the specified
;                 structures with the string or string variable specified
;       V1LABEL   Replace the first attribute label or the first vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       V2LABEL   Replace the second attribute label or the second vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       V3LABEL   Replace the third attribute label or the third vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       V4LABEL   Replace the fourth attribute label or the fourth vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       V5LABEL   Replace the fifth attribute label or the fifth vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       V6LABEL   Replace the sixth attribute label or the sixth vector label
;                 for the specified structures with the string or string 
;                 variable specified
;       VALABEL   Replace the all attribute labels or all vector labels
;                 for the specified structures with the string or string 
;                 variable specified
;       MXDIM     Multiply the first spatial dimension by this value
;       MYDIM     Multiply the second spatial dimension by this value
;       MZDIM     Multiply the third spatial dimension by this value
;       MTDIM     Multiply the fourth spatial dimension by this value
;       MADIM     Multiply all spatial dimensions by this value
;                 MADIM depends on the first spare word in the first structure
;                 If SPARE[0] eq 2 then ONLY MXDIM = MZDIM = MADIM
;                 If SPARE[0] eq 3 then ONLY MXDIM = MADIM
;                 Else:  MXDIM = MYDIM = MZDIM = MTDIM = MADIM
;       AXDIM     Add this offset to the first spatial dimension
;       AYDIM     Add this offset to the second spatial dimension
;       AZDIM     Add this offset to the third spatial dimension
;       ATDIM     Add this offset to the fourth spatial dimension
;       AADIM     Add this offset to all the spatial dimensions
;       NOTE:  Multiplication and division are performed according to:
;              Xnew = Xold * MXDIM + AXDIM
;       S1V       Multiply or scale the first vector or attribute component
;       S2V       Multiply or scale the second vector or attribute component
;       S3V       Multiply or scale the third vector or attribute component
;       S4V       Multiply or scale the fourth vector or attribute component
;       S5V       Multiply or scale the fifth vector or attribute component
;       S6V       Multiply or scale the sixth vector or attribute component
;       SCALE     Multiply all the vector amplitudes by a scale factor.
;                 This is identical to:  mulstr, str, 0, scale
;       SPACE     If SPACE is present and not zero, then S1V, S2V, ..., SCALE
;                 must have values of 1, 2, or 3. The scaling will be of the 
;                 form: V1 = X1(i)*v1(i, *, *) for s1v = 1
;                   or  V1 = X2(i)*v1(*, i, *) for s1v = 2 
;                   or  V1 = X3(i)*v1(*, *, i) for s1v = 3 
;                   for example to multiply vector 2 by r (r*btheta)
;                     USE: chastr, Bfield, s2v =1, /space
;                 or if SPACE is a string function of X then:
;                   if s1v = 1 then 
;                        x = X1
;                       V1 = string *v1(i, *,*)
;                   if s1v = 2 then 
;                        x = X2 ... etc 
;                 for example to multiply vector 1 by sin(theta)
;                  (string may be any executable idl or user function)
;                    chastr, str, s1v =2, space = 'sin(x)'
;                 If V1LABEL is not specified it will be modified to reflect 
;                   this scaling, ie, v1label = labx+' * '+labv 
;                   where labx is the spatial label and labv is the vecor label
;                   (See example)
;       FILE      Replace the file name with the string specified
; OUTPUTS:
;       NONE    The specified structures are modified and stored in their
;               original locations
; COMMON BLOCKS:
;       QSCOM, QSSTRCOM 
; RESTRICTIONS:
;       STR array must be defined and have valid data
; EXAMPLES:
;       Change the dataset comment to 'Stream Functions at 65 ns' on structure
;          array 2
;            CHASTR, 2, c = 'Stream Functions at 65 ns'
;       Change the x and y spatial coordinates of an image by 1/maginfication
;            CHASTR,IMAGE,mx=1/.56, my = 1/.56,xl='Target(cm)',y='Target(cm)'
;       Change the spatial coordinates of arrays 2 through 9 to cm from meters
;            CHASTR,indgen(8)+2,ma=1e2,x= 'X (cm)',y='Y (cm)',z='Z (cm)'
;         Note in cylindrical coordinates we would use:
;            CHASTR,indgen(8)+2,mx=1e2, mz = 1e2, x ='Radius(cm)',z='Z(cm)'
;       Change the units of electric field to MV/cm from V/m on structure, F
;            CHASTR, F, sa= 1./1e8,v1='Ex(MV/cm)',v2='Ey (MV/cm)',v3='Ez(MV/cm)'
;       Change the radial component of the field in structure, F, from Er to r*Er
;            CHASTR, F,s1v =1, /space
;       
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, November 16, 1993
;-
;
on_error, 1
;     include the common block
;
@qscom.dat
;  Common block
@qsstrcom.dat
;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements (qsplotinit) eq 0 then qsinit
;
      nparams = n_params()
      if nparams ne 1 then begin
        print,'% Format:', $ 
 ' CHASTR, STRARRAY [, CLABEL= clab] [, YLABEL= ylab] [, XLABEL= xlab]'
 print, '         [, ZLABEL= zlab] [, TLABEL= tlab] [, ALABEL= alab]'
 print, '         [, V1LABEL= v1lab] [, V2LABEL= v2lab] [, V3LABEL= v3lab]'
 print, '         [, V4LABEL= v4lab] [, V5LABEL= v5lab] [, V6LABEL= v6lab]'
 print, '         [, VALABEL= valab]'
 print, '         [, MXDIM= mxd] [, MYDIM= myd] [, MZDIM= mzd] [, MTDIM= mtd]'
 print, '         [, MADIM= mad]'
 print, '         [, AXDIM= axd] [, AYDIM= ayd] [, AZDIM= azd] [, ATDIM= atd]'
 print, '         [, AADIM= aad]'
 print, '         [, S1V= s1v] [, S2V= s2v] [, S3V= s3v] [, S4V= s4v]'
 print, '         [, S5V= s5v] [, s6v = s6v]'
 print, '         [, FILE = file]  [, SCALE= scale] [, SPACE = space]'
;
        return
      endif
;
;     determine if strarray is specified
;
      nelstr = n_elements(strarray)
      if nelstr eq 0 then begin
        print, '% CHASTR: Structures must be defined'
        return
      endif
;  check the parameters
      if n_elements (alab) ne 0 then begin
        xlab = alab
        ylab = alab
        zlab = alab
        tlab = alab
      endif
      if n_elements (valab) ne 0 then begin
        v1lab = valab 
        v2lab = valab 
        v3lab = valab 
        v4lab = valab 
        v5lab = valab 
        v6lab = valab 
        v7lab = valab 
        v8lab = valab 
      endif

      if n_elements(aad) ne 0 then begin
        axd = aad
        ayd = aad
        azd = aad
        atd = aad
      endif
      if n_elements(axd) gt 0 then axd = axd(0) else axd = 0
      if n_elements(ayd) gt 0 then ayd = ayd(0) else ayd = 0
      if n_elements(azd) gt 0 then azd = azd(0) else azd = 0
      if n_elements(atd) gt 0 then atd = atd(0) else atd = 0
;
      if n_elements(scale) ne 0 then begin
        s1v = scale
        s2v = scale
        s3v = scale
        s4v = scale
        s5v = scale
        s6v = scale
        s7v = scale
        s8v = scale
      endif
;
;  loop over the structures
;
    for iarray = 0, nelstr-1 do begin
      str = get_valid_structure(strarray[iarray], 'CHASTR', lcstr=lcstr, $
        ierror=ier)
      if IER ne 0 then begin
        print, '% CHASTR: Returning immediately'
        return
      endif 
      if lcstr eq 0 then a = strarray(iarray) else a = 0 

;     determine if we have valid data
      ityp = str.type
      valid_typ = [1, 2, 3, 4, 7, 8, 9, 10]
      ok = where (ityp eq valid_typ, nok)
      if nok eq 0 then begin
        print, '% CHASTR: This structure type is not supported', ityp
        return
      endif
;     MADIM     Multiply all spatial dimensions by this value
;               If SPARE[0] eq 2 then ONLY MXDIM = MZDIM = MADIM
;               If SPARE[0] eq 3 then ONLY MXDIM = MADIM
;               Else:  MXDIM = MYDIM = MZDIM = MTDIM = MADIM
      if iarray eq 0 then begin
        if n_elements(mad) ne 0 then begin
          mxd = mad[0]
          if str.spare[0] ne 3 then begin
            mzd = mad[0]
            if str.spare[0] ne 2 then begin
              myd = mad[0]
              mtd = mad[0]
            endif
          endif
        endif
;
        if n_elements(mxd) gt 0 then mxd = mxd(0) else mxd = 1
        if n_elements(myd) gt 0 then myd = myd(0) else myd = 1
        if n_elements(mzd) gt 0 then mzd = mzd(0) else mzd = 1
        if n_elements(mtd) gt 0 then mtd = mtd(0) else mtd = 1
      endif

      if n_elements(file) gt 0 then str.file = strtrim(file, 2)
      if n_elements(clab) ne 0 then begin
        str.clab = clab
        if a gt 0 then comment(a-1) = clab
      endif
;

      ok = where(strpos(tag_names(str), 'X4') ge 0 , nok4)
      if ((max([mxd, myd, mzd, mtd], min = mm) ne 1) or $
          (mm ne 1)) then begin

        if mxd ne 1 and ityp ne 4 then str.x1 = str.x1 *mxd
        if myd ne 1 and ityp ne 4 then str.x2 = str.x2 *myd
        if mzd ne 1 and ityp ne 4 then str.x3 = str.x3 *mzd
        if mtd ne 1 and nok4 gt 0 then str.x4 = str.x4 *mtd
        ok = where(strpos(tag_names(str), 'DX') ge 0 , nok)
        if nok ne 0 then begin
          str.dx1 = str.dx1*mxd
          str.dx2 = str.dx2*myd
          str.dx3 = str.dx3*mzd
          if nok4 gt 0 then str.dx4 = str.dx4*mtd
        endif
      endif

      if ((max([axd, ayd, azd, atd], min = mm) ne 0) or $
         (mm ne 0)) then begin
        if axd ne 0 and ityp ne 4 then str.x1 = str.x1 +axd
        if ayd ne 0 and ityp ne 4 then str.x2 = str.x2 +ayd
        if azd ne 0 and ityp ne 4 then str.x3 = str.x3 +azd
        if atd ne 0 and nok4 gt 0 then str.x4 = str.x4 +atd
      endif
;
      if n_elements (xlab) ne 0 then begin
        if ityp eq 4 then str.xlab(0)= xlab $
          else  str.x1lab = xlab
      endif
;
      if n_elements (ylab) ne 0 then begin
        if ityp eq 4 then str.xlab(1)= ylab $
          else  str.x2lab = ylab
      endif
;
      if n_elements (zlab) ne 0 then begin
        if ityp eq 4 then str.xlab(2)= zlab $
          else  str.x3lab = zlab
      endif
;
      if n_elements (tlab) ne 0 and nok4 gt 0 then begin
        if ityp lt 3 then str.x4lab = tlab
      endif
;
      valid_vect=[1, 2, 8, 9]
      ok = where (ityp eq valid_vect, nok)
      if nok gt 0 then begin
        if keyword_set(space) then begin
          ss = size(space(0))
          if ss(1) eq 7 then lcstr = 1 else lcstr = 0
          if ityp eq 1 then nv = str.nvect else nv = str.natt
          maxblock = str.block
          if n_elements(s1v) gt 0 then begin
            if s1v gt 0 and s1v le str.ndim then begin
              if nv ge 1 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s1v, i, lab = labx)
                  v1 = get_array (str,-1 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s1v eq 1 then begin
                    for ii = 0, nel-1 do v1(ii, *, *) = x(ii)*v1(ii, *, *)
                  endif else if s1v eq 2 then begin
                    for ii = 0, nel-1 do v1(*, ii, *) = x(ii)*v1(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v1(*, *, ii) = x(ii)*v1(*, *, ii)
                  endelse
                if  n_elements(v1lab) eq 0 then $
                  put_array, str, v1, -1,i, lab = labx+' * '+labv else $ 
                  put_array, str, v1, -1,i, lab = v1lab
                endfor
              endif 
            endif else print, $
              'S1V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
          if n_elements(s2v) gt 0 then begin
            if s2v gt 0 and s2v le str.ndim then begin
              if nv ge 2 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s2v, i, lab = labx)
                  v2 = get_array (str,-2 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s2v eq 1 then begin
                    for ii = 0, nel-1 do v2(ii, *, *) = x(ii)*v2(ii, *, *)
                  endif else if s2v eq 2 then begin
                    for ii = 0, nel-1 do v2(*, ii, *) = x(ii)*v2(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v2(*, *, ii) = x(ii)*v2(*, *, ii)
                  endelse
                if  n_elements(v2lab) eq 0 then $
                  put_array, str, v2, -2,i, lab = labx+' * '+labv else $ 
                  put_array, str, v2, -2,i, lab = v2lab
                endfor
              endif 
            endif else print, $
              'S2V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
          if n_elements(s3v) gt 0 then begin
            if s3v gt 0 and s3v le str.ndim then begin
              if nv ge 3 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s3v, i, lab = labx)
                  v3 = get_array (str,-3 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s3v eq 1 then begin
                    for ii = 0, nel-1 do v3(ii, *, *) = x(ii)*v3(ii, *, *)
                  endif else if s3v eq 2 then begin
                    for ii = 0, nel-1 do v3(*, ii, *) = x(ii)*v3(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v3(*, *, ii) = x(ii)*v3(*, *, ii)
                  endelse
                if  n_elements(v3lab) eq 0 then $
                  put_array, str, v3, -3,i, lab = labx+' * '+labv else $ 
                  put_array, str, v3, -3,i, lab = v3lab
                endfor
              endif 
            endif else print, $
              'S3V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
          if n_elements(s4v) gt 0 then begin
            if s4v gt 0 and s4v le str.ndim then begin
              if nv ge 4 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s4v, i, lab = labx)
                  v4 = get_array (str,-4 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s4v eq 1 then begin
                    for ii = 0, nel-1 do v4(ii, *, *) = x(ii)*v4(ii, *, *)
                  endif else if s4v eq 2 then begin
                    for ii = 0, nel-1 do v4(*, ii, *) = x(ii)*v4(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v4(*, *, ii) = x(ii)*v4(*, *, ii)
                  endelse
                if  n_elements(v4lab) eq 0 then $
                  put_array, str, v4, -4,i, lab = labx+' * '+labv else $ 
                  put_array, str, v4, -4,i, lab = v4lab
                endfor
              endif 
            endif else print, $
              'S4V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
          if n_elements(s5v) gt 0 then begin
            if s5v gt 0 and s5v le str.ndim then begin
              if nv ge 5 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s5v, i, lab = labx)
                  v5 = get_array (str,-5 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s5v eq 1 then begin
                    for ii = 0, nel-1 do v5(ii, *, *) = x(ii)*v5(ii, *, *)
                  endif else if s5v eq 2 then begin
                    for ii = 0, nel-1 do v5(*, ii, *) = x(ii)*v5(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v5(*, *, ii) = x(ii)*v5(*, *, ii)
                  endelse
                if  n_elements(v5lab) eq 0 then $
                  put_array, str, v5, -5,i, lab = labx+' * '+labv else $ 
                  put_array, str, v5, -5,i, lab = v5lab
                endfor
              endif 
            endif else print, $
              'S5V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
          if n_elements(s6v) gt 0 then begin
            if s6v gt 0 and s6v le str.ndim then begin
              if nv ge 6 then begin
                for i = 1, maxblock do begin
                  x = get_array (str, s6v, i, lab = labx)
                  v6 = get_array (str,-6 , i, lab = labv)
                  nel = n_elements(x)
                  if lcstr ne 0 then begin
                    ok = execute('x = '+ space(0))
                    ntemp = n_elements(x)
                    if ntemp lt nel then begin
                      print, 'Function is too short. Additional values will be' $
                        + ' added'
                      if ntemp gt 1 then begin
                        temp = x
                        x= make_array(nel, value = x(0))
                        x(0:ntemp-1) = temp
                      endif else x= make_array(nel, value = x(0))
                    endif
                  endif
                  if s6v eq 1 then begin
                    for ii = 0, nel-1 do v6(ii, *, *) = x(ii)*v6(ii, *, *)
                  endif else if s6v eq 2 then begin
                    for ii = 0, nel-1 do v6(*, ii, *) = x(ii)*v6(*, ii, *)
                  endif else begin
                    for ii = 0, nel-1 do v6(*, *, ii) = x(ii)*v6(*, *, ii)
                  endelse
                if  n_elements(v6lab) eq 0 then $
                  put_array, str, v6, -6,i, lab = labx+' * '+labv else $ 
                  put_array, str, v6, -6,i, lab = v6lab
                endfor
              endif 
            endif else print, $
              'S6V must be in the range [1,'+strtrim(str.ndim,2)+']'
          endif
        endif else begin
          if n_elements(s1v) gt 0 then str.v1 = str.v1 * s1v(0)
;
          if n_elements(s2v) gt 0 then str.v2 = str.v2 * s2v(0)
;
          if n_elements(s3v) gt 0 then str.v3 = str.v3 * s3v(0)
;
          if n_elements(s4v) gt 0 then str.v4 = str.v4 * s4v(0)
;
          if n_elements(s5v) gt 0 then str.v5 = str.v5 * s5v(0)
;
          if n_elements(s6v) gt 0 then str.v6 = str.v6 * s6v(0)
;
          ok = where (tag_names(str) eq 'V7')
          if ok[0] gt 0 then begin
            if n_elements(s7v) gt 0 then str.v7 = str.v7 * s7v(0)
;
            if n_elements(s8v) gt 0 then str.v8 = str.v8 * s8v(0)
;
            if n_elements(v7lab) gt 0 then str.v7lab = v7lab
;
            if n_elements(v8lab) gt 0 then str.v8lab = v8lab
          endif
;
          if n_elements(v1lab) gt 0 then str.v1lab = v1lab
;
          if n_elements(v2lab) gt 0 then str.v2lab = v2lab
;
          if n_elements(v3lab) gt 0 then str.v3lab = v3lab
;
          if n_elements(v3lab) gt 0 then str.v3lab = v3lab
;
          if n_elements(v4lab) gt 0 then str.v4lab = v4lab
;
          if n_elements(v5lab) gt 0 then str.v5lab = v5lab
; old way was dieing with particle data
;         if n_elements(v5lab) gt 0 then str.v5lab(*) = v5lab
;
          if n_elements(v6lab) gt 0 then str.v6lab = v6lab
        endelse
      endif
      if ityp eq 1 or ityp eq 8 then if str.two ne 0 then begin
        if str.two eq 1 then str.slice = mxd *str.slice + axd else $
        if str.two eq 2 then str.slice = myd *str.slice + ayd else $
        if str.two eq 3 then str.slice = mzd *str.slice + azd 
      endif
      if ((ityp eq 1) or (ityp eq 3)) or ityp eq 8 then begin
        str.range(*, 0, *) = str.range (*, 0, *) * mxd + axd
        str.range(*, 1, *) = str.range (*, 1, *) * myd + ayd
        if str.ndim gt 2 then begin
          str.range(*, 2, *) = str.range (*, 2, *) * mzd + azd
          if str.ndim gt 3 then $
            str.range(*, 3, *) = str.range (*, 3, *) * mtd + atd
        endif
      endif
      if ityp eq 4 then begin
        str.plane(*, 0, *) = str.plane(*, 0, *) *mxd + axd
        str.plane(*, 1, *) = str.plane(*, 1, *) *myd + ayd
        str.plane(*, 2, *) = str.plane(*, 2, *) *mzd + azd
;
        if n_elements(str.slant) gt 2 then  begin
           str.slant(0, *, *)=str.slant(0, *, *)*mxd + axd
           str.slant(1, *, *)=str.slant(1, *, *)*myd + ayd
           str.slant(2, *, *)=str.slant(2, *, *)*mzd + azd
        endif
        ok = where (tag_names(str) eq 'SPACE', nok)

        if nok gt 0 then if n_elements(str.space) gt 3 then begin
           str.space[0, *] = str.space[0, *]*mxd + axd
           str.space[1, *] = str.space[1, *]*myd + ayd
           str.space[2, *] = str.space[2, *]*mzd + azd
        endif
      endif
;
      if a eq 0 then begin
        strarray = str
;       since we can have only one structure as an arguement - return
        return
      endif else i2s, str, a, /no_copy
    endfor
      return
      end
