; ------------------------------------------------------------------------ 
; $Id: dirs.pro,v 1.3 2007/06/06 20:09:33 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro dirs, wminin, wmaxin, xsize = xsize, truncate = truncate, $
          page = page , reset = reset, base = tlb
;+
;
; NAME:
;      DIRS
; PURPOSE:
;      Show the structures in the qs common area which contain data
; CALLING SEQUENCE:
;      DIRS  [, SMIN] [, SMAX] [, TRUNCATE = truncate] [, Xsize = xsize]
;            [, PAGE = page]
; INPUTS:
;      SMIN   - First sturcture array number to be listed or an array of 
;               structure array numbers or a QS structure
;         or
;             If a single parameter, SMIN can be a search string.
;             Those arrays containing the string in the dataset comment will
;               be listed. The full set of wild card options is available.
;               See GET_MATCH
;      SMAX   - Last structure array number
;      TRUNCATE - If set, the dataset comments will be truncated at ' - '
;               This is a toggle; If not specified the last value will
;               be used.  Default value =0 -- no truncate
;      PAGE   - Number of lines on the directory
;      XSIZE  - Width of the window in pixels (generally not set)
;      RESET  - If set, a new window will be opened.
; Outputs:
;      Structure parameters in a widget 
; Example:
;      See the contents of all structure arrays
;         DIRS
;      See the structures with E-field in the comment
;         DIRS, 'E-field'
;      See the full contents of structure arrays 1 through 13
;         DIRS, 1, 13
; (or)    DIRS, 1+indgen(13)
;-
;  Common block
@qsstrcom.dat
@qscom.dat
;
      common dirsave, dirbase, dirlist, oldpage,  text1, truncate_save
;
      common pffcom, initpff, pff_exe
      common dirs_common, base, list

;
      if n_elements (structinit) eq 0 then qsstruct_init
      if n_elements(qsplotinit) eq 0 then qsinit
      if n_elements(initpff) eq 0 then initpff = 0
      if initpff eq 0 then init_pff

;
      on_error, 1
;
      if n_elements (truncate_save) eq 0 then truncate_save =0
      if n_elements (truncate) gt 0 then truncate_save = truncate
      if keyword_set (reset) then base = 0
      lcpar = 0
      npar = n_params()
      if npar eq 0 then begin
        wmin = where (lcvalidstr ne 0 , nwdf)+1
      endif else if npar eq 1 then begin
        ss = size(wminin)
        if ss (ss(0)+1) eq 7 then begin
          wmin= wminin(0)
          full = 0
          lst = strlen(wmin)
          if lst eq 0  then wmin = '*'
          ok = get_match (wmin, /str, nelement=nwdf)
          if ok(0) eq -1 then begin
           print, '% DIRS: No COMMENTS contain the string: ', wmin
           return
          endif
          wmin = temporary(ok)
        endif else begin
          nwdf = n_elements(wminin)
          if nwdf eq 0 then begin
            print, '% DIRS: First parameter is undefined'
            return
          endif else wmin = wminin
        endelse
        if nwdf eq 1 then begin
          b = size(wmin)
          if  b(b(0)+1) gt 3 then begin
            print,'% DIRS: First parameter to DIRS ', $
                 'must be a search string or a structure array number'
            return
          endif
        endif  else begin
          ok = where (lcvalidstr(wmin-1) ne 0, nwdf)
          if nwdf eq 0 then begin
            print, '% DIRS: Specified structure arrays are empty'
            return
          endif else wmin = wmin(ok)
        endelse
      endif else begin
        nwdf = min ([n_elements(wminin), n_elements(wmaxin)])
        if nwdf eq 0 then begin
          print, '% DIRS: Parameters are undefined'
          return
        endif else begin
          wmax = wmaxin(0)
          wmin = wminin(0)
        endelse
        wmin = min([wmin, wmax], max = wmax) >1
        if wmax le 0 then wmax = maxstructure
        wmax = wmax < maxstructure
        ok = where(lcvalidstr(wmin-1:wmax-1) ne 0, nwdf)
        if nwdf eq 0 then begin
          print, '% DIRS: Specified structure arrays are empty'
          return
        endif else begin
          wmin = findgen(wmax-wmin+1)+wmin
          wmin = wmin(ok)
        endelse
      endelse
      wmin = fix(wmin)
      line = make_array(nwdf> 1L, /string)
      if nwdf eq 0 then line(0) = 'All Structure Arrays are empty' else begin
        for i = 0, nwdf-1 do begin
          str = comment(wmin(i)-1)
          if keyword_set(truncate_save) then begin
            ii = strpos(str, ' - ')
            if ii gt 0 then str = strmid(str, 0, ii)
          endif
          line(i) = strtrim(wmin(i), 2) + ' - '+str
        endfor
      endelse
      if n_elements (oldpage) eq 0 then oldpage = directorypage
      if n_elements (page) eq 0 then page = oldpage
      page = page >4
;
      if n_elements (base) eq 0 then begin
        base = 0
        list = 0
      endif
      text = 0
;
      if oldpage ne page then page=-page

;
      widget_directory, base, page, list, line, xsize = xsize, $
        base_title = 'Structure Array Directory', text1 = text, $
        dirtype = 12, value1 = 'Str #          Structure Comment' 

      oldpage = page
      tlb = base
      return
      end
