; ------------------------------------------------------------------------ 
; $Id: pixplot.pro,v 1.4 2010/12/08 16:26:39 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function pixplot, fname, skip = skip, ignore = ignore, circle = circle, $
             xdelta = xdelta, ydelta = ydelta, nx= nx, ny = ny, np = np, $
             xlog = xlog, ylog = ylog, zlog = zlog, xrange = xrange, $
             yrange = yrange, default = default, highres = highres, $
             xlabel = xlabel, ylabel = ylabel,  $
             plotonly= plotonly, levels=levels, ccolor=ccolor,nlevel=nlevel, $
             _extra = extrastuff, thermometer = thermometer, tparam = tparam, $
            clabel = clabel, tlabel = tlabel, fast = fast
;+
;    read an ascii file of amplitude, x and y values and optionally delta-X and
;      delta-Y values. Generate a field type structure of the data.
;      blank lines are ignored and lines with less than 3 numbers are ignored.
;      TQEDIT can be used to modify/add/delete regions.
;    Alternatively, if the plotonly keyword is set a contour pixplot will
;      be generated with the data, but no structure will be returned.
;
;    CAUTION:
;         The TQBUILD code suite is used. Current data in this commmon
;           block can be lost.
;
;    format: str = pixplot([, fname] [, skip = skip] [, ignore = ignore]
;                [, fast = fast]
;                [, circle = circle] [,xdelta = xdelta] [,ydelta = ydelta]
;                [,nx = nx] [,ny = ny] [,np = np] [, xlog = xlog] 
;                [, ylog = ylog] [, zlog = zlog] [, xrange = xrange]
;                [, yrange = yrange] [, default = default] [, highres = highres]
;                [,xlabel = xlabel] [, ylabel = ylabel] 
;                [, clabel = clabel] [, tlabel = tlabel] 
;                [, plotonly= plotonly] [, levels = levels] [, ccolor = ccolor]
;                [, thermometer = thermometer] [, tparam = tparam]
;                [, Nlevel = nlevel] [, _extra = extrastuff] )
;    where
;      str     - PIXPLOT data is returned in structure, STR
;              - If an error occurs, STR = 0
;      fname   - the file name or a unit number
;                if a file name then the file is opened and closed at the end
;                if a number then the file is read directly
;                if unit is equal to zero or fname is undefined then 
;                  pickfile will be used to determine a file
;                if pickfile is called then fname will contain the file name
;                  on return from pixplot
;                file should contain data of the form:
;                   amplitude xvalue yvalue 
;                or optionally (where absolute values are used for the last 2):
;                   amplitude xvalue yvalue deltax deltay
;      skip    - If the file has several lines of header information which
;                must be skipped, set skip equal to this number of lines.
;                (DEFAULT: SKIP = 0)
;     ignore   - String array. Space is used as a delimiter.  Any nonnumeric 
;                characters to be ignored are listed in ignore. See example.
;     fast     - If set polyfillv will be used to determine values; slight
;                  errors can occur
;     circle   - If set, an ellipse is drawn around the specified points
;                Default is CIRCLE = 0.  A rectangle is drawn
;     Xdelta   - Default deltax value if not listed in the file
;                Default is 0.5; must be greater than zero.
;     Ydelta   - Default deltay value if not listed in the file
;                Default is 0.5; must be greater than zero.
;     nx, ny   - number of resolution elements in the X and Y directions
;                Default = np > 3
;     np       - Default for nx and ny if not specified.
;                Default value is 250 or 1250 depending on HIGHRES
;     highres  - If set, Default value of np is 1250 otherwise np = 250
;                If highres is set then fast is set.
;                If np is specified, this keywork is ignored.
;     lowres   - If set, Default value of np is 50 otherwise np = 250
;                If np is specified, this keywork is ignored.
;     xrange   - Range of x-Values
;     yrange   - Range of y-Values
;     default  - Default value for the two dimensional array 
;                Default = 0.0 unless zlog is set.
;     xlog     - If set, X-data is the log of the actual data
;     ylog     - If set, Y-data is the log of the actual data
;     zlog     - If set, Z-data is the log of the actual data
;     xlabel   - X-label for the data- Default ='X Axis'
;     ylabel   - Y-label for the data- Default ='X Axis'
;     clabel   - Comment label for the dataset- Default ='PixPlot Data'
;     tlabel   - Type label for the dataset- Default ='PixPlot Data'
;     plotonly - If set, no structure is returned but the PixPlot data is
;                plotted in a contour plot.
;  The following keywords are used only for PLOTONLY
;     nlevel   - Number of contour levels. Default is 20
;     levels   - Contour level values, These will be sorted.
;                If levels has <2 elements, the actual values will be returned.
;                If levels has more than 1 value, these values will be used for
;                  the contour plot and nlevel will be set to this number
;                Default levels:
;                levels = vmin + (vmax-vmin)/nlevel*findgen(nlevel)
;     ccolor   - C_color keyword values for the contour plot
;                If ccolor has <2 elements, the actual values will be returned.
;                If ccolor has more than 1 value, these values will be used for
;                  the contour plot with the c_color keyword.
;                Default levels
;                ccolor = 9+round((!d.table_size-11)*(1+indgen(nlevel))/nlevel)
;                Note: If Ccolor is passed and levels are passed, both will be
;                      sorted.
;                Note: If the number of elements in CColor is more than levels,
;                      they will be truncated.  If the number of levels is less,
;                      the CColor values will be replicated.
;    THERMOMETER- If keyword is set a thermometer will be generated
;                 with the contour plot.
;                 Default IDL margins are [10, 3] for x and [4, 2] for y
;                 If TPARAM does not have a Tag Name beginning with 'CU'
;                 then:
;                 If no tag names begin with 'H' xmargin will be set to
;                   [!x.margin(0), 10]
;                   otherwise ymargin will be set to [8, !y.margin(1)]
;           Note: xmargin or ymargin passed to plotfld will override
;                       these settings.
;           Note: If THERMOMETER is a structure, then TPARAM is ignored
;                 and set equal to THERMOMETER
;       TPARAM  - Structure containing data for the thermometer, Tag names
;                 must be valid for the THERMOMETER procedure.
;       _extra - Any valid keywords for the plot command
;               Default values for the contour plot are:
;               plot, x, y, /nodata, $
;               xrange = xrange, yrange = yrange, title = clabel, $
;               xlog = xlog, ylog = ylog, xtitle = xlabel, ytitle = ylabel
;               _extra = extrastuff
;     Procedure:
;       calculate a center and a radius for x and y
;       if a log plot:                     for a linear plot:
;            center = sqrt((x+dx)(x-dx))      center = x
;            radius = log((x+dx)/center)      radius = dx
;       generate the region:
;         circle set: tqcond,centerx+radx*cos(angle), centery+rady*sin(angle)
;              where angle is an array of angles between [0 , 2pi]
;         circle not set: 
;              xx = [centerx-radx, centerx+radx, centerx+radx, centerx-radx, $
;                   centerx-radx]
;              yy = [centery-rady, centery-rady, centery+rady, centery+rady, $
;                   centery-rady]
;              tqcond, xx, yy
;       edit the regions then fit to a grid and edit again if necessary
;       fill the regions and condstruct a structure.
;     Example:
;       assume a data file containing:
; ;     1 6 6 5 5 
; ;     2 6 6 4 4
; ;     3 6 6 3 3
; ;     4 6 6 2 2
; ;     5 6 6 1 1
;       make a pixplot of this data - several plot options exist
;       str = pixplot(ignore =';', xran=[0,12], yran =[0, 12])
;       plotstr, str, /shade ; shade surface of the result
;       plotsrt, str, /fill  ; filled contour plot
;       plotstr, str, /fill, col=-1 , nl = 5; RGB type contour plot
;
;       Make a pixplot with full grid with file 'junk.dat' and set background
;         to 3
;       junk = pixplot(ignore =';', xran=[0,12], /plot, 'junk.dat', $
;              ticklen = 1, /xgridstyle, /ygridstyle, default = 3)
;-
      on_error, 1
      if n_elements(fname) eq 0 then begin
        fname = dialog_pickfile(/must_exist, /read)
        if strlen(fname) lt 1 then begin
          print, ' No File was Selected - RETURNING'
          return, 0
        endif
      endif
      fname = fname(0)
      b = size(fname)
      if b(1) eq 7 then begin
        openr,unit,fname, /get_lun, error = err
        lcopen = 1
        if err ne 0 then return, 0
      endif else begin
        lcopen = 0
        unit = fname
        if unit eq 0 then begin
          fname = dialog_pickfile(/must_exist, /read)
          if strlen(fname) lt 1 then begin
            print, ' No File was Selected - RETURNING'
            return, 0
          endif
          openr,unit,fname, /get_lun, error = err
          lcopen = 1
          if error ne 0 then return, 0
        endif 
      endelse
      if n_elements(skip) eq 0 then skip = 0 else skip = abs(skip(0))
      if keyword_set(circle) then circle =1 else circle =0
      if n_elements(xdelta) eq 0 then xdelta=0.5 else xdelta=abs(xdelta(0))
      if n_elements(ydelta) eq 0 then ydelta=0.5 else ydelta=abs(ydelta(0))
      if xdelta eq 0 or ydelta eq 0 then begin
        Print, ' Cannot do a pixplot with zero value of x/ydelta'
        return, 0
      endif
      if n_elements(xlabel) eq 0 then xlabel ='X Axis'
      if n_elements(ylabel) eq 0 then ylabel ='Y Axis'
      if n_elements(clabel) eq 0 then clabel ='PixPlot Data'
      if n_elements(tlabel) eq 0 then tlabel ='PixPlot Data'
      if keyword_set(plotonly) then begin
        np = 1024
        if keyword_set(thermometer) then begin
          ss = size (thermometer)
          if ss(ss(0)+1) eq 8 then tparam = thermometer
          xmargin = [!x.margin(0), 10]
          ymargin = !y.margin
          if n_elements (tparam) gt 0 then begin
            b = size(tparam)
            if b(b(0)+1) ne 8 then begin
              winformation, $
              ['TPARAM must be a structure', 'TPARAM will be set to null']
              tparam = {title:''}
            endif else begin
              b = tag_names(tparam)
              ok = where (strmid(b, 0, 2) eq 'CU', nok)
              if nok eq 0 then begin
                ok = where (strmid(b, 0, 1) eq 'H', nok)
                if nok gt 0 then begin
                  ymargin = [8, !y.margin(1)]
                  xmargin = !x.margin
                endif
              endif else xmargin = !x.margin
            endelse
          endif
        endif else begin
          xmargin = !x.margin
          ymargin = !y.margin
        endelse

      endif else begin
        if n_elements(np) eq 0 then begin
          if keyword_set(highres) then np = 1250L else begin
            if keyword_set(lowres) then np = 50L else np = 250L
          endelse
        endif
        if n_elements(nx) eq 0 then nx = np else nx = abs (nx(0)) > 3
        if n_elements(ny) eq 0 then ny = np else ny = abs (ny(0)) > 3
        nx = long(nx)
        ny = long(ny)
      endelse
      strin = ' '
      ncon = 0
      nval = 2048
      if keyword_set(circle) then begin
        if np lt 600 then $
          npcircle = 101 else npcircle = 201
        angle = (!pi+!pi)/(npcircle-1)*findgen(npcircle) 
        sinang = sin(angle)
        cosang = cos(angle)
      endif
      nvalue = 32L
      value= make_array(nvalue)
      xarr = make_array(nval)
      yarr = make_array(nval)
      locar = make_array(nvalue, /long, value = 0L)
      reset = 1
      if skip gt 0 then for i = 1, skip do readf, unit, strin
      while (not (eof(unit))) do begin
        on_ioerror, baddata
        readf,unit,strin
        parser, strin, array, np, ignore = ignore
        if np ge 3 then begin
          ncon = ncon+1
          if ncon eq nvalue then begin
            nvalue = nvalue+nvalue
            value = [value, value]
            locar = [locar, locar]
          endif
          value(ncon-1) =array(0)
          xx = array(1)
          yy = array(2)
          if np gt 3 then dx = abs(array(3)) else dx = xdelta
          if np gt 4 then dy = abs(array(4)) else dy = ydelta
          lccond = 1 
          if keyword_set(xlog) then begin
            if xx-dx gt 0 then begin
              center = sqrt ((xx-dx)*(xx+dx))
              radx = alog((xx+dx)/center)
              centerx = alog(center)
            endif else begin
              print, 'These values cannot be used with a log grid (x-dx<0)'
              print, ' z, x, y, dx, dy: ', array(0), xx, yy, dx, dy
              lccond = 0
            endelse
          endif else begin
            centerx = xx
            radx = dx
          endelse
          if keyword_set(ylog) then begin
            if yy-dy gt 0 then begin
              center = sqrt ((yy-dy)*(yy+dy))
              rady = alog((yy+dy)/center)
              centery = alog(center)
            endif else begin
              print, 'These values cannot be used with a log grid (y-dy<0)'
              print, ' z, x, y, dx, dy: ', array(0), xx, yy, dx, dy
              lccond = 0
            endelse
          endif else begin
            centery = yy
            rady = dy
          endelse
          if lccond ne 0 then begin
            if keyword_set(circle) then begin
              xx =  centerx+radx*cosang
              yy =  centery+rady*sinang
              np = npcircle
            endif else begin
              xx = [centerx-radx, centerx+radx, centerx+radx, centerx-radx, $
                  centerx-radx]
              yy = [centery-rady, centery-rady, centery+rady, centery+rady, $
                  centery-rady]
              np = 5
            endelse
            locar(ncon) = locar(ncon-1) + np
            if  locar(ncon) ge nval then begin
              nval = nval+nval
              xarr = [xarr, xarr]
              yarr = [yarr, yarr]
            endif 
            xarr(locar(ncon-1):locar(ncon)-1) = xx
            yarr(locar(ncon-1):locar(ncon)-1) = yy

          endif else ncon = ncon-1
        endif
      endwhile
      if ncon eq 0 then begin
        print, 'No conductors input - RETURNING'
        return, 0
      endif
      value = value (0: ncon-1)
      xarr = xarr(0:locar(ncon)-1)
      yarr = yarr(0:locar(ncon)-1)
      locar = locar(0:ncon)
      vmin = min(value, max = vmax)
      if vmin le 0 and keyword_set(zlog) then begin
        print, ' Log Z-axis not possible with negative/zero values'
        zlog = 0
      endif
      if n_elements(default) gt 0 then begin
        if keyword_set(zlog) then begin
          if default le 0 then begin
            default = floor(alog10(vmin))
            print, 'Value of default changed to :', default
          endif
        endif
      endif else begin
        if keyword_set(zlog) then $
            default = floor(alog10(vmin)) $
        else default = 0.0
        print, 'Value of default set to :', default
      endelse
      if keyword_set(plotonly) then begin
        if keyword_set(xlog) then xarr =exp(xarr)
        if keyword_set(ylog) then yarr = exp(yarr)
        if keyword_set(zlog) then yarr = 10^(default)
        if n_elements(xrange) lt 2 then xrange = [min(xarr, max = mm), mm]
        if n_elements(yrange) lt 2 then yrange = [min(yarr, max = mm), mm]
        if n_elements(nlevel) eq 0 then nlevel = 20 else nlevel = nlevel> 2
        if n_elements(levels) lt 2 then begin
          if keyword_set(zlog) then begin
            levels = alog(vmin)+ $
              (alog(vmax)-alog(vmin))/(nlevel)*findgen(nlevel)
            levels = exp(levels)
          endif else levels = vmin + (vmax-vmin)/(nlevel)*findgen(nlevel)
        endif else begin
          oklev = sort(levels)
          sort= sort(oklev)
          nlevel = n_elements(levels)
        endelse
                    
        if n_elements(ccolor) lt 2 then $
          ccolor = 9+ round((!d.table_size-11)*(1+indgen(nlevel))/nlevel ) $
        else begin 
          nelcol = n_elements(ccolor)
          if nelcol lt nlevel then begin
            junk = make_array(/long,nlevel)
            junk(0:nelcol-1) = ccolor
            ccolor = temporary(junk)
            for i = nelcol, nlevel-1 do ccolor(i) = ccolor(i mod nelcol) 
          endif
          if n_elements(oklev) eq nlevel then ccolor = ccolor (oklev)
        endelse

        plot, xarr, yarr, /nodata, $
               xrange = xrange, yrange = yrange, title = clabel, $
               xlog = xlog, ylog = ylog, xtitle = xlabel, ytitle = ylabel, $
               xmargin = xmargin, ymargin = ymargin, _extra = extrastuff
        if default gt levels(0) then begin
          ok = where (default ge levels, nok)
          cval = ccolor(ok(nok-1))
          polyfill, [!x.crange, rotate(!x.crange, 2)], color = cval, $
                    [!y.crange(0), !y.crange(0), !y.crange(1), !y.crange(1)]
          !p.multi(0) = !p.multi(0)+1
          plot, xarr, yarr, /nodata, $
               xrange = xrange, yrange = yrange, title = clabel, $
               xlog = xlog, ylog = ylog, xtitle = xlabel, ytitle = ylabel, $
               xmargin = xmargin, ymargin = ymargin, _extra = extrastuff
        endif
        epx = 0.0020*(!x.crange(1)-!x.crange(0))
        epy = 0.0020*(!y.crange(1)-!y.crange(0))
        clipit = [!x.crange(0)+epx, !y.crange(0)+epy, $
                  !x.crange(1)-epx, !y.crange(1)-epy]
        for i = 0, ncon-1 do begin
          ok = where (value(i) ge levels, nok)
          if nok gt 0 then begin
            cval = ccolor (ok(nok-1))
            polyfill, xarr(locar(i):locar(i+1)-1),yarr(locar(i):locar(i+1)-1), $
              color = cval, noclip=0, $
             clip = clipit
          endif else print, 'PixPlot ', strtrim(i, 2), ' below lowest level'
        endfor
        if keyword_set (thermometer) then $
          thermometer, levels, ccolor, _extra = tparam
        return, 0
      endif
      tqcond, xarr, yarr, locar, reset = reset
      print, ' Edit the regions if necessary. Press DONE when ready.'
      qsedit, /cond, /fast, /reset
      tqhelp, cond = cond
      if Ncon ne cond.ncon then begin
        print, ' Number of conductors may not be changed - RETURNING'
        return, 0
      endif
      if (n_elements(xrange) lt 2 or n_elements(yrange) lt 2) then begin
        if n_elements(xrange) lt 2 then begin
          cx = cond.x
          ran = max(cx, min = minv)-minv
          if nx lt 200 then begin
            dx = ran/(nx-2)
            xrange = [minv-dx, minv+ran+dx]
          endif else begin
            dx = ran/(nx-4)
            xrange = [minv-2*dx, minv+ran+2*dx]
          endelse
        endif else begin
          xrange = xrange(0:1)
          if keyword_set(xlog) then xrange = alog(xrange)
          dx = (xrange(1)-xrange(0))/nx
        endelse
        if n_elements(yrange) lt 2 then begin
          cy = cond.y
          ran = max(cy, min = minv)-minv
          if ny lt 200 then begin
            dy = ran/(ny-2)
            yrange = [minv-dy, minv+ran+dy]
          endif else begin
            dy = ran/(ny-4)
            yrange = [minv-2*dy, minv+ran+2*dy]
          endelse
        endif else begin
          yrange = yrange(0:1)
          if keyword_set(ylog) then yrange = alog(yrange)
          dy = (yrange(1)-yrange(0))/ny
        endelse
      endif else begin
        xrange = xrange(0:1)
        if keyword_set(xlog) then xrange = alog(xrange)
        dx = float(xrange(1)-xrange(0))/nx
        yrange = yrange(0:1)
        if keyword_set(ylog) then yrange = alog(yrange)
        dy = float(yrange(1)-yrange(0))/ny
      endelse  
      qsgrid, xrange, [dx, dx], /x, /quiet
      qsgrid, yrange, [dy, dy], /y, /quiet
      qsfit, /close
      print, ' Modify regions as desired.'
      print, ' Remove any bad points and close all conductors'
      qsedit, /fast
      tqhelp, cond = cond, gx = xx, gy = yy
      nx = n_elements(xx)
      ny = n_elements(yy)
      if keyword_set(zlog) then value= alog10(value)
      if keyword_set(xlog) then  begin
        xx= xx/alog(10.0)
        xrange = xrange/alog(10.0)
      endif
      if keyword_set(ylog) then begin
        yy= yy/alog(10.0)
        yrange = yrange/alog(10.0)
      endif
      array = make_array(nx, ny, /float, value = float(default))
      if keyword_set(fast) or keyword_set(highres) then begin
        for ii = 0, ncon-1 do $
          array(polyfillv(cond.ix(cond.loci(ii):cond.loci(ii+1)-1), $
           cond.iy(cond.loci(ii):cond.loci(ii+1)-1), nx, ny)) = value(ii)
      endif else begin
        tqfill, arrayi
        for i = 1, ncon do begin
          ok = where (arrayi eq i, nok)
          if nok gt 0 then array(ok) = value(i-1)
        endfor
      endelse
      if lcopen eq 1 then free_lun, unit
      if n_elements(xlabel) eq 0 then begin
        if keyword_set(xlog) then xlabel = 'Log X Axis'   else $
          xlabel  ='X Axis'
      endif
      if n_elements(ylabel) eq 0 then begin
        if keyword_set(ylog) then ylabel = 'Log Y Axis'   else $
          ylabel  ='Y Axis'
      endif
      if n_elements(clabel) eq 0 then begin
        if keyword_set(zlog) then clabel = 'Log PixPlot Data'   else $
          clabel = 'PixPlot Data'
      endif
      if n_elements(tlabel) eq 0 then tlabel = 'PixPlot Data'
      if lcopen ne 1 then fname = 'Unit '+strtrim(unit)
      locx = make_array(2,2,/long)
      locx (*, 0) = 0
      locx (0, 1) = nx
      locx (1, 1) = ny
      ntot = (nx)*(ny)
      range = make_array(2,2,2)
      range(0, 0, *) = xx(0) 
      range(0, 1, *) = yy(0)
      range(1, 0, *) = xx(nx-1)
      range(1, 1, *) = yy(ny-1)
      spare = [0,4,0,0,0]
      ssize = make_array(2, 1)
      ssize (0:*, 0)= [nx, ny]
      locv = [0, ntot]
      v1 = reform(array, ntot, /overwrite)

      dx1 = 0.0
      dx2 = 0.0
      dx3 = 0.0
      dx4 = 0.0
      if n_elements(xx) gt 1 then check_uniform, xx, dx1
      if n_elements(yy) gt 1 then check_uniform, yy, dx2
      str={type:1L, ndim:2L, nvect: 1L, block :1L, two: 3L, $
         slice: 0.0, size: ssize, range: range, order: 1L, $
          locx:locx, locv: locv, x1:xx, x2:yy, $
          x3:0.0, x4:0.0, v1:temporary(v1), v2:0.0, $
          v3:0.0, v4:0.0, v5:0.0, v6:0.0,x1lab : xlabel, x2lab :ylabel, $
          dx1: dx1, dx2: dx2, dx3: dx3, dx4: dx4, $
          x3lab : '', x4lab :'',v1lab : clabel, v2lab :'PIX array', $
          v3lab : '', v4lab :'',v5lab : '', v6lab :'', $
          file : fname, spare : spare,  tlab : tlabel, clab: clabel}
      return, str
baddata:
      print, 'Reading error on the following:'
      print, strin
      return, 0
      end
