; ------------------------------------------------------------------------ 
; $Id: scanp.pro,v 1.4 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro ScanP, xaxis, yaxis, Weight, dsmin, dsmax, dsstp, $
             con=con, scale=scale, index=index, winnum=winnum, $
             xsize = xsize, ysize = ysize,  _extra = extrastuff

;+
; PURPOSE:
;       Use XINTERANIMATE to build an animation of a DO-list of 
;       datasets of FIELD snapshots
; CALLING SEQUENCE:
;       ScanP, Xaxis, Yaxis, Weight, Start [, Stop] [, Skip]
;               [, CON=CON] [, SCALE=SCALE ] [, INDEX=INDEX ]'
;               [, XSIZE = xsize] [, YSIZE = ysize]
;               [, WINNUM=WINNUM ] [, _EXTRA = EXTRASTUFF]
; INPUTS:
;       Xaxis :  Particle parameter for the horizontal axis
;       Ysxis :  Particle parameter for the vertical axis
;       Weight : Value of normal ordinate (PLOTFLD argument)
;       Start \  
;       Stop -  DO list for frames in file.
;       Skip  /  
;       Default Value for Stop is number of datasets in current PFF file
;       If Stop le start then stop = ndspff()
;       If Skip le 0 or undefined then skip = 1
; OPTIONAL INPUT:
;       -none-
; KEYWORDS:
;       CON:    Conductor structure
;       INDEX:  Index of plane (PLOTFLD option)
;       WINNUM: IDL window number
;       XSIZE:  X Size of the window; Default = 448
;       YSIZE:  Y Size of the window; Default = 384
;
; Example
;       Make a movie using datasets 215 to 228, with axis 3, 1, and 
;          color each particle using charge as weight
;      ScanP, 3, 1, -4,  215,  228, 1
;-

on_error, 1

nparams = n_params()
if nparams lt 4 then begin
  print, '% FORMAT: ScanP, Xaxis, Yaxis, Weight, Start [, Stop] [, Skip]'
  print, '         [, con=con] [, index=index ]'
  print, '         [, winnum=winnum ] [, _extra=extra ]'
  return
endif

if n_elements(con)    eq 0 then con = 0
if n_elements(index)  eq 0 then index = 0
if n_elements(winnum) eq 0 then winnum = 1
if n_elements(dsmax) eq 0 then dsmax = ndspff()
if n_elements(dsstp) eq 0 then dsstp = 1
if n_elements(xaxis) eq 0 or n_elements (yaxis) eq 0 then begin
  print, '% ScanP: Axes must be defined'
  return
endif
if xaxis lt 1 or xaxis gt 3 then begin
  print, '% ScanP: X axis must be in the range (1, 3).
  return
endif
if yaxis lt 1 or yaxis gt 3 then begin
  print, '% ScanP: Y axis must be in the range (1, 3).
  return
endif
  
if n_elements(dsmin) eq 0 then begin
  print, '% ScanP: First dataset must be specified.'
  return
endif

  
if dsmax le dsmin then dsmax = ndspff()

nds   = (dsmax - dsmin) / dsstp + 1
if n_elements(xsize) eq 0 then xsize = 448
if n_elements(ysize) eq 0 then ysize = 384

print, nds, ' Frames'

window, winnum, xsiz=xsize, ysiz=ysize
xinteranimate, set = [xsize, ysize, nds]

; Loop over requested frames

frame_num = 0

for ds=dsmin,dsmax,dsstp do begin
  readpar, f, ds

  loctl  = strpos (f.clab, 't = ')
  if loctl lt 0 then loctl  = strpos (f.clab, 't=')
  len    = strlen(f.clab) - loctl + 1
  tlabel = strmid (f.clab, loctl, len)

  plotpar, f,  xaxis, yaxis, Weight, con=con, index=index, $
           tit=tlabel, _extra = extrastuff 
  xinteranimate, frame=frame_num, window=winnum
  frame_num = frame_num + 1
endfor

wdelete, winnum
xinteranimate, /block 
;; xinteranimate, /close

return
end
