; ------------------------------------------------------------------------ 
; $Id: shelp.pro,v 1.5 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro shelp, wminin, wmaxin, full = full, idlhelp = idlhelp, comment = cmment
;+
;
; NAME:
;      SHELP
; PURPOSE:
;      Show the elements of a QS structure or show the structures 
;        in the qs common area
; CALLING SEQUENCE:
;      SHELP  [, SMIN] [, SMAX] [, /FULL] [, /IDLHELP]
; INPUTS:
;      SMIN   - First sturcture array number to be listed or an array of 
;               structure array numbers or a QS structure
;         or
;             If a single parameter, SMIN can be a search string.
;             Those arrays containing the string in the dataset comment will
;               be listed. The full set of wild card options is available.
;               See GET_MATCH
;       Note: FULL = 0 if SMIN is a string
;      SMAX   - Last structure array number
;      FULL   - If present and not zero, the full structure array
;               information is printed.  If full is zero, only the dataset
;               comment is printed
;               Default:
;               SMIN and SMAX are missing, then FULL = 0
;               SMAX is missing, then FULL = 1
;               Otherwise, FULL = 0
;      IDLHELP- If present and not zero, the structure component information
;               provided by "help, /structure" is provided. Default is 
;               IDLHELP =0
;      Comment- If set, full comment will be printed. ';' or ';;'will be taken as a
;               carriage return. If comment is set, then full = 0.
;         Note: If one instance of ';;' is found that will be the delimiter
;               for the remainder of the line.

; Outputs:
;      Structure parameters
; Example:
;      See the contents of all structure arrays
;         shelp
;      See the full contents of structure, f12
;         shelp, f12
;      See the dataset comment only of structure array, f12
;         shelp, f12, full= 0
;      See the full contents of structure arrays 1 through 3
;         shelp, 1, 3, /full
; (or)    shelp, [1,2,3]
;      See the contents of structure arrays 10 through 20
;         shelp, 10, 20
;-
;  Common block
@qsstrcom.dat
;
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      on_error, 1
      lcpar = 0
      npar = n_params()
      if keyword_set(cmment) then full = 0
      if n_elements(idlhelp) eq 0 then idlhelp = 0 else idlhelp = idlhelp(0)
      if npar eq 0 then begin
        wmin = where (lcvalidstr ne 0 , nwdf)+1
        if nwdf eq 0 then begin
          Print, ' All Structure Arrays are Empty.'
          return
        endif
      endif else if npar eq 1 then begin
        ss = size(wminin)
        if ss (ss(0)+1) eq 7 then begin
          wmin= wminin(0)
          full = 0
          lst = strlen(wmin)
          if lst eq 0  then wmin = '*'
          ok = get_match (wmin, /str, nelement=nwdf)
          if ok(0) eq -1 then begin
           print, '% SHELP: No COMMENTS contain the string: ', wmin
           return
          endif
          wmin = temporary(ok)
        endif else begin
          if n_elements(full) eq 0 then full = 1
          nwdf = n_elements(wminin)
          if nwdf eq 0 then begin
            print, ' First parameter to SHELP is undefined'
            return
          endif else wmin = wminin
        endelse
        if nwdf eq 1 then begin
          b = size(wmin)
          if  b(b(0)+1) ne 8 then begin
            if wmin gt 0 and wmin le maxstructure then begin
              lcpar = wmin
              s2i, wmin, wmin, /no_copy
              b = size (wmin)
              if b(b(0)+1) ne 8 then begin
                print,' First parameter to SHELP ', $
                 'must be a structure or a structure array number'
                return
              endif
            endif  else begin
              print,' First parameter to SHELP ', $
                'must be a structure or a structure array number'
              return
            endelse
          endif
          pristr, wmin
          if idlhelp ne 0 then help, /structure, wmin
          if lcpar gt 0 then i2s, wmin, lcpar, /no_copy
            return
        endif else begin
          if n_elements (full) eq 0 then full = 1
          ok = where (lcvalidstr(wmin-1) ne 0, nwdf)
          if nwdf eq 0 then begin
            print, ' Specified structure arrays are empty'
            return
          endif else wmin = wmin(ok)
        endelse
      endif else begin
        nwdf  =min ([n_elements(wminin), n_elements(wmaxin)])
        if nwdf eq 0 then begin
          print, ' Parameters to SHELP are undefined'
          return
        endif else begin
          wmax = wmaxin(0)
          wmin = wminin(0)
        endelse
        wmin = min([wmin, wmax], max = wmax) >1
        if wmax le 0 then wmax = maxstructure
        wmax = wmax < maxstructure
        ok = where(lcvalidstr(wmin-1:wmax-1) ne 0, nwdf)
        if nwdf eq 0 then begin
          print, ' Specified structure arrays are empty'
          return
        endif else begin
          wmin = findgen(wmax-wmin+1)+wmin
          wmin = wmin(ok)
        endelse
      endelse
      if n_elements(full) eq 0 then full = 0
      wmin = fix(wmin)
      if full eq 0 then begin
        print, ' Structure Arrays with Valid Data are Listed:'
        print, ' Array No.   Dataset Comment'
        for i = 0, nwdf-1 do begin
          if keyword_set(cmment) then begin
            comm = comment(wmin(i)-1)
            ldelim = 1
            delimiter = ';'
            ok = strpos(comm, delimiter, 0)
            lcom = strlen(comm)
            if ok lt 0 or ok+1 ge lcom then  print,wmin(i),format='(i4,4x,a)', $
              strmid(comment(wmin(i)-1),0,66) else begin
              print, wmin(i),format='(i4,4x, a)', strmid(comm,0,ok)
              oko=ok+ldelim
              if strpos(comm, ';', oko) eq oko then begin
                delimiter = ';;'
                ldelim = 2
                oko = oko+1
              endif
              ok = strpos(comm, delimiter, oko)
              while ok gt 0 do begin
                print, format='(8x, a)', strmid (comm, oko, ok-oko)
                oko= ok+ldelim
                if ldelim eq 1 and strpos(comm, ';', oko) eq oko then begin
                  delimiter = ';;'
                  ldelim = 2
                  oko = oko+1
                endif
                if oko ge lcom+1-ldelim then ok = -1 else $
                  ok = strpos(comm, delimiter, oko)
              endwhile
              if oko le lcom+1-ldelim then $
                print, format='(8x, a)', strmid (comm, oko, lcom-oko)
            endelse
          endif else begin
            print, wmin(i),'    ', strmid(comment(wmin(i)-1),0,66)
            if idlhelp ne 0 then help, /structure, wmin
          endelse
        endfor
      endif else for i = 0, nwdf-1 do begin
        print, ' -----------------'
        print, ' Structure array number', wmin(i)
        pristr, wmin(i)
        temp = 0
        if idlhelp ne 0 then begin
          handle_value, strarraycom(wmin(i)-1), temp, /no_copy
          help, /structure, temp
          handle_value, strarraycom(wmin(i)-1), temp, /set, /no_copy
        endif
      end
      return
      end
