; ------------------------------------------------------------------------ 
; $Id: spec.pro,v 1.3 2007/06/06 20:09:37 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
;  Procedure SPEC
;  This procedure produces the energy values and time of flights for a
;  magnetic spectrometer.  The geometry assumes a planar anode on the left.
;  Next a planar gas cell with a three element scattering foil in the center.
;  Gas pressure and type may be different on either side of the scatterer.
;  The spectrometer is assumed to be at an angle to the direct beam and to 
;  have one foil at its entrance. The magnetic field in the magnet is assumed
;  constant over the diameter.
;  
;  format: SPEC, fileout, filein, quiet= quiet, length = length
;    where: 
;    fileout  - Name of file with the output
;    filein   - Input file with spectrometer values
;    length   - length of trajectory in the magnetic field
;    quiet    - Range information will be printed if quiet is 0
; Note: if the ion charge is greater than z or less than 0  then 
;       all possible charge states will be calculated.
;       
;               Default: QUIET=1
;  
;  Example:
;            spec
;-  
   pro read_spec_geom, unit
on_error, 1
@spec_common.dat
   str = '  '
   lcadd = 0
   sstr = ''
   while (not eof(unit)) do begin
     on_ioerror, badeof
     readf, unit, str
     str = strtrim(str)
     d = strlen(str) 
;    print, strmid(str, d-1 , 1), str
     if strmid(str, d-1 , 1) eq '$' then begin
       if lcadd eq 0 then begin
         lcadd = 1
         sstr = strmid(str, 0, d-1)
       endif else sstr = sstr+strmid(str, 0, d-1)
     endif else begin
       if lcadd eq 0 then sstr = str else sstr = sstr+str
       ok = execute(sstr)
       lcadd = 0
     endelse
   endwhile
badeof:
   return
   end

   Pro spec, fileout, filein, quiet = quiet, length = arc_len

     on_error, 1

@spec_common.dat
@spec_default.dat
     matname=['Hydrogen','Helium','Lithium','Beryllium', $
      'Boron','Carbon','Nitrogen','Oxygen','Fluorine','Neon','Sodium', $
      'Magnesium','Aluminum','Silicon','Phosphorus','Sulpher','Chlorine', $
      'Argon','Potassium','Calcium','Scansium','Titanium','Vanadium', $
      'Chromium','Manganese','Iron','Cobalt','Nickel','Copper','Zinc', $
      'Gallium','Germanium','Arsenic','Selenium','Bromine','Krypton', $
      'Rubidium','Stontium','Yttrium','Zirconium','Niobium', $
      'Molybendum','Technetium','Ruthenium','Rhodium','Palladium', $
      'Silver','Cadmium','Indium','Tin','Antimony','Tellerium', $
      'Iodine','Xenon','Cesium','Barrium','Lanthanum','Cerium']
     matname =[matname, $
      'Praseodymium','Neodymium','Promethium','Samarium','Europium', $
      'Gadolinium','Terbium','Dysprosium','Holmium','Erbium', $
      'Thulium','Ytterbium','Lutetium','Hafnium','Tantalum', $
      'Tungston','Thenium','Osmium','Iridium','Platinum','Gold', $
      'Mercury','Thallium','Lead','Bismuth','Polonium','Astatine', $
      'Radon','Francium','Radium','Actinium','Thorium', $
      'Protactinium','Uranium','dud-93','dud-94','dud-95','dud-96', $
      'dud-97','dud-98','dud-99','dud-100','Poly','Mylar','LiF', $
      'Teflon','Kimfol','Kapton','Parylene-N','Parylene-C', $
      'Parylene-D','ErD2','Brass','Monel','Inconel','LB4','LB5']

      ionmass = [1.0079,  4.0026, 6.941,  9.0122,10.81,  12.011,  14.007, $
         15.9994, 18.998, 20.179, 22.990, 24.305, 26.9815, 28.0855, $
         30.974,  32.06,  35.453, 39.948, 39.098, 40.08,   44.956, $
         47.90,   50.9415,51.996, 54.938, 55.847, 58.933,  58.70, $
         63.546,  65.38,  69.72,  72.59,  74.922, 78.96,   79.904, $
         83.80,   85.468, 87.62,  88.906, 91.22,  92.906,  95.94, $
         98.0,   101.1,  102.91, 106.4,  107.87, 112.41,  114.82, $
        118.69,  121.75, 127.6,  126.90, 131.3,  132.905, 137.33, $
        138.91,  140.12, 140.91, 144.24, 145.0,  150.4,   151.96]
      ionmass =[ionmass, $
        157.25,  158.93, 162.5,  164.93, 167.26, 168.93,  173.04, $
        174.97,  178.49, 180.95, 183.85, 186.21, 190.2,   192.22, $
        195.09,  196.97, 200.59, 204.37, 207.2,  208.98,  209., $
        210.,    222.,   223.,   226.0,  227.03, 232.04,  231.04, $
        238.03,  237.05, 244.,   243.,   247.,   247.,    251., $
        252.,    257. ]

      if n_elements(quiet) eq 0 then quiet = 1
      if n_elements(fileout) eq 0 then begin
        get_tqstring, iq, 'ENTER the Output File Name:', fileout
        if iq ne 0 then return
      endif
;
      if n_elements(filein) gt 0 then begin
        openr, unit, /get, filein, error = err
        if err ne 0 then begin
          filein = strtrim(dialog_pickfile(/must_exist, filter= '*.geom'), 2)
          if strlen(filein) gt 0 then openr, unit, /get, filein, error = err
          if err ne 0 then begin
            print, 'Unable to open input file - Stopping'
            return
          endif
        endif
        read_spec_geom, unit
        free_lun, unit
      endif 

      spec_unknown

      if lcquit ne 0 then return

      fileg = find_newfile(fileout + '.geom')
      filep = find_newfile(fileout + '.pff')
      filev = find_newfile(fileout + '.vida')
      openw, gunit, /get, fileg, error= err
      if err ne 0 then begin
        print, '********* Error opening output geometry file, STOPPING *********'
        return
      endif
      openw, vunit, /get, filev, error= err
      if err ne 0 then begin
        print, '********* Error opening output vida file, STOPPING *********'
        return
      endif

      createpff, filep
      spec_write, filep, gunit

      if (scat_th_2 le 0.0) then begin
        print, 'Scattering foil thickness must be gt 0.0'
        return
      endif
;     calculate x and y of exit
      magnet_r = magnet_dia*0.5
      magnet_angr = magnet_angle*!dtor
      npmag = n_elements(magnet_angle) 
;     X and Y are exit points for the ions in the spectrometer
;     x = magnet_r*sin(magnet_angr)
;     y = magnet_r*(1.0+cos(magnet_angr))
      x = sin(magnet_angr)
      y = 1.0+cos(magnet_angr)

; gyro radius - this can be rewritten
;     gr =  (x+y*y/x)*(0.5*magnet_r)
      gr =  y/x*magnet_r

; Classical Exit Energy
;     ionmas = ion_a*amu_mass
;     velocity in cm/sec
;     velc= gr*ion_q *echarge*magnet_b*1.0e-4/ionmas
;     energy in 10^6 MeV
;     energy = 0.5*ionmas*velc*velc*1e-10/echarge
     

; Exit energy
;     pc = 1.0e-12*cqB*gr where c(m/s), q(1,2,...), B(gauss), gr(cm)
      if (ion_q lt 0) or (ion_q gt ion_z) then begin
        pc = 2.9979246e-4*magnet_b*gr ; momentum*c in MeV for charge =1
        if ion_z gt 1 then begin
          n = n_elements(pc)
          temp = temporary(pc)
          gtemp = temporary(gr)
          pc = make_array(ion_z*n)
          gr = make_array(ion_z*n)
          pc (0:n-1) = temp
          gr (0:n-1) = gtemp
          for i = 1, ion_z-1 do pc(i*n:(i+1)*n-1) = (i+1)*temp
          for i = 1, ion_z-1 do gr(i*n:(i+1)*n-1) = gtemp
          temp = 0
        endif
      endif else pc = 2.9979246e-4*ion_q*magnet_b*gr ; momentum*c in MeV

;     a = 1
      c = pc*pc ; actually *(-1)
      rest_ener = amu_energy*ion_a
      b =  rest_ener+rest_ener
      bsq = rest_ener*rest_ener

      enersp = c/(rest_ener+sqrt(bsq+c))
;     enerp = 2*c/(b+2*sqrt(bsq+c))
; print,' Difference:', enersp -enerp
      velocity = 2.9979246e10*sqrt(c/(bsq+c))
;     velcc = 0.5*ionmas*velocity*velocity*1e-10/echarge
      
      np = n_elements(velocity)
;     for i = 0, np-1 do  print, magnet_angle(i), energy(i), $
;          enersp(i), velc(i), velocity(i), velcc(i)

; Arc length in the magnet
      arc_len = (gr+gr)*atan(magnet_r/gr)
      time = (arc_len-magnet_r+cr39_radius-ph_dist_1+magnet_dist)/velocity

; Range in silicon
      ein = enersp
      eout = make_array(np, value = 0.0)
      pin_range = make_array(np, value = -9.0)
      zion = ion_z
      mat = 14L
      quiet = quiet(0)
      stopping, zion, ein, eout, pin_range, mat, 0.0, edel, ier, aion = ion_a, $
           ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
; Energy loss in foil in front of pinhole
      if ph_thick gt 0.0 then begin
        eout = enersp
        ein = make_array(np, value = -9.0)
        thick = make_array(np, value = ph_thick)
        stopping, zion, ein, eout, thick, ph_mat,0.0, edel, ier, aion = ion_a, $
           ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
        pc2 = (b+ein)*ein
        velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
      endif
      spect_angle_r = spect_angle*!dtor
      if lc90 eq 0 then $
        fudge = abs(cos(spect_angle_r)) else $
        fudge = abs(sin(spect_angle_r))
      if fudge lt .001 then begin
        print, 'This code is not designed for 90 degree scattering'
        print, '********************Angle set to 89 degrees******************'
        spect_angle_r = 89*!dtor
        fudge = cos(spect_angle_r)
      endif
      dd = (ph_dist_1-GCell_2_dist/fudge)
; Time to get to the gascell
      if dd gt 0.0 then time = time +(dd)/velocity else print, $
        '************* Spectrometer is in the gascell *******'
; Energy loss in second gas cell foil
      if GCell_2_th   gt  0.0 then begin
        eout = ein
        ein = make_array(np, value = -9.0)
        thick = make_array(np, value = GCell_2_th/fudge)
        stopping, zion, ein, eout, thick, 102L,0.0, edel, ier, aion = ion_a, $
           ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
        if gcell_2_press gt 0.0 then begin
          eout= ein
          ein = make_array(np, value = -9.0)
; put in a multiplier of 1e4 to convert cm to microns
          thick = make_array(np, value = GCell_2_dist/fudge*1.0e4)
          stopping, zion, ein, eout, thick, GCell_2_gas,GCell_2_press, edel, $
             ier, aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, $
             quiet = quiet
          pc2 = (b+ein)*ein
          velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
          x = edel/ein
;  linear approximation i.e. - dE/dx = constant in gas cell
          correction=1.0+x*0.25*(1.0+0.5*x*(1.0+.625*x*(1.0+0.7*x*(1.0+0.75*x))))
          time = time +GCell_2_dist/velocity*correction/fudge
        endif else begin
          pc2 = (b+ein)*ein
          velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
          time = time +GCell_2_dist/velocity/fudge
        endelse 
      endif else begin
;  Velocity has not changed so just calculate the additional time
        time = time +GCell_2_dist/velocity/fudge
      endelse
;     energy loss in scattering foil - first calculate an effective thickness
      fudge = abs(cos((scat_angle+spect_angle)*!dtor))
      if fudge lt .001 then begin
        print, 'This code is not designed for 90 degree scattering'
        print, '********************Angle set to 89 degrees******************'
        scat_angle_r = 89*!dtor
        fudge = cos(scat_angle_r)
      endif
;  determine if particle is in reflection or transmission

      if spect_angle ge -(90.0 + scat_angle) and $
         spect_angle le  (90.0 - scat_angle) then begin
;  particle is in transmission
        if scat_th_3 gt 0.0 then begin
          eout = ein
          ein = make_array(np, value = -9.0)
          thick = make_array(np, value = scat_th_3/fudge)
          stopping, zion, ein, eout, thick, scat_mat_3,0.0, edel, ier, $
            aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, $
            quiet = quiet
        endif
      endif else begin
;  particle is in reflection
        if scat_th_1 gt 0.0 then begin
          eout = ein
          ein = make_array(np, value = -9.0)
          thick = make_array(np, value = scat_th_1/fudge)
          stopping, zion, ein, eout, thick, scat_mat_3,0.0, edel, ier, $
            aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
        endif
      endelse
;     Transport through the rear/front half of the scattering foil
      eout = ein
      ein = make_array(np, value = -9.0)
      thick = make_array(np, value = scat_th_2/fudge*0.5)
      stopping, zion, ein, eout, thick, scat_mat_2,0.0, edel, ier, aion=ion_a, $
           ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
;     Rutherford scattering energy loss
;     calculate the scattering angle
;     first the direction cosines to the anode
      if anode_y eq 0.0 and anode_z eq 0.0 then begin
        aa = 1.0
        bb = 0.0
        cc = 0.0
        cscat = cos(spect_angle*!dtor)
      endif else begin
        dist = sqrt(anode_dist*anode_dist + anode_y*Anode_y + anode_y*Anode_y)
        aa = anode_dist/dist
        bb = anode_y/dist
        cc = anode_z/dist
        spect_ang_rad = spect_angle*!dtor
        cscat = aa*cos(spect_ang_rad) - cc*sin(spect_ang_rad)
      endelse
      scat_a = ionmass(scat_mat_2-1)
; note cos^2 + sin^2 = 1 
      if cscat gt 0.0 then begin
        fudge =(sqrt(scat_a^2 +(cscat-1.0)*(cscat+1)*ion_a*ion_a)+ion_a*cscat) $
          /(scat_a + ion_a)
;       fudge is the velocity ratio
;       change to the energy ratio
        fudge = fudge*fudge
      endif else if cscat lt 0.0 then begin
        fudge =(scat_a - ion_a)/(  $
          sqrt(scat_a^2 +(cscat-1.0)*(cscat+1)*ion_a*ion_a)- ion_a*cscat)
;       fudge is the velocity ratio- change to the energy ratio
        fudge = fudge*fudge
      endif else begin
        fudge =(scat_a - ion_a)/(scat_a + ion_a)
      endelse
;     print, ' fudge =', fudge, cscat, scat_mat_2, ion_a
;     Transport through the front of the foil
;     print, ' fudge =', fudge
      eout = ein/fudge
      ein = make_array(np, value = -9.0)
      foil_angle= scat_angle*!dtor
      th_fudge = cos(foil_angle)*(aa+ cc*tan(foil_angle))
;  print, ' thickness fudge = ', th_fudge
      thick = make_array(np, value = 0.5*scat_th_2/th_fudge)
      stopping, zion, ein, eout, thick, scat_mat_2,0.0, edel, ier, $
        aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
      if scat_th_1 gt 0.0 then begin
        eout = ein
        ein = make_array(np, value = -9.0)
        thick = make_array(np, value = scat_th_1/th_fudge)
        stopping, zion, ein, eout, thick, scat_mat_1,0.0, edel, ier, $
          aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
      endif
      if gcell_1_press gt 0 then begin
        eout= ein
        ein = make_array(np, value = -9.0)
; put in a multiplier of 1e4 to convert cm to microns
        thick = make_array(np, value = GCell_1_dist/aa*1.0e4)
        stopping, zion, ein, eout, thick, GCell_1_gas,GCell_1_press, edel, $
          ier, aion = ion_a, ion_name= ion_name, foil_mat = foil_mat, $
          quiet = quiet
        pc2 = (b+ein)*ein
        velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
        x = (ein-eout)/ein
;  linear approximation i.e. - dE/dx = constant in gas cell
        correction=1.0+x*0.25*(1.0+0.5*x*(1.0+.625*x*(1.0+0.7*x*(1.0+0.75*x))))
        time = time +GCell_1_dist/velocity*correction
      endif else begin
        pc2 = (b+ein)*ein
        velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
        time = time +GCell_1_dist/velocity/aa
      endelse
      if GCell_1_th   gt  0.0 then begin
        eout = ein
        ein = make_array(np, value = -9.0)
        thick = make_array(np, value = GCell_1_th/aa)
        stopping, zion, ein, eout, thick, 102L,0.0, edel, ier, aion = ion_a, $
           ion_name= ion_name, foil_mat = foil_mat, quiet = quiet
      endif     
      pc2 = (b+ein)*ein
      velocity =  2.9979246e10*sqrt(pc2/(bsq+pc2))
      time = time +(anode_dist-GCell_1_dist)/velocity/aa
      time = time*1.0e9
str='   PIN Angle  Spec.Energy  Fract.  Ion_Range  Diode_Energy  Transit_Time'
      print, str
      printf, gunit, ';'+str
str='     (deg)       (Mev)             (microns)     (MeV)          (ns)'
      print, str
      printf, gunit, ';'+str
      tlabel = 'WDF Array'

      if npmag lt np then begin
        printf, vunit, ' Magnet radius (cm)  B Field (kG)  CR39 radius (cm)'
        printf, vunit, magnet_r, magnet_b*1.0e-3, cr39_radius, $
               format ='(f13.3, f15.3, f15.3)'
        for j = 1, ion_z do begin 
          print, ' Ion charge = ', strtrim (j, 2)
          printf, gunit, ';  Ion charge = ', strtrim(j, 2)
          sti = (j-1)*npmag ; start position for the output
          stq = j*npmag-1
          espec = enersp(sti:stq)
          frac = eq_charge(ion_z, j, espec, ier)
          if n_elements(frac) ne npmag then $
             frac = make_array(npmag, value = -1.000)
          for i = sti, stq do print,magnet_angle(i-sti), enersp(i), $
            frac(i-sti), pin_range(i), ein(i), time(i), $
            format='(f10.2, f12.3,f10.3, f10.1, f13.3, f14.2)' 
          for i = sti, stq do printf, gunit ,';', magnet_angle(i-sti), $
            enersp(i), frac(i-sti), pin_range(i), ein(i), time(i), $
            format='(a,f10.2, f12.3,f10.3, f10.1,f13.3,f14.2)'
          printf, vunit, ' Zsp   Z   A  Angles'
          printf, vunit, j, ion_z, ion_a, npmag, format='(4i4)'
          printf, vunit, '   Angle (deg)   Vd (MeV)'
          for i = sti, stq do printf, vunit, magnet_angle(i-sti), ein(i), $
            format=' (2f12.4)'
          printf, vunit, ' '
          writepff, 0, enersp(sti:stq), magnet_angle,$
             tlabel = tlabel,clabel= 'Ion Charge = ' + strtrim(j, 2) + $
            ', Spectrometer Ion Energy vs PIN Angle', xlabel= $
            'Spec. Angle (deg)', Ylabel = 'Energy (MeV)'
          writepff, 0, ein(sti:stq), magnet_angle, $
            tlabel = tlabel,clabel= 'Ion Charge = ' + strtrim(j, 2) + $
            ', Diode Ion Energy vs PIN Angle', xlabel= $
            'Spec. Angle (deg)', Ylabel = 'Energy (MeV)'
          writepff, 0, pin_range(sti:stq), magnet_angle, $
            tlabel = tlabel,clabel= 'Ion Charge = ' + strtrim(j, 2) + $
            ', Ion Range in PIN vs PIN Angle', xlabel= $
            'Spec. Angle (deg)', Ylabel = 'Ion Range (micron)'
          writepff, 0, time(sti:stq), magnet_angle, $
            tlabel = tlabel,clabel= 'Ion Charge = ' + strtrim(j, 2) + $
            ', Ion Transit Time vs PIN  Angle', xlabel= $
            'Spec. Angle (deg)', Ylabel = 'Ion Transit Time (ns)'
        endfor
      endif else begin
        espec = enersp(0:npmag-1)
        frac = eq_charge(ion_z, ion_q, espec, ier)
        if n_elements(frac) ne npmag then $
             frac = make_array(npmag, value = -1.000)
        for i = 0, npmag-1 do print,magnet_angle(i), enersp(i), $
          frac(i),pin_range(i),ein(i),time(i), $
          format='(f10.2, f12.3, f10.3, f10.1, f13.3, f14.2)' 
        for i = 0, npmag-1 do printf, gunit ,';', magnet_angle(i), enersp(i), $
          frac(i), pin_range(i),ein(i),time(i), $
          format='(a,f10.2, f12.3, f10.3, f10.1, f13.3, f14.2)'
        printf, vunit, ' Magnet radius (cm)  B Field (kG)  CR39 radius (cm)'
        printf, vunit, magnet_r, magnet_b*1.0e-3, cr39_radius, $
               format ='(f13.3, f15.3, f15.3)'
        printf, vunit, ' Zsp   Z   A  Angles'
        printf, vunit, ion_q, ion_z, ion_a, npmag, format='(4i4)'
        printf, vunit, '   Angle (deg)   Vd (MeV)'
        for i = 0, npmag-1 do printf, vunit, magnet_angle(i), ein(i), $
           format=' (2f12.4)'
        writepff, 0, enersp, magnet_angle, tlabel = tlabel,clabel= $
          'Spectrometer Ion Energy vs PIN Angle', xlabel= 'Spec. Angle (deg)', $
          Ylabel = 'Energy (MeV)'
        writepff, 0, ein, magnet_angle, tlabel = tlabel,clabel= $
          'Diode Ion Energy vs PIN Angle', xlabel= 'Spec. Angle (deg)', $
          Ylabel = 'Energy (MeV)'
        writepff, 0, pin_range, magnet_angle, tlabel = tlabel,clabel= $
          'Ion Range in PIN vs PIN Angle', xlabel= 'Spec. Angle (deg)', $
          Ylabel = 'Ion Range (micron)'
        writepff, 0, time, magnet_angle, tlabel = tlabel,clabel= $
          'Ion Transit Time vs PIN  Angle', xlabel= 'Spec. Angle (deg)', $
          Ylabel = 'Ion Transit Time (ns)'
      endelse

      free_lun, vunit
      free_lun, gunit
      return
      end
