; ------------------------------------------------------------------------ 
; $Id: strcomment.pro,v 1.4 2010/09/17 20:51:05 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function strcomment, wdf

;
;+
;  This function returns the structure comment for an array. 
;  Format:  com = STRCOMMENT(STR)
;  where:
;    STR - Valid Structure number or PFIDL structure
;  Example:
;    Print the comment label for structure array 5
;      Print, strcomment(5)
;    Add the shot number to a comment of structure array 3, 
;       where shot number is SHOT.
;      chastr, 3, c= strcomment(3)+', Shot '+strtrim(shot, 2)
;
;-
;
;
@qsstrcom.dat
on_error, 1
      ok = '' 
      if n_elements(structinit) eq 0 then return, ok
;
      if n_elements(wdf) eq 0 then begin 
        print, '% STRCOMMENT - Structure array number must be defined'
        return, ''
      endif
      st = size(wdf, /type)
      if st ne 8 then begin
        if st gt 0 and st lt 6 then wd = round(wdf) else begin
          print, '% STRCOMMENT - Structure array must be a real number'
          return, ok
        endelse 
        if wd lt 1 or wd gt maxstructure then begin
          print, $
            '% STRCOMMENT - Structure array must be between 1 and ', $
             strtrim(maxstructure, 2)
          print, '          Value = ', strtrim (wd, 2)
          return, ''
        endif
;
        if lcvalidstr(wd-1) ne 0 then return, comment(wd-1) else begin
          print, '% STRCOMMENT - Not a valid structure number: ', strtrim(wd,2)
          return, ok
        endelse
      endif
      ok = where(strpos(tag_names(wdf), 'CLAB') ge 0 , nok)
      if nok eq 1 then return, wdf.clab
      print, '% STRCOMMENT - Not a valid PFIDL structure'
      return, ok
      end

