; ------------------------------------------------------------------------ 
; $Id: dirsaf.pro,v 1.5 2007/06/06 20:10:58 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro dirsaf, quiet = quiet, PAGE = page, RESET = reset, ndset = ndset, $
       truncate = truncate, xsize = xsize, line= line
;+
;
;     this routine prints a directory of pff files
;
;     DIRSAF [, Page = Page] [, NDSET = ndset]
;         [, TRUNCATE = truncate] [, /QUIET]
;         [, Reset = reset] [, Xsize = xsize]
;     where:
;           page is the number of lines on the directory
;                default is directorypage
;           reset if present and not zero, resets the directory
;                use this if the window has been closed
;             also: if the window is still open and you wish to leave it
;                open, use /reset to put the current command in a second
;                window.
;           ndset is the number of datasets returned
;           xsize is the width of the window in pixels (generally not set)
;           truncate - if set, the dataset comments will be truncated at ' - '
;                      This is a toggle; If not specified the last value will
;                      be used.  Default value =0 -- no truncate
;     Example:
;         Get a directory of the current SAF file and the number of datasets.
;           dirsaf, ndset = ndset
;
;-
;
      on_error, 1
      common safcom, initsaf, saf_exe
      common dirsaf_save, base, list, oldpage,  text1, truncate_save
;
@saf_com.dat
;
@qscom.dat
      if n_elements(qsplotinit) eq 0 then qsinit
;
;     check to see if saf has been initialized
;
      if (n_elements(initsaf) eq 0) then init_saf else $
        if (initsaf eq 0) then init_saf
      if initsaf eq 0 then begin
        print, 'SAF libraries are not supported by this Operating System'
        return
      endif
;
;     see if a file is open
      if strlen(saf_filename) eq 0 then begin
        print, '% DIRSAF:  No files are open'
        print, '           Directory is not possible.'
        return
      endif
      if n_elements (oldpage) eq 0 then oldpage = directorypage
      if n_elements (page) eq 0 then page = oldpage
      page = page >4
;
      if n_elements (base) eq 0 then begin
        base = 0
        text1 = 0
        text2 = 0
      endif
;
      filename = saf_filename 
; get_saf_dir, p1, p2, p3, line = line, filename = filename, number = ndset, $
  get_saf_dir, line = line, filename = filename, number = ndset, $
              sortit = sortit, dataset = dataset
      if n_elements(line) lt 2 then begin
        print, '% DIRSAF: No Data'
        if n_elements(line) gt 0 then print, line[0]
        return
      endif
      saf_directory_list = dataset
      n_elem_saf_dir = ndset
;
      if keyword_set(truncate_save) then begin
        for i = 0, ndset-1 do begin
          ii = strpos(line(i), ' - ', 25)
          if ii gt 0 then line(i) = strmid(line(i), 0, ii)
        endfor
      endif

      if ((ndset eq 0) or (keyword_set(quiet))) then return
      if keyword_set(reset) then base = 0
      if oldpage ne page then page = -page
      widget_directory, base, page, list, line, base_title = 'SAF Directory', $
        xsize = xsize, text1 = text1, value1 = filename,  dirtype =4, $
        text2 = text2, fileid = fileid, no_block = no_block,  $
        value2 = ' #   Data Type       Dataset Comment'

      oldpage = page
      end
