; ------------------------------------------------------------------------ 
; $Id: readsaf.pro,v 1.5 2007/06/06 20:10:58 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro readsaf, farray, datasetin , true_location = true_location, $
     structure = structure, oldversion = oldversion

;+
;    % Format: READSAF, structure  , dataset
;                 [, /OLDVERSION]
;                 [, /structure] [, /true_location]'
;   oldversion - If set transpose the hyperslab
;
;-
      on_error, 1
      common safcom, initsaf, saf_exe
;  Common block
@qsstrcom.dat
;
@saf_com.dat
;
;     check to see if saf has been initialized
;
      if (n_elements(initsaf) eq 0) then initsaf =0
      if initsaf eq 0 then begin
        print, '% READSAF: No SAF files are open
        return
      endif
;
      if n_elements (structinit) eq 0 then qsstruct_init
      nparams =  n_params()
      
;      farray = 0.0
      if (nparams lt 1 ) then begin
        print,'% Format: READSAF, structure  , dataset'
        print,'               [, /OLDVERSION]
        print,'               [, /structure] [, /true_location]'
        return
      endif
      if nparams lt 2 then begin
        print,'% READSAF:  READSAF must have 2 parameters'
        print,'% Format: READSAF, structure  , dataset'
        print,'               [, /OLDVERSION]
        print,'         [, /structure] [, /true_location]
        return
      endif
      str=0
      slice = 0.0
      if not keyword_set(structure) then begin
        if n_elements(farray) eq 0 then begin
          print, '% READSAF: Structure must be provided'
          return
        endif
        str = farray
        if str lt 1 or str gt maxstructure then begin
          print, '% READSAF: QS structure array number must be greater than'
          print, '  zero and less than or equal to', maxstructure
          return
        endif
      endif else farray =0.0

      if n_elements (datasetin) eq 0 then dataset = 0L else $
        dataset = datasetin(0)
      if dataset eq 0 then begin
        print, '% ReadSAF:  Dataset must be defined.'
        return 
      endif
      ndset = n_elem_saf_dir
      if ndset eq 0 then begin
        dirsaf, /quiet
        ndset = n_elem_saf_dir
        if ndset eq 0 then begin
          print, '% READSAF: No files are open'
          return
        endif
      endif

      if dataset gt ndset then begin
        print, '% ReadSAF:  Dataset must be in the range 1:'+ $
          strtrim(ndset, 2)
        return
      endif 
      dataset = dataset -1
;
        readsafhdr,dataset, nblo = nblo,  nxyz= nxyz, nvec= nvec
       
      
      mblo = nblo
      mdim = nvec

      farray = make_array(mdim)
      lenx = make_array(mblo, /long)
      leny = make_array(mblo, /long)
      lenz = make_array(mblo, /long)
      mx = nxyz[0]
      my = nxyz[1]
      mz = nxyz[2]
      x = make_array(mx)
      y = make_array(my)
      z = make_array(mz)
      
      locx = make_array(mblo+1, /LONG)
      locy = make_array(mblo+1, /LONG)
      locz = make_array(mblo+1, /LONG)
      locfa = make_array(mblo+1, /LONG)
      lenfa = make_array(mblo, /LONG)
      olocx = make_array(mblo+1, /LONG)
      olocy = make_array(mblo+1, /LONG)
      olocz = make_array(mblo+1, /LONG)
      olocfa = make_array(mblo+1, /LONG)

      tapp = 0L

      if keyword_set(oldversion) then h_tran = 1L else h_tran = 0L

      if  initsaf lt 3 then begin
      ; unix
        junk = call_external(SAF_EXE,'saf_read',  dataset, tapp,  $
                lenx, leny, lenz,  x, y, z, farray, h_tran)

      endif else begin
         PRINt, '% READSAF: SAF is not available on this machine.'
      endelse
      tapp = tapp+1
      
      lenfa = lenx*leny*lenz
      olocfa[1:*] = round(total(lenfa, /cum, /double))
      olocx[1:*] = round(total(lenx, /cum, /double))
      olocy[1:*] = round(total(leny, /cum, /double))
      olocz[1:*] = round(total(lenz, /cum, /double))
; help, true_location
      for i = 0, mblo-1 do begin
        xx = x[olocx[i]:olocx[i+1]-1]
        yy = y[olocy[i]:olocy[i+1]-1]
        zz = z[olocz[i]:olocz[i+1]-1]
        nx = lenx[i]
        ny = leny[i]
        nz = lenz[i]
        vv = reform (farray[olocfa[i]:olocfa[i+1]-1], nx, ny, nz, /over)
        reduce_saf_array, xx, yy, zz, nx, ny, nz, vv, type = tapp, $
          true_location = true_location, tolerance = tolerance
        lenx[i] = n_elements(xx)
        leny[i] = n_elements(yy)
        lenz[i] = n_elements(zz)
        lenfa[i] = n_elements(vv)
        if (lenx[i]*leny[i]*lenz[i]-lenfa[i] ne 0) then begin
          print, '% READSAF: Error with numbers from REDUCE_SAF_ARRAY', $
                 i, lenx[i],leny[i],lenz[i],lenfa[i] 
          return
        endif
        locx[i+1] = locx[i]+lenx[i]       
        locy[i+1] = locy[i]+leny[i]       
        locz[i+1] = locz[i]+lenz[i]       
        locfa[i+1] = locfa[i]+lenfa[i]       

        x[locx[i]:locx[i+1]-1] = xx
        y[locy[i]:locy[i+1]-1] = yy
        z[locz[i]:locz[i+1]-1] = zz
        farray[locfa[i]:locfa[i+1]-1] = vv
      endfor
      x = temporary(x[0:locx[mblo]-1])
      y = temporary(y[0:locy[mblo]-1])
      z = temporary(z[0:locz[mblo]-1])
      farray = temporary(farray[0:locfa[mblo]-1])

     range = make_array(2, 3, mblo+1)
     for i = 0, mblo-1 do begin
       range (0, 0, i+1) = min(x(locx(i): locx(i+1)-1), max = mm)
       range (1, 0, i+1) = mm
       range (0, 1, i+1) = min(y(locy(i): locy(i+1)-1), max = mm)
       range (1, 1, i+1) = mm
       range (0, 2, i+1) = min(z(locz(i): locz(i+1)-1), max = mm)
       range (1, 2, i+1) = mm
     endfor
     locx = transpose(reform([locx, locy, locz], mblo+1, 3))
     range (0, 0, 0) = min(range(0, 0, 1:mblo))
     range (0, 1, 0) = min(range(0, 1, 1:mblo))
     range (0, 2, 0) = min(range(0, 2, 1:mblo))
     range (1, 0, 0) = max(range(1, 0, 1:mblo))
     range (1, 1, 0) = max(range(1, 1, 1:mblo))
     range (1, 2, 0) = max(range(1, 2, 1:mblo))

     clabel = saf_directory_list[dataset]
     filename = saf_filename
     ssize = make_array(3, mblo, /long)
     ssize(0, *)= lenx
     ssize(1, *)= leny
     ssize(2, *)= lenz

     x1lab = make_array(mblo, /string, value = 'X (m)')
     x2lab = make_array(mblo, /string, value = 'Y (m)')
     x3lab = make_array(mblo, /string, value = 'Z (m)')
     x4lab = ''
     if tapp lt 5 then $
     v1lab = make_array(mblo, /string, value = 'Electric Field (V/m)') $
     else v1lab = make_array(mblo, /string, value = 'Magnetic Field (T)') 
     v2lab = ''
     v3lab = ''
     v4lab = ''
     v5lab = ''
     v6lab = ''
     v2= 0.0
     v3= 0.0
     v4= 0.0
     v5= 0.0
     v6= 0.0
;
; common saf_common, saf_directory_list, saf_filename
;     check on the number of parameters
;
     x4 =0.0
     ispare = [0,0,0,0,0]
     tlabel ='SAF Field Data'
     if mblo gt 1 then begin
       order = make_array(3, mblo, /long)
       for i = 0, 2 do order (i, *) = sort(range(0,i, 1:mblo))
     endif else order = 1L

     farray = {type:1L, ndim:3L, nvect: 1L, block :mblo, two: 0, $
              slice: slice, size: ssize, range: range, $
              order: order, locx:locx, locv: locfa, $
              x1:x, x2:y, x3:z, x4:x4,  $
              v1:farray, v2:v2, v3:v3, v4:v4, $
              v5:v5, v6:v6,x1lab : x1lab, x2lab :x2lab, $
              x3lab : x3lab, x4lab :x4lab,v1lab : v1lab, v2lab :v2lab, $
              v3lab : v3lab, v4lab :v4lab,v5lab : v5lab, v6lab :v6lab, $
              file : filename, spare : ispare, tlab : tlabel, clab:clabel }

      if str eq 0 then return
      i2s, farray, str, /no_copy
      farray = temporary(str)

      return
      end
