; ------------------------------------------------------------------------ 
; $Id: get_grid_parm.pro,v 1.3 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
pro get_grid_parm, axis, a, b, x, n, a=ak, b = bk, x0= x0, np = np

;+
;
;  This is a utility function for returning the grid parameters for a quadratic
;  grid.
;         Grid is of the form: 
;  Quadratic:  X = x0 + A*I + B*I*I  for i= 0, 1, ...
;  Geometric:  X = DX/log(R) * (R^I -1) + x0
;
; format: GET_GRID_PARM, AXIS, A, B, X0, NP, A=a, B=b, X0= x0, NP = np
;    where:
;          AXIS  - Axis direction = 1, 2, 3
;          A     - Linear coefficient or (Delta X0= DX0)
;          B     - Quadratic coefficient or (Ratio coefficient -1)
;                  (R-1 is used for geometric to increase precision for a given
;                   number of numbers.) 
;          X0    - Initial value
;          NP    - Number of grid points for Quicksilver grids 
;    Note: Keywords or positional parameters may be used
;          
;
; Example:
;    Get grid parameters for current radial grid (direction 1)
;       GET_GRID_PARM, 1, A, B, X, N
;    Get the a and b coeficients for a Y grid (direction 2)
;       GET_GRID_PARM, 2, A, B
; <or>  GET_GRID_PARM, 2, a = a, b = b
;-
  on_error, 1
;
@tq_qs_gridparam.com
;common tq_qs_gridparam, grid_param_1, grid_param_2, grid_param_3
; 

a = 0 & b=0 & x= 0 & n = 0

if n_elements(axis) lt 0 then begin
  print, '% QS_GET_GRID_PARM: Axis must be defined
  return
endif

if axis gt 3 or axis lt 1 then begin
  print, '% QS_GET_GRID_PARM:  Axis value must be in the range [1, 3]
  return
endif

if axis eq 1 then begin
  if n_elements(grid_param_1) eq 0 then begin
    print, '% QS_GET_GRID_PARM:  Parameters are not defined for X axis'
    print, '      Use TQGRID or QSGRID to calculate parameters.'
    return
  endif
  str = grid_param_1 
endif else if axis eq 2 then begin
  if n_elements(grid_param_2) eq 0 then begin
    print, '% QS_GET_GRID_PARM:  Parameters are not defined for Y axis'
    print, '      Use TQGRID or QSGRID to calculate parameters.'
    return
  endif
   str = grid_param_2 
endif else begin
  if n_elements(grid_param_3) eq 0 then begin
    print, '% QS_GET_GRID_PARM:  Parameters are not defined for Z axis'
    print, '      Use TQGRID or QSGRID to calculate parameters.'
    return
  endif
  str = grid_param_3
endelse

 a = str.a
 b = str.b
 x = str.x0
 n = str.n
 ak = a
 bk = b
 x0 = x
 np = n
 str = 0
end
