; ------------------------------------------------------------------------ 
; $Id: restore_markers.pro,v 1.3 2007/06/06 20:12:25 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro restore_markers, fname, reset = reset
 on_error, 1
;+ 
; read markers from an ascii file
; file generated from save_markers
; file format:
;         name
;         x, y
;         comment
;  Name must be have one or more ascii characters.
;  # indicates a comment and is ignored.
;  a blank line is ignored except for the comment line
; 
; Command format:
;   RESTORE_MARKERS [, FILENAME] [/RESET]
;   where
;      FILENAME  - Name of the file containing the marker information
;                  Default: Use pickfile
;      RESET     - If set, all current markers are removed.
;-
 if n_elements(fname) eq 0 then fname = '' else fname = strtrim(fname, 2)
 i = strlen(fname)
 if i lt 1 then begin
   fname = dialog_pickfile(/must_exist, /read)
   if strlen(fname) lt 1 then begin
     print, ' No File was Selected - RETURNING'
     return
   endif
 endif
 openr,unit,fname, /get_lun, error = error
 if error ne 0 then begin
   print, '% RESTORE_MARKERS: Error opening file'
   return
 endif
 if keyword_set(reset) then rm_markers
 mxpoint = 512L
 x = make_array(mxpoint)
 y = make_array(mxpoint)
 str1 = make_array(mxpoint, /string)
 str2 = make_array(mxpoint, /string)
 strin  = ' '
 strin2 = ' '
 n = 0
 again =1 
; Get the name of the first marker
 while again eq 1 do begin
   readf, unit, strin
   strin1 = strtrim(strin, 2) 
   if ((strlen(strin1) gt 0) and  $
     (strmid(strin, 0, 1) ne '#')) then again = 0
 endwhile
 again = 1
 while (again eq 1) do begin
;  Get the x, y locations
   while again eq 1 do begin
     readf, unit, strin
     strin2 = strtrim(strin, 2) 
     if ((strlen(strin2) gt 0) and  $
       (strmid(strin, 0, 1) ne  '#')) then again = 0
   endwhile
   reads, strin2, xx, yy
;  Get the cmoment string
   readf, unit, strin2
   while (strmid(strin2, 0, 1) eq '#') do readf, unit, strin2
   if n eq mxpoint  then begin
     mxpoint = mxpoint+mxpoint
     x = [x, x]
     y = [y, y]
     str2 = [str2, str2]
     str1 = [str1, str1]
   endif
   x(n) = xx
   y(n) = yy
   str2(n) = strin2
   str1(n) = strin1
   n = n+1
;  Get the next marker name or get rid of any trailing lines 
   again = 1
   while ((not eof(unit)) and (again eq 1)) do begin
     readf, unit, strin
     strin1 = strtrim(strin, 2) 
     if ((strlen(strin1) gt 0) and  $
       (strmid(strin, 0, 1) ne '#')) then again = 2
   endwhile
   again = again - 1 
 endwhile

 free_lun, unit
 for i = 0L, n-1 do begin
   tqhelp, gx = gx, gy = gy
   junk = min(abs(gx-x(i)), ii)
   junk = min(abs(gy-y(i)), jj)
   add_tqmrk, [ii, jj], str2[i], str1[i]
 endfor
 end
