; ------------------------------------------------------------------------ 
; $Id: tq2s.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tq2s, str, all = all
;+
;  TQ2S
;  Purpose: Convert a conductor fitted to a grid to a conductor structure 
;           or multiple conductor structures.
;
;  Format:  TQ2S [, str] [, /ALL ]
;    where:
;           str   - Structure array array number for the first conductor
;                   Default is 50.
;                   Caution: Str can be changed for each conductor.
;                            No checks are make to insure that conductors
;                            are not overwritten.
;           ALL   - If keyword is set, then all conductors will be 
;                   put into a single structure array.
;-
;
@pfidltq.com
;
    on_error, 1
;   see if conductors are defined
    if n_elements(cond) eq 0 then message, $
      'Conductors must be defined to fill an array'
    if n_elements (gx) lt 3 or n_elements (gy) lt 3 then begin
        wwarning, '  Use TQGRID to generate a grid   '
        return
    endif
    if n_elements(cond.iy) lt 3 or n_elements(cond.ix) lt 3 then begin
        wwarning, '  Use TQFIT to fit data to the grid   '
        return
    endif
    if total(cond.lcfit) ne n_elements(cond.lcfit) then begin
        wwarning, ' Use TQFIT to fit data to the grid   '
        return
    endif
;
    nct = cond.ncon
    if nct eq 0 then $
      message, 'No conductors have been defined.'
    get_tqbad
    if cond.nbad(0) ne 0 then begin
      print, ' Bad points must be removed before calling TQ2S.'
      return
    endif
;
    location = cond.loci
    nix = cond.ix
    niy = cond.iy
    xxx = gx
    yyy = gy
    xran = [min(gx, max = mxx), mxx]
    yran = [min(gy, max = myy), myy]
;
    nx = n_elements(xxx)
    ny = n_elements(yyy)

    if keyword_set(all) then step = nct else step = 1
    if n_elements(str) eq 0 then str = 50 else str = round(str(0))
    for i = 0, nct-1, step do begin
      loc = location(i:i+step)- location(i)
      ix = nix(location(i): location(i+step)-1)
      iy = niy(location(i): location(i+step)-1)
      np = min([n_elements(ix), n_elements(iy)])
      xx = gx(ix)
      yy = gy(iy)
      plot, gx, gy, /nodata, /xst, /yst, xran= xran,yran= yran
      for j = 0, nct-1 do  oplot, gx(nix(location(j):location(j+1)-1)), $
         gy(niy(location(j):location(j+1)-1)), line = 2, color = 4
      if step eq 1 then oplot, xx, yy, color = 7, thick = 1.5
      strnumb = i+ str
      get_cxy_labels, lcquit, strnumb, clabel, xlabel, ylabel, offset
      if lcquit ne 0 then return
      cond2str, s, xx* mult, yy* mult, loc, clabel = clabel, $
          xlabel = xlabel, ylabel= ylabel
      i2s, s, strnumb
    endfor

    return
    end
