; ------------------------------------------------------------------------ 
; $Id: tqfill.pro,v 1.3 2007/06/06 20:12:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqfill, array, xxx, yyy, value = valuein
;
;+
;   purpose:  Fill an array with number for conductors 
;             If an error occurs, array = 0
;   format:  tqfill, array, x, y [, value = value]
;
;   NOTE: Conductors must be closed with all bad points removed.
;     where:
;          Array is the grid array of integers
;          x are the X values
;          y are the Y values
;          value is optional fill value, must be an integer
;                default is the conductor number 
;
;    example: 
;        Return a array of the conductor locations.
;            tqfill , array, xx, yy 
;-
@pfidltq.com
;
    on_error, 1
    array = 0
;   see if conductors are defined
    if n_elements(cond) eq 0 then message, $
      'Conductors must be defined to fill an array'
    if n_elements (gx) lt 3 or n_elements (gy) lt 3 then begin
        wwarning, '  Use TQGRID to generate a grid   '
        return
    endif
    if n_elements(cond.iy) lt 3 or n_elements(cond.ix) lt 3 then begin
        wwarning, '  Use TQFIT to fit data to the grid   '
        return
    endif
    if total(cond.lcfit) ne n_elements(cond.lcfit) then begin
        wwarning, ' Use TQFIT to fit data to the grid   '
        return
    endif

;
    nct = cond.ncon
    if nct eq 0 then $
      message, 'No conductors have been defined.'
    get_tqbad
    if cond.nbad(0) ne 0 then begin
      print, ' Bad points must be removed before calling TQFILL.'
      return
    endif
;
    location = cond.loci
    nix = cond.ix
    niy = cond.iy
    xxx = gx
    yyy = gy
    nx = n_elements(xxx)
    ny = n_elements(yyy)
    array = make_array(nx, ny, /int)
    for i = 0, nct-1 do begin
      temp = array*0
      value = 1
      ix = nix(location(i): location(i+1)-1)
      iy = niy(location(i): location(i+1)-1)
      np = n_elements(ix)
      if (ix(0) ne ix(np-1) or iy(0) ne iy(np-1)) then begin
        print, ' Conductors must be closed: ', i
        print, ix(0) ,  ix(np-1),iy(0) ,  iy(np-1)
        array =0
        return
      endif
      if n_elements(valuein) gt 0 then value = valuein(0) else value = i+1
      ind2fill,value,ix, iy, np, temp, nx, ny 
      if n_elements(temp) lt 2 then return
      if i eq 0 then array = temp else begin
        ok = where (temp ne 0, nok)
        if nok gt 0 then array(ok) = value
;       ok = where (array eq 0, nok)
;       if nok gt 0 then array(ok) = temp(ok)
      endelse
    endfor
    return
    end
