; ------------------------------------------------------------------------ 
; $Id: tqreadcon.pro,v 1.3 2007/06/06 20:12:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqreadcon, ain, xin, yin, ZVALUE, nosort = nosort, _extra= extrastuff
;
;+
; read a conductor into a tq conductor structure.
;
; Format: tqreadcon, dataset, x-axis, y-axis [, zvalue] 
;        [, nosort = nosort] [, _extra = extrastuff]
;  where:
;       dataset - Conductor structure or structure array number
;       x-axia   - 1, 2, or 3 - the x-axix coordinate value
;       y-axia   - 1, 2, or 3 - the y-axix coordinate value
;       zvalue   - optional value of the remaining coordinate.
;                  Default is to take the midpoint of the first block.
;       nosort   - if set, conductor segments will be left separate.
;       extrastuff - valid keywords for TQCOND
;       example;
;          Read the structure array 2 into the tq structure with 
;          Z as x and R as y. Read all blocks. 
;            TQREADCON, 2, 3, 1 , 0
;-
@pfidltq.com
;
@qsstrcom.dat
;
on_error, 1
;
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      if n_elements(ain) eq 0 then begin
        print, 'Grid structure must be defined'
        return
      endif else a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 4 then begin
        print, ' Only Conductor type structures may be used for TQREADCON'
        return
      end

      nx = n_elements (xin)
      if nx eq 0 then begin
        Print, ' X array must be defined'
        return
      endif else xt = xin(0)
      if xt lt 1 or xt gt 3 then begin
        print, ' Abscissa must be one of the there dimensions not =', xt
        return
      endif
;
      ny = n_elements (yin)
      if ny eq 0 then begin
        Print, ' Y array must be defined'
        return
      endif else yt = yin(0)
;
      if yt lt 1 or yt gt 3 then begin
        print, ' Ordinate must be one of the there dimensions not =', yt
        return
      endif
      
      zplane = 6-xt-yt
      mmax = max(a.plane(1, zplane-1, *))
      mmin = min(a.plane(0, zplane-1, *))
      if n_elements(zvalue) eq 0 then $
            zvalue = (mmin + mmax)*0.5 else if n_elements(zvalue) gt 1 then $
            zvalue = ((zvalue (0)> mmin) + (zvalue(1)< mmax) )*0.5
      if zvalue lt mmin or zvalue gt mmax then zvalue = (mmin + mmax)*0.5
         
      err = 4.57764e-5*( max(a.plane(1, *, *))- min(a.plane(0, *, *)))
      nn= zplane
      if nn eq 3 then begin
;       arr = [a.n0,a.n1,a.n2]
        arr = [a.n1,a.n2]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where(a.plane(0,2, arr) le zvalue+err and $
                 a.plane(1,2, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endif else if nn eq 2 then begin
;       arr =  [a.n0,a.n1,a.n3]
        arr =  [a.n1,a.n3]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where( a.plane(0,1, arr) le zvalue+err and $
               a.plane(1,1, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endif else begin
;       arr = [a.n0,a.n2,a.n3]
        arr = [a.n2,a.n3]
        n = where (arr ge 0, np)
        if np gt 0 then begin
          arr = arr(n)
            n = where(a.plane(0 ,0, arr) le zvalue+err and $
               a.plane(1,0, arr) ge zvalue-err, np)
            if np gt 0 then arr = arr(n)
        endif
      endelse
;     we will just throw the slant surfaces in-
;     They are only used on 2D data.
;     If slants are used on 3D data this logic will need to be improved!!
;       see use of xtemp and ytemp below
      if a.size(0) gt 0 then begin
        nslant = a.size(0)
        xslant = reform (a.slant(xt-1, *,*))
        yslant = reform (a.slant(yt-1, *,*))
      endif else nslant = 0
      if np eq 0  and nslant eq 0 then begin
        Print, ' No conductor available for these x, y, zvalue parameters'
        print, x, y, zvalue
      endif
      xx = a.plane(*, xt-1, arr)
      yy = a.plane(*, yt-1, arr)
      xx = reform (xx, 2*np)
      yy = reform (yy, 2*np)
      if nslant gt 0 then begin
        xx = [xx , reform (xslant, 2*nslant)]
        yy = [yy , reform (yslant, 2*nslant)]
      endif
      resolution = err
      resol2 = err*err/10.
      nline = nslant+np
      locg = lindgen(nline+1)*2
        change = 0
      if keyword_set(nosort) then goto, finished
        i = 0
newline:
        i = i+1
        if i eq nline then goto, returnhere
again3:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i-1)) - xx(locg(check))) lt resolution and $
            abs( yy(locg(i-1))- yy(locg(check))) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i-1)) - xx(locg(check(ok))))^2+ $
                 (yy(locg(i-1))-yy(locg(check(ok))))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, rotate(xx(locg(n-1):locg(n)-1),2), $
               xx(locg(i-1):locg(i)-1), rotate(yy(locg(n-1):locg(n)-1), 2), $
               yy(locg(i-1):locg(i)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again3
          endif
again4:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i-1)) - xx(locg(check+1)-1)) lt resolution $
            and abs( yy(locg(i-1))- yy(locg(check+1)-1)) lt resolution, ns)
          if ns gt 0 then begin
            if ns gt 1 then begin
              change = 1
              junk = min((xx(locg(i-1)) - xx(locg(check(ok)+1)-1))^2+ $
                 (yy(locg(i-1))-yy(locg(check(ok)+1)-1))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(n-1):locg(n)-1), $
               xx(locg(i-1):locg(i)-1), yy(locg(n-1):locg(n)-1), $
               yy(locg(i-1):locg(i)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again4
          endif
again5:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i)-1) - xx(locg(check))) lt resolution and $
            abs( yy(locg(i)-1)- yy(locg(check))) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i)-1) - xx(locg(check(ok))))^2+ $
                 (yy(locg(i)-1)-yy(locg(check(ok))))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(i-1):locg(i)-1), $
               xx(locg(n-1):locg(n)-1), yy(locg(i-1):locg(i)-1), $
               yy(locg(n-1):locg(n)-1), i, nline, locg, xx, yy, resol= resol2
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if i eq nline then goto, returnhere
            goto, again5
          endif
again6:
        check = indgen(nline)
        ok = where (check ge i)
        check = check(ok)
          ok = where (abs(xx(locg(i)-1) - xx(locg(check+1)-1)) lt resolution $
            and abs( yy(locg(i)-1)- yy(locg(check+1)-1)) lt resolution, ns)
          if ns gt 0 then begin
            change = 1
            if ns gt 1 then begin
              junk = min((xx(locg(i)-1) - xx(locg(check(ok)+1)-1))^2+ $
                 (yy(locg(i)-1)-yy(locg(check(ok)+1)-1))^2,k)
              ok(0) = ok(k)
            endif
            n = check(ok(0))+1
            join2pack, xx(locg(i-1):locg(i)-1), $
               rotate(xx(locg(n-1):locg(n)-1),2) , yy(locg(i-1):locg(i)-1), $
               rotate(yy(locg(n-1):locg(n)-1),2), i, nline, locg, xx, yy, $
                resol= resol2
;
            array2pack, n, nline, locg, 0, xx, vx, yy, vy
            if nline eq i then goto, returnhere
            goto, again6
          endif
          goto, newline
returnhere:
          if change eq 1 then begin
            change = 0
            i = 0
            goto, newline
          endif
finished:
locg = locg(0:nline)
 xx = xx(0:locg(nline)-1)
 yy = yy(0:locg(nline)-1)
tqcond, xx, yy, locg, _extra = extrastuff
   return
   end
