; ------------------------------------------------------------------------ 
; $Id: tqreadgrd.pro,v 1.3 2007/06/06 20:12:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro tqreadgrd, ain, xin, yin, zvalue, block = block
;
;+
; read a grid into a tq gridding structure.
;
; Format: tqreadgrd, dataset, x-axis, y-axis [, zvalue] [, block = block] 
;  where:
;       dataset - Grid structure or structure array number
;       x-axia   - 1, 2, or 3 - the x-axix coordinate value
;       y-axia   - 1, 2, or 3 - the y-axix coordinate value
;       zvalue   - optional value of the remaining coordinate.
;                  Default is to take the midpoint of the problem .
;       block    - optional block specifier, 
;                  NOTE: Grid must be continuous in the specified blocks.
;       example;
;          Read the structure array 2 into the tq structure with 
;          Z as x and R as y. Read all blocks. 
;            TQREADGRD, 2, 3, 1  
;-
@pfidltq.com
@qsstrcom.dat
;
on_error, 1
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      if n_elements(ain) eq 0 then begin
        print, 'Grid structure must be defined'
        return
      endif else a = ain
      b = size(a)
      if b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          s2i, a, a
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      if a.type ne 3 then begin
        print, ' Only Grid type structures may be read with TQREADGRD'
        return
      end

      nx = n_elements (xin)
      if nx eq 0 then begin
        Print, ' X array must be defined'
        return
      endif else xt = xin(0)
      if xt lt 1 or xt gt 3 then begin
        print, ' Abscissa must be one of the there dimensions not =', xt
        return
      endif
;
      ny = n_elements (yin)
      if ny eq 0 then begin
        Print, ' Y array must be defined'
        return
      endif else yt = yin(0)
;
      if yt lt 1 or yt gt 3 then begin
        print, ' Ordinate must be one of the there dimensions not =', yt
        return
      endif
      if n_elements (block) eq 0 then block = 0
      if n_elements (block) gt 1 then begin
        good = where (block gt 0 and block le a.block,i)
        if i lt 1 then begin
          print, ' No valid blocks specified for PLOTGRD'
          return
        endif
        blockarray = block(good)
      endif else begin
        if block lt 0 or block gt a.block then block = 0
        if block gt 0 then ts = block else ts = 1
        if block gt 0 then tf = block else tf = a.block
        blockarray = indgen(tf-ts+1)+ts
      endelse
      zplane = 6-xt-yt
      if n_elements(zvalue) eq 0 then $
            zvalue = total(a.range(0:1, zplane-1, 0))*0.5
      ok = where (zvalue ge a.range(0, zplane-1, blockarray) and $
                zvalue le a.range(1, zplane-1, blockarray), i)
      if i eq 0 then begin
        print, ' None of the specifed blocks exist at a dimension',$
           zplane, ' value of' , zvalue
        return
      endif else blockarray = blockarray(ok)

      iblo = blockarray(0)
      xx = get_array (a, xt, iblo)
      yy = get_array (a, yt, iblo)
      if i gt 1 then for ii = 1, i-1 do begin
        iblo = blockarray(ii)
        xx = [xx, get_array (a, xt, iblo)]
        yy = [yy, get_array (a, yt, iblo)]
      endfor
      nx = n_elements(xx)
      ny = n_elements(yy)
      i = sort(xx)
      xx = xx(i)
      i = sort(yy)
      yy = yy(i)
      gdx = xx(1:nx-1)-xx(0:nx-2)
      gdy = yy(1:ny-1)-yy(0:ny-2)
      ave = total (gdx)/nx
      ok = where(gdx gt ave*0.01, nok)
      if nok eq 0 then begin
        Print, ' No valid X-points found'
        return
      endif
      gx = [xx(0), xx(ok+1)]
      x = gx
      nx = n_elements(xx)
      gdx = xx(1:nx-1)-xx(0:nx-2)
      dx = gdx
      ave = total (gdy)/ny
      ok = where(gdy gt ave*0.01, nok)
      if nok eq 0 then begin
        Print, ' No valid Y-points found'
        return
      endif
      gy = [yy(0), yy(ok+1)]
      y = gy
      ny = n_elements(yy)
      gdy = yy(1:ny-1)-yy(0:ny-2)
      dy = gdy
      mult = 1.0
      x1grid = ' Input for grid is from a PFF file.'
      x2grid = ' Input for grid is from a PFF file.'
      return
      end
