; ------------------------------------------------------------------------ 
; $Id: con2indqs.pro,v 1.3 2007/06/06 20:12:40 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro con2indqs, x, y, cx, cy, ix, iy, ni, close = close
;
; x and y are the grids
; cx and cy are the conductors
; ix and iy are the nearest grid locations
; ni = the last index of points in ix, iy
;
; scheme is grid based.
; the closest grid point to the line segment is selected
; points in a buffer cell around the entire grid are taken to the nearest
;      grid point
on_error, 1
    nc = N_elements(cx)
    ni = -1L
    if nc ne n_elements(cy) then begin
      print,'X and Y conductor arrays must be equal'
      ni = 0
      return
    endif
    if keyword_set (close) and nc gt 2 then begin
      if cx(0) ne cx(nc-1) or cy(0) ne cy(nc-1) then begin
        cx = [cx, cx(0)]
        cy = [cy, cy(0)]
        nc = nc+1
      endif
    endif
;   define maximum number of x and y grid elements
    ixmax = n_elements (x) -1
    iymax = n_elements (y) -1
;   define min and max values which include a buffer cell
    minx = x(0)-x(1)+x(0)
    miny = y(0)-y(1)+y(0)
    maxx = x(ixmax)-x(ixmax-1)+x(ixmax)
    maxy = y(iymax)-y(iymax-1)+y(iymax)
;   define real min and max values
    rminx = x(0)
    rmaxx = x(ixmax)
    rminy = y(0)
    rmaxy = y(iymax)
;   set up ix and iy arrays
    nimax = 128L
    ix = make_array(/int, nimax)
    iy = make_array(/int, nimax)
    smode = make_array(/int, nimax)
;   index on ok
    iok = 0
;   ist is the first index on the end point for the search 
;   mode is the type of step
    mode = 0
; use mode = 3, -3 for diagonal
;     mode = 1, -1 for delta x
;     mode = 2, -2 for delta y
;
; determine which conductor points are within the region of interest
    ok = where (cx ge minx and cx le maxx and cy ge miny and cy le maxy, nok)
    if nok eq 0 then begin
      print,'Warning: One conductor has no points within the grid area.'
      return
    endif
;   actually starting
;   if the start point is in the buffer cell, take the nearest grid point
;   if the first point is in the grid
;         take the nearest grid point if it is the first point of the conductor
;         extrapolate to the edge of the grid otherwise.
;
entergrid:
    ist = ok(iok) 
    if ok(iok) gt 0 then begin
;     this is not the first point of the conductor since not zero
;     see if the grid is being entered- If the point before is outside the
;     grid and the current point is inside the grid then interpolate to the
;     grid boundary.
      i = ist -1
      iix = -1
      if cx(i) lt rminx and cx(ist) gt rminx then begin
        yent = (rminx - cx(i))/(cx(ist)-cx(i))*(cy(ist)-cy(i))+cy(i)
        if yent ge rminy and yent le rmaxy then begin
          iix = 0
          a = min(abs(y-yent), iiy)
        endif
      endif else if cx(i) gt rmaxx and cx(ist) lt rmaxx then begin
        yent = (rmaxx - cx(i))/(cx(ist)-cx(i))*(cy(ist)-cy(i))+cy(i)
        if yent ge rminy and yent le rmaxy then begin
          iix = ixmax
          a = min(abs(y-yent), iiy)
        endif
      endif
      if iix lt 0 then begin
        if cy(i) lt rminy and cy(ist) gt rminy then begin
          xent = (rminy - cy(i))/(cy(ist)-cy(i))*(cx(ist)-cx(i))+cx(i)
          a = min(abs(x-xent), iix)
          iiy = 0
        endif else if cy(i) gt rmaxy and cy(ist) lt rmaxy then begin
          xent = (rmaxy - cy(i))/(cy(ist)-cy(i))*(cx(ist)-cx(i))+cx(i)
          iiy = iymax
          a = min(abs(x-xent), iix)
        endif
      endif
      if iix lt 0 then begin
        a = min(abs(x-cx(ist)), iix)
        a = min(abs(y-cy(ist)), iiy)
        ist = ist+1
      endif
    endif else begin
;   ifirst conductor point, get the nearest grid point as the first point
      a = min(abs(x-cx(ist)), iix)
      a = min(abs(y-cy(ist)), iiy)
      ist = ist +1
    endelse
    if ni ge 0 then begin
      if iix eq ix(ni) then begin
        if iiy lt iy(ni) then begin
          nmode =-2
        endif else if iiy gt iy(ni) then begin
          nmode = 2
        endif else if iiy eq iy(ni) then begin
          nmode = 4
        endif 
      endif else if iiy eq iy(ni) then begin
        if iix lt ix(ni) then begin
          nmode =-1
        endif else if iix gt ix(ni) then begin
          nmode = 1
        endif
      endif else nmode = -4
    endif else nmode = -4
;   If mode = 4 then ignore the point
;   If mode = -4 then add the point unconditionally and set mode to  0

    if (nmode ne mode and nmode lt 4) then begin
      if abs(mode) eq abs(nmode) then begin
        if abs(mode) eq 1 then begin
          if iix eq ix(ni-1) then begin
;         if this happens then drop the last point.
            mode = smode(ni-1)
            ni = ni-1
          endif else begin
            ix(ni)=iix
            if iix gt ix(ni-1) then mode = 1 else mode = -1
            smode(ni) = mode
          endelse
        endif else begin
          if iiy eq iy(ni-1) then begin
;         if this happens then drop the last point.    
            mode = smode(ni-1)
            ni = ni-1
          endif else begin
            iy(ni)=iiy
            if iiy gt iy(ni-1) then mode = 2 else mode = -2
            smode(ni) = mode
          endelse
        endelse
      endif else begin
        ni = ni+1
        if ni eq nimax then begin
          nimax = nimax+nimax
          ix = [ix, ix]
          iy = [iy, iy]
          smode = [smode, smode]
        endif
        ix(ni) = iix
        iy(ni) = iiy
        if nmode eq -4 then nmode = 0
        smode(ni) = nmode
        mode = nmode
      endelse
    endif else if nmode eq mode then begin
      ix(ni) = iix
      iy(ni) = iiy
    endif
    if ist ge nc then goto, finish
;
;   loop over the rest of the conductors
; **********************************************************************
    for i = ist, nc-1 do begin
      nexti = 0
      a = min(abs(x-cx(i)), iix)
      a = min(abs(y-cy(i)), iiy)
      lcreenter = 0
;     if we are at the same point go to the next segment
      if (iix ne ix(ni)) or (iiy ne iy(ni)) then begin
        if iix eq ix(ni)  then begin
          if iiy lt iy(ni) then nmode = -2 else nmode = 2
          nexti = 1
        endif else if iiy eq iy(ni) then begin
          if iix lt ix(ni) then nmode = -1 else nmode = 1
          nexti = 1
        endif else  begin
;         look for the grid point which has the shortest perpendicular distance
;         to the line between conductor points.
          la = [cx(i-1), cy(i-1)]
          lb = [cx(i), cy(i)]
again:
          iix = -1
          if cx(i) ge x(ix(ni)) then dx = 1 else dx = -1
          if cy(i) ge y(iy(ni)) then dy = 1 else dy = -1
          if (ix(ni) eq ixmax and dx eq 1)  then begin
            if cx(i) le maxx then begin
              iix = ixmax
              a = min(abs(y-cy(i)), iiy)
              if iiy eq iy(ni) then begin
                iix = -1 
              endif else if iiy lt iy(ni) then begin
                nmode = -2
              endif else nmode = 2
            endif else lcreenter = 1
          endif else if (ix(ni) eq 0 and dx eq -1) then begin
            if cx(i) ge minx then begin
              iix = 0
              a = min(abs(y-cy(i)), iiy)
              if iiy eq iy(ni) then begin
                iix = -1 
              endif else if iiy lt iy(ni) then begin
                nmode = -2
              endif else nmode = 2
            endif else lcreenter = 1
          endif else if (iy(ni) eq iymax and dy eq 1)  then begin
            if cy(i) le maxy then begin
              iiy = iymax
              a = min(abs(x-cx(i)), iix)
              if iix eq ix(ni) then begin
                iix = -1 
              endif else if iix lt ix(ni) then begin
                nmode = -1
              endif else nmode = 1
            endif else lcreenter = 1
          endif else if (iy(ni) eq 0 and dy eq -1) then begin
            if cy(i) ge miny then begin
              iiy = 0
              a = min(abs(x-cx(i)), iix)
              if iix eq ix(ni) then begin
                iix = -1 
              endif else if iix lt ix(ni) then begin
                nmode = -1
              endif else nmode = 1
            endif else lcreenter = 1
          endif else begin
;           get initial and final points
            pi = [ x(ix(ni)),y(iy(ni))]
            pf = [ x(ix(ni)+dx),y(iy(ni)+dy)]
            adist = abs((lb -pi)/(pf - pi))
            if adist(0) ge 0.5 or adist(1) ge 0.5 then begin
;     a = pnt_line([x(ix(ni)+dx), y(iy(ni)+dy)], la, lb)
              b = pnt_line([x(ix(ni)+dx), y(iy(ni))], la, lb)
              c = pnt_line([x(ix(ni)), y(iy(ni)+dy)], la, lb)
              r = min([b,c], iabc)
              iabc = iabc+1
              if iabc eq 1 and adist(0) ge 0.5 then begin
                if dx lt 0 then nmode = -1 else nmode = 1
                iix = ix(ni)+dx
                iiy = iy(ni)
              endif else if iabc eq 2 and adist(1) ge 0.5 then begin
                if dy lt 0 then nmode = -2 else nmode = 2
                iix = ix(ni)
                iiy = iy(ni)+dy
              endif
            endif
          endelse
          if lcreenter eq 1 then begin
            j= where(ok gt i, nok)
            if nok gt 0 then begin
              iok = j(0)
              goto, entergrid
            endif else goto, finish
          endif
        endelse
        if iix ge 0 then begin
          if nmode eq mode then begin
            ix(ni) = iix
            iy(ni) = iiy
          endif else if abs(mode) eq abs(nmode) then begin
            if abs(mode) eq 1 then begin
              if iix eq ix(ni-1) then begin
;               if this happens then drop the last point.
                mode = smode(ni-1)
                ni = ni-1
              endif else begin
                ix(ni)=iix
                if iix gt ix(ni-1) then mode = 1 else mode = -1
                smode(ni) = mode
              endelse
            endif else begin
              if iiy eq iy(ni-1) then begin
;             if this happens then drop the last point.
                mode = smode(ni-1)
                ni = ni-1
              endif else begin
                iy(ni)=iiy
                if iiy gt iy(ni-1) then mode = 2 else mode = -2
                smode(ni) = mode
              endelse
            endelse
          endif else begin
            ni = ni+1
            if ni eq nimax then begin
              nimax = nimax+nimax
              ix = [ix, ix]
              iy = [iy, iy]
              smode = [smode, smode]
            endif
            ix(ni) = iix
            iy(ni) = iiy
            smode(ni) = nmode
            mode = nmode
          endelse
        endif
        if iix ge 0 and nexti eq 0 and ( $
           (mode eq 1 and x(ix(ni)) le cx(i)) or $
           (mode eq -1 and x(ix(ni)) ge cx(i)) or $
           (mode eq 2 and y(iy(ni)) le cy(i)) or $
           (mode eq -2 and y(iy(ni)) ge cy(i))) then goto, again
;       if we are at the same point go to the next segment
;       if (iix ne ix(ni)) or (iiy ne iy(ni)) then goto, again
      endif
    endfor
finish:
    if (ni eq 0) then begin
      ni = ni+1
      ix(1) = ix(0)
      iy(1) = iy(0)
    endif
    if (ix(0) eq ix(ni)) and (iy(0) eq iy(ni)) and $
      (smode(1) eq -smode(ni)) and (ni gt 1) then begin
      if abs(smode(1)) eq 2 then begin
        dy1 = (iy(1)-iy(0))
        dy2 = (iy(ni)-iy(ni-1))
        if abs(dy1) gt abs(dy2) then begin
          iy(0) = iy(ni-1)
          ni = ni-1
        endif else if abs(dy1) lt abs(dy2) then begin
          iy(ni) = iy(1)
          iy = iy(1:*)
          ix = ix(1:*)
          ni = ni-1
        endif else begin
          ni = ni-2
          iy= iy(1:*)
          ix= ix(1:*)
        endelse
      endif else begin
        dx1 = (ix(1)-ix(0))
        dx2 = (ix(ni)-ix(ni-1))
        if abs(dx1) gt abs(dx2) then begin
          ix(0) = ix(ni-1)
          ni = ni-1
        endif else if abs(dx1) lt abs(dx2) then begin
          ix(ni) = ix(1)
          ix = ix(1:*)
          iy = iy(1:*)
          ni = ni-1
        endif else begin
          ni = ni-2
          ix= ix(1:*)
          iy= iy(1:*)
        endelse 
      endelse
    endif
    if keyword_set (close) and ni gt 1 then begin
      if ix(0) ne ix(ni) or iy(0) ne iy(ni) then begin
        ni = ni+1
       print, ' Adding a point to close conductor', ix(0), iy(0)
        ix(ni) = ix(0)
        iy(ni) = iy(0)
      endif
    endif
    ix = ix(0:ni)
    iy = iy(0:ni)
    ni = ni+1
    smode = 0
    return
    end
