; ------------------------------------------------------------------------ 
; $Id: get_tq_conduct.pro,v 1.4 2010/09/29 20:59:14 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_tq_conduct , xp, yp,xout, yout, lcmode, lcsegment, $
       lcregion, region_size, condstr
; get closest [i, j] conductor value grid point to xp, yp
; return, n corresponding to ix(n), iy(n) of closest point
;
@pfidltq.com
 on_error, 1
 if n_elements (xp) eq 0 or n_elements(yp) eq 0 then message, $
      'Input parameters must be defined.'
 xrange = (!x.crange(1)-!x.crange(0))
 yrange = (!y.crange(1)-!y.crange(0))
 ratio = (xrange/yrange)^2
 if lcmode eq 'INPUT' then begin
   if lcsegment ge 0 then $
     dist =(gx(cond.ix(cond.loci(lcsegment-1):cond.loci(lcsegment)-1))-xp)^2+ $
        ratio*(gy(cond.iy(cond.loci(lcsegment-1):cond.loci(lcsegment)-1))-yp)^2 $
     else dist = (gx(cond.ix)-xp)^2 + ratio*(gy(cond.iy)-yp)^2
   junk = min(dist, j)
   if lcsegment lt 1 then begin
     xout = gx(cond.ix(j))
     yout = gy(cond.iy(j))
     ok=where(j ge cond.loci(0:cond.ncon-1) and j lt cond.loci(1:cond.ncon),np)
     if np ne 1 then message, strtrim(np, 2)+' Conductor segments found'
     lcsegment  = ok(0) +1
     j = j - cond.loci(lcsegment-1)
   endif else begin
     xout = gx(cond.ix(cond.loci(lcsegment-1)+j))
     yout = gy(cond.iy(cond.loci(lcsegment-1)+j))
   endelse
 endif else begin
   if lcsegment ge 0 then $
     dist = (cond.x(cond.locx(lcsegment-1):cond.locx(lcsegment)-1) -xp)^2 + $
      ratio*(cond.y(cond.locx(lcsegment-1):cond.locx(lcsegment)-1)-yp)^2 else $
           dist = (cond.x-xp)^2 + ratio*(cond.y-yp)^2
   junk = min(dist, j)
   if lcregion le 0 then begin
     if lcregion lt 0 then begin
       lcregion = 1
       cond = temporary(condstr)
     endif
     if lcsegment lt 1 then begin
       xout = cond.x(j)  
       yout = cond.y(j)
       ok=where(j ge cond.locx(0:cond.ncon-1) and j lt cond.locx(1:cond.ncon),np)
       if np ne 1 then message,strtrim(np,2)+' Conductor segments found ' + $
             string(j)
       lcsegment  = ok(0)+1
       j = j - cond.locx(lcsegment-1)
     endif else begin
       xout = cond.x(cond.locx(lcsegment-1)+j)
       yout = cond.y(cond.locx(lcsegment-1)+j)
     endelse
   endif else begin
     delx = 1.5*region_size*xrange
     dely = 1.5*region_size*yrange
     regionsize = junk + (delx^2 + dely^2)/2.25
     
     ok = where (dist le regionsize, nok)
     if nok eq 1 then begin
       if lcsegment lt 1 then begin
         xout = cond.x(j)
         yout = cond.y(j)
         ok=where(j ge cond.locx(0:cond.ncon-1) and $
                j lt cond.locx(1:cond.ncon),np)
         if np ne 1 then message,strtrim(np,2)+' Conductor segments found ' + $
             string(j)
         lcsegment  = ok(0)+1
         j = j - cond.locx(lcsegment-1)
       endif else begin
         xout = cond.x(cond.locx(lcsegment-1)+j)
         yout = cond.y(cond.locx(lcsegment-1)+j)
       endelse
     endif else begin
       if lcsegment lt -1 then begin
         oks = where(ok(0) ge cond.locx(0:cond.ncon-1) and $
                ok(0) lt cond.locx(1:cond.ncon),np)
         if np ne 1 then message,strtrim(np,2)+' Conductor segments found ' + $
             string(j)
         for i = 1, nok-1 do begin
           okt = where(ok(i) ge cond.locx(0:cond.ncon-1) and $
                ok(i) lt cond.locx(1:cond.ncon),np)
           if np ne 1 then message,strtrim(np,2)+' Conductor segments found ' + $
               string(j)
           if okt(0) ne oks(0) then  goto, spread_them
         endfor
         lcsegment = okt(0)+1
         xout = cond.x(j)
         yout = cond.y(j)
         j = j - cond.locx(lcsegment-1)
         return, j
       endif
spread_them:
       condstr = cond
       lcregion = -1
       if lcsegment lt 1 then begin
         for i = 0, nok-1 do cond.x(ok(i))= cond.x(ok(i))+i*delx
         for i = 0, nok-1 do cond.y(ok(i))= cond.y(ok(i))+i*dely
         if cond.x(ok(nok-1)) gt !x.crange(1) or $
           cond.y(ok(nok-1)) gt !y.crange(1) then plot, gx, gy, /nodata , $
          xrange= [!x.crange(0), max([!x.crange(1), cond.x(ok(nok-1))])], $
          yrange= [!y.crange(0), max([!y.crange(1), cond.y(ok(nok-1))])], $
          /xstyle, /ystyle
         xout = cond.x(ok(0))
         yout = cond.y(ok(0))
         j = ok(0)
       endif else begin
         for i = 0, nok-1 do cond.x(cond.locx(lcsegment-1)+ok(i))= $
                  cond.x(cond.locx(lcsegment-1)+ok(i))+i*delx
         for i = 0, nok-1 do cond.y(cond.locx(lcsegment-1)+ok(i))= $
                  cond.y(cond.locx(lcsegment-1)+ok(i))+i*dely
         if cond.x(cond.locx(lcsegment-1)+ok(nok-1)) gt !x.crange(1) or $
           cond.y(cond.locx(lcsegment-1)+ok(nok-1)) gt !y.crange(1) then $
           plot, gx, gy, /nodata , $
          xrange= [!x.crange(0), max([!x.crange(1), $
               cond.x(cond.locx(lcsegment-1)+ok(nok-1))])], $
          yrange= [!y.crange(0), max([!y.crange(1), $
               cond.y(cond.locx(lcsegment-1)+ok(nok-1))])], $
          /xstyle, /ystyle
         j = ok(0)
         xout = cond.x(cond.locx(lcsegment-1)+j)
         yout = cond.y(cond.locx(lcsegment-1)+j)
       endelse
     endelse   
   endelse
 endelse
 return,j
 end
