; ------------------------------------------------------------------------ 
; $Id: yfit_tqgrid.pro,v 1.3 2007/06/06 20:12:43 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  yfit_tqgrid, p1, p2
;
;+
;   Purpose: Fit the Y-grid to a line between two points P1, and P2
;   format:
;       yfit_tqgrid, p1, p2
;       where P1 and P2 are 2-element vectors
;   Example: 
;       fit the grid between the  points, [0, 2], [2, -3]
;       yfit_tqgrid, [0, 2], [2, -3]
;
;-
;
@pfidltq.com
;
;     common pfidltq, x, dx, y, dy, gx, gy, gdx, gdy, mult, x1grid, x2grid, $
;        cond, markx, marky, markerstring, lcfit
;
      on_error,2
      if p2(0) eq p1(0) then begin
        print, ' Everything fits a verticle line'
        return
      endif
      if p2(1) lt p1(1) then begin
        temp = p1
        p1 = p2
        p2 = temp
      endif
;
      slope = (p2(1)-p1(1))/(p2(0)-p1(0))
      if slope eq 0 then begin
        print , ' Current gridding is adequate'
        return
      endif
      if slope lt 0 then begin
        aslope = -slope
      endif else aslope = slope
;     clip the line to the grid
      nx = n_elements (gx)
      ny = n_elements (gy)
      xmin =  x(0)
      xmax = max(x)
      ymin = y(0)
      ymax = max(y)
;     clip to the grid
      if p1(0) lt xmin then begin
        p1(1) = p1(1) + (xmin-p1(0))*slope
        p1(0) = xmin
      endif
      if p1(0) gt xmax then begin
        p1(1) = p1(1) + (xmax-p1(0))*slope
        p1(0) = xmax
      endif
      if p1(1) lt ymin then begin
        p1(0) = p1(0) +(ymin-p1(1))/slope
        p1(1) = ymin
      endif
      if p2(0) gt xmax then begin
        p2(1) = p2(1) + (xmax-p2(0))*slope
        p2(0) = xmax
      endif
      if p2(1) gt ymax then begin
        p2(0) = p2(0) +(ymax-p2(1))/slope
        p2(1) = ymax
      endif
      if p2(0) lt xmin then begin
        p2(1) = p2(1) +(xmin-p2(0))*slope
        p2(0) = xmin
      endif
      x1 = p1(0)
      x2 = p2(0)
      if p2(1) lt p1(1) then begin
        y1 = p2(1)
        y2 = p1(1)
      endif else begin
        y1 = p1(1)
        y2 = p2(1)
      endelse
;
      ist = get_tqgrid (x1, y1)
      iend = get_tqgrid (x2, y2)
;
      if ist(0) lt 2 then xd = xmin else xd = gx(ist(0)-1)
      if iend(0) +2 ge nx then xu = xmax else xu = gx(iend(0)+1)
;     find the x and y values which are within 4 grid lines of the region
      okx =where (x ge xd-gdx(ist(0)) and x le xu+gdx(iend(0)<nx-2), nokx)
;     fix the y values
      if nokx gt 0 then begin
        ynew = make_array(nokx)
        dynew = ynew
        for i = 0, nokx-1 do begin
          j = okx(i)
          xx =  x(j)
          ynew(i) = p1(1) + (xx - p1(0))*slope
          del = min (abs(xx-gx), n)
          dynew(i) = -gdx(n < ny-2)*aslope
        endfor
        ok = sort(ynew)
        ynew = ynew(ok)
        dynew = dynew(ok)
      endif
;
      if ist(1) le 3 then yd = ymin else yd = gy(ist(1)-3)
      if iend(1) +4 ge ny then yu = ymax else yu = gy(iend(1)+3)
;
      oky =where (y ge yd-gdy(ist(1)) and  y le yu+gdy(iend(1)<ny-2), noky)
;     
      if noky gt 0 then begin 
        if oky(0) eq 0 then begin
          xx = p1(0) + (y(0)-p1(1))/slope
          del =  min (abs(xx-gx), n)
          dy(0)= -gdx(n< nx-2)*aslope
          noky = noky -1
          if noky gt 0 then oky = oky(1:*)
        endif
        nya = n_elements(y)-1
        if noky gt 0 then begin
          if oky(noky-1) eq nya then begin
            xx = p2(0) + (y(nya)-p2(1))/slope
            del =  min (abs(xx-gx), n)
            dy(nya)= -gdx(n< nx-2)*aslope
            noky = noky -1
            if noky gt 0 then oky = oky(0:noky-1)
          endif
        endif
        print, y
        print, noky, oky
        if noky gt 0 then y(oky) = y(0)-10.*gdy(0)
      endif
      if n_elements (ynew) gt 0 then begin
        y = [y, ynew]
        dy = [dy, dynew]
      endif
;
      del = min(abs(y-y1), n)
      if del gt 2.0*(dy(n)) then begin
        y = [y, y1]
        del = min(abs(gx-p1(0)), j)
        dy  = [dy, -gdx(j< nx-2)*aslope]
      endif 
      del = min(abs(y-y2), n)
      if del gt 2.0*abs(dy(n)) then begin
        y = [y, y2]
        del = min(abs(gx-p2(0)), j)
        dy  = [dy, -gdx(j< nx-2)*aslope]
      endif 
      print,y, dy
;
      tqgrid, /yaxis
      return
      end
