; ------------------------------------------------------------------------ 
; $Id: becky.pro,v 1.4 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro becky , ima,imb, x, y, im1, x1, y1, im2, x2, y2, master, cubbic, ier, $
             default = default
;
;
; no error checking 
; assume im1 is 2 d array of dimension n_elements(x1), n_elements (y1)
; etc
; If master gt 0, then regions where there is no overlap are set to default.
;   default value of default is 0.
; x, y are sorted but not uniform
; 
 COMPILE_OPT hidden
 on_error, 1
 if n_elements(cubbic) eq 0 then  cubic = 0 else cubic = cubbic
 ier = 0
 if n_elements(master) eq 0 then master = 0
 nx1 = n_elements(x1)
 nx2 = n_elements(x2)
 ny1 = n_elements(y1)
 ny2 = n_elements(y2)
 if master eq 0 then begin
   dx1 = x1(1:nx1-1)-x1(0:nx1-2)
   dx2 = x2(1:nx2-1)-x2(0:nx2-2)
   dy1 = y1(1:ny1-1)-y1(0:ny1-2)
   dy2 = y2(1:ny2-1)-y2(0:ny2-2)
   dxmin =min([dx1, dx2]) 
   dymin =min([dy1, dy2]) 
 endif
 if master le 0 then begin
   x0 = max([x1(0), x2(0)])
   y0 = max([y1(0), y2(0)])
   xf = min([x1(nx1-1), x2(nx2-1)])
   yf = min([y1(ny1-1), y2(ny2-1)])
   if master eq 0 then begin
     nx = long((xf-x0)/dxmin+1.00)
     ny = long((yf-y0)/dymin+1.00)
     if nx lt 2 or ny lt 2 then goto, no_overlap
     x = findgen(nx)*dxmin+x0
     y = findgen(ny)*dymin+y0
;     p1 = get_yarr(x1, findgen(nx1), x)
;     q1 = get_yarr(y1, findgen(ny1), y)
;     ima = interpolate(im1, p1, q1, cubic = cubic, /grid) 
     ima = interpolate(im1, findex(x1, x), findex(y1, y), cubic = cubic, /grid) 
;     p2 = get_yarr(x2, findgen(nx2), x)
;     q2 = get_yarr(y2, findgen(ny2), y)
;     imb = interpolate(im2, p2, q2, cubic = cubic, /grid) 
     imb = interpolate(im2, findex(x2, x), findex(y2, y), cubic = cubic, /grid) 
   endif else begin
     xerr = max(abs([x0, xf]))*3.05e-5
     yerr = max(abs([y0, yf]))*3.05e-5
     if master eq -1 then begin
       okx = where (x1 gt x0-xerr and x1 lt xf+xerr, nokx)
       if nokx lt 2 then goto, no_overlap
       oky = where (y1 gt y0-yerr and y1 lt yf+yerr, noky)
       if noky lt 2 then goto, no_overlap
       x = x1(okx)
       y = y1(oky)
;       p2 = get_yarr(x2, findgen(nx2), x)
;       q2 = get_yarr(y2, findgen(ny2), y)
       ima = im1(okx(0):okx(nokx-1), oky(0):oky(noky-1))
;       imb = interpolate(im2, p2, q2, cubic = cubic, /grid) 
       imb = interpolate(im2, findex(x2, x), findex(y2, y), cubic=cubic, /grid) 
     endif else begin
       okx = where (x2 gt x0-xerr and x2 lt xf+xerr, nokx)
       if nokx lt 2 then goto, no_overlap
       oky = where (y2 gt y0-yerr and y2 lt yf+yerr, noky)
       if noky lt 2 then goto, no_overlap
       x = x2(okx)
       y = y2(oky)
;       p1 = get_yarr(x1, findgen(nx1), x)
;       q1 = get_yarr(y1, findgen(ny1), y)
;       ima = interpolate(im1, p1, q1, cubic = cubic, /grid) 
       ima = interpolate(im1, findex(x1, x), findex(y1, y), cubic=cubic, /grid) 
       imb = im2(okx(0):okx(nokx-1), oky(0):oky(noky-1))
     endelse
   endelse
   return
 endif
 if master eq 1 then begin
   imb = 0.0 * im1
   if n_elements (default) gt 0 then imb = imb+default(0)
   xerr = max(abs([x1(0), x1(nx1-1)]))*3.05e-5
   yerr = max(abs([y1(0), y1(ny1-1)]))*3.05e-5
   okx = where (x1 gt x2(0)-xerr and x1 lt x2(nx2-1)+xerr, nokx)
   if nokx lt 2 then goto, no_overlap
   oky = where (y1 gt y2(0)-yerr and y1 lt y2(ny2-1)+yerr, noky)
   if noky lt 2 then goto, no_overlap
   x = x1(okx)
   y = y1(oky)
;   p2 = get_yarr(x2, findgen(nx2), x)
;   q2 = get_yarr(y2, findgen(ny2), y)
;   imb(okx(0):okx(nokx-1), oky(0):oky(noky-1)) = $
;        interpolate(im2, p2, q2, cubic = cubic, /grid)
   imb(okx(0):okx(nokx-1), oky(0):oky(noky-1)) = $
        interpolate(im2, findex(x2, x), findex(y2, y), cubic = cubic, /grid)
   ima = im1
   x = x1
   y = y1
   return
 endif
 ima = 0.0*im2
 if n_elements(default) gt 0 then ima = ima+default(0)
 xerr = max(abs([x2(0), x2(nx2-1)]))*3.05e-5
 yerr = max(abs([y2(0), y2(ny2-1)]))*3.05e-5
 okx = where (x2 gt x1(0)-xerr and x2 lt x1(nx1-1)+xerr, nokx)
 if nokx lt 2 then goto, no_overlap
 oky = where (y2 gt y1(0)-yerr and y2 lt y1(ny1-1)+yerr, noky)
 if noky lt 2 then goto, no_overlap
 x = x2(okx)
 y = y2(oky)
; p1 = get_yarr(x1, findgen(nx1), x)
; q1 = get_yarr(y1, findgen(ny1), y)
; ima(okx(0):okx(nokx-1), oky(0):oky(noky-1)) = $
;     interpolate(im1, p1, q1, cubic = cubic, /grid) 
 ima(okx(0):okx(nokx-1), oky(0):oky(noky-1)) = $
     interpolate(im1, findex(x1, x), findex(y1, y), cubic = cubic, /grid) 
 imb = im2
 return
;
 no_overlap:
      print, 'Cannot perform operation - Overlap of less than 2 points'
      ier = -1
      return
      end
