;
; ------------------------------------------------------------------------ 
; $Id: calc_temp.pro,v 1.3 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function  calc_temp, e, rat, lo, hi, np = np, limits = limits
;
; calculate temperture given total power, e, in a range of energies
;   from lo to hi , where e is in Watts/m2
;
; If rat is undefined or le 0 then rat starts at 1
; If lo is missing then the energy goes from 0 to infinity.
; If hi is missing then the energy goes from LO to 25* t0 where 
;    t0 = (e/1.028300544e+09/rat)^.25
; Rat is the ration of e to the total blackbody emission.
; NP is the number of points between LO and HI used to calculate the 
;   spectrum.  This is ignored if LO is undefined.
;   Default = 1024
; LIMITS is the factor of temperature which limits the values of lo and hi
;    If lo gt limits*t0 then return, 0.0
;    If hi lt t0/limits then return, 0.0
;    Default: LIMITS = [0.5, 10.0]
;
 if n_elements(rat) eq 0 then rat = 1.
 if rat le 0.0 then rat = 1
 err = 1
 tt = (e/1.028300544e+09/rat)^.25
 if n_elements(lo) eq 0 then return, tt
 if n_elements(hi) eq 0 then hi = 25*tt
 ii = 0
 if n_elements(np) eq 0 then np = 1024
 wid= (hi-lo)/float(np)
 if n_elements (limits) lt 2 then limits = [.5, 10.0]
 if hi lt limits[0]*tt then return, 0.0
 if lo gt limits[1]*tt then return, 0.0
 ener = lo+wid/2 +wid*findgen(np)
 ww = get_maxwdf()
 while ii lt 25 and err gt 1e-5 do begin
   blackbody, ww, tt, ener, tot = tot, power= power, /no_save
   rat = total(power, /double)*wid/tot
   t = tt
   tt = (e/1.028300544e+09/rat)^.25
   err = abs(tt-t)/tt
   ii = ii+1
 endwhile
 if err gt 1e-5 then return, 0.0 else return, tt
 end
