; ------------------------------------------------------------------------ 
; $Id: change_alpha.pro,v 1.3 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro change_alpha, obj, alpha
;
; OBJ - IDLgrPolygon object
;/remote/lpmix/consult/mfpasik
;         Value from 0 to 255
;
 if n_params() eq 0 then begin
   print, '% Format:  CHANGE_ALPHA, Poly_Obj, Alpha
   print, '            (Alpha range = [0, 255])
   return
 endif
 if n_elements(obj) eq 0 or n_elements(alpha) eq 0 then begin
   Print, '% CHANGE_ALPHA: Object and Alpha must be defined'
   return
 endif
 if size (obj, /typ) ne 11 then begin
   Print, '% CHANGE_ALPHA: Object must be the first parameter.
   return 
 endif
 if obj_class(obj) ne 'IDLGRPOLYGON' then begin
   Print, '% CHANGE_ALPHA: IDLGRPOLYGON must be provided.'
   return 
 endif

 if alpha lt 0 or alpha gt 255 then begin
   Print, '% CHANGE_ALPHA: Alpha must be in the range [0, 255]
   return 
 endif
 
 obj -> getproperty, texture_map=oimage, color=color, vert_color=vertcolor
 obj -> getproperty, data = Vertices, palette = opalette
 if obj_class(opalette) eq 'IDLGRPALETTE' then lcpal = 1 else lcpal = 0
 nvert = n_elements(vertices[0, *])

 if obj_class(oimage) ne 'IDLGRIMAGE'  then begin
   rgbTable = MAKE_ARRAY(4,1,NVert, /BYTE, VALUE = 0)
   oimage = obj_new('IDLgrImage')
   if n_elements (vertcolor) lt nvert then begin
     if n_elements(color) eq 1 then begin
       if lcpal eq 1 then color = opalette -> getRGB(color) else begin
          print, '% CHANGE_ALPHA:  Color set to grey
         color = [160, 160, 160]
         obj -> setproperty, color=color
       endelse
     endif
     rgbtable [0, *,*] = color[0]
     rgbtable [1, *,*] = color[1]
     rgbtable [2, *,*] = color[2]


   endif else begin
     if lcpal eq 0 then begin
         Print, '% CHANGE_ALPHA: Must have a palette'
         return
     endif
       opalette -> getproperty, red_val= r, green_val=g, blue_val = b
       rr = make_array(1, nvert, /byte)
       gg = make_array(1, nvert, /byte)
       bb = make_array(1, nvert, /byte)
       rr[*,*]  = r[vertcolor]
       gg[*,*]  = g[vertcolor]
       bb[*,*]  = b[vertcolor]
       rgbtable [0, *,*] = rr
       rgbtable [1, *,*] = gg
       rgbtable [2, *,*] = bb
   endelse
   tcoord = make_array(2, nvert)
   xr = [min(vertices[0, *] , max = m), m]
   yr = [min(vertices[1, *] , max = m), m]
   tcoord [0, *] = (vertices[0, *]-xr[0])/(xr[1]-xr[0])
   tcoord [1, *] = (vertices[1, *]-yr[0])/(yr[1]-yr[0])
   rgbtable [3, *,*] = byte(alpha)
   oimage -> setproperty, data = rgbtable
   obj -> setproperty, texture_map = oimage, texture_coord = tcoord, $
        vert_color = 0, color = [255b,255b, 255b], /texture_interp
   return
   plot, reform (tcoord[0, *]), reform (vertices[0, *])
 endif

 oimage -> getproperty, data = rgbtable
 help, rgbtable
 rgbtable[3, *, *] = byte(alpha)
 oimage -> setproperty, data = rgbtable
 end
