; ------------------------------------------------------------------------ 
; $Id: check_equal.pro,v 1.3 2007/06/06 19:34:26 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function check_equal, array1, array2, error = errorin
; 
;+
;   Check to see if two arrays are equal
;   format:  equal = check_equal( ar1, ar2, error = error)
;      where: Equal  0 (zero) if they ar not equal
;                    1 (one ) if they are equal
;             ar1    the first array
;             ar2    the second array
;             error  tolerance for the comparison.
;                    Default = (max(ar1)-min(ar1))/2^16
;-
      on_error, 2
      a = size (array1)
      b = size (array2)
;
      if max(abs(a-b)) ne 0 then return, 0
      if n_elements(a) ne n_elements(b) then return, 0
;
      if n_elements(errorin) eq 0 then error=(max(array1, min=m)-m)/(2.0^16) $
        else error = abs(errorin[0])
;
      if max([max(array1-array2, min = m), -m]) gt error then return, 0
;
      return, 1
      end
