; ------------------------------------------------------------------------ 
; $Id: find_newfile.pro,v 1.7 2009/02/17 17:01:49 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function find_newfile, filein, initial = initial, lcquit, quiet = quiet
;+
;  Find a unique file name 
;  Format: NEW_FILE = FIND_NEWFILE( file [, lcquit] [, initial = initial])
;  If file is undefined or not a string then set the file to 
;    file = 'NEWfile.default'
;    Append numbers to get a unique name.
;    If file contains a . then add _# before the period
;    If file has a leading . the add _# to the end
;    If file has no . then add _#
;  If lcquit is not equal to 0 then the user will be asked if he wants
;    to make a new file
;  INITIAL is the first number to try and is returned with one more than 
;    the last value to try.
;-
 COMPILE_OPT hidden
   if n_elements (filein) eq 0 then file='NEWfile.default' else file=filein(0)
   a = size (file)
   if n_elements(initial) eq 0 then initial = 1L
   if a(a(0)+1) ne 7 then file = 'NEWfile.default'
   file = strtrim(file, 2)
   lfile = strlen(file)
   if lfile eq 0 then begin
     file = 'NEWfile.default'
     lfile = strlen(file)
   endif
   if n_elements (lcquit) eq 0 then lcquit=0
   if lcquit ne 0 then quiet = 0
   sep = strpos(file, path_sep(), /reverse_search)
   period = strpos(file, '.', /reverse_search)
   if period lt sep then period =-1
   if period eq lfile-1 then period = -2
   try = initial 
   junk = file_search(file, count = count)
   while (count gt 0) do begin
     if not keyword_set(quiet) then $
        print, '% Find_NewFile: ', strtrim(file, 2),' already exists'
     if lcquit ne 0 then begin
        ans= ' '
        read, ans, prompt='    Do you wany to make a new version? (y or n)[n]:'
        if ans eq 'n'  or ans eq '' or ans eq 'N' then return, file
        lcquit = 0
     endif
     if try gt 9999 then begin
       Print, '% FIND_NEWFILE: Unable to find a unique file in 10,000 trys'
       return, ''
     endif
     lval = (try lt 1000) ?   '3' : '4'
     val = strtrim(string(try, format='( I'+lval+'.'+lval+')') ,2)
     if try eq initial then begin
       if period ge -1 then begin
         file= (period gt 0) ? strmid(file,0,period)+'_'+val+ $
            strmid(file,period) :  file + '_' + val 
         if period ge 0 then lf = period+1 else lf = lfile+1
       endif else begin
         file = file+val
         lf = lfile
       endelse
     endif else file= (period gt 0) ? strmid(file,0,lf)+val+ $
          strmid(file,lf+lval)  : strmid(file,0,lf)+val
     try = try+1
     junk = file_search(file, count = count)
   endwhile
   initial = try
   return, file
   end
