; ------------------------------------------------------------------------ 
; $Id: get_array_union.pro,v 1.3 2007/06/06 19:34:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
; 
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 Pro get_array_union, ar1, ar2, artotal, ind1, ind2, range, ier
;
; Require that ar1 and ar2 are part of a larger grid and check to ~3e-5 
; Find the super grid for the union of ar1 ar2
; Find the indices of ar1 and ar2 which correspond to the super grid.
; If AR1 and AR2 are not part of a super grid, ier = 1, else, ier = 0
;  epsilon =  max([drange/32768., range*2.3841858e-07]) 
;  range = [min, max] and drange = max-min
;
on_error,1
range = [min([ar1, ar2], max = mm), mm]
drange = (range[1]-range[0]) 
eps = max([drange/32768., range*2.3841858e-07])
n1 = n_elements(ar1)
n2 = n_elements(ar2)
ier = 1

; See if the arrays are identical
if (n1 eq n2) then begin
  if (max(abs(ar1-ar2) le eps)) then begin
    ier = 0
    ind1 = [0, n1-1]
    ind2 = ind1
    artotal = ar1
    return
  endif
endif

r1 = [min(ar1, max = mm), mm]
r2 = [min(ar2, max = mm), mm]

; Four possibilities: Identical (already checked), same start, same end, 
;                     different start and end.

; See if same beginning

if abs(r1[0]-r2[0]) le eps then begin
  if n1 lt n2 then begin
    if (max(abs(ar1[0:n1-1]-ar2[0:n1-1])) le eps) then begin
      ier = 0
      artotal = [ar1, ar2[n1:*]]
      ind1 = [0, n1-1]
      ind2 = [0, n2-1]
    endif
  endif
  if n2 lt n1 then begin
    if (max(abs(ar1[0:n2-1]-ar2[0:n2-1])) le eps) then begin
      ier = 0
      artotal = ar1
      ind1 = [0, n1-1]
      ind2 = [0, n2-1]
    endif
  endif
  range[0] = ar1[0]
  return
endif

; See if same ending

if abs(r1[1]-r2[1]) le eps then begin
  if n1 lt n2 then begin
    if (max(abs(ar1[0:n1-1]-ar2[n2-n1:n2-1])) le eps) then begin
      ier = 0
      artotal = [ar2[0:n2-n1-1], ar1]
      ind1 = [n2-n1, n2-1]
      ind2 = [0, n2-1]
    endif
  endif
  if n2 lt n1 then begin
    if (max(abs(ar1[n1-n2:n1-1]-ar2[0:n2-1])) le eps) then begin
      ier = 0
      artotal = ar1
      ind1 = [0, n1-1]
      ind2 = [n1-n2, n1-1]
    endif
  endif
  range[1] = ar1[n1-1]
  return
endif
  
; neither same beginning or same ending

if r1[0] lt r2[0] then begin
  ok = where (ar1 lt r2[0]-eps, nok)
  ind1 = [0, n1-1]
  ind2 = [nok, n2+nok-1]
  lcfirst = 1
endif else begin
  ok = where (ar2 lt r1[0]-eps, nok)
  ind1 = [nok, n1-1+nok]
  ind2 = [0, n2-1]
  lcfirst = 2
endelse

if r1[1] gt r2[1] then begin
  okend = where (ar1 gt r2[1]+eps, nokend)
  if lcfirst eq 1 then begin
    if (max(abs(ar1[nok:okend[0]-1]-ar2) le eps) and (n2+nok+nokend eq n1)) $
        then begin
      artotal = ar1
      ier = 0
    endif
  endif else begin
    if (max(abs(ar1[0:okend[0]-1]-ar2[nok:*]) le eps) and $
        (n1+nok eq n2+nokend)) then begin
      artotal = [ar2[0:nok-1], ar1]
      ier = 0
    endif
  endelse
endif else begin
  okend = where (ar2 gt r1[1]+eps, nokend)
  if lcfirst eq 1 then begin
    if (max(abs(ar1[nok:*]-ar2[0:n2-nokend-1]) le eps) and $
        (n2+nok eq nokend+n1)) then begin
      artotal = [ar1, ar2[okend]]
      ier = 0
    endif
  endif else begin 
    if (max(abs(ar1-ar2[nok:okend[0]-1]) le eps) and $
        (nok+nokend+n1 eq n2)) then begin
      artotal = [ar2[0:nok-1], ar1, ar2[okend]]
      ier = 0
    endif
  endelse
endelse
end

