; ------------------------------------------------------------------------ 
; $Id: get_aspect.pro,v 1.6 2008/04/09 15:25:36 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Function Get_Aspect, xrange, yrange, xmargin = xmargin, ymargin= ymargin
;+
; Get aspect returns the positon for an aspect ratio 1 plot.
; note xstyle and ystyle must be set. ie
; xstyle = 1 and ystyle = 1 or 1+ystyle if ystyle = 4 or something 
;
; format: position = Get_Aspect(xrange, yrange $
;                 [, xmargin = xmargin] [, ymargin = ymargin]) 
; 
; where:
;     xrange   - 2 element vector with desired range of x data
;     yrange   - 2 element vector with desired range of y data
;     xmargin  - 2 element vector with x margins in character.
;                Default = [10, 3]
;     ymargin  - 2 element vector with y margins in character.
;                Default = [4, 2]
;    
; 
; Example:
;   plo, 4, position= get_aspect(xrange, yrange), /xst, /yst, $
;          xrange = xrange, yrange = yrange
;-

 COMPILE_OPT hidden

      on_error, 1
      if n_elements (xrange) lt 2 then begin
        print, '% Get_Aspect: Xrange must be provided'
        return, 0
      endif else xrange = float(xrange)
      if n_elements (yrange) lt 2 then begin
        print, '% Get_Aspect: Yrange must be provided'
        return, 0
      endif else yrange = float(yrange)
   
      p = !p.multi
      nx = !p.multi(1) > 1
      ny = !p.multi(2) > 1
      nt = nx*ny
      if nt gt 4 then chars = 0.5 else chars = 1.0
      if !d.window lt 0 then begin
        if !d.name ne 'PS' and !d.name ne 'PCL' and !d.name ne 'PRINTER' then $
         window
      endif
      xsize = !d.x_size
      xch = !d.x_ch_size*chars
      if !d.X_PX_CM eq  !d.Y_PX_CM then begin
        ysize = !d.y_size
        ych = !d.y_ch_size*chars
      endif else begin
        pix_rat = !d.X_PX_CM / !d.Y_PX_CM
        pix_rat = 1
        ysize = round(!d.y_size*pix_rat)
        ych = !d.y_ch_size*chars*pix_rat
      endelse
      npm = !p.multi(0)-1
      if npm lt 0 then npm = nt-1
      if p[4] eq 0 then begin
        py0 = (npm/nx)
        px0 = nx - 1 - npm mod nx
      endif else begin
        px0 = nx-1 - npm/ny
        py0 = npm mod ny
      endelse
      dpx = 1.0/nx
      dpy = 1.0/ny
      xregion = [dpx*px0, dpx*(px0+1)]
      yregion = [dpy*py0, dpy*(py0+1)]

      if n_elements(xmargin) lt 2 then xmargin = [10, 3] else $
         xmargin = xmargin(0:1)
      if n_elements(ymargin) lt 2 then ymargin = [4, 2] else $
         ymargin = ymargin(0:1)
      xwin = dpx*xsize-xch*total(xmargin)
      ywin = dpy*ysize-ych*total(ymargin)
      xst = xregion(0)*xsize +xmargin(0)*xch
      yst = yregion(0)*ysize +ymargin(0)*ych
      delran = abs((xrange(1)-xrange(0))/(yrange(1)-yrange(0)))
      if xwin/ywin gt delran then xwin = ywin*delran else ywin = xwin/delran
      return, [xst/xsize, yst/ysize, (xst+xwin)/xsize , (yst+ywin)/ysize]
      end
      
