; ------------------------------------------------------------------------ 
; $Id: get_blockconnect.pro,v 1.3 2007/06/06 19:34:27 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 function get_blockconnect, order, range, two, ndim, mxbl 

;
;+
;
;  Generate block connections for a multi-block dataset
;
;  format: 
;        connect = get_blockconnect(order, range, two, ndim, mxbl)
;           where:
;                order is the order array
;                range is the range array
;                two is the degenerate dimension
;                ndim is the number of dimensions (assume 3)
;                mxbl is the maximum number of blocks
;
;    connect (Npar, nconnect) is zero for single block data
;            nconnect is the connection number definded below
;            Npar is 7 with each value indicating the following:
;            0 - plane for the connection 1, 2, 3 (normal direction)
;            1 - block with smallest minimum in the normal direction
;            2 - block with the minimum equal to the connection plane
;            3 - low value of first remaining dimension of the overlap
;            4 - high value of first remaining dimension of the overlap
;            5 - low value of the last remaining dimension of the overlap
;            6 - high value of the last remaining dimension of the overlap
;            Example: coordinates x, y, z, blocks 1 and 2 connect at y = 10
;            and 1 is below 2 then parameter 0 is 2,1 is 1, 2 is 2, 3 and 4 
;            are the minimum and maximum values of x for the overlap and 5 
;            and 6 are the minimum and maximum values of z for the overlap.
;-
;
      on_error, 1
;
; generate the connections 
;
        if mxbl le 1 then return, 0L
;
        npar = 7
;       make a big array for connections and reduce it later
        connect = make_array(npar, 4*mxbl)
;       look for block connections  -- ONLY consider 3D
        nconnect = 0
        for i= 0 , ndim-1 do begin
          if i+1 ne two or ndim ne 3 then begin
;         calculate other dimensions
            j = i+1
            if j eq ndim then j = 0
            k = j+1 
            if k eq ndim then k = 0
            j = min([j,k], max = k)
            err = max(abs([range(0,i,0), range(1,i,0)]))*6.1e-5
;           look in the i direction 
;           use bs for the start block bf for the final block
            for bs = 1, mxbl-1 do begin
              bbs = order(i, bs-1)+1
              for bf = bs+1, mxbl do begin
;               calculate block numbers, block bf = bbf, and block bs = bbs
                bbf = order(i, bf-1)+1
                if abs(range(1,i,bbs)-range(0,i,bbf)) $
                  lt err then begin
                  connect(3, nconnect) = max([range(0, j, bbs), $
                      range(0, j, bbf)])
                  connect(4, nconnect) = min([range(1, j, bbs), $
                      range(1, j, bbf)])
;                 save a few values in case we have a connection
                  connect(0, nconnect) = i
                  connect(1, nconnect) = bbs
                  connect(2, nconnect) = bbf
                  if ndim gt 2 then begin
                    connect(5, nconnect) = max([range(0, k, bbs), $
                      range(0, k, bbf)])
                    connect(6, nconnect) = min([range(1, k, bbs), $
                      range(1, k,bbf)])
                    if connect(6, nconnect) - connect(5, nconnect) gt err $
                      or connect(4, nconnect) - connect(3, nconnect) gt err $
                      then nconnect = nconnect+1
                  endif else if connect(4,nconnect) - connect(3,nconnect) $
                      gt err then nconnect = nconnect+1
                endif
              endfor
            endfor
          endif
        endfor
        if nconnect eq 0 then connect=0L else connect=connect(*,0:nconnect-1)
        return, connect
        end
