; ------------------------------------------------------------------------ 
; $Id: get_master_time.pro,v 1.4 2008/06/12 21:57:50 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro  get_master_time, array1, array2, max1, max2, np1, np2, xydata, $
         junk, master = masterin, default = default
;
;      put two WDF arrays on a common time base
;
      on_error, 1
;
      if n_elements(default) eq 0 then default = 0
      if not keyword_set(masterin) then begin
        print, '% GET_MASTER_TIME: Keyword master is not set'
        return
      endif
      master = masterin
      if master lt 0 then mast = -master else mast = master
      if round (mast) eq 2 then mast = 2 else mast = 1

      nel1 = n_elements (array1)
      nel2 = n_elements (array2)
      if nel1 eq np1+ np1 then begin
        xy1 = 1 
        min1 = min(array1[0:np1-1])
      endif else begin
        xy1 = 0
        min1 = array1[0]
      endelse
      if nel2 eq np2+ np2 then begin
        xy2 = 1 
        min2 = min(array2[0:np2-1])
      endif else begin
        xy2 = 0
        min2 = array2[0]
      endelse
;     check the mean against machine accuracy and the range against pff.
      start_err = max([abs([min1, max1, min2, max2])/32, $
              (max1-min1), (max2- min2)]) /32768.
      start_err2 = start_err+start_err
;      xydata = xy1

;
;     See if we are nearly equal already
;
      if np1 eq np2 and (abs(min1 - min2) le start_err) then begin
;       First check XY data
        if  xy1*xy2 eq 1 then begin
;         we have xydata, and the number of points and start values are equal
          if max((array1(0:np1-1) - array2(0:np1-1)), min=mm)- mm le $
            start_err2 then begin
            xydata = 1
            return
          endif
        endif else if (xy1 ne xy2) then begin
          if xy1 eq 0 then begin
;           Uniform array 1
            if max((array1[0]+findgen(np1)*array1[1]-array2(0:np1-1)),min=mm) $
               -mm le start_err2 then begin
              if mast eq 1 then begin
                xydata = 0
                array2 = [array1[0:1], array2[np2:*]]
              endif else begin
                xydata = 1
                array1 = [array2[0:np1-1], array1[2:np1+1]]
              endelse
              return
            endif
          endif else begin
;           Uniform array 2
            if max((array2[0]+findgen(np2)*array2[1]-array1(0:np1-1)),min=mm) $
               -mm le start_err2 then begin
              if mast eq 1 then begin
                xydata = 1
                array2 = [array1[0:np1-1], array2[2:np1+1]]
              endif else begin
                xydata = 0
                array1 = [array2[0:1], array1[np1:*]]
              endelse
              return
            endif 
          endelse
        endif else if abs(array1[1] -array2[1]) lt start_err then begin
;         Both arrays are uniform uniform data
            xydata = 0
            return
        endif
      endif 
; 
;     arrays are not equal
; 
      
      if mast eq 2 then begin
;
;       work with 1 as the master array
;       switch 1 and 2
        xydata = xy2
        temp = temporary(array1)
        array1 = temporary(array2)
        array2 = temporary(temp)
        temp = min1 & min1 = min2 & min2 = temp
        temp = xy1 & xy1 = xy2 & xy2 = temp
        temp = max1 & max1 = max2 & max2 = temp
        temp = np1 & np1 = np2 & np2 = temp
      endif else xydata = xy1
;
;     make sure that array1 (master array) is an increasing function of x
      if xy1 gt 0 then begin
        dx = array1[1:np1-1]-array1[0:np1-2]
        if min(dx) lt 0 then begin
          lcsort = 1
          xind = sort(array1[0:np1-1])
          array1 = array1[ [xind, np1+xind] ]
          xind = sort(xind)
          if master gt 0 then begin
            print, '% GET_MASTER_TIME: MASTER set to negative'
            master = -3
          endif
        endif else lcsort = 0
      endif
;
;  find the first point of match in the master array
;
      if min2 le min1 then st = 0 else begin
        if xy1 gt 0 then begin
          ok = where (array1[0:np1-1] ge min2 , nok)
          if nok lt 2 then begin
            print, '% GET_MASTER_TIME: Less than 2 points overlap'
            array1=0
            array2 =0
            return
          endif 
          st = ok[0]
        endif else begin
          st = ceil((min2-min1)/array1[1])
          if st ge np1-3 then begin
            print, '% GET_MASTER_TIME: Less than 2 points overlap' 
            array1=0
            array2=0
            return
          endif
        endelse
      endelse
;
;  find the last point of match in the master array
;
      if max2 ge max1 then en = np1-1 else begin
        if xy1 gt 0 then begin
          ok = where (array1[0:np1-1] le max2 , nok)
          if nok lt 2 then begin
            print, '% GET_MASTER_TIME: Less than 2 points overlap' 
            array1=0
            array2 =0
            return
          endif
          en = ok[nok-1]
        endif else begin
          en = floor((max2-min1)/array1[1])
          if en lt  2 then begin
            print, '% GET_MASTER_TIME: Less than 2 points overlap' 
            array1=0
            array2=0
            return
          endif
        endelse
      endelse

      if xy1 eq 0 then begin
        xst = array1[0]+st*array1[1]
        xen = array1[0]+en*array1[1]
      endif
      if xy2 eq 1 then begin
        dx = array2[1:np2-1]-array2[0:np2-2]
        if min(dx) lt 0 then begin
          a = sort(array2[0:np2-1])
          array2= array2[ [a, np2+a]]
        endif
        space = array2[0:np2-1]
        image = array2[np2:*]
      endif else begin
        space = array2[0]+array2[1]*findgen(np2)
        image = array2[2:*]
      endelse

      if xy1 eq 0 then $
        put2uniform, space, image, np2, xst, array1[1], xen, array2, /yonly $
      else array2 = get_yarr(space, image, array1[st:en])
      if master lt 0 then begin
        if st gt 0 then $
            array2 = [make_array(st, value = float(default)), array2]
        en = en+1
        if en lt np1 then $
            array2 = [ array2, make_array(np1-en, value = float(default))]
      endif

      if master lt 0 then begin
        if xy1 eq 0 then array2 = [array1[0:1], array2] else begin
          if lcsort ne 0 then begin
            xval = (array1[0:np1-1])[xind]
            array2 = [xval, array2[xind]]
            array1 = [xval, (array1[np1:*])[xind]]
          endif else $
            array2 = [array1[0:np1-1], array2]
        endelse
      endif else begin
        if xy1 eq 0 then begin
          array2 = [ xst, array1[1], array2]
          array1 = [ xst, array1[1], array1[2+st:2+en] ]
        endif else begin
          xx = array1[st:en]
          yy = array1[np1+st:np1+en]
          array1 = [xx, yy]
          array2=[xx, array2]
        endelse
        np1 = en-st+1
      endelse

      np2 = np1
      if mast eq 1 then return

      temp = temporary(array1)
      array1 = temporary(array2)
      array2 = temporary(temp)
      temp = min1 & min1 = min2 & min2 = temp
      temp = xy1 & xy1 = xy2 & xy2 = temp
      temp = max1 & max1 = max2 & max2 = temp
      end
