; ------------------------------------------------------------------------ 
; $Id: get_pfidl_directory.pro,v 1.14 2011/05/05 18:36:04 lpmix Exp $
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; Function Get_PFIDL_Directory
;
;   Return the PFIDL Directory ending in a path separator
;
;   If this is a unix system, check for a link to the distribution .../idl/lib/
;     Specifically look for "/idl*/lib/pfidl/pfidl_1d" in the path(!PATH).
;   Otherwise, look for a "/pfidl/pfidl/pfidl_1d" in the path (!PATH).
;   In all cases the "/" is PATH_SEP() for the current system.
;
;  EXAMPLE:
;    Get the pfidl directory and store it in pdir
;      PDIR = Get_PFIDL_Directory()
;-
 function  Get_PFIDL_Directory
 COMPILE_OPT hidden
on_error, 1
   common pfidl_directory_common, pdirect
   if n_elements(pdirect) eq 1 then return, pdirect
   pdirect = getenv('PFIDL_DIRECTORY')
   sep = path_sep()
   if strlen (pdirect) gt 1 then $
     if get_match(sep+'$', pdirect, casesen =0, regexp = 0) lt 0 $
       then return, pdirect+sep else return, pdirect
;  get a list of directories 
   aa = strsplit(!path, path_sep(/search_path), /extract)
;  See if we have a unix installation with a soft link for PFIDL
   
   if !version.os_family eq 'unix' then begin
;    Search path for installations with a link
;    ln -s ../../pfidl/pfidl . in ".../rsi/idl/lib" or ".../itt/idl/lib"
     pdirl = sep+'lib'+sep+'pfidl'+sep+'pfidl_1d'
;    use idl* for other idl versions other than most current version
     pdir = sep+'idl*'+pdirl
     j = get_match(pdir, aa, nelements = nlink, casesen =1, regexp = 0)
     if nlink gt 0 then begin
       a = aa[j[0]]
       j = strpos(a, pdirl)
       pdir = strmid (a, 0, j+11)
       jj = file_info(pdir)
       if jj.symlink gt 0 then begin
         a = file_readlink(pdir)
;        a = file_readlink(pdir, /allow_nonsymlink)
         if strlen(a) gt 1 then begin
;          see if we have a complete path or a relative path
           jj = strpos(a, 'pfidl'+sep+'pfidl')
           if strmid(a, 0, 1) eq sep then begin
             if jj gt 0 then pdir = strmid(a, 0, jj+6) else pdir = a
;             help, pdir
           endif else begin
             if jj gt 0 then begin
               pdir = strmid(pdir, 0,j+5) +strmid(a, 0, jj+6)
               j = file_info(pdir)
               pdir = j.name
             endif else pdir = '' ; No pfidl/pfidl in the link
           endelse
         endif else pdir = '' ; Unable to get link!  unlikely
       endif else pdir = '' ; No symlink
     endif else pdir = '' ; No matches for "/lib/pfidl/pfidl_1d" in !path
     l = strlen (pdir)
;    We have a directory - see if PFIDL is really there
     if l gt 1 then begin
       pdirect = pdir
       if strmid(pdir, l-1) ne sep then pdirect = pdir+sep
       aa = file_search(pdirect+'pfidl'+sep+'pfidl_1d', 'add.pro', count=n)
;      next line for original version of PFIDL with PFIDL_1D files in PFIDe
;      if n eq 0 then  aa = file_search(pdirect, 'add.pro', count=n)
       if n eq 0 then begin
         print, '% GET_PFIDL_DIRECTORY: PFIDL linked directory not found'
         print, '  Please set the system variable: PFIDL_DIRECTORY'
         print, '  or use the SET_PFIDL_DIRECTORY procedure.'
         pdirect = ''
         return, pdirect
       endif else return, pdirect
     endif
   endif
         
;  search path for independent installation with PFIDL in IDL_PATH
   pdir = sep+'pfidl'+sep+'pfidl'+sep+'pfidl_1d'
   j = get_match(pdir, aa, nelements = n, casesen =0, regexp = 0)
   if n eq 0 then begin
     print, '% GET_PFIDL_DIRECTORY: PFIDL directory not found'
     print, '  Please set the system variable: PFIDL_DIRECTORY'
     print, '  or use the SET_PFIDL_DIRECTORY procedure.'
     pdirect = ''
     return, pdirect
   endif 
   aa = aa[j[0]]
   j = strpos(strlowcase(aa), pdir)
   pdirect = strmid(aa, 0, j+7)
   return, pdirect
 end

