; ------------------------------------------------------------------------ 
; $Id: get_scale_image.pro,v 1.5 2008/01/24 22:21:08 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;

 pro get_scale_image, image, xc, yc, newimage, newx, newy, magin, $
     maxx, maxy, xrange, yrange, dx, dy
;
; Note: Magnification can be slightly different in the X and Y directions.
;
; image = original image
; xc   X Center of new image
; yc   Y Center of new image
; newimag = new magnified image
; newx    = X values of new image
; newy    = Y values of new image
; mag     = Magnification
; maxx    = maximum size in x
; maxy    = maximum size in y
;
on_error,1

ssim = size(image)
if (ssim[0] eq 3) then begin
  trucolor = 1
  nlev = ssim[1]
  mx = ssim[2]
  my = ssim[3]
endif else begin
  trucolor =0
  mx = ssim[1]
  my = ssim[2]
  nlev = 1
endelse
; 
; Minmag is minimum magnification which just fills the screen
;
minmag = min([float(maxx-1)/(mx-1), float(maxy-1)/(my-1)])
mag = magin > minmag
; Range of the data is the number of bins in the output
; divided by the maginfication
xr = float(maxx-1)/mag < (mx-1)
yr = float(maxy-1)/mag < (my-1)


x0 = (xc - xr/2) > 0
x1 = (x0+xr) < (mx -1)
if x1 eq (mx-1) then x0 = (x1-xr)>0

y0 = (yc - yr/2) > 0
y1 = (y0+yr) < (my -1)
if y1 eq (my-1) then y0 = (y1-yr)>0

nnx = round(mag*(x1-x0))+1 < maxx
nny = round(mag*(y1-Y0))+1 < maxy

dx = (x1-x0)/(nnx-1)
dy = (y1-y0)/(nny-1)
newx = x0 + findgen(nnx)*dx
newy = y0 + findgen(nny)*dy
xrange = [x0, newx[nnx-1]]
yrange = [y0, newy[nny-1]]
newimage= image
m = mag
if size (newimage, /type ) eq 1 and trucolor eq 0 then $
    newimage = bytscl (newimage)
while ( mag lt 0.375) do begin
  if mx mod 2 eq 0 then mx = mx/2 else mx = mx/2 +1
  if my mod 2 eq 0 then my = my/2 else my = my/2 +1
  if nlev eq 1 then sim = smooth(newimage,3) else begin
    sim = newimage
    for i = 0, nlev-1 do sim[i, *, *]= smooth(newimage[i, *, *], 3)
  endelse
  newimage = (nlev eq 1) ? ((sim)[lindgen(mx)*2,*])[*,lindgen(my)*2] : $
   ((sim)[*, lindgen(mx)*2,*])[*, *,lindgen(my)*2]
  
  if size (newimage, /type ) eq 1 and trucolor eq 0 then $
    newimage = bytscl (newimage)
;  if size (newimage, /type ) eq 1 then newimage = bytscl (newimage)
  mag = mag*2
  newx = newx*0.5
  newy = newy*0.5
endwhile
if mag lt 0.75 then newimage = smooth(newimage, 3)
if nlev eq 1 then newimage = interpolate(newimage, newx, newy, /grid) else begin
   tmp = newimage
   newimage = make_array(nlev, n_elements(newx), n_elements(newy), type = ssim[4])
   for i = 0, nlev-1 do newimage[i, *, *]= $
         interpolate(tmp[i,*,*], newx, newy, /grid)
endelse
if m lt mag then begin
  newx = x0 + findgen(nnx)*dx
  newy = y0 + findgen(nny)*dy
endif
end

