; ------------------------------------------------------------------------ 
; $Id: get_wdfarray.pro,v 1.4 2007/06/06 19:34:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro get_wdfarray, array, wdf, xydata = xydata, npoint= npoint, $
              clabel = clab, file = file, xlabel = xlab, ylabel = ylab, $
              nel = nel, no_copy= no_copy
      
;+
;     this procedure moves an array from the common area indicated by wdf
;     If xydata is absent or zero, then if the number of points is
;     half the length of the array, the xy data will be converted to a wdf
;     array on a uniform time base usint i2w; if NPOINT is specified,
;     delta will be specified. nel is the number of points in the stored array
;     format:
;         get_wdfarray, array, wdf, xydata = xydata, npoint= npoint, $
;             clabel = clab, file = file, xlabel = xlab, ylabel = ylab, $
;             nel = nel, no_copy= no_copy
;      where:
;             array - array of the form [t0, dt, y-values] unless keyword
;                       xydata is set
;                     if xydata is set AND the data is xy data the array is
;                       of the form [x-values, yvalues]
;             wdf   - WDF array number
;             xydata - If set xydata will be returned
;                      If not set, all data will be returned on a uniform grid
;             npoint - Ignored unless data is xy and xydata is not set.
;                      For XY data npoint will be used to put on a uniform grid
;             clabel, xlabel, ylabel - dataset labels
;             nel    - Number of points in the original array.
;                      For uniform data, n_elements(array)= nel+2
;                      For X-Y data, n_elements(array)= nel+nel if XYDATA is set
;                      For X-Y data, n_elements(array)= ?? if XYDATA is not set
;             File   - File associated with this data
;             no_copy- If set, this wdfarray will become undefined. (npoint = 0)
;
;   
;-
;     include the common block
 COMPILE_OPT hidden
@com.dat
;
;
      on_error, 1
      if n_elements(initwdf) eq 0 then wdfinit
;     start
      array = 0
      if n_elements(wdf) eq 0 then begin
        print,'Bad parameters to GET_WDFARRAY'
        return
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print,'%GET_WDFARRAY: Bad WDF array number = ',strtrim (wdf, 2)
        print,'     (WDF array number must be >0 and </=',strtrim(maxwdfarray,2)
        return
      end
      wdf1 = long(wdf(0))-1
      nel = wdfnp(wdf1)
      if nel eq 0 then begin
        print, '%GET_WDFARRAY: No data in array ' ,strtrim (wdf1+1, 2)
        return
      endif
      clab = wdfcomment(wdf1)
      xlab = wdfxlabel(wdf1)
      ylab = wdfylabel(wdf1)
      file = wdffile(wdf1)

;
;     set the number of points to zero if no_copy is set
;
      if keyword_set(no_copy) then wdfnp(wdf1) = 0 else no_copy = 0
      handle_value, wdfarraycom(wdf1), array, no_copy = no_copy
      if keyword_set(xydata) then return

      if n_elements(array) eq nel+nel and nel gt 2 then begin
        if n_elements(npoint) eq 0 then i2w, array(0: nel-1), array(nel:*), $
          maxwdfarray else i2w, array(0: nel-1), array(nel:*),maxwdfarray, $
          delta = (wdfmaxx(wdf1)-wdfminx(wdf1))/float((npoint>2)-1)
        handle_value, wdfarraycom(maxwdfarray-1), array
      endif
      return
      end
