; ------------------------------------------------------------------------ 
; $Id: get_y.pro,v 1.3 2007/06/06 19:34:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
function get_y, xarrayin, yarrayin, x
;+
;     Format: y = get_Y(xarray, yarray, x)
;     this function returns the y value of a function for a passed x value
;     xarray is assumed to have at least as many elements as yarray
;     The routine looks form the minimum of (xarray -x) and interpolates
;      Zero is returned if x is outside the values of xarray within an error
;     of xerr = max([2.38e-7*nel, 6.1e-5]) of the range of data
;     If x eq xarray +/- xerr yarray is returned
;        else linear interpolation is used
;-
      nel = n_elements(yarrayin)
      if nel le 1 then return, 0.0
      if (nel gt n_elements(xarrayin)) then begin
        print, ' X-array must have as many elements as the Y array.'
        return,0.0
      endif
      xarray = xarrayin(0: nel-1)
      yarray = yarrayin(0: nel-1)
; add some fudge room
      xerr = max([2.38e-7*nel, 6.1e-5]) *(max(xarray, min = mn)- mn)
      if (x le xarray(0)-xerr  or x ge xarray(nel-1)+xerr) then begin
;        print, ' X value is outside the range of the data'
        return, 0.0
      endif
      x = float(x)
      delx= min(abs(xarray -x), i)
      if abs(delx) le xerr then return, yarray(i)
      if x le xarray(i) then begin
        delta = abs(xarray(i)-xarray(i-1))
        return, (delx*yarray(i-1)+(delta-delx)*yarray(i))/delta
      endif
      delta = abs(xarray(i+1)-xarray(i))
      if delta eq 0 then return ,0.5*(yarray(i+1)+yarray(i))
      return, (delx*yarray(i+1) + (delta-delx)*yarray(i))/delta
      end
