; ------------------------------------------------------------------------ 
; $Id: i2xy.pro,v 1.5 2009/06/04 18:40:47 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
  pro i2xy, space_in, image_in, wdf, clabel = clab, file = file, $
      xlabel = xlab, ylabel = ylab
;+
; This procedure converts an x and y array to a wdf array but does not 
;   interpolate to a uniform time base.  The x adn y array are stored 
;   exactly as passed; if they are used for other routines, such as add,
;   subtract, etc. then they are sent to I2W to be put on a uniform timebase.
; format:  i2xy, space, image, wdf, clabel = clab, file = file, $
;      xlabel = xlab, ylabel = ylab
;     this procedure converts two idl arrays to an wdf array of x and y values.
;     space is the x array
;     image is the y array
;     wdf is the wdf array number
;     clabel is dataset comment (used for title of plots)
;     ylabel is y-axis (ordinate)  or block label
;     xlabel is x-axis (abscissa) label
;     file is file or source of these dataset values
;-
;     include the common block
@com.dat
;
      on_error, 1
;     move input variables to work arrays
      if n_elements(space_in) gt 2 then space = space_in else begin
        print, '% I2XY - First parameter must have 3 or more elements'
        return
      endelse
      nel = n_elements(image_in)
      if nel gt 2 then image = image_in else begin
        print, '% I2XY - Second parameter must have 3 or more elements'
        return
      endelse
;     see if we are initialized
      if n_elements(initwdf) eq 0 then wdfinit
;
;     remove any unused dimensions
      space = reform (space)
      image = reform (image)
      n = size(space)
      m = size(image)
;     Make sure the arrays are good
;
      if (n[0] ne 1) or (m[0] ne 1) then begin
        print, 'x and y arrays must be one dimensional to convert to WDF array'
        return
      endif
      if (n[1] lt nel) then begin
        print,'x array must be at least as large as the y array'
        return
      endif
;     set arrays to the same size
      if n[1] gt nel then space= space (0:nel-1)
;
;     determine if provided wdf array is good
      if n_params() lt 3 then begin
        print, '% I2XY: Three parameters must be specified '
	return
      endif
      wdf = get_save_array(wdf, 0)
      if wdf eq 0 then begin
        print, '% I2XY: Waveform data array must be specified.'
	print, '        All existing arrays have data'
      endif
      if wdf lt 1 or wdf gt maxwdfarray then begin
        print, 'WDF array must be greater than 0 and </=', maxwdfarray
        return
      endif
      if n_elements(clab) gt 0 then wdfcomment(wdf-1)= strtrim(clab, 2) else  $
        wdfcomment(wdf-1) = '  '
      if n_elements(file) gt 0 then wdffile(wdf-1)= strtrim(file, 2) else  $
        wdffile(wdf-1) = '  '
      if n_elements(xlab) gt 0 then wdfxlabel(wdf-1)= strtrim(xlab, 2) else  $
        wdfxlabel(wdf-1) = '  '
      if n_elements(ylab) gt 0 then wdfylabel(wdf-1)= strtrim(ylab, 2) else  $
        wdfylabel(wdf-1) = '  '
      wdfarray = [space, image]
      put_wdfarray, wdfarray, wdf, /xy, /no_copy
      space = 0
      image = 0
      end
