; ------------------------------------------------------------------------ 
; $Id: mk_poly_field_object.pro,v 1.3 2007/06/06 19:34:30 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+ 
; Make a polygon object from an image.
; Image is a two dimensional image and x, y, and z are coordinates.
;
; Written: March 8, 2000 lpmix
;-
 function mk_poly_field_object,imagein, x, y, z, lev = lev, palette = palette, $
   min_color = min_color, filter = filter, logscale = logscale

 on_error, 2

; common pfidl_obj_common, palette_save
 ss = size(imagein)
 if ((ss[0] lt 2) or (ss[0] gt 3)) then begin
   print, '% MK_POLY_FIELD_OBJECT: Image must have two or three dimensions"
   return, 0
 endif
 image = imagein
  if n_elements (palette) eq 0 then begin
    if n_elements (palette_save) gt 0 then palette= palette_save else begin
      tvlct, /get, red, green, blue
      palette =  OBJ_NEW('IDLgrPalette', red, green, blue)
      palette_save = palette
    endelse
  endif
 if n_elements (palette) eq 0 then return , 0 
 if n_elements (z) eq 0 then z = 0.0
 nx = ss[1] 
 ny = ss[2] 
 if ss[0] eq 3 then nz = ss[3]  else nz = n_elements(z)
 if nx eq 1 then begin
   nxx = ny
   nyy = nz
   xx = make_array (nxx, nyy, value = float(x[0]))
   yy = y # make_array(nz, value = 1.0)
   zz = make_array ( ny, value = 1.0) # z
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_FIELD_OBJECT: Bad data NX = 1
     print, nx, ny, nz, ss
     return, 0
   endif
 endif else if ny eq 1 then begin
   nxx = nx
   nyy = nz
   xx = x # make_array ( nyy, value = 1.0)
   yy = make_array (nxx, nyy, value = float(y[0]))
   zz = make_array (nxx, value = 1.0) # z
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_FIELD_OBJECT: Bad data NY = 1
     print, nx, ny, nz, ss
     return, 0
   endif
 endif else if nz eq 1 then begin
   nxx = nx
   nyy = ny
   xx = x# make_array ( nyy, value = 1.0)
   yy = make_array (nxx, value = 1.0) # y
   zz = make_array (nxx, nyy, value = float(z[0]))
   ima = reform (image, /overwrite)
   ss = size(ima)
   if ss[0] ne 2 and ss[1] ne nxx and ss[2] ne nyy then begin
     print, '% MK_POLY_FIELD_OBJECT: Bad data NZ = 1
     print, nx, ny, nz, ss
     return, 0
   endif
 endif else begin
   print, '% MK_POLY_FIELD_OBJECT: Bad data - period
   print, nx, ny, nz
   return, 0
 endelse
 if keyword_set(filter) then begin
   ss = size(filter)
   if ss[0] ne 2 then begin
     print, '% MK_POLY_FIELD_OBJECT: Filter must be 2 dimensions.'
     return, 0
   endif
   if ss[1] ne nxx or ss[2] ne nyy then begin
     if ss[2] eq nxx and ss[1] eq nyy then filter = transpose (filter) $
       else begin
       print, '% MK_POLY_FIELD_OBJECT: Filter must be the same size as field'
       help, filter
       help, nxx, nyy
       return, 0
     endelse
     print, '% MK_POLY_FIELD_OBJECT: Filter has been transposed.
   endif
 endif
 nxm = nxx-1
 nym = nyy-1
 npoly = nxm*nym
 conn = lonarr(5*npoly)
 conn [5*lindgen(npoly)] = 4L
 
 row = [ 0L, 1, nxx+1,nxx]
 elem = [1,2,3,4l]

if keyword_set (filter) then begin
 nc = 0L
 for j = 0L, nym-1 do begin
   offset = j*nxm
   offset2 = j*nxx
   for i = 0L, nxm-1 do begin
     if min(filter[(offset2+i)+row]) gt 0 then begin
       conn[nc+ elem] = (offset2+i)+row
       nc = nc+5
     endif
   endfor
 endfor
 if nc eq 0 then begin
   print, '% MK_POLY_FIELD_OBJECT: No values for filtered polygons.
   return, 0
 endif
 conn= conn[0:nc-1]
endif else begin
 for j = 0L, nym-1 do begin
   offset = j*nxm
   offset2 = j*nxx
   for i = 0L, nxm-1 do conn[(i+offset)*5+ elem] = (offset2+i)+row
 endfor
endelse
 if n_elements(palette) ne 0 then begin
  if size(palette, /type) eq 11 then opal = palette
 endif
 if n_elements (opal) eq 0 then begin
   loadct, 5
   tvlct, /get, r, g, b
   opal = obj_new ('IDLgrPalette', r, g, b)
 endif
 xx = reform (xx, nxx*nyy, /over)
 yy = reform (yy, nxx*nyy, /over)
 if keyword_set(logscale) then begin
   if n_elements(lev) lt 2 then begin
     lev = [min(ima, max= m), m]
     if (lev[0] lt (1e-5*m)) then lev[0] = (1e-5*m)
   endif
   lev = alog (lev)
   ima = alog (ima > lev[0])
 endif 
 if n_elements(lev) lt 2 then lev = [min(ima, max = mm), mm] else $
   lev= [min(lev, max= mm), mm]
 if n_elements (min_color) eq 0 then $
 vv = bytscl(reform (ima, nxx*nyy, /over), min = lev[0], max = lev[1]) else $
 vv = bytscl(reform (ima, nxx*nyy, /over), min = lev[0], max = lev[1], $
       top = 255-min_color)+byte (min_color > 0)

 omodel = obj_new('idlgrpolygon', xx, yy, zz, polygons = conn, $
    vert_colors = vv, shading =1, palette = opal)

 return, omodel
 end

