; ------------------------------------------------------------------------ 
; $Id: pristr.pro,v 1.4 2007/06/06 19:34:31 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 Pro pristr, ain
;+
;
; NAME:
;      PRISTR
; PURPOSE:
;      Show the elements of a QS structure 
; CALLING SEQUENCE:
;      PRISTR,   STRUCTURE
; INPUTS:
;      STURCTURE  - Particle, field, conductor, or grid sturcture
; Outputs:
;      Structure parameters
; Example:
;      See the contents of structure array, a
;         pristr, a
;      See the contents of structure f12
;         pristr, f12
;-
;  Common block
@qsstrcom.dat
;
;
      if n_elements (structinit) eq 0 then qsstruct_init
;
      on_error, 1
      lcrep = 0
      npar = n_params()
      if npar eq 0 then begin 
        print, 'QS Structure Must Be Specified'
        return
      endif
      if n_elements (ain) gt 0 then a = ain else a= 0
      b= size(a)
      if  b(b(0)+1) ne 8 then begin
        if a gt 0 and a le maxstructure then begin
          lcrep  = a
          s2i, a, a, /no_copy
          b = size (a)
          if b(b(0)+1) ne 8 then begin
            print, ' First parameter must be a structure or a QS array number'
            return
          endif
        endif  else begin
          print, ' First parameter must be a structure or a QS array number'
          return
        endelse
      endif
      c = tag_names(a)
      for i=0,n_elements(c)-1 do if strpos(c(i),'TYPE') ge 0 then goto, check
      print, 'Only qs structures can be printed'
      if lcrep gt 0 then i2s, a, lcrep, /no_copy
      return
check:
      valid =[1, 2, 3, 4, 5, 7, 8, 9]
      ok = where (a.type eq valid, nok)
      if nok eq 0 then begin
        print, ' Only Field, particle, conductor, grid or acis type structures'
        print, '    may be displayed'
        if lcrep gt 0 then i2s, a, lcrep, /no_copy
        return
      endif
      if a.type eq 1 then print, 'FIELD STRUCTURE' else if $
         a.type eq 2 then print, 'PARTICLE STRUCTURE' else if $
         a.type eq 3 then print, 'GRID STRUCTURE' else if $
         a.type eq 7 then print, 'ACIS SLICE STRUCTURE' else if $
         a.type eq 8 then print, 'Finite Elem Field STRUCTURE' else if $
         a.type eq 9 then print, 'Exodus-II Field STRUCTURE' else if $
         a.type eq 5 then print, 'Integer-Float List' else if $
         a.type eq 4 then print, 'CONDUCTOR STRUCTURE'
      if a.type ne 7 then print, 'Data from file: ', a.file
      print, 'Dataset Type: ', a.tlab
      print, 'Dataset Comment: ', a.clab
      if a.type eq 5 then begin
        print, ' Number in Integer Array is ', strtrim(a.size(0), 2)
        print, ' Number in Float List is ', strtrim(a.size(1), 2)
        print, ' Number in Float Array is ', strtrim(a.size(2), 2)
        print, ' Application Dataset Type is ', strtrim(a.spare, 2)
        return
      endif
      if a.type eq 1 or a.type ge 8 then begin
        print, $
        'Number of spatial dimensions is ', strtrim(a.ndim, 2)
        if a.type eq 9 then print, $
        'Number of time samples is ', strtrim(a.ntime, 2)
      endif
      if a.type ne 4 then $
        if a.two eq 1 then print, ' First dimension is degenerate' else $
        if a.two eq 2 then print, ' Second dimension is degenerate' else $
        if a.two eq 3 then print, ' Third dimension is degenerate' 
      if a.type eq 1 or a.type ge 8 then $
        print, 'Number of vector conponents is ' , strtrim(a.nvect, 2) else $
        if a.type eq 2 then $
          print, 'Number of attributes is ' , strtrim(a.natt, 2)
      if a.type eq 1 or a.type eq 8 then begin
        print, 'Number of blocks is ',  strtrim(a.block, 2)
        print, ' Size of the spatial arrays for each block is'
        print, '     Block        Size of each dimension'
        if a.type lt 9 then for i = 0, a.block-1 do print, i+1, a.size (*, i)
          
      endif else if a.type eq 2 then begin
        print, ' Number of particles in this dataset is ', strtrim(a.nvert, 2)
      endif else if a.type eq 3 then begin
        print, 'Number of blocks is ',  strtrim(a.block, 2)
        print, ' Size of the spatial arrays for each block is'
        print, '     Block        Size of each dimension'
        for i = 0, a.block-1 do print, i+1, a.size (*, i)
      endif else if a.type eq 4 then begin
        print, 'Spatial labels are:'
        for i = 0, 2 do begin
          print,'Dimension ',strtrim(string( i+1), 2), ' - ' , a.xlab(i)
          print, '  Range of values is', min(a.plane(0, i, *)),' to', $
                max(a.plane(1, i, *))
        endfor
        print, 'Number of slant surfaces is' , a.size(0)
        print, 'Number of surfaces normal to X-axis is' , a.size(1)
        print, 'Number of surfaces normal to Y-axis is' , a.size(2)
        print, 'Number of surfaces normal to Z-axis is' , a.size(3)
        print, 'Total number of conductor surfaces is ', a.size (4)
        if lcrep gt 0 then i2s, a, lcrep, /no_copy
        return
      endif
  if a.type eq 7 then begin
     print, 'Parts sliced: '
       print, a.part, format='(5x, 15(i3, :, '',''))' 
     print, ' Range of the spatial components for all blocks are:'
     print, a.x1lab, '- Min = ', a.range(0, 0), ', Max = ',a.range(1, 0)
     print, a.x2lab, '- Min = ', a.range(0, 1), ', Max = ',a.range(1, 1)
     print, a.x3lab, '- Min = ', a.range(0, 2), ', Max = ',a.range(1, 2)
     return
  endif
  print, ' Range of the spatial components for all blocks are:'
  if (a.type eq 2 or a.block eq 1 or a.type ge 8) then begin
      print, a.x1lab, '- Min = ', min(a.x1, max = mm), ', Max = ', mm
  if a.ndim gt 1 then print,a.x2lab,'- Min =',min(a.x2,max=mm),', Max =',mm 
  if a.ndim gt 2 then print,a.x3lab,'- Min =',min(a.x3,max=mm),', Max =',mm 
  if a.ndim gt 3 then print,a.x4lab,'- Min =',min(a.x4,max=mm),', Max =',mm 
  if a.ndim gt 4 then print,a.x5lab(0),'- Min =',min(a.x5,max=mm),', Max =',mm
  if a.ndim gt 5 then print,a.x6lab(0),'- Min =',min(a.x6,max=mm),', Max =',mm
  if a.type eq 9 then print,a.x4lab,'- Min =',min(a.x4,max=mm),', Max =',mm 
 endif else begin
      print, a.x1lab(0), '- Min = ', min(a.x1, max = mm), ', Max = ', mm
  if a.ndim gt 1 then print,a.x2lab(0),'- Min =',min(a.x2,max=mm),', Max =',mm
  if a.ndim gt 2 then print,a.x3lab(0),'- Min =',min(a.x3,max=mm),', Max =',mm
  if a.ndim gt 3 then print,a.x4lab(0),'- Min =',min(a.x4,max=mm),', Max =',mm
  if a.ndim gt 4 then print,a.x5lab(0),'- Min =',min(a.x5,max=mm),', Max =',mm
  if a.ndim gt 5 then print,a.x6lab(0),'- Min =',min(a.x6,max=mm),', Max =',mm
 endelse
  if a.type eq 3 then begin
    if lcrep gt 0 then i2s, a, lcrep, /no_copy
    return
  endif
  if a.type eq 1 or a.type ge 8 then begin
  if a.nvect eq 0 then print, 'This dataset has no vector components' else $
print, ' Range of the vector components for all blocks are:'
  if a.nvect gt 0 then print,(a.v1lab)(0),'- Min =',min(a.v1,max=mm),',Max =',mm
  if a.nvect gt 1 then print,(a.v2lab)(0),'- Min =',min(a.v2,max=mm),',Max =',mm
  if a.nvect gt 2 then print,(a.v3lab)(0),'- Min =',min(a.v3,max=mm),',Max =',mm
  if a.nvect gt 3 then print,(a.v4lab)(0),'- Min =',min(a.v4,max=mm),',Max =',mm
  if a.nvect gt 4 then print,(a.v5lab)(0),'- Min =',min(a.v5,max=mm),',Max =',mm
  if a.nvect gt 5 then print,(a.v6lab)(0),'- Min =',min(a.v6,max=mm),',Max =',mm
  endif else begin
  if a.natt eq 0 then print,'This dataset has no attribute components'else $
print,' Range of the attribute components for all blocks are:'
  if a.natt gt 0 then print,a.v1lab,'- Min =',min(a.v1,max=mm),', Max =',mm
  if a.natt gt 1 then print,a.v2lab,'- Min =',min(a.v2,max=mm),', Max =',mm
  if a.natt gt 2 then print,a.v3lab,'- Min =',min(a.v3,max=mm),', Max =',mm
  if a.natt gt 3 then print,a.v4lab,'- Min =',min(a.v4,max=mm),', Max =',mm
  if a.natt gt 4 then print,a.v5lab,'- Min =',min(a.v5,max=mm),', Max =',mm
  if a.natt gt 5 then print,a.v6lab,'- Min =',min(a.v6,max=mm),', Max =',mm
  if a.natt gt 6 then print,a.v7lab,'- Min =',min(a.v7,max=mm),', Max =',mm
  if a.natt gt 7 then print,a.v8lab,'- Min =',min(a.v8,max=mm),', Max =',mm
      endelse
      print, ' Components of the spare array are:' 
      print,  a.spare
      if lcrep gt 0 then i2s, a, lcrep, /no_copy
      return
end
