; ------------------------------------------------------------------------ 
; $Id: make_ufo_coef.pro,v 1.5 2008/02/27 22:32:34 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;+
; NAME:  MADE_UFO_COEF
; 
; PURPOSE:  Generate the coefficient for UFO or for signal synthesis.
;
; MODIFICATION HISTORY:
;       Written by:     L. P. Mix, June 6, 2001
;
;-
;/remote/lpmix/consult/dlfehl/xrd

 pro MAKE_UFO_COEF, shotin, losin, coefficient = arr, str = str, $
    wdfarray = wdfarray, xrdsignal = xrd_data, $
      cathode = cathode, filter = filter, ierror = ier
;
   on_error, 1
@xrd_common_data.dat
@ufo_common_savedata.dat
;
   ier = 1
   str_save = 0
   maxwdf = get_maxwdf()
   if maxwdf lt 512 then set_maxwdfarray, 512

   pff_info, fid= fid, curr = curr, nfile = nfile, /quiet

   if n_elements (shotin) eq 0 then shot = 0 else shot = shotin
   if size(shot, /type) lt 8 then begin
     if hfid ne 0 and shot gt 0 then begin
       if n_elements (shotin) eq 0 then $
         str = '^Shot ' else $
         str = 'Shot'+string (shotin[0], format='(i5)') + ', LOS '
       if n_elements (losin) gt 0 then $
         str = str+strupcase(strtrim(losin, 2))+';;'
       get_dir, hfid, str, line=line, dataset = dataset, number = number
       if number gt 1 then begin
         please_select, lcquit, n, line
         if lcquit eq 0 then number = 1 else return
         dataset = dataset[n]
       endif
       If number gt  0 then $
         readifl, shot, /structure, dataset[0], fil = hfid $
       else begin
         print, '% MAKE_UFO_COEF: No shot header designated.  Returning'
         return
       endelse
     endif
   endif
   if size (shot, /type) eq 8 then begin
     str = temporary(shot)
     if str.type ne 101 then begin
   print, '% Edit_Header: Only shot configuration (101) structures may be input.'
       return
     endif
     shot= str.shot
     los = str.los
     comm = str.comment
     ndet = str.ndet
     dist = str.dist
     stype = str.source
     srcarea = str.src_area
     az_ang = str.az_ang
     Aper_hght = str.aper_ht
     Pinhght = str.pinch_ht
     ele_ang= str.elevation
     signal= str.signal[0:ndet-1]
     cathode= str.cathode[0:ndet-1]
     calibration = str.calibration[0:ndet-1]
     diameter = str.diameter[0:ndet-1]
     filter = str.filter[0:ndet-1]
     energy_bin = str.energy_bin[0:ndet]

   endif else begin    
     print, '% MAKE_UFO_COEF: Shot header must be provided'
     return
   endelse

 st = get_maxwdf() - 5*ndet
 stcath = st
; get the cathodes

 if keyword_set(wdfarray) then begin
   for i = 0, ndet-1 do begin
     name = '^Cathode: '+cathode[i]+', Calibration: '+calibration[i] + ','
     c = get_match(name, nele= n, /wdfarray)
     if n eq 0 then begin
        print, '% MAKE_UFO_COEF: Unable to find WDF array: ', name
        return
     endif
     if n gt 1 then begin
       please_select, lcquit, n, wdcomment(c)
       if lcquit gt 0 then return else c = c[n]
     endif
     xfr, c[0], st+i 
   endfor
 
 endif else begin
   setpff, cfid,  ier
   if ier ne 0 then return
  
   for i = 0, ndet-1 do begin
     name = '^Cathode: '+cathode[i]+', Calibration: '+calibration[i] + ','
     c = get_dset(name, ndset = ndset)
     if ndset eq 0 then begin
       print, '% MAKE_UFO_COEF: Unable to find PFF dataset: ', name
       return
     endif
     if ndset gt 1 then begin
       get_dir, name, comment = c
       please_select, lcquit, n, c
       if lcquit gt 0 then return else c = c[n]
     endif
     re, st+i, c[0]
   endfor
 endelse
; get the filters
 st = st+ndet
 stfilt = st

 if keyword_set(wdfarray) then begin
   for i = 0, ndet-1 do begin
     name = 'Filter: '+filter[i]+','
     c = get_match(name, nele= n, /wdfarray)
     if n eq 0 then begin
        print, '% MAKE_UFO_COEF: Unable to find WDF array: ', name
        return
     endif
     if n gt 1 then begin
       please_select, lcquit, n, wdcomment(c)
       if lcquit gt 0 then return else c = c[n]
     endif
     xfr, c[0], st+i
   endfor

 endif else begin
   setpff, ffid, ier
   if ier ne 0 then return
 
   for i = 0, ndet-1 do begin
     name = 'Filter: '+filter[i]+','
     c = get_dset(name, ndset = ndset)
     if ndset eq 0 then begin
       print, '% MAKE_UFO_COEF: Unable to find PFF dataset: ', name
       return
     endif
     if ndset gt 1 then begin
       get_dir, name, comment = c
       please_select, lcquit, n, c
       if lcquit gt 0 then return else c = c[n]
     endif
     re, st+i, c[0]
   endfor
 endelse

 st = st+ndet
 stdet = st 
 for i = 0, ndet-1 do mul, stcath+i, stfilt+i, stdet+i, clabel = $
    'Detector Response: '+cathode[i]+', '+calibration[i]+', '+ filter[i]
 
 for i = stdet, stdet+ndet-1 do int, i, /double

 if stype eq 1 then begin
;  pinch shot
; Power into the detectors for a pinch is given by;
;   P = (Aperature Height)/(Pinch Length) ; to account for blockage of source
;       * !pi*diameter^2/distance^2/400   ; to account for solid angle
;                                         ; Diameter in mm for the 400
;       * cos(elevation)                  ; to account for Lambertian source
;       * 50                              ; to convert the Amperes to Volts
;       * total power output              ; in MWatts/each energy bin/SR
;       Therefore Total_Power_Output = P/r2 where r2 is given above
;          and P is Power in MW into each of the detectors.
;
   r2 = diameter*diameter *(.1250*!pi/(dist*dist)*cos(!dtor*str.elevation) $
          *str.aper_ht / str.pinch_ht)

;  convert to W/SR/ev from MW/bin  (!pi^2 Sr / pinch)
;  xrays/r2/!pi/!pi/de
;  Let R1 = r2/!pi/!pi
   r1 = r2*!pi*!pi
;
;  To get W/SR divide total Watts by (!pi)^2
 endif else begin
;  hohlraum shot
   print, 'Set Factor to 1'
   r2 = diameter*diameter *(.1250/(!pi*dist*dist)*cos(!dtor*str.elevation) $
          )
   r1 = r2*!pi*!pi
 endelse
 print, ' r2 = ',  r2


 arr = make_array(ndet, ndet, /double)

 for i = 0, ndet-1 do begin
   val = get_value(stdet+i, energy_bin)
   arr[*, i] = (val[1:ndet]-val[0:ndet-1])*r2[i]
 endfor

 scale_save = r2
 str_save = str
 coef_save = arr
 ier = 0
 return
 end
