; ------------------------------------------------------------------------ 
; $Id: showxrd.pro,v 1.3 2007/06/29 13:42:44 lpmix Exp $ 
; ------------------------------------------------------------------------ 

; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
 pro  showxrd, Cathode = cathode, filter = filter, detector = detector, $
       header = header, cfid = ccfid, hfid = hhfid, ffid = fffid, dfid = ddfid
;+
; NAME:
;       SHOWXRD
;
; PURPOSE:
;       Show the current files for cathode, filter, and detector response and 
;         the shot header configuration.
;
; CALLING SEQUENCE:
;       SHOWXRD
;
; INPUTS:
;       None
;
; KEYWORD PARAMETERS:
;       
;       Cathode  - File name to be used for cathode response
;       CFID     - File id of the file to be used for the cathode response
;       Filter   - File name to be used for filter response
;       FFID     - File id of the file to be used for the filter response
;       Detector - File name to be used for detector response
;       DFID     - File id of the file to be used for the detector response
;       Header   - File name to be used for shot configuration header
;       HFID     - File id of the file to be used for shot configuration
;
; MODIFICATION HISTORY:
;      Original version, L. P. Mix, February 12, 2001
;
;-
;
on_error, 1
;
@xrd_common_data.dat
common pffcom, initpff, pff_exe

  if n_elements(initxrd) eq 0 then initxrd = 0
  if initxrd eq 0 then begin
    print, '% SHOWXRD: No files have been set.'
    return
  endif

  if hfid ne 0 then begin
    hhfid = hfid
    header = header_file
  endif else begin
    hhfid = 0
    header = ''
  endelse

  if dfid ne 0 then begin
    ddfid = dfid
    detector = detector_file
  endif  else begin
    ddfid = 0
    detector = ''
  endelse

  if ffid ne 0 then begin
    fffid = ffid
    filter = filter_file
  endif else begin
    fffid = 0
    filter = ''
  endelse

  if cfid ne 0 then begin
    ccfid = cfid
    cathode = cathode_file
  endif else begin
    ccfid = 0
    cathode = ''
  endelse

PRINT, 'File Type  FileID  File Name'
print, 'Header       ', strtrim(hfid, 2) , '     ', header
print, 'Cathode      ', strtrim(cfid, 2) , '     ', cathode
print, 'Filter       ', strtrim(ffid, 2) , '     ', filter
print, 'Detector     ', strtrim(dfid, 2) , '     ', detector

  end
