; ------------------------------------------------------------------------ 
; $Id: filter_fit_function.pro,v 1.3 2007/06/29 13:45:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       Filter_fit_function
;
; PURPOSE:
;       Calculates filter parameters for common block variables
;
; CALLING SEQUENCE:
;       Called by Curvefit
;       filter_fit_function, x, a, f, pder
;       X is x values
;       A is Coefficients
;       F is the function value
;       PDER is the partial derivative with respect to A
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 17, 2001
;-

PRO filter_fit_function, x, a, f, pder

 common filter_fit_funct_common, xval, wdf, fval
;common filter_fit_funct_common
;
nc = n_elements(a)
nelx = n_elements (x)
lccalc = 0
if nelx eq n_elements(xval) then begin
  if (max((x-xval)/x, min= mindif)-mindif) gt 5e-6 then lccalc= 1
endif else lccalc = 1


if lccalc eq 0 then ff = fval else  begin
  print, '% Filter_fit_function: Calculate function'
  ff = make_array(nelx, nc)
  for i = 0, n_elements(wdf)-1 do $
    ff[*, i]= get_value(wdf[i], x)
endelse

f = make_array(nelx, /double, value = 0.0d0)
for i = 0, nc-1 do f = f+a[i]*ff[*, i]
f = exp(-f)

if n_params() gt 3 then begin
  pder = make_array( nelx, nc, /double)
  for i = 0, nc-1 do $
    pder[*, i] = -ff[*, i]*exp(-a[i]*ff[*,i])
endif

END
