; ------------------------------------------------------------------------ 
; $Id: filter_lmfit_function.pro,v 1.3 2007/06/29 13:45:29 lpmix Exp $ 
; ------------------------------------------------------------------------ 
;
; Copyright 2007 Sandia Corporation.
; Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
; license for use of this work by or on behalf of the U.S. Government.
;
;+
; NAME:
;       Filter_lmfit_function
;
; PURPOSE:
;       Calculates filter parameters for common block variables
;
; CALLING SEQUENCE:
;       Called by Curvefit
;       filter_fit_function, x, a, f, pder
;       X is x values
;       A is Coefficients
;       F is the function value
;       PDER is the partial derivative with respect to A
;
; MODIFICATION HISTORY:
; 	Written by:	L. P. Mix, April 17, 2001
;-

function filter_lmfit_function, x, a

common filter_fit_funct_common, xval, wdf, fval
;
nc = n_elements(a)
lccalc = 0
ok = where (x eq xval, nok)
if nok gt 0 then ff= reform(fval[ok[0], *]) else print, '% lmfit error'

f = total(a*ff, /double)
f = exp(-f)

pder = -ff*exp(-a*ff)
return, [f, pder]

END

